@@ -14,6 +14,8 @@ recurse=1
 recurse_given=NO
 use_modulename=1
 cleanup=YES
+preprocess=0
+manpages=0
 
 while test -n "$1" ; do
 case "x$1" in
@@ -34,6 +36,12 @@ case "x$1" in
 "x--modulename" )
 	use_modulename=1
 	;;
+"x--preprocess" )
+	preprocess=1
+	;;
+"x--manpages" )
+        manpages=1
+        ;;
 "x--help" )
 	echo "doxygen.sh usage:"
 	echo "doxygen.sh [--options] <srcdir> [<subdir>]"
@@ -42,6 +50,8 @@ case "x$1" in
 	echo "  --no-modulename  Build in apidocs/, not apidocs-module/"
 	echo "  --doxdatadir=dir Use dir as the source of global Doxygen files"
 	echo "  --installdir=dir Use dir as target to install to"
+	echo "  --preprocess     Generate source code (KConfigXT, uic, etc.)"
+        echo "  --manpages       Generate man pages in addition to html"
 	exit 2
 	;;
 x--doxdatadir=* )
@@ -161,6 +171,32 @@ if test -z "$TOPNAME" ; then
     TOPNAME="API Reference"
 fi
 
+#
+# Preprocess source dir and generate source files (KConfigXT, uic, etc.)
+# @param $1: create or cleanup (parameter for doxygen-preprocess-foo.sh)
+#
+preprocess_sources()
+{
+	for dir in `find $top_srcdir/$subdir -type d | grep -v ".svn"`; do
+		# execute global preprocessing modules
+		local preproc_base="`dirname $0`"
+		for proc in `find "$preproc_base" -name "doxygen-preprocess-*.sh"`; do
+			(cd $dir && $proc $1)
+		done
+		# execute local preprocessing modules
+		for proc in `find $dir -maxdepth 1 -name "doxygen-preprocess-*.sh"`; do
+			echo "* running $proc"
+			(cd $dir && $proc $1)
+		done
+	done
+}
+
+if test "$preprocess" = "1"; then
+	echo "* Generating source code in $top_srcdir/$subdir"
+	preprocess_sources create
+	echo "* Generating source code completed"
+fi
+
 ### We need some values from top-level files, which
 ### are not preserved between invocations of this
 ### script, so factor it out for easy use.
@@ -190,7 +226,7 @@ if test -z "$subdir" ; then
 	test -f "Doxyfile.in" || create_doxyfile_in
 
 	# Copy in logos and the like
-	for i in "favicon.ico" "kde_gear_64.png"
+	for i in "favicon.ico" "block_title_bottom.png" "block_title_mid.png" "block_title_top.png" "top-kde.jpg" "top-left.jpg" "top-right.jpg" "top.jpg" "kde.css" "flat.css" "print.css"
 	do
 		cp "$DOXDATA/$i" . > /dev/null 2> /dev/null
 	done
@@ -244,6 +280,8 @@ do
 	# referenced tag files.
 	list=""
 	if test -d "$dir" && test -f "$dir/Mainpage.dox" ; then
+		# Need this dir at the very least, even if it doesn't reference
+		echo "$dir" "$dir"
 		list=`grep DOXYGEN_REFERENCES $dir/Mainpage.dox | sed 's/.*=//'`
 	fi
 
@@ -258,6 +296,18 @@ done ) | grep -v / | tsort 2> /dev/null > subdirs.top
 }
 
 
+apidox_manpages()
+{
+        mp=`extract_line DOXYGEN_GENERATE_MAN`
+        if test -z "$mp" ; then
+               mp="YES"
+        fi
+	echo "GENERATE_MAN           = $mp" >> "$subdir/Doxyfile"
+        echo "MAN_OUTPUT             = man" >> "$subdir/Doxyfile"
+        echo "MAN_EXTENSION          = .3"  >> "$subdir/Doxyfile"
+        echo "MAN_LINKS              = YES" >> "$subdir/Doxyfile"
+}
+
 ### Add HTML header, footer, CSS tags to Doxyfile.
 ### Assumes $subdir is set. Argument is a string
 ### to stick in front of the file if needed.
@@ -448,6 +498,9 @@ apidox_toplevel()
 		echo "HTML_OUTPUT            = ." 
 	} >> Doxyfile
 	apidox_htmlfiles "main"
+        if test "$manpages" = "1"; then
+                apidox_manpages
+        fi
 
 	# KDevelop has a top-level Makefile.am with settings.
 	for i in "$top_srcdir/Mainpage.dox"
@@ -540,16 +593,22 @@ apidox_subdir()
 		echo "RECURSIVE              = NO"
 	fi
 	echo "HTML_OUTPUT            = $subdir/html"
-	test -d "$top_srcdir/doc/api" && \
-		echo "IMAGE_PATH             = $top_srcdir/doc/api"
+	if test -d "$top_srcdir/doc/api"; then
+		echo "IMAGE_PATH             = $top_srcdir/doc/api $srcdir"
+	else
+		echo "IMAGE_PATH             = $srcdir"
+	fi
 	} >> "$subdir/Doxyfile"
 
 	apidox_htmlfiles ""
+        if test "$manpages" = "1"; then
+                apidox_manpages
+        fi
 
 	# Mainpage.doxs may contain overrides to our settings,
 	# so copy them in.
 	grep '^//[[:space:]]*DOXYGEN_SET_' "$srcdir/Mainpage.dox" | \
-		sed -e 's+//\s*DOXYGEN_SET_++' >> "$subdir/Doxyfile"
+		sed -e 's+//[[:space:]]*DOXYGEN_SET_++' >> "$subdir/Doxyfile"
 	apidox_specials "$srcdir/Mainpage.dox" "$subdir/Doxyfile"
 
 	excludes=`extract_line DOXYGEN_EXCLUDE`
@@ -623,9 +682,9 @@ apidox_subdir()
 			else
 				if test -z "$file" ; then
 					# Really no Qt tags
-					echo "" >> "$subdir/Doxyfile"
+					echo "\\" >> "$subdir/Doxyfile"
 				else
-					echo "  $file=$QTDOCDIR" >> "$subdir/Doxyfile"
+					echo "  $file=$QTDOCDIR \\" >> "$subdir/Doxyfile"
 				fi
 			fi
 		else
@@ -636,17 +695,19 @@ apidox_subdir()
 		fi
 	done
 
-	apidox_local
-
 	if test -z "$1" ; then
+		echo "" >> "$subdir/Doxyfile"
 		echo "GENERATE_HTML=NO" >> "$subdir/Doxyfile"
 		echo "GENERATE_TAGFILE=$subdir/$subdirname.tag" >> "$subdir/Doxyfile"
 		test -z "$NO_APPEND_TAG" && echo "	$subdir/$subdirname.tag \\" >> subdirs.tag
 	else
+		#test -s subdirs.tag && grep -v "$subdir/$subdirname.tag" subdirs.tag >> "$subdir/Doxyfile"
+		echo "" >> "$subdir/Doxyfile"
 		echo "GENERATE_HTML=YES" >> "$subdir/Doxyfile"
-		test -s subdirs.tag && grep -v "$subdir/$subdirname.tag" subdirs.tag >> "$subdir/Doxyfile"
 	fi
 
+	apidox_local
+
 	if grep '^DOXYGEN_EMPTY' "$srcdir/Mainpage.dox" > /dev/null 2>&1 ; then
 		# This directory is empty, so don't process it, but
 		# *do* handle subdirs that might have dox.
@@ -667,7 +728,7 @@ apidox_subdir()
 #
 do_subdir()
 {
-	subdir=`echo "$1" | sed -e 's+/$++'`
+	local subdir=`echo "$1" | sed -e 's+/$++'`
 	srcdir="$top_srcdir/$subdir"
 	subdirname=`basename "$subdir"`
 	mkdir -p "$subdir" 2> /dev/null
@@ -897,8 +958,13 @@ fi
 if test "YES" = "$cleanup" ; then
 	rm -f subdirs.in  subdirs.later subdirs.sort subdirs.top Doxyfile.in
 	rm -f `find . -name Doxyfile`
-	rm -f qt/qt.tag
-	rmdir qt > /dev/null 2>&1
+#	rm -f qt/qt.tag
+#	rmdir qt > /dev/null 2>&1
+	if test "$preprocess" = "1"; then
+		echo "* Cleaning up previously generated source code in $top_srcdir/$subdir"
+		preprocess_sources cleanup
+		echo "* Cleaning up previously generated source code done"
+	fi
 fi
 
 
