@@ -37,7 +37,6 @@
 
 #include "kimiface.h"
 
-#include "kimproxy.h"
 #include <kservicetype.h>
 
 static KStaticDeleter<KIMProxy> _staticDeleter;
@@ -167,7 +166,7 @@ AppPresenceCurrent ContactPresenceListCurrent::best()
 
 OrgKdeKIMInterface * findInterface( const QString & app )
 {
-        return new OrgKdeKIMInterface( app, "/KIMIface", QDBus::sessionBus() );
+        return new OrgKdeKIMInterface( app, "/KIMIface", QDBusConnection::sessionBus() );
 }
 
 KIMProxy * KIMProxy::instance() 
@@ -181,7 +180,7 @@ KIMProxy::KIMProxy() : QObject(), d( new Private )
 {
 	//QDBus::sessionBus().registerObject( "/KIMProxy", this);
 	m_initialized = false;
-	connect( QDBus::sessionBus().interface(),
+	connect( QDBusConnection::sessionBus().interface(),
 		 SIGNAL(serviceOwnerChanged(QString,QString,QString)),
 		 SLOT(nameOwnerChanged(QString,QString,QString)) );
 
@@ -202,7 +201,7 @@ KIMProxy::KIMProxy() : QObject(), d( new Private )
 	//DCOPCString method = "contactPresenceChanged( QString, QCString, int )";
 	//QCString receiverObjectId = "KIMProxyIface";
 
-	QDBus::sessionBus().connect( QString(), "/KIMIface", "org.kde.KIM", "contactPresenceChanged",
+	QDBusConnection::sessionBus().connect( QString(), "/KIMIface", "org.kde.KIM", "contactPresenceChanged",
 				     this, SLOT(contactPresenceChanged(QString,QString,int)) );
 }
 
@@ -222,7 +221,7 @@ bool KIMProxy::initialize()
 			// see what apps implementing our service type are out there
 			const KService::List offers = KServiceTypeTrader::self()->query( IM_SERVICE_TYPE );
 			KService::List::const_iterator offer;
-			QStringList registeredApps = QDBus::sessionBus().interface()->registeredServiceNames();
+			QStringList registeredApps = QDBusConnection::sessionBus().interface()->registeredServiceNames();
 			foreach (QString app, registeredApps)
 			{
 				//kDebug( 790 ) << " considering: " << *app << endl;
@@ -546,17 +545,17 @@ bool KIMProxy::startPreferredApp()
 #ifdef __GNUC__
 # warning "unused variable: preferences"
 #endif
-	QString preferences = QString("[X-DCOP-ServiceName] = '%1'").arg( preferredApp() );
-	// start/find an instance of DCOP/InstantMessenger
+	QString preferences = QString("[X-DBUS-ServiceName] = '%1'").arg( preferredApp() );
+	// start/find an instance of DBUS/InstantMessenger
 	QString error;
-	QString dcopService;
+	QString dbusService;
 	// Get a preferred IM client.
 	// The app will notify itself to us using nameOwnerChanged, so we don't need to record a stub for it here
 	// FIXME: error in preferences, see debug output
 	preferences.clear();
-	int result = KDBusServiceStarter::self()->findServiceFor( IM_SERVICE_TYPE, QString("Application"), &error, &dcopService );
+	int result = KDBusServiceStarter::self()->findServiceFor( IM_SERVICE_TYPE, QString("Application"), &error, &dbusService );
 
-	kDebug( 790 ) << k_funcinfo << "error was: " << error << ", dcopService: " << dcopService << endl;
+	kDebug( 790 ) << k_funcinfo << "error was: " << error << ", dbusService: " << dbusService << endl;
 
 	return ( result == 0 );
 }
