@@ -23,13 +23,13 @@
 // the very important KTextEditor::Cursor class
 #include <ktexteditor/cursor.h>
 #include <ktexteditor/range.h>
-#include <kdocument/document.h>
 
 // our main baseclass of the KTextEditor::Document
 #include <kparts/part.h>
 
 // the list of views
-#include <QList>
+#include <QtCore/QList>
+#include <QtCore/QMetaType>
 
 namespace KTextEditor
 {
@@ -54,7 +54,7 @@ class View;
  * synchronized. Support for text selection is handeled by a View and text
  * format attribues by the Attribute class.
  *
- * To load a document call KParts::ReadOnlyPart::openURL().
+ * To load a document call KParts::ReadOnlyPart::openUrl().
  * To reload a document from a file call documentReload(), to save the
  * document call documentSave() or documentSaveAs(). Whenever the modified
  * state of the document changes the signal modifiedChanged() is emitted.
@@ -63,9 +63,6 @@ class View;
  * with setEncoding(), however this will only take effect on file reload and
  * file save.
  *
- * Every document has a application wide unique document number, accessible
- * with documentNumber().
- *
  * \section doc_manipulation Text Manipulation
  *
  * Get the whole content with text() and set new content with setText().
@@ -108,7 +105,7 @@ class View;
  *      KTextEditor::VariableInterface
  * \author Christoph Cullmann \<cullmann@kde.org\>
  */
-class KTEXTEDITOR_EXPORT Document : public KDocument::Document
+class KTEXTEDITOR_EXPORT Document : public KParts::ReadWritePart
 {
   Q_OBJECT
 
@@ -141,6 +138,13 @@ class KTEXTEDITOR_EXPORT Document : public KDocument::Document
      */
     virtual Editor *editor () = 0;
 
+    /**
+     * Create a new view attached to @p parent.
+     * @param parent parent widget
+     * @return the new view
+     */
+    virtual View *createView ( QWidget *parent ) = 0;
+
     /**
      * Return the view which is currently has user focus, if any.
      */
@@ -149,7 +153,7 @@ class KTEXTEDITOR_EXPORT Document : public KDocument::Document
     /**
      * Returns the views pre-casted to KTextEditor::View%s
      */
-    QList<View*> textViews() const;
+    virtual const QList<View*> &views() const = 0;
 
   Q_SIGNALS:
    /**
@@ -293,16 +297,11 @@ class KTEXTEDITOR_EXPORT Document : public KDocument::Document
      * If the texteditor part does not support these transactions,
      * both calls just do nothing.
      *
-     * \param view here you can optional give a view which does the editing
-     *        this can cause the editor part implementation to do some
-     *        special cursor handling in this view. Important: this only will
-     *        work if you pass here a view which parent document is this
-     *        document, otherwise the view is just ignored.
      * \return \e true on success, otherwise \e false. Parts not supporting
      *         it should return \e false
      * \see endEditing()
      */
-    virtual bool startEditing (View *view = 0) = 0;
+    virtual bool startEditing () = 0;
 
     /**
      * End an editing sequence.
@@ -495,7 +494,7 @@ class KTEXTEDITOR_EXPORT Document : public KDocument::Document
      * is treated as line delimiter, so it is possible to insert multiple
      * lines. To append lines at the end of the document, use
      * \code
-     *   insertLine( numLines(), text )
+     * insertLine( numLines(), text )
      * \endcode
      * \param line line where to insert the text
      * \param text text which should be inserted
@@ -509,7 +508,7 @@ class KTEXTEDITOR_EXPORT Document : public KDocument::Document
      * is treated as line delimiter, so it is possible to insert multiple
      * lines. To append lines at the end of the document, use
      * \code
-     *   insertLine( numLines(), text )
+     * insertLine( numLines(), text )
      * \endcode
      * \param line line where to insert the text
      * \param text text which should be inserted
@@ -580,10 +579,35 @@ class KTEXTEDITOR_EXPORT Document : public KDocument::Document
      * \param document the document being closed
      */
     void aboutToClose(KTextEditor::Document *document);
+
+  private:
+    class DocumentPrivate* const d;
+
+  public:
+    /**
+     * by default dialogs should be displayed.
+     * In any case (dialog shown or suppressed)
+     * openingErrors and openingErrorMessage should have meaningfull values
+     *
+     * \param suppress true/false value if dialogs should be displayed
+     */
+    void setSuppressOpeningErrorDialogs(bool suppress);
+    bool suppressOpeningErrorDialogs() const;
+    /**
+     * True, eg if the file for opening could not be read
+     * This doesn't have to handle the KPart job cancled cases
+     */
+    bool openingError() const;
+    QString openingErrorMessage() const;
+  protected:
+    void setOpeningError(bool errors);
+    void setOpeningErrorMessage(const QString& message);
 };
 
 }
 
+Q_DECLARE_METATYPE(KTextEditor::Document*)
+
 #endif
 
 // kate: space-indent on; indent-width 2; replace-tabs on;
