@@ -21,6 +21,8 @@
 
 #include <ktexteditor/smartrange.h>
 
+class QMutex;
+
 namespace KTextEditor
 {
 class Document;
@@ -85,20 +87,27 @@ class SmartCursor;
  * \section smartiface_access Accessing the Interface
  *
  * The SmartInterface is supposed to be an extension interface for a Document,
- * i.e. the Document inherits the interface \e provided that the 
+ * i.e. the Document inherits the interface \e provided that the
  * KTextEditor library in use implements the interface. Use dynamic_cast to access
  * the interface:
  * \code
- *   // doc is of type KTextEditor::Document*
- *   KTextEditor::SmartInterface *iface =
- *       qobject_cast<KTextEditor::SmartInterface*>( doc );
+ * // doc is of type KTextEditor::Document*
+ * KTextEditor::SmartInterface *iface =
+ *     qobject_cast<KTextEditor::SmartInterface*>( doc );
  *
- *   if( iface ) {
- *       // the implementation supports the interface
- *       // do stuff
- *   }
+ * if( iface ) {
+ *     // the implementation supports the interface
+ *     // do stuff
+ * }
  * \endcode
  *
+ * \section smartiface_threadsafety Thread safety
+ * The smart interface is designed to be usable in multithreaded environments.
+ * If you use the interface from threads other than the main thread, you must
+ * lock the smartMutex() whenever you are making a non-const call to a smart object.
+ * This allows the text editor to guarantee that the objects will not change
+ * when it locks the mutex (for example, when performing layout or rendering).
+ *
  * \author Hamish Rodda \<rodda@kde.org\>
  */
 class KTEXTEDITOR_EXPORT SmartInterface
@@ -109,6 +118,14 @@ class KTEXTEDITOR_EXPORT SmartInterface
     SmartInterface();
     virtual ~SmartInterface();
 
+    /**
+     * Provides access to the recursive mutex used to protect write access to
+     * smart interface objects (cursors + ranges and their associated properties).
+     * If you use this interface  from a thread other than the main thread,
+     * you must lock this mutex whenever you call a non-const function on a smart object.
+     */
+    QMutex* smartMutex() const;
+
     /**
      * Clears or deletes all instances of smart objects, ie:
      * \li deletes all SmartCursor%s
@@ -116,8 +133,6 @@ class KTEXTEDITOR_EXPORT SmartInterface
      * \li clears all arbitrary highlight ranges
      * \li clears all action binding
      *
-     * This method is likely to be noticably faster than deleting and clearing each instance yourself.
-     *
      * Deletion occurs without modification to the underlying text.
      */
     virtual void clearSmartInterface() = 0;
@@ -137,6 +152,50 @@ class KTEXTEDITOR_EXPORT SmartInterface
     void setClearOnDocumentReload(bool clearOnReload);
 
     //BEGIN New cursor methods
+    /**
+     * Retrieve a token representing the current version of the document. This can
+     * be used later to create cursors and ranges as they would have been at this revision.
+     *
+     * Once you have finished with the token, release it with releaseRevision();
+     */
+    virtual int currentRevision() const = 0;
+
+    /**
+     * Release a revision token provided by currentRevision().  You will no longer be able to
+     * create cursors and ranges agaist this revision.
+     */
+    virtual void releaseRevision(int revision) const = 0;
+
+    /**
+     * Tell the smart interface to work against the given \a revision when creating cursors and
+     * ranges.
+     *
+     * \param revision the token representing a revision retrieved by currentRevision(), or -1 to
+     *                 clear any previous setting and use the current document revision.
+     */
+    virtual void useRevision(int revision) = 0;
+
+    /**
+     * Clear any previous setting to use a specific revision.
+     */
+    void clearRevision();
+
+    /**
+     * Translate the given \a cursor against the revision specified through useRevision(),
+     * using the given \a insertBehavior.
+     *
+     * If no revision is set, simply returns the cursor.
+     */
+    virtual KTextEditor::Cursor translateFromRevision(const KTextEditor::Cursor& cursor, KTextEditor::SmartCursor::InsertBehavior insertBehavior = KTextEditor::SmartCursor::StayOnInsert) const;
+
+    /**
+     * Translate the given \a range against the revision specified through useRevision(),
+     * using the given \a insertBehavior.
+     *
+     * If no revision is set, simply returns the range.
+     */
+    virtual KTextEditor::Range translateFromRevision(const KTextEditor::Range& range, KTextEditor::SmartRange::InsertBehaviors insertBehavior = KTextEditor::SmartRange::ExpandLeft | KTextEditor::SmartRange::ExpandRight) const;
+
     /**
      * \name Smart Cursors
      *
@@ -152,9 +211,9 @@ class KTEXTEDITOR_EXPORT SmartInterface
      *
      * \param position The initial cursor position assumed by the new cursor.
      *                 If not specified, it will start at position (0, 0).
-     * \param insertBehaviour Define whether the cursor should move when text is inserted at the cursor position.
+     * \param insertBehavior Define whether the cursor should move when text is inserted at the cursor position.
      */
-    virtual SmartCursor* newSmartCursor(const Cursor& position = Cursor::start(), SmartCursor::InsertBehaviour insertBehaviour = SmartCursor::MoveOnInsert) = 0;
+    virtual SmartCursor* newSmartCursor(const Cursor& position = Cursor::start(), SmartCursor::InsertBehavior insertBehavior = SmartCursor::MoveOnInsert) = 0;
 
     /**
      * \overload
@@ -167,10 +226,10 @@ class KTEXTEDITOR_EXPORT SmartInterface
      *
      * \param line the line number of the cursor's initial position
      * \param column the line number of the cursor's initial position
-     * \param insertBehaviour Define whether the cursor should move when text is inserted at the cursor position.
+     * \param insertBehavior Define whether the cursor should move when text is inserted at the cursor position.
      */
-    inline SmartCursor* newSmartCursor(int line, int column, SmartCursor::InsertBehaviour insertBehaviour = SmartCursor::MoveOnInsert)
-      { return newSmartCursor(Cursor(line, column), insertBehaviour); }
+    inline SmartCursor* newSmartCursor(int line, int column, SmartCursor::InsertBehavior insertBehavior = SmartCursor::MoveOnInsert)
+      { return newSmartCursor(Cursor(line, column), insertBehavior); }
 
     /**
      * Delete all SmartCursor%s from this document, with the exception of those
@@ -242,8 +301,8 @@ class KTEXTEDITOR_EXPORT SmartInterface
      *
      * First, extract the cursors yourself using:
      * \code
-     *  SmartCursor* start = &range->smartStart();
-     *  SmartCursor* end = &range->smartEnd();
+     * SmartCursor* start = &range->smartStart();
+     * SmartCursor* end = &range->smartEnd();
      * \endcode
      *
      * Then, call this function to delete the SmartRange instance.  The underlying text will not be affected.
@@ -440,7 +499,7 @@ class KTEXTEDITOR_EXPORT SmartInterface
     virtual void attributeNotDynamic(Attribute::Ptr a) = 0;
 
   private:
-    bool m_clearOnDocumentReload;
+    class SmartInterfacePrivate* const d;
 };
 
 }
