@@ -62,13 +62,13 @@ class SmartRangeWatcher;
  *
  * Create a new SmartRange like this:
  * \code
- *   // Retrieve the SmartInterface
- *   KTextEditor::SmartInterface* smart =
- *       qobject_cast<KTextEditor::SmartInterface*>( yourDocument );
+ * // Retrieve the SmartInterface
+ * KTextEditor::SmartInterface* smart =
+ *     qobject_cast<KTextEditor::SmartInterface*>( yourDocument );
  *
- *   if ( smart ) {
- *       KTextEditor::SmartRange* range = smart->newSmartRange();
- *   }
+ * if ( smart ) {
+ *     KTextEditor::SmartRange* range = smart->newSmartRange();
+ * }
  * \endcode
  *
  * When finished with a SmartRange, simply delete it.
@@ -687,6 +687,14 @@ class KTEXTEDITOR_EXPORT SmartRange : public Range
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(SmartRange::InsertBehaviors)
 
+/**
+ * kDebug() stream operator.  Writes this range to the debug output in a nicely formatted way.
+ */
+inline kdbgstream& operator<<(kdbgstream& s, const SmartRange& range)
+{
+    return s << static_cast<const Range&>(range);
+}
+
 }
 
 #endif
