@@ -25,8 +25,6 @@
 // gui merging
 #include <kxmlguiclient.h>
 
-#include <kdocument/view.h>
-
 // widget
 #include <qwidget.h>
 
@@ -59,7 +57,7 @@ class Document;
  * method to insert text is insertText(), which inserts the given text
  * at the current cursor position and emits the signal textInserted().
  *
- * Usually a view is created by using KDocument::createView().
+ * Usually a view is created by using Document::createView().
  * Furthermore a view can have a context menu. Set it with setContextMenu()
  * and get it with contextMenu().
  *
@@ -69,14 +67,14 @@ class Document;
  * menu entries and toolbar items) can be merged into the application's GUI
  * (or into a KXMLGUIFactory) by calling
  * \code
- *   // view is of type KTextEditor::View*
- *   mainWindow()->guiFactory()->addClient( view );
+ * // view is of type KTextEditor::View*
+ * mainWindow()->guiFactory()->addClient( view );
  * \endcode
  * You can add only one view as client, so if you have several views, you first
  * have to remove the current view, and then add the new one, like this
  * \code
- *   mainWindow()->guiFactory()->removeClient( currentView );
- *   mainWindow()->guiFactory()->addClient( newView );
+ * mainWindow()->guiFactory()->removeClient( currentView );
+ * mainWindow()->guiFactory()->addClient( newView );
  * \endcode
  *
  * \section view_selection Text Selection
@@ -101,7 +99,7 @@ class Document;
  * of the current cursor position in pixel by using
  * cursorPositionCoordinates(). The signal cursorPositionChanged() is emitted
  * whenever the cursor position changed.
- * 
+ *
  * \section view_mouse_tracking Mouse Tracking
  *
  * It is possible to get notified via the signal mousePositionChanged() for
@@ -139,7 +137,7 @@ class Document;
  *      KXMLGUIClient
  * \author Christoph Cullmann \<cullmann@kde.org\>
  */
-class KTEXTEDITOR_EXPORT View : public KDocument::View
+class KTEXTEDITOR_EXPORT View :  public QWidget, public KXMLGUIClient
 {
   Q_OBJECT
 
@@ -149,14 +147,14 @@ class KTEXTEDITOR_EXPORT View : public KDocument::View
      *
      * Create a view attached to the widget \p parent.
      * \param parent parent widget
-     * \see KDocument::createView()
+     * \see Document::createView()
      */
-    View ( QWidget *parent ) : KDocument::View( parent ) {}
+    View ( QWidget *parent );
 
     /**
      * Virtual destructor.
      */
-    virtual ~View () {}
+    virtual ~View ();
 
   /*
    * Accessor for the document
@@ -169,6 +167,15 @@ class KTEXTEDITOR_EXPORT View : public KDocument::View
      */
     virtual Document *document () const = 0;
 
+    /**
+     * Check whether this view is the document's active view.
+     * This is equal to the code:
+     * \code
+     * document()->activeView() == view
+     * \endcode
+     */
+    bool isActiveView() const;
+
   /*
    * General information about this view
    */
@@ -315,7 +322,7 @@ class KTEXTEDITOR_EXPORT View : public KDocument::View
      * \return \e true on success, otherwise \e false
      * \see cursorPosition()
      */
-    virtual bool setCursorPosition (const Cursor &position) = 0;
+    virtual bool setCursorPosition (Cursor position) = 0;
 
     /**
      * Get the view's current cursor position. A \e TAB character is
@@ -323,7 +330,7 @@ class KTEXTEDITOR_EXPORT View : public KDocument::View
      * \return current cursor position
      * \see setCursorPosition()
      */
-    virtual const Cursor &cursorPosition () const = 0;
+    virtual Cursor cursorPosition () const = 0;
 
     /**
      * Get the current \e virtual cursor position, \e virtual means the
@@ -534,6 +541,9 @@ class KTEXTEDITOR_EXPORT View : public KDocument::View
      * \see textInserted()
      */
     virtual bool insertText (const QString &text);
+
+  private:
+    class ViewPrivate* const d;
 };
 
 }
