@@ -79,6 +79,7 @@ const QStringList &KateCommands::CoreCommands::cmds()
     << "set-indent-width"
     << "set-indent-mode" << "set-auto-indent"
     << "set-line-numbers" << "set-folding-markers" << "set-icon-border"
+    << "set-wrap-cursor"
     << "set-word-wrap" << "set-word-wrap-column"
     << "set-replace-tabs-save" << "set-remove-trailing-space-save"
     << "set-highlight" << "set-show-indent";
@@ -86,29 +87,6 @@ const QStringList &KateCommands::CoreCommands::cmds()
   return l;
 }
 
-QString KateCommands::CoreCommands::name(const QString& cmd) const
-{
-  // TODO: return meaningful names, like "Indent", "Unindent",
-  // "Clean Indentation", "Comment", "Uncomment", "Goto", "Kill line",
-  return QString();
-}
-
-QString KateCommands::CoreCommands::description(const QString& cmd) const
-{
-  // TODO: return meaningful descriptions, like
-  // "Increase indentation level.", "Decrease indentation level.",
-  // "Remove indentation."
-  return QString();
-}
-
-QString KateCommands::CoreCommands::category(const QString& cmd) const
-{
-  // TODO: return meaningful descriptions, like
-  // "Increase indentation level.", "Decrease indentation level.",
-  // "Remove indentation."
-  return QString();
-}
-
 bool KateCommands::CoreCommands::exec(KTextEditor::View *view,
                             const QString &_cmd,
                             QString &errorMsg)
@@ -172,15 +150,9 @@ bool KateCommands::CoreCommands::exec(KTextEditor::View *view,
   }
   else if ( cmd == "set-highlight" )
   {
-    QString val = _cmd.section( ' ', 1 ).toLower();
-    for ( uint i=0; i < v->doc()->hlModeCount(); i++ )
-    {
-      if ( v->doc()->hlModeName( i ).toLower() == val )
-      {
-        v->doc()->setHlMode( i );
-        return true;
-      }
-    }
+    if ( v->doc()->setHighlighting( args.first()) )
+      return true;
+      
     KCC_ERR( i18n("No such highlight '%1'",  args.first() ) );
   }
 
@@ -236,6 +208,7 @@ bool KateCommands::CoreCommands::exec(KTextEditor::View *view,
             cmd == "set-remove-trailing-space" ||
             cmd == "set-show-tabs" ||
             cmd == "set-word-wrap" ||
+            cmd == "set-wrap-cursor" ||
             cmd == "set-replace-tabs-save" ||
             cmd == "set-remove-trailing-space-save" ||
             cmd == "set-show-indent" )
@@ -259,10 +232,14 @@ bool KateCommands::CoreCommands::exec(KTextEditor::View *view,
         setDocFlag( KateDocumentConfig::cfRemoveTrailingDyn, enable, v->doc() );
       else if ( cmd == "set-show-tabs" )
         setDocFlag( KateDocumentConfig::cfShowTabs, enable, v->doc() );
+      else if ( cmd == "set-show-trailing-spaces" )
+        setDocFlag( KateDocumentConfig::cfShowSpaces, enable, v->doc() );
       else if ( cmd == "set-word-wrap" )
         v->doc()->setWordWrap( enable );
       else if ( cmd == "set-remove-trailing-space-save" )
         setDocFlag( KateDocumentConfig::cfRemoveSpaces, enable, v->doc() );
+      else if ( cmd == "set-wrap-cursor" )
+        setDocFlag( KateDocumentConfig::cfWrapCursor, enable, v->doc() );
 
       return true;
     }
@@ -535,30 +512,13 @@ bool KateCommands::SedReplace::exec (KTextEditor::View *view, const QString &cmd
     res += sedMagic(doc, line, find, replace, d, !noCase, repeat);
   }
 
-  msg = i18np("1 replacement done", "%n replacements done",res );
+  msg = i18np("1 replacement done", "%1 replacements done",res );
 
   doc->editEnd();
 
   return true;
 }
 
-QString KateCommands::SedReplace::name (const QString& cmd) const
-{
-  Q_UNUSED(cmd);
-  return i18n("Sed Replace");
-}
-
-QString KateCommands::SedReplace::description (const QString& cmd) const
-{
-  Q_UNUSED(cmd);
-  return i18n("Support for sed-like search and replace.");
-}
-
-QString KateCommands::SedReplace::category (const QString& cmd) const
-{
-  Q_UNUSED(cmd);
-  return i18n("Edit");
-}
 //END SedReplace
 
 //BEGIN Character
@@ -604,29 +564,12 @@ bool KateCommands::Character::exec (KTextEditor::View *view, const QString &_cmd
   return true;
 }
 
-QString KateCommands::Character::name (const QString& cmd) const
-{
-  Q_UNUSED(cmd);
-  return i18n("Insert Character");
-}
-
-QString KateCommands::Character::description (const QString& cmd) const
-{
-  Q_UNUSED(cmd);
-  return i18n("Insert a unicode or ASCII character.");
-}
-
-QString KateCommands::Character::category (const QString& cmd) const
-{
-  Q_UNUSED(cmd);
-  return i18n("Edit");
-}
 //END Character
 
 //BEGIN Date
 bool KateCommands::Date::exec (KTextEditor::View *view, const QString &cmd, QString &)
 {
-  if (cmd.left(4) != "date")
+  if (!cmd.startsWith(QLatin1String("date")))
     return false;
 
   if (QDateTime::currentDateTime().toString(cmd.mid(5, cmd.length()-5)).length() > 0)
@@ -637,23 +580,6 @@ bool KateCommands::Date::exec (KTextEditor::View *view, const QString &cmd, QStr
   return true;
 }
 
-QString KateCommands::Date::name (const QString& cmd) const
-{
-  Q_UNUSED(cmd);
-  return i18n("Insert Date/Time");
-}
-
-QString KateCommands::Date::description (const QString& cmd) const
-{
-  Q_UNUSED(cmd);
-  return i18n("Insert the current Date and time.");
-}
-
-QString KateCommands::Date::category (const QString& cmd) const
-{
-  Q_UNUSED(cmd);
-  return i18n("Edit");
-}
 //END Date
 
 // kate: space-indent on; indent-width 2; replace-tabs on;
