@@ -32,7 +32,7 @@
 #include <klocale.h>
 #include <kfinddialog.h>
 #include <kreplacedialog.h>
-#include <kinstance.h>
+#include <kcomponentdata.h>
 #include <kstaticdeleter.h>
 #include <kfind.h>
 #include <kdebug.h>
@@ -112,9 +112,9 @@ KateDocumentConfig::KateDocumentConfig ()
   m_pluginsSet.fill (true);
 
   // init with defaults from config or really hardcoded ones
-  KConfig *config = KGlobal::config();
+  KSharedConfig::Ptr config = KGlobal::config();
   config->setGroup("Kate Document Defaults");
-  readConfig (config);
+  readConfig (config.data());
 }
 
 KateDocumentConfig::KateDocumentConfig (KateDocument *doc)
@@ -166,7 +166,6 @@ void KateDocumentConfig::readConfig (KConfig *config)
   setUndoSteps(config->readEntry("Undo Steps", 0));
 
   setConfigFlags (config->readEntry("Basic Config Flags", KateDocumentConfig::cfTabIndents
-    | KateDocumentConfig::cfKeepIndentProfile
     | KateDocumentConfig::cfWrapCursor
     | KateDocumentConfig::cfShowTabs
     | KateDocumentConfig::cfSmartHome));
@@ -650,9 +649,9 @@ KateViewConfig::KateViewConfig ()
   s_global = this;
 
   // init with defaults from config or really hardcoded ones
-  KConfig *config = KGlobal::config();
+  KSharedConfig::Ptr config = KGlobal::config();
   config->setGroup("Kate View Defaults");
-  readConfig (config);
+  readConfig (config.data());
 }
 
 KateViewConfig::KateViewConfig (KateView *view)
@@ -1029,6 +1028,7 @@ KateRendererConfig::KateRendererConfig ()
    m_tabMarkerColorSet(true),
    m_iconBarColorSet (true),
    m_lineNumberColorSet (true),
+   m_templateColorsSet(true),
    m_lineMarkerColorSet (m_lineMarkerColor.size()),
    m_renderer (0)
 {
@@ -1038,9 +1038,9 @@ KateRendererConfig::KateRendererConfig ()
   s_global = this;
 
   // init with defaults from config or really hardcoded ones
-  KConfig *config = KGlobal::config();
+  KSharedConfig::Ptr config = KGlobal::config();
   config->setGroup("Kate Renderer Defaults");
-  readConfig (config);
+  readConfig (config.data());
 }
 
 KateRendererConfig::KateRendererConfig (KateRenderer *renderer)
@@ -1058,6 +1058,7 @@ KateRendererConfig::KateRendererConfig (KateRenderer *renderer)
    m_tabMarkerColorSet(false),
    m_iconBarColorSet (false),
    m_lineNumberColorSet (false),
+   m_templateColorsSet(false),
    m_lineMarkerColorSet (m_lineMarkerColor.size()),
    m_renderer (renderer)
 {
@@ -1188,6 +1189,14 @@ void KateRendererConfig::setSchemaInternal( int schema )
   m_font = config->readEntry("Font", f);
   m_fontMetrics = QFontMetrics(m_font);
   m_fontSet = true;
+
+  kDebug()<<"Loading template colors "<<this<<endl;
+  m_templateBackgroundColor=config->readEntry(QString("Color Template Background"),QColor(0xcc,0xcc,0xcc));
+  m_templateEditablePlaceholderColor = config->readEntry(QString("Color Template Editable Placeholder"),QColor(0xcc,0xff,0xcc));
+  m_templateFocusedEditablePlaceholderColor=config->readEntry(QString("Color Template Focused Editable Placeholder"),QColor(0x66,0xff,0x66));
+  m_templateNotEditablePlaceholderColor=config->readEntry(QString("Color Template Not Editable Placeholder"),QColor(0xff,0xcc,0xcc));
+
+  m_templateColorsSet=true;
 }
 
 const QFont& KateRendererConfig::font() const
@@ -1391,6 +1400,33 @@ void KateRendererConfig::setIconBarColor (const QColor &col)
   configEnd ();
 }
 
+
+const QColor &KateRendererConfig::templateBackgroundColor() const {
+  if (m_templateColorsSet || isGlobal())
+    return m_templateBackgroundColor;
+
+  return s_global->templateBackgroundColor();
+}
+const QColor &KateRendererConfig::templateEditablePlaceholderColor() const {
+  if (m_templateColorsSet || isGlobal())  
+    return m_templateEditablePlaceholderColor;
+
+  return s_global->templateEditablePlaceholderColor();
+}
+const QColor &KateRendererConfig::templateFocusedEditablePlaceholderColor() const {
+  if (m_templateColorsSet || isGlobal())  
+    return m_templateFocusedEditablePlaceholderColor;
+
+  return s_global->templateFocusedEditablePlaceholderColor();
+}
+const QColor &KateRendererConfig::templateNotEditablePlaceholderColor() const {
+  if (m_templateColorsSet || isGlobal())  
+    return m_templateNotEditablePlaceholderColor;
+  
+  return s_global->templateNotEditablePlaceholderColor();
+}
+
+
 const QColor& KateRendererConfig::lineNumberColor() const
 {
   if (m_lineNumberColorSet || isGlobal())
