@@ -2,6 +2,7 @@
    Copyright (C) 2002, 2003 Anders Lund <anders.lund@lund.tdcadsl.dk>
    Copyright (C) 2003 Christoph Cullmann <cullmann@kde.org>
    Copyright (C) 2001 Joseph Wenninger <jowenn@kde.org>
+   Copyright (C) 2006 Dominik Haumann <dhdev@gmx.de>
 
    Based on work of:
      Copyright (C) 1999 Jochen Wilhelmy <digisnap@cs.tu-berlin.de>
@@ -40,6 +41,7 @@
 #include "ui_cursorconfigwidget.h"
 #include "ui_editconfigwidget.h"
 #include "ui_hlconfigwidget.h"
+#include "ui_indentationconfigwidget.h"
 #include "ui_opensaveconfigwidget.h"
 
 #include <ktexteditor/plugin.h>
@@ -76,7 +78,7 @@
 #include <krun.h>
 #include <kseparator.h>
 #include <kstandarddirs.h>
-#include <ktempfile.h>
+#include <ktemporaryfile.h>
 #include <kpushbutton.h>
 #include <kvbox.h>
 #include <kactioncollection.h>
@@ -102,6 +104,8 @@
 #include <qtextstream.h>
 #include <qtoolbutton.h>
 #include <QComboBox>
+#include <QWhatsThis>
+#include <QKeyEvent>
 
 // trailing slash is important
 #define HLDOWNLOADPATH "http://kate.kde.org/syntax/"
@@ -133,120 +137,18 @@ void KateConfigPage::somethingHasChanged ()
 //END KateConfigPage
 
 //BEGIN KateIndentConfigTab
-const int KateIndentConfigTab::flags[] = {
-    KateDocumentConfig::cfKeepIndentProfile,
-    KateDocumentConfig::cfKeepExtraSpaces,
-    KateDocumentConfig::cfBackspaceIndents,
-    KateDocumentConfig::cfDoxygenAutoTyping,
-};
-
 KateIndentConfigTab::KateIndentConfigTab(QWidget *parent)
   : KateConfigPage(parent)
 {
-  QVBoxLayout *layout = new QVBoxLayout(this);
-  layout->setMargin(0);
-  layout->setSpacing( KDialog::spacingHint() );
-  int configFlags = KateDocumentConfig::global()->configFlags();
+  ui = new Ui::IndentationConfigWidget();
+  ui->setupUi( this );
 
-  QGroupBox *gbAuto = new QGroupBox(i18n("Automatic Indentation"), this);
-  QVBoxLayout *vb = new QVBoxLayout (gbAuto);
+  ui->cmbMode->addItems (KateAutoIndent::listModes());
 
-  QHBoxLayout *indentLayout = new QHBoxLayout();
-  vb->addItem(indentLayout);
-  indentLayout->setSpacing(KDialog::spacingHint());
-  QLabel *indentLabel = new QLabel(i18n("Default &Indentation mode:"),gbAuto);
-  indentLayout->addWidget(indentLabel);
-  m_indentMode = new KComboBox (gbAuto);
-  indentLayout->addWidget(m_indentMode);
-  m_indentMode->addItems (KateAutoIndent::listModes());
-  indentLabel->setBuddy(m_indentMode);
-  m_configPage = new QPushButton(SmallIconSet("configure"), i18n("Configure..."), gbAuto);
-  indentLayout->addWidget(m_configPage);
+  ui->label->setTextInteractionFlags(Qt::LinksAccessibleByMouse | Qt::LinksAccessibleByKeyboard);
+  connect(ui->label, SIGNAL(linkActivated(const QString&)), this, SLOT(showWhatsThis(const QString&)));
 
-  opt[3] = new QCheckBox(i18n("Insert leading Doxygen \"*\" when typing"), gbAuto);
-  vb->addWidget (opt[3]);
-
-  QGroupBox *gbSpaces = new QGroupBox(i18n("Indentation with Spaces"), this);
-  vb = new QVBoxLayout (gbSpaces);
-
-  indentationWidth = new KIntNumInput(KateDocumentConfig::global()->indentationWidth(),gbSpaces);
-  indentationWidth->setRange(1, 16, 1, false);
-  indentationWidth->setLabel(i18n("Number of spaces:"), Qt::AlignVCenter);
-  vb->addWidget (indentationWidth);
-
-  opt[0] = new QCheckBox(i18n("Keep indent &profile"), this);
-  opt[1] = new QCheckBox(i18n("&Keep extra spaces"), this);
-
-  QGroupBox *keys = new QGroupBox(i18n("Keys to Use"), this);
-  vb = new QVBoxLayout (keys);
-  opt[2] = new QCheckBox(i18n("&Backspace key indents"), keys);
-  vb->addWidget (opt[2]);
-
-  m_tabs = new QGroupBox(i18n("Tab Key Mode"), this );
-  QVBoxLayout *tablayout=new QVBoxLayout(m_tabs);
-
-  tablayout->addWidget( rb1=new QRadioButton( i18n("Always advance to the next tab position"), m_tabs ));
-  tablayout->addWidget( rb2=new QRadioButton( i18n("Always indent current &line"), m_tabs ));
-  tablayout->addWidget( rb3=new QRadioButton( i18n("Indent if in leading blank space"), m_tabs ));
-
-  opt[0]->setChecked(configFlags & flags[0]);
-  opt[1]->setChecked(configFlags & flags[1]);
-  opt[2]->setChecked(configFlags & flags[2]);
-  opt[3]->setChecked(configFlags & flags[3]);
-
-  layout->addWidget(gbAuto);
-  layout->addWidget(gbSpaces);
-  layout->addWidget(opt[0]);
-  layout->addWidget(opt[1]);
-  layout->addWidget(keys);
-  layout->addWidget(m_tabs, 0);
-
-  layout->addStretch();
-
-  // What is this? help
-  opt[1]->setWhatsThis( i18n(
-        "Indentations of more than the selected number of spaces will not be "
-        "shortened."));
-  opt[2]->setWhatsThis( i18n(
-        "This allows the <b>Backspace</b> key to be used to decrease the "
-        "indentation level."));
-  opt[3]->setWhatsThis( i18n(
-        "Automatically inserts a leading \"*\" while typing within a Doxygen "
-        "style comment."));
-  indentationWidth->setWhatsThis( i18n("The number of spaces to indent with."));
-
-  m_configPage->setWhatsThis( i18n(
-        "If this button is enabled, additional indenter specific options are "
-        "available and can be configured in an extra dialog.") );
-
-  m_indentMode->setWhatsThis( i18n(
-        "The specified indentation mode will be used for all new documents. Be aware "
-        "that it is also possible to set the indentation mode with document variables, "
-        "filetypes or a .kateconfig file." ) );
-
-  rb1->setWhatsThis( i18n(
-        "If this option is selected, the <b>Tab</b> key always inserts "
-        "whitespace so that the next tab postion is reached. "
-        "If the option <b>Insert spaces instead of tabulators</b> "
-        "in the section <b>Editing</b> is enabled, spaces are inserted; "
-        "otherwise, a single tabulator is inserted.") );
-  rb2->setWhatsThis( i18n(
-        "If this option is selected, the <b>Tab</b> key always indents "
-        "the current line by the number of character positions specified "
-        "in <b>Number of spaces</b>.") );
-  rb3->setWhatsThis( i18n(
-        "If this option is selected, the <b>Tab</b> key either indents "
-        "the current line or advances to the next tab position.<p>"
-        "If the insertion point is at or before the first non-space "
-        "character in the line, or if there is a selection, "
-        "the current line is indented by the number of character "
-        "positions specified in <b>Number of spaces</b>.<p>"
-        "If the insertion point is located after the first non-space "
-        "character in the line and there is no selection, "
-        "whitespace is inserted so that the next tab postion is reached: "
-        "if the option <b>Insert spaces instead of tabulators</b> "
-        "in the section <b>Editing</b> is enabled, spaces are inserted; "
-        "otherwise, a single tabulator is inserted.") );
+  // What's This? help can be found in the ui file
 
   reload ();
 
@@ -254,65 +156,22 @@ KateIndentConfigTab::KateIndentConfigTab(QWidget *parent)
   // after initial reload, connect the stuff for the changed () signal
   //
 
-  connect(m_indentMode, SIGNAL(activated(int)), this, SLOT(slotChanged()));
-  connect(m_indentMode, SIGNAL(activated(int)), this, SLOT(indenterSelected(int)));
+  connect(ui->cmbMode, SIGNAL(activated(int)), this, SLOT(slotChanged()));
 
-  connect( opt[0], SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
-  connect( opt[1], SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
-  connect( opt[2], SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
-  connect( opt[3], SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
+  connect(ui->chkKeepExtraSpaces, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(ui->chkIndentPaste, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(ui->chkBackspaceUnindents, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
 
-  connect(indentationWidth, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
+  connect(ui->sbIndentWidth, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
 
-  connect(rb1, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-  connect(rb2, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-  connect(rb3, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-
-  connect(m_configPage, SIGNAL(clicked()), this, SLOT(configPage()));
+  connect(ui->rbTabAdvances, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(ui->rbTabIndents, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(ui->rbTabSmart, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
 }
 
-void KateIndentConfigTab::indenterSelected (int index)
+void KateIndentConfigTab::showWhatsThis(const QString& text)
 {
-#ifdef __GNUC__
-#warning fixme
-#endif
-  /*if (index == KateDocumentConfig::imCStyle || index == KateDocumentConfig::imCSAndS)
-    opt[3]->setEnabled(true);
-  else
-    opt[3]->setEnabled(false);
-*/
-  m_configPage->setEnabled( KateAutoIndent::hasConfigPage(index) );
-}
-
-void KateIndentConfigTab::configPage()
-{
-  uint index = m_indentMode->currentIndex();
-  if ( KateAutoIndent::hasConfigPage(index) )
-  {
-    KDialog dlg( this );
-    dlg.setObjectName( "indenter_config_dialog" );
-    dlg.setModal( true );
-    dlg.setCaption( i18n("Configure Indenter") );
-    dlg.setButtons( KDialog::Ok | KDialog::Cancel );
-    dlg.setDefaultButton( KDialog::Cancel );
-    dlg.showButtonSeparator( true );
-
-    KVBox *box = new KVBox(&dlg);
-    box->setSpacing( KDialog::spacingHint() );
-    dlg.setMainWidget(box);
-    new QLabel("<qt><b>" + KateAutoIndent::modeDescription(index) + "</b></qt>", box);
-    new KSeparator(box);
-
-    IndenterConfigPage* page = KateAutoIndent::configPage(box, index);
-
-    if (!page) return;
-    box->setStretchFactor(page, 1);
-
-    connect( &dlg, SIGNAL(okClicked()), page, SLOT(apply()) );
-
-    dlg.resize(400, 300);
-    dlg.exec();
-  }
+  QWhatsThis::showText(QCursor::pos(), text);
 }
 
 void KateIndentConfigTab::apply ()
@@ -324,22 +183,23 @@ void KateIndentConfigTab::apply ()
 
   KateDocumentConfig::global()->configStart ();
 
-  int configFlags, z;
+  uint configFlags = KateDocumentConfig::global()->configFlags();
 
-  configFlags = KateDocumentConfig::global()->configFlags();
-  for (z = 0; z < numFlags; z++) {
-    configFlags &= ~flags[z];
-    if (opt[z]->isChecked()) configFlags |= flags[z];
-  }
+  configFlags &= ~KateDocumentConfig::cfKeepExtraSpaces;
+  configFlags &= ~KateDocumentConfig::cfIndentPastedText;
+  configFlags &= ~KateDocumentConfig::cfBackspaceIndents;
+
+  if (ui->chkKeepExtraSpaces->isChecked()) configFlags |= KateDocumentConfig::cfKeepExtraSpaces;
+  if (ui->chkIndentPaste->isChecked()) configFlags |= KateDocumentConfig::cfIndentPastedText;
+  if (ui->chkBackspaceUnindents->isChecked()) configFlags |= KateDocumentConfig::cfBackspaceIndents;
 
   KateDocumentConfig::global()->setConfigFlags(configFlags);
-  KateDocumentConfig::global()->setIndentationWidth(indentationWidth->value());
+  KateDocumentConfig::global()->setIndentationWidth(ui->sbIndentWidth->value());
+  KateDocumentConfig::global()->setIndentationMode(KateAutoIndent::modeName(ui->cmbMode->currentIndex()));
 
-  KateDocumentConfig::global()->setIndentationMode(KateAutoIndent::modeName(m_indentMode->currentIndex()));
-
-  if (rb1->isChecked())
+  if (ui->rbTabAdvances->isChecked())
     KateDocumentConfig::global()->setTabHandling( KateDocumentConfig::tabInsertsTab );
-  else if (rb2->isChecked())
+  else if (ui->rbTabIndents->isChecked())
     KateDocumentConfig::global()->setTabHandling( KateDocumentConfig::tabIndents );
   else
     KateDocumentConfig::global()->setTabHandling( KateDocumentConfig::tabSmart );
@@ -349,13 +209,18 @@ void KateIndentConfigTab::apply ()
 
 void KateIndentConfigTab::reload ()
 {
-  rb1->setChecked( KateDocumentConfig::global()->tabHandling() == KateDocumentConfig::tabInsertsTab );
-  rb2->setChecked( KateDocumentConfig::global()->tabHandling() == KateDocumentConfig::tabIndents );
-  rb3->setChecked( KateDocumentConfig::global()->tabHandling() == KateDocumentConfig::tabSmart );
+  uint configFlags = KateDocumentConfig::global()->configFlags();
 
-  m_indentMode->setCurrentIndex (KateAutoIndent::modeNumber (KateDocumentConfig::global()->indentationMode()));
+  ui->sbIndentWidth->setValue(KateDocumentConfig::global()->indentationWidth());
+  ui->chkKeepExtraSpaces->setChecked(configFlags & KateDocumentConfig::cfKeepExtraSpaces);
+  ui->chkIndentPaste->setChecked(configFlags & KateDocumentConfig::cfIndentPastedText);
+  ui->chkBackspaceUnindents->setChecked(configFlags & KateDocumentConfig::cfBackspaceIndents);
 
-  indenterSelected (m_indentMode->currentIndex());
+  ui->rbTabAdvances->setChecked( KateDocumentConfig::global()->tabHandling() == KateDocumentConfig::tabInsertsTab );
+  ui->rbTabIndents->setChecked( KateDocumentConfig::global()->tabHandling() == KateDocumentConfig::tabIndents );
+  ui->rbTabSmart->setChecked( KateDocumentConfig::global()->tabHandling() == KateDocumentConfig::tabSmart );
+
+  ui->cmbMode->setCurrentIndex (KateAutoIndent::modeNumber (KateDocumentConfig::global()->indentationMode()));
 }
 //END KateIndentConfigTab
 
@@ -363,7 +228,7 @@ void KateIndentConfigTab::reload ()
 KateSelectConfigTab::KateSelectConfigTab(QWidget *parent)
   : KateConfigPage(parent)
 {
-  int configFlags = KateDocumentConfig::global()->configFlags();
+  uint configFlags = KateDocumentConfig::global()->configFlags();
 
   ui = new Ui::CursorConfigWidget();
   ui->setupUi( this );
@@ -402,7 +267,7 @@ void KateSelectConfigTab::apply ()
   KateViewConfig::global()->configStart ();
   KateDocumentConfig::global()->configStart ();
 
- int configFlags = KateDocumentConfig::global()->configFlags();
+  uint configFlags = KateDocumentConfig::global()->configFlags();
 
   configFlags &= ~KateDocumentConfig::cfSmartHome;
   configFlags &= ~KateDocumentConfig::cfWrapCursor;
@@ -432,7 +297,7 @@ void KateSelectConfigTab::reload ()
 KateEditConfigTab::KateEditConfigTab(QWidget *parent)
   : KateConfigPage(parent)
 {
-  int configFlags = KateDocumentConfig::global()->configFlags();
+  uint configFlags = KateDocumentConfig::global()->configFlags();
 
   ui = new Ui::EditConfigWidget();
   ui->setupUi( this );
@@ -443,6 +308,9 @@ KateEditConfigTab::KateEditConfigTab(QWidget *parent)
   ui->chkShowTabs->setChecked( configFlags & KateDocumentConfig::cfShowTabs );
   connect(ui->chkShowTabs, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
 
+  ui->chkShowSpaces->setChecked( configFlags & KateDocumentConfig::cfShowSpaces );
+  connect(ui->chkShowSpaces, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+
   ui->sbTabWidth->setValue( KateDocumentConfig::global()->tabWidth() );
   connect(ui->sbTabWidth, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
 
@@ -487,15 +355,17 @@ void KateEditConfigTab::apply ()
   KateViewConfig::global()->configStart ();
   KateDocumentConfig::global()->configStart ();
 
-  int configFlags = KateDocumentConfig::global()->configFlags();
+  uint configFlags = KateDocumentConfig::global()->configFlags();
 
   configFlags &= ~KateDocumentConfig::cfAutoBrackets;
   configFlags &= ~KateDocumentConfig::cfShowTabs;
+  configFlags &= ~KateDocumentConfig::cfShowSpaces;
   configFlags &= ~KateDocumentConfig::cfReplaceTabsDyn;
   configFlags &= ~KateDocumentConfig::cfRemoveTrailingDyn;
 
   if (ui->chkAutoBrackets->isChecked()) configFlags |= KateDocumentConfig::cfAutoBrackets;
   if (ui->chkShowTabs->isChecked()) configFlags |= KateDocumentConfig::cfShowTabs;
+  if (ui->chkShowSpaces->isChecked()) configFlags |= KateDocumentConfig::cfShowSpaces;
   if (ui->chkReplaceTabs->isChecked()) configFlags |= KateDocumentConfig::cfReplaceTabsDyn;
   if (ui->chkRemoveTrailingSpaces->isChecked()) configFlags |= KateDocumentConfig::cfRemoveTrailingDyn;
 
@@ -707,7 +577,7 @@ void KateSaveConfigTab::apply()
 
   KateDocumentConfig::global()->setSearchDirConfigDepth(ui->sbConfigFileSearchDepth->value());
 
-  int configFlags = KateDocumentConfig::global()->configFlags();
+  uint configFlags = KateDocumentConfig::global()->configFlags();
 
   configFlags &= ~KateDocumentConfig::cfRemoveSpaces; // clear flag
   if (ui->chkRemoveTrailingSpaces->isChecked()) configFlags |= KateDocumentConfig::cfRemoveSpaces; // set flag if checked
@@ -752,7 +622,7 @@ void KateSaveConfigTab::reload()
   ui->cmbEOL->setCurrentIndex(KateDocumentConfig::global()->eol());
   ui->chkDetectEOL->setChecked(KateDocumentConfig::global()->allowEolDetection());
 
-  const int configFlags = KateDocumentConfig::global()->configFlags();
+  const uint configFlags = KateDocumentConfig::global()->configFlags();
   ui->chkRemoveTrailingSpaces->setChecked(configFlags & KateDocumentConfig::cfRemoveSpaces);
   ui->sbConfigFileSearchDepth->setValue(KateDocumentConfig::global()->searchDirConfigDepth());
 
@@ -785,29 +655,16 @@ class KatePartPluginListItem : public QTreeWidgetItem
     KatePartPluginListItem(bool checked, uint i, const QString &name, QTreeWidget *parent);
     uint pluginIndex () const { return index; }
 
-  protected:
-    void stateChange(bool);
-
   private:
     uint index;
-    bool silentStateChange;
 };
 
 KatePartPluginListItem::KatePartPluginListItem(bool checked, uint i, const QString &name, QTreeWidget *parent)
   : QTreeWidgetItem(parent)
   , index(i)
-  , silentStateChange(false)
 {
   setText(0, name);
-  silentStateChange = true;
   setCheckState(0, checked ? Qt::Checked : Qt::Unchecked);
-  silentStateChange = false;
-}
-
-void KatePartPluginListItem::stateChange(bool b)
-{
-  if(!silentStateChange)
-    static_cast<KatePartPluginListView *>(treeWidget())->stateChanged(this, b);
 }
 //END
 
@@ -816,11 +673,6 @@ KatePartPluginListView::KatePartPluginListView(QWidget *parent)
   : QTreeWidget(parent)
 {
 }
-
-void KatePartPluginListView::stateChanged(KatePartPluginListItem *item, bool b)
-{
-  emit stateChange(item, b);
-}
 //END
 
 //BEGIN KatePartPluginConfigPage
@@ -833,6 +685,7 @@ KatePartPluginConfigPage::KatePartPluginConfigPage (QWidget *parent) : KateConfi
   listView = new KatePartPluginListView(this);
   listView->setColumnCount(2);
   listView->setHeaderLabels(QStringList() << i18n("Name") << i18n("Comment"));
+  listView->setRootIsDecorated(false);
 
   grid->addWidget(listView, 0, 0);
 
@@ -844,6 +697,7 @@ KatePartPluginConfigPage::KatePartPluginConfigPage (QWidget *parent) : KateConfi
 
     m_items.append (item);
   }
+  listView->resizeColumnToContents(0);
 
   // configure button
 
@@ -853,9 +707,7 @@ KatePartPluginConfigPage::KatePartPluginConfigPage (QWidget *parent) : KateConfi
   connect( btnConfigure, SIGNAL(clicked()), this, SLOT(slotConfigure()) );
 
   connect( listView, SIGNAL(currentItemChanged(QTreeWidgetItem*, QTreeWidgetItem*)), this, SLOT(slotCurrentChanged(QTreeWidgetItem*)) );
-  connect( listView, SIGNAL(stateChange(KatePartPluginListItem *, bool)),
-    this, SLOT(slotStateChanged(KatePartPluginListItem *, bool)));
-  connect(listView, SIGNAL(stateChange(KatePartPluginListItem *, bool)), this, SLOT(slotChanged()));
+  connect( listView, SIGNAL(itemChanged (QTreeWidgetItem *, int)), this, SLOT(slotChanged()));
 }
 
 KatePartPluginConfigPage::~KatePartPluginConfigPage ()
@@ -881,12 +733,6 @@ void KatePartPluginConfigPage::apply ()
   KateDocumentConfig::global()->configEnd ();
 }
 
-void KatePartPluginConfigPage::slotStateChanged( KatePartPluginListItem *item, bool b )
-{
-  if ( b )
-    slotCurrentChanged( (QTreeWidgetItem*)item );
-}
-
 void KatePartPluginConfigPage::slotCurrentChanged( QTreeWidgetItem* i )
 {
   KatePartPluginListItem *item = static_cast<KatePartPluginListItem *>(i);
@@ -1056,13 +902,13 @@ void KateHlConfigPage::showMTDlg()
 {
   QString text = i18n("Select the MimeTypes you want highlighted using the '%1' syntax highlight rules.\nPlease note that this will automatically edit the associated file extensions as well.",  ui->cmbHl->currentText() );
   QStringList list = ui->edtMimeTypes->text().split( QRegExp("\\s*;\\s*") );
-  KMimeTypeChooserDialog *d = new KMimeTypeChooserDialog( i18n("Select Mime Types"), text, list, "text", this );
+  KMimeTypeChooserDialog d( i18n("Select Mime Types"), text, list, "text", this );
 
-  if ( d->exec() == KDialog::Accepted ) {
+  if ( d.exec() == KDialog::Accepted ) {
     // do some checking, warn user if mime types or patterns are removed.
     // if the lists are empty, and the fields not, warn.
-    ui->edtFileExtensions->setText(d->chooser()->patterns().join(";"));
-    ui->edtMimeTypes->setText(d->chooser()->mimeTypes().join(";"));
+    ui->edtFileExtensions->setText(d.chooser()->patterns().join(";"));
+    ui->edtMimeTypes->setText(d.chooser()->mimeTypes().join(";"));
   }
 }
 //END KateHlConfigPage
@@ -1073,7 +919,7 @@ KateHlDownloadDialog::KateHlDownloadDialog(QWidget *parent, const char *name, bo
 {
   setCaption( i18n("Highlight Download") );
   setButtons( User1 | Close );
-  setButtonGuiItem( User1, i18n("&Install") );
+  setButtonGuiItem( User1, KGuiItem(i18n("&Install")) );
   setDefaultButton( User1 );
   setObjectName( name );
   setModal( modal );
@@ -1091,7 +937,7 @@ KateHlDownloadDialog::KateHlDownloadDialog(QWidget *parent, const char *name, bo
   //list->setAllColumnsShowFocus(true);
 
   new QLabel(i18n("<b>Note:</b> New versions are selected automatically."), vbox);
-  setButtonIcon(User1, SmallIconSet("ok"));
+  setButtonIcon(User1, KIcon("ok"));
 
   transferJob = KIO::get(
     KUrl(QString(HLDOWNLOADPATH)
@@ -1102,6 +948,7 @@ KateHlDownloadDialog::KateHlDownloadDialog(QWidget *parent, const char *name, bo
     this, SLOT(listDataReceived(KIO::Job *, const QByteArray &)));
 //        void data( KIO::Job *, const QByteArray &data);
   resize(450, 400);
+  connect(this,SIGNAL(user1Clicked()),this,SLOT(slotUser1()));
 }
 
 KateHlDownloadDialog::~KateHlDownloadDialog(){}
@@ -1188,38 +1035,60 @@ void KateHlDownloadDialog::slotUser1()
 }
 //END KateHlDownloadDialog
 
-//BEGIN KateGotoLineDialog
-KateGotoLineDialog::KateGotoLineDialog(QWidget *parent, int line, int max)
-  : KDialog( parent )
+//BEGIN KateGotoBar
+KateGotoBar::KateGotoBar(KateViewBar *parent)
+  : KateViewBarWidget( parent )
 {
-  setModal( true );
-  setCaption( i18n("Go to Line") );
-  setButtons( Ok | Cancel );
-  setDefaultButton( Ok );
+  QHBoxLayout *topLayout = new QHBoxLayout( centralWidget() );
+  topLayout->setMargin(0);
+  //topLayout->setSpacing(spacingHint());
+  gotoRange = new KIntNumInput(1, centralWidget());
+  gotoRange->setRange(1, 100);
+  gotoRange->setEditFocus(true);
 
-  QWidget *page = new QWidget(this);
-  setMainWidget(page);
+  QLabel *label = new QLabel(i18n("&Go to line:"), centralWidget() );
+  label->setBuddy(gotoRange);
 
-  QVBoxLayout *topLayout = new QVBoxLayout( page);
-  topLayout->setMargin( 0);
-  topLayout->setSpacing(spacingHint());
-  e1 = new KIntNumInput(line,page);
-  e1->setRange(1, max);
-  e1->setEditFocus(true);
+  btnOK = new QToolButton();
+  btnOK->setAutoRaise(true);
+  btnOK->setIcon(QIcon(SmallIcon("goto")));
+  btnOK->setText(i18n("Go"));
+  btnOK->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
+  connect(btnOK, SIGNAL(clicked()), this, SLOT(gotoLine()));
 
-  QLabel *label = new QLabel(i18n("&Go to line:"), page );
-  label->setBuddy(e1);
   topLayout->addWidget(label);
-  topLayout->addWidget(e1);
-  topLayout->addSpacing(spacingHint()); // A little bit extra space
-  topLayout->addStretch(10);
-  e1->setFocus();
+  topLayout->addWidget(gotoRange, 1);
+  topLayout->addWidget(btnOK);
 }
 
-int KateGotoLineDialog::getLine() {
-  return e1->value();
+void KateGotoBar::showBar()
+{
+  KateView* view = viewBar()->view();
+  gotoRange->setMaximum(view->doc()->lines());
+  if (!isVisible())
+    gotoRange->setValue(view->cursorPosition().line() + 1);
+  gotoRange->setEditFocus(true);
+
+  KateViewBarWidget::showBar();
 }
-//END KateGotoLineDialog
+
+void KateGotoBar::keyPressEvent(QKeyEvent* event)
+{
+  int key = event->key();
+  if (key == Qt::Key_Return || key == Qt::Key_Enter) {
+    gotoLine();
+    return;
+  }
+  KateViewBarWidget::keyPressEvent(event);
+}
+
+void KateGotoBar::gotoLine()
+{
+  viewBar()->view()->setCursorPosition( KTextEditor::Cursor(gotoRange->value() - 1, 0) );
+  viewBar()->view()->setFocus();
+  hideBar();
+}
+//END KateGotoBar
 
 //BEGIN KateModOnHdPrompt
 KateModOnHdPrompt::KateModOnHdPrompt( KateDocument *doc,
@@ -1311,19 +1180,26 @@ void KateModOnHdPrompt::slotDiff()
 void KateModOnHdPrompt::slotPRead( KProcIO *p)
 {
   // create a file for the diff if we haven't one already
-  if ( ! m_tmpfile )
-    m_tmpfile = new KTempFile();
+  if ( ! m_tmpfile ) {
+    m_tmpfile = new KTemporaryFile();
+    m_tmpfile->setAutoRemove(false);
+    m_tmpfile->open();
+  }
+
+  QTextStream stream(m_tmpfile);
+
   // put all the data we have in it
   QString stmp;
   bool readData = false;
   while ( p->readln( stmp, false ) > -1 )
   {
-    *m_tmpfile->textStream() << stmp << endl;
+    stream << stmp << endl;
     readData = true;
   }
+  stream.flush();
 
   // dominik: only ackRead(), when we *really* read data, otherwise, this slot
-  // is called initity times, which leads to a crash (#123887)
+  // is called infinity times, which leads to a crash (#123887)
   if( readData )
     p->ackRead();
 }
@@ -1345,6 +1221,8 @@ void KateModOnHdPrompt::slotPDone( KProcess *p )
                         i18n("The diff command failed. Please make sure that "
                              "diff(1) is installed and in your PATH."),
                         i18n("Error Creating Diff") );
+    delete m_tmpfile;
+    m_tmpfile = 0;
     return;
   }
 
@@ -1356,7 +1234,7 @@ void KateModOnHdPrompt::slotPDone( KProcess *p )
     return;
   }
 
-  KRun::runUrl( KUrl::fromPath(m_tmpfile->name()), "text/x-diff", this, true );
+  KRun::runUrl( KUrl::fromPath(m_tmpfile->fileName()), "text/x-diff", this, true );
   delete m_tmpfile;
   m_tmpfile = 0;
 }
@@ -1379,7 +1257,7 @@ void KateModOnHdPrompt::slotButtonClicked(int button)
            "will overwrite the file on disk; if you do not save then the disk "
            "file (if present) is what you have."),
            i18n("You Are on Your Own"),
-           KStdGuiItem::cont(),
+           KStandardGuiItem::cont(),
            "kate_ignore_modonhd" ) != KMessageBox::Continue )
         return;
       done( Ignore );
