@@ -2,6 +2,7 @@
    Copyright (C) 2001-2004 Christoph Cullmann <cullmann@kde.org>
    Copyright (C) 2001 Joseph Wenninger <jowenn@kde.org>
    Copyright (C) 1999 Jochen Wilhelmy <digisnap@cs.tu-berlin.de>
+   Copyright (C) 2006 Hamish Rodda <rodda@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -21,7 +22,15 @@
 #ifndef _KATE_DOCUMENT_H_
 #define _KATE_DOCUMENT_H_
 
-#include "katetextline.h"
+#include <QLinkedList>
+#include <QMap>
+#include <QDateTime>
+#include <QClipboard>
+#include <QStack>
+
+#include <kmimetype.h>
+#include <klocale.h>
+#include <kshortcut.h>
 
 #include <ktexteditor/document.h>
 #include <ktexteditor/sessionconfiginterface.h>
@@ -33,23 +42,16 @@
 #include <ktexteditor/smartinterface.h>
 #include <ktexteditor/rangefeedback.h>
 
-#include <kmimetype.h>
-#include <klocale.h>
-#include <kshortcut.h>
+#include "katetextline.h"
+#include "kateautoindent.h"
+#include "katenamespace.h"
 
-#include <qlinkedlist.h>
-#include <qmap.h>
-#include <qdatetime.h>
-#include <QClipboard>
-
-namespace KTextEditor { class Plugin; }
+namespace KTextEditor { class Plugin; class Attribute; }
 
 namespace KIO { class TransferJob; }
 
 class KateUndoGroup;
 class KateCmd;
-class KTextEditor::Attribute;
-class KateAutoIndent;
 class KateCodeFoldingTree;
 class KateBuffer;
 class KateView;
@@ -62,7 +64,7 @@ class KatePartPluginItem;
 class KatePartPluginInfo;
 class KateSmartManager;
 
-class KTempFile;
+class KTemporaryFile;
 
 class QTimer;
 
@@ -95,7 +97,7 @@ class KateDocument : public KTextEditor::Document,
                   QWidget *parentWidget = 0, QObject * = 0);
     ~KateDocument ();
 
-    bool closeURL();
+    bool closeUrl();
 
     KTextEditor::Editor *editor ();
 
@@ -146,8 +148,8 @@ class KateDocument : public KTextEditor::Document,
   // KTextEditor::Document stuff
   //
   public:
-    KDocument::View *createView( QWidget *parent );
-    const QList<KDocument::View*> &views () const;
+    KTextEditor::View *createView( QWidget *parent );
+    const QList<KTextEditor::View*> &views () const;
 
     virtual KTextEditor::View* activeView() const { return m_activeView; }
     // Invalid covariant returns my a$$... for some reason gcc won't let me return a KateView above!
@@ -158,7 +160,7 @@ class KateDocument : public KTextEditor::Document,
 
   private:
     QLinkedList<KateView*> m_views;
-    QList<KDocument::View*> m_textEditViews;
+    QList<KTextEditor::View*> m_textEditViews;
     KTextEditor::View *m_activeView;
 
   //
@@ -193,7 +195,7 @@ class KateDocument : public KTextEditor::Document,
     int lineLength(int line) const;
 
   Q_SIGNALS:
-    void charactersSemiInteractivelyInserted(int ,int ,const QString&);
+    void charactersSemiInteractivelyInserted(const KTextEditor::Cursor& position, const QString& text);
 
   public:
 //BEGIN editStart/editEnd (start, end, undo, cursor update, view update)
@@ -202,16 +204,19 @@ class KateDocument : public KTextEditor::Document,
      * them.
      * @param withUndo if true, add undo history
      */
-    void editStart (bool withUndo = true, KTextEditor::View *view = 0);
+    void editStart (bool withUndo = true, Kate::EditSource editSource = Kate::NoEditSource);
     /** Same as editStart() with undo */
-    void editBegin (KTextEditor::View *view = 0) { editStart(true, view); }
+    void editBegin (Kate::EditSource editSource = Kate::NoEditSource) { editStart(true, editSource); }
     /**
      * End a editor operation.
      * @see editStart()
      */
     void editEnd ();
 
-    bool startEditing (KTextEditor::View *view = 0) { editStart (true, view); return true; }
+    void pushEditState();
+    void popEditState();
+
+    bool startEditing () { editStart (true, Kate::ThirdPartyEdit); return true; }
     bool endEditing () { editEnd (); return true; }
 
 //END editStart/editEnd
@@ -224,7 +229,7 @@ class KateDocument : public KTextEditor::Document,
      * @param s string to be inserted
      * @return true on success
      */
-    bool editInsertText ( int line, int col, const QString &s );
+    bool editInsertText ( int line, int col, const QString &s, Kate::EditSource editSource = Kate::NoEditSource );
     /**
      * Remove a string in the given line/column
      * @param line line number
@@ -232,7 +237,7 @@ class KateDocument : public KTextEditor::Document,
      * @param len length of text to be removed
      * @return true on success
      */
-    bool editRemoveText ( int line, int col, int len );
+    bool editRemoveText ( int line, int col, int len, Kate::EditSource editSource = Kate::NoEditSource );
 
     /**
      * Mark @p line as @p autowrapped. This is necessary if static word warp is
@@ -270,13 +275,13 @@ class KateDocument : public KTextEditor::Document,
      * @param s string to insert
      * @return true on success
      */
-    bool editInsertLine ( int line, const QString &s );
+    bool editInsertLine ( int line, const QString &s, Kate::EditSource editSource = Kate::NoEditSource );
     /**
      * Remove a line
      * @param line line number
      * @return true on success
      */
-    bool editRemoveLine ( int line );
+    bool editRemoveLine ( int line, Kate::EditSource editSource = Kate::NoEditSource );
 
     /**
      * Remove a line
@@ -312,17 +317,18 @@ class KateDocument : public KTextEditor::Document,
     void setUndoDontMergeComplex(bool dontMerge);
 
     bool isEditRunning() const;
-
+    bool isWithUndo() const {return editWithUndo;}
   private Q_SLOTS:
     void undoCancel();
 
   private:
     void editAddUndo (int type, uint line, uint col, uint len, const QString &text);
 
-    uint editSessionNumber;
+    int editSessionNumber;
+    QStack<int> editStateStack;
+    QStack<Kate::EditSource> m_editSources;
     bool editIsRunning;
     bool editWithUndo;
-    KateView *editView;
     bool m_undoComplexMerge;
     KateUndoGroup* m_editCurrentUndo;
 
@@ -364,7 +370,6 @@ class KateDocument : public KTextEditor::Document,
 
   Q_SIGNALS:
     void undoChanged ();
-    void textInserted(int line,int column);
 
   //
   // KTextEditor::SearchInterface stuff
@@ -378,22 +383,48 @@ class KateDocument : public KTextEditor::Document,
   //
   // KTextEditor::HighlightingInterface stuff
   //
-  public Q_SLOTS:
-    bool setHlMode (uint mode);
+  public:
+    /**
+     * Return the name of the currently used highlighting
+     * \return name of the used highlighting
+     *
+     */
+    virtual QString highlighting() const;
 
+    /**
+     * Return a list of the names of all possible highlighting
+     * \return list of highlighting names
+     */
+    virtual QStringList highlightings() const;
+
+    /**
+     * Set the current highlighting of the document by giving it's name
+     * \param name name of the highlighting to use for this document
+     * \return \e true on success, otherwise \e false
+     */
+    virtual bool setHighlighting(const QString &name);
+
+  Q_SIGNALS:
+    /**
+     * Warn anyone listening that the current document's highlighting has
+     * changed.
+     *
+     * \param document the document which's highlighting has changed
+     */
+    virtual void highlightingChanged(KTextEditor::Document *document);
+
+  /*
+   * Helpers....
+   */
   public:
     uint hlMode();
-    uint hlModeCount();
-    QString hlModeName(uint mode);
-    QString hlModeSectionName(uint mode);
+    uint hlModeCount() const;
+    QString hlModeName(uint mode) const;
     void bufferHlChanged();
 
   private:
     void setDontChangeHlOnSave();
 
-  Q_SIGNALS:
-    void hlChanged ();
-
   //
   // KTextEditor::ConfigInterface stuff
   //
@@ -425,7 +456,7 @@ class KateDocument : public KTextEditor::Document,
     QPixmap markPixmap( MarkInterface::MarkTypes ) const;
     QString markDescription( MarkInterface::MarkTypes ) const;
     QColor markColor( MarkInterface::MarkTypes ) const;
-	uint editableMarks() const;
+    uint editableMarks() const;
 
   Q_SIGNALS:
     void marksChanged( KTextEditor::Document* );
@@ -451,7 +482,7 @@ class KateDocument : public KTextEditor::Document,
     /**
      * @return the name of the mimetype for the document.
      *
-     * This method is using KMimeType::findByURL, and if the pointer
+     * This method is using KMimeType::findByUrl, and if the pointer
      * is then still the default MimeType for a nonlocal or unsaved file,
      * uses mimeTypeForContent().
      */
@@ -483,7 +514,13 @@ class KateDocument : public KTextEditor::Document,
   public:
     virtual void clearSmartInterface();
 
-    virtual KTextEditor::SmartCursor* newSmartCursor(const KTextEditor::Cursor& position, KTextEditor::SmartCursor::InsertBehaviour insertBehaviour = KTextEditor::SmartCursor::MoveOnInsert);
+    virtual int currentRevision() const;
+    virtual void releaseRevision(int revision) const;
+    virtual void useRevision(int revision = -1);
+    virtual KTextEditor::Cursor translateFromRevision(const KTextEditor::Cursor& cursor, KTextEditor::SmartCursor::InsertBehavior insertBehavior = KTextEditor::SmartCursor::StayOnInsert) const;
+    virtual KTextEditor::Range translateFromRevision(const KTextEditor::Range& range, KTextEditor::SmartRange::InsertBehaviors insertBehavior = KTextEditor::SmartRange::ExpandLeft | KTextEditor::SmartRange::ExpandRight) const;
+
+    virtual KTextEditor::SmartCursor* newSmartCursor(const KTextEditor::Cursor& position, KTextEditor::SmartCursor::InsertBehavior insertBehavior = KTextEditor::SmartCursor::MoveOnInsert);
     virtual void deleteCursors();
 
     virtual KTextEditor::SmartRange* newSmartRange(const KTextEditor::Range& range, KTextEditor::SmartRange* parent = 0L, KTextEditor::SmartRange::InsertBehaviors insertBehavior = KTextEditor::SmartRange::DoNotExpand);
@@ -513,6 +550,8 @@ class KateDocument : public KTextEditor::Document,
 
     KateSmartManager* smartManager() const { return m_smartManager; }
 
+    bool isSmartLocked() const;
+
   Q_SIGNALS:
     void dynamicHighlightAdded(KateSmartRange* range);
     void dynamicHighlightRemoved(KateSmartRange* range);
@@ -538,7 +577,7 @@ class KateDocument : public KTextEditor::Document,
   // KParts::ReadWrite stuff
   //
   public:
-    bool openURL( const KUrl &url );
+    bool openUrl( const KUrl &url );
 
     /* Anders:
       I reimplemented this, since i need to check if backup succeeded
@@ -576,7 +615,7 @@ class KateDocument : public KTextEditor::Document,
     bool typeChars ( KateView *type, const QString &chars );
 
     /**
-     * gets the last line number (numLines() -1)
+     * gets the last line number (lines() - 1)
      */
     inline int lastLine() const { return lines()-1; }
 
@@ -606,8 +645,8 @@ class KateDocument : public KTextEditor::Document,
 
     bool ownedView(KateView *);
 
-    uint currentColumn( const KTextEditor::Cursor& );
-    void newLine(             KTextEditor::Cursor&, KateView* ); // Changes input
+    uint toVirtualColumn( const KTextEditor::Cursor& );
+    void newLine( KateView*view ); // Changes input
     void backspace(     KateView *view, const KTextEditor::Cursor& );
     void del(           KateView *view, const KTextEditor::Cursor& );
     void transpose(     const KTextEditor::Cursor& );
@@ -635,8 +674,8 @@ class KateDocument : public KTextEditor::Document,
     void joinLines( uint first, uint last );
 
   private:
-    bool removeStringFromBegining(int line, QString &str);
-    bool removeStringFromEnd(int line, QString &str);
+    bool removeStringFromBeginning(int line, const QString &str);
+    bool removeStringFromEnd(int line, const QString &str);
 
     /**
       Find the position (line and col) of the next char
@@ -686,22 +725,22 @@ class KateDocument : public KTextEditor::Document,
      * Add a comment marker as defined by the language providing the attribute
      * @p attrib to each line in the selection.
      */
-           void addStartStopCommentToSelection( KateView *view, int attrib=0 );
+    void addStartStopCommentToSelection( KateView *view, int attrib=0 );
     /**
-    * @see addStartStopCommentToSelection.
-    */
-          void addStartLineCommentToSelection( KateView *view, int attrib=0 );
+     * @see addStartStopCommentToSelection.
+     */
+    void addStartLineCommentToSelection( KateView *view, int attrib=0 );
 
     /**
-    * Removes comment markers relevant to the language providing
-    * the attribuge @p attrib from each line in the selection.
-    *
-    * @return whether the operation succeeded.
-    */
+     * Removes comment markers relevant to the language providing
+     * the attribuge @p attrib from each line in the selection.
+     *
+     * @return whether the operation succeeded.
+     */
     bool removeStartStopCommentFromSelection( KateView *view, int attrib=0 );
     /**
-    * @see removeStartStopCommentFromSelection.
-    */
+     * @see removeStartStopCommentFromSelection.
+     */
     bool removeStartLineCommentFromSelection( KateView *view, int attrib=0 );
 
   public:
@@ -823,7 +862,13 @@ class KateDocument : public KTextEditor::Document,
      * and the active view cursor is not on line and behind
      * the last nonspace character.
      */
-    void removeTrailingSpace( int line );
+    void removeTrailingSpace(int line);
+    inline void blockRemoveTrailingSpaces(bool block)
+    { m_blockRemoveTrailingSpaces = block; }
+
+  private:
+    /// if true, removeTrailingSpace() returns immediately.
+    bool m_blockRemoveTrailingSpaces;
 
   public:
     void updateFileType (int newType, bool user = false);
@@ -837,7 +882,8 @@ class KateDocument : public KTextEditor::Document,
     // text buffer
     KateBuffer *m_buffer;
 
-    KateAutoIndent *m_indenter;
+    // indenter
+    KateAutoIndent m_indenter;
 
     bool hlSetByUser;
 
@@ -929,7 +975,7 @@ class KateDocument : public KTextEditor::Document,
     static QRegExp kvVar;
 
     KIO::TransferJob *m_job;
-    KTempFile *m_tempFile;
+    KTemporaryFile *m_tempFile;
 
     bool s_fileChangedDialogsActivated;
 
@@ -944,8 +990,17 @@ class KateDocument : public KTextEditor::Document,
       KateKeyInterceptorFunctor *m_tabInterceptor;
 
   protected Q_SLOTS:
-      //void testTemplateCode();
+      void testTemplateCode();
       void dumpRegionTree();
+  public:
+      class LoadSaveFilterCheckPlugins;
+  private:
+      void setPreSavePostDialogFilterChecks(QStringList plugins) {m_preSavePostDialogFilterChecks=plugins;}
+      QStringList m_preSavePostDialogFilterChecks;
+      void setPostLoadFilterChecks(QStringList plugins) {m_postLoadFilterChecks=plugins;}
+      QStringList m_postLoadFilterChecks;
+      static LoadSaveFilterCheckPlugins *s_loadSaveFilterCheckPlugins;
+      static LoadSaveFilterCheckPlugins* loadSaveFilterCheckPlugins();
 };
 
 #endif
