@@ -27,6 +27,7 @@
 #include "katehighlight.h"
 #include "kateview.h"
 #include "katerenderrange.h"
+#include "katetextlayout.h"
 
 #include <limits.h>
 
@@ -44,6 +45,7 @@ KateRenderer::KateRenderer(KateDocument* doc, KateView *view)
     , m_drawCaret(true)
     , m_showSelections(true)
     , m_showTabs(true)
+    , m_showSpaces(true)
     , m_printerFriendly(false)
     , m_dynamicRegion(doc)
 {
@@ -97,6 +99,11 @@ void KateRenderer::setShowTabs(bool showTabs)
   m_showTabs = showTabs;
 }
 
+void KateRenderer::setShowTrailingSpaces(bool showSpaces)
+{
+  m_showSpaces = showSpaces;
+}
+
 void KateRenderer::setTabWidth(int tabWidth)
 {
   m_tabWidth = tabWidth;
@@ -149,6 +156,7 @@ void KateRenderer::setPrinterFriendly(bool printerFriendly)
 {
   m_printerFriendly = printerFriendly;
   setShowTabs(false);
+  setShowTrailingSpaces(false);
   setShowSelections(false);
   setDrawCaret(false);
 }
@@ -219,13 +227,39 @@ void KateRenderer::paintTextLineBackground(QPainter& paint, KateLineLayoutPtr la
   }
 }
 
-void KateRenderer::paintWhitespaceMarker(QPainter &paint, uint x, uint y)
+void KateRenderer::paintTabstop(QPainter &paint, qreal x, qreal y)
 {
   QPen penBackup( paint.pen() );
-  paint.setPen( config()->tabMarkerColor() );
-  paint.drawPoint(x,     y);
-  paint.drawPoint(x + 1, y);
-  paint.drawPoint(x,     y - 1);
+  QPen pen( config()->tabMarkerColor() );
+  pen.setWidthF(qMax(0.5, spaceWidth() * .1));
+  pen.setCapStyle(Qt::RoundCap);
+  paint.setPen( pen );
+
+  // FIXME: optimize for speed!
+  qreal dist = spaceWidth() * 0.3;
+  QPointF points[8];
+  points[0] = QPointF(x - dist, y - dist);
+  points[1] = QPointF(x, y);
+  points[2] = QPointF(x, y);
+  points[3] = QPointF(x - dist, y + dist);
+  x += spaceWidth() / 3.0;
+  points[4] = QPointF(x - dist, y - dist);
+  points[5] = QPointF(x, y);
+  points[6] = QPointF(x, y);
+  points[7] = QPointF(x - dist, y + dist);
+  paint.drawLines(points, 4);
+  paint.setPen( penBackup );
+}
+
+void KateRenderer::paintTrailingSpace(QPainter &paint, qreal x, qreal y)
+{
+  QPen penBackup( paint.pen() );
+  QPen pen( config()->tabMarkerColor() );
+  pen.setWidthF(spaceWidth() / 3.5);
+  pen.setCapStyle(Qt::RoundCap);
+  paint.setPen( pen );
+
+  paint.drawPoint( QPointF(x, y) );
   paint.setPen( penBackup );
 }
 
@@ -258,18 +292,22 @@ QList<QTextLayout::FormatRange> KateRenderer::decorationsForLine( const KateText
 {
   QList<QTextLayout::FormatRange> newHighlight;
 
-  if (textLine->attributesList().count() || m_view->externalHighlights().count() || m_view->internalHighlights().count() || m_doc->documentHighlights().count()) {
+  // Don't compute the highlighting if there isn't going to be any highlighting
+  if (selectionsOnly || textLine->attributesList().count() || m_view->externalHighlights().count() || m_view->internalHighlights().count() || m_doc->documentHighlights().count()) {
     RenderRangeList renderRanges;
 
     if (!completionHighlight) {
+      // Add arbitrary highlighting ranges to the list
       renderRanges.appendRanges(m_view->internalHighlights(), selectionsOnly, view());
       renderRanges.appendRanges(m_view->externalHighlights(), selectionsOnly, view());
       renderRanges.appendRanges(m_doc->documentHighlights(), selectionsOnly, view());
 
     } else {
+      // Add the code completion arbitrary highlight to the list
       renderRanges.append(completionHighlight);
     }
 
+    // Add the inbuilt highlighting to the list
     NormalRenderRange* inbuiltHighlight = new NormalRenderRange();
     const QVector<int> &al = textLine->attributesList();
     for (int i = 0; i+2 < al.count(); i += 3) {
@@ -277,15 +315,18 @@ QList<QTextLayout::FormatRange> KateRenderer::decorationsForLine( const KateText
     }
     renderRanges.append(inbuiltHighlight);
 
-    NormalRenderRange* selectionHighlight = 0L;
+    // Add selection highlighting if we're creating the selection decorations
     if ((selectionsOnly && showSelections() && m_view->selection()) || (completionHighlight && completionSelected)) {
-      selectionHighlight = new NormalRenderRange();
-      static KTextEditor::Attribute::Ptr backgroundAttribute;
-      if (!backgroundAttribute) {
-        backgroundAttribute = KTextEditor::Attribute::Ptr(new KTextEditor::Attribute());
-        backgroundAttribute->setBackground(config()->selectionColor());
-      }
+      NormalRenderRange* selectionHighlight = new NormalRenderRange();
 
+      // Set up the selection background attribute TODO: move this elsewhere, eg. into the config?
+      static KTextEditor::Attribute::Ptr backgroundAttribute;
+      if (!backgroundAttribute)
+        backgroundAttribute = KTextEditor::Attribute::Ptr(new KTextEditor::Attribute());
+
+      backgroundAttribute->setBackground(config()->selectionColor());
+
+      // Create a range for the current selection
       if (completionHighlight && completionSelected)
         selectionHighlight->addRange(new KTextEditor::Range(line, 0, line + 1, 0), backgroundAttribute);
       else
@@ -296,6 +337,7 @@ QList<QTextLayout::FormatRange> KateRenderer::decorationsForLine( const KateText
 
     KTextEditor::Cursor currentPosition, endPosition;
 
+    // Calculate the range which we need to iterate in order to get the highlighting for just this line
     if (selectionsOnly) {
       KTextEditor::Range rangeNeeded = m_view->selectionRange().encompass(m_dynamicRegion.boundingRange());
       rangeNeeded &= KTextEditor::Range(line, 0, line + 1, 0);
@@ -308,28 +350,29 @@ QList<QTextLayout::FormatRange> KateRenderer::decorationsForLine( const KateText
       endPosition = KTextEditor::Cursor(line + 1, 0);
     }
 
+    // Main iterative loop.  This walks through each set of highlighting ranges, and stops each
+    // time the highlighting changes.  It then creates the corresponding QTextLayout::FormatRanges.
     do {
       renderRanges.advanceTo(currentPosition);
 
       if (!renderRanges.hasAttribute()) {
+        // No attribute, don't need to create a FormatRange for this text range
         currentPosition = renderRanges.nextBoundary();
         continue;
       }
 
       KTextEditor::Cursor nextPosition = renderRanges.nextBoundary();
 
+      // Create the format range and populate with the correct start, length and format info
       QTextLayout::FormatRange fr;
       fr.start = currentPosition.column();
 
-      if (nextPosition < endPosition) {
+      if (nextPosition < endPosition || endPosition.line() <= line) {
         fr.length = nextPosition.column() - currentPosition.column();
 
       } else {
-        if (endPosition.line() > line)
-          // +1 to force background drawing at the end of the line when it's warranted
-          fr.length = textLine->length() - currentPosition.column() + 1;
-        else
-          fr.length = endPosition.column() - currentPosition.column();
+        // +1 to force background drawing at the end of the line when it's warranted
+        fr.length = textLine->length() - currentPosition.column() + 1;
       }
 
       KTextEditor::Attribute a = renderRanges.generateAttribute();
@@ -343,8 +386,11 @@ QList<QTextLayout::FormatRange> KateRenderer::decorationsForLine( const KateText
 
     } while (currentPosition < endPosition);
 
-    delete inbuiltHighlight;
-    delete selectionHighlight;
+    if (completionHighlight)
+      // Don't delete external completion render range
+      renderRanges.removeAll(completionHighlight);
+
+    qDeleteAll(renderRanges);
   }
 
   return newHighlight;
@@ -376,7 +422,7 @@ void KateRenderer::paintTextLine(QPainter& paint, KateLineLayoutPtr range, int x
   // Draws the dashed underline at the start of a folded block of text.
   if (range->startsInvisibleBlock()) {
     paint.setPen(QPen(config()->wordWrapMarkerColor(), 1, Qt::DashLine));
-    paint.drawLine(0, (config()->fontMetrics().height() * range->viewLineCount()) - 1, xEnd - xStart, (config()->fontMetrics().height() * range->viewLineCount()) - 1);
+    paint.drawLine(0, (fm.height() * range->viewLineCount()) - 1, xEnd - xStart, (fm.height() * range->viewLineCount()) - 1);
   }
 
   if (range->layout()) {
@@ -399,17 +445,21 @@ void KateRenderer::paintTextLine(QPainter& paint, KateLineLayoutPtr range, int x
     for (int i = 0; i < range->viewLineCount(); ++i) {
       KateTextLayout line = range->viewLine(i);
 
-      // Determine the background color to use, if any, for the end of this view line
-      QColor backgroundColor;
+      // Determine the background to use, if any, for the end of this view line
+      QBrush backgroundBrush;
+      bool backgroundBrushSet = false;
       while (it2.hasNext()) {
         const QTextLayout::FormatRange& fr = it2.peekNext();
-        if (fr.start >= line.endCol())
-          goto backgroundFound;
+        if (fr.start > line.endCol())
+          goto backgroundDetermined;
 
         if (fr.start + fr.length > line.endCol()) {
-          if (fr.format.hasProperty(QTextFormat::BackgroundBrush))
-            backgroundColor = fr.format.background().color();
-          goto backgroundFound;
+          if (fr.format.hasProperty(QTextFormat::BackgroundBrush)) {
+            backgroundBrushSet = true;
+            backgroundBrush = fr.format.background();
+          }
+
+          goto backgroundDetermined;
         }
 
         it2.next();
@@ -417,29 +467,32 @@ void KateRenderer::paintTextLine(QPainter& paint, KateLineLayoutPtr range, int x
 
       while (it.hasNext()) {
         const QTextLayout::FormatRange& fr = it.peekNext();
-        if (fr.start >= line.endCol())
+        if (fr.start > line.endCol())
           break;
 
         if (fr.start + fr.length > line.endCol()) {
-          if (fr.format.hasProperty(QTextFormat::BackgroundBrush))
-            backgroundColor = fr.format.background().color();
+          if (fr.format.hasProperty(QTextFormat::BackgroundBrush)) {
+            backgroundBrushSet = true;
+            backgroundBrush = fr.format.background();
+          }
+
           break;
         }
 
         it.next();
       }
 
-      backgroundFound:
+      backgroundDetermined:
 
       // Draw selection outside of areas where text is rendered
       if (m_view->selection() && m_view->lineEndSelected(line.end(true))) {
-        QRect area(line.endX() + line.xOffset() - xStart, config()->fontMetrics().height() * i, xEnd - xStart, config()->fontMetrics().height() * (i + 1));
+        QRect area(line.endX() + line.xOffset() - xStart, fm.height() * i, xEnd - xStart, fm.height() * (i + 1));
         paint.fillRect(area, config()->selectionColor());
 
-      } else if (backgroundColor.isValid()) {
+      } else if (backgroundBrushSet) {
         // Draw text background outside of areas where text is rendered.
-        QRect area(line.endX() + line.xOffset() - xStart, config()->fontMetrics().height() * i, xEnd - xStart, config()->fontMetrics().height() * (i + 1));
-        paint.fillRect(area, backgroundColor);
+        QRect area(line.endX() + line.xOffset() - xStart, fm.height() * i, xEnd - xStart, fm.height() * (i + 1));
+        paint.fillRect(area, backgroundBrush);
       }
 
       // Draw indent lines
@@ -454,29 +507,39 @@ void KateRenderer::paintTextLine(QPainter& paint, KateLineLayoutPtr range, int x
         }
       }
 
-      // Draw tab stops
-      if (showTabs()) {
+      // Draw tab stops and trailing spaces
+      if (showTabs() || showTrailingSpaces()) {
         const QString& text = range->textLine()->string();
-        int tabIndex = text.indexOf(tabChar);
-        int y = (config()->fontMetrics().height() * i) + fm.ascent();
-        while (tabIndex != -1) {
-          paintWhitespaceMarker(paint, (int)line.lineLayout().cursorToX(tabIndex) - xStart, y);
-          tabIndex = text.indexOf(tabChar, tabIndex + 1);
+        int y = fm.height() * i + fm.ascent() - fm.strikeOutPos();
+
+        if (showTabs()) {
+          int tabIndex = text.indexOf(tabChar, line.startCol());
+          while (tabIndex != -1 && tabIndex < line.endCol()) {
+            paintTabstop(paint, line.lineLayout().cursorToX(tabIndex) - xStart + spaceWidth()/2.0, y);
+            tabIndex = text.indexOf(tabChar, tabIndex + 1);
+          }
         }
 
-        // FIXME need to add config option for draw trailing spaces
-        int spaceIndex = text.count() - 1;
-        while (spaceIndex >= 0 && text.at(spaceIndex).isSpace()) {
-          paintWhitespaceMarker(paint, (int)line.lineLayout().cursorToX(spaceIndex) - xStart, y);
-          --spaceIndex;
+        if (showTrailingSpaces()) {
+          int spaceIndex = line.endCol() - 1;
+          int trailingPos = range->textLine()->lastChar();
+          if (trailingPos < 0)
+            trailingPos = 0;
+          if (spaceIndex >= trailingPos) {
+            while (spaceIndex >= line.startCol() && text.at(spaceIndex).isSpace()) {
+              if (text.at(spaceIndex) != '\t' || !showTabs())
+                paintTrailingSpace(paint, line.lineLayout().cursorToX(spaceIndex) - xStart + spaceWidth()/2.0, y);
+              --spaceIndex;
+            }
+          }
         }
       }
     }
 
     // draw word-wrap-honor-indent filling
-    if (range->viewLineCount() > 1 && range->shiftX() && range->shiftX() > xStart)
+    if ( (range->viewLineCount() > 1)  && range->shiftX() && (range->shiftX() > xStart) )
     {
-      paint.fillRect(0, config()->fontMetrics().height(), range->shiftX() - xStart, config()->fontMetrics().height() * (range->viewLineCount() - 1),
+      paint.fillRect(0, fm.height(), range->shiftX() - xStart, fm.height() * (range->viewLineCount() - 1),
         QBrush(config()->wordWrapMarkerColor(), Qt::DiagCrossPattern));
     }
 
@@ -499,7 +562,7 @@ void KateRenderer::paintTextLine(QPainter& paint, KateLineLayoutPtr range, int x
       QColor c;
       // Could actually use the real highlighting system for this... would be slower but more accurate for corner cases
       foreach (QTextLayout::FormatRange r, range->layout()->additionalFormats())
-        if (r.start <= cursor->column() && r.start + r.length > cursor->column()) {
+        if ( (r.start <= cursor->column() ) && ( (r.start + r.length)  > cursor->column()) ) {
           c = r.format.foreground().color();
           break;
         }
@@ -514,7 +577,7 @@ void KateRenderer::paintTextLine(QPainter& paint, KateLineLayoutPtr range, int x
         paint.setPen(QPen(c, caretWidth));
 
         // Clip the caret - Qt's caret has a habit of intruding onto other lines
-        paint.setClipRect(0, line.lineNumber() * config()->fontMetrics().height(), xEnd - xStart, config()->fontMetrics().height());
+        paint.setClipRect(0, line.lineNumber() * fm.height(), xEnd - xStart, fm.height());
 
         // Draw the cursor, start drawing in the middle as the above sets the width from the center of the line
         range->layout()->drawCursor(&paint, QPoint(caretWidth/2 - xStart,0), cursor->column());
@@ -525,8 +588,8 @@ void KateRenderer::paintTextLine(QPainter& paint, KateLineLayoutPtr range, int x
         // Off the end of the line... must be block mode. Draw the caret ourselves.
         const KateTextLayout& lastLine = range->viewLine(range->viewLineCount() - 1);
         int x = range->widthOfLastLine() + spaceWidth() * (cursor->column() - range->length());
-        if (x >= xStart && x <= xEnd)
-          paint.fillRect(x, (int)lastLine.lineLayout().y(), caretWidth, config()->fontMetrics().height(), c);
+        if ( (x >= xStart) && (x <= xEnd))
+          paint.fillRect(x, (int)lastLine.lineLayout().y(), caretWidth, fm.height(), c);
       }
     }
   }
@@ -550,10 +613,6 @@ void KateRenderer::paintTextLine(QPainter& paint, KateLineLayoutPtr range, int x
           // indentation lines OR
           || (showIndentLines() && curCol < lastIndentColumn)
 
-          // the next char is a tab (removed the "and this isn't" because that's dealt with above)
-          // i.e. we have to draw the current text so the tab can be rendered as above.
-          || (textLine->string()[nextCol] == tabChar)
-
           // input method edit area
           || ( m_view && (isIMEdit != m_view->isIMEdit( line, nextCol )) )
 
@@ -582,30 +641,6 @@ void KateRenderer::paintTextLine(QPainter& paint, KateLineLayoutPtr range, int x
               paint.setPen( m_view->colorGroup().color( QPalette::BrightText ) );
             }
 
-            // Draw indentation markers.
-            if (showIndentLines() && curCol < lastIndentColumn)
-            {
-              // Draw multiple guides when tab width greater than indent width.
-              const int charWidth = isTab ? m_tabWidth - curPos % m_tabWidth : 1;
-
-              // Do not draw indent guides on the first line.
-              int i = 0;
-              if (curPos == 0 || curPos % m_indentWidth > 0)
-                i = m_indentWidth - curPos % m_indentWidth;
-
-              for (; i < charWidth; i += m_indentWidth)
-              {
-                // In most cases this is done one or zero times.
-                paintIndentMarker(paint, xPos - xStart + i * spaceWidth, line);
-
-                // Draw highlighted line.
-                if (curPos+i == minIndent)
-                {
-                  paintIndentMarker(paint, xPos - xStart + 1 + i * spaceWidth, line+1);
-                }
-              }
-            }
-
             // Draw preedit's underline
             if (isIMEdit) {
               QRect r( oldXPos - xStart, 0, xPosAfter - oldXPos, config()->fontMetrics().height() );
@@ -614,21 +649,11 @@ void KateRenderer::paintTextLine(QPainter& paint, KateLineLayoutPtr range, int x
 
             // Put pen color back
             if (isIMSel) paint.restore();
-
-      // Update the current indentation pos.
-      if (isTab)
-      {
-        curPos += m_tabWidth - (curPos % m_tabWidth);
-      }
-      else
-      {
-        curPos++;
-      }
     }
   */
 
   // show word wrap marker if desirable
-  if (!isPrinterFriendly() && config()->wordWrapMarker() && config()->font().fixedPitch())
+  if ((!isPrinterFriendly()) && config()->wordWrapMarker() && config()->font().fixedPitch())
   {
     paint.setPen( config()->wordWrapMarkerColor() );
     int _x = m_doc->config()->wordWrapAt() * fm.width('x') - xStart;
@@ -740,6 +765,17 @@ void KateRenderer::layoutLine(KateLineLayoutPtr lineLayout, int maxwidth, bool c
   opt.setFlags(QTextOption::IncludeTrailingSpaces);
   opt.setTabStop(m_tabWidth * config()->fontMetrics().width(spaceChar));
   opt.setWrapMode(QTextOption::WrapAnywhere);//QTextOption::WrapAtWordBoundaryOrAnywhere);
+
+  // Find the first strong character in the string.
+  // If it is an RTL character, set the base layout direction of the string to RTL.
+  //
+  // See http://www.unicode.org/reports/tr9/#The_Paragraph_Level (Sections P2 & P3).
+  // Qt's text renderer ("scribe") version 4.2 assumes a "higher-level protocol"
+  // (such as KatePart) will specify the paragraph level, so it does not apply P2 & P3
+  // by itself. If this ever change in Qt, the next code block could be removed.
+  if (lineLayout->textLine()->string().isRightToLeft())
+    opt.setTextDirection(Qt::RightToLeft);
+
   l->setTextOption(opt);
 
   // Syntax highlighting, inbuilt and arbitrary
