@@ -29,9 +29,10 @@
 #include "kateconfig.h"
 
 #include <klocale.h>
-#include <kstdaction.h>
+#include <kstandardaction.h>
 #include <kpushbutton.h>
 #include <kaction.h>
+#include <kactioncollection.h>
 #include <kmessagebox.h>
 #include <kstringhandler.h>
 #include <kdebug.h>
@@ -62,20 +63,20 @@ KateSearch::~KateSearch()
 
 void KateSearch::createActions( KActionCollection* ac )
 {
-  KAction* a = KStdAction::find( this, SLOT(find()), ac );
-  a->setWhatsThis(i18n("Look up the first occurrence of a piece of text or regular expression."));
-  m_view->addAction(a);
+    QAction* a = ac->addAction( KStandardAction::Find, this, SLOT(find()) );
+    a->setWhatsThis(i18n("Look up the first occurrence of a piece of text or regular expression."));
+    m_view->addAction(a);
 
-  a = KStdAction::findNext( this, SLOT(slotFindNext()), ac );
-  a->setWhatsThis(i18n("Look up the next occurrence of the search phrase."));
-  m_view->addAction(a);
+    a = ac->addAction( KStandardAction::FindNext, this, SLOT(slotFindNext()) );
+    a->setWhatsThis(i18n("Look up the next occurrence of the search phrase."));
+    m_view->addAction(a);
 
-  a = KStdAction::findPrev( this, SLOT(slotFindPrev()), ac, "edit_find_prev" );
-  a->setWhatsThis(i18n("Look up the previous occurrence of the search phrase."));
-  m_view->addAction(a);
+    a = ac->addAction( KStandardAction::FindPrev, "edit_find_prev", this, SLOT(slotFindPrev()) );
+    a->setWhatsThis(i18n("Look up the previous occurrence of the search phrase."));
+    m_view->addAction(a);
 
-  a = KStdAction::replace( this, SLOT(replace()), ac );
-  a->setWhatsThis(i18n("Look up a piece of text or regular expression and replace the result with some given text."));
+    a = ac->addAction( KStandardAction::Replace, this, SLOT(replace()) );
+    a->setWhatsThis(i18n("Look up a piece of text or regular expression and replace the result with some given text."));
 }
 
 void KateSearch::addToList( QStringList& list, const QString& s )
@@ -271,17 +272,17 @@ void KateSearch::search( SearchFlags flags )
 void KateSearch::wrapSearch()
 {
   if( s.flags.selected )
-  {      
+  {
     KTextEditor::Cursor start (s.selection.start());
     KTextEditor::Cursor end (s.selection.end());
-      
+
     // recalc for block sel, to have start with lowest col, end with highest
     if (m_view->blockSelectionMode())
     {
       start.setColumn (qMin(s.selection.start().column(), s.selection.end().column()));
       end.setColumn (qMax(s.selection.start().column(), s.selection.end().column()));
     }
-    
+
     s.cursor = s.flags.backward ? end : start;
   }
   else
@@ -341,7 +342,7 @@ void KateSearch::replaceAll()
     }
   } else {
     KMessageBox::information( view(),
-        i18np("%n replacement made.","%n replacements made.",replaces),
+        i18np("%1 replacement made.","%1 replacements made.",replaces),
         i18n("Replace") );
   }
 }
@@ -358,7 +359,7 @@ void KateSearch::promptReplace()
   } else {
     replacePrompt->hide();
     KMessageBox::information( view(),
-        i18np("%n replacement made.","%n replacements made.",replaces),
+        i18np("%1 replacement made.","%1 replacements made.",replaces),
         i18n("Replace") );
   }
 }
@@ -383,11 +384,11 @@ void KateSearch::replaceOne()
           kDebug()<<"KateSearch::replaceOne(): you don't have "<<ccap<<" backreferences in regexp '"<<m_re.pattern()<<"'"<<endl;
         }
       }
-      pos = br.indexIn( replaceWith, pos + qMax(br.matchedLength(), (int)sc.length()) );
+      pos = br.indexIn( replaceWith, pos + (int)sc.length());
     }
   }
 
-  doc()->editStart();
+  doc()->editStart(true, Kate::SearchReplaceEdit);
 
   doc()->removeText( KTextEditor::Range(s.cursor, s.cursor + KTextEditor::Cursor(0,s.matchedLength)) );
   doc()->insertText( s.cursor, replaceWith );
@@ -461,8 +462,8 @@ void KateSearch::replaceSlot() {
 bool KateSearch::askContinue()
 {
   QString made =
-     i18np( "%n replacement made.",
-           "%n replacements made.",
+     i18np( "%1 replacement made.",
+           "%1 replacements made.",
            replaces );
 
   QString reached = !s.flags.backward ?
@@ -481,12 +482,12 @@ bool KateSearch::askContinue()
      i18n( "Continue from the end?" );
 
   QString text = s.flags.replace ?
-     made + "\n" + reached + "\n" + question :
-     reached + "\n" + question;
+     made + '\n' + reached + '\n' + question :
+     reached + '\n' + question;
 
   return KMessageBox::Yes == KMessageBox::questionYesNo(
      view(), text, s.flags.replace ? i18n("Replace") : i18n("Find"),
-     KStdGuiItem::cont(), i18n("&Stop") );
+     KStandardGuiItem::cont(), KGuiItem(i18n("&Stop")) );
 }
 
 QString KateSearch::getSearchText()
@@ -567,16 +568,16 @@ bool KateSearch::doSearch( const QString& text )
     {
       KTextEditor::Cursor start (s.selection.start());
       KTextEditor::Cursor end (s.selection.end());
-        
+
       // recalc for block sel, to have start with lowest col, end with highest
       if (m_view->blockSelectionMode())
       {
         start.setColumn (qMin(s.selection.start().column(), s.selection.end().column()));
         end.setColumn (qMax(s.selection.start().column(), s.selection.end().column()));
       }
-    
-      if ( !s.flags.backward && match.start() >= end
-        ||  s.flags.backward && match.start() < start )
+
+      if ( (!s.flags.backward && match.start() >= end)
+        ||  (s.flags.backward && match.start() < start) )
         match = KTextEditor::Range::invalid();
       else if (m_view->blockSelection())
       {
@@ -589,7 +590,7 @@ bool KateSearch::doSearch( const QString& text )
   }
   while (s.flags.selected && m_view->blockSelectionMode() && match.isValid());
   // in the case we want to search in selection + blockselection we need to loop
-  
+
   if( !match.isValid() ) return false;
 
   // save the search result
@@ -633,12 +634,12 @@ KateReplacePrompt::KateReplacePrompt ( QWidget *parent )
   setModal( false );
   setCaption( i18n( "Replace Confirmation" ) );
   setButtons( User3 | User2 | User1 | Close | Ok );
-  setButtonGuiItem( User1, i18n("Replace &All") );
-  setButtonGuiItem( User2, i18n("Re&place && Close") );
-  setButtonGuiItem( User3, i18n("&Replace") );
+  setButtonGuiItem( User1, KGuiItem(i18n("Replace &All")) );
+  setButtonGuiItem( User2, KGuiItem(i18n("Re&place && Close")) );
+  setButtonGuiItem( User3, KGuiItem(i18n("&Replace")) );
   setDefaultButton( Ok );
   showButtonSeparator( true );
-  setButtonGuiItem( Ok, i18n("&Find Next") );
+  setButtonGuiItem( Ok, KGuiItem(i18n("&Find Next")) );
 
   QWidget *page = new QWidget(this);
   setMainWidget(page);
@@ -850,31 +851,6 @@ const QStringList &SearchCommand::cmds()
   return l;
 }
 
-QString SearchCommand::name (const QString& cmd) const
-{
-  if( cmd == "find" )
-    return i18n("Search");
-  else if( cmd == "repace" )
-    return i18n("Replace");
-  else
-    return i18n("Incremental Find");
-}
-
-QString SearchCommand::description (const QString& cmd) const
-{
-  if( cmd == "find" )
-    return i18n("Search.");
-  else if( cmd == "repace" )
-    return i18n("Replace.");
-  else
-    return i18n("Search for text while typing.");
-}
-
-QString SearchCommand::category (const QString& cmd) const
-{
-  return i18n("Edit");
-}
-
 bool SearchCommand::wantsToProcessText( const QString &cmdname )
 {
   return  cmdname == "ifind";
