@@ -28,7 +28,6 @@
 #include <ktexteditor/texthintinterface.h>
 #include <ktexteditor/markinterface.h>
 #include <ktexteditor/codecompletioninterface.h>
-#include <ktexteditor/codecompletion2.h>
 #include <ktexteditor/sessionconfiginterface.h>
 #include <ktexteditor/templateinterface.h>
 #include <ktexteditor/rangefeedback.h>
@@ -45,7 +44,6 @@ class KateDocument;
 class KateBookmarks;
 class KateSearch;
 class KateCmdLine;
-class KateCodeCompletion;
 class KateViewConfig;
 class KateViewSchemaAction;
 class KateRenderer;
@@ -53,6 +51,9 @@ class KateSpell;
 class KateCompletionWidget;
 class KateSmartRange;
 class KateViewInternal;
+class KateSearchBar;
+class KateViewBar;
+class KateGotoBar;
 
 class KToggleAction;
 class KAction;
@@ -71,22 +72,21 @@ namespace KTextEditor
 //
 class KateView : public KTextEditor::View,
                  public KTextEditor::TextHintInterface,
-                 public KTextEditor::CodeCompletionInterface,
                  public KTextEditor::SessionConfigInterface,
                  public KTextEditor::TemplateInterface,
-                 public KTextEditor::CodeCompletionInterface2,
+                 public KTextEditor::CodeCompletionInterface,
                  public KTextEditor::ConfigInterface,
 		 private KTextEditor::SmartRangeWatcher
 {
     Q_OBJECT
     Q_INTERFACES(KTextEditor::TextHintInterface)
-    Q_INTERFACES(KTextEditor::CodeCompletionInterface)
     Q_INTERFACES(KTextEditor::SessionConfigInterface)
     Q_INTERFACES(KTextEditor::TemplateInterface)
     Q_INTERFACES(KTextEditor::ConfigInterface)
+    Q_INTERFACES(KTextEditor::CodeCompletionInterface)
     friend class KateViewInternal;
     friend class KateIconBorder;
-    friend class KateCodeCompletion;
+    friend class KateSearchBar;
 
   public:
     KateView( KateDocument* doc, QWidget* parent );
@@ -161,9 +161,9 @@ class KateView : public KTextEditor::View,
   // KTextEditor::ViewCursorInterface
   //
   public:
-    bool setCursorPosition (const KTextEditor::Cursor &position);
+    bool setCursorPosition (KTextEditor::Cursor position);
 
-    const KTextEditor::Cursor &cursorPosition () const;
+    KTextEditor::Cursor cursorPosition () const;
 
     KTextEditor::Cursor cursorPositionVirtual () const;
 
@@ -174,11 +174,11 @@ class KateView : public KTextEditor::View,
     bool setCursorPositionVisual( const KTextEditor::Cursor& position );
 
     /**
-     * calculate the virtual column position of the cursor
-     * one tab maybe multiple columns
-     * @return virtual cursor column
+     * Return the virtual cursor column, each tab is expanded into the
+     * document's tabWidth characters. If word wrap is off, the cursor may be
+     * behind the line's length.
      */
-    int cursorColumn() const;
+    int virtualCursorColumn() const;
 
     virtual bool mouseTrackingEnabled() const;
     virtual bool setMouseTrackingEnabled(bool enable);
@@ -199,42 +199,24 @@ class KateView : public KTextEditor::View,
      void setConfigValue(const QString &key, const QVariant &value);
 
   //
-  // KTextEditor::CodeCompletionInterface
-  //
-  public:
-    bool registerCompletionProvider(KTextEditor::CompletionProvider*);
-    bool unregisterCompletionProvider(KTextEditor::CompletionProvider*);
-    void invokeCompletion(enum KTextEditor::CompletionType type);
-    void invokeCompletion(KTextEditor::CompletionProvider*,enum KTextEditor::CompletionType);
-    void completionAborted();
-    void completionDone();
-    void completingInProgress(bool val) {m_cc_cleanup=val;}
-  private:
-    QLinkedList<KTextEditor::CompletionProvider*> m_completionProviders;
-    QHash<KTextEditor::CompletionProvider*,KTextEditor::CompletionData> m_completionProviderData;
-    bool m_customComplete;
-    bool m_cc_cleanup;
-    enum KTextEditor::CompletionType m_delayed_cc_type;
-    KTextEditor::CompletionProvider* m_delayed_cc_provider;
-
-#if 0
-  public Q_SLOTS:
-    void showArgHint( QStringList arg1, const QString& arg2, const QString& arg3 );
-#endif
-
-  //
-  // KTextEditor::CodeCompletionInterface
+  // KTextEditor::CodeCompletionInterface2
   //
   public:
     virtual bool isCompletionActive() const;
     virtual void startCompletion(const KTextEditor::Range& word, KTextEditor::CodeCompletionModel* model);
     virtual void abortCompletion();
     virtual void forceCompletion();
+    virtual void registerCompletionModel(KTextEditor::CodeCompletionModel* model);
+    virtual void unregisterCompletionModel(KTextEditor::CodeCompletionModel* model);
+    virtual bool isAutomaticInvocationEnabled() const;
+    virtual void setAutomaticInvocationEnabled(bool enabled = true);
 
-    inline KateCompletionWidget* completionWidget() const { return m_completionWidget; }
+  public Q_SLOTS:
+    void userInvokedCompletion();
 
   private:
-    KateCompletionWidget* m_completionWidget;
+    KateCompletionWidget* completionWidget() const;
+    mutable KateCompletionWidget* m_completionWidget;
 
   //
   // KTextEditor::TextHintInterface
@@ -411,6 +393,7 @@ class KateView : public KTextEditor::View,
     void joinLines();
 
     // Note - the following functions simply forward to KateViewInternal
+    void keyReturn();
     void backspace();
     void deleteWordLeft();
     void keyDelete();
@@ -462,9 +445,8 @@ class KateView : public KTextEditor::View,
     void find( const QString&, long, bool add=true ); ///< proxy for KateSearch
     void replace();
     void replace( const QString&, const QString &, long ); ///< proxy for KateSearch
-    void findAgain( bool back );
-    void findAgain()              { findAgain( false );          }
-    void findPrev()               { findAgain( true );           }
+    void findNext();
+    void findPrevious();
 
     void setFoldingMarkersOn( bool enable ); // Not in KTextEditor::View, but should be
     void setIconBorder( bool enable );
@@ -540,11 +522,10 @@ class KateView : public KTextEditor::View,
     void setupActions();
     void setupEditActions();
     void setupCodeFolding();
-    void setupCodeCompletion();
 
     KActionCollection*     m_editActions;
-    KAction*               m_editUndo;
-    KAction*               m_editRedo;
+    QAction*               m_editUndo;
+    QAction*               m_editRedo;
     KRecentFilesAction*    m_fileRecent;
     KToggleAction*         m_toggleFoldingMarkers;
     KToggleAction*         m_toggleIconBar;
@@ -553,16 +534,16 @@ class KateView : public KTextEditor::View,
     KToggleAction*         m_toggleDynWrap;
     KSelectAction*         m_setDynWrapIndicators;
     KToggleAction*         m_toggleWWMarker;
-    KAction*               m_switchCmdLine;
+    QAction*               m_switchCmdLine;
 
     KSelectAction*         m_setEndOfLine;
 
-    KAction *m_cut;
-    KAction *m_copy;
-    KAction *m_copyHTML;
-    KAction *m_paste;
-    KAction *m_selectAll;
-    KAction *m_deSelect;
+    QAction *m_cut;
+    QAction *m_copy;
+    QAction *m_copyHTML;
+    QAction *m_paste;
+    QAction *m_selectAll;
+    QAction *m_deSelect;
 
     KToggleAction *m_toggleBlockSelection;
     KToggleAction *m_toggleInsert;
@@ -574,9 +555,10 @@ class KateView : public KTextEditor::View,
     KateSearch*            m_search;
     KateSpell             *m_spell;
     KateBookmarks*         m_bookmarks;
-    KateCodeCompletion*    m_codeCompletion;
 
     KateCmdLine *m_cmdLine;
+    KateSearchBar *m_searchBar;
+    KateGotoBar *m_gotoBar;
     bool m_cmdLineOn;
 
     QVBoxLayout *m_vBox;
@@ -618,9 +600,8 @@ class KateView : public KTextEditor::View,
    */
   public:
     void setIMSelectionValue( const KTextEditor::Range& imRange, const KTextEditor::Range& imSelection, bool imComposeEvent );
-    void getIMSelectionValue( KTextEditor::Range* imRange, KTextEditor::Range* imSelection );
-    bool isIMSelection( const KTextEditor::Cursor& pos );
-    bool isIMEdit( const KTextEditor::Cursor& pos );
+    const KTextEditor::Range& imSelection() const;
+    const KTextEditor::Range& imEdit() const;
     bool imComposeEvent () const { return m_imComposeEvent; }
 
   ///Template stuff
@@ -630,6 +611,8 @@ class KateView : public KTextEditor::View,
   private:
     KTextEditor::Range m_imRange, m_imSelection;
     bool m_imComposeEvent;
+
+    KateViewBar *m_viewBar;
 };
 
 #endif
