@@ -30,6 +30,7 @@
 #include "kateview.h"
 #include "kateviewinternal.h"
 #include "katelayoutcache.h"
+#include "katetextlayout.h"
 
 #include <kapplication.h>
 #include <kglobalsettings.h>
@@ -38,17 +39,21 @@
 #include <kglobal.h>
 #include <kcharsets.h>
 #include <kmenu.h>
-#include <kmenu.h>
+#include <kiconloader.h>
 
 #include <qcursor.h>
 #include <qpainter.h>
 #include <qstyle.h>
-#include <qtimer.h>
+#include <QTimer>
 #include <qregexp.h>
 #include <qtextcodec.h>
 #include <QMouseEvent>
 #include <QPainterPath>
 #include <QStyleOption>
+#include <QPalette>
+#include <QPen>
+#include <QVBoxLayout>
+#include <QToolButton>
 
 #include <math.h>
 
@@ -232,8 +237,13 @@ void KateScrollBar::recomputeMarksPositions()
 
 void KateScrollBar::sliderMaybeMoved(int value)
 {
-  if (m_middleMouseDown)
+  if (m_middleMouseDown) {
+    // we only need to emit this signal once, as for the following slider
+    // movements the signal sliderMoved() is already emitted.
+    // Thus, set m_middleMouseDown to false right away.
+    m_middleMouseDown = false;
     emit sliderMMBMoved(value);
+  }
 }
 //END
 
@@ -400,10 +410,10 @@ void KateCmdLine::slotReturnPressed ( const QString& text )
   m_cmdend = 0;
 
   m_view->setFocus ();
-  QTimer::singleShot( 4000, this, SLOT(hideMe()) );
+  QTimer::singleShot( 4000, this, SLOT(hideBar()) );
 }
 
-void KateCmdLine::hideMe () // unless i have focus ;)
+void KateCmdLine::hideBar () // unless i have focus ;)
 {
   if ( isVisibleTo(parentWidget()) && ! hasFocus() ) {
      m_view->toggleCmdLine ();
@@ -427,7 +437,7 @@ void KateCmdLine::keyPressEvent( QKeyEvent *ev )
   if (ev->key() == Qt::Key_Escape)
   {
     m_view->setFocus ();
-    hideMe();
+    hideBar();
   }
   else if ( ev->key() == Qt::Key_Up )
     fromHistory( true );
@@ -574,40 +584,6 @@ void KateCmdLine::fromHistory( bool up )
 //BEGIN KateIconBorder
 using namespace KTextEditor;
 
-static const char* const plus_xpm[] = {
-"11 11 3 1",
-"       c None",
-".      c #000000",
-"+      c #FFFFFF",
-"...........",
-".+++++++++.",
-".+++++++++.",
-".++++.++++.",
-".++++.++++.",
-".++.....++.",
-".++++.++++.",
-".++++.++++.",
-".+++++++++.",
-".+++++++++.",
-"..........."};
-
-static const char* const minus_xpm[] = {
-"11 11 3 1",
-"       c None",
-".      c #000000",
-"+      c #FFFFFF",
-"...........",
-".+++++++++.",
-".+++++++++.",
-".+++++++++.",
-".+++++++++.",
-".++.....++.",
-".+++++++++.",
-".+++++++++.",
-".+++++++++.",
-".+++++++++.",
-"..........."};
-
 static const char * bookmark_xpm[] = {
 "14 13 82 1",
 "   c None",
@@ -706,7 +682,6 @@ static const char * bookmark_xpm[] = {
 "   JKL  MNO   ",
 "   P      Q   "};
 
-const int iconPaneWidth = 16;
 const int halfIPW = 8;
 
 KateIconBorder::KateIconBorder ( KateViewInternal* internalView, QWidget *parent )
@@ -721,18 +696,61 @@ KateIconBorder::KateIconBorder ( KateViewInternal* internalView, QWidget *parent
   , m_dynWrapIndicators( 0 )
   , m_cachedLNWidth( 0 )
   , m_maxCharWidth( 0 )
-  , minus_px ((const char**)minus_xpm)
-  , plus_px ((const char**)plus_xpm)
+  , iconPaneWidth (16)
+  , m_blockRange(0)
+  , m_lastBlockLine(-1)
 {
-  setAttribute( Qt::WA_StaticContents );
-  setSizePolicy( QSizePolicy(  QSizePolicy::Fixed, QSizePolicy::Minimum ) );
 
+        for (int i=0;i<MAXFOLDINGCOLORS;i++) {
+          int r,g,b;
+  /*
+          if (info.depth<16) {
+            r=0xff;
+            g=0xff;
+            b=0xff-0x10*info.depth;
+          } else if (info.depth<32) {
+            r=0xff-0x10*(info.depth-16); g=0xff;b=0;
+          } else if (info.depth<48) {
+            //r=0;g=0xff;b=0x10*(info.depth-32);
+            r=0;g=0xff-0x10*(info.depth-32);b=0;
+          } else {
+            //r=0;g=0xff;b=0xff;
+            r=0;g=0;b=0;
+          }
+  */
+          if (i==0) {
+            r=0xff;
+            g=0xff;
+            b=0xff;
+          } else
+          if (i<3) {
+            r=0xff;
+            g=0xff;
+            b=0xff-0x40-0x40*i;
+          } else if (i<7) {
+            r=0xff-0x40*(i-3); g=0xff;b=0;
+          } else if (i<15) {
+            //r=0;g=0xff;b=0x10*(info.depth-32);
+            r=0;g=0xff-0x40*(i-7);b=0;
+          } else {
+            //r=0;g=0xff;b=0xff;
+            r=0;g=0;b=0;
+          }
+          m_foldingColors[i]=QBrush(QColor(r,g,b,0x88),Qt::SolidPattern /*Qt::Dense4Pattern*/);
+          m_foldingColorsSolid[i]=QBrush(QColor(r,g,b),Qt::SolidPattern);
+       }
+
+  setAttribute( Qt::WA_StaticContents );
+  setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Minimum );
+  setMouseTracking(true);
   m_doc->setMarkDescription( MarkInterface::markType01, i18n("Bookmark") );
   m_doc->setMarkPixmap( MarkInterface::markType01, QPixmap((const char**)bookmark_xpm) );
 
   updateFont();
 }
 
+KateIconBorder::~KateIconBorder() {delete m_blockRange;}
+
 void KateIconBorder::setIconBorderOn( bool enable )
 {
   if( enable == m_iconBorderOn )
@@ -814,6 +832,13 @@ void KateIconBorder::updateFont()
     int charWidth = fm.width( QChar(i) );
     m_maxCharWidth = qMax(m_maxCharWidth, charWidth);
   }
+
+  // the icon pane scales with the font...
+  iconPaneWidth = fm.height();
+
+  updateGeometry();
+
+  QTimer::singleShot( 0, this, SLOT(update()) );
 }
 
 int KateIconBorder::lineNumberWidth() const
@@ -858,11 +883,72 @@ int KateIconBorder::lineNumberWidth() const
   return width;
 }
 
+const QBrush& KateIconBorder::foldingColor(KateLineInfo *info,int realLine, bool solid) {
+  int depth;
+  if (info!=0) {
+    depth=info->depth;
+  } else {
+    KateLineInfo tmp;
+    m_doc->lineInfo(&tmp,realLine);
+    depth=tmp.depth;
+  }
+
+  if (solid) {
+    if (depth<MAXFOLDINGCOLORS)
+      return m_foldingColorsSolid[depth];
+    else
+      return m_foldingColorsSolid[MAXFOLDINGCOLORS-1];
+  } else {
+    if (depth<MAXFOLDINGCOLORS)
+      return m_foldingColors[depth];
+    else
+      return m_foldingColors[MAXFOLDINGCOLORS-1];
+  }
+
+}
+
 void KateIconBorder::paintEvent(QPaintEvent* e)
 {
   paintBorder(e->rect().x(), e->rect().y(), e->rect().width(), e->rect().height());
 }
 
+static void paintTriangle (QPainter &painter, QColor baseColor, int xOffset, int yOffset, int width, int height, bool open)
+{
+  float size = qMin (width, height);
+
+  QColor c = baseColor.dark ();
+
+  // just test if that worked, else use light..., black on black is evil...
+  if (c == baseColor)
+    c = baseColor.light ();
+
+  QPen pen;
+  pen.setJoinStyle (Qt::RoundJoin);
+  pen.setColor (c);
+  pen.setWidthF (1.5);
+  painter.setPen ( pen );
+
+  painter.setBrush ( c );
+
+  // let some border, if possible
+  size *= 0.6;
+
+  float halfSize = size / 2;
+  float halfSizeP = halfSize * 0.6;
+  QPointF middle (xOffset + (float)width / 2, yOffset + (float)height / 2);
+
+  if (open)
+  {
+    QPointF points[3] = { middle+QPointF(-halfSize, -halfSizeP), middle+QPointF(halfSize, -halfSizeP), middle+QPointF(0, halfSizeP) };
+    painter.drawConvexPolygon(points, 3);
+  }
+  else
+  {
+    QPointF points[3] = { middle+QPointF(-halfSizeP, -halfSize), middle+QPointF(-halfSizeP, halfSize), middle+QPointF(halfSizeP, 0) };
+    painter.drawConvexPolygon(points, 3);
+  }
+}
+
 void KateIconBorder::paintBorder (int /*x*/, int y, int /*width*/, int height)
 {
   uint h = m_view->renderer()->config()->fontMetrics().height();
@@ -896,10 +982,12 @@ void KateIconBorder::paintBorder (int /*x*/, int y, int /*width*/, int height)
   int w( this->width() );                     // sane value/calc only once
 
   QPainter p ( this );
+  p.setRenderHints (QPainter::Antialiasing);
   p.setFont ( m_view->renderer()->config()->font() ); // for line numbers
   // the line number color is for the line numbers, vertical separator lines
   // and for for the code folding lines.
   p.setPen ( m_view->renderer()->config()->lineNumberColor() );
+  p.setBrush ( m_view->renderer()->config()->lineNumberColor() );
 
   KateLineInfo oldInfo;
   if (startz < lineRangesSize)
@@ -926,6 +1014,9 @@ void KateIconBorder::paintBorder (int /*x*/, int y, int /*width*/, int height)
     // icon pane
     if( m_iconBorderOn )
     {
+      p.setPen ( m_view->renderer()->config()->lineNumberColor() );
+      p.setBrush ( m_view->renderer()->config()->lineNumberColor() );
+
       p.drawLine(lnX+iconPaneWidth, y, lnX+iconPaneWidth, y+h);
 
       if( (realLine > -1) && (m_viewInternal->cache()->viewLine(z).startCol() == 0) )
@@ -986,42 +1077,25 @@ void KateIconBorder::paintBorder (int /*x*/, int y, int /*width*/, int height)
         KateLineInfo info;
         m_doc->lineInfo(&info,realLine);
 
+        QBrush brush (foldingColor(&info,realLine,true));
+        p.fillRect(lnX, y, iconPaneWidth, h, brush);
+
         if (!info.topLevel)
         {
           if (info.startsVisibleBlock && (m_viewInternal->cache()->viewLine(z).startCol() == 0))
           {
-            if (oldInfo.topLevel)
-              p.drawLine(lnX+halfIPW,y+m_px,lnX+halfIPW,y+h-1);
-            else
-              p.drawLine(lnX+halfIPW,y,lnX+halfIPW,y+h-1);
-
-            p.drawPixmap(lnX+3,y+m_px,minus_px);
+            paintTriangle (p, brush.color(), lnX, y, iconPaneWidth, h, true);
           }
-          else if (info.startsInVisibleBlock)
+          else if (info.startsInVisibleBlock && m_viewInternal->cache()->viewLine(z).startCol() == 0)
           {
-            if (m_viewInternal->cache()->viewLine(z).startCol() == 0)
-            {
-              if (oldInfo.topLevel)
-                p.drawLine(lnX+halfIPW,y+m_px,lnX+halfIPW,y+h-1);
-              else
-                p.drawLine(lnX+halfIPW,y,lnX+halfIPW,y+h-1);
-
-              p.drawPixmap(lnX+3,y+m_px,plus_px);
-            }
-            else
-            {
-              p.drawLine(lnX+halfIPW,y,lnX+halfIPW,y+h-1);
-            }
-
-            if (!m_viewInternal->cache()->viewLine(z).wrap())
-              p.drawLine(lnX+halfIPW,y+h-1,lnX+iconPaneWidth-2,y+h-1);
+            paintTriangle (p, brush.color(), lnX, y, iconPaneWidth, h, false);
           }
           else
           {
-            p.drawLine(lnX+halfIPW,y,lnX+halfIPW,y+h-1);
+           // p.drawLine(lnX+halfIPW,y,lnX+halfIPW,y+h-1);
 
-            if (info.endsBlock && !m_viewInternal->cache()->viewLine(z).wrap())
-              p.drawLine(lnX+halfIPW,y+h-1,lnX+iconPaneWidth-2,y+h-1);
+           // if (info.endsBlock && !m_viewInternal->cache()->viewLine(z).wrap())
+            //  p.drawLine(lnX+halfIPW,y+h-1,lnX+iconPaneWidth-2,y+h-1);
           }
         }
 
@@ -1059,7 +1133,6 @@ void KateIconBorder::mousePressEvent( QMouseEvent* e )
   const KateTextLayout& t = m_viewInternal->yToKateTextLayout(e->y());
   if (t.isValid()) {
     m_lastClickedLine = t.line();
-
     if ( positionToArea( e->pos() ) != IconBorder )
     {
       QMouseEvent forward( QEvent::MouseButtonPress,
@@ -1072,10 +1145,42 @@ void KateIconBorder::mousePressEvent( QMouseEvent* e )
   QWidget::mousePressEvent(e);
 }
 
+void KateIconBorder::showBlock(int line) {
+  //kDebug()<<"showBlock: 1"<<endl;
+  if (line==m_lastBlockLine) return;
+  //kDebug()<<"showBlock: 2"<<endl;
+  m_lastBlockLine=line;
+  delete m_blockRange;
+  m_blockRange=0;
+  KateCodeFoldingTree *tree=m_doc->foldingTree();
+  if (tree) {
+    //kDebug()<<"showBlock: 3"<<endl;
+    KateCodeFoldingNode *node = tree->findNodeForLine(line);
+    KTextEditor::Cursor beg;
+    KTextEditor::Cursor end;
+    if (node->getBegin(tree,&beg) && node->getEnd(tree,&end)) {
+      kDebug()<<"BEGIN"<<beg<<"END"<<end<<endl;
+      m_blockRange=m_doc->newSmartRange(KTextEditor::Range(beg,end));
+      KTextEditor::Attribute::Ptr attr(new KTextEditor::Attribute());
+      attr->setBackground(foldingColor(0,line,false));
+      m_blockRange->setAttribute(attr);
+      m_doc->addHighlightToView(m_view,m_blockRange,false);
+    }
+  }
+}
+
+void KateIconBorder::hideBlock() {
+  m_lastBlockLine=-1;
+  delete m_blockRange;
+  m_blockRange=0;
+}
+
 void KateIconBorder::mouseMoveEvent( QMouseEvent* e )
 {
   const KateTextLayout& t = m_viewInternal->yToKateTextLayout(e->y());
   if (t.isValid()) {
+    if ( positionToArea( e->pos() ) == FoldingMarkers) showBlock(t.line());
+    else hideBlock();
     if ( positionToArea( e->pos() ) != IconBorder )
     {
       QMouseEvent forward( QEvent::MouseMove,
@@ -1182,9 +1287,9 @@ void KateIconBorder::showMarkMenu( uint line, const QPoint& pos )
   {
      KateViewConfig::global()->setDefaultMarkType (vec[result-100]);
      // flush config, otherwise it isn't nessecarily done
-     KConfig *config = KGlobal::config();
+     KSharedConfig::Ptr config = KGlobal::config();
      config->setGroup("Kate View Defaults");
-     KateViewConfig::global()->writeConfig( config );
+     KateViewConfig::global()->writeConfig(config.data());
   }
   else
   {
@@ -1198,8 +1303,8 @@ void KateIconBorder::showMarkMenu( uint line, const QPoint& pos )
 }
 //END KateIconBorder
 
-KateViewEncodingAction::KateViewEncodingAction(KateDocument *_doc, KateView *_view, const QString& text, KActionCollection* parent, const char* name)
-       : KActionMenu (text, parent, name), doc(_doc), view (_view)
+KateViewEncodingAction::KateViewEncodingAction(KateDocument *_doc, KateView *_view, const QString& text, QObject *parent)
+       : KActionMenu (text, parent), doc(_doc), view (_view)
 {
   m_actions=new QActionGroup(this);
   m_actions->setExclusive(true);
@@ -1249,4 +1354,115 @@ void KateViewEncodingAction::setMode (QAction* a)
   view->reloadFile();
 }
 
+
+//BEGIN KateViewBar related classes
+
+KateViewBarWidget::KateViewBarWidget (KateViewBar *viewBar)
+ : QWidget (), m_viewBar (viewBar)
+{
+  m_viewBar->addBarWidget (this);
+
+  QHBoxLayout *layout = new QHBoxLayout;
+
+  // NOTE: Here be cosmetics.
+  layout->setMargin(2);
+
+  // hide button
+  QToolButton *hideButton = new QToolButton(this);
+  hideButton->setAutoRaise(true);
+  hideButton->setIcon(QIcon(SmallIcon("cancel")));
+  connect(hideButton, SIGNAL(clicked()), this, SLOT(hideBar()));
+  layout->addWidget(hideButton);
+  layout->setAlignment( hideButton, Qt::AlignLeft|Qt::AlignTop );
+
+  // widget to be used as parent for the real content
+  m_centralWidget = new QWidget ();
+  layout->addWidget(m_centralWidget);
+
+  setLayout(layout);
+  setFocusProxy(m_centralWidget);
+}
+
+void KateViewBarWidget::showBar ()
+{
+  m_viewBar->showBarWidget (this);
+}
+
+void KateViewBarWidget::hideBar ()
+{
+  // let the barwidget do some stuff on hide, perhaps even say: no, don't hide me...
+  if (!hideIsTriggered ())
+    return;
+
+  m_viewBar->hideBarWidget ();
+}
+
+
+
+KateStackedLayout::KateStackedLayout(QWidget* parent)
+  : QStackedLayout(parent)
+{}
+
+QSize KateStackedLayout::sizeHint() const
+{
+  if (currentWidget())
+    return currentWidget()->sizeHint();
+  return QStackedLayout::sizeHint();
+}
+
+QSize KateStackedLayout::minimumSize() const
+{
+  if (currentWidget())
+    return currentWidget()->minimumSize();
+  return QStackedLayout::minimumSize();
+}
+
+
+
+KateViewBar::KateViewBar (KateView *view)
+ : QWidget (view), m_view (view)
+{
+  m_stack = new KateStackedLayout(this);
+  hide ();
+}
+
+void KateViewBar::addBarWidget (KateViewBarWidget *newBarWidget)
+{
+  // add new widget, invisible...
+  m_stack->addWidget (newBarWidget);
+
+  kDebug(13025)<<"add barwidget " << newBarWidget <<endl;
+}
+
+void KateViewBar::showBarWidget (KateViewBarWidget *barWidget)
+{
+  // raise correct widget
+  m_stack->setCurrentWidget (barWidget);
+  kDebug(13025)<<"show barwidget " << barWidget <<endl;
+  show ();
+}
+
+void KateViewBar::hideBarWidget ()
+{
+  hide();
+  kDebug(13025)<<"hide barwidget"<<endl;
+}
+
+void KateViewBar::keyPressEvent(QKeyEvent* event)
+{
+  if (event->key() == Qt::Key_Escape) {
+    hideBarWidget();
+    return;
+  }
+  QWidget::keyPressEvent(event);
+}
+
+void KateViewBar::hideEvent(QHideEvent* event)
+{
+  if (!event->spontaneous())
+    m_view->setFocus();
+}
+
+//END KateViewBar related classes
+
 // kate: space-indent on; indent-width 2; replace-tabs on;
