@@ -30,14 +30,14 @@
 #include "katecodefoldinghelpers.h"
 #include "kateviewhelpers.h"
 #include "katehighlight.h"
-#include "katesmartcursor.h"
+#include "katesmartrange.h"
 #include "katerenderer.h"
-#include "katecodecompletion.h"
 #include "kateconfig.h"
 #include "katelayoutcache.h"
 #include "katedynamicanimation.h"
 #include "katesmartmanager.h"
 #include "katecompletionwidget.h"
+#include "katetextlayout.h"
 
 #include <kcursor.h>
 #include <kdebug.h>
@@ -51,8 +51,9 @@
 #include <QPixmap>
 #include <QWheelEvent>
 #include <QMouseEvent>
-#include <QClipboard>
 #include <QStack>
+#include <QMutex>
+#include <QThread>
 
 KateViewInternal::KateViewInternal(KateView *view, KateDocument *doc)
   : QWidget (view)
@@ -63,11 +64,16 @@ KateViewInternal::KateViewInternal(KateView *view, KateDocument *doc)
   , m_cursor(doc)
   , m_mouse()
   , m_possibleTripleClick (false)
-  , m_bm(doc)
-  , m_bmStart(doc, &m_bm)
-  , m_bmEnd(doc, &m_bm)
+#if 0
+  , m_bm(doc->smartManager()->newSmartRange())
+  , m_bmStart(doc->smartManager()->newSmartRange(KTextEditor::Range()/*, m_bm*/))
+  , m_bmEnd(doc->smartManager()->newSmartRange(KTextEditor::Range()/*, m_bm*/))
+#endif
   , m_dummy (0)
-  , m_startPos(doc)
+
+  // stay on cursor will avoid that the view scroll around on press return at beginning
+  , m_startPos(doc, KTextEditor::SmartCursor::StayOnInsert)
+
   , m_madeVisible(false)
   , m_shiftKeyPressed (false)
   , m_autoCenterLines (false)
@@ -87,7 +93,9 @@ KateViewInternal::KateViewInternal(KateView *view, KateDocument *doc)
   , m_textHintEnabled(false)
   , m_textHintMouseX(-1)
   , m_textHintMouseY(-1)
+  , m_smartDirty(false)
 {
+#if 0
   // Set up bracket marking
   static KTextEditor::Attribute::Ptr bracketOutline, bracketFill;
   if (!bracketOutline) {
@@ -101,19 +109,15 @@ KateViewInternal::KateViewInternal(KateView *view, KateDocument *doc)
     bracketFill->setFontBold();
   }
 
-  m_bm.setAttribute(bracketOutline);
-  m_bm.setInternal();
-
-  m_bmStart.setAttribute(bracketFill);
-  m_bmStart.setInternal();
-
-  m_bmEnd.setAttribute(bracketFill);
-  m_bmEnd.setInternal();
+  m_bm->setAttribute(bracketOutline);
+  m_bmStart->setAttribute(bracketFill);
+  m_bmEnd->setAttribute(bracketFill);
+#endif
 
   setMinimumSize (0,0);
 
   // cursor
-  m_cursor.setInsertBehaviour (KTextEditor::SmartCursor::MoveOnInsert);
+  m_cursor.setInsertBehavior (KTextEditor::SmartCursor::MoveOnInsert);
   m_cursor.setInternal();
 
   m_startPos.setInternal();
@@ -126,12 +130,12 @@ KateViewInternal::KateViewInternal(KateView *view, KateDocument *doc)
   m_lineScroll = new KateScrollBar(Qt::Vertical, this);
   m_lineScroll->show();
   m_lineScroll->setTracking (true);
-  m_lineScroll->setSizePolicy( QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding));
+  m_lineScroll->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Expanding );
 
   // bottom corner box
   m_dummy = new QWidget(m_view);
   m_dummy->setFixedSize(m_lineScroll->width(), m_lineScroll->width());
-  m_dummy->setSizePolicy( QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
+  m_dummy->setSizePolicy( QSizePolicy::Fixed, QSizePolicy::Fixed );
 
   if (m_view->dynWordWrap())
     m_dummy->hide();
@@ -143,6 +147,7 @@ KateViewInternal::KateViewInternal(KateView *view, KateDocument *doc)
   // Hijack the line scroller's controls, so we can scroll nicely for word-wrap
   connect(m_lineScroll, SIGNAL(actionTriggered(int)), SLOT(scrollAction(int)));
   connect(m_lineScroll, SIGNAL(sliderMoved(int)), SLOT(scrollLines(int)));
+  connect(m_lineScroll, SIGNAL(sliderMMBMoved(int)), SLOT(scrollLines(int)));
 
   // catch wheel events, completing the hijack
   //m_lineScroll->installEventFilter(this);
@@ -177,7 +182,7 @@ KateViewInternal::KateViewInternal(KateView *view, KateDocument *doc)
            this, SLOT(slotCodeFoldingChanged()) );
 
   m_displayCursor.setPosition(0, 0);
-  m_cursor.setInsertBehaviour(KTextEditor::SmartCursor::MoveOnInsert);
+  m_cursor.setInsertBehavior(KTextEditor::SmartCursor::MoveOnInsert);
   m_cursorX = 0;
 
   setAcceptDrops( true );
@@ -234,6 +239,13 @@ void removeWatcher(KTextEditor::SmartRange* range, KTextEditor::SmartRangeWatche
     removeWatcher(child, watcher);
 }
 
+void addWatcher(KTextEditor::SmartRange* range, KTextEditor::SmartRangeWatcher* watcher)
+{
+  range->addWatcher(watcher);
+  foreach (KTextEditor::SmartRange* child, range->childRanges())
+    addWatcher(child, watcher);
+}
+
 KateViewInternal::~KateViewInternal ()
 {
   // crashes on close without
@@ -243,6 +255,11 @@ KateViewInternal::~KateViewInternal ()
     removeWatcher(dynamic->top, this);
 
   qDeleteAll(m_dynamicHighlights);
+
+  // FIXME
+  //delete m_bmEnd;
+  //delete m_bmStart;
+  //delete m_bm;
 }
 
 void KateViewInternal::prepareForDynWrapChange()
@@ -362,7 +379,7 @@ void KateViewInternal::scrollViewLines(int offset)
 
 void KateViewInternal::scrollAction( int action )
 {
-  switch (action) {
+  switch  (action) {
     case QAbstractSlider::SliderSingleStepAdd:
       scrollNextLine();
       break;
@@ -380,11 +397,11 @@ void KateViewInternal::scrollAction( int action )
       break;
 
     case QAbstractSlider::SliderToMinimum:
-      topOfView();
+      top_home();
       break;
 
     case QAbstractSlider::SliderToMaximum:
-      bottomOfView();
+      bottom_end();
       break;
   }
 }
@@ -519,6 +536,22 @@ void KateViewInternal::scrollColumns ( int x )
 // If changed is true, the lines that have been set dirty have been updated.
 void KateViewInternal::updateView(bool changed, int viewLinesScrolled)
 {
+#ifndef KTEXTEDITOR_NO_SMART_THREADSAFE
+  QMutexLocker lock(m_doc->smartMutex());
+#endif
+
+  doUpdateView(changed, viewLinesScrolled);
+
+  if (changed)
+    updateDirty(); //paintText(0, 0, width(), height(), true);
+}
+
+void KateViewInternal::doUpdateView(bool changed, int viewLinesScrolled)
+{
+#ifndef KTEXTEDITOR_NO_SMART_THREADSAFE
+  Q_ASSERT(m_doc->isSmartLocked() || thread() == QThread::currentThread());
+#endif
+
   m_updatingView = true;
 
   if (changed)
@@ -571,10 +604,10 @@ void KateViewInternal::updateView(bool changed, int viewLinesScrolled)
     m_columnScroll->blockSignals(false);
   }
 
-  m_updatingView = false;
+  if (m_smartDirty)
+    m_smartDirty = false;
 
-  if (changed)
-    updateDirty(); //paintText(0, 0, width(), height(), true);
+  m_updatingView = false;
 }
 
 /**
@@ -671,7 +704,9 @@ int KateViewInternal::linesDisplayed() const
   int h = height();
   int fh = renderer()->fontHeight();
 
-  return (h - (h % fh)) / fh;
+  // default to 1, there is always one line around....
+  // too many places calc with linesDisplayed() - 1
+  return qMax (1, (h - (h % fh)) / fh);
 }
 
 QPoint KateViewInternal::cursorToCoordinate( const KTextEditor::Cursor & cursor, bool realCursor ) const
@@ -722,27 +757,18 @@ QVariant KateViewInternal::inputMethodQuery ( Qt::InputMethodQuery query ) const
 
 void KateViewInternal::doReturn()
 {
-  KTextEditor::Cursor c = m_cursor;
-  m_doc->newLine( c, view() );
-  // TODO Might not be needed
-  updateCursor( c );
+  m_doc->newLine( view() );
   updateView();
 }
 
 void KateViewInternal::doDelete()
 {
   m_doc->del( m_view, m_cursor );
-  if (m_view->m_codeCompletion->codeCompletionVisible()) {
-    m_view->m_codeCompletion->updateBox();
-  }
 }
 
 void KateViewInternal::doBackspace()
 {
   m_doc->backspace( m_view, m_cursor );
-  if (m_view->m_codeCompletion->codeCompletionVisible()) {
-    m_view->m_codeCompletion->updateBox();
-  }
 }
 
 void KateViewInternal::doTranspose()
@@ -993,17 +1019,11 @@ void KateViewInternal::cursorLeft(  bool sel )
     return;
 
   moveChar( KateViewInternal::left, sel );
-  if (m_view->m_codeCompletion->codeCompletionVisible()) {
-    m_view->m_codeCompletion->updateBox();
-  }
 }
 
 void KateViewInternal::cursorRight( bool sel )
 {
   moveChar( KateViewInternal::right, sel );
-  if (m_view->m_codeCompletion->codeCompletionVisible()) {
-    m_view->m_codeCompletion->updateBox();
-  }
 }
 
 void KateViewInternal::wordLeft ( bool sel )
@@ -1100,17 +1120,6 @@ void KateViewInternal::moveEdge( KateViewInternal::Bias bias, bool sel )
 
 void KateViewInternal::home( bool sel )
 {
-  if (m_view->isCompletionActive()) {
-    m_view->completionWidget()->top();
-    return;
-  }
-
-  if (m_view->m_codeCompletion->codeCompletionVisible()) {
-    QKeyEvent e(QEvent::KeyPress, Qt::Key_Home, 0);
-    m_view->m_codeCompletion->handleKey(&e);
-    return;
-  }
-
   if (m_view->dynWordWrap() && currentLayout().startCol()) {
     // Allow us to go to the real start if we're already at the start of the view line
     if (m_cursor.column() != currentLayout().startCol()) {
@@ -1151,25 +1160,16 @@ void KateViewInternal::end( bool sel )
     return;
   }
 
-  if (m_view->m_codeCompletion->codeCompletionVisible()) {
-    QKeyEvent e(QEvent::KeyPress, Qt::Key_End, 0);
-    m_view->m_codeCompletion->handleKey(&e);
-    return;
-  }
+  KateTextLayout layout = currentLayout();
 
-  // FIXME: Both "smart end" and "smart home" use the current range's last/first character
-  //        when jumping to the "absolute" extreme. For 4.0 and 3.5.5 (kling)
-  bool alreadyAtEndOfLine = false;
-
-  if (m_view->dynWordWrap() && currentLayout().wrap()) {
+  if (m_view->dynWordWrap() && layout.wrap()) {
     // Allow us to go to the real end if we're already at the end of the view line
-    if (m_cursor.column() < currentLayout().endCol() - 1) {
-      KTextEditor::Cursor c(m_cursor.line(), currentLayout().endCol() - 1);
+    if (m_cursor.column() < layout.endCol() - 1) {
+      KTextEditor::Cursor c(m_cursor.line(), layout.endCol() - 1);
       updateSelection( c, sel );
       updateCursor( c );
       return;
     }
-    alreadyAtEndOfLine = true;
   }
 
   if( !(m_doc->config()->configFlags() & KateDocumentConfig::cfSmartHome) ) {
@@ -1183,21 +1183,14 @@ void KateViewInternal::end( bool sel )
     return;
 
   // "Smart End", as requested in bugs #78258 and #106970
-  KTextEditor::Cursor c = m_cursor;
-  int lc = l->lastChar();
-
-  if (lc < 0 || c.column() == (lc + 1)) {
-    if (alreadyAtEndOfLine) {
-      moveEdge(right, sel);
-      return;
-    }
-    c.setColumn(currentLayout().endCol());
+  if (m_cursor.column() == m_doc->lineLength(m_cursor.line())) {
+    KTextEditor::Cursor c = m_cursor;
+    c.setColumn(l->lastChar() + 1);
+    updateSelection(c, sel);
+    updateCursor(c, true);
   } else {
-    c.setColumn(lc + 1);
+    moveEdge(right, sel);
   }
-
-  updateSelection( c, sel );
-  updateCursor( c, true );
 }
 
 KateTextLayout KateViewInternal::currentLayout() const
@@ -1380,12 +1373,6 @@ void KateViewInternal::cursorUp(bool sel)
     return;
   }
 
-  if (m_view->m_codeCompletion->codeCompletionVisible()) {
-    QKeyEvent e(QEvent::KeyPress, Qt::Key_Up, 0);
-    m_view->m_codeCompletion->handleKey(&e);
-    return;
-  }
-
   if (m_displayCursor.line() == 0 && (!m_view->dynWordWrap() || cache()->viewLine(m_cursor) == 0))
     return;
 
@@ -1419,12 +1406,6 @@ void KateViewInternal::cursorDown(bool sel)
     return;
   }
 
-  if (m_view->m_codeCompletion->codeCompletionVisible()) {
-    QKeyEvent e(QEvent::KeyPress, Qt::Key_Down, 0);
-    m_view->m_codeCompletion->handleKey(&e);
-    return;
-  }
-
   if ((m_displayCursor.line() >= (int)m_doc->numVisLines() - 1) && (!m_view->dynWordWrap() || cache()->viewLine(m_cursor) == cache()->lastViewLine(m_cursor.line())))
     return;
 
@@ -1523,12 +1504,6 @@ void KateViewInternal::pageUp( bool sel )
     return;
   }
 
-  if (m_view->m_codeCompletion->codeCompletionVisible()) {
-    QKeyEvent e(QEvent::KeyPress, Qt::Key_PageUp, 0);
-    m_view->m_codeCompletion->handleKey(&e);
-    return;
-  }
-
   // remember the view line and x pos
   int viewLine = cache()->displayViewLine(m_displayCursor);
   bool atTop = startPos().atStartOfDocument();
@@ -1574,12 +1549,6 @@ void KateViewInternal::pageDown( bool sel )
     return;
   }
 
-  if (m_view->m_codeCompletion->codeCompletionVisible()) {
-    QKeyEvent e(QEvent::KeyPress, Qt::Key_PageDown, 0);
-    m_view->m_codeCompletion->handleKey(&e);
-    return;
-  }
-
   // remember the view line
   int viewLine = cache()->displayViewLine(m_displayCursor);
   bool atEnd = startPos() >= m_cachedMaxStartPos;
@@ -1678,12 +1647,6 @@ void KateViewInternal::top_home( bool sel )
     return;
   }
 
-  if (m_view->m_codeCompletion->codeCompletionVisible()) {
-    QKeyEvent e(QEvent::KeyPress, Qt::Key_Home, 0);
-    m_view->m_codeCompletion->handleKey(&e);
-    return;
-  }
-
   KTextEditor::Cursor c( 0, 0 );
   updateSelection( c, sel );
   updateCursor( c );
@@ -1696,12 +1659,6 @@ void KateViewInternal::bottom_end( bool sel )
     return;
   }
 
-  if (m_view->m_codeCompletion->codeCompletionVisible()) {
-    QKeyEvent e(QEvent::KeyPress, Qt::Key_End, 0);
-    m_view->m_codeCompletion->handleKey(&e);
-    return;
-  }
-
   KTextEditor::Cursor c( m_doc->lastLine(), m_doc->lineLength( m_doc->lastLine() ) );
   updateSelection( c, sel );
   updateCursor( c );
@@ -1748,7 +1705,7 @@ void KateViewInternal::updateSelection( const KTextEditor::Cursor& _newCursor, b
 
             KateTextLine::Ptr l = m_doc->kateTextLine( newCursor.line() );
 
-            for ( c = newCursor.column(); c > 0; c-- )
+            for ( c = newCursor.column(); c >= 0; c-- )
               if ( !m_doc->highlight()->isInWord( l->at( c ) ) )
                 break;
 
@@ -1813,7 +1770,7 @@ void KateViewInternal::updateCursor( const KTextEditor::Cursor& newCursor, bool
 {
   if ( !force && (m_cursor == newCursor) )
   {
-    if ( !m_madeVisible )
+    if ( !m_madeVisible && m_view == m_doc->activeView() )
     {
       // unfold if required
       m_doc->foldingTree()->ensureVisible( newCursor.line() );
@@ -1833,7 +1790,8 @@ void KateViewInternal::updateCursor( const KTextEditor::Cursor& newCursor, bool
   m_displayCursor = toVirtualCursor(m_cursor);
 
   m_cursorX = renderer()->cursorToX(cache()->textLayout(m_cursor), m_cursor);
-  makeVisible ( m_displayCursor, m_displayCursor.column(), false, center, calledExternally );
+  if ( m_view == m_doc->activeView() )
+    makeVisible ( m_displayCursor, m_displayCursor.column(), false, center, calledExternally );
 
   updateBracketMarks();
 
@@ -1868,34 +1826,36 @@ void KateViewInternal::updateCursor( const KTextEditor::Cursor& newCursor, bool
 
 void KateViewInternal::updateBracketMarks()
 {
-  if ( m_bm.isValid() ) {
-    tagRange(m_bm, true);
-    tagRange(m_bmStart, true);
-    tagRange(m_bmEnd, true);
+#if 0
+  if ( m_bm->isValid() ) {
+    tagRange(*m_bm, true);
+    tagRange(*m_bmStart, true);
+    tagRange(*m_bmEnd, true);
   }
 
-  //m_bmStart.setValid(false);
-  //m_bmEnd.setValid(false);
+  //m_bmStart->setValid(false);
+  //m_bmEnd->setValid(false);
 
   // add some limit to this, this is really endless on big files without limit
   int maxLines = linesDisplayed () * 3;
-  m_doc->newBracketMark( m_cursor, m_bm, maxLines );
+  m_doc->newBracketMark( m_cursor, *m_bm, maxLines );
 
-  if ( m_bm.isValid() ) {
-    m_bmStart.start() = m_bm.start();
-    m_bmStart.end().setPosition(m_bm.start().line(), m_bm.start().column() + 1);
-    //m_bmStart.setValid(true);
+  if ( m_bm->isValid() ) {
+    m_bmStart->start() = m_bm->start();
+    m_bmStart->end().setPosition(m_bm->start().line(), m_bm->start().column() + 1);
+    //m_bmStart->setValid(true);
 
-    m_bmEnd.start() = m_bm.end();
-    m_bmEnd.end().setPosition(m_bm.end().line(), m_bmEnd.end().column() + 1);
-    //m_bmEnd.setValid(true);
+    m_bmEnd->start() = m_bm->end();
+    m_bmEnd->end().setPosition(m_bm->end().line(), m_bmEnd->end().column() + 1);
+    //m_bmEnd->setValid(true);
 
-    m_bm.end().setColumn(m_bm.end().column() + 1);
+    m_bm->end().setColumn(m_bm->end().column() + 1);
 
-    tagRange(m_bm, true);
-    tagRange(m_bmStart, true);
-    tagRange(m_bmEnd, true);
+    tagRange(*m_bm, true);
+    tagRange(*m_bmStart, true);
+    tagRange(*m_bmEnd, true);
   }
+#endif
 }
 
 bool KateViewInternal::tagLine(const KTextEditor::Cursor& virtualCursor)
@@ -2058,29 +2018,19 @@ bool KateViewInternal::eventFilter( QObject *obj, QEvent *e )
   {
     case QEvent::KeyPress:
     {
-      QKeyEvent *k = (QKeyEvent *)e;
+      QKeyEvent *k = static_cast<QKeyEvent *>(e);
 
-
-      if (m_view->isCompletionActive())
-      {
-        //kDebug (13030) << "hint around" << endl;
-
-        if( k->key() == Qt::Key_Escape )
+      if (k->key() == Qt::Key_Escape) {
+        if (m_view->isCompletionActive()) {
           m_view->abortCompletion();
-      }
-
-      if( ! m_view->m_codeCompletion->codeCompletionVisible() )
-      {
-        //kDebug (13030) << "hint around" << endl;
-
-        if( k->key() == Qt::Key_Escape )
-          m_view->m_codeCompletion->abortCompletion();
-      }
-
-      if ((k->key() == Qt::Key_Escape) && !m_view->config()->persistentSelection() )
-      {
-        m_view->clearSelection();
-        return true;
+          return true;
+        } else if (m_view->m_viewBar->isVisible()) {
+          m_view->m_viewBar->hide();
+          return true;
+        } else if (!m_view->config()->persistentSelection()) {
+          m_view->clearSelection();
+          return true;
+        }
       }
       else if ( !((k->modifiers() & Qt::ControlModifier) || (k->modifiers() & Qt::AltModifier)) )
       {
@@ -2130,8 +2080,6 @@ void KateViewInternal::keyPressEvent( QKeyEvent* e )
 {
   int key = e->key() | e->modifiers();
 
-  bool codeComp = m_view->m_codeCompletion->codeCompletionVisible ();
-
   if (m_view->isCompletionActive())
   {
     if( key == Qt::Key_Enter || key == Qt::Key_Return  ||
@@ -2142,18 +2090,6 @@ void KateViewInternal::keyPressEvent( QKeyEvent* e )
     }
   }
 
-  if (codeComp)
-  {
-    //kDebug (13030) << "hint around" << endl;
-
-    if( key == Qt::Key_Enter || key == Qt::Key_Return  ||
-    (key == Qt::SHIFT + Qt::Key_Return) || (key == Qt::SHIFT + Qt::Key_Enter)) {
-      m_view->m_codeCompletion->doComplete();
-      e->accept();
-      return;
-    }
-  }
-
   if( !m_doc->isReadWrite() )
   {
     e->ignore();
@@ -2182,7 +2118,7 @@ void KateViewInternal::keyPressEvent( QKeyEvent* e )
       pos = line->length(); // stay indented
     }
     m_doc->editStart();
-    m_doc->insertText( KTextEditor::Cursor(m_cursor.line(), line->length()), "\n" +  line->string(0, pos)
+    m_doc->insertText( KTextEditor::Cursor(m_cursor.line(), line->length()), '\n' +  line->string(0, pos)
       + line->string().right( line->length() - m_cursor.column() ) );
     m_cursor.setPosition(KTextEditor::Cursor(ln + 1, pos));
     if (col < int(line->length()))
@@ -2200,9 +2136,6 @@ void KateViewInternal::keyPressEvent( QKeyEvent* e )
     //m_view->backspace();
     e->accept();
 
-    if (codeComp)
-      m_view->m_codeCompletion->updateBox ();
-
     return;
   }
 
@@ -2244,9 +2177,6 @@ void KateViewInternal::keyPressEvent( QKeyEvent* e )
 
       e->accept();
 
-      if (codeComp)
-        m_view->m_codeCompletion->updateBox ();
-
       return;
     }
     else if (m_doc->config()->tabHandling() != KateDocumentConfig::tabInsertsTab)
@@ -2255,20 +2185,14 @@ void KateViewInternal::keyPressEvent( QKeyEvent* e )
       m_doc->indent( m_view, m_cursor.line(), -1 );
       e->accept();
 
-      if (codeComp)
-        m_view->m_codeCompletion->updateBox ();
-
       return;
     }
-}
-  if ( !(e->modifiers() & Qt::ControlModifier) && !(e->modifiers() & Qt::AltModifier)
-       && m_doc->typeChars ( m_view, e->text() ) )
+  }
+
+  if ( !(e->modifiers() & Qt::ControlModifier) && !e->text().isEmpty() && m_doc->typeChars ( m_view, e->text() ) )
   {
     e->accept();
 
-    if (codeComp)
-      m_view->m_codeCompletion->updateBox ();
-
     return;
   }
 
@@ -2608,6 +2532,13 @@ void KateViewInternal::updateDirty( )
 
 void KateViewInternal::paintEvent(QPaintEvent *e)
 {
+#ifndef KTEXTEDITOR_NO_SMART_THREADSAFE
+  QMutexLocker lock(m_doc->smartMutex());
+
+  if (m_smartDirty)
+    doUpdateView();
+#endif
+
   //kDebug (13030) << "GOT PAINT EVENT: x: " << e->rect().x() << " y: " << e->rect().y()
   //  << " width: " << e->rect().width() << " height: " << e->rect().height() << endl;
 
@@ -2619,10 +2550,12 @@ void KateViewInternal::paintEvent(QPaintEvent *e)
   uint lineRangesSize = cache()->viewCacheLineCount();
 
   QPainter paint(this);
+  paint.setRenderHints (QPainter::Antialiasing);
 
   // TODO put in the proper places
   renderer()->setCaretStyle(m_view->isOverwriteMode() ? KateRenderer::Replace : KateRenderer::Insert);
   renderer()->setShowTabs(m_doc->config()->configFlags() & KateDocumentConfig::cfShowTabs);
+  renderer()->setShowTrailingSpaces(m_doc->config()->configFlags() & KateDocumentConfig::cfShowSpaces);
 
   int sy = startz * h;
   paint.translate(e->rect().x(), startz * h);
@@ -2779,17 +2712,9 @@ void KateViewInternal::focusOutEvent (QFocusEvent *)
   if (m_view->isCompletionActive())
     m_view->abortCompletion();
 
-  if( ! m_view->m_codeCompletion->codeCompletionVisible() )
-  {
-    m_cursorTimer.stop();
-
-    renderer()->setDrawCaret(true);
-
-    paintCursor();
-
-    // this will handle focus stuff in kateview
-    m_view->slotLostFocus ();
-  }
+  m_cursorTimer.stop();
+  m_view->renderer()->setDrawCaret(true);
+  paintCursor();
 
   m_textHintTimer.stop();
 }
@@ -3011,11 +2936,8 @@ void KateViewInternal::editEnd(int editTagLineStart, int editTagLineEnd, bool ta
     m_madeVisible = false;
     updateCursor ( m_cursor, true );
   }
-  else //if ( m_view->isActive() )
+  else if ( m_view == m_doc->activeView() )
   {
-#ifdef __GNUC__
-#warning fixme, this needs to be fixed app transparent to only be done if this view is the view where the editing did happen
-#endif
     makeVisible(m_displayCursor, m_displayCursor.column());
   }
 
@@ -3044,6 +2966,14 @@ void KateViewInternal::inputMethodEvent(QInputMethodEvent* e)
     return;
   }
 
+  if ( m_view->selection() )
+    m_view->removeSelectedText();
+
+  if ( !e->commitString().isEmpty() ) {
+      m_doc->insertText( m_cursor, e->commitString() );
+  }
+
+#if 0
   if ( m_view->selection() )
     m_view->removeSelectedText();
 
@@ -3053,6 +2983,7 @@ void KateViewInternal::inputMethodEvent(QInputMethodEvent* e)
   m_view->setIMSelectionValue( m_imPreedit, m_imPreedit, true );
 
   QWidget::inputMethodEvent(e);
+#endif
 }
 
 KateLayoutCache* KateViewInternal::cache( ) const
@@ -3080,6 +3011,8 @@ void KateViewInternal::dynamicHighlightAdded( KateSmartRange * range )
   DynamicRangeHL* hl = new DynamicRangeHL(range);
   hl->isView = view() == sender();
 
+  addWatcher(range, this);
+
   m_dynamicHighlights.insert(range, hl);
 
   if (m_mouse.isValid())
@@ -3091,6 +3024,8 @@ void KateViewInternal::dynamicHighlightAdded( KateSmartRange * range )
 
 void KateViewInternal::dynamicHighlightRemoved( KateSmartRange * range )
 {
+  removeWatcher(range, this);
+
   delete m_dynamicHighlights.take(range);
 }
 
@@ -3239,10 +3174,19 @@ void KateViewInternal::cursorMoved( )
 
 void KateViewInternal::relayoutRange( const KTextEditor::Range & range, bool realCursors )
 {
+
   int startLine = realCursors ? range.start().line() : toRealCursor(range.start()).line();
   int endLine = realCursors ? range.end().line() : toRealCursor(range.end()).line();
   cache()->relayoutLines(startLine, endLine);
+#ifndef KTEXTEDITOR_NO_SMART_THREADSAFE
+  Q_ASSERT(m_doc->isSmartLocked() || thread() == QThread::currentThread());
+  if (!m_smartDirty) {
+    m_smartDirty = true;
+    emit requestViewUpdate();
+  }
+#else
   emit requestViewUpdate();
+#endif
 }
 
 void KateViewInternal::rangePositionChanged( KTextEditor::SmartRange * range )
@@ -3258,6 +3202,7 @@ void KateViewInternal::rangeDeleted( KTextEditor::SmartRange * range )
 void KateViewInternal::childRangeInserted( KTextEditor::SmartRange *, KTextEditor::SmartRange * child )
 {
   relayoutRange(*child);
+  // This is already a dynamically highlighted range if we're watching it, thus add a watcher
   child->addWatcher(this);
 }
 
@@ -3270,19 +3215,24 @@ void KateViewInternal::rangeAttributeChanged( KTextEditor::SmartRange * range, K
 void KateViewInternal::childRangeRemoved( KTextEditor::SmartRange *, KTextEditor::SmartRange * child )
 {
   relayoutRange(*child);
+  // This is already a dynamically highlighted range if we're watching it, thus remove the watcher
   child->removeWatcher(this);
 }
 
 void KateViewInternal::addHighlightRange(KTextEditor::SmartRange* range)
 {
-  range->addWatcher(this);
+  relayoutRange(*range);
+  // Watchers are only added for dynamically highlighted ranges
+  //range->addWatcher(this);
   foreach (KTextEditor::SmartRange* child, range->childRanges())
     addHighlightRange(child);
 }
 
 void KateViewInternal::removeHighlightRange(KTextEditor::SmartRange* range)
 {
-  range->removeWatcher(this);
+  relayoutRange(*range);
+  // Watchers are only removed for dynamically highlighted ranges
+  //range->removeWatcher(this);
   foreach (KTextEditor::SmartRange* child, range->childRanges())
     removeHighlightRange(child);
 }
