@@ -18,13 +18,6 @@
    Boston, MA 02110-1301, USA.
 */
 
-// remove when QVBoxLayout::setAutoAdd is ported
-#define QT3_SUPPORT
-#define QT3_SUPPORT_WARNINGS
-#ifdef __GNUC__
-#warning TODO: remove QT3_SUPPORT
-#endif
-
 #include "katetest.h"
 #include "katetest.moc"
 
@@ -38,7 +31,7 @@
 #include <kio/netaccess.h>
 
 #include <kdeversion.h>
-#include <kaboutapplication.h>
+#include <kaboutapplicationdialog.h>
 #include <kicon.h>
 #include <kencodingfiledialog.h>
 #include <kdiroperator.h>
@@ -46,7 +39,7 @@
 #include <kstandarddirs.h>
 #include <kaboutdata.h>
 #include <kstatusbar.h>
-#include <kstdaction.h>
+#include <kstandardaction.h>
 #include <kaction.h>
 #include <kdebug.h>
 #include <kglobal.h>
@@ -60,11 +53,7 @@
 #include <kedittoolbar.h>
 #include <kparts/event.h>
 #include <kmenubar.h>
-#include <kiconloader.h>
-#include <klocale.h>
 #include <ksqueezedtextlabel.h>
-#include <kconfig.h>
-#include <kdebug.h>
 #include <kstringhandler.h>
 #include <krecentfilesaction.h>
 #include <kactioncollection.h>
@@ -124,7 +113,7 @@ KWrite::KWrite (KTextEditor::Document *doc)
     docList.append(doc);
   }
 
-  new ArbitraryHighlightTest(doc);
+  //new ArbitraryHighlightTest(doc);
 
   m_view = qobject_cast<KTextEditor::View*>(doc->createView (this));
 
@@ -190,45 +179,59 @@ KWrite::~KWrite()
 
 void KWrite::setupActions()
 {
-  KStdAction::close( this, SLOT(slotFlush()), actionCollection(), "file_close" )->setWhatsThis(i18n("Use this to close the current document"));
+    actionCollection()->addAction( KStandardAction::Close, "file_close", this, SLOT(slotFlush()) )
+        ->setWhatsThis(i18n("Use this to close the current document"));
 
-  // setup File menu
-  KStdAction::print(this, SLOT(printDlg()), actionCollection())->setWhatsThis(i18n("Use this command to print the current document"));
-  KStdAction::openNew( this, SLOT(slotNew()), actionCollection(), "file_new" )->setWhatsThis(i18n("Use this command to create a new document"));
-  KStdAction::open( this, SLOT( slotOpen() ), actionCollection(), "file_open" )->setWhatsThis(i18n("Use this command to open an existing document for editing"));
+    // setup File menu
+    actionCollection()->addAction( KStandardAction::Print, this, SLOT(printDlg()) )
+        ->setWhatsThis(i18n("Use this command to print the current document"));
+    actionCollection()->addAction( KStandardAction::New, "file_new", this, SLOT(slotNew()) )
+        ->setWhatsThis(i18n("Use this command to create a new document"));
+    actionCollection()->addAction( KStandardAction::Open, "file_open", this, SLOT(slotOpen()) )
+        ->setWhatsThis(i18n("Use this command to open an existing document for editing"));
 
-  m_recentFiles = KStdAction::openRecent(this, SLOT(slotOpen(const KUrl&)),
-                                         actionCollection());
-  m_recentFiles->setWhatsThis(i18n("This lists files which you have opened recently, and allows you to easily open them again."));
+    m_recentFiles = KStandardAction::openRecent(this, SLOT(slotOpen(const KUrl&)), this );
+    actionCollection()->addAction(m_recentFiles->objectName(), m_recentFiles);
+    m_recentFiles->setWhatsThis(i18n("This lists files which you have opened recently, and allows you to easily open them again."));
 
-  KAction *a = new KAction(KIcon("window_new"), i18n("&New Window"), actionCollection(), "view_new_view");
-  connect(a, SIGNAL(triggered(bool)), SLOT(newView()));
-  a->setWhatsThis(i18n("Create another view containing the current document"));
+    QAction *a = new KAction(KIcon("window_new"), i18n("&New Window"), this);
+    actionCollection()->addAction("view_new_view", a);
+    connect(a, SIGNAL(triggered(bool)), SLOT(newView()));
+    a->setWhatsThis(i18n("Create another view containing the current document"));
 
-  a=new KAction(i18n("Choose Editor..."), actionCollection(),"settings_choose_editor");
-  connect(a, SIGNAL(triggered(bool)), SLOT(changeEditor()));
-  a->setWhatsThis(i18n("Override the system wide setting for the default editing component"));
+    a=new KAction(i18n("Choose Editor..."), this);
+    actionCollection()->addAction("settings_choose_editor", a);
+    connect(a, SIGNAL(triggered(bool)), SLOT(changeEditor()));
+    a->setWhatsThis(i18n("Override the system wide setting for the default editing component"));
 
-  KStdAction::quit(this, SLOT(close()), actionCollection())->setWhatsThis(i18n("Close the current document view"));
+    actionCollection()->addAction( KStandardAction::Quit, this, SLOT(close()) )
+        ->setWhatsThis(i18n("Close the current document view"));
 
-  // setup Settings menu
-  setStandardToolBarMenuEnabled(true);
+    // setup Settings menu
+    setStandardToolBarMenuEnabled(true);
 
-  m_paShowStatusBar = KStdAction::showStatusbar(this, SLOT(toggleStatusBar()), actionCollection(), "settings_show_statusbar");
-  m_paShowStatusBar->setWhatsThis(i18n("Use this command to show or hide the view's statusbar"));
+    m_paShowStatusBar = KStandardAction::showStatusbar(this, SLOT(toggleStatusBar()), this);
+    actionCollection()->addAction("settings_show_statusbar", m_paShowStatusBar);
+    m_paShowStatusBar->setWhatsThis(i18n("Use this command to show or hide the view's statusbar"));
 
-  m_paShowPath = new KToggleAction(i18n("Sho&w Path"), 0, this, SLOT(documentNameChanged()),
-                    actionCollection(), "set_showPath");
-  m_paShowPath->setCheckedState(i18n("Hide Path"));
-  m_paShowPath->setWhatsThis(i18n("Show the complete document path in the window caption"));
-  a=KStdAction::keyBindings(this, SLOT(editKeys()), actionCollection());
-  a->setWhatsThis(i18n("Configure the application's keyboard shortcut assignments."));
+    m_paShowPath = new KToggleAction(i18n("Sho&w Path"), this);
+    actionCollection()->addAction("set_showPath", m_paShowPath);
+    m_paShowPath->setShortcuts(KShortcut());
+    m_paShowPath->setCheckedState(KGuiItem(i18n("Hide Path")));
+    m_paShowPath->setWhatsThis(i18n("Show the complete document path in the window caption"));
+    connect(m_paShowPath, SIGNAL(triggered()), this, SLOT(documentNameChanged()));
 
-  a=KStdAction::configureToolbars(this, SLOT(editToolbars()), actionCollection(), "set_configure_toolbars");
-  a->setWhatsThis(i18n("Configure which items should appear in the toolbar(s)."));
+    a=actionCollection()->addAction( KStandardAction::KeyBindings, this, SLOT(editKeys()));
+    a->setWhatsThis(i18n("Configure the application's keyboard shortcut assignments."));
 
-  a=new KAction(i18n("&About Editor Component"),0,this,SLOT(aboutEditor()),actionCollection(),"help_about_editor");
+    a=actionCollection()->addAction( KStandardAction::ConfigureToolbars, "set_configure_toolbars",
+                                     this, SLOT(editToolbars()) );
+    a->setWhatsThis(i18n("Configure which items should appear in the toolbar(s)."));
 
+    a=new KAction(this);
+    actionCollection()->addAction("help_about_editor", a);
+    a->setText(i18n("&About Editor Component"));
+    connect(a, SIGNAL(triggered()), this, SLOT(aboutEditor()));
 }
 
 void KWrite::setupStatusBar()
@@ -265,7 +268,7 @@ void KWrite::setupStatusBar()
 // load on url
 void KWrite::loadURL(const KUrl &url)
 {
-  m_view->document()->openURL(url);
+  m_view->document()->openUrl(url);
 }
 
 // is closing the window wanted by user ?
@@ -292,7 +295,7 @@ void KWrite::changeEditor()
 
 void KWrite::slotFlush ()
 {
-   m_view->document()->closeURL();
+   m_view->document()->closeUrl();
 }
 
 void KWrite::slotNew()
@@ -300,12 +303,12 @@ void KWrite::slotNew()
   if (m_view->document()->isModified() || !m_view->document()->url().isEmpty())
     new KWrite();
   else
-    m_view->document()->openURL(KUrl());
+    m_view->document()->openUrl(KUrl());
 }
 
 void KWrite::slotOpen()
 {
-	KEncodingFileDialog::Result r=KEncodingFileDialog::getOpenURLsAndEncoding(m_view->document()->encoding(), m_view->document()->url().url(),QString(),this,i18n("Open File"));
+	KEncodingFileDialog::Result r=KEncodingFileDialog::getOpenUrlsAndEncoding(m_view->document()->encoding(), m_view->document()->url().url(),QString(),this,i18n("Open File"));
 
   for (KUrl::List::Iterator i=r.URLs.begin(); i != r.URLs.end(); ++i)
   {
@@ -407,9 +410,9 @@ void KWrite::slotDropEvent( QDropEvent *event )
 
 void KWrite::slotEnableActions( bool enable )
 {
-  Q3ValueList<KAction *> actions = actionCollection()->actions();
-  Q3ValueList<KAction *>::ConstIterator it = actions.begin();
-  Q3ValueList<KAction *>::ConstIterator end = actions.end();
+  Q3ValueList<QAction *> actions = actionCollection()->actions();
+  Q3ValueList<QAction *>::ConstIterator it = actions.begin();
+  Q3ValueList<QAction *>::ConstIterator end = actions.end();
 
   for (; it != end; ++it )
       (*it)->setEnabled( enable );
@@ -457,14 +460,14 @@ void KWrite::writeConfig(KConfig *config)
 //config file
 void KWrite::readConfig()
 {
-  KConfig *config = KGlobal::config();
-  readConfig(config);
+  KSharedConfig::Ptr config = KGlobal::config();
+  readConfig(config.data());
 }
 
 void KWrite::writeConfig()
 {
-  KConfig *config = KGlobal::config();
-  writeConfig(config);
+  KSharedConfig::Ptr config = KGlobal::config();
+  writeConfig(config.data());
 }
 
 // session management
@@ -563,7 +566,7 @@ void KWrite::restore()
 
 void KWrite::aboutEditor()
 {
-  KAboutApplication *ad = new KAboutApplication (m_view->document()->editor()->aboutData(), this);
+  KAboutApplicationDialog *ad = new KAboutApplicationDialog (m_view->document()->editor()->aboutData(), this);
 
   ad->exec();
 
@@ -719,7 +722,7 @@ extern "C" KDE_EXPORT int main(int argc, char **argv)
   KApplication a;
 
   KGlobal::locale()->insertCatalog("katepart");
-#if 0 
+#if 0
   DCOPClient *client = KApplication::dcopClient();
   if (!client->isRegistered())
   {
@@ -770,7 +773,7 @@ extern "C" KDE_EXPORT int main(int argc, char **argv)
           do
           {
             line = input.readLine();
-            text.append( line + "\n" );
+            text.append( line + '\n' );
           } while( !line.isNull() );
 
 
@@ -830,10 +833,11 @@ KWriteEditorChooser::KWriteEditorChooser(QWidget *parent):
   setDefaultButton( KDialog::Cancel );
 
   QWidget *widget = new QWidget();
-	(new QVBoxLayout(widget))->setAutoAdd(true);
-	m_chooser=new KTextEditor::EditorChooser(widget);
-	setMainWidget(widget);
-	m_chooser->readAppSetting();
+  QVBoxLayout* layout = new QVBoxLayout(widget);
+  m_chooser=new KTextEditor::EditorChooser(widget);
+  layout->addWidget(m_chooser);
+  setMainWidget(widget);
+  m_chooser->readAppSetting();
 }
 
 KWriteEditorChooser::~KWriteEditorChooser() {
