@@ -20,7 +20,7 @@
 
 #include "kcertpart.h"
 #include <kparts/genericfactory.h>
-#include <kinstance.h>
+#include <kcomponentdata.h>
 #include <kaboutdata.h>
 #include <qframe.h>
 #include <klocale.h>
@@ -35,7 +35,7 @@
 #include <kopenssl.h>
 #include <ksslpemcallback.h>
 #include <kfiledialog.h>
-#include <kprocess.h>
+#include <qprocess.h>
 #include <qtabwidget.h>
 #include <kseparator.h>
 #include <k3listview.h>
@@ -128,12 +128,12 @@ class KCertPartPrivate {
 KCertPart::KCertPart(QWidget *parentWidget,
 					 QObject *parent,
 					 const QStringList & /*args*/ )
-	: KParts::ReadWritePart(parent),d(new KCertPartPrivate)
+    : KParts::ReadWritePart(parent),
+    d(new KCertPartPrivate)
 {
-
-	KInstance *instance = new KInstance("KCertPart");
-	QGridLayout *grid;
-	setInstance(instance);
+    KComponentData componentData("KCertPart");
+    setComponentData(componentData);
+    QGridLayout *grid;
 
 
 	_signers = new KSSLSigners;
@@ -189,8 +189,8 @@ KCertPart::KCertPart(QWidget *parentWidget,
 
 	grid->addWidget(new QLabel(i18n("Subject:"), _pkcsFrame), 2, 0);
 	grid->addWidget(new QLabel(i18n("Issued by:"), _pkcsFrame), 2, 3);
-	_p12_subject = KSSLInfoDlg::certInfoWidget(_pkcsFrame, QString(""));
-	_p12_issuer = KSSLInfoDlg::certInfoWidget(_pkcsFrame, QString(""));
+	_p12_subject = KSSLInfoDialog::certInfoWidget(_pkcsFrame, QString(""));
+	_p12_issuer = KSSLInfoDialog::certInfoWidget(_pkcsFrame, QString(""));
 	grid->addWidget(_p12_subject, 3, 0, 4, 3);
 	grid->addWidget(_p12_issuer, 3, 3, 4, 3);
 
@@ -280,8 +280,8 @@ KCertPart::KCertPart(QWidget *parentWidget,
 
 	grid->addWidget(new QLabel(i18n("Subject:"), _x509Frame), 1, 0);
 	grid->addWidget(new QLabel(i18n("Issued by:"), _x509Frame), 1, 3);
-	_ca_subject = KSSLInfoDlg::certInfoWidget(_x509Frame, QString(""));
-	_ca_issuer = KSSLInfoDlg::certInfoWidget(_x509Frame, QString(""));
+	_ca_subject = KSSLInfoDialog::certInfoWidget(_x509Frame, QString(""));
+	_ca_issuer = KSSLInfoDialog::certInfoWidget(_x509Frame, QString(""));
 	grid->addWidget(_ca_subject, 2, 0, 4, 3);
 	grid->addWidget(_ca_issuer, 2, 3, 4, 3);
 
@@ -491,18 +491,20 @@ bool KCertPart::openFile() {
 //		 x-pkcs12 loading
 /////////////////////////////////////////////////////////////////////////////
 		if (whatType == "application/x-pkcs12") {
-			QByteArray pass;
 			_p12 = KSSLPKCS12::loadCertFile(m_file);
 
 			while (!_p12) {
 				// try prompting for a password.
-				int rc = KPasswordDialog::getPassword(_frame,pass, i18n("Certificate Password"));
-				if (rc != KPasswordDialog::Accepted) break;
-
-				_p12 = KSSLPKCS12::loadCertFile(m_file, QString(pass));
+				KPasswordDialog dlg(_frame);
+				dlg.setCaption(i18n("Certificate Password"));
+				dlg.setPrompt(i18n("Certificate Password"));
+				if( !dlg.exec() )
+					break;
+				
+				_p12 = KSSLPKCS12::loadCertFile(m_file, dlg.password());
 
 				if (!_p12) {
-					rc = KMessageBox::warningContinueCancel(_frame, i18n("The certificate file could not be loaded. Try a different password?"), i18n("Certificate Import"),i18n("Try Different"));
+					int rc = KMessageBox::warningContinueCancel(_frame, i18n("The certificate file could not be loaded. Try a different password?"), i18n("Certificate Import"),KGuiItem(i18n("Try Different")));
 					if (rc == KMessageBox::Continue) continue;
 					break;
 				}
@@ -735,7 +737,7 @@ void KCertPart::slotImport() {
 
 		if (cfg.hasGroup(_p12->getCertificate()->getSubject())) {
 			QString msg = _curName + '\n' + i18n("A certificate with that name already exists. Are you sure that you wish to replace it?");
-			int rc= KMessageBox::warningContinueCancel(_frame, msg, i18n("Certificate Import"),i18n("Replace"));
+			int rc= KMessageBox::warningContinueCancel(_frame, msg, i18n("Certificate Import"),KGuiItem(i18n("Replace")));
 			if (rc == KMessageBox::Cancel) {
 				return;
 			}
@@ -751,7 +753,7 @@ void KCertPart::slotImport() {
 		KConfig cfg("ksslcalist", true, false);
 		if (cfg.hasGroup(_ca->getSubject())) {
 			QString msg = _curName + '\n' + i18n("A certificate with that name already exists. Are you sure that you wish to replace it?");
-			int rc= KMessageBox::warningContinueCancel(_frame, msg, i18n("Certificate Import"),i18n("Replace"));
+			int rc= KMessageBox::warningContinueCancel(_frame, msg, i18n("Certificate Import"),KGuiItem(i18n("Replace")));
 			if (rc == KMessageBox::Cancel) {
 				return;
 			}
@@ -781,9 +783,7 @@ void KCertPart::slotDone() {
 
 
 void KCertPart::slotLaunch() {
-	KShellProcess p;
-	p << "kcmshell" << "crypto";
-	p.start(KProcess::DontCare);
+	QProcess::startDetached("kcmshell", QStringList() << "crypto");
 }
 
 
