@@ -34,9 +34,10 @@
 #include <klineedit.h>
 #include <klocale.h>
 #include <kpushbutton.h>
-#include <kstdguiitem.h>
+#include <kstandardguiitem.h>
 
 #include "k3sconfig.h"
+#include <kconfiggroup.h>
 
 class K3SpellConfigPrivate
 {
@@ -47,7 +48,6 @@ public:
 
 K3SpellConfig::K3SpellConfig (const K3SpellConfig &_ksc)
   : QWidget(0), nodialog(true)
-  , kc(0)
   , cb0(0)
   , cb1(0)
   , cb2(0)
@@ -176,7 +176,7 @@ K3SpellConfig::K3SpellConfig( QWidget *parent,
 
   if( addHelpButton )
   {
-    QPushButton *pushButton = new KPushButton( KStdGuiItem::help(), this );
+    QPushButton *pushButton = new KPushButton( KStandardGuiItem::help(), this );
     connect( pushButton, SIGNAL(clicked()), this, SLOT(sHelp()) );
     glay->addWidget(pushButton, 6, 2);
   }
