@@ -34,6 +34,7 @@
 #include <QKeyEvent>
 
 #include "k3syntaxhighlighter.h"
+#include <kconfiggroup.h>
 
 static int dummy, dummy2, dummy3, dummy4;
 static int *Okay = &dummy;
@@ -315,7 +316,6 @@ K3DictSpellingHighlighter::K3DictSpellingHighlighter( Q3TextEdit *textEdit,
     }
 
     slotDictionaryChanged();
-    startTimer( 2 * 1000 );
 }
 
 K3DictSpellingHighlighter::~K3DictSpellingHighlighter()
