@@ -282,7 +282,7 @@ static QString enumName(const QString &n)
   return result;
 }
 
-static QString setFunction(const QString &n, const QString &className = QString::null)
+static QString setFunction(const QString &n, const QString &className = QString())
 {
   QString result = "set"+n;
   result[3] = result[3].toUpper();
@@ -293,7 +293,7 @@ static QString setFunction(const QString &n, const QString &className = QString:
 }
 
 
-static QString getFunction(const QString &n, const QString &className = QString::null)
+static QString getFunction(const QString &n, const QString &className = QString())
 {
   QString result = n;
   result[0] = result[0].toLower();
@@ -306,8 +306,8 @@ static QString getFunction(const QString &n, const QString &className = QString:
 
 static void addQuotes( QString &s )
 {
-  if ( s.left( 1 ) != "\"" ) s.prepend( "\"" );
-  if ( s.right( 1 ) != "\"" ) s.append( "\"" );
+  if ( !s.startsWith( '"' ) ) s.prepend( '"' );
+  if ( !s.endsWith( '"' ) ) s.append( '"' );
 }
 
 static QString quoteString( const QString &s )
@@ -344,7 +344,7 @@ static QString dumpNode(const QDomNode &node)
   return msg;
 }
 
-static QString filenameOnly(QString path)
+static QString filenameOnly(const QString& path)
 {
    int i = path.lastIndexOf(QRegExp("[/\\]"));
    if (i >= 0)
@@ -450,9 +450,7 @@ CfgEntry *parseEntry( const QString &group, const QDomElement &element )
   QString maxValue;
   int paramMax = 0;
 
-  QDomNode n;
-  for ( n = element.firstChild(); !n.isNull(); n = n.nextSibling() ) {
-    QDomElement e = n.toElement();
+  for ( QDomElement e = element.firstChildElement(); !e.isNull(); e = e.nextSiblingElement() ) {
     QString tag = e.tagName();
     if ( tag == "label" ) label = e.text();
     else if ( tag == "whatsthis" ) whatsThis = e.text();
@@ -484,14 +482,10 @@ CfgEntry *parseEntry( const QString &group, const QDomElement &element )
       }
       else if (paramType == "Enum")
       {
-         QDomNode n2;
-         for ( n2 = e.firstChild(); !n2.isNull(); n2 = n2.nextSibling() ) {
-           QDomElement e2 = n2.toElement();
+         for ( QDomElement e2 = e.firstChildElement(); !e2.isNull(); e2 = e2.nextSiblingElement() ) {
            if (e2.tagName() == "values")
            {
-             QDomNode n3;
-             for ( n3 = e2.firstChild(); !n3.isNull(); n3 = n3.nextSibling() ) {
-               QDomElement e3 = n3.toElement();
+             for ( QDomElement e3 = e2.firstChildElement(); !e3.isNull(); e3 = e3.nextSiblingElement() ) {
                if (e3.tagName() == "value")
                {
                   paramValues.append( e3.text() );
@@ -525,18 +519,14 @@ CfgEntry *parseEntry( const QString &group, const QDomElement &element )
       }
     }
     else if ( tag == "choices" ) {
-      QDomNode n2;
-      for( n2 = e.firstChild(); !n2.isNull(); n2 = n2.nextSibling() ) {
-        QDomElement e2 = n2.toElement();
+      for( QDomElement e2 = e.firstChildElement(); !e2.isNull(); e2 = e2.nextSiblingElement() ) {
         if ( e2.tagName() == "choice" ) {
-          QDomNode n3;
           CfgEntry::Choice choice;
           choice.name = e2.attribute( "name" );
           if ( choice.name.isEmpty() ) {
             std::cerr << "Tag <choice> requires attribute 'name'." << std::endl;
           }
-          for( n3 = e2.firstChild(); !n3.isNull(); n3 = n3.nextSibling() ) {
-            QDomElement e3 = n3.toElement();
+          for( QDomElement e3 = e2.firstChildElement(); !e3.isNull(); e3 = e3.nextSiblingElement() ) {
             if ( e3.tagName() == "label" ) choice.label = e3.text();
             if ( e3.tagName() == "whatsthis" ) choice.whatsThis = e3.text();
           }
@@ -565,9 +555,9 @@ CfgEntry *parseEntry( const QString &group, const QDomElement &element )
 
   if ( nameIsEmpty ) {
     name = key;
-    name.replace( " ", QString::null );
+    name.replace( " ", QString() );
   } else if ( name.contains( ' ' ) ) {
-      std::cout<<"Entry '"<<qPrintable(name)<<"' contains spaces! <name> elements can't contain speces!"<<std::endl;
+      std::cout<<"Entry '"<<qPrintable(name)<<"' contains spaces! <name> elements can't contain spaces!"<<std::endl;
     name.remove( ' ' );
   }
 
@@ -598,16 +588,14 @@ CfgEntry *parseEntry( const QString &group, const QDomElement &element )
   {
     // Adjust name
     paramName = name;
-    name.replace("$("+param+')', QString::null);
+    name.replace("$("+param+')', QString());
     // Lookup defaults for indexed entries
     for(int i = 0; i <= paramMax; i++)
     {
-      paramDefaultValues.append(QString::null);
+      paramDefaultValues.append(QString());
     }
 
-    QDomNode n;
-    for ( n = element.firstChild(); !n.isNull(); n = n.nextSibling() ) {
-      QDomElement e = n.toElement();
+    for ( QDomElement e = element.firstChildElement(); !e.isNull(); e = e.nextSiblingElement() ) {
       QString tag = e.tagName();
       if ( tag == "default" )
       {
@@ -751,7 +739,7 @@ QString defaultValue( const QString &type )
 {
     if ( type == "String" )           return "\"\""; // Use empty string, not null string!
     else if ( type == "StringList" )  return "QStringList()";
-    else if ( type == "Font" )        return "KGlobalSettings::generalFont()";
+    else if ( type == "Font" )        return "QFont()";
     else if ( type == "Rect" )        return "QRect()";
     else if ( type == "Size" )        return "QSize()";
     else if ( type == "Color" )       return "QColor(128, 128, 128)";
@@ -787,13 +775,13 @@ QString itemType( const QString &type )
 static QString itemDeclaration(const CfgEntry *e)
 {
   if (itemAccessors)
-     return QString::null;
+     return QString();
 
   QString fCap = e->name();
   fCap[0] = fCap[0].toUpper();
   return "  KConfigSkeleton::Item"+itemType( e->type() ) +
          "  *item" + fCap +
-         ( (!e->param().isEmpty())?(QString("[%1]").arg(e->paramMax()+1)) : QString::null) +
+         ( (!e->param().isEmpty())?(QString("[%1]").arg(e->paramMax()+1)) : QString()) +
          ";\n";
 }
 
@@ -837,7 +825,7 @@ static QString itemPath(const CfgEntry *e)
 }
 
 QString newItem( const QString &type, const QString &name, const QString &key,
-                 const QString &defaultValue, const QString &param = QString::null)
+                 const QString &defaultValue, const QString &param = QString())
 {
   QString t = "new KConfigSkeleton::Item" + itemType( type ) +
               "( currentGroup(), " + key + ", " + varPath( name ) + param;
@@ -896,7 +884,7 @@ QString paramString(const QString &group, const QList<Param> &parameters)
 }
 
 /* int i is the value of the parameter */
-QString userTextsFunctions( CfgEntry *e, QString itemVarStr=QString::null, QString i=QString::null )
+QString userTextsFunctions( CfgEntry *e, QString itemVarStr=QString(), QString i=QString() )
 {
   QString txt;
   if (itemVarStr.isNull()) itemVarStr=itemPath(e);
@@ -999,7 +987,7 @@ QString memberMutatorBody( CfgEntry *e )
   out << " = v;" << endl;
 
   if ( !e->signalList().empty() ) {
-    foreach(Signal signal, e->signalList()) {
+    foreach(const Signal &signal, e->signalList()) {
       out << "  " << This << varPath("settingsChanged") << " |= " << signalEnumName(signal.name) << ";" << endl;
     }
     out << "}" << endl;
@@ -1119,10 +1107,7 @@ int main( int argc, char **argv )
 
   QList<CfgEntry*> entries;
 
-  QDomNode n;
-  for ( n = cfgElement.firstChild(); !n.isNull(); n = n.nextSibling() ) {
-    QDomElement e = n.toElement();
-
+  for ( QDomElement e = cfgElement.firstChildElement(); !e.isNull(); e = e.nextSiblingElement() ) {
     QString tag = e.tagName();
 
     if ( tag == "include" ) {
@@ -1133,9 +1118,7 @@ int main( int argc, char **argv )
     } else if ( tag == "kcfgfile" ) {
       cfgFileName = e.attribute( "name" );
       cfgFileNameArg = e.attribute( "arg" ).toLower() == "true";
-      QDomNode n2;
-      for( n2 = e.firstChild(); !n2.isNull(); n2 = n2.nextSibling() ) {
-        QDomElement e2 = n2.toElement();
+      for( QDomElement e2 = e.firstChildElement(); !e2.isNull(); e2 = e2.nextSiblingElement() ) {
         if ( e2.tagName() == "parameter" ) {
           Param p;
           p.name = e2.attribute( "name" );
@@ -1152,9 +1135,7 @@ int main( int argc, char **argv )
         std::cerr << "Group without name" << std::endl;
         return 1;
       }
-      QDomNode n2;
-      for( n2 = e.firstChild(); !n2.isNull(); n2 = n2.nextSibling() ) {
-        QDomElement e2 = n2.toElement();
+      for( QDomElement e2 = e.firstChildElement(); !e2.isNull(); e2 = e2.nextSiblingElement() ) {
         if ( e2.tagName() != "entry" ) continue;
         CfgEntry *entry = parseEntry( group, e2 );
         if ( entry ) entries.append( entry );
@@ -1173,9 +1154,7 @@ int main( int argc, char **argv )
      Signal theSignal;
      theSignal.name = signalName;
 
-     QDomNode n2;
-     for( n2 = e.firstChild(); !n2.isNull(); n2 = n2.nextSibling() ) {
-       QDomElement e2 = n2.toElement();
+     for( QDomElement e2 = e.firstChildElement(); !e2.isNull(); e2 = e2.nextSiblingElement() ) {
        if ( e2.tagName() == "argument") {
          SignalArguments argument;
          argument.type = e2.attribute("type");
@@ -1303,6 +1282,9 @@ int main( int argc, char **argv )
     QStringList values = (*itEntry)->paramValues();
     if ( !values.isEmpty() ) {
       if ( globalEnums ) {
+        // ### FIXME!!
+        // make the following string table an index-based string search!
+        // ###
         h << "    enum { " << values.join( ", " ) << " };" << endl;
         h << "    static const char* const " << enumName( (*itEntry)->param() ) << "ToString[];" << endl;
         cppPreamble += "const char* const " + className + "::" + enumName( (*itEntry)->param() ) +
@@ -1337,7 +1319,7 @@ int main( int argc, char **argv )
     h << "    " << className << "(";
     if (cfgFileNameArg)
        h << " KSharedConfig::Ptr config"
-         << (parameters.isEmpty() ? " = KSharedConfig::Ptr ( KGlobal::sharedConfig() )" : ", ");
+         << (parameters.isEmpty() ? " = KGlobal::config()" : ", ");
     for (QList<Param>::ConstIterator it = parameters.begin();
          it != parameters.end(); ++it)
     {
@@ -1455,7 +1437,7 @@ int main( int argc, char **argv )
   if( hasSignals ) {
     h << endl;
     h << "  Q_SIGNALS:";
-    foreach(Signal signal, signalList) {
+    foreach(const Signal &signal, signalList) {
       h << endl;
       if ( !signal.label.isEmpty() ) {
         h << "    /**" << endl;
@@ -1466,7 +1448,7 @@ int main( int argc, char **argv )
       QList<SignalArguments>::ConstIterator it, itEnd = signal.arguments.constEnd();
       for ( it = signal.arguments.constBegin(); it != itEnd; ) {
         SignalArguments argument = *it;
-        h << param(argument.type) << argument.variableName;
+        h << param(argument.type) << " " << argument.variableName;
         if ( ++it != itEnd ) {
          h << ", ";
         }
@@ -1850,7 +1832,7 @@ int main( int argc, char **argv )
   if ( hasSignals ) {
     cpp << "void " << className << "::" << "usrWriteConfig()" << endl;
     cpp << "{" << endl;
-    foreach(Signal signal, signalList) {
+    foreach(const Signal &signal, signalList) {
       cpp << "  if ( " << varPath("settingsChanged") << " & " << signalEnumName(signal.name) << " ) " << endl;
       cpp << "    emit " << signal.name << "(";
       QList<SignalArguments>::ConstIterator it, itEnd = signal.arguments.constEnd();
