@@ -542,31 +542,38 @@ void KResolver::emitFinished()
 QString KResolver::errorString(int errorcode, int syserror)
 {
   // no i18n now...
-  static const char * const messages[] =
+  static const char messages[] =
   {
-    I18N_NOOP("no error"),	// NoError
-    I18N_NOOP("requested family not supported for this host name"), // AddrFamily
-    I18N_NOOP("temporary failure in name resolution"),	// TryAgain
-    I18N_NOOP("non-recoverable failure in name resolution"), // NonRecoverable
-    I18N_NOOP("invalid flags"),			// BadFlags
-    I18N_NOOP("memory allocation failure"),	// Memory
-    I18N_NOOP("name or service not known"),	// NoName
-    I18N_NOOP("requested family not supported"),	// UnsupportedFamily
-    I18N_NOOP("requested service not supported for this socket type"), // UnsupportedService
-    I18N_NOOP("requested socket type not supported"),	// UnsupportedSocketType
-    I18N_NOOP("unknown error"),			// UnknownError
+    I18N_NOOP("no error")"\0"	// NoError
+    I18N_NOOP("requested family not supported for this host name")"\0" // AddrFamily
+    I18N_NOOP("temporary failure in name resolution")"\0"	// TryAgain
+    I18N_NOOP("non-recoverable failure in name resolution")"\0" // NonRecoverable
+    I18N_NOOP("invalid flags")"\0"		// BadFlags
+    I18N_NOOP("memory allocation failure")"\0"	// Memory
+    I18N_NOOP("name or service not known")"\0"	// NoName
+    I18N_NOOP("requested family not supported")"\0"	// UnsupportedFamily
+    I18N_NOOP("requested service not supported for this socket type")"\0" // UnsupportedService
+    I18N_NOOP("requested socket type not supported")"\0"	// UnsupportedSocketType
+    I18N_NOOP("unknown error")"\0"			// UnknownError
     I18N_NOOP2("1: the i18n'ed system error code, from errno",
-	      "system error: %1")		// SystemError
+	      "system error: %1")"\0"		// SystemError
+    "\0"
+  };
+  // index table generated by generate_string_table.pl
+  static const int messages_indices[] = {
+      0,    9,   59,   96,  139,  153,  179,  205,
+      236,  289,  325,    0
   };
 
   // handle the special value
   if (errorcode == Canceled)
     return i18n("request was canceled");
 
+  Q_ASSERT(int(SystemError) <= -(int)(sizeof(messages_indices)/sizeof(messages_indices[0])));
   if (errorcode > 0 || errorcode < SystemError)
     return QString();
 
-  QString msg = i18n(messages[-errorcode]);
+  QString msg = i18n(messages + messages_indices[-errorcode]);
   if (errorcode == SystemError)
     msg.arg(QString::fromLocal8Bit(strerror(syserror)));
 
