@@ -27,9 +27,9 @@
 
 //////////////////
 // Needed includes
-#include <QList>
-#include <QObject>
-#include <QSharedDataPointer>
+#include <QtCore/QList>
+#include <QtCore/QObject>
+#include <QtCore/QSharedDataPointer>
 #include "ksocketaddress.h"
 
 
@@ -459,7 +459,7 @@ public:
    * @param nodename	The host name we want resolved.
    * @param servicename	The service name associated, like "http".
    */
-  KResolver(const QString& nodename, const QString& servicename = QString(),
+  explicit KResolver(const QString& nodename, const QString& servicename = QString(),
 	    QObject * = 0L);
 
   /**
