@@ -56,8 +56,8 @@
 #ifndef KSOCKETBASE_H
 #define KSOCKETBASE_H
 
-#include <QIODevice>
-#include <QString>
+#include <QtCore/QIODevice>
+#include <QtCore/QString>
 
 #include "ksocketaddress.h"
 #include <kdelibs_export.h>
@@ -459,6 +459,7 @@ private:
  */
 class KDECORE_EXPORT KActiveSocketBase: public QIODevice, virtual public KSocketBase
 {
+  Q_OBJECT
 public:
   /**
    * Constructor.
