@@ -19,7 +19,8 @@
 #ifndef ksavefiletest_h
 #define ksavefiletest_h
 
-#include <qobject.h>
+#include <QtCore/QObject>
+#include <QtCore/QStringList>
 
 class KSaveFileTest : public QObject
 {
@@ -27,10 +28,15 @@ class KSaveFileTest : public QObject
 
 private Q_SLOTS:
 
+    void test_fstream();
+    void test_ksavefile();
     void test_numberedBackupFile();
     void test_rcsBackupFile();
-    void test_dataStream();
+    void test_simpleBackupFile();
+    void cleanupTestCase();
     
+private:
+    QStringList filesToRemove;
 };
 
 #endif
