@@ -6,7 +6,9 @@
 #include <ksimpleconfig.h>
 #include <kaboutdata.h>
 #include <kcmdlineargs.h>
-#include <kapplication.h>
+#include <kcomponentdata.h>
+#include <QApplication>
+#include <QDir>
 #include <unistd.h>
 #include <stdio.h>
 
@@ -14,19 +16,15 @@
 #include <paths.h>
 #endif
 
-#ifndef _PATH_TMP
-#define _PATH_TMP "/tmp/"
-#endif
-
 int main( int argc, char **argv )
 {
-   KAboutData about("kconfigtest", "kconfigtest", "version");
-   KCmdLineArgs::init(argc, argv, &about);
-
-  KApplication a;
+  KAboutData about("kconfigtest", "kconfigtest", "version");
+  KComponentData cData(&about);
+  //KCmdLineArgs::init(argc, argv, &about);
+  //KApplication a;
 
 
-  KSimpleConfig aConfig( _PATH_TMP"/simpleconfig.cfg" );
+  KSimpleConfig aConfig( QDir::tempPath() + "/simpleconfig.cfg" );
 
   aConfig.setGroup( "Test group" );
   aConfig.writeEntry( "key1", "value1" );
