@@ -32,6 +32,9 @@ private Q_SLOTS:
     void testFindAllResources();
     void testFindDirs();
     void testFindResourceDir();
+    void testFindExe();
+    void testLocate();
+    void testRelativeLocation();
 };
 
 #endif
