@@ -45,10 +45,22 @@ void KUrlTest::testEmptyURL()
   QVERIFY( !emptyURL.isValid() );
   QVERIFY( emptyURL.isEmpty() );
   QVERIFY( emptyURL.prettyUrl().isEmpty() );
+  QCOMPARE( emptyURL.url(), QString() );
+  QVERIFY( emptyURL.url().isEmpty() );
+  QVERIFY( !emptyURL.url().isNull() ); // well, a null string would be correct too...
 
-  emptyURL = "";
-  QVERIFY( !emptyURL.isValid() );
-  QVERIFY( emptyURL.isEmpty() );
+  KUrl emptyStringURL("");
+  QVERIFY( !emptyStringURL.isValid() );
+  QVERIFY( emptyStringURL.isEmpty() );
+  QVERIFY( emptyStringURL.url().isEmpty() );
+  QVERIFY( !emptyStringURL.url().isNull() );
+  QCOMPARE( emptyURL, emptyStringURL );
+
+  // Roundtrip via .url()
+  KUrl emptyCopy( emptyURL.url() );
+  QCOMPARE( emptyURL, emptyCopy );
+  KUrl emptyStringCopy = KUrl( emptyStringURL.url() );
+  QCOMPARE( emptyStringURL, emptyCopy );
 
   KUrl fileURL( "file:/" );
   QVERIFY( !fileURL.isEmpty() );
@@ -86,17 +98,10 @@ void KUrlTest::testSetQuery()
   QCOMPARE( url1.query(), QString("?toto=titi&kde=rocks") );
   url1.setQuery( "?kde=rocks&a=b" );
   QCOMPARE( url1.query(), QString("?kde=rocks&a=b") );
-#if defined( QT_KDE_QT_COPY ) || QT_VERSION >= 0x040200
   url1.setQuery( "?" );
   QCOMPARE( url1.query(), QString() ); // KDE3 difference: I expected QString("?"), but QUrl::setEncodedQuery translates empty to no query. Is that a problem though?
   url1.setQuery( "" );
   QCOMPARE( url1.query(), QString() ); // idem
-#else // bug was: we get no query
-  url1.setQuery( "?" );
-  QCOMPARE( url1.query(), QString() );
-  url1.setQuery( "" );
-  QCOMPARE( url1.query(), QString() );
-#endif
   url1.setQuery( QString() );
   QCOMPARE( url1.query(), QString() );
 }
@@ -138,6 +143,10 @@ void KUrlTest::testQUrl()
 {
   QUrl url1( "file:///home/dfaure/my#%2f" );
   QCOMPARE( url1.toString(), QString( "file:///home/dfaure/my#%2f" ) );
+#ifdef Q_WS_WIN
+  QUrl url2( "file:///c:/home/dfaure/my#%2f" );
+  QCOMPARE( url2.toString(), QString( "file:///c:/home/dfaure/my#%2f" ) );
+#endif
 }
 
 
@@ -158,9 +167,20 @@ void KUrlTest::testIsLocalFile()
   KUrl local_file_4("file:///my/file");
   QVERIFY( local_file_4.isLocalFile() );
 
+#ifdef Q_WS_WIN
+  KUrl local_file_4a("file:///c:/my/file");
+  QVERIFY( local_file_4a.isLocalFile() );
+#endif
+
   KUrl local_file_5;
   local_file_5.setPath("/foo?bar");
   QCOMPARE( local_file_5.url(), QString("file:///foo%3Fbar") );
+
+#ifdef Q_WS_WIN
+  KUrl local_file_5a;
+  local_file_5a.setPath("c:/foo?bar");
+  QCOMPARE( local_file_5a.url(), QString("file:///c:/foo%3Fbar") );
+#endif
 }
 
 void KUrlTest::testSimpleMethods() // to test parsing, mostly
@@ -184,7 +204,7 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
   QVERIFY( !fileURL.hasRef() );
 
   QString u1 = "file:/home/dfaure/my#myref";
-  KUrl url1 = u1;
+  KUrl url1(u1);
   // KDE3 difference: QUrl doesn't resolve file:/ into file:///
   QCOMPARE( url1.url(), QString("file:///home/dfaure/my#myref") );
   QVERIFY( url1.hasRef() );
@@ -283,6 +303,13 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
   KUrl charles2("file:/home/charles/foo%20moo");
   QCOMPARE( charles2.path(), QString("/home/charles/foo moo") );
 
+#ifdef Q_WS_WIN
+#ifdef Q_CC_MSVC
+#pragma message ("port KUser")
+#else
+#warning port KUser
+#endif
+#else
   KUrl tilde;
   KUser currentUser;
   const QString userName = currentUser.loginName();
@@ -290,6 +317,7 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
   tilde.setPath( QString::fromUtf8( "~%1/Matériel" ).arg( userName ) );
   QString homeDir = currentUser.homeDir();
   QCOMPARE( tilde.url(), QString("file://%1/Mat%C3%A9riel").arg(homeDir));
+#endif
 }
 
 void KUrlTest::testEmptyQueryOrRef()
@@ -301,7 +329,6 @@ void KUrlTest::testEmptyQueryOrRef()
   QCOMPARE( url.toEncoded(), QByteArray( "http://www.kde.org?" ) );
   QCOMPARE( url.encodedQuery(), QByteArray() ); // note that QByteArray() == QByteArray("")
 
-#if defined( QT_KDE_QT_COPY ) || QT_VERSION >= 0x040200
   url = QUrl::fromEncoded( "http://www.kde.org" );
   QVERIFY( url.encodedQuery().isEmpty() );
   QVERIFY( !url.hasQuery() );
@@ -313,6 +340,8 @@ void KUrlTest::testEmptyQueryOrRef()
   KUrl noQuery( "http://www.kde.org");
   QCOMPARE( noQuery.query(), QString() ); // query at all
   QVERIFY( !noQuery.hasQuery() );
+  QVERIFY( !noQuery.hasRef());
+  QVERIFY( noQuery.ref().isNull() );
 
   // Empty queries should be preserved!
   QUrl qurl = QUrl::fromEncoded("http://www.kde.org/cgi/test.cgi?", QUrl::TolerantMode);
@@ -340,9 +369,6 @@ void KUrlTest::testEmptyQueryOrRef()
   tobi1 = "http://host.net/path/#no-query";
   QCOMPARE(tobi1.query(), QString("")); // no query
   QVERIFY( !tobi1.hasQuery() );
-#else
-  QSKIP( "Qt-4.2 or qt-copy needed for hasQuery, to distinguish empty query and no query", SkipSingle );
-#endif
 }
 
 void KUrlTest::testParsingTolerance()
@@ -493,7 +519,7 @@ void KUrlTest::testSetFileName() // and addPath
   if ( tmpurl.at(tmpurl.length() - 1) == '/')
       // It's a directory, so strip the trailing slash first
       tmpurl.truncate( tmpurl.length() - 1);
-  KUrl newUrl = tmpurl;
+  KUrl newUrl(tmpurl);
   newUrl.setFileName( "mynewdir" );
   QCOMPARE( newUrl.url(), QString("file:///home/dfaure/mynewdir") );
 
@@ -656,6 +682,11 @@ void KUrlTest::testRelativeURL()
 
 void KUrlTest::testAdjustPath()
 {
+    KUrl url0("file:///");
+    QCOMPARE( url0.url( KUrl::RemoveTrailingSlash ), QString("file:///") );
+    url0.adjustPath( KUrl::RemoveTrailingSlash );
+    QCOMPARE( url0.url(), QString("file:///") );
+
     KUrl url1("file:///home/kde/");
     url1.adjustPath( KUrl::LeaveTrailingSlash );
     QCOMPARE(  url1.path(), QString("/home/kde/" ) );
@@ -707,10 +738,6 @@ void KUrlTest::testAdjustPath()
 
 void KUrlTest::testIPV6()
 {
-#if QT_VERSION < 0x040200
-  QSKIP( "Qt-4.2 needed for ipv6 hosts using []", SkipSingle );
-#endif
-
   kDebug() << k_funcinfo << endl;
   // IPV6
   KUrl waba1( "http://[::FFFF:129.144.52.38]:81/index.html" );
@@ -779,7 +806,6 @@ void KUrlTest::testBaseURL() // those are tests for the KUrl(base,relative) cons
   qurl = "http://www.foo.bar:80";
   QCOMPARE( qurl.toEncoded(), QByteArray("http://www.foo.bar:80") );
 
-#if defined( QT_KDE_QT_COPY ) || QT_VERSION >= 0x040200
   qurl.setHost( QString() );
   qurl.setPath( QString() );
   QCOMPARE( qurl.toEncoded(), QByteArray("http://:80") );
@@ -858,7 +884,6 @@ void KUrlTest::testBaseURL() // those are tests for the KUrl(base,relative) cons
      QCOMPARE( waba2.url(), QString("http://www.website.com/directory/relative.html#with_reference") );
   }
 
-#if defined( QT_KDE_QT_COPY ) || QT_VERSION >= 0x040200
   {
      KUrl waba2( waba1, "#");
      QCOMPARE( waba2.url(), QString("http://www.website.com/directory/?hello#") );
@@ -937,21 +962,13 @@ void KUrlTest::testBaseURL() // those are tests for the KUrl(base,relative) cons
   KUrl sadEagleCombined( sadEagleBase, "/personal/diary/rpc.php?C=jsrs1&F=getDiaryDay&P0=[2006-3-8]&U=1141858921458" );
   QCOMPARE( sadEagleCombined.url(), sadEagleExpectedResult.url() );
 
-#else
-  QSKIP( "QUrl::setFragment(\"\") doesn't make hasFragment return true", SkipSingle ); // TT task 105559 (Thiago, Andreas)
-#endif
-
-#else
-  QSKIP( "QUrl::setPort(-1) doesn't remove the port; breaks base url tests", SkipSingle );
-#endif
-
 }
 
 void KUrlTest::testSubURL()
 {
   kDebug() << k_funcinfo << endl;
   QString u1 = "file:/home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/#myref";
-  KUrl url1 = u1;
+  KUrl url1(u1);
   QCOMPARE( url1.url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/%23tar:/%23myref") ); // KDE3 would say # 3 times
   QVERIFY( url1.hasRef() );
   QVERIFY( !url1.isLocalFile() );  // Not strictly local!
@@ -1262,9 +1279,7 @@ void KUrlTest::testBrokenStuff()
 
   broken = "ptal://mlc:usb:PC_970";
   QVERIFY( !broken.isValid() );
-#if QT_VERSION < 0x040201
   QSKIP( "QUrl doesn't provide the initial string if it's an invalid url; asked for 4.2...", SkipSingle );
-#endif
   QCOMPARE( broken.url(), QString("ptal://mlc:usb:PC_970") ); // FAILS - but we need it...
 
   QUrl brokenUrl( "ptal://mlc:usb:PC_970" );
@@ -1405,11 +1420,19 @@ void KUrlTest::testPathOrURL()
   QVERIFY( uloc.isValid() ); // KDE3: was invalid; same as above
   uloc = KUrl( "" );
   QVERIFY( !uloc.isValid() );
+#ifdef Q_WS_WIN
+#ifdef Q_CC_MSVC
+#pragma message ("port KUser")
+#else
+#warning port KUser
+#endif
+#else
   KUser currentUser;
   const QString userName = currentUser.loginName();
   QVERIFY( !userName.isEmpty() );
   uloc = KUrl(QString::fromUtf8("~%1/konqtests/Matériel").arg(userName));
   QCOMPARE( uloc.path(), QString::fromUtf8("%1/konqtests/Matériel").arg(currentUser.homeDir()) );
+#endif
 
   // pathOrUrl tests
   uloc = KUrl( "/home/dfaure/konqtests/Mat%C3%A9riel" );
@@ -1444,6 +1467,13 @@ void KUrlTest::testAssignment()
   QVERIFY( uloc.isValid() );
   uloc = KUrl( "" );
   QVERIFY( !uloc.isValid() );
+#ifdef Q_WS_WIN
+#ifdef Q_CC_MSVC
+#pragma message ("port KUser")
+#else
+#warning port KUser
+#endif
+#else
   KUser currentUser;
   const QString userName = currentUser.loginName();
   QVERIFY( !userName.isEmpty() );
@@ -1455,9 +1485,10 @@ void KUrlTest::testAssignment()
   // Assigning a KUrl to a QUrl and back
   QUrl qurl = uloc;
   QCOMPARE( qurl.toEncoded(), uloc.toEncoded() );
-  uloc = qurl;
+  uloc = KUrl(qurl);
   QCOMPARE( qurl.toEncoded(), uloc.toEncoded() );
   QCOMPARE( uloc.path(), QString::fromUtf8("%1/konqtests/Matériel").arg(currentUser.homeDir()) );
+#endif
 }
 
 void KUrlTest::testQueryItem()
