@@ -34,7 +34,7 @@
 #include <kbuildservicegroupfactory.h>
 #include <kbuildprotocolinfofactory.h>
 #include <kctimefactory.h>
-#include <ktempfile.h>
+#include <ktemporaryfile.h>
 #include <qdatastream.h>
 #include <qfile.h>
 #include <qtimer.h>
@@ -181,7 +181,8 @@ KSycocaEntry::Ptr KBuildSycoca::createEntry(const QString &file, bool addToFacto
    quint32 timeStamp = g_ctimeInfo->ctime(file);
    if (!timeStamp)
    {
-      timeStamp = KGlobal::dirs()->calcResourceHash( g_resource, file, true);
+      timeStamp = KGlobal::dirs()->calcResourceHash( g_resource, file,
+                                                     KStandardDirs::Recursive);
    }
    KSycocaEntry::Ptr entry;
    if (g_allEntries)
@@ -310,8 +311,8 @@ bool KBuildSycoca::build()
 
      (void) KGlobal::dirs()->findAllResources( g_resource,
                                                QString(),
-                                               true, // Recursive!
-                                               true, // unique
+                                               KStandardDirs::Recursive |
+                                               KStandardDirs::NoDuplicates,
                                                relFiles);
 
 
@@ -426,7 +427,8 @@ void KBuildSycoca::createMenu(QString caption, QString name, VFolderMenu::SubMen
      quint32 timeStamp = g_ctimeInfo->ctime(directoryFile);
      if (!timeStamp)
      {
-        timeStamp = KGlobal::dirs()->calcResourceHash( g_resource, directoryFile, true);
+        timeStamp = KGlobal::dirs()->calcResourceHash( g_resource, directoryFile,
+                                                       KStandardDirs::Recursive );
      }
 
      KServiceGroup::Ptr entry;
@@ -476,16 +478,18 @@ bool KBuildSycoca::recreate()
   QString path(sycocaPath());
 
   // KSaveFile first writes to a temp file.
-  // Upon close() it moves the stuff to the right place.
-  std::auto_ptr<KSaveFile> database( new KSaveFile(path) );
-  if (database->status() == EACCES && QFile::exists(path))
+  // Upon finalize() it moves the stuff to the right place.
+  KSaveFile database(path);
+  bool openedOK = database.open();
+  if (!openedOK && database.error() == QFile::PermissionsError && QFile::exists(path))
   {
     QFile::remove( path );
-    database.reset( new KSaveFile(path) ); // try again
+    openedOK = database.open();
   }
-  if (database->status() != 0)
+  if (!openedOK)
   {
-    fprintf(stderr, "kbuildsycoca: ERROR creating database '%s'! %s\n", path.toLocal8Bit().data(),strerror(database->status()));
+    fprintf(stderr, "kbuildsycoca: ERROR creating database '%s'! %s\n",
+      path.toLocal8Bit().data(), database.errorString().toLocal8Bit().data());
 #ifdef KBUILDSYCOCA_GUI // KBUILDSYCOCA_GUI is used on win32 to build
                         // GUI version of kbuildsycoca, so-called "kbuildsycocaw".
     if (!silent)
@@ -494,7 +498,7 @@ bool KBuildSycoca::recreate()
     return false;
   }
 
-  m_str = database->dataStream();
+  m_str = new QDataStream ( &database );
   m_str->setVersion(QDataStream::Qt_3_1);
 
   kDebug(7021) << "Recreating ksycoca file (" << path << ", version " << KSycoca::version() << ")" << endl;
@@ -511,11 +515,12 @@ bool KBuildSycoca::recreate()
   {
     save(); // Save database
     if (m_str->status() != QDataStream::Ok) // ######## TODO: does this detect write errors, e.g. disk full?
-      database->abort(); // Error
+      database.abort(); // Error
+    delete m_str;
     m_str = 0L;
-    if (!database->close())
+    if (!database.finalize())
     {
-      fprintf(stderr, "kbuildsycoca: ERROR writing database '%s'!\n", database->name().toLocal8Bit().data());
+      fprintf(stderr, "kbuildsycoca: ERROR writing database '%s'!\n", database.fileName().toLocal8Bit().data());
       fprintf(stderr, "kbuildsycoca: Disk full?\n");
 #ifdef KBUILDSYCOCA_GUI
       if (!silent)
@@ -526,8 +531,9 @@ bool KBuildSycoca::recreate()
   }
   else
   {
+    delete m_str;
     m_str = 0L;
-    database->abort();
+    database.abort();
     if (bMenuTest)
        return true;
     kDebug(7021) << "Database is up to date" << endl;
@@ -579,7 +585,8 @@ void KBuildSycoca::save()
    (*m_str) << KGlobal::dirs()->kfsstnd_prefixes();
    (*m_str) << newTimestamp;
    (*m_str) << KGlobal::locale()->language();
-   (*m_str) << KGlobal::dirs()->calcResourceHash("services", "update_ksycoca", true);
+   (*m_str) << KGlobal::dirs()->calcResourceHash("services", "update_ksycoca",
+                                                 KStandardDirs::Recursive );
    (*m_str) << (*g_allResourceDirs);
 
    // Calculate per-servicetype/mimetype data
@@ -780,14 +787,14 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
    {
      // kapp registered already, but with the PID in the name.
      // We need to re-register without it, to detect already-running kbuildsycoca instances.
-     if (QDBus::sessionBus().registerService(appFullName))
+     if (QDBusConnection::sessionBus().registerService(appFullName))
      {
        break; // Go
      }
      fprintf(stderr, "Waiting for already running %s to finish.\n", appName);
 
      QEventLoop eventLoop;
-     QObject::connect(QDBus::sessionBus().interface(), SIGNAL(serviceAcquired(QString)),
+     QObject::connect(QDBusConnection::sessionBus().interface(), SIGNAL(serviceAcquired(QString)),
                       &eventLoop, SLOT(quit()));
      eventLoop.exec( QEventLoop::ExcludeUserInputEvents );
    }
@@ -795,13 +802,19 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
 
    bool checkfiles = bGlobalDatabase || args->isSet("checkfiles");
 
+#ifdef Q_WS_WIN
+   // incremental does not work under win32 because of shared access violation 
+   bool incremental = 0;
+#else
    bool incremental = !bGlobalDatabase && args->isSet("incremental") && checkfiles;
+#endif
    if (incremental || !checkfiles)
    {
      KSycoca::self()->disableAutoRebuild(); // Prevent deadlock
      QString current_language = KGlobal::locale()->language();
      QString ksycoca_language = KSycoca::self()->language();
-     quint32 current_update_sig = KGlobal::dirs()->calcResourceHash("services", "update_ksycoca", true);
+     quint32 current_update_sig = KGlobal::dirs()->calcResourceHash("services", "update_ksycoca",
+                                                                    KStandardDirs::Recursive );
      quint32 ksycoca_update_sig = KSycoca::self()->updateSignature();
 
      if ((current_update_sig != ksycoca_update_sig) ||
@@ -865,7 +878,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
       g_ctimeDict = 0;
       if (incremental)
       {
-         qWarning("Reusing existing ksycoca");
+         kDebug() << "Reusing existing ksycoca" << endl;
          KSycoca *oldSycoca = KSycoca::self();
          KSycocaFactoryList *factories = new KSycocaFactoryList;
          g_allEntries = new KSycocaEntryListList;
@@ -917,9 +930,9 @@ extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
    if (args->isSet("signal"))
    {
      // Notify ALL applications that have a ksycoca object, using a signal
-     QDBusMessage signal = QDBusMessage::signal("/", "org.kde.KSycoca", "notifyDatabaseChanged", QDBus::sessionBus());
+     QDBusMessage signal = QDBusMessage::createSignal("/", "org.kde.KSycoca", "notifyDatabaseChanged" );
      signal << *g_changeList;
-     signal.send();
+     QDBusConnection::sessionBus().send(signal);
    }
 
 #ifdef KBUILDSYCOCA_GUI
