@@ -42,7 +42,6 @@
 #include <klocale.h>
 #include <kglobal.h>
 #include <kconfig.h>
-#include <kprocess.h>
 #include <kdebug.h>
 #include <kdirwatch.h>
 #include <kstandarddirs.h>
@@ -68,6 +67,8 @@ extern QDBUS_EXPORT void qDBusAddSpyHook(void (*)(const QDBusMessage&));
 
 static void runBuildSycoca(QObject *callBackObj=0, const char *callBackSlot=0)
 {
+   const QString exe = KStandardDirs::findExe("kbuildsycoca");
+   Q_ASSERT(!exe.isEmpty());
    QStringList args;
    args.append("--incremental");
    if(checkStamps)
@@ -78,14 +79,14 @@ static void runBuildSycoca(QObject *callBackObj=0, const char *callBackSlot=0)
       checkStamps = false; // useful only during kded startup
    if (callBackObj)
    {
-      QVariantList args;
-      args << QString("kbuildsycoca") << args;
-      KToolInvocation::klauncher()->callWithArgumentList("kdeinit_exec_wait", args, callBackObj,
+      QVariantList argList;
+      argList << exe << args << QStringList() << QString();
+      KToolInvocation::klauncher()->callWithCallback("kdeinit_exec_wait", argList, callBackObj,
                                                          callBackSlot);
    }
    else
    {
-      KToolInvocation::kdeinitExecWait( "kbuildsycoca", args );
+      KToolInvocation::kdeinitExecWait( exe, args );
    }
 }
 
@@ -111,7 +112,7 @@ Kded::Kded(bool checkUpdates)
   new KBuildsycocaAdaptor(this);
   new KdedAdaptor(this);
 
-  QDBusConnection session = QDBus::sessionBus();
+  QDBusConnection session = QDBusConnection::sessionBus();
   session.registerObject("/kbuildsycoca", this);
   session.registerObject("/kded", this);
 
@@ -175,9 +176,11 @@ void Kded::messageFilter(const QDBusMessage &message)
 void Kded::initModules()
 {
      m_dontLoad.clear();
-     KConfig *config = KGlobal::config();
+     KSharedConfig::Ptr config = KGlobal::config();
      bool kde_running = !( getenv( "KDE_FULL_SESSION" ) == NULL || getenv( "KDE_FULL_SESSION" )[ 0 ] == '\0' );
-
+    // not the same user like the one running the session (most likely we're run via sudo or something)
+    if( getenv( "KDE_SESSION_UID" ) != NULL && uid_t( atoi( getenv( "KDE_SESSION_UID" ))) != getuid())
+        kde_running = false;
      // Preload kded modules.
      KService::List kdedModules = KServiceTypeTrader::self()->query("KDEDModule");
      for(KService::List::ConstIterator it = kdedModules.begin(); it != kdedModules.end(); ++it)
@@ -221,7 +224,7 @@ void Kded::initModules()
 void Kded::loadSecondPhase()
 {
      kDebug(7020) << "Loading second phase autoload" << endl;
-     KConfig *config = KGlobal::config();
+     KSharedConfig::Ptr config = KGlobal::config();
      KService::List kdedModules = KServiceTypeTrader::self()->query("KDEDModule");
      for(KService::List::ConstIterator it = kdedModules.begin(); it != kdedModules.end(); ++it)
      {
@@ -483,7 +486,7 @@ void Kded::recreateDone()
    for(; m_recreateCount; m_recreateCount--)
    {
       QDBusMessage msg = m_recreateRequests.takeFirst();
-      msg.sendReply();
+      QDBusConnection::sessionBus().send(msg.createReply());
    }
    m_recreateBusy = false;
 
@@ -528,7 +531,7 @@ void Kded::recreate(const QDBusMessage &msg)
 void Kded::readDirectory( const QString& _path )
 {
   QString path( _path );
-  if ( path.right(1) != "/" )
+  if ( !path.endsWith( '/' ) )
     path += '/';
 
   if ( m_pDirWatch->contains( path ) ) // Already seen this one?
@@ -809,7 +812,7 @@ public:
 extern "C" KDE_EXPORT int kdemain(int argc, char *argv[])
 {
      KAboutData aboutData( "kded", I18N_NOOP("KDE Daemon"),
-        "$Id: kded.cpp 556890 2006-07-01 20:37:17Z hausmann $",
+        "$Id: kded.cpp 634212 2007-02-16 16:24:28Z lunakl $",
         I18N_NOOP("KDE Daemon - triggers Sycoca database updates when needed"));
 
      KCmdLineArgs::init(argc, argv, &aboutData);
@@ -827,8 +830,8 @@ extern "C" KDE_EXPORT int kdemain(int argc, char *argv[])
      // Parse command line before checking DCOP
      KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
 
-     KInstance *instance = new KInstance(&aboutData);
-     KConfig *config = instance->config(); // Enable translations.
+     KComponentData componentData(&aboutData);
+     KSharedConfig::Ptr config = componentData.config(); // Enable translations.
 
      if (args->isSet("check"))
      {
@@ -880,7 +883,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char *argv[])
      if (bCheckHostname)
         (void) new KHostnameD(HostnamePollInterval); // Watch for hostname changes
 
-     QObject::connect(QDBus::sessionBus().interface(),
+     QObject::connect(QDBusConnection::sessionBus().interface(),
                       SIGNAL(serviceOwnerChanged(QString,QString,QString)),
                       kded, SLOT(slotApplicationRemoved(QString,QString,QString)));
 
@@ -892,13 +895,12 @@ extern "C" KDE_EXPORT int kdemain(int argc, char *argv[])
      // unconditionnally (David)
 
      // FIXME: rename the signal
-     QDBusMessage msg = QDBusMessage::signal("/kbuildsycoca", "org.kde.ksycoca",
-                                             "notifyDatabaseChanged", QDBus::sessionBus());
-     msg.send();
-
-     QDBusInterface("org.kde.ksplash", "/")
-         .call(QDBus::NoBlock, "upAndRunning", QString("kded"));
+     QDBusMessage msg = QDBusMessage::createSignal("/kbuildsycoca", "org.kde.KSycoca", "notifyDatabaseChanged" );
+     msg << QStringList();
+     QDBusConnection::sessionBus().send(msg);
 #ifdef Q_WS_X11
+     QDBusInterface("org.kde.ksplash", "/KSplash")
+         .call(QDBus::NoBlock, "upAndRunning", QString("kded"));
      XEvent e;
      e.xclient.type = ClientMessage;
      e.xclient.message_type = XInternAtom( QX11Info::display(), "_KDE_SPLASH_PROGRESS", False );
@@ -911,7 +913,6 @@ extern "C" KDE_EXPORT int kdemain(int argc, char *argv[])
      int result = k.exec(); // keep running
 
      delete kded;
-     delete instance; // Deletes config as well
 
      return result;
 }
