@@ -33,9 +33,6 @@
 #include <qfile.h>
 #include <qdir.h>
 #include <qregexp.h>
-#ifndef Q_OS_WIN
-#include <sys/dir.h>
-#endif
 
 #include "vfolder_menu.h"
 
@@ -651,12 +648,13 @@ VFolderMenu::mergeMenus(QDomElement &docElem, QString &name)
          if (!QDir::isRelativePath(dir))
          {
             // Absolute
-            fileList = KGlobal::dirs()->findAllResources("xdgconf-menu", dir+"*.menu", false, false);
+            fileList = KGlobal::dirs()->findAllResources("xdgconf-menu", dir+"*.menu");
          }
          else
          {
             // Relative
-            (void) KGlobal::dirs()->findAllResources("xdgconf-menu", dir+"*.menu", false, true, fileList);
+            (void) KGlobal::dirs()->findAllResources("xdgconf-menu", dir+"*.menu",
+                                                     KStandardDirs::NoDuplicates, fileList);
          }
 
          for(QStringList::ConstIterator it=fileList.begin();
@@ -770,8 +768,27 @@ VFolderMenu::locateMenuFile(const QString &fileName)
       return QString();
    }
 
-   QString baseName = QDir::cleanPath(m_docInfo.baseDir + fileName);
-   QString result = KStandardDirs::locate("xdgconf-menu", baseName);
+   QString result;
+
+   QString xdgMenuPrefix = QString::fromLocal8Bit(getenv("XDG_MENU_PREFIX"));
+   if (!xdgMenuPrefix.isEmpty())
+   {
+      QFileInfo fileInfo(fileName);
+
+      QString fileNameOnly = fileInfo.fileName();
+      if (!fileNameOnly.startsWith(xdgMenuPrefix))
+         fileNameOnly = xdgMenuPrefix + fileNameOnly;
+
+      QString baseName = QDir::cleanPath(m_docInfo.baseDir +
+                                         fileInfo.path() + "/" + fileNameOnly);
+      result = KStandardDirs::locate("xdgconf-menu", baseName);
+   }
+
+   if (result.isEmpty())
+   {
+       QString baseName = QDir::cleanPath(m_docInfo.baseDir + fileName);
+       result = KStandardDirs::locate("xdgconf-menu", baseName);
+   }
 
    return result;
 }
@@ -830,7 +847,10 @@ VFolderMenu::loadMenu(const QString &fileName)
 
    if (m_doc.isNull())
    {
-      kWarning(7021) << "Load error (" << m_docInfo.path << ")" << endl;
+      if (m_docInfo.path.isEmpty())
+         kError(7021) << fileName << " not found in " << m_allDirectories << endl;
+      else
+         kWarning(7021) << "Load error (" << m_docInfo.path << ")" << endl;
       return;
    }
 
@@ -1021,8 +1041,8 @@ kDebug(7021) << "processKDELegacyDirs()" << endl;
 
    (void) KGlobal::dirs()->findAllResources( "apps",
                                              QString(),
-                                             true, // Recursive!
-                                             true, // unique
+                                             KStandardDirs::Recursive |
+                                             KStandardDirs::NoDuplicates,
                                              relFiles);
    for(QStringList::ConstIterator it = relFiles.begin();
        it != relFiles.end(); ++it)
