@@ -30,10 +30,10 @@ class QDirLineEdit : public QWidget
 {
 	Q_OBJECT
 public:
-	QDirLineEdit(bool file, QWidget *parent = 0);
+	explicit QDirLineEdit(bool file, QWidget *parent = 0);
 	~QDirLineEdit();
 
-	void setURL(const QString& txt);
+	void setUrl(const QString& txt);
 	QString url();
 	void setFileEdit(bool on = true);
 
