@@ -45,9 +45,9 @@
 #include <klocale.h>
 #include <kconfig.h>
 #include <kstandarddirs.h>
+#include <kbufferedsocket.h>
 #include <klibloader.h>
 #include <kactioncollection.h>
-#include <kbufferedsocket.h>
 #include <kmessagebox.h>
 #include <kaction.h>
 #include <kprocess.h>
@@ -60,9 +60,9 @@ using namespace KNetwork;
 
 #define ppdi18n(s)	i18n(QString::fromLocal8Bit(s).toUtf8())
 
-void extractMaticData(QString& buf, const QString& filename);
-QString printerURI(KMPrinter *p, bool useExistingURI = false);
-QString downloadDriver(KMPrinter *p);
+static void extractMaticData(QString& buf, const QString& filename);
+static QString printerURI(KMPrinter *p, bool useExistingURI);
+static QString downloadDriver(KMPrinter *p);
 
 static int trials = 5;
 
@@ -89,6 +89,7 @@ KMCupsManager::KMCupsManager(QObject *parent, const QStringList & /*args*/)
 
 KMCupsManager::~KMCupsManager()
 {
+       delete m_socket;
 }
 
 QString KMCupsManager::driverDbCreationProgram()
@@ -215,7 +216,7 @@ bool KMCupsManager::setPrinterState(KMPrinter *p, int state)
 	QString		uri;
 
 	req.setOperation(state);
-	uri = printerURI(p);
+	uri = printerURI(p, true);
 	req.addURI(IPP_TAG_OPERATION,"printer-uri",uri);
 	if (req.doRequest("/admin/"))
 		return true;
@@ -806,15 +807,17 @@ QStringList KMCupsManager::detectLocalPrinters()
 
 void KMCupsManager::createPluginActions(KActionCollection *coll)
 {
-	KAction	*act = new KAction(i18n("&Export Driver..."), coll, "plugin_export_driver");
-  act->setIcon( KIcon( "kdeprint_uploadsmb" ) );
+	KAction	*act = new KAction(i18n("&Export Driver..."), this);
+	coll->addAction( "plugin_export_driver", act );
+	act->setIcon( KIcon( "kdeprint_uploadsmb" ) );
 	act->setActionGroup(pluginGroup());
-  connect( act, SIGNAL( triggered( bool ) ), this, SLOT(exportDriver()) );
+	connect( act, SIGNAL( triggered( bool ) ), this, SLOT(exportDriver()) );
 
-	act = new KAction(i18n("&Printer IPP Report"), coll, "plugin_printer_ipp_report");
-  act->setIcon( KIcon( "kdeprint_report" ) );
+	act = new KAction(i18n("&Printer IPP Report"), this);
+	coll->addAction( "plugin_printer_ipp_report", act );
+	act->setIcon( KIcon( "kdeprint_report" ) );
 	act->setActionGroup(pluginGroup());
-  connect( act, SIGNAL( triggered( bool ) ), this, SLOT(printerIppReport()) );
+	connect( act, SIGNAL( triggered( bool ) ), this, SLOT(printerIppReport()) );
 }
 
 void KMCupsManager::validatePluginActions(KActionCollection *coll, KMPrinter *pr)
@@ -888,6 +891,12 @@ QString KMCupsManager::testPage()
 void KMCupsManager::checkUpdatePossibleInternal()
 {
 	kDebug(500) << "Checking for update possible" << endl;
+	m_socket = new KNetwork::KBufferedSocket;
+	m_socket->setTimeout( 1500 );
+	connect( m_socket, SIGNAL( connected(const KNetwork::KResolverEntry&) ),
+                SLOT( slotConnectionSuccess() ) );
+	connect( m_socket, SIGNAL( gotError( int ) ), SLOT( slotConnectionFailed( int ) ) );
+	trials = 5;
 	QTimer::singleShot( 1, this, SLOT( slotAsyncConnect() ) );
 }
 
@@ -920,16 +929,13 @@ void KMCupsManager::slotConnectionSuccess()
 void KMCupsManager::slotAsyncConnect()
 {
 	kDebug(500) << "Starting async connect " << CupsInfos::self()->hostaddr() << endl;
-	http_t	*HTTP = httpConnect(CupsInfos::self()->host().toLatin1(), CupsInfos::self()->port());
-	if(!HTTP)
-		slotConnectionFailed(0);
-	else {
-		slotConnectionSuccess();
-		httpClose(HTTP);
-	}
+        if (CupsInfos::self()->host().startsWith("/"))
+            m_socket->connect( QString(), CupsInfos::self()->host());
+        else
+            m_socket->connect( CupsInfos::self()->host(), QString::number(CupsInfos::self()->port()) );
 }
 
-void KMCupsManager::slotConnectionFailed( int )
+void KMCupsManager::slotConnectionFailed( int errcode )
 {
 	kDebug(500) << "Connection failed trials=" << trials << endl;
 	if ( trials > 0 )
@@ -939,7 +945,23 @@ void KMCupsManager::slotConnectionFailed( int )
 		return;
 	}
 
-	setErrorMsg( i18n( "Connection to CUPS server failed. Check that the CUPS server is correctly installed and running. " ) );
+	QString einfo;
+	switch (errcode) {
+	    case KNetwork::KSocketBase::ConnectionRefused:
+	    case KNetwork::KSocketBase::ConnectionTimedOut:
+		einfo = i18n("connection refused") + QString(" (%1)").arg(errcode);
+		break;
+	    case KNetwork::KSocketBase::LookupFailure:
+		einfo = i18n("host not found") + QString(" (%1)").arg(errcode);
+		break;
+	    case KNetwork::KSocketBase::WouldBlock:
+	    default:
+		einfo = i18n("read failed (%1)", errcode);
+		break;
+	}
+
+	setErrorMsg( i18n( "Connection to CUPS server failed. Check that the CUPS server is correctly installed and running. "
+        "Error: %2: %1", einfo, CupsInfos::self()->hostaddr() ) );
 	setUpdatePossible( false );
 }
 
@@ -955,7 +977,7 @@ void KMCupsManager::hostPingFailedSlot() {
 
 //*****************************************************************************************************
 
-void extractMaticData(QString& buf, const QString& filename)
+static void extractMaticData(QString& buf, const QString& filename)
 {
 	QFile	f(filename);
 	if (f.exists() && f.open(QIODevice::ReadOnly))
@@ -971,7 +993,7 @@ void extractMaticData(QString& buf, const QString& filename)
 	}
 }
 
-QString printerURI(KMPrinter *p, bool use)
+static QString printerURI(KMPrinter *p, bool use)
 {
 	QString	uri;
 	if (use && !p->uri().isEmpty())
@@ -984,7 +1006,7 @@ QString printerURI(KMPrinter *p, bool use)
 	return uri;
 }
 
-QString downloadDriver(KMPrinter *p)
+static QString downloadDriver(KMPrinter *p)
 {
 	QString	driverfile, prname = p->printerName();
 	bool	changed(false);
