@@ -45,6 +45,7 @@
 
 #include <klocale.h>
 #include <kiconloader.h>
+#include <kicon.h>
 #include <kfiledialog.h>
 #include <kurlrequester.h>
 #include <klineedit.h>
@@ -52,11 +53,11 @@
 #include <kglobal.h>
 #include <kconfig.h>
 #include <kguiitem.h>
-#include <kstdguiitem.h>
+#include <kstandardguiitem.h>
 #include <kapplication.h>
 #include <ktoolinvocation.h>
 #include <kauthorized.h>
-#include <kio/renamedlg.h>
+#include <kio/renamedialog.h>
 #include <kmessagebox.h>
 
 #include <time.h>
@@ -292,14 +293,14 @@ KPrintDialog::KPrintDialog(QWidget *parent)
 	d->m_default = new KPushButton(KGuiItem(i18n("Set as &Default"), "kdeprint_defaultsoft"), m_pbox);
 	d->m_default->setWhatsThis(whatsThisSetDefaultPrinter);
 	d->m_filter = new QPushButton(m_pbox);
-	d->m_filter->setIcon(SmallIcon("filter"));
+	d->m_filter->setIcon(KIcon("filter"));
 	d->m_filter->setMinimumSize(QSize(d->m_printers->minimumHeight(),d->m_printers->minimumHeight()));
 	d->m_filter->setCheckable(true);
 	d->m_filter->setDown(KMManager::self()->isFilterEnabled());
 	d->m_filter->setToolTip(i18n("Toggle selective view on printer list"));
 	d->m_filter->setWhatsThis(whatsThisPrinterFilter);
 	d->m_wizard = new QPushButton(m_pbox);
-	d->m_wizard->setIcon(SmallIcon("wizard"));
+	d->m_wizard->setIcon(KIcon("wizard"));
 	d->m_wizard->setMinimumSize(QSize(d->m_printers->minimumHeight(),d->m_printers->minimumHeight()));
 	d->m_wizard->setToolTip(i18n("Add printer..."));
 	d->m_wizard->setWhatsThis(whatsThisAddPrinterWizard);
@@ -307,7 +308,7 @@ KPrintDialog::KPrintDialog(QWidget *parent)
         d->m_ok->setWhatsThis(whatsThisPrintButton);
 	d->m_ok->setDefault(true);
 	d->m_ok->setEnabled( false );
-	QPushButton	*m_cancel = new KPushButton(KStdGuiItem::cancel(), main);
+	QPushButton	*m_cancel = new KPushButton(KStandardGuiItem::cancel(), main);
         m_cancel->setWhatsThis(whatsThisCancelButton);
 	d->m_preview = new QCheckBox(i18n("Previe&w"), m_pbox);
 	d->m_preview->setWhatsThis(whatsThisPreviewCheckBox);
@@ -330,7 +331,7 @@ KPrintDialog::KPrintDialog(QWidget *parent)
 	d->m_extbtn->setWhatsThis(whatsThisOptions);
 	d->m_persistent = new QCheckBox(i18n("&Keep this dialog open after printing"), main);
         d->m_persistent->setWhatsThis(whatsThisKeepDialogOpenCheckbox);
-		QPushButton	*m_help = new KPushButton(KStdGuiItem::help(), main);
+		QPushButton	*m_help = new KPushButton(KStandardGuiItem::help(), main);
         m_help->setWhatsThis(whatsThisHelpButton);
 
 	QWidget::setTabOrder( d->m_printers, d->m_filter );
@@ -452,7 +453,7 @@ KPrintDialog::KPrintDialog(QWidget *parent)
 	}
 	else
 	{
-		KConfig	*config = KGlobal::config();
+		KSharedConfig::Ptr config = KGlobal::config();
 		config->setGroup("KPrinter Settings");
 		expandDialog(!config->readEntry("DialogReduced", QVariant(KMFactory::self()->settings()->application != KPrinter::StandAlone)).toBool());
 	}
@@ -460,7 +461,7 @@ KPrintDialog::KPrintDialog(QWidget *parent)
 
 KPrintDialog::~KPrintDialog()
 {
-	KConfig	*config = KGlobal::config();
+	KSharedConfig::Ptr config = KGlobal::config();
 	config->setGroup("KPrinter Settings");
 	config->writeEntry("DialogReduced", d->m_reduced);
 
@@ -752,8 +753,9 @@ bool KPrintDialog::checkOutputFile()
 			{
 				//value = (KMessageBox::warningYesNo(this,i18n("File \"%1\" already exists. Overwrite?").arg(f.absoluteFilePath())) == KMessageBox::Yes);
 				time_t mtimeDest = f.lastModified().toTime_t();
-				KIO::RenameDlg dlg( this, i18n( "Print" ), QString(), d->m_file->url().toString(),
-                                                    KIO::M_OVERWRITE, ( time_t ) -1, f.size(), ( time_t ) -1, f.created().toTime_t() , mtimeDest+1, mtimeDest );
+				KIO::RenameDialog dlg( this, i18n( "Print" ), KUrl(), d->m_file->url(),
+							  		   KIO::M_OVERWRITE, (time_t)-1, f.size(), (time_t)-1,
+									   f.created().toTime_t(), mtimeDest+1, mtimeDest );
 				int result = dlg.exec();
 				switch ( result )
 				{
@@ -765,7 +767,7 @@ bool KPrintDialog::checkOutputFile()
 						value = false;
 						break;
 					case KIO::R_RENAME:
-						url = dlg.newDestURL();
+						url = dlg.newDestUrl();
 						d->m_file->setUrl( url );
 						value = true;
 						anotherCheck = true;
@@ -890,7 +892,7 @@ void KPrintDialog::expandDialog(bool on)
 			if (d->b_systemEnabled)
 				d->m_plugin->show();
 		}
-		d->m_extbtn->setIcon(SmallIconSet("up"));
+		d->m_extbtn->setIcon(KIcon("up"));
 		d->m_extbtn->setText(i18n("&Options <<"));
 		d->m_reduced = false;
 	}
@@ -902,7 +904,7 @@ void KPrintDialog::expandDialog(bool on)
 			if (d->b_systemEnabled)
 				d->m_plugin->hide();
 		}
-		d->m_extbtn->setIcon(SmallIconSet("down"));
+		d->m_extbtn->setIcon(KIcon("down"));
 		d->m_extbtn->setText(i18n("&Options >>"));
 		d->m_reduced = true;
 	}
