@@ -42,7 +42,6 @@
 #include <knotification.h>
 #include <kdebug.h>
 #include <klocale.h>
-#include <kprocess.h>
 #include <klibloader.h>
 #include <kmessagebox.h>
 
@@ -73,7 +72,7 @@ void KPrinterWrapper::setMargins( uint top, uint left, uint bottom, uint right )
 	// QPainter::pageRect() is actually taken from e.g. CUPs or whether a full
 	// page is being used.
 	
-	QRect pageRect = paperRect().adjusted( left, top, -right, -bottom );
+	QRect pageRect = paperRect().adjusted( left, top, 0U-right, 0U-bottom );
 	printEngine()->setProperty( QPrintEngine::PPK_PageRect, pageRect );
 }
 
@@ -295,7 +294,8 @@ void KPrinter::loadSettings()
 	setSearchName(option("kde-searchname"));
 	d->m_options.remove("kde-searchname");
 
-	KConfig	*conf = KGlobal::config(), *pconf = KMFactory::self()->printConfig();
+    KSharedConfig::Ptr conf = KGlobal::config();
+    KConfig *pconf = KMFactory::self()->printConfig();
 	conf->setGroup("KPrinter Settings");
 	pconf->setGroup("General");
 
@@ -320,7 +320,7 @@ void KPrinter::saveSettings()
 	}
 
 	// save latest used printer to config file
-	KConfig	*conf = KGlobal::config();
+	KSharedConfig::Ptr conf = KGlobal::config();
 	conf->setGroup("KPrinter Settings");
 	conf->writeEntry("Printer",searchName());
 	// latest used print command
@@ -768,7 +768,7 @@ void KPrinter::initOptions(const QMap<QString,QString>& opts)
 	for (QMap<QString,QString>::ConstIterator it=opts.begin(); it!=opts.end(); ++it)
 	{
 		setOption(it.key(), it.value());
-		if (it.key().left(4) != "kde-")
+		if (!it.key().startsWith(QLatin1String("kde-")))
 			d->m_impl->broadcastOption(it.key(),it.value());
 	}
 }
