@@ -37,9 +37,10 @@
 #include <krandom.h>
 #include <kstandarddirs.h>
 #include <kdebug.h>
-#include <kmimemagic.h>
+#include <kmimetype.h>
 #include <kmessagebox.h>
 #include <kprocess.h>
+#include <qprocess.h>
 #include <kconfig.h>
 #include <kapplication.h>
 
@@ -158,9 +159,7 @@ bool KPrinterImpl::printFiles(KPrinter *p, const QStringList& f, bool flag)
 				}
 				else
 				{
-					KProcess proc;
-					proc << (flag?"mv":"cp") << f[0] << p->outputFileName();
-					if (!proc.start(KProcess::Block) || !proc.normalExit() || proc.exitStatus() != 0)
+					if (QProcess::execute(flag?"mv":"cp", QStringList() << f[0] << p->outputFileName()) != 0)
 					{
 						p->setErrorMessage(i18n("Cannot save print file to %1. Check that you have write access to it.", p->outputFileName()));
 						return false;
@@ -212,7 +211,7 @@ void KPrinterImpl::statusMessage(const QString& msg, KPrinter *printer)
 		message.prepend(i18n("Printing document: %1", printer->docName())+"\n");
 
         QDBusInterface kdeprintd( "org.kde.kded", "/modules/kdeprintd", "org.kde.KDEPrintd" );
-        (void)kdeprintd.call( "statusMessage", msg, int(getpid()), kapp->caption() );
+        (void)kdeprintd.call( "statusMessage", msg, int(getpid()), KGlobal::caption() );
 }
 
 bool KPrinterImpl::startPrinting(const QString& cmd, KPrinter *printer, const QStringList& files, bool flag)
@@ -226,8 +225,8 @@ bool KPrinterImpl::startPrinting(const QString& cmd, KPrinter *printer, const QS
 	for (QStringList::ConstIterator it=files.begin(); it!=files.end(); ++it)
 		if (QFile::exists(*it))
 		{
-			// quote and encode filenames
-			filestr.append(quote(QFile::encodeName(*it))).append(" ");
+			// quote filenames
+			filestr.append(quote(*it)).append(" ");
 			printfiles.append(*it);
 		}
 		else
@@ -347,7 +346,7 @@ int KPrinterImpl::doFilterFiles(KPrinter *printer, QStringList& files, const QSt
 	QString	ps = pageSizeToPageName( printer->option( "kde-printsize" ).isEmpty() ? printer->pageSize() : ( KPrinter::PageSize )printer->option( "kde-printsize" ).toInt() );
 	for (QStringList::Iterator it=files.begin(); it!=files.end(); ++it)
 	{
-		QString	mime = KMimeMagic::self()->findFileType(*it)->mimeType();
+		QString	mime = KMimeType::findByPath(*it)->name();
 		if (!inputMimeTypes.contains(mime))
 		{
 			if (KMessageBox::warningContinueCancel(0,
@@ -355,7 +354,7 @@ int KPrinterImpl::doFilterFiles(KPrinter *printer, QStringList& files, const QSt
 				     "(this may happen with non-CUPS spoolers when performing page selection "
 				     "on a non-PostScript file). Do you want KDE to convert the file to a supported "
 				     "format?</p>", mime),
-				QString(), i18n("Convert")) == KMessageBox::Continue)
+				QString(), KGuiItem(i18n("Convert"))) == KMessageBox::Continue)
 			{
 				QStringList	ff;
 				int	done(0);
@@ -448,7 +447,7 @@ int KPrinterImpl::autoConvertFiles(KPrinter *printer, QStringList& files, bool f
 	int		status(0), result;
 	for (QStringList::Iterator it=files.begin(); it!=files.end(); )
 	{
-		QString	mime = KMimeMagic::self()->findFileType(*it)->mimeType();
+		QString	mime = KMimeType::findByPath(*it)->name();
 		if ( mime == "application/x-zerosize" )
 		{
 			// special case of empty file
@@ -475,8 +474,8 @@ int KPrinterImpl::autoConvertFiles(KPrinter *printer, QStringList& files, bool f
 						    "</ul> "
 						    "Do you want KDE to attempt and convert this file to %2?</qt>", mime, primaryMimeType),
 					       QString(),
-					       i18n("Convert"),
-					       i18n("Keep"),
+					       KGuiItem(i18n("Convert")),
+					       KGuiItem(i18n("Keep")),
 					       QLatin1String("kdeprintAutoConvert"))) == KMessageBox::Yes)
 			{
 				// find the filter chain
@@ -534,6 +533,7 @@ bool KPrinterImpl::setupSpecialCommand(QString& cmd, KPrinter *p, const QStringL
 	s = KMFactory::self()->specialManager()->setupCommand(s, p->options());
 
 	QString	ps = pageSizeToPageName( p->option( "kde-printsize" ).isEmpty() ? p->pageSize() : ( KPrinter::PageSize )p->option( "kde-printsize" ).toInt() );
+	// XXX use KMacroExpander
 	s.replace("%psl", ps.toLower());
 	s.replace("%psu", ps);
 	s.replace("%out", "$out{" + p->outputFileName() + "}"); // Replace as last
@@ -552,7 +552,7 @@ void KPrinterImpl::saveOptions(const QMap<QString,QString>& opts)
 
 void KPrinterImpl::loadAppOptions()
 {
-	KConfig	*conf = KGlobal::config();
+	KSharedConfig::Ptr conf = KGlobal::config();
 	conf->setGroup("KPrinter Settings");
 	QStringList	opts = conf->readEntry("ApplicationOptions", QStringList());
 	for (int i=0; i<opts.count(); i+=2)
@@ -567,7 +567,7 @@ void KPrinterImpl::saveAppOptions()
 		if (it.key().startsWith("app-"))
 			optlist << it.key() << it.value();
 
-	KConfig	*conf = KGlobal::config();
+	KSharedConfig::Ptr conf = KGlobal::config();
 	conf->setGroup("KPrinter Settings");
 	conf->writeEntry("ApplicationOptions", optlist);
 }
