@@ -29,7 +29,6 @@
 #include <qcombobox.h>
 #include <QGroupBox>
 #include <QVBoxLayout>
-#include <q3widgetstack.h>
 #include <qtoolbutton.h>
 #include <kpushbutton.h>
 #include <qcheckbox.h>
@@ -42,7 +41,7 @@
 #include <kiconloader.h>
 #include <kdialog.h>
 #include <kseparator.h>
-#include <klistbox.h>
+#include <klistwidget.h>
 #include <kmimetype.h>
 #include <kmessagebox.h>
 #include <krandom.h>
@@ -89,17 +88,17 @@ KXmlCommandAdvancedDlg::KXmlCommandAdvancedDlg( QWidget *parent )
 	m_view->header()->hide();
 	m_view->setSorting(-1);
 	m_apply = new QToolButton(this);
-	m_apply->setIcon( QApplication::isRightToLeft()? SmallIconSet( "forward" ) : SmallIconSet("back"));
+	m_apply->setIcon( QApplication::isRightToLeft()? KIcon( "forward" ) : KIcon("back"));
 	m_addgrp = new QToolButton(this);
-	m_addgrp->setIcon(SmallIconSet("folder"));
+	m_addgrp->setIcon(KIcon("folder"));
 	m_addopt = new QToolButton(this);
-	m_addopt->setIcon(SmallIconSet("document"));
+	m_addopt->setIcon(KIcon("document"));
 	m_delopt = new QToolButton(this);
-	m_delopt->setIcon(SmallIconSet("editdelete"));
+	m_delopt->setIcon(KIcon("editdelete"));
 	m_up = new QToolButton(this);
-	m_up->setIcon(SmallIconSet("up"));
+	m_up->setIcon(KIcon("up"));
 	m_down = new QToolButton(this);
-	m_down->setIcon(SmallIconSet("down"));
+	m_down->setIcon(KIcon("down"));
 	m_dummy = new QWidget(this);
 	m_desc = new QLineEdit(m_dummy);
 	m_name = new QLineEdit(m_dummy);
@@ -145,9 +144,9 @@ KXmlCommandAdvancedDlg::KXmlCommandAdvancedDlg( QWidget *parent )
 	m_values->setSorting(-1);
 	m_values->setMaximumHeight(110);
 	m_addval = new QToolButton(w2);
-	m_addval->setIcon(SmallIconSet("editcopy"));
+	m_addval->setIcon(KIcon("editcopy"));
 	m_delval = new QToolButton(w2);
-	m_delval->setIcon(SmallIconSet("editdelete"));
+	m_delval->setIcon(KIcon("editdelete"));
 	m_addval->setToolTip(i18n("Add value"));
 	m_delval->setToolTip(i18n("Delete value"));
 
@@ -806,7 +805,7 @@ void KXmlCommandAdvancedDlg::recreateGroup(Q3ListViewItem *item, DrGroup *grp)
 			}
 			else
 			{
-				opt->setName("_kde-"+m_xmlcmd->name()+"-"+opt->name());
+				opt->setName("_kde-"+m_xmlcmd->name()+'-'+opt->name());
 				grp->addOption(opt);
 			}
 			m_opts.remove(child->text(1));
@@ -889,17 +888,17 @@ KXmlCommandDlg::KXmlCommandDlg(QWidget *parent, const char *name)
 	m_requirements->addColumn("");
 	m_requirements->header()->hide();
 	m_addreq = new QToolButton(m_gb2);
-	m_addreq->setIcon(SmallIconSet("filenew"));
+	m_addreq->setIcon(KIcon("filenew"));
 	m_removereq = new QToolButton(m_gb2);
-	m_removereq->setIcon(SmallIconSet("editdelete"));
+	m_removereq->setIcon(KIcon("editdelete"));
 	QPushButton	*m_edit = new KPushButton(KGuiItem(i18n("&Edit Command..."), "edit"), topmain);
 	m_mimetype = new QComboBox(dummy);
-	m_availablemime = new KListBox(m_gb1);
-	m_selectedmime = new KListBox(m_gb1);
+	m_availablemime = new KListWidget(m_gb1);
+	m_selectedmime = new KListWidget(m_gb1);
 	m_addmime = new QToolButton(m_gb1);
-	m_addmime->setIcon(QApplication::isRightToLeft()? SmallIconSet("forward") : SmallIconSet("back"));
+	m_addmime->setIcon(QApplication::isRightToLeft()? KIcon("forward") : KIcon("back"));
 	m_removemime = new QToolButton(m_gb1);
-	m_removemime->setIcon(QApplication::isRightToLeft()? SmallIconSet("back" ) : SmallIconSet("forward"));
+	m_removemime->setIcon(QApplication::isRightToLeft()? KIcon("back" ) : KIcon("forward"));
 	m_gb2->setMinimumWidth(380);
 	m_gb1->setMinimumHeight(180);
 	m_requirements->setMaximumHeight(80);
@@ -988,10 +987,11 @@ KXmlCommandDlg::KXmlCommandDlg(QWidget *parent, const char *name)
 
 	m_mimelist.sort();
 	m_mimetype->addItems(m_mimelist);
-	m_availablemime->insertStringList(m_mimelist);
+	m_availablemime->addItems(m_mimelist);
 
 	setMainWidget(topmain);
 	setDetailsWidget(dummy);
+	connect(this,SIGNAL(okClicked()),this,SLOT(slotOk()));
 }
 
 void KXmlCommandDlg::setCommand(KXmlCommand *xmlCmd)
@@ -1020,11 +1020,14 @@ void KXmlCommandDlg::setCommand(KXmlCommand *xmlCmd)
 	list = xmlCmd->inputMimeTypes();
 	m_selectedmime->clear();
 	m_availablemime->clear();
-	m_availablemime->insertStringList(m_mimelist);
+	m_availablemime->addItems(m_mimelist);
 	for (QStringList::ConstIterator it=list.begin(); it!=list.end(); ++it)
 	{
-		m_selectedmime->insertItem(*it);
-		delete m_availablemime->findItem(*it, Q3ListView::ExactMatch);
+		m_selectedmime->addItem(*it);
+		QList<QListWidgetItem*> items = m_availablemime->findItems(*it,Qt::MatchExactly);
+        foreach ( QListWidgetItem* item , items ) {
+            delete item;
+        }
 	}
 }
 
@@ -1043,8 +1046,8 @@ void KXmlCommandDlg::slotOk()
 		}
 		m_cmd->setRequirements(l);
 		l.clear();
-		for (uint i=0; i<m_selectedmime->count(); i++)
-			l << m_selectedmime->text(i);
+		for (int i=0; i<m_selectedmime->count(); i++)
+			l << m_selectedmime->item(i)->text();
 		m_cmd->setInputMimeTypes(l);
 	}
 	KDialog::accept();
@@ -1063,23 +1066,25 @@ bool KXmlCommandDlg::editCommand(KXmlCommand *xmlCmd, QWidget *parent)
 
 void KXmlCommandDlg::slotAddMime()
 {
-	int	index = m_availablemime->currentItem();
+	int	index = m_availablemime->currentRow();
 	if (index != -1)
 	{
-		m_selectedmime->insertItem(m_availablemime->currentText());
-		m_availablemime->removeItem(index);
-		m_selectedmime->sort();
+		m_selectedmime->addItem(m_availablemime->currentItem()->text());
+		QListWidgetItem* item = m_availablemime->takeItem(index);
+        delete item;
+		m_selectedmime->model()->sort(0);
 	}
 }
 
 void KXmlCommandDlg::slotRemoveMime()
 {
-	int	index = m_selectedmime->currentItem();
+	int	index = m_selectedmime->currentRow();
 	if (index != -1)
 	{
-		m_availablemime->insertItem(m_selectedmime->currentText());
-		m_selectedmime->removeItem(index);
-		m_availablemime->sort();
+		m_availablemime->addItem(m_selectedmime->currentItem()->text());
+		QListWidgetItem* item = m_selectedmime->takeItem(index);
+        delete item;
+		m_availablemime->model()->sort(0);
 	}
 }
 
