@@ -27,6 +27,7 @@
 #include <QHBoxLayout>
 #include <QCloseEvent>
 #include <QTextCodec>
+#include <QProcess>
 
 #include <kaboutdata.h>
 #include "ktoolinvocation.h"
@@ -36,10 +37,9 @@
 #include <klineedit.h>
 #include <klocale.h>
 #include <kmessagebox.h>
-#include <kprocess.h>
 #include <kstandarddirs.h>
-#include <kstdguiitem.h>
-#include <kinstance.h>
+#include <kstandardguiitem.h>
+#include <kcomponentdata.h>
 #include <kurl.h>
 #include <kurllabel.h>
 
@@ -54,8 +54,6 @@
 #include "kdepackages.h"
 #include <kcombobox.h>
 #include <config.h>
-#include <ktempfile.h>
-#include <qtextstream.h>
 #include <qfile.h>
 
 #include <config-compiler.h>
@@ -85,12 +83,11 @@ KBugReport::KBugReport( QWidget * _parent, bool modal, const KAboutData *aboutDa
   showButtonSeparator(true);
   setModal(modal);
 
-  // Use supplied aboutdata, otherwise the one from the active instance
+  // Use supplied aboutdata, otherwise the one from the active componentData
   // otherwise the KGlobal one. _activeInstance should neved be 0L in theory.
-  m_aboutData = aboutData
-    ? aboutData
-    : ( KGlobal::activeInstance() ? KGlobal::activeInstance()->aboutData()
-                                  : KGlobal::instance()->aboutData() );
+  m_aboutData = aboutData ? aboutData
+      : (KGlobal::activeComponent().isValid() ? KGlobal::activeComponent().aboutData()
+                                  : KGlobal::mainComponent().aboutData());
   m_process = 0;
   QWidget * parent = new QWidget(this);
   d->submitBugButton = 0;
@@ -99,7 +96,7 @@ KBugReport::KBugReport( QWidget * _parent, bool modal, const KAboutData *aboutDa
   {
     // This is a core KDE application -> redirect to the web form
     d->submitBugButton = new QPushButton( parent );
-    setButtonGuiItem( Cancel, KStdGuiItem::close() );
+    setButtonGuiItem( Cancel, KStandardGuiItem::close() );
   }
 
   QLabel * tmpLabel;
@@ -331,10 +328,10 @@ void KBugReport::appChanged(int i)
 void KBugReport::slotConfigureEmail()
 {
   if (m_process) return;
-  m_process = new KProcess;
-  *m_process << QString::fromLatin1("kcmshell") << QString::fromLatin1("kcm_useraccount");
-  connect(m_process, SIGNAL(processExited(KProcess *)), SLOT(slotSetFrom()));
-  if (!m_process->start())
+  m_process = new QProcess;
+  connect( m_process, SIGNAL(finished( int, QProcess::ExitStatus )), SLOT(slotSetFrom()) );
+  m_process->start( QString::fromLatin1("kcmshell"), QStringList() << QString::fromLatin1("kcm_useraccount") );
+  if ( !m_process->waitForStarted() )
   {
     kDebug() << "Couldn't start kcmshell.." << endl;
     delete m_process;
@@ -399,7 +396,7 @@ void KBugReport::accept()
                 "<li>cause serious data loss</li>"
                 "<li>introduce a security hole on the system where the affected package is installed</li></ul>\n"
                 "<p>Does the bug you are reporting cause any of the above damage? "
-                "If it does not, please select a lower severity. Thank you!</p>" ),QString(),KStdGuiItem::cont(),KStdGuiItem::cancel() ) == KMessageBox::No )
+                "If it does not, please select a lower severity. Thank you!</p>" ),QString(),KStandardGuiItem::cont(),KStandardGuiItem::cancel() ) == KMessageBox::No )
                 return;
             break;
         case 1: // grave
@@ -410,7 +407,7 @@ void KBugReport::accept()
                 "<li>cause data loss</li>"
                 "<li>introduce a security hole allowing access to the accounts of users who use the affected package</li></ul>\n"
                 "<p>Does the bug you are reporting cause any of the above damage? "
-                "If it does not, please select a lower severity. Thank you!</p>" ),QString(),KStdGuiItem::cont(),KStdGuiItem::cancel() ) == KMessageBox::No )
+                "If it does not, please select a lower severity. Thank you!</p>" ),QString(),KStandardGuiItem::cont(),KStandardGuiItem::cancel() ) == KMessageBox::No )
                 return;
             break;
         default:
@@ -436,7 +433,7 @@ void KBugReport::closeEvent( QCloseEvent * e)
   {
     int rc = KMessageBox::warningYesNo( this,
              i18n( "Close and discard\nedited message?" ),
-             i18n( "Close Message" ), KStdGuiItem::discard(), KStdGuiItem::cont() );
+             i18n( "Close Message" ), KStandardGuiItem::discard(), KStandardGuiItem::cont() );
     if( rc == KMessageBox::No )
     {
         e->ignore();
@@ -467,7 +464,7 @@ QString KBugReport::text() const
 */
   bodyText=m_lineedit->toPlainText();
   if (bodyText.length()>0)
-        if (bodyText[bodyText.length()-1]!='\n') bodyText+="\n";
+        if (bodyText[bodyText.length()-1]!='\n') bodyText+='\n';
   if (severity == QString::fromLatin1("i18n") && KGlobal::locale()->language() != KLocale::defaultLanguage()) {
       // Case 1 : i18n bug
       QString package = QString::fromLatin1("i18n_%1").arg(KGlobal::locale()->language());
@@ -501,51 +498,26 @@ bool KBugReport::sendBugReport()
   if (command.isEmpty())
       command = KStandardDirs::findExe( QString::fromLatin1("ksendbugmail") );
 
-  KTempFile outputfile;
-  outputfile.close();
-
-  QString subject = m_subject->text();
-  command += " --subject ";
-  command += KProcess::quote(subject);
-  command += " --recipient ";
-  command += KProcess::quote(recipient);
-  command += " > ";
-  command += KProcess::quote(outputfile.name());
-
-  fflush(stdin);
-  fflush(stderr);
-
-  FILE * fd = popen(QFile::encodeName(command), "w");
-  if (!fd)
+  QProcess proc;
+  proc.start( command, QStringList() << " --subject " << m_subject->text() << " --recipient " << recipient );
+  if (!proc.waitForStarted())
   {
     kError() << "Unable to open a pipe to " << command << endl;
     return false;
   }
+  proc.write( text().toAscii() );
 
-  QByteArray btext = text().toAscii();
-  fwrite(btext.data(),btext.length(),1,fd);
-  fflush(fd);
+  proc.waitForFinished();
+  kDebug() << "kbugreport: sendbugmail exit, status " << proc.exitStatus() << " code " << proc.exitCode() << endl;
 
-  int error = pclose(fd);
-  kDebug() << "exit status1 " << error << " " << (WIFEXITED(error)) << " " <<  WEXITSTATUS(error) << endl;
-
-  if ((WIFEXITED(error)) && WEXITSTATUS(error) == 1) {
-      QFile of(outputfile.name());
-      if (of.open(QIODevice::ReadOnly )) {
-          QTextStream is(&of);
-          //is.setEncoding(QTextStream::UnicodeUTF8);
-          is.setCodec(QTextCodec::codecForName("UTF-8"));
-          QString line;
-          while (!is.atEnd())
-              line = is.readLine();
-          d->lastError = line;
-      } else {
-          d->lastError.clear();
-      }
-      outputfile.unlink();
+  QByteArray line;
+  if (proc.exitStatus() == QProcess::NormalExit && proc.exitCode() == 1) { // XXX too restrictive?
+      // XXX not stderr?
+      while (!proc.atEnd())
+          line = proc.readLine();
+      d->lastError = QString::fromUtf8( line );
       return false;
   }
-  outputfile.unlink();
   return true;
 }
 
