@@ -20,9 +20,9 @@
  *  Boston, MA 02110-1301, USA.
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
+
+#include "kdialog.h"
 
 #include <QApplication>
 #include <QDesktopWidget>
@@ -37,15 +37,15 @@
 #include <QVBoxLayout>
 #include <QWhatsThis>
 
-#include <kapplication.h>
 #include <kconfig.h>
 #include <kglobalsettings.h>
 #include <kguiitem.h>
+#include <kcomponentdata.h>
 #include <klocale.h>
 #include <kpushbutton.h>
 #include <kseparator.h>
 #include <kstaticdeleter.h>
-#include <kstdguiitem.h>
+#include <kstandardguiitem.h>
 #include <ktoolinvocation.h>
 #include <kurllabel.h>
 #include <kwhatsthismanager_p.h>
@@ -55,12 +55,10 @@
 #include <netwm.h>
 #endif
 
-#include "kdialog.h"
-
 struct KDialog::Private
 {
   Private( KDialog *parent )
-    : mParent( parent ), mDetails( false ), mSettingDetails( false ), mDetailsWidget( 0 ),
+    : mParent( parent ), mDetailsVisible( false ), mSettingDetails( false ), mDetailsWidget( 0 ),
       mTopLayout( 0 ), mMainWidget( 0 ), mUrlHelp( 0 ), mActionSeparator( 0 ),
       mButtonOrientation( Qt::Horizontal ),
       mButtonBox( 0 )
@@ -74,12 +72,12 @@ struct KDialog::Private
 
 
   KDialog* mParent;
-  bool mDetails;
+  bool mDetailsVisible;
   bool mSettingDetails;
   QWidget *mDetailsWidget;
   QSize mIncSize;
   QSize mMinSize;
-  QString mDetailsButton;
+  QString mDetailsButtonText;
 
   QBoxLayout *mTopLayout;
   QWidget *mMainWidget;
@@ -153,16 +151,22 @@ void KDialog::Private::appendButton( ButtonCode key, const KGuiItem &item )
       role = QDialogButtonBox::DestructiveRole;
       break;
     case Ok:
+      role = QDialogButtonBox::AcceptRole;
+      break;
     case Apply:
+      role = QDialogButtonBox::ApplyRole;
+      break;
     case Try:
     case Close:
     case Yes:
-      role = QDialogButtonBox::AcceptRole;
+      role = QDialogButtonBox::YesRole;
       break;
     case Cancel:
-    case No:
       role = QDialogButtonBox::RejectRole;
       break;
+    case No:
+      role = QDialogButtonBox::NoRole;
+      break;
     case User1:
     case User2:
     case User3:
@@ -187,12 +191,6 @@ void KDialog::Private::appendButton( ButtonCode key, const KGuiItem &item )
            &mButtonSignalMapper, SLOT( map() ) );
 }
 
-KPushButton *KDialog::Private::button( ButtonCode id ) const
-{
-  return mButtonList.contains( id ) ? mButtonList[ id ] : 0;
-}
-
-
 KDialog::KDialog( QWidget *parent, Qt::WFlags flags )
   : QDialog( parent, flags | Qt::MSWindowsFixedSizeDialogHint ),
     d( new Private( this ) )
@@ -240,9 +238,9 @@ void KDialog::setButtons( ButtonCodes buttonMask )
   d->mButtonBox = new QDialogButtonBox( this );
 
   if ( buttonMask & Help )
-    d->appendButton( Help, KStdGuiItem::help() );
+    d->appendButton( Help, KStandardGuiItem::help() );
   if ( buttonMask & Default )
-    d->appendButton( Default, KStdGuiItem::defaults() );
+    d->appendButton( Default, KStandardGuiItem::defaults() );
   if ( buttonMask & User3 )
     d->appendButton( User3, KGuiItem() );
   if ( buttonMask & User2 )
@@ -250,22 +248,22 @@ void KDialog::setButtons( ButtonCodes buttonMask )
   if ( buttonMask & User1 )
     d->appendButton( User1, KGuiItem() );
   if ( buttonMask & Ok )
-    d->appendButton( Ok, KStdGuiItem::ok() );
+    d->appendButton( Ok, KStandardGuiItem::ok() );
   if ( buttonMask & Apply )
-    d->appendButton( Apply, KStdGuiItem::apply() );
+    d->appendButton( Apply, KStandardGuiItem::apply() );
   if ( buttonMask & Try )
-    d->appendButton( Try, i18n( "&Try" ) );
+    d->appendButton( Try, KGuiItem(i18n( "&Try" )) );
   if ( buttonMask & Cancel )
-    d->appendButton( Cancel, KStdGuiItem::cancel() );
+    d->appendButton( Cancel, KStandardGuiItem::cancel() );
   if ( buttonMask & Close )
-    d->appendButton( Close, KStdGuiItem::close() );
+    d->appendButton( Close, KStandardGuiItem::close() );
   if ( buttonMask & Yes )
-    d->appendButton( Yes, KStdGuiItem::yes() );
+    d->appendButton( Yes, KStandardGuiItem::yes() );
   if ( buttonMask & No )
-    d->appendButton( No, KStdGuiItem::no() );
+    d->appendButton( No, KStandardGuiItem::no() );
   if ( buttonMask & Details ) {
-    d->appendButton( Details, QString() );
-    setDetails( false );
+    d->appendButton( Details, KGuiItem(QString()) );
+    setDetailsWidgetVisible( false );
   }
 
   d->setupLayout();
@@ -293,7 +291,7 @@ void KDialog::setEscapeButton( ButtonCode id )
 void KDialog::setDefaultButton( ButtonCode defaultButton )
 {
   if ( defaultButton != NoDefault ) {
-    KPushButton *button = d->button( defaultButton );
+    KPushButton *button = this->button( defaultButton );
     if ( button )
       d->setButtonFocus( button, true, false );
   }
@@ -319,12 +317,14 @@ void KDialog::setMainWidget( QWidget *widget )
 
 QWidget *KDialog::mainWidget()
 {
-  return d->mMainWidget;
+    if (!d->mMainWidget)
+        setMainWidget( new QWidget(this) );
+    return d->mMainWidget;
 }
 
 QSize KDialog::sizeHint() const
 {
- return d->mMinSize.expandedTo( minimumSizeHint() ) + d->mIncSize;
+    return d->mMinSize.expandedTo( minimumSizeHint() ) + d->mIncSize;
 }
 
 QSize KDialog::minimumSizeHint() const
@@ -361,7 +361,7 @@ QSize KDialog::minimumSizeHint() const
   s1.rwidth()  = qMax( s1.rwidth(), s2.rwidth() );
   s1.rheight() += s2.rheight();
 
-  if ( d->mDetailsWidget && d->mDetails ) {
+  if ( d->mDetailsWidget && d->mDetailsVisible ) {
     s2 = d->mDetailsWidget->sizeHint() + zeroByS;
     s2 = s2.expandedTo( d->mDetailsWidget->minimumSize() );
     s2 = s2.expandedTo( d->mDetailsWidget->minimumSizeHint() );
@@ -405,7 +405,7 @@ void KDialog::keyPressEvent( QKeyEvent *event )
 {
   if ( event->modifiers() == 0 ) {
     if ( event->key() == Qt::Key_F1 ) {
-      KPushButton *button = d->button( Help );
+      KPushButton *button = this->button( Help );
 
       if ( button ) {
         button->animateClick();
@@ -415,7 +415,7 @@ void KDialog::keyPressEvent( QKeyEvent *event )
     }
 
     if ( event->key() == Qt::Key_Escape ) {
-      KPushButton *button = d->button( d->mEscapeButton );
+      KPushButton *button = this->button( d->mEscapeButton );
 
       if ( button ) {
         button->animateClick();
@@ -431,7 +431,7 @@ void KDialog::keyPressEvent( QKeyEvent *event )
   } else if ( event->modifiers() == Qt::ControlModifier &&
             ( event->key() == Qt::Key_Return || event->key() == Qt::Key_Enter ) ) {
     // accept the dialog when Ctrl-Return is pressed
-    KPushButton *button = d->button( Ok );
+    KPushButton *button = this->button( Ok );
 
     if ( button ) {
       button->animateClick();
@@ -453,12 +453,52 @@ int KDialog::spacingHint()
   return Private::mSpacingSize;
 }
 
+QString KDialog::makeStandardCaption( const QString &userCaption,
+                                      QWidget* window,
+                                      CaptionFlags flags )
+{
+  Q_UNUSED(window);
+  QString caption = KGlobal::caption();
+  QString captionString = userCaption.isEmpty() ? caption : userCaption;
+
+  // If the document is modified, add '[modified]'.
+  if (flags & ModifiedCaption)
+      captionString += QString::fromUtf8(" [") + i18n("modified") + QString::fromUtf8("]");
+
+  if ( !userCaption.isEmpty() ) {
+      // Add the application name if:
+      // User asked for it, it's not a duplication  and the app name (caption()) is not empty
+      if ( flags & AppNameCaption &&
+           !caption.isEmpty() &&
+           !userCaption.endsWith(caption)  ) {
+           // TODO: check to see if this is a transient/secondary window before trying to add the app name
+           //       on platforms that need this
+          captionString += QString::fromUtf8(" - ") + caption;
+      }
+  }
+
+  return captionString;
+}
+
 void KDialog::setCaption( const QString &_caption )
 {
-  QString caption = kapp ? kapp->makeStdCaption( _caption ) : _caption;
+  QString caption = makeStandardCaption( _caption, this );
   setPlainCaption( caption );
 }
 
+void KDialog::setCaption( const QString &caption, bool modified )
+{
+    CaptionFlags flags = HIGCompliantCaption;
+
+    if ( modified )
+    {
+        flags |= ModifiedCaption;
+    }
+
+    setPlainCaption( makeStandardCaption(caption, this, flags) );
+}
+
+
 void KDialog::setPlainCaption( const QString &caption )
 {
   QDialog::setWindowTitle( caption );
@@ -620,17 +660,21 @@ void KDialog::incrementInitialSize( const QSize &size )
   adjustSize();
 }
 
+KPushButton *KDialog::button( ButtonCode id ) const
+{
+  return d->mButtonList.contains( id ) ? d->mButtonList[ id ] : 0;
+}
 
 void KDialog::enableButton( ButtonCode id, bool state )
 {
-  KPushButton *button = d->button( id );
+  KPushButton *button = this->button( id );
   if ( button )
     button->setEnabled( state );
 }
 
 bool KDialog::isButtonEnabled( ButtonCode id ) const
 {
-  KPushButton *button = d->button( id );
+  KPushButton *button = this->button( id );
   if ( button )
     return button->isEnabled();
 
@@ -654,68 +698,71 @@ void KDialog::enableButtonCancel( bool state )
 
 void KDialog::showButton( ButtonCode id, bool state )
 {
-  KPushButton *button = d->button( id );
+  KPushButton *button = this->button( id );
   if ( button )
     state ? button->show() : button->hide();
 }
 
 void KDialog::setButtonGuiItem( ButtonCode id, const KGuiItem &item )
 {
-  KPushButton *button = d->button( id );
+  KPushButton *button = this->button( id );
   if ( !button )
     return;
 
   button->setGuiItem( item );
 }
 
-void KDialog::setButtonMenu( ButtonCode id, QMenu *menu )
+void KDialog::setButtonMenu( ButtonCode id, QMenu *menu, ButtonPopupMode popupmode)
 {
-  KPushButton *button = d->button( id );
+  KPushButton *button = this->button( id );
   if ( button )
-    button->setMenu( menu );
+    if (popupmode==InstantPopup)
+      button->setMenu( menu );
+    else
+      button->setDelayedMenu(menu);
 }
 
 void KDialog::setButtonText( ButtonCode id, const QString &text )
 {
   if ( !d->mSettingDetails && (id == Details) ) {
-    d->mDetailsButton = text;
-    setDetails( d->mDetails );
+    d->mDetailsButtonText = text;
+    setDetailsWidgetVisible( d->mDetailsVisible );
     return;
   }
 
-  KPushButton *button = d->button( id );
+  KPushButton *button = this->button( id );
   if ( button )
     button->setText( text );
 }
 
 QString KDialog::buttonText( ButtonCode id ) const
 {
-  KPushButton *button = d->button( id );
+  KPushButton *button = this->button( id );
   if ( button )
     return button->text();
   else
     return QString();
 }
 
-void KDialog::setButtonIcon( ButtonCode id, const QIcon &icon )
+void KDialog::setButtonIcon( ButtonCode id, const KIcon &icon )
 {
-  KPushButton *button = d->button( id );
+  KPushButton *button = this->button( id );
   if ( button )
     button->setIcon( icon );
 }
 
-QIcon KDialog::buttonIcon( ButtonCode id ) const
+KIcon KDialog::buttonIcon( ButtonCode id ) const
 {
-  KPushButton *button = d->button( id );
+  KPushButton *button = this->button( id );
   if ( button )
-    return button->icon();
+    return KIcon(button->icon());
   else
-    return QIcon();
+    return KIcon();
 }
 
 void KDialog::setButtonToolTip( ButtonCode id, const QString &text )
 {
-  KPushButton *button = d->button( id );
+  KPushButton *button = this->button( id );
   if ( button ) {
     if ( text.isEmpty() )
       button->setToolTip( QString() );
@@ -726,7 +773,7 @@ void KDialog::setButtonToolTip( ButtonCode id, const QString &text )
 
 QString KDialog::buttonToolTip( ButtonCode id ) const
 {
-  KPushButton *button = d->button( id );
+  KPushButton *button = this->button( id );
   if ( button )
     return button->toolTip();
   else
@@ -735,7 +782,7 @@ QString KDialog::buttonToolTip( ButtonCode id ) const
 
 void KDialog::setButtonWhatsThis( ButtonCode id, const QString &text )
 {
-  KPushButton *button = d->button( id );
+  KPushButton *button = this->button( id );
   if ( button ) {
     if ( text.isEmpty() )
       button->setWhatsThis( QString() );
@@ -746,7 +793,7 @@ void KDialog::setButtonWhatsThis( ButtonCode id, const QString &text )
 
 QString KDialog::buttonWhatsThis( ButtonCode id ) const
 {
-  KPushButton *button = d->button( id );
+  KPushButton *button = this->button( id );
   if ( button )
     return button->whatsThis();
   else
@@ -755,7 +802,7 @@ QString KDialog::buttonWhatsThis( ButtonCode id ) const
 
 void KDialog::setButtonFocus( ButtonCode id )
 {
-  KPushButton *button = d->button( id );
+  KPushButton *button = this->button( id );
   if ( button )
     d->setButtonFocus( button, false, true );
 }
@@ -772,19 +819,24 @@ void KDialog::setDetailsWidget( QWidget *detailsWidget )
   d->setupLayout();
 
   if ( !d->mSettingDetails )
-    setDetails( d->mDetails );
+    setDetailsWidgetVisible( d->mDetailsVisible );
 }
 
-void KDialog::setDetails( bool showDetails )
+bool KDialog::isDetailsWidgetVisible() const
 {
-  if ( d->mDetailsButton.isEmpty() )
-    d->mDetailsButton = i18n( "&Details" );
+    return d->mDetailsVisible;
+}
+
+void KDialog::setDetailsWidgetVisible( bool visible )
+{
+  if ( d->mDetailsButtonText.isEmpty() )
+    d->mDetailsButtonText = i18n( "&Details" );
 
   d->mSettingDetails = true;
-  d->mDetails = showDetails;
-  if ( d->mDetails ) {
+  d->mDetailsVisible = visible;
+  if ( d->mDetailsVisible ) {
     emit aboutToShowDetails();
-    setButtonText( Details, d->mDetailsButton + " <<" );
+    setButtonText( Details, d->mDetailsButtonText + " <<" );
     if ( d->mDetailsWidget ) {
       if ( layout() )
         layout()->setEnabled( false );
@@ -799,7 +851,7 @@ void KDialog::setDetails( bool showDetails )
       }
     }
   } else {
-    setButtonText( Details, d->mDetailsButton + " >>" );
+    setButtonText( Details, d->mDetailsButtonText + " >>" );
     if ( d->mDetailsWidget )
       d->mDetailsWidget->hide();
 
@@ -844,7 +896,7 @@ void KDialog::slotButtonClicked( int button )
       break;
     case User1:
         emit user1Clicked();
-      break;
+        break;
     case Yes:
       emit yesClicked();
       done( Yes );
@@ -870,7 +922,7 @@ void KDialog::slotButtonClicked( int button )
       emit defaultClicked();
       break;
     case Details:
-      setDetails( !d->mDetails );
+      setDetailsWidgetVisible( !d->mDetailsVisible );
       break;
   }
 }
@@ -939,7 +991,7 @@ void KDialog::hideEvent( QHideEvent *event )
 
 void KDialog::closeEvent( QCloseEvent *event )
 {
-  KPushButton *button = d->button( d->mEscapeButton );
+  KPushButton *button = this->button( d->mEscapeButton );
   if ( button && !isHidden() )
     button->animateClick();
   else
