@@ -24,6 +24,7 @@
 #ifndef KDIALOG_H
 #define KDIALOG_H
 
+class KPushButton;
 class QMenu;
 
 #include <QtGui/QDialog>
@@ -41,7 +42,7 @@ class QMenu;
  * It offers the standard action buttons you'd expect to find in a
  * dialog as well as the capability to define at most three configurable
  * buttons. You can define a main widget that contains your specific
- * dialog layout 
+ * dialog layout
  *
  * The class takes care of the geometry management. You only need to define
  * a minimum size for the widget you want to use as the main widget.
@@ -103,7 +104,7 @@ class QMenu;
  *   dialog->setCaption( "My title" );
  *   dialog->setButtons( KDialog::Ok | KDialog::Cancel | KDialog::Apply );
  *
- *   FooWidget widget = new FooWidget( dialog );
+ *   FooWidget *widget = new FooWidget( dialog );
  *   dialog->setMainWidget( widget );
  *   connect( dialog, SIGNAL( applyClicked() ), widget, SLOT( save() ) );
  *   connect( dialog, SIGNAL( okClicked() ), widget, SLOT( save() ) );
@@ -122,13 +123,14 @@ class QMenu;
  * @see KPageDialog
  * @author Thomas Tanghus <tanghus@earthling.net>
  * @author Espen Sand <espensa@online.no>
- * @author Mirko Boehm <mirko@kde.org> 
+ * @author Mirko Boehm <mirko@kde.org>
  * @author Olivier Goffart <ogoffart at kde.org>
  * @author Tobias Koenig <tokoe@kde.org>
  */
 class KDEUI_EXPORT KDialog : public QDialog
 {
   Q_OBJECT
+  Q_ENUMS(ButtonCode)
 
   public:
 
@@ -148,10 +150,17 @@ class KDEUI_EXPORT KDialog : public QDialog
       User1   = 0x00001000, ///< Show User defined button 1.
       User2   = 0x00002000, ///< Show User defined button 2.
       User3   = 0x00004000, ///< Show User defined button 3.
-      NoDefault = 0x00008000 ///< Used when specifying a default button; indicates that no button should be marked by default. 
+      NoDefault = 0x00008000 ///< Used when specifying a default button; indicates that no button should be marked by default.
     };
     Q_DECLARE_FLAGS(ButtonCodes, ButtonCode)
 
+    enum ButtonPopupMode
+    {
+      InstantPopup = 0,
+      DelayedPopup = 1
+    };
+    Q_DECLARE_FLAGS(ButtonPopupModes, ButtonPopupMode)
+
   public:
     /**
      * Creates a dialog.
@@ -167,7 +176,7 @@ class KDEUI_EXPORT KDialog : public QDialog
     ~KDialog();
 
     /**
-     * Creates (or recreates) the button box and all the buttons in it. 
+     * Creates (or recreates) the button box and all the buttons in it.
      *
      * This will reset all default KGuiItem of all button.
      *
@@ -251,12 +260,12 @@ class KDEUI_EXPORT KDialog : public QDialog
      * @param id The button identifier.
      * @param icon Button icon.
      */
-    void setButtonIcon( ButtonCode id, const QIcon &icon );
+    void setButtonIcon( ButtonCode id, const KIcon &icon );
 
     /**
      * Returns the icon of any button.
      */
-    QIcon buttonIcon( ButtonCode id ) const;
+    KIcon buttonIcon( ButtonCode id ) const;
 
     /**
      * Sets the tooltip text of any button.
@@ -300,8 +309,9 @@ class KDEUI_EXPORT KDialog : public QDialog
      *
      * @param id The button identifier.
      * @param menu The menu.
+     * @param popupmode Choose if KPushButton setMenu or setDelayedMenu is used
      */
-    void setButtonMenu( ButtonCode id, QMenu *menu );
+    void setButtonMenu( ButtonCode id, QMenu *menu, ButtonPopupMode popupmode=InstantPopup);
 
     /**
      * Sets the focus to the button of the passed @param id.
@@ -365,6 +375,18 @@ class KDEUI_EXPORT KDialog : public QDialog
      */
     bool isButtonEnabled( ButtonCode id ) const;
 
+    /**
+     * Returns the button that corresponds to the @p id.
+     *
+     * Normally you should not use this function.
+     * @em Never delete the object returned by this function.
+     * See also enableButton(), showButton(), setButtonGuiItem().
+     *
+     * @param id Identifier of the button.
+     * @return The button or 0 if the button does not exist.
+     */
+    KPushButton* button( ButtonCode id ) const;
+
     /**
      * Return the number of pixels you shall use between a
      * dialog edge and the outermost widget(s) according to the KDE standard.
@@ -377,6 +399,51 @@ class KDEUI_EXPORT KDialog : public QDialog
      */
     static int spacingHint();
 
+    /**
+     * @enum StandardCaptionFlag
+     * Used to specify how to construct a window caption
+     *
+     * @value AppName Indicates that the method shall include
+     * the application name when making the caption string.
+     * @value Modified Causes a 'modified' sign will be included in the
+     * returned string. This is useful when indicating that a file is
+     * modified, i.e., it contains data that has not been saved.
+     * @value HIGCompliant The base minimum flags required to align a
+     * caption with the KDE Human Interface Guidelines
+     */
+    enum CaptionFlag
+    {
+        NoCaptionFlags = 0,
+        AppNameCaption = 1,
+        ModifiedCaption = 2,
+        HIGCompliantCaption = AppNameCaption
+    };
+    Q_DECLARE_FLAGS(CaptionFlags, CaptionFlag)
+
+    /**
+     * Builds a caption that contains the application name along with the
+     * userCaption using a standard layout.
+     *
+     * To make a compliant caption for your window, simply do:
+     * @p setWindowTitle(KDialog::makeStandardCaption(yourCaption));
+     *
+     * To ensure that the caption is appropriate to the desktop in which the
+     * application is running, pass in a pointer to the window the caption will
+     * be applied to.
+     *
+     * If using a KDialog or KMainWindow subclass, call setCaption instead and
+     * an appropraite standard caption will be created for you
+     *
+     * @param userCaption The caption string you want to display in the
+     * window caption area. Do not include the application name!
+     * @param window a pointer to the window this application will apply to
+     * @param flags
+     * @return the created caption
+     */
+    static QString makeStandardCaption( const QString &userCaption,
+                                        QWidget* window = 0,
+                                        CaptionFlags flags = HIGCompliantCaption );
+
     /**
      * Resize every layout manager used in @p widget and its nested children.
      *
@@ -422,7 +489,12 @@ class KDEUI_EXPORT KDialog : public QDialog
     void setMainWidget( QWidget *widget );
 
     /**
-     * @return The current main widget. Can be 0 if no widget has been defined.
+     * @return The current main widget. Will create a QWidget as the mainWidget
+     * if none was set before. This way you can write
+     * \code
+     *   ui.setupUi(mainWidget());
+     * \endcode
+     * when using designer.
      */
     QWidget *mainWidget();
 
@@ -446,6 +518,17 @@ class KDEUI_EXPORT KDialog : public QDialog
      */
     virtual void setCaption( const QString &caption );
 
+    /**
+     * Makes a KDE compliant caption.
+     *
+     * @param caption Your caption. @em Do @em not include the application name
+     * in this string. It will be added automatically according to the KDE
+     * standard.
+     * @param modified Specify whether the document is modified. This displays
+     * an additional sign in the title bar, usually "**".
+     */
+    virtual void setCaption( const QString &caption, bool modified );
+
     /**
      * Make a plain caption without any modifications.
      *
@@ -514,10 +597,15 @@ class KDEUI_EXPORT KDialog : public QDialog
      */
     void setHelp( const QString &anchor, const QString &appname = QString() );
 
+    /**
+     * Returns the status of the Details button.
+     */
+    bool isDetailsWidgetVisible() const;
+
     /**
      * Sets the status of the Details button.
      */
-    void setDetails( bool showDetails );
+    void setDetailsWidgetVisible( bool visible );
 
     /**
      * Sets the widget that gets shown when "Details" is enabled.
@@ -693,6 +781,7 @@ class KDEUI_EXPORT KDialog : public QDialog
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(KDialog::ButtonCodes)
+Q_DECLARE_OPERATORS_FOR_FLAGS(KDialog::CaptionFlags)
 
 
 /**
