@@ -32,7 +32,7 @@ class QFont;
 class QLabel;
 class QLineEdit;
 class QStringList;
-class KListBox;
+class KListWidget;
 class KIntNumInput;
 
 /**
@@ -73,8 +73,8 @@ public:
    * <p>Consider the following code snippet;
    * \code
    *    QStringList list;
-   *    KFontChooser::getFontList(list,SmoothScalableFonts);
-   *    KFontChooser chooseFont = new KFontChooser(0, "FontList", false, list);
+   *    KFontChooser::getFontList(list, KFontChooser::SmoothScalableFonts);
+   *    KFontChooser *chooseFont = new KFontChooser(0, false, list);
    * \endcode
    * <p>
    * The above creates a font chooser dialog with only SmoothScaleble fonts.
@@ -98,7 +98,6 @@ public:
    *        *sizeIsRelativeState, user choice may be retrieved by
    *        calling sizeIsRelative().
    */
-    /// KDE4: copy to KDE3SUPPORT
   explicit KFontChooser(QWidget *parent = 0L,
                         bool onlyFixed = false,
                         const QStringList &fontList = QStringList(),
@@ -278,9 +277,9 @@ private:
   QCheckBox    *styleCheckbox;
   QCheckBox    *sizeCheckbox;
   QLabel       *sizeLabel;
-  KListBox     *familyListBox;
-  KListBox     *styleListBox;
-  KListBox     *sizeListBox;
+  KListWidget     *familyListBox;
+  KListWidget     *styleListBox;
+  KListWidget     *sizeListBox;
   QCheckBox    *sizeIsRelativeCheckBox;
 
   QFont        selFont;
