@@ -34,19 +34,20 @@
 #include <QHeaderView>
 
 #include <kaction.h>
-#include <kapplication.h>
 #include <kconfig.h>
 #include <kdebug.h>
 #include <kglobal.h>
 #include <kglobalaccel.h>
 #include <kicon.h>
+#include <kcomponentdata.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kxmlguifactory.h>
 #include <kaboutdata.h>
 #include <kshortcut.h>
 #include <kstaticdeleter.h>
-#include <kstdaccel.h>
+#include <kstandardshortcut.h>
+#include <kconfiggroup.h>
 
 #include "kactioncollection.h"
 
@@ -78,15 +79,15 @@ class KKeyChooserItem : public QTreeWidgetItem
 
 	KAction* action() const;
 	QString actionName() const;
-	const KShortcut& shortcut() const;
+	KShortcut shortcut() const;
 	bool isConfigurable() const;
-	const KShortcut& defaultShortcut() const;
+	KShortcut defaultShortcut() const;
 
 	void setLocalShortcut( const KShortcut& cut );
 	void setGlobalShortcut( const KShortcut& cut );
-	
-	const KShortcut& globalShortcut() const;
-	const KShortcut& defaultGlobalShortcut() const;
+
+        KShortcut globalShortcut() const;
+	KShortcut defaultGlobalShortcut() const;
 
 	void commitChanges();
 
@@ -94,8 +95,7 @@ class KKeyChooserItem : public QTreeWidgetItem
 
  private:
 	void checkModified();
-	bool checkChanged(const KShortcut& cut1, const KShortcut& cut2, int sequence) const;
- 
+
 	KAction* m_action;
 	bool m_modified;
 	KShortcut m_newLocalShortcut, m_newGlobalShortcut;
@@ -112,17 +112,17 @@ class KKeyChooserPrivate
  		: ui(new Ui::KKeyDialog())
 	{
 	}
-	
+
 	~KKeyChooserPrivate()
 	{
 		delete ui;
 	}
 
-	QList< QList<KAction*> > rgpLists;
+	QList< QList<QAction*> > rgpLists;
 	QList< KActionCollection* > collections;
 
 	Ui::KKeyDialog* ui;
-	
+
 	// If this is set, then shortcuts require a modifier:
 	//  so 'A' would not be valid, whereas 'Ctrl+A' would be.
 	// Note, however, that this only applies to printable characters.
@@ -167,14 +167,15 @@ KKeyChooser::~KKeyChooser()
 
 bool KKeyChooser::insert( KActionCollection* pColl, const QString &title )
 {
-	QString str = title;
-	if ( title.isEmpty() && pColl->instance() && pColl->instance()->aboutData() )
-		str = pColl->instance()->aboutData()->programName();
+    QString str = title;
+    if (title.isEmpty() && pColl->componentData().isValid() && pColl->componentData().aboutData()) {
+        str = pColl->componentData().aboutData()->programName();
+    }
 
 	d->rgpLists.append( pColl->actions() );
 	d->collections.append( pColl );
 	buildListView(d->rgpLists.count() - 1, str);
-	
+
 	return true;
 }
 
@@ -211,8 +212,8 @@ void KKeyChooser::initGUI( ActionTypes type, LetterShortcuts allowLetterShortcut
 	d->ui->list->header()->setStretchLastSection(false);
 	if (!(d->type & GlobalAction)) {
 		d->ui->list->header()->hideSection(3);
-	
-	} else if (d->type & ~GlobalAction) {
+
+	} else if (!(d->type & ~GlobalAction)) {
 		d->ui->list->header()->hideSection(1);
 		d->ui->list->header()->hideSection(2);
 	}
@@ -234,7 +235,7 @@ void KKeyChooser::initGUI( ActionTypes type, LetterShortcuts allowLetterShortcut
 	connect( d->ui->globalCustom, SIGNAL(clicked()), SLOT(slotGlobalCustomKey()) );
 	connect( d->ui->globalCustomSelector, SIGNAL(capturedShortcut(const KShortcut&)), SLOT(slotGlobalCapturedShortcut(const KShortcut&)) );
 
-	connect( kapp, SIGNAL( settingsChanged( int )), SLOT( slotSettingsChanged( int )));
+	connect( KGlobalSettings::self(), SIGNAL( settingsChanged( int )), SLOT( slotSettingsChanged( int )));
 
 	if( allChoosers == NULL )
 		allChoosers = allChoosersDeleter.setObject( allChoosers, new QList< KKeyChooser* > );
@@ -244,16 +245,16 @@ void KKeyChooser::initGUI( ActionTypes type, LetterShortcuts allowLetterShortcut
 // Add all shortcuts to the list
 void KKeyChooser::buildListView( uint iList, const QString &title )
 {
-	const QList<KAction*> pList = d->rgpLists[iList];
+	const QList<QAction*> pList = d->rgpLists[iList];
 
 	QTreeWidgetItem *pProgramItem, *pGroupItem = 0, *pParentItem, *pItem;
 
 	QString str = (title.isEmpty() ? i18n("Shortcuts") : title);
 	pParentItem = pProgramItem = pItem = new QTreeWidgetItem( d->ui->list, QStringList() << str );
 	d->ui->list->expandItem(pParentItem);
-	//pParentItem->setFlags(pParentItem->flags() & !Qt::ItemIsSelectable);
-	
-	foreach (KAction* action, pList) {
+	pParentItem->setFlags(pParentItem->flags() & ~Qt::ItemIsSelectable);
+
+	foreach (QAction* action, pList) {
 		ActionTypes thisType = 0;
 		switch (action->shortcutContext()) {
 			case Qt::ApplicationShortcut:
@@ -267,7 +268,9 @@ void KKeyChooser::buildListView( uint iList, const QString &title )
 				break;
 		}
 
-		if (action->globalShortcutAllowed())
+                KAction *kaction = qobject_cast<KAction*>(action);
+
+		if (kaction!=0 && kaction->globalShortcutAllowed())
 			thisType &= GlobalAction;
 
 		if (!(thisType & d->type))
@@ -278,7 +281,7 @@ void KKeyChooser::buildListView( uint iList, const QString &title )
 		if( sName.startsWith( "Program:" ) ) {
 			pItem = new QTreeWidgetItem( d->ui->list, pProgramItem );
 			pItem->setText(0, action->text());
-			//pItem->setFlags(pItem->flags() & !Qt::ItemIsSelectable);
+			pItem->setFlags(pItem->flags() & ~Qt::ItemIsSelectable);
 			d->ui->list->expandItem(pItem);
 			if( !pProgramItem->childCount() )
 				delete pProgramItem;
@@ -286,13 +289,13 @@ void KKeyChooser::buildListView( uint iList, const QString &title )
 		} else if( sName.startsWith( "Group:" ) ) {
 			pItem = new QTreeWidgetItem( pProgramItem, pParentItem );
 			pItem->setText(0, action->text());
-			//pItem->setFlags(pItem->flags() & !Qt::ItemIsSelectable);
+			pItem->setFlags(pItem->flags() & ~Qt::ItemIsSelectable);
 			d->ui->list->expandItem(pItem);
 			if( pGroupItem && !pGroupItem->childCount() )
 				delete pGroupItem;
 			pGroupItem = pParentItem = pItem;
-		} else if( !sName.isEmpty() && sName != "unnamed"  && action->isShortcutConfigurable() ) {
-			pItem = new KKeyChooserItem( pParentItem, pItem, action );
+		} else if( !sName.isEmpty() && sName != "unnamed"  && kaction!=0 && kaction->isShortcutConfigurable() ) {
+			pItem = new KKeyChooserItem( pParentItem, pItem, kaction );
 		}
 	}
 	if( !pProgramItem->childCount() )
@@ -328,19 +331,19 @@ void KKeyChooser::updateButtons()
 
 		d->ui->localCustomSelector->setShortcut( pItem->shortcut() );
 		d->ui->globalCustomSelector->setShortcut( pItem->globalShortcut() );
-		
+
 		d->ui->localDefault->setText( i18nc("Default (default shortcut)", "De&fault (%1)", localKeyStrDef.isEmpty() ? i18n("none") : localKeyStrDef) );
 		d->ui->globalDefault->setText( i18nc("Default (default shortcut)", "De&fault (%1)" , globalKeyStrDef.isEmpty() ? i18n("none") : globalKeyStrDef) );
 
 		// Select the appropriate radio button.
-		if (pItem->shortcut().isNull())
+		if (pItem->shortcut().isEmpty())
 			d->ui->localNone->setChecked( true );
 		else if (pItem->shortcut() == pItem->defaultShortcut())
 			d->ui->localDefault->setChecked( true );
 		else
 			d->ui->localCustom->setChecked( true );
 
-		if (pItem->globalShortcut().isNull())
+		if (pItem->globalShortcut().isEmpty())
 			d->ui->globalNone->setChecked( true );
 		else if (pItem->globalShortcut() == pItem->defaultGlobalShortcut())
 			d->ui->globalDefault->setChecked( true );
@@ -350,10 +353,10 @@ void KKeyChooser::updateButtons()
 		// Enable buttons if this key is configurable.
 		// The 'Default Key' button must also have a default key.
 		d->ui->localGroup->setEnabled( pItem->isConfigurable() );
-		d->ui->localDefault->setEnabled( pItem->isConfigurable() && pItem->defaultShortcut().count() != 0 );
+		d->ui->localDefault->setEnabled( pItem->isConfigurable() && !pItem->defaultShortcut().isEmpty() );
 
 		d->ui->globalGroup->setEnabled( pItem->action()->globalShortcutAllowed() );
-		d->ui->globalDefault->setEnabled( pItem->action()->globalShortcutAllowed() && pItem->defaultGlobalShortcut().count() != 0 );
+		d->ui->globalDefault->setEnabled( pItem->action()->globalShortcutAllowed() && !pItem->defaultGlobalShortcut().isEmpty() );
 	}
 }
 
@@ -409,8 +412,8 @@ void KKeyChooser::slotGlobalCustomKey()
 
 void KKeyChooser::slotSettingsChanged( int category )
 {
-	if( category == KApplication::SETTINGS_SHORTCUTS )
-		KGlobalAccel::self()->readSettings();
+    if( category == KGlobalSettings::SETTINGS_SHORTCUTS )
+        KGlobalAccel::self()->readSettings();
 }
 
 // KDE4 IMHO this shouldn't be here at all - it cannot check whether the default
@@ -448,7 +451,7 @@ void KKeyChooser::captureCurrentItem()
 
 void KKeyChooser::slotLocalCapturedShortcut( const KShortcut& cut )
 {
-	if( cut.isNull() )
+	if( cut.isEmpty() )
 		slotLocalNoKey();
 	else
 		setLocalShortcut( cut );
@@ -456,7 +459,7 @@ void KKeyChooser::slotLocalCapturedShortcut( const KShortcut& cut )
 
 void KKeyChooser::slotGlobalCapturedShortcut( const KShortcut& cut )
 {
-	if( cut.isNull() )
+	if( cut.isEmpty() )
 		slotGlobalNoKey();
 	else
 		setGlobalShortcut( cut );
@@ -469,10 +472,7 @@ void KKeyChooser::setLocalShortcut( const KShortcut& cut )
 	if( !pItem )
 		return;
 
-	foreach (const QKeySequence& seq, cut.sequences()) {
-		if (seq.isEmpty())
-			continue;
-		
+	foreach (const QKeySequence& seq, cut) {
 		int key = seq[0];
 
 		if( !d->allowLetterShortcuts && (key & Qt::KeyboardModifierMask) == 0
@@ -502,10 +502,10 @@ void KKeyChooser::setGlobalShortcut( const KShortcut& cut )
 	if( !pItem )
 		return;
 
-	foreach (const QKeySequence& seq, cut.sequences()) {
+	foreach (const QKeySequence& seq, cut) {
 		if (seq.isEmpty())
 			continue;
-		
+
 		int key = seq[0];
 
 		if( !d->allowLetterShortcuts && (key & Qt::KeyboardModifierMask) == 0
@@ -530,22 +530,39 @@ void KKeyChooser::setGlobalShortcut( const KShortcut& cut )
 
 // Returns iSeq index if cut2 has a sequence of equal or higher priority to a sequence in cut.
 // else -1
-static int keyConflict( const KShortcut& cut, const KShortcut& cut2 )
+static QKeySequence keyConflict( const KShortcut& cut, const KShortcut& cut2 )
 {
-	for( int iSeq = 0; iSeq < cut.count(); iSeq++ ) {
-		for( int iSeq2 = 0; iSeq2 < cut2.count(); iSeq2++ ) {
-			if( cut.seq(iSeq) == cut2.seq(iSeq2) )
-				return iSeq;
+/*	QList<QKeySequence> sl = seqList(cut);
+	QList<QKeySequence> sl2 = seqList(cut2);
+	for( int iSeq = 0; iSeq < sl.count(); iSeq++ ) {
+		for( int iSeq2 = 0; iSeq2 < sl2.count(); iSeq2++ ) {
+			if( sl[iSeq] == sl2[iSeq2] )
+				return sl[iSeq];
 		}
 	}
-	return -1;
+	return QKeySequence();*/
+
+//looks stupid but probably runs faster
+	const QKeySequence &cPri = cut.primary();
+	const QKeySequence &cAlt = cut.alternate();
+	const QKeySequence &c2Pri = cut2.primary();
+	const QKeySequence &c2Alt = cut2.alternate();
+	if (!cPri.isEmpty()) {
+		if (cPri == c2Pri || cPri == c2Alt)
+			return cPri;
+	}
+	if (!cAlt.isEmpty()) {
+		if (cAlt == c2Pri || cAlt == c2Alt)
+			return cAlt;
+	}
+	return QKeySequence();
 }
 
 // Removes the sequences in cut2 from cut1
-static void removeFromShortcut(  KShortcut & cut1, const KShortcut &cut2)
+static void removeFromShortcut( KShortcut & cut1, const KShortcut &cut2 )
 {
-	for( int iSeq2 = 0; iSeq2 < cut2.count(); iSeq2++ )
-		cut1.remove(cut2.seq(iSeq2));
+	cut1.removeAll(cut2.primary());
+	cut1.removeAll(cut2.alternate());
 }
 
 bool KKeyChooser::isKeyPresent( const KShortcut& cut, bool bWarnUser )
@@ -561,8 +578,8 @@ bool KKeyChooser::isKeyPresent( const KShortcut& cut, bool bWarnUser )
 	for( QList< KKeyChooser* >::ConstIterator it = allChoosers->begin();
 		 it != allChoosers->end();
 		 ++it ) {
-		has_global_chooser |= ((*it)->d->type & GlobalAction);
-		has_standard_chooser |= ((*it)->d->type & WidgetAction);
+		has_global_chooser |= (((*it)->d->type & GlobalAction) == GlobalAction);
+		has_standard_chooser |= (((*it)->d->type & WidgetAction) == WidgetAction);
 	}
 
 	// If editing global shortcuts, check them for conflicts with the stdaccels.
@@ -602,10 +619,10 @@ bool KKeyChooser::isKeyPresentLocally( const KShortcut& cut, KKeyChooserItem* ig
 	for( QTreeWidgetItemIterator it( d->ui->list ); *it; ++it ) {
 		KKeyChooserItem* pItem2 = dynamic_cast<KKeyChooserItem*>(*it);
 		if( pItem2 && pItem2 != ignoreItem ) {
-			int iSeq = keyConflict( cut, pItem2->shortcut() );
-			if( iSeq > -1 ) {
+			QKeySequence conflict = keyConflict( cut, pItem2->shortcut() );
+			if( !conflict.isEmpty() ) {
 				if( bWarnUser ) {
-					if( !promptForReassign( cut.seq(iSeq), pItem2->actionName(), WindowAction, this ))
+					if( !promptForReassign( conflict, pItem2->actionName(), WindowAction, this ))
 						return true;
 					// else remove the shortcut from it
 					KShortcut cut2 = pItem2->shortcut();
@@ -615,10 +632,10 @@ bool KKeyChooser::isKeyPresentLocally( const KShortcut& cut, KKeyChooserItem* ig
 					emit keyChange();
 				}
 			}
-			iSeq = keyConflict( cut, pItem2->globalShortcut() );
-			if( iSeq > -1 ) {
+			conflict = keyConflict( cut, pItem2->globalShortcut() );
+			if( !conflict.isEmpty() ) {
 				if( bWarnUser ) {
-					if( !promptForReassign( cut.seq(iSeq), pItem2->actionName(), WindowAction, this ))
+					if( !promptForReassign( conflict, pItem2->actionName(), WindowAction, this ))
 						return true;
 					// else remove the shortcut from it
 					KShortcut cut2 = pItem2->globalShortcut();
@@ -636,14 +653,14 @@ bool KKeyChooser::isKeyPresentLocally( const KShortcut& cut, KKeyChooserItem* ig
 bool KKeyChooser::checkStandardShortcutsConflict( const KShortcut& cut, bool bWarnUser, QWidget* parent )
 {
 	// For each key sequence in the shortcut,
-	foreach (const QKeySequence& seq, cut.sequences()) {
-		KStdAccel::StdAccel id = KStdAccel::findStdAccel( seq );
-		if( id != KStdAccel::AccelNone
-			&& keyConflict( cut, KStdAccel::shortcut( id ) ) > -1 ) {
+	foreach (const QKeySequence& seq, cut) {
+		KStandardShortcut::StandardShortcut id = KStandardShortcut::findStandardShortcut( seq );
+		if( id != KStandardShortcut::AccelNone
+			&& !keyConflict( cut, KStandardShortcut::shortcut( id ) ).isEmpty() ) {
 			if( bWarnUser ) {
-				if( !promptForReassign( seq, KStdAccel::label(id), WidgetAction, parent ))
+				if( !promptForReassign( seq, KStandardShortcut::label(id), WidgetAction, parent ))
 					return true;
-				removeStandardShortcut( KStdAccel::label(id), dynamic_cast< KKeyChooser* > ( parent ), cut);
+				removeStandardShortcut( KStandardShortcut::label(id), dynamic_cast< KKeyChooser* > ( parent ), cut);
 			}
 		}
 	}
@@ -653,11 +670,11 @@ bool KKeyChooser::checkStandardShortcutsConflict( const KShortcut& cut, bool bWa
 bool KKeyChooser::checkGlobalShortcutsConflictInternal( const KShortcut& cut, bool bWarnUser, QWidget* parent, const QString& ignoreAction )
 {
 	foreach (KAction* action, KGlobalAccel::self()->actionsWithGlobalShortcut()) {
-		int iSeq = keyConflict( cut, action->globalShortcut() );
-		if( iSeq > -1 ) {
+		QKeySequence conflict = keyConflict( cut, action->globalShortcut() );
+		if( !conflict.isEmpty() ) {
 			if( ignoreAction.isEmpty() || action->objectName() != ignoreAction ) {
 				if( bWarnUser ) {
-					if( !promptForReassign( cut.seq(iSeq), action->objectName(), GlobalAction, parent ))
+					if( !promptForReassign( conflict, action->objectName(), GlobalAction, parent ))
 						return true;
 					removeGlobalShortcut( action, dynamic_cast< KKeyChooser* >( parent ), cut);
 				}
@@ -679,9 +696,9 @@ void KKeyChooser::removeStandardShortcut( const QString& name, KKeyChooser* choo
 	}
 
 	if( !was_in_choosers ) { // not edited, needs to be changed in config file
-		KStdAccel::StdAccel id = KStdAccel::findStdAccel( name.toAscii().constData() );
-		if (id != KStdAccel::AccelNone)
-			KStdAccel::saveShortcut(id, cut);
+		KStandardShortcut::StandardShortcut id = KStandardShortcut::findStandardShortcut( name.toAscii().constData() );
+		if (id != KStandardShortcut::AccelNone)
+			KStandardShortcut::saveShortcut(id, cut);
 		else
 			kWarning(125) << k_funcinfo << "Cannot find stdaccel matching name " << name << endl;
 	}
@@ -723,7 +740,7 @@ bool KKeyChooser::removeShortcut( const QString& name, const KShortcut &cut )
 			return true;
 		}
 	}
-	
+
 	return false;
 }
 
@@ -755,7 +772,7 @@ bool KKeyChooser::promptForReassign( const QKeySequence& cut, const QString& sAc
 		cut.toString(), sAction.trimmed());
 	}
 
-	return KMessageBox::warningContinueCancel( parent, s, sTitle, i18n("Reassign") ) == KMessageBox::Continue;
+	return KMessageBox::warningContinueCancel( parent, s, sTitle, KGuiItem(i18n("Reassign")) ) == KMessageBox::Continue;
 }
 
 //---------------------------------------------------
@@ -783,7 +800,7 @@ QVariant KKeyChooserItem::data(int column, int role) const
 		case 0:
 			switch(role) {
 				case Qt::DisplayRole:
-					return m_action->text().remove("&");
+					return m_action->text().remove('&');
 				case Qt::DecorationRole:
 					return m_action->icon();
 				case Qt::WhatsThisRole:
@@ -793,56 +810,47 @@ QVariant KKeyChooserItem::data(int column, int role) const
 				case Qt::StatusTipRole:
 					return m_action->statusTip();
 			}
+			break;
 
 		case 1:
+                    if (role==Qt::DisplayRole)
+                        return shortcut().primary();
+                    break;
 		case 2:
-			switch (role) {
-				case Qt::DisplayRole:
-					if (column - 1 < m_newLocalShortcut.count())
-						return m_newLocalShortcut.seq(column - 1).toString();
-					break;
-				case Qt::FontRole:
-					if (checkChanged(m_newLocalShortcut, m_action->shortcut(), column - 1)) {
-						QFont parentFont = treeWidget()->font();
-						parentFont.setBold(true);
-						return parentFont;
-					}
-					break;
-				case Qt::TextColorRole:
-					if (checkChanged(m_newLocalShortcut, m_action->shortcut(), column - 1))
-						return QColor(Qt::blue);
-				default:
-					break;
-			}
-			break;
+                    if (role==Qt::DisplayRole)
+                        return shortcut().alternate();
+                    break;
 
 		case 3:
 			switch (role) {
 				case Qt::DisplayRole:
-					if (m_newGlobalShortcut.count())
-						return m_newGlobalShortcut.seq(0).toString();
+					if (!m_newGlobalShortcut.isEmpty())
+						return m_newGlobalShortcut.toString();
+					//H4X
+					/*else
+						return "glob:none";*/
+					/*else
+						return globalShortcut().seq(column - 1).toString();*/
 					break;
-				case Qt::FontRole:
+/*				case Qt::FontRole:
 					if (checkChanged(m_newGlobalShortcut, m_action->globalShortcut(), 0)) {
 						QFont parentFont = treeWidget()->font();
 						parentFont.setBold(true);
 						return parentFont;
 					}
 					break;
-				case Qt::TextColorRole:
+				case Qt::ForegroundRole:
 					if (checkChanged(m_newGlobalShortcut, m_action->globalShortcut(), 0))
 						return QColor(Qt::blue);
 					break;
-				case Qt::BackgroundColorRole:
+				case Qt::BackgroundRole:
 					if (!m_action->globalShortcutAllowed())
-						return QColor(Qt::gray);
-					break;
-				default:
+					return QColor(Qt::gray);*/
 					break;
 			}
 			break;
 	}
-	
+
 	return QVariant();
 }
 
@@ -861,9 +869,9 @@ void KKeyChooserItem::commitChanges()
 {
 	if( m_modified ) {
 		if ( m_newLocalShortcut != m_action->shortcut() )
-			m_action->setShortcut( m_newLocalShortcut, KAction::CustomShortcut );
+			m_action->setShortcut( m_newLocalShortcut, KAction::ActiveShortcut );
 		if ( m_newGlobalShortcut != m_action->globalShortcut() )
-			m_action->setGlobalShortcut( m_newGlobalShortcut, KAction::CustomShortcut );
+			m_action->setGlobalShortcut( m_newGlobalShortcut, KAction::ActiveShortcut );
 	}
 }
 
@@ -931,7 +939,7 @@ int KKeyDialog::configure( KActionCollection* coll, KKeyChooser::LetterShortcuts
 }
 
 
-const KShortcut& KKeyChooserItem::shortcut( ) const
+KShortcut KKeyChooserItem::shortcut( ) const
 {
 	return m_newLocalShortcut;
 }
@@ -941,9 +949,9 @@ bool KKeyChooserItem::isConfigurable( ) const
 	return m_action->isShortcutConfigurable();
 }
 
-const KShortcut& KKeyChooserItem::defaultShortcut( ) const
+KShortcut KKeyChooserItem::defaultShortcut( ) const
 {
-	return m_action->defaultShortcut();
+	return m_action->shortcut( KAction::DefaultShortcut );
 }
 
 void KKeyChooser::resizeColumns( )
@@ -959,24 +967,22 @@ void KKeyChooser::showEvent( QShowEvent * event )
 	QTimer::singleShot(0, this, SLOT(resizeColumns()));
 }
 
-const KShortcut & KKeyChooserItem::globalShortcut( ) const
+KShortcut KKeyChooserItem::globalShortcut( ) const
 {
 	return m_newGlobalShortcut;
 }
 
-const KShortcut & KKeyChooserItem::defaultGlobalShortcut( ) const
+KShortcut KKeyChooserItem::defaultGlobalShortcut( ) const
 {
-	return m_action->defaultGlobalShortcut();
+	return m_action->globalShortcut( KAction::DefaultShortcut );
 }
 
 void KKeyChooserItem::checkModified( )
 {
 	m_modified = (m_newLocalShortcut != m_action->shortcut()) || (m_newGlobalShortcut != m_action->globalShortcut());
-	// Force redraw
-	if (text(0) == "a")
-		setText(0, "b");
-	else
-		setText(0, "a");
+
+	// why do I still have to use viewport() (with Qt 4.2.0)?
+	treeWidget()->viewport()->update();
 }
 
 void KKeyChooserItem::setGlobalShortcut( const KShortcut & cut )
@@ -985,25 +991,6 @@ void KKeyChooserItem::setGlobalShortcut( const KShortcut & cut )
 	checkModified();
 }
 
-bool KKeyChooserItem::checkChanged( const KShortcut & cut1, const KShortcut & cut2, int sequence ) const
-{
-	if (cut1.count() == cut2.count())
-		if (sequence < cut1.count())
-			return cut1.seq(sequence) != cut2.seq(sequence);
-		else
-			return false;
-	else if (cut1.count() < cut2.count())
-		if (sequence < cut1.count())
-			return cut1.seq(sequence) != cut2.seq(sequence);
-		else
-			return false;
-	else
-		if (sequence < cut2.count())
-			return cut1.seq(sequence) != cut2.seq(sequence);
-		else
-			return true;
-}
-
 KAction * KKeyChooserItem::action( ) const
 {
 	return m_action;
