@@ -42,7 +42,7 @@ class KFindDialog::KFindDialogPrivate
 public:
     KFindDialogPrivate() : m_regexpDialog(0),
         m_regexpDialogQueryDone(false),
-        m_enabled(KFind::WholeWordsOnly | KFind::FromCursor |  KFind::SelectedText | Qt::CaseSensitive | KFind::FindBackwards | KFind::RegularExpression), m_initialShowDone(false) {}
+        m_enabled(KFind::WholeWordsOnly | KFind::FromCursor |  KFind::SelectedText | KFind::CaseSensitive | KFind::FindBackwards | KFind::RegularExpression), m_initialShowDone(false) {}
     QDialog* m_regexpDialog;
     bool m_regexpDialogQueryDone;
     long m_enabled; // uses Options to define which search options are enabled
@@ -65,7 +65,7 @@ KFindDialog::KFindDialog(QWidget *parent, const char *name, long options, const
 
     init(false, findStrings, hasSelection);
     setOptions(options);
-    setButtonGuiItem( KDialog::Cancel, KStdGuiItem::close() );
+    setButtonGuiItem( KDialog::Cancel, KStandardGuiItem::close() );
 }
 
 KFindDialog::KFindDialog(bool modal, QWidget *parent, const char *name, long options, const QStringList &findStrings, bool hasSelection) :
@@ -82,7 +82,7 @@ KFindDialog::KFindDialog(bool modal, QWidget *parent, const char *name, long opt
 
     init(false, findStrings, hasSelection);
     setOptions(options);
-    setButtonGuiItem( KDialog::Cancel, KStdGuiItem::close() );
+    setButtonGuiItem( KDialog::Cancel, KStandardGuiItem::close() );
 }
 
 KFindDialog::KFindDialog(QWidget *parent, const char *name, bool /*forReplace*/) :
@@ -97,7 +97,7 @@ KFindDialog::KFindDialog(QWidget *parent, const char *name, bool /*forReplace*/)
     setButtons( Ok | Cancel );
     setDefaultButton( Ok );
 
-    setButtonGuiItem( KDialog::Cancel,  KStdGuiItem::close() );
+    setButtonGuiItem( KDialog::Cancel,  KStandardGuiItem::close() );
 }
 
 KFindDialog::~KFindDialog()
@@ -110,7 +110,7 @@ QWidget *KFindDialog::findExtension()
     if (!m_findExtension)
     {
       m_findExtension = new QWidget(m_findGrp);
-      m_findLayout->addWidget(m_findExtension, 3, 3, 0, 1);
+      m_findLayout->addWidget(m_findExtension, 3, 0, 1, 2);
     }
 
     return m_findExtension;
@@ -215,6 +215,15 @@ void KFindDialog::init(bool forReplace, const QStringList &findStrings, bool has
 
     connect(m_find, SIGNAL(editTextChanged(const QString &)),this, SLOT(textSearchChanged(const QString &)));
 
+    connect(m_regExp, SIGNAL(toggled(bool)), this, SIGNAL(optionsChanged()));
+    connect(m_backRef, SIGNAL(toggled(bool)), this, SIGNAL(optionsChanged()));
+    connect(m_caseSensitive, SIGNAL(toggled(bool)), this, SIGNAL(optionsChanged()));
+    connect(m_wholeWordsOnly, SIGNAL(toggled(bool)), this, SIGNAL(optionsChanged()));
+    connect(m_fromCursor, SIGNAL(toggled(bool)), this, SIGNAL(optionsChanged()));
+    connect(m_findBackwards, SIGNAL(toggled(bool)), this, SIGNAL(optionsChanged()));
+    connect(m_selectedText, SIGNAL(toggled(bool)), this, SIGNAL(optionsChanged()));
+    connect(m_promptOnReplace, SIGNAL(toggled(bool)), this, SIGNAL(optionsChanged()));
+
     // tab order
     setTabOrder(m_find, m_regExp);
     setTabOrder(m_regExp, m_regExpItem);
@@ -289,6 +298,8 @@ void KFindDialog::init(bool forReplace, const QStringList &findStrings, bool has
             "Search backwards.") );
     m_promptOnReplace->setWhatsThis(i18n(
             "Ask before replacing each match found.") );
+
+    connect(this,SIGNAL(okClicked()),this,SLOT(slotOk()));
 }
 
 void KFindDialog::textSearchChanged( const QString & text)
@@ -319,7 +330,7 @@ long KFindDialog::options() const
     long options = 0;
 
     if (m_caseSensitive->isChecked())
-        options |= Qt::CaseSensitive;
+        options |= KFind::CaseSensitive;
     if (m_wholeWordsOnly->isChecked())
         options |= KFind::WholeWordsOnly;
     if (m_fromCursor->isChecked())
@@ -398,10 +409,10 @@ void KFindDialog::setSupportsBackwardsFind( bool supports )
 void KFindDialog::setSupportsCaseSensitiveFind( bool supports )
 {
     // ########## This should hide the checkbox instead
-    if (supports) d->m_enabled |= Qt::CaseSensitive;
-    else d->m_enabled &= ~Qt::CaseSensitive;
+    if (supports) d->m_enabled |= KFind::CaseSensitive;
+    else d->m_enabled &= ~KFind::CaseSensitive;
     m_caseSensitive->setEnabled( supports );
-    m_caseSensitive->setChecked( supports && (options() & Qt::CaseSensitive) );
+    m_caseSensitive->setChecked( supports && (options() & KFind::CaseSensitive) );
 }
 
 void KFindDialog::setSupportsWholeWordsFind( bool supports )
@@ -424,7 +435,7 @@ void KFindDialog::setSupportsRegularExpressionFind( bool supports )
 
 void KFindDialog::setOptions(long options)
 {
-    m_caseSensitive->setChecked((d->m_enabled & Qt::CaseSensitive) && (options & Qt::CaseSensitive));
+    m_caseSensitive->setChecked((d->m_enabled & KFind::CaseSensitive) && (options & KFind::CaseSensitive));
     m_wholeWordsOnly->setChecked((d->m_enabled & KFind::WholeWordsOnly) && (options & KFind::WholeWordsOnly));
     m_fromCursor->setChecked((d->m_enabled & KFind::FromCursor) && (options & KFind::FromCursor));
     m_findBackwards->setChecked((d->m_enabled & KFind::FindBackwards) && (options & KFind::FindBackwards));
@@ -595,7 +606,6 @@ void KFindDialog::slotOk()
         }
     }
     m_find->addToHistory(pattern());
-    emit okClicked();
     if ( windowModality() != Qt::NonModal )
         accept();
 }
