@@ -1,15 +1,15 @@
 #ifndef TST_KLIVEUI_H
 #define TST_KLIVEUI_H
 
-#include <QtTest/QtTest>
-#include <QMainWindow>
+#include <qtest_kde.h>
 #include <QAction>
 #include <QMenu>
 #include <QMenuBar>
 
+#include <kmainwindow.h>
 #include "../kliveui.h"
 
-class tst_GuiEditor : public QObject
+class tst_KLiveUi : public QObject
 {
     Q_OBJECT
 public slots:
@@ -20,9 +20,12 @@ private slots:
 //    void recursiveActionGroups();
     void menuDeletion();
     void changeExistingMenu();
+    void implicitBeginMenuBar();
+    void subComponents();
+    void deleteActions();
 
 private:
-    QMainWindow *mw;
+    KMainWindow *mw;
     QAction *testAction1;
     QAction *testAction2;
     QAction *testAction3;
