@@ -13,9 +13,9 @@ int main( int argc, char **argv )
 
     KActionCollection* coll = new KActionCollection( &app );
 
-    KAction* action = new KAction(coll, "test");
+    KAction* action = static_cast<KAction *>(coll->addAction("test"));
     action->setGlobalShortcutAllowed(true);
-    action->setGlobalShortcut(Qt::META + Qt::Key_F5);
+    action->setGlobalShortcut(KShortcut(Qt::META + Qt::Key_F5));
 
     QObject::connect(action, SIGNAL(triggered(bool)), &app, SLOT(quit()));
 
