@@ -5,7 +5,6 @@
 #include <kapplication.h>
 #include <kcmdlineargs.h>
 
-#include "kprogressbar.h"
 #include "kprogressdialog.h"
 
 class MyWidget : public QWidget {
@@ -37,7 +36,7 @@ private:
 
 void MyWidget::timerEvent(QTimerEvent *)
 {
-    Progress->progressBar()->advance(1);
+    Progress->progressBar()->setValue(Progress->progressBar()->value()+1);
     Cancelled->setCheckState(Progress->wasCancelled() ? Qt::Checked : Qt::Unchecked);
 }
 
