@@ -11,7 +11,7 @@ public:
     Client() {}
 
     void setXMLFile( const QString &f, bool merge = true ) { KXMLGUIClient::setXMLFile( f, merge ); }
-    void setInstance( KInstance *inst ) { KXMLGUIClient::setInstance( inst ); }
+    void setComponentData(const KComponentData &inst) { KXMLGUIClient::setComponentData(inst); }
 
 public Q_SLOTS:
     void slotSec();
