@@ -468,18 +468,21 @@ bool keyQtToCodeX( int keyQt, int& keyCode )
 	//  E.g., XK_Plus requires SHIFT on the en layout.
 	uint modExtra = getModsRequired(sym);
 	// Get the X modifier equivalent.
-	if( !sym || !keyQtToModX( keyQt & Qt::KeyboardModifierMask | modExtra, mod ) ) {
+	if( !sym || !keyQtToModX( (keyQt & Qt::KeyboardModifierMask) | modExtra, mod ) ) {
 		keyCode = 0;
 		return false;
 	}
 
 #ifdef Q_WS_X11
-	// FIXME: Accomadate non-standard layouts
 	// XKeysymToKeycode returns the wrong keycode for XK_Print and XK_Break.
 	// Specifically, it returns the code for SysReq instead of Print
-	if( sym == XK_Print && !(mod & Mod1Mask) )
+	// Only do this for the default Xorg layout, other keycode mappings
+	// (e.g. evdev) don't need or want it.
+	if( sym == XK_Print && !(mod & Mod1Mask) &&
+			XKeycodeToKeysym( QX11Info::display(), 111, 0 ) == XK_Print )
 		keyCode = 111; // code for Print
-	else if( sym == XK_Break || (sym == XK_Pause && (mod & ControlMask)) )
+	else if( sym == XK_Break || ((sym == XK_Pause && (mod & ControlMask)) &&
+			XKeycodeToKeysym( QX11Info::display(), 114, 0 ) == XK_Pause) )
 		keyCode = 114;
 	else
 		keyCode = XKeysymToKeycode( QX11Info::display(), sym );
