@@ -22,7 +22,7 @@
 #ifndef KCOMBOBOX_H
 #define KCOMBOBOX_H
 
-#include <QComboBox>
+#include <QtGui/QComboBox>
 
 #include <kcompletion.h>
 
@@ -68,7 +68,7 @@ class KUrl;
  * and consider using QGuardedPtr<KCompletion>.
  *
  * The default key-bindings for completion and rotation is determined from the
- * global settings in KStdAccel. These values, however, can be overridden
+ * global settings in KStandardShortcut. These values, however, can be overridden
  * locally by invoking KCompletionBase::setKeyBinding(). The values can
  * easily be reverted back to the default setting, by simply calling
  * useGlobalSettings(). An alternate method would be to default individual
@@ -171,18 +171,18 @@ public:
     */
     virtual ~KComboBox();
 
-	/**
+    /**
      * Deprecated to reflect Qt api changes
-	 * @deprecated
-	 */
-	KDE_DEPRECATED void insertURL( const KUrl& url, int index = -1 )
-		{ insertUrl( index < 0 ? count() : index, url ); }
-	KDE_DEPRECATED void insertURL( const QPixmap& pixmap, const KUrl& url, int index = -1 )
-		{ insertUrl( index < 0 ? count() : index, QIcon(pixmap), url ); }
-	KDE_DEPRECATED void changeURL( const KUrl& url, int index )
-		{ changeUrl( index, url ); }
-  KDE_DEPRECATED void changeURL( const QPixmap& pixmap, const KUrl& url, int index )
-		{ changeUrl( index, QIcon(pixmap), url ); }
+     * @deprecated
+     */
+    KDE_DEPRECATED void insertURL( const KUrl& url, int index = -1 )
+    { insertUrl( index < 0 ? count() : index, url ); }
+    KDE_DEPRECATED void insertURL( const QPixmap& pixmap, const KUrl& url, int index = -1 )
+    { insertUrl( index < 0 ? count() : index, QIcon(pixmap), url ); }
+    KDE_DEPRECATED void changeURL( const KUrl& url, int index )
+    { changeUrl( index, url ); }
+    KDE_DEPRECATED void changeURL( const QPixmap& pixmap, const KUrl& url, int index )
+    { changeUrl( index, QIcon(pixmap), url ); }
 
 	/**
      * Sets @p url into the edit field of the combobox. It uses
@@ -190,7 +190,7 @@ public:
      * displaying.
      */
     void setEditUrl( const KUrl& url );
-   
+
 	/**
      * Appends @p url to the combobox.
      * KUrl::prettyUrl() is used so that the url is properly decoded
@@ -647,7 +647,7 @@ public:
      * method KPixmapProvider::pixmapFor(). This method is called whenever
      * an item is added to the KHistoryComboBox. Implement it to return your
      * own custom pixmaps, or use the KUrlPixmapProvider from libkio,
-     * which uses KMimeType::pixmapForURL to resolve icons.
+     * which uses KMimeType::pixmapForUrl to resolve icons.
      *
      * Set @p prov to 0L if you want to disable pixmaps. Default no pixmaps.
      *
@@ -660,13 +660,13 @@ public:
      * @see setPixmapProvider
      * @see KPixmapProvider
      */
-    KPixmapProvider * pixmapProvider() const { return myPixProvider; }
+    KPixmapProvider * pixmapProvider() const;
 
     /**
      * Resets the current position of the up/down history. Call this
      * when you manually call setCurrentItem() or clearEdit().
      */
-    void reset() { slotReset(); }
+    void reset(); 
 
 public Q_SLOTS:
     /**
@@ -754,26 +754,12 @@ private:
     void rotateUp();
     void rotateDown();
 
-    /**
-     * The current position (index) in the combobox, used for Up and Down
-     */
-    int myIterateIndex;
-
-    /**
-     * The text typed before Up or Down was pressed.
-     */
-    QString myText;
-
-    /**
-     * Indicates that the user at least once rotated Up through the entire list
-     * Needed to allow going back after rotation.
-     */
-    bool myRotated;
-    KPixmapProvider *myPixProvider;
-
 private:
     class KHistoryComboPrivate;
+    friend class KHistoryComboPrivate;
     KHistoryComboPrivate* const d;
+    
+    Q_DISABLE_COPY(KHistoryCombo)
 };
 
 
