@@ -25,17 +25,13 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <qclipboard.h>
-#include <qtimer.h>
-#include <qevent.h>
-#include <qstyle.h>
+#include "klineedit.h"
 
-#include <QPainter>
 #include <kconfig.h>
 #include <qtooltip.h>
 #include <kcursor.h>
 #include <klocale.h>
-#include <kstdaccel.h>
+#include <kstandardshortcut.h>
 #include <kmenu.h>
 #include <kdebug.h>
 #include <kcompletionbox.h>
@@ -43,8 +39,17 @@
 #include <kiconloader.h>
 #include <kapplication.h>
 #include <kauthorized.h>
+#include <kicon.h>
+
+#include <QtCore/QTimer>
+#include <QtGui/QClipboard>
+#include <QtGui/QKeyEvent>
+#include <QtGui/QLabel>
+#include <QtGui/QPainter>
+#include <QtGui/QStyle>
+#include <QtGui/QStyleOption>
+#include <kconfiggroup.h>
 
-#include "klineedit.h"
 #include "klineedit.moc"
 
 
@@ -72,6 +77,9 @@ public:
             initialized = true;
         }
 
+        clearButton = 0;
+        clickButtonState = -1;
+        clickInClear = false;
     }
 
     ~KLineEditPrivate()
@@ -102,7 +110,18 @@ public:
     bool enableClickMsg:1;
     bool drawClickMsg:1;
 
+    bool bEnableMenu :1; 
+    bool possibleTripleClick :1;  // set in mousePressEvent, deleted in tripleClickTimeout
+
+    bool clickInClear:1;
+    int  clickButtonState;
+    QLabel* clearButton;
+
     KCompletionBox *completionBox;
+
+    QAction *noCompletionAction, *shellCompletionAction, *autoCompletionAction, *popupCompletionAction, *shortAutoCompletionAction, *popupAutoCompletionAction, *defaultAction;
+
+    QMap<KGlobalSettings::Completion, bool> disableCompletionMap;
 };
 
 bool KLineEdit::KLineEditPrivate::backspacePerformsCompletion = false;
@@ -129,7 +148,7 @@ KLineEdit::~KLineEdit ()
 
 void KLineEdit::init()
 {
-    possibleTripleClick = false;
+    d->possibleTripleClick = false;
     d->bgRole = backgroundRole();
 
     // Enable the context menu by default.
@@ -155,6 +174,86 @@ QString KLineEdit::clickMessage() const
     return d->clickMessage;
 }
 
+void KLineEdit::setClearButtonShown(bool show)
+{
+    if (show) {
+        if (d->clearButton) {
+            return;
+        }
+
+        d->clearButton = new QLabel( this );
+        d->clearButton->setAlignment( Qt::AlignLeft | Qt::AlignVCenter );
+        d->clearButton->setCursor( Qt::ArrowCursor );
+        d->clearButton->setToolTip( i18n( "Clear text" ) );
+
+        updateClearButton();
+        updateClearButtonIcon(text());
+        d->clearButton->show();
+        connect(this, SIGNAL(textChanged(QString)), this, SLOT(updateClearButtonIcon(QString)));
+    } else {
+        disconnect(this, SIGNAL(textChanged(QString)), this, SLOT(updateClearButtonIcon(QString)));
+        delete d->clearButton;
+        d->clearButton = 0;
+        d->clickInClear = false;
+        setStyleSheet(QString());
+    }
+}
+
+bool KLineEdit::isClearButtonShown() const
+{
+    return d->clearButton != 0;
+}
+
+void KLineEdit::updateClearButtonIcon(const QString& text)
+{
+    if (!d->clearButton || isReadOnly()) {
+        return;
+    }
+
+    if (text.length() > 0) {
+        if (d->clickButtonState == K3Icon::DefaultState) {
+            return;
+        }
+
+        d->clickButtonState = K3Icon::DefaultState;
+    } else {
+        if (d->clickButtonState == K3Icon::DisabledState) {
+            return;
+        }
+
+        d->clickButtonState = K3Icon::DisabledState;
+    }
+
+    if ( qApp->isLeftToRight() ) {
+        d->clearButton->setPixmap( SmallIcon( "clear_left.png", 0, d->clickButtonState ) );
+    } else {
+        d->clearButton->setPixmap( SmallIcon("locationbar_erase.png", 0, d->clickButtonState ) );
+    }
+}
+
+void KLineEdit::updateClearButton()
+{
+    if (!d->clearButton || isReadOnly()) {
+        return;
+    }
+
+    QSize geom = size();
+    int frameWidth = style()->pixelMetric(QStyle::PM_DefaultFrameWidth);
+    int buttonWidth = d->clearButton->sizeHint().width();
+    QSize newButtonSize(buttonWidth, geom.height());
+
+    if (newButtonSize != d->clearButton->size()) {
+        d->clearButton->resize(newButtonSize);
+        setStyleSheet(QString("QLineEdit { padding-right: %1; }").arg(buttonWidth));
+    }
+
+    if (qApp->isLeftToRight()) {
+        d->clearButton->move(geom.width() - frameWidth - buttonWidth - 1, 0);
+    } else {
+        d->clearButton->move(frameWidth + 1, 0);
+    }
+}
+
 void KLineEdit::setCompletionMode( KGlobalSettings::Completion mode )
 {
     KGlobalSettings::Completion oldMode = completionMode();
@@ -182,6 +281,11 @@ void KLineEdit::setCompletionMode( KGlobalSettings::Completion mode )
     KCompletionBase::setCompletionMode( mode );
 }
 
+void KLineEdit::setCompletionModeDisabled( KGlobalSettings::Completion mode, bool disable )
+{
+  d->disableCompletionMap[ mode ] = disable;
+}
+
 void KLineEdit::setCompletedText( const QString& t, bool marked )
 {
     if ( !d->autoSuggest )
@@ -288,6 +392,11 @@ void KLineEdit::setReadOnly(bool readOnly)
             d->squeezedText = text();
             setSqueezedText();
         }
+
+        if (d->clearButton) {
+            d->clearButton->hide();
+            setStyleSheet(QString());
+        }
     }
     else
     {
@@ -297,6 +406,12 @@ void KLineEdit::setReadOnly(bool readOnly)
            d->squeezedText.clear();
         }
         setBackgroundRole(d->bgRole);
+
+        if (d->clearButton) {
+            int buttonWidth = d->clearButton->sizeHint().width();
+            setStyleSheet(QString("QLineEdit { padding-right: %1; }").arg(buttonWidth));
+            d->clearButton->show();
+        }
     }
 }
 
@@ -397,14 +512,19 @@ void KLineEdit::setSqueezedText()
     {
       QLineEdit::setText(fullText);
 
-          this->setToolTip("" );
-          QToolTip::showText(pos(), QString()); // hide
-       }
+      this->setToolTip( "" );
+      QToolTip::showText(pos(), QString()); // hide
+    }
 
-       setCursorPosition(0);
+    setCursorPosition(0);
 }
 
 void KLineEdit::copy() const
+{
+    copy(true);
+}
+
+void KLineEdit::copy(bool clipboard) const
 {
    if (!d->squeezedText.isEmpty() && d->squeezedStart)
    {
@@ -425,9 +545,9 @@ void KLineEdit::copy() const
       QString t = d->squeezedText;
       t = t.mid(start, end - start);
       disconnect( QApplication::clipboard(), SIGNAL(selectionChanged()), this, 0);
-      QApplication::clipboard()->setText( t );
+      QApplication::clipboard()->setText( t, clipboard ? QClipboard::Clipboard : QClipboard::Selection );
       connect( QApplication::clipboard(), SIGNAL(selectionChanged()), this,
-               SLOT(clipboardChanged()) );
+               SLOT(_q_clipboardChanged()) );
       return;
    }
 
@@ -439,6 +559,7 @@ void KLineEdit::resizeEvent( QResizeEvent * ev )
     if (!d->squeezedText.isEmpty())
         setSqueezedText();
 
+    updateClearButton();
     QLineEdit::resizeEvent(ev);
 }
 
@@ -446,17 +567,17 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
 {
     int key = e->key() | e->modifiers();
 
-    if ( KStdAccel::copy().contains( key ) )
+    if ( KStandardShortcut::copy().contains( key ) )
     {
         copy();
         return;
     }
-    else if ( KStdAccel::paste().contains( key ) )
+    else if ( KStandardShortcut::paste().contains( key ) )
     {
         paste();
         return;
     }
-    else if ( KStdAccel::pasteSelection().contains( key ) )
+    else if ( KStandardShortcut::pasteSelection().contains( key ) )
     {
         QString text = QApplication::clipboard()->text( QClipboard::Selection);
         insert( text );
@@ -464,22 +585,22 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
         return;
     }
 
-    else if ( KStdAccel::cut().contains( key ) )
+    else if ( KStandardShortcut::cut().contains( key ) )
     {
         cut();
         return;
     }
-    else if ( KStdAccel::undo().contains( key ) )
+    else if ( KStandardShortcut::undo().contains( key ) )
     {
         undo();
         return;
     }
-    else if ( KStdAccel::redo().contains( key ) )
+    else if ( KStandardShortcut::redo().contains( key ) )
     {
         redo();
         return;
     }
-    else if ( KStdAccel::deleteWordBack().contains( key ) )
+    else if ( KStandardShortcut::deleteWordBack().contains( key ) )
     {
         cursorWordBackward(true);
         if ( hasSelectedText() )
@@ -488,7 +609,7 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
         e->accept();
         return;
     }
-    else if ( KStdAccel::deleteWordForward().contains( key ) )
+    else if ( KStandardShortcut::deleteWordForward().contains( key ) )
     {
         // Workaround for QT bug where
         cursorWordForward(true);
@@ -498,25 +619,25 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
         e->accept();
         return;
     }
-    else if ( KStdAccel::backwardWord().contains( key ) )
+    else if ( KStandardShortcut::backwardWord().contains( key ) )
     {
       cursorWordBackward(false);
       e->accept();
       return;
     }
-    else if ( KStdAccel::forwardWord().contains( key ) )
+    else if ( KStandardShortcut::forwardWord().contains( key ) )
     {
       cursorWordForward(false);
       e->accept();
       return;
     }
-    else if ( KStdAccel::beginningOfLine().contains( key ) )
+    else if ( KStandardShortcut::beginningOfLine().contains( key ) )
     {
       home(false);
       e->accept();
       return;
     }
-    else if ( KStdAccel::endOfLine().contains( key ) )
+    else if ( KStandardShortcut::endOfLine().contains( key ) )
     {
       end(false);
       e->accept();
@@ -728,8 +849,8 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
         {
             // Handles completion.
             KShortcut cut;
-            if ( keys[TextCompletion].isNull() )
-                cut = KStdAccel::shortcut(KStdAccel::TextCompletion);
+            if ( keys[TextCompletion].isEmpty() )
+                cut = KStandardShortcut::shortcut(KStandardShortcut::TextCompletion);
             else
                 cut = keys[TextCompletion];
 
@@ -757,8 +878,8 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
         {
             // Handles previous match
             KShortcut cut;
-            if ( keys[PrevCompletionMatch].isNull() )
-                cut = KStdAccel::shortcut(KStdAccel::PrevCompletion);
+            if ( keys[PrevCompletionMatch].isEmpty() )
+                cut = KStandardShortcut::shortcut(KStandardShortcut::PrevCompletion);
             else
                 cut = keys[PrevCompletionMatch];
 
@@ -772,8 +893,8 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
             }
 
             // Handles next match
-            if ( keys[NextCompletionMatch].isNull() )
-                cut = KStdAccel::shortcut(KStdAccel::NextCompletion);
+            if ( keys[NextCompletionMatch].isEmpty() )
+                cut = KStandardShortcut::shortcut(KStandardShortcut::NextCompletion);
             else
                 cut = keys[NextCompletionMatch];
 
@@ -791,8 +912,8 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
         if ( compObj() )
         {
             KShortcut cut;
-            if ( keys[SubstringCompletion].isNull() )
-                cut = KStdAccel::shortcut(KStdAccel::SubstringCompletion);
+            if ( keys[SubstringCompletion].isEmpty() )
+                cut = KStandardShortcut::shortcut(KStandardShortcut::SubstringCompletion);
             else
                 cut = keys[SubstringCompletion];
 
@@ -823,7 +944,7 @@ void KLineEdit::mouseDoubleClickEvent( QMouseEvent* e )
 {
     if ( e->button() == Qt::LeftButton  )
     {
-        possibleTripleClick=true;
+        d->possibleTripleClick=true;
         QTimer::singleShot( QApplication::doubleClickInterval(),this,
                             SLOT(tripleClickTimeout()) );
     }
@@ -832,82 +953,116 @@ void KLineEdit::mouseDoubleClickEvent( QMouseEvent* e )
 
 void KLineEdit::mousePressEvent( QMouseEvent* e )
 {
-    if ( possibleTripleClick && e->button() == Qt::LeftButton )
-    {
+    if  ( (e->button() == Qt::LeftButton ||
+           e->button() == Qt::MidButton ) &&
+          d->clearButton ) {
+        d->clickInClear = d->clearButton->underMouse();
+
+        if ( d->clickInClear ) {
+            d->possibleTripleClick = false;
+        }
+    }
+
+    if ( e->button() == Qt::LeftButton && d->possibleTripleClick ) {
         selectAll();
         e->accept();
         return;
     }
+
     QLineEdit::mousePressEvent( e );
 }
 
+void KLineEdit::mouseReleaseEvent( QMouseEvent* e )
+{
+    if ( d->clickInClear ) {
+        if ( d->clearButton->underMouse() ) {
+            QString newText;
+            if ( e->button() == Qt::MidButton ) {
+                newText = QApplication::clipboard()->text( QClipboard::Selection );
+                setText( newText );
+            } else {
+                clear();
+            }
+            emit textChanged( newText );
+        }
+
+        d->clickInClear = false;
+        e->accept();
+        return;
+    }
+
+    QLineEdit::mouseReleaseEvent( e );
+
+   // Call our own copy() to fix copying of squeezed text
+   if (QApplication::clipboard()->supportsSelection() ) {
+       if ( e->button() == Qt::LeftButton ) {
+           copy( FALSE );
+       }
+   }
+}
+
 void KLineEdit::tripleClickTimeout()
 {
-    possibleTripleClick=false;
+    d->possibleTripleClick=false;
 }
 
-void KLineEdit::contextMenuEvent(QContextMenuEvent *e)
+QMenu* KLineEdit::createStandardContextMenu()
 {
-    if (!m_bEnableMenu)
-        return;
-    QMenu *popup = createStandardContextMenu();
-    QList<QAction *> lstAction = popup->actions ();
-
-    if ( !lstAction.isEmpty() )
-    {
-        enum { UndoAct, RedoAct, CutAct, CopyAct, PasteAct, ClearAct, SelectAllAct, NCountActs };
-        lstAction[UndoAct]->setIcon( SmallIconSet("undo") );
-        lstAction[RedoAct]->setIcon( SmallIconSet("redo") );
-        lstAction[CutAct]->setIcon( SmallIconSet("editcut") );
-        lstAction[CopyAct]->setIcon( SmallIconSet("editcopy") );
-        lstAction[PasteAct]->setIcon( SmallIconSet("editpaste") );
-        lstAction[ClearAct]->setIcon( SmallIconSet("editclear") );
-    }
-
+    QMenu *popup = QLineEdit::createStandardContextMenu();
+    KIconTheme::assignIconsToContextMenu( KIconTheme::TextEditor, popup->actions () );
 
     // If a completion object is present and the input
     // widget is not read-only, show the Text Completion
     // menu item.
     if ( compObj() && !isReadOnly() && KAuthorized::authorize("lineedit_text_completion") )
     {
-        QMenu *subMenu = popup->addMenu( SmallIconSet("completion"), i18n("Text Completion") );
+        QMenu *subMenu = popup->addMenu( KIcon("completion"), i18n("Text Completion") );
         connect( subMenu, SIGNAL( triggered ( QAction* ) ),
                  this, SLOT( completionMenuActivated( QAction* ) ) );
 
         popup->addSeparator();
 
         QActionGroup* ag = new QActionGroup( this );
-        noCompletionAction = ag->addAction( i18n("None"));
-        shellCompletionAction = ag->addAction( i18n("Manual") );
-        autoCompletionAction = ag->addAction( i18n("Automatic") );
-        popupCompletionAction = ag->addAction( i18n("Dropdown List") );
-        shortAutoCompletionAction = ag->addAction( i18n("Short Automatic") );
-        popupAutoCompletionAction = ag->addAction( i18n("Dropdown List && Automatic"));
+        d->noCompletionAction = ag->addAction( i18n("None"));
+        d->shellCompletionAction = ag->addAction( i18n("Manual") );
+        d->autoCompletionAction = ag->addAction( i18n("Automatic") );
+        d->popupCompletionAction = ag->addAction( i18n("Dropdown List") );
+        d->shortAutoCompletionAction = ag->addAction( i18n("Short Automatic") );
+        d->popupAutoCompletionAction = ag->addAction( i18n("Dropdown List && Automatic"));
         subMenu->addActions( ag->actions() );
 
-        //subMenu->setAccel( KStdAccel::completion(), ShellCompletion );
+        //subMenu->setAccel( KStandardShortcut::completion(), ShellCompletion );
 
-        shellCompletionAction->setCheckable(true);
-        noCompletionAction->setCheckable(true);
-        popupCompletionAction->setCheckable(true);
-        autoCompletionAction->setCheckable(true);
-        shortAutoCompletionAction->setCheckable(true);
-        popupAutoCompletionAction->setCheckable(true);
+        d->shellCompletionAction->setCheckable( !d->disableCompletionMap[ KGlobalSettings::CompletionShell ] );
+        d->noCompletionAction->setCheckable( !d->disableCompletionMap[ KGlobalSettings::CompletionNone ] );
+        d->popupCompletionAction->setCheckable( !d->disableCompletionMap[ KGlobalSettings::CompletionPopup ] );
+        d->autoCompletionAction->setCheckable( !d->disableCompletionMap[ KGlobalSettings::CompletionAuto ] );
+        d->shortAutoCompletionAction->setCheckable( !d->disableCompletionMap[ KGlobalSettings::CompletionMan ] );
+        d->popupAutoCompletionAction->setCheckable( !d->disableCompletionMap[ KGlobalSettings::CompletionPopupAuto ] );
 
         KGlobalSettings::Completion mode = completionMode();
-        noCompletionAction->setChecked( mode == KGlobalSettings::CompletionNone );
-        shellCompletionAction->setChecked( mode == KGlobalSettings::CompletionShell );
-        popupCompletionAction->setChecked( mode == KGlobalSettings::CompletionPopup );
-        autoCompletionAction->setChecked(  mode == KGlobalSettings::CompletionAuto );
-        shortAutoCompletionAction->setChecked( mode == KGlobalSettings::CompletionMan );
-        popupAutoCompletionAction->setChecked( mode == KGlobalSettings::CompletionPopupAuto );
+        d->noCompletionAction->setChecked( mode == KGlobalSettings::CompletionNone );
+        d->shellCompletionAction->setChecked( mode == KGlobalSettings::CompletionShell );
+        d->popupCompletionAction->setChecked( mode == KGlobalSettings::CompletionPopup );
+        d->autoCompletionAction->setChecked(  mode == KGlobalSettings::CompletionAuto );
+        d->shortAutoCompletionAction->setChecked( mode == KGlobalSettings::CompletionMan );
+        d->popupAutoCompletionAction->setChecked( mode == KGlobalSettings::CompletionPopupAuto );
         if ( mode != KGlobalSettings::completionMode() )
         {
             subMenu->addSeparator();
-            defaultAction = subMenu->addAction( i18n("Default") );
+            d->defaultAction = subMenu->addAction( i18n("Default") );
         }
     }
 
+    return popup;
+}
+
+void KLineEdit::contextMenuEvent( QContextMenuEvent *e )
+{
+    if ( !d->bEnableMenu )
+      return;
+    QMenu *popup = createStandardContextMenu();
+
     // ### do we really need this?  Yes, Please do not remove!  This
     // allows applications to extend the popup menu without having to
     // inherit from this class! (DA)
@@ -921,31 +1076,31 @@ void KLineEdit::completionMenuActivated( QAction  *act)
 {
     KGlobalSettings::Completion oldMode = completionMode();
 
-    if( act == noCompletionAction )
+    if( act == d->noCompletionAction )
     {
         setCompletionMode( KGlobalSettings::CompletionNone );
     }
-    else if( act ==  shellCompletionAction)
+    else if( act ==  d->shellCompletionAction)
     {
         setCompletionMode( KGlobalSettings::CompletionShell );
     }
-    else if( act == autoCompletionAction)
+    else if( act == d->autoCompletionAction)
     {
         setCompletionMode( KGlobalSettings::CompletionAuto );
     }
-    else if( act == popupCompletionAction)
+    else if( act == d->popupCompletionAction)
     {
         setCompletionMode( KGlobalSettings::CompletionPopup );
     }
-    else if( act == shortAutoCompletionAction)
+    else if( act == d->shortAutoCompletionAction)
     {
         setCompletionMode( KGlobalSettings::CompletionMan );
     }
-    else if( act == popupAutoCompletionAction)
+    else if( act == d->popupAutoCompletionAction)
     {
         setCompletionMode( KGlobalSettings::CompletionPopupAuto );
     }
-    else if( act == defaultAction )
+    else if( act == d->defaultAction )
     {
         setCompletionMode( KGlobalSettings::completionMode() );
     }
@@ -1115,24 +1270,24 @@ bool KLineEdit::overrideAccel (const QKeyEvent* e)
     int key = e->key() | e->modifiers();
     KeyBindingMap keys = getKeyBindings();
 
-    if (keys[TextCompletion].isNull())
-        scKey = KStdAccel::shortcut(KStdAccel::TextCompletion);
+    if (keys[TextCompletion].isEmpty())
+        scKey = KStandardShortcut::shortcut(KStandardShortcut::TextCompletion);
     else
         scKey = keys[TextCompletion];
 
     if (scKey.contains( key ))
         return true;
 
-    if (keys[NextCompletionMatch].isNull())
-        scKey = KStdAccel::shortcut(KStdAccel::NextCompletion);
+    if (keys[NextCompletionMatch].isEmpty())
+        scKey = KStandardShortcut::shortcut(KStandardShortcut::NextCompletion);
     else
         scKey = keys[NextCompletionMatch];
 
     if (scKey.contains( key ))
         return true;
 
-    if (keys[PrevCompletionMatch].isNull())
-        scKey = KStdAccel::shortcut(KStdAccel::PrevCompletion);
+    if (keys[PrevCompletionMatch].isEmpty())
+        scKey = KStandardShortcut::shortcut(KStandardShortcut::PrevCompletion);
     else
         scKey = keys[PrevCompletionMatch];
 
@@ -1140,27 +1295,27 @@ bool KLineEdit::overrideAccel (const QKeyEvent* e)
         return true;
 
     // Override all the text manupilation accelerators...
-    if ( KStdAccel::copy().contains( key ) )
+    if ( KStandardShortcut::copy().contains( key ) )
         return true;
-    else if ( KStdAccel::paste().contains( key ) )
+    else if ( KStandardShortcut::paste().contains( key ) )
         return true;
-    else if ( KStdAccel::cut().contains( key ) )
+    else if ( KStandardShortcut::cut().contains( key ) )
         return true;
-    else if ( KStdAccel::undo().contains( key ) )
+    else if ( KStandardShortcut::undo().contains( key ) )
         return true;
-    else if ( KStdAccel::redo().contains( key ) )
+    else if ( KStandardShortcut::redo().contains( key ) )
         return true;
-    else if (KStdAccel::deleteWordBack().contains( key ))
+    else if (KStandardShortcut::deleteWordBack().contains( key ))
         return true;
-    else if (KStdAccel::deleteWordForward().contains( key ))
+    else if (KStandardShortcut::deleteWordForward().contains( key ))
         return true;
-    else if (KStdAccel::forwardWord().contains( key ))
+    else if (KStandardShortcut::forwardWord().contains( key ))
         return true;
-    else if (KStdAccel::backwardWord().contains( key ))
+    else if (KStandardShortcut::backwardWord().contains( key ))
         return true;
-    else if (KStdAccel::beginningOfLine().contains( key ))
+    else if (KStandardShortcut::beginningOfLine().contains( key ))
         return true;
-    else if (KStdAccel::endOfLine().contains( key ))
+    else if (KStandardShortcut::endOfLine().contains( key ))
         return true;
 
     if (d->completionBox && d->completionBox->isVisible ())
@@ -1197,23 +1352,34 @@ void KLineEdit::setCompletedItems( const QStringList& items, bool autoSuggest )
 
         if ( d->completionBox->isVisible() )
         {
-            bool wasSelected = d->completionBox->isSelected( d->completionBox->currentItem() );
-            const QString currentSelection = d->completionBox->currentText();
+            QListWidgetItem* currentItem = d->completionBox->currentItem();
+
+            bool wasSelected = false;
+            QString currentSelection;
+
+            if ( currentItem != 0 ) {
+                wasSelected = currentItem->isSelected();
+                currentSelection = currentItem->text();
+            }
+
             d->completionBox->setItems( items );
-            Q3ListBoxItem* item = d->completionBox->findItem( currentSelection, Q3ListBox::ExactMatch );
+
+            QList<QListWidgetItem*> matchedItems = d->completionBox->findItems( currentSelection , Qt::MatchExactly);
+            QListWidgetItem* matchedItem = matchedItems.isEmpty() ? 0 : matchedItems.first();
+
             // If no item is selected, that means the listbox hasn't been manipulated by the user yet,
             // because it's not possible otherwise to have no selected item. In such case make
             // always the first item current and unselected, so that the current item doesn't jump.
-            if( !item || !wasSelected )
+            if( !matchedItem || !wasSelected )
             {
                 wasSelected = false;
-                item = d->completionBox->item( 0 );
+                matchedItem = d->completionBox->item( 0 );
             }
-            if ( item )
+            if ( matchedItem )
             {
                 d->completionBox->blockSignals( true );
-                d->completionBox->setCurrentItem( item );
-                d->completionBox->setSelected( item, wasSelected );
+                d->completionBox->setCurrentItem( matchedItem );
+                matchedItem->setSelected(wasSelected);
                 d->completionBox->blockSignals( false );
             }
         }
@@ -1333,7 +1499,13 @@ void KLineEdit::paintEvent( QPaintEvent *ev )
         QPainter p( this );
         QPen tmp = p.pen();
         p.setPen( palette().color( QPalette::Disabled, QPalette::Text ) );
+
+        //FIXME: this rect is not where the text actually starts
+        // qlineedit uses an internal qstyleoption set to figure this out
+        // and then adds a hardcoded 2 pixel interior to that.
+        // probably requires fixes to Qt itself to do this cleanly
         QRect cr = contentsRect();
+
         p.drawText( cr, Qt::AlignLeft|Qt::AlignVCenter, d->clickMessage );
         p.setPen( tmp );
     }
@@ -1363,6 +1535,40 @@ void KLineEdit::setClickMessage( const QString &msg )
 {
     d->enableClickMsg = true;
     d->clickMessage = msg;
-	d->drawClickMsg = text().isEmpty();
+    d->drawClickMsg = text().isEmpty();
     update();
 }
+
+void KLineEdit::setContextMenuEnabled( bool showMenu )
+{
+    d->bEnableMenu = showMenu;
+}
+
+bool KLineEdit::isContextMenuEnabled() const
+{
+    return  d->bEnableMenu;
+}
+
+void KLineEdit::setPasswordMode(bool b)
+{
+    if(b)
+    {
+        KConfigGroup cg(KGlobal::config(), "Passwords");
+        const QString val = cg.readEntry("EchoMode", "OneStar");
+        if (val == "NoEcho")
+            setEchoMode(NoEcho);
+        else
+            setEchoMode(Password);
+       //KDE3 has also a "ThreeStars" mode, but QLineEdit doesn't support it.
+    }
+    else
+    {
+        setEchoMode( Normal );
+    }
+}
+
+bool KLineEdit::passwordMode() const
+{
+    return echoMode() == NoEcho || echoMode() == Password;
+}
+
