@@ -30,7 +30,7 @@
 #ifndef KLINEEDIT_H
 #define KLINEEDIT_H
 
-#include <QLineEdit>
+#include <QtGui/QLineEdit>
 
 #include <kcompletion.h>
 
@@ -73,7 +73,7 @@ class KUrl;
  * boolean parameter in the above functions to false.
  *
  * The default key-bindings for completion and rotation is determined
- * from the global settings in KStdAccel.  These values, however,
+ * from the global settings in KStandardShortcut.  These values, however,
  * can be overridden locally by invoking KCompletionBase::setKeyBinding().
  * The values can easily be reverted back to the default setting, by simply
  * calling useGlobalSettings(). An alternate method would be to default
@@ -153,6 +153,8 @@ class KDEUI_EXPORT KLineEdit : public QLineEdit, public KCompletionBase
     Q_PROPERTY( bool trapEnterKeyEvent READ trapReturnKey WRITE setTrapReturnKey )
     Q_PROPERTY( bool squeezedTextEnabled READ isSqueezedTextEnabled WRITE setSqueezedTextEnabled )
     Q_PROPERTY( QString clickMessage READ clickMessage WRITE setClickMessage )
+    Q_PROPERTY( bool showClearButton READ isClearButtonShown WRITE setClearButtonShown )
+    Q_PROPERTY( bool passwordMode READ passwordMode WRITE setPasswordMode )
 
 public:
 
@@ -192,6 +194,14 @@ public:
      */
     virtual void setCompletionMode( KGlobalSettings::Completion mode );
 
+   /**
+    * Disables completion modes by makeing them non-checkable.
+    *
+    * The context menu allows to change the completion mode.
+    * This method allows to disable some modes.
+    */
+    void setCompletionModeDisabled( KGlobalSettings::Completion mode, bool disable = true );
+
    /**
     * Enables/disables the popup (context) menu.
     *
@@ -206,12 +216,12 @@ public:
     *
     * @param showMenu If @p true, show the context menu.
     */
-    virtual void setContextMenuEnabled( bool showMenu ) {  m_bEnableMenu = showMenu; }
+    virtual void setContextMenuEnabled( bool showMenu );
 
     /**
      * Returns @p true when the context menu is enabled.
      */
-    bool isContextMenuEnabled() const { return m_bEnableMenu; }
+    bool isContextMenuEnabled() const;
 
     /**
      * Enables/Disables handling of URL drops. If enabled and the user
@@ -321,6 +331,17 @@ public:
      */
     QString clickMessage() const;
 
+    /**
+     * This makes the line edit display an icon on one side of the line edit
+     * which, when clicked, clears the contents of the line edit.
+     * This is useful for such things as location or search bars.
+     **/
+    void setClearButtonShown(bool show);
+
+    /**
+     * @return whether or not the clear button is shown
+     **/
+    bool isClearButtonShown() const;
 
 Q_SIGNALS:
 
@@ -436,6 +457,18 @@ public Q_SLOTS:
      */
     virtual void setText ( const QString& );
 
+    /**
+     * @brief set the line edit in password mode.
+     * this change the EchoMode according to KDE preferences.
+     * @param b true to set in password mode
+     */
+    void setPasswordMode( bool b = true );
+
+    /**
+     * @return returns true if the lineedit is set to password mode echoing
+     */
+    bool passwordMode( ) const;
+
 
 protected Q_SLOTS:
 
@@ -475,6 +508,13 @@ protected:
     */
     virtual void mousePressEvent( QMouseEvent * );
 
+    /**
+    * Re-implemented for internal reasons.  API not affected.
+    *
+    * See QLineEdit::mouseReleaseEvent().
+    */
+    virtual void mouseReleaseEvent( QMouseEvent * );
+
     /**
     * Re-implemented for internal reasons.  API not affected.
     *
@@ -489,6 +529,13 @@ protected:
     */
     virtual void contextMenuEvent( QContextMenuEvent * );
 
+    /**
+    * Re-implemented for internal reasons.  API not affected.
+    *
+    * See QLineEdit::createStandardContextMenu().
+    */
+    QMenu* createStandardContextMenu();
+
     /**
     * Re-implemented to handle URI drops.
     *
@@ -530,13 +577,17 @@ protected:
 
     virtual void focusOutEvent( QFocusEvent *ev );
 
-
 private Q_SLOTS:
     void completionMenuActivated( QAction *act );
     void tripleClickTimeout();  // resets possibleTripleClick
     void slotRestoreSelectionColors();
     void setTextWorkaround( const QString& text );
 
+    /**
+     * updates the icon of the clear button on text change
+     **/
+    void updateClearButtonIcon(const QString&);
+
 private:
 
 
@@ -545,6 +596,8 @@ private:
      */
     void init();
 
+    void copy( bool clipboard ) const;
+
     /**
      * Checks whether we should/should not consume a key used as
      * an accelerator.
@@ -557,11 +610,10 @@ private:
      */
     void setSqueezedText ();
 
-    bool m_bEnableMenu;
-
-    bool possibleTripleClick;  // set in mousePressEvent, deleted in tripleClickTimeout
-
-    QAction *noCompletionAction, *shellCompletionAction, *autoCompletionAction, *popupCompletionAction, *shortAutoCompletionAction, *popupAutoCompletionAction, *defaultAction;
+    /**
+     * updates the geometry of the clear button on resize events
+     **/
+    void updateClearButton();
 
 private:
     class KLineEditPrivate;
