@@ -92,6 +92,8 @@ static const int computedProperties[] = {
     CSS_PROP_ORPHANS,
     CSS_PROP_OUTLINE_STYLE,
     CSS_PROP_OVERFLOW,
+    CSS_PROP_OVERFLOW_X,
+    CSS_PROP_OVERFLOW_Y,
     CSS_PROP_PADDING_TOP,
     CSS_PROP_PADDING_RIGHT,
     CSS_PROP_PADDING_BOTTOM,
@@ -124,7 +126,7 @@ const unsigned numComputedProperties = sizeof(computedProperties) / sizeof(compu
 static CSSValueImpl *valueForLength(const Length &length, int max)
 {
     if (length.isPercent()) {
-        return new CSSPrimitiveValueImpl(length._length, CSSPrimitiveValue::CSS_PERCENTAGE);
+        return new CSSPrimitiveValueImpl(length.value(), CSSPrimitiveValue::CSS_PERCENTAGE);
     }
     else {
         return new CSSPrimitiveValueImpl(length.minWidth(max), CSSPrimitiveValue::CSS_PX);
@@ -265,6 +267,14 @@ DOMString khtml::stringForListStyleType(EListStyleType type)
     return "";
 }
 
+static CSSPrimitiveValueImpl* valueForColor(QColor color)
+{
+    if (color.isValid())
+        return new CSSPrimitiveValueImpl(color.rgb());//### KDE4: use rgba!
+    else
+        return new CSSPrimitiveValueImpl(khtml::transparentColor);
+}
+
 static CSSValueImpl* valueForShadow(const ShadowData *shadow)
 {
     if (!shadow)
@@ -274,7 +284,7 @@ static CSSValueImpl* valueForShadow(const ShadowData *shadow)
         CSSPrimitiveValueImpl *x = new CSSPrimitiveValueImpl(s->x, CSSPrimitiveValue::CSS_PX);
         CSSPrimitiveValueImpl *y = new CSSPrimitiveValueImpl(s->y, CSSPrimitiveValue::CSS_PX);
         CSSPrimitiveValueImpl *blur = new CSSPrimitiveValueImpl(s->blur, CSSPrimitiveValue::CSS_PX);
-        CSSPrimitiveValueImpl *color = new CSSPrimitiveValueImpl(s->color.rgb());
+        CSSPrimitiveValueImpl *color = valueForColor(s->color);
         list->append(new ShadowValueImpl(x, y, blur, color));
     }
     return list;
@@ -309,26 +319,25 @@ static CSSValueImpl *getPositionOffsetValue(RenderObject *renderer, int property
 
     if (renderer->isPositioned())
         return valueForLength(l, renderer->contentWidth());
-    
+
     if (renderer->isRelPositioned())
         // FIXME: It's not enough to simply return "auto" values for one offset if the other side is defined.
         // In other words if left is auto and right is not auto, then left's computed value is negative right.
         // So we should get the opposite length unit and see if it is auto.
         return valueForLength(l, renderer->contentWidth());
-    
+
     return new CSSPrimitiveValueImpl(CSS_VAL_AUTO);
  }
 
-
 RenderStyleDeclarationImpl::RenderStyleDeclarationImpl( DOM::NodeImpl *node )
     : CSSStyleDeclarationImpl(0), m_node(node)
 {
-    kDebug() << "Render Style Declaration created" << endl;
+    //kDebug() << "Render Style Declaration created" << endl;
 }
 
 RenderStyleDeclarationImpl::~RenderStyleDeclarationImpl()
 {
-    kDebug() << "Render Style Declaration destroyed" << endl;
+    //kDebug() << "Render Style Declaration destroyed" << endl;
 }
 
 DOM::DOMString RenderStyleDeclarationImpl::cssText() const
@@ -374,7 +383,7 @@ CSSValueImpl *RenderStyleDeclarationImpl::getPropertyCSSValue( int propertyID )
     switch(propertyID)
     {
     case CSS_PROP_BACKGROUND_COLOR:
-        return new CSSPrimitiveValueImpl(style->backgroundColor().rgb());
+        return valueForColor(style->backgroundColor());
     case CSS_PROP_BACKGROUND_IMAGE:
         if (style->backgroundImage())
             return new CSSPrimitiveValueImpl(style->backgroundImage()->url(),
@@ -403,13 +412,13 @@ CSSValueImpl *RenderStyleDeclarationImpl::getPropertyCSSValue( int propertyID )
         DOMString string;
         Length length(style->backgroundXPosition());
         if (length.isPercent())
-            string = QString::number(length._length) + "%";
+            string = QString::number(length.value()) + "%";
         else
             string = QString::number(length.minWidth(renderer->contentWidth()));
         string += " ";
         length = style->backgroundYPosition();
         if (length.isPercent())
-            string += QString::number(length._length) + "%";
+            string += QString::number(length.value()) + "%";
         else
             string += QString::number(length.minWidth(renderer->contentWidth()));
         return new CSSPrimitiveValueImpl(string, CSSPrimitiveValue::CSS_STRING);
@@ -438,13 +447,13 @@ CSSValueImpl *RenderStyleDeclarationImpl::getPropertyCSSValue( int propertyID )
         return new CSSPrimitiveValueImpl(style->borderVerticalSpacing(),
                                          CSSPrimitiveValue::CSS_PX);
     case CSS_PROP_BORDER_TOP_COLOR:
-        return new CSSPrimitiveValueImpl(style->borderTopColor().rgb());
+        return valueForColor(style->borderTopColor());
     case CSS_PROP_BORDER_RIGHT_COLOR:
-        return new CSSPrimitiveValueImpl(style->borderRightColor().rgb());
+        return valueForColor(style->borderRightColor());
     case CSS_PROP_BORDER_BOTTOM_COLOR:
-        return new CSSPrimitiveValueImpl(style->borderBottomColor().rgb());
+        return valueForColor(style->borderBottomColor());
     case CSS_PROP_BORDER_LEFT_COLOR:
-        return new CSSPrimitiveValueImpl(style->borderLeftColor().rgb());
+        return valueForColor(style->borderLeftColor());
     case CSS_PROP_BORDER_TOP_STYLE:
         return valueForBorderStyle(style->borderTopStyle());
     case CSS_PROP_BORDER_RIGHT_STYLE:
@@ -492,7 +501,7 @@ CSSValueImpl *RenderStyleDeclarationImpl::getPropertyCSSValue( int propertyID )
     case CSS_PROP_CLIP:
         break;
     case CSS_PROP_COLOR:
-        return new CSSPrimitiveValueImpl(style->color().rgb());
+        return valueForColor(style->color());
     case CSS_PROP_CONTENT:
         break;
     case CSS_PROP_COUNTER_INCREMENT:
@@ -657,16 +666,21 @@ CSSValueImpl *RenderStyleDeclarationImpl::getPropertyCSSValue( int propertyID )
         return new CSSPrimitiveValueImpl(style->letterSpacing(), CSSPrimitiveValue::CSS_PX);
     case CSS_PROP_LINE_HEIGHT:
     {
+        // Note: internally a specified <number> value gets encoded as a percentage,
+        // so the isPercent() case corresponds to the <number> case; 
+        // values < 0  are used to mark "normal"; and specified %%
+        // get computed down to px by the time they get to RenderStyle 
+        // already
         Length length(style->lineHeight());
-        if (length.value() < 0)
+        if (length.value() < 0) 
             return new CSSPrimitiveValueImpl(CSS_VAL_NORMAL);
         if (length.isPercent()) {
             //XXX: merge from webcore the computedStyle/specifiedStyle distinction in rendering/font.h
             float computedSize = style->htmlFont().getFontDef().size;
-            return new CSSPrimitiveValueImpl((int)(length._length * computedSize) / 100, CSSPrimitiveValue::CSS_PX);
+            return new CSSPrimitiveValueImpl((int)(length.value() * computedSize) / 100, CSSPrimitiveValue::CSS_PX);
         }
         else {
-            return new CSSPrimitiveValueImpl(length._length, CSSPrimitiveValue::CSS_PX);
+            return new CSSPrimitiveValueImpl(length.value(), CSSPrimitiveValue::CSS_PX);
         }
     }
     case CSS_PROP_LIST_STYLE_IMAGE:
@@ -769,8 +783,20 @@ CSSValueImpl *RenderStyleDeclarationImpl::getPropertyCSSValue( int propertyID )
     case CSS_PROP_OUTLINE_WIDTH:
         break;
     case CSS_PROP_OVERFLOW:
-    {
-        switch (style->overflow()) {
+    case CSS_PROP_OVERFLOW_X:
+    case CSS_PROP_OVERFLOW_Y: {
+        EOverflow overflow;
+        switch (propertyID) {
+        case CSS_PROP_OVERFLOW_X:
+            overflow = style->overflowX();
+            break;
+        case CSS_PROP_OVERFLOW_Y:
+            overflow = style->overflowY();
+            break;
+        default:
+            overflow = qMax(style->overflowX(), style->overflowY());
+        }
+        switch (overflow) {
         case OVISIBLE:
             return new CSSPrimitiveValueImpl(CSS_VAL_VISIBLE);
         case OHIDDEN:
@@ -824,14 +850,11 @@ CSSValueImpl *RenderStyleDeclarationImpl::getPropertyCSSValue( int propertyID )
         Q_ASSERT(0);
         break;
     case CSS_PROP_PAGE_BREAK_INSIDE:
-        switch (style->pageBreakInside()) {
-        case PBAUTO:
+        if (style->pageBreakInside())
             return new CSSPrimitiveValueImpl(CSS_VAL_AUTO);
-        case PBAVOID:
+        else
             return new CSSPrimitiveValueImpl(CSS_VAL_AVOID);
-        case PBALWAYS:
-            break; // not allowed
-        }
+        Q_ASSERT(0);
         break;
     case CSS_PROP_POSITION:
         switch (style->position()) {
@@ -1112,7 +1135,7 @@ CSSProperty RenderStyleDeclarationImpl::property( int id ) const
 {
     CSSProperty prop;
     prop.m_id = id;
-    prop.m_bImportant = false;
+    prop.m_important = false;
     prop.nonCSSHint = false;
 
     CSSValueImpl* v = getPropertyCSSValue( id );
