@@ -39,7 +39,7 @@
 #include <kconfigbase.h>
 #include <kapplication.h>
 #include <kstringhandler.h>
-#include <kstdaction.h>
+#include <kstandardaction.h>
 #include <kxmlguifactory.h>
 
 #include <ktexteditor/highlightinginterface.h>
@@ -69,7 +69,6 @@
 #include <QTabWidget>
 #include <QToolButton>
 
-#include "numberedtextview.h"
 #include "breakpointsdock.h"
 #include "consoledock.h"
 #include "localvariabledock.h"
@@ -77,6 +76,7 @@
 #include "callstackdock.h"
 #include "scriptsdock.h"
 
+#include "debugdocument.h"
 #include "debugwindow.h"
 #include "debugwindow.moc"
 
@@ -114,7 +114,7 @@ void DebugWindow::setNextSourceInfo(QString url, int baseLine)
 
 DebugWindow::DebugWindow(QWidget *parent)
   : KMainWindow(parent, "DebugWindow", Qt::WType_TopLevel),
-    KInstance("kjs_debugger")
+    KComponentData("kjs_debugger")
 {
     setCaption(i18n("JavaScript Debugger"));
     kDebug() << "creating DebugWindow" << endl;
@@ -165,36 +165,40 @@ DebugWindow::DebugWindow(QWidget *parent)
 
 void DebugWindow::createActions()
 {
-    // Standard actions
-    m_exitAct = new KAction(i18n("E&xit"), actionCollection(), "exit");
-    m_exitAct->setShortcut(i18n("Ctrl+Q"));
-    m_exitAct->setStatusTip(i18n("Exit the application"));
-    connect(m_exitAct, SIGNAL(triggered()), this, SLOT(close()));
-
     // Flow control actions
-    m_continueAct = new KAction(KIcon(":/images/continue.png"), i18n("Continue"), actionCollection(), "continue");
+    m_continueAct = new KAction(KIcon(":/images/continue.png"), i18n("Continue"), this );
+    actionCollection()->addAction( "continue", m_continueAct );
     m_continueAct->setStatusTip(i18n("Continue script execution"));
     m_continueAct->setToolTip(i18n("Continue script execution"));
+    m_continueAct->setEnabled(false);
     connect(m_continueAct, SIGNAL(triggered(bool)), this, SLOT(continueExecution()));
 
-    m_stopAct = new KAction(KIcon(":/images/stop.png"), i18n("Stop"), actionCollection(), "stop");
+    m_stopAct = new KAction(KIcon(":/images/stop.png"), i18n("Stop"), this );
+    actionCollection()->addAction( "stop", m_stopAct );
     m_stopAct->setStatusTip(i18n("Stop script execution"));
     m_stopAct->setToolTip(i18n("Stop script execution"));
+    m_stopAct->setEnabled(false);
     connect(m_stopAct, SIGNAL(triggered(bool)), this, SLOT(stopExecution()));
 
-    m_stepIntoAct = new KAction(KIcon(":/images/step-into.png"), i18n("Step Into"), actionCollection(), "stepInto");
+    m_stepIntoAct = new KAction(KIcon(":/images/step-into.png"), i18n("Step Into"), this );
+    actionCollection()->addAction( "stepInto", m_stepIntoAct );
     m_stepIntoAct->setStatusTip(i18n("Step Into"));
     m_stepIntoAct->setToolTip(i18n("Step Into"));
+    m_stepIntoAct->setEnabled(false);
     connect(m_stepIntoAct, SIGNAL(triggered(bool)), this, SLOT(stepInto()));
 
-    m_stepOutAct = new KAction(KIcon(":/images/step-out.png"), i18n("Step Out"), actionCollection(), "stepOut");
+    m_stepOutAct = new KAction(KIcon(":/images/step-out.png"), i18n("Step Out"), this );
+    actionCollection()->addAction( "stepOut", m_stepOutAct );
     m_stepOutAct->setStatusTip(i18n("Step Out"));
     m_stepOutAct->setToolTip(i18n("Step Out"));
+    m_stepOutAct->setEnabled(false);
     connect(m_stepOutAct, SIGNAL(triggered(bool)), this, SLOT(stepOut()) );
 
-    m_stepOverAct = new KAction(KIcon(":/images/step-over.png"), i18n("Step Over"), actionCollection(), "stepOver");
+    m_stepOverAct = new KAction(KIcon(":/images/step-over.png"), i18n("Step Over"), this );
+    actionCollection()->addAction( "stepOver", m_stepOverAct );
     m_stepOverAct->setStatusTip(i18n("Step Over"));
     m_stepOverAct->setToolTip(i18n("Step Over"));
+    m_stepOverAct->setEnabled(false);
     connect(m_stepOverAct, SIGNAL(triggered(bool)), this, SLOT(stepOver()) );
 }
 
@@ -210,7 +214,6 @@ void DebugWindow::createMenus()
     menuBar()->insertItem("&Debug", debugMenu);
 */
     KMenu *fileMenu = new KMenu("F&ile", this);
-    fileMenu->addAction(m_exitAct);
     menuBar()->addMenu(fileMenu);
 }
 
@@ -228,15 +231,7 @@ void DebugWindow::createTabWidget()
 {
     QVBoxLayout *layout = new QVBoxLayout(m_docFrame);
     m_tabWidget = new QTabWidget;
-/*
-    QToolButton *newTabButton = new QToolButton(this);
-    m_tabWidget->setCornerWidget(newTabButton, Qt::TopLeftCorner);
-    newTabButton->setCursor(Qt::ArrowCursor);
-    newTabButton->setAutoRaise(true);
-    newTabButton->setIcon(QIcon(ImageLocation + QLatin1String("addtab.png")));
-    QObject::connect(newTabButton, SIGNAL(clicked()), this, SLOT(newTab()));
-    newTabButton->setToolTip(tr("Add page"));
-*/
+
     QToolButton *closeTabButton = new QToolButton(m_tabWidget);
     m_tabWidget->setCornerWidget(closeTabButton, Qt::TopRightCorner);
     closeTabButton->setCursor(Qt::ArrowCursor);
@@ -257,14 +252,17 @@ void DebugWindow::createStatusBar()
 void DebugWindow::stopExecution()
 {
     m_mode = Stop;
-
-//    while (!m_execStates.isEmpty())
-//        leaveSession();
 }
 
 void DebugWindow::continueExecution()
 {
-    enableOtherWindows();
+    m_continueAct->setEnabled(false);
+    m_stopAct->setEnabled(false);
+    m_stepIntoAct->setEnabled(false);
+    m_stepOutAct->setEnabled(false);
+    m_stepOverAct->setEnabled(false);
+
+    exitLoop();
     m_mode = Continue;
 }
 
@@ -293,7 +291,7 @@ DebugWindow::~DebugWindow()
 
 // -------------------------------------------------------------
 bool DebugWindow::sourceParsed(ExecState *exec, int sourceId, const UString &sourceURL,
-                               const UString &source, int startingLineNumber, int errorLine, const UString &errorMsg)
+                               const UString &source, int /* startingLineNumber */, int errorLine, const UString &/* errorMsg */)
 {
     Q_UNUSED(exec);
 
@@ -414,7 +412,7 @@ bool DebugWindow::atStatement(ExecState *exec, int sourceId, int firstLine, int
 //             kDebug() << "Hey! we actually found a breakpoint!" << endl;
             // Lets try a dump of the scope chain now..
             // m_localVariables->display(exec);
-            enterDebugSession(exec);
+            enterDebugSession(exec, document);
         }
     }
 
@@ -425,54 +423,79 @@ bool DebugWindow::atStatement(ExecState *exec, int sourceId, int firstLine, int
 
 bool DebugWindow::callEvent(ExecState *exec, int sourceId, int lineno, JSObject *function, const List &args)
 {
-    Q_UNUSED(exec);
-    Q_UNUSED(sourceId);
-    Q_UNUSED(lineno);
-    Q_UNUSED(function);
-    Q_UNUSED(args);
+    kDebug() << "***************************** callEvent **************************************************" << endl;
+    kDebug() << "  sourceId: " << sourceId << endl
+             << "lineNumber: " << lineno << endl;
 
-//     kDebug() << "callEvent" << endl;
+    DebugDocument *document = m_sourceIdLookup[sourceId];
+    if (document)
+    {
+        if (function->inherits(&InternalFunctionImp::info))
+        {
+            KJS::InternalFunctionImp *func = static_cast<InternalFunctionImp*>(function);
+            if (func)
+            {
+                QString functionName = func->functionName().qstring();
+                kDebug() << "  function: " << functionName << endl;
+                func = 0;
+
+                document->addCall(functionName, lineno);
+            }
+        }
+
+        for( KJS::ListIterator item = args.begin();
+            item != args.end();
+            ++item)
+        {
+            KJS::JSValue *value = (*item);
+            kDebug() << "arg: " << value->toString(exec).qstring() << endl;
+        }
+    }
+
+
+    kDebug() << "****************************************************************************************" << endl;
 
     return (m_mode != Stop);
 }
 
 bool DebugWindow::returnEvent(ExecState *exec, int sourceId, int lineno, JSObject *function)
 {
-    Q_UNUSED(exec);
-    Q_UNUSED(sourceId);
-    Q_UNUSED(lineno);
-    Q_UNUSED(function);
+    kDebug() << "***************************** returnEvent **************************************************" << endl;
+    kDebug() << "  sourceId: " << sourceId << endl
+             << "lineNumber: " << lineno << endl;
 
-//     kDebug() << "returnEvent" << endl;
+    DebugDocument *document = m_sourceIdLookup[sourceId];
+    if (document)
+    {
+        if (function->inherits(&InternalFunctionImp::info))
+        {
+            KJS::InternalFunctionImp *func = static_cast<InternalFunctionImp*>(function);
+            if (func)
+            {
+                QString functionName = func->functionName().qstring();
+                kDebug() << "  function: " << functionName << endl;
+                func = 0;
+
+                document->removeCall(functionName, lineno);
+            }
+        }
+    }
+
+    kDebug() << "****************************************************************************************" << endl;
 
     return (m_mode != Stop);
 }
 
 // End KJS::Debugger overloads
 
+
 void DebugWindow::enableKateHighlighting(KTextEditor::Document *document)
 {
     KTextEditor::HighlightingInterface *highlightingInterface = qobject_cast<KTextEditor::HighlightingInterface*>(document);
-    if (!m_highlightingMode)
-    {
-        if (highlightingInterface)
-        {
-            int count = highlightingInterface->hlModeCount();
-            for (int i=0; i<count; i++)
-            {
-                QString modeName = highlightingInterface->hlModeName(i);
-                QString sectionName = highlightingInterface->hlModeSectionName(i);
-                if (modeName == "JavaScript")
-                {
-                    m_highlightingMode = i;
-                    break;
-                }
-            }
-        }
-    }
+    if (!highlightingInterface)
+        return;
 
-    if (m_highlightingMode && highlightingInterface)
-        highlightingInterface->setHlMode(m_highlightingMode);
+    highlightingInterface->setHighlighting("JavaScript");
 }
 
 void DebugWindow::displayScript(KJS::DebugDocument *document)
@@ -577,7 +600,7 @@ void DebugWindow::closeTab()
 }
 
 
-void DebugWindow::enterDebugSession(KJS::ExecState *exec)
+void DebugWindow::enterDebugSession(KJS::ExecState *exec, DebugDocument *document)
 {
     // This "enters" a new debugging session, i.e. enables usage of the debugging window
     // It re-enters the qt event loop here, allowing execution of other parts of the
@@ -589,7 +612,6 @@ void DebugWindow::enterDebugSession(KJS::ExecState *exec)
         show();
 
     m_mode = Stop;
-
 //    if (m_execStates.isEmpty())
     {
         m_continueAct->setEnabled(true);
@@ -599,22 +621,10 @@ void DebugWindow::enterDebugSession(KJS::ExecState *exec)
         m_stepOverAct->setEnabled(true);
     }
 
-
+    m_callStack->displayStack(document);
     m_localVariables->display(exec);
-    disableOtherWindows();
-/*
-    bool done = false;
-    while (!done)
-    {
-        if (m_mode == Continue)
-            done = true;
-        kapp->processEvents(QEventLoop::ExcludeSocketNotifiers | 
-                            QEventLoop::DeferredDeletion | QEventLoop::X11ExcludeTimers);
 
-    }
-
-
-    m_localVariables->clear(); */
+    enterLoop();
 }
 
 
@@ -624,13 +634,6 @@ void DebugWindow::enterDebugSession(KJS::ExecState *exec)
 
 bool DebugWindow::eventFilter(QObject *object, QEvent *event)
 {
-    /*
-    if (object == this)
-        return QObject::eventFilter(object, event);
-    return true;
-    */
-
-/*
     switch (event->type())
     {
         case QEvent::MouseButtonPress:
@@ -642,200 +645,47 @@ bool DebugWindow::eventFilter(QObject *object, QEvent *event)
         case QEvent::Destroy:
         case QEvent::Close:
         case QEvent::Quit:
-            if (object == this)
-                return QWidget::eventFilter(object, event);
-            else
-                return true;
+            {
+                while (object->parent())
+                    object = object->parent();
+                if (object == this)
+                    return QWidget::eventFilter(object, event);
+                else
+                    return true;
+            }
             break;
         default:
             return QWidget::eventFilter(object, event);
     }
-    */
-    return QWidget::eventFilter(object, event);
+
 }
 
-void DebugWindow::disableOtherWindows()
+void DebugWindow::enterLoop()
 {
-    kapp->installEventFilter(this);
-/*
-  QWidgetList widgets = QApplication::allWidgets();
-  QListIterator<QWidget*> it(widgets);
-  while (it.hasNext()) {
-    QWidget* widget = it.next();
-    widget->installEventFilter(this);
-  }
-*/
+    QEventLoop eventLoop;
+    connect(this, SIGNAL(quitLoop()), &eventLoop, SLOT(quit()));
+    enterModality();
+
+//    eventLoop.exec(QEventLoop::X11ExcludeTimers | QEventLoop::ExcludeSocketNotifiers);
+    eventLoop.exec();
 }
 
-void DebugWindow::enableOtherWindows()
+void DebugWindow::exitLoop()
 {
-    kapp->removeEventFilter(this);
-/*
-  QWidgetList widgets = QApplication::allWidgets();
-  QListIterator<QWidget*> it(widgets);
-  while (it.hasNext()) {
-    QWidget* widget = it.next();
-    widget->removeEventFilter(this);
-  }
-*/
+    leaveModality();
+    emit quitLoop();
 }
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-////////////////////// DEBUG DOCUMENT
-
-
-class DebugDocument::Private : public QSharedData
+void DebugWindow::enterModality()
 {
-public:
-    QString url;
-    QString name;
-    QString source;
-    Interpreter *interpreter;
-    QHash<int, SourceFragment> codeFragments;
-    QVector<int> breakpoints;
-
-};
-
-DebugDocument::DebugDocument(const QString &url, Interpreter *interpreter)
-{
-    d = new DebugDocument::Private;
-    d->url = url;
-    d->interpreter = interpreter;
-
-    QStringList splitUrl = url.split('/');
-    if (!splitUrl.isEmpty())
-    {
-        qDebug() << splitUrl;
-        while (d->name.isEmpty() && !splitUrl.isEmpty())
-            d->name = splitUrl.takeLast();
-
-        if (d->name.isEmpty())
-            d->name = "undefined";
-    }
-    else
-        d->name = "undefined";
-
-    if (d->interpreter)
-    {
-        ScriptInterpreter *scriptInterpreter = static_cast<ScriptInterpreter*>(d->interpreter);
-        KHTMLPart *part = qobject_cast<KHTMLPart*>(scriptInterpreter->part());
-        if (part &&
-            d->url == part->url().url())
-        {
-//            connect(part, SIGNAL(completed()), this, SLOT(readSource()));
-        }
-    }
+    QWidgetList widgets = QApplication::allWidgets();
+    foreach (QWidget *widget, widgets)
+        widget->installEventFilter(this);
 }
 
-DebugDocument::DebugDocument(const DebugDocument &other)
+void DebugWindow::leaveModality()
 {
-    d = other.d;
+    QWidgetList widgets = QApplication::allWidgets();
+    foreach (QWidget *widget, widgets)
+        widget->removeEventFilter(this);
 }
-
-DebugDocument::~DebugDocument()
-{
-}
-
-QString DebugDocument::name() const
-{
-    return d->name;
-}
-
-QString DebugDocument::url() const
-{
-    return d->url;
-}
-
-Interpreter * DebugDocument::interpreter() const
-{
-    return d->interpreter;
-}
-
-QList<SourceFragment> DebugDocument::fragments()
-{
-    return d->codeFragments.values();
-}
-
-bool DebugDocument::deleteFragment(int sourceId)
-{
-    if (d->codeFragments.contains(sourceId))
-    {
-        d->codeFragments.remove(sourceId);
-        return true;
-    }
-    return false;
-}
-
-SourceFragment DebugDocument::fragment(int sourceId)
-{
-    if (d->codeFragments.contains(sourceId))
-        return d->codeFragments[sourceId];
-    else
-        return SourceFragment();
-}
-
-QString DebugDocument::source() const
-{
-    return d->source;
-}
-
-void DebugDocument::addCodeFragment(int sourceId, int baseLine, const QString &source)
-{
-    SourceFragment code;
-    code.sourceId = sourceId;
-    code.baseLine = baseLine;
-    code.source = source;
-
-    d->codeFragments[sourceId] = code;
-}
-
-void DebugDocument::setBreakpoint(int lineNumber)
-{
-    d->breakpoints.append(lineNumber);
-}
-
-void DebugDocument::removeBreakpoint(int lineNumber)
-{
-    int idx = d->breakpoints.indexOf(lineNumber);
-    if (idx != -1)
-        d->breakpoints.remove(idx);
-}
-
-bool DebugDocument::hasBreakpoint(int lineNumber)
-{
-    return d->breakpoints.contains(lineNumber);
-}
-
-QVector<int> DebugDocument::breakpoints()
-{
-    return d->breakpoints;
-}
-
-void DebugDocument::readSource()
-{
-    if (d->interpreter)
-    {
-        ScriptInterpreter *scriptInterpreter = static_cast<ScriptInterpreter*>(d->interpreter);
-        KHTMLPart *part = qobject_cast<KHTMLPart*>(scriptInterpreter->part());
-        if (part &&
-            d->url == part->url().url() &&
-            !part->inProgress())
-        {
-            d->source = part->documentSource();
-        }
-    }
-}
-
