@@ -74,7 +74,7 @@ static QString cssPropertyName( const Identifier &p, bool& hadPixelPrefix )
     QString prop = p.qstring();
     int i = prop.length();
     while ( --i ) {
-        char c = prop[i].latin1();
+        char c = prop[i].toLatin1();
         if ( c >= 'A' && c <= 'Z' )
             prop.insert( i, '-' );
     }
@@ -134,6 +134,8 @@ KJS_DEFINE_PROTOTYPE(DOMCSSStyleDeclarationProto)
 KJS_IMPLEMENT_PROTOFUNC(DOMCSSStyleDeclarationProtoFunc)
 KJS_IMPLEMENT_PROTOTYPE("DOMCSSStyleDeclaration", DOMCSSStyleDeclarationProto, DOMCSSStyleDeclarationProtoFunc)
 
+IMPLEMENT_PSEUDO_CONSTRUCTOR(CSSStyleDeclarationPseudoCtor, "DOMCSSStyleDeclaration",DOMCSSStyleDeclarationProto)
+
 const ClassInfo DOMCSSStyleDeclaration::info = { "CSSStyleDeclaration", 0, &DOMCSSStyleDeclarationTable, 0 };
 
 DOMCSSStyleDeclaration::DOMCSSStyleDeclaration(ExecState *exec, DOM::CSSStyleDeclarationImpl* s)
@@ -243,8 +245,13 @@ void DOMCSSStyleDeclaration::put(ExecState *exec, const Identifier &propertyName
     if (int pId = cssPropertyId(prop)) {
       if (propvalue.isEmpty())
         styleDecl.removeProperty(pId);
-      else
-        styleDecl.setProperty(pId,DOM::DOMString(propvalue),"", exception); // ### is "" ok for priority?
+      else {
+        int important = propvalue.indexOf("!important", 0, Qt::CaseInsensitive);
+        if (important == -1)
+            styleDecl.setProperty(pId, DOM::DOMString(propvalue), "", exception);
+        else
+            styleDecl.setProperty(pId, DOM::DOMString(propvalue.left(important - 1)), "important", exception);
+      }
     }
     else
       // otherwise add it as a JS property
