@@ -56,7 +56,6 @@
 #include <kshortcut.h>
 #include <kconfig.h>
 #include <kconfigbase.h>
-#include <kapplication.h>
 #include <kstringhandler.h>
 
 #include "kjs_dom.h"
@@ -85,7 +84,7 @@ SourceDisplay::SourceDisplay(KJSDebugWin *debugWin, QWidget *parent, const char
 {
   verticalScrollBar()->setSingleStep(QFontMetrics(m_font).height());
   viewport()->setAttribute(Qt::WA_NoSystemBackground);
-  m_breakpointIcon = KGlobal::iconLoader()->loadIcon("stop",K3Icon::Small);
+  m_breakpointIcon = KIconLoader::global()->loadIcon("stop",K3Icon::Small);
 }
 
 SourceDisplay::~SourceDisplay()
@@ -306,9 +305,9 @@ KJSErrorDialog::KJSErrorDialog(QWidget *parent, const QString& errorMessage, boo
   setMainWidget(page);
 
   QLabel *iconLabel = new QLabel("",page);
-  iconLabel->setPixmap(KGlobal::iconLoader()->loadIcon("messagebox_critical",
-						       K3Icon::NoGroup,K3Icon::SizeMedium,
-						       K3Icon::DefaultState,0,true));
+  iconLabel->setPixmap(KIconLoader::global()->loadIcon("messagebox_critical",
+                                                    K3Icon::NoGroup,K3Icon::SizeMedium,
+                                                    K3Icon::DefaultState,0,true));
 
   QWidget *contents = new QWidget(page);
   QLabel *label = new QLabel(errorMessage,contents);
@@ -328,6 +327,7 @@ KJSErrorDialog::KJSErrorDialog(QWidget *parent, const QString& errorMessage, boo
   topLayout->addStretch(10);
 
   m_debugSelected = false;
+  connect(this,SIGNAL(user1Clicked()),this,SLOT(slotUser1()));
 }
 
 KJSErrorDialog::~KJSErrorDialog()
@@ -361,7 +361,7 @@ void EvalMultiLineEdit::keyPressEvent(QKeyEvent * e)
 }
 //-------------------------------------------------------------------------
 KJSDebugWin::KJSDebugWin(QWidget *parent, const char *name)
-  : KMainWindow(parent, name, Qt::WType_TopLevel), KInstance("kjs_debugger")
+  : KMainWindow(parent, name, Qt::WType_TopLevel), KComponentData("kjs_debugger")
 {
   m_breakpoints = 0;
   m_breakpointCount = 0;
@@ -375,7 +375,7 @@ KJSDebugWin::KJSDebugWin(QWidget *parent, const char *name)
   m_execsAlloc = 0;
   m_steppingDepth = 0;
 
-  m_stopIcon = KGlobal::iconLoader()->loadIcon("stop",K3Icon::Small);
+  m_stopIcon = KIconLoader::global()->loadIcon("stop",K3Icon::Small);
   m_emptyIcon = QPixmap(m_stopIcon.width(),m_stopIcon.height());
   QBitmap emptyMask(m_stopIcon.width(),m_stopIcon.height());
   emptyMask.clear();
@@ -455,25 +455,25 @@ KJSDebugWin::KJSDebugWin(QWidget *parent, const char *name)
   KMenu *debugMenu = new KMenu(this);
   menuBar()->insertItem("&Debug",debugMenu);
 
-  m_nextAction = new KAction(i18nc("Next breakpoint","&Next"), actionCollection(),"next");
-  m_nextAction->setIcon( KIcon( "dbgnext" ) );
-  connect( m_nextAction, SIGNAL( triggered( bool ) ), this, SLOT(slotNext()) );
+  m_actionCollection = new KActionCollection(this);
+  m_actionCollection->setComponentData(*this);
 
-  m_stepAction = new KAction(i18n("&Step"), actionCollection(),"step");
-  m_stepAction->setIcon( KIcon( "dbgstep" ) );
-  connect( m_stepAction, SIGNAL( triggered( bool ) ), this,SLOT(slotStep()) );
-
-  m_continueAction = new KAction(i18n("&Continue"), actionCollection(),"cont");
-  m_continueAction->setIcon( KIcon( "dbgrun" ) );
-  connect( m_continueAction, SIGNAL( triggered( bool ) ),this,SLOT(slotContinue()) );
-
-  m_stopAction = new KAction(i18n("St&op"), actionCollection(),"stop");
-  m_stopAction->setIcon( KIcon( "stop" ) );
-  connect( m_stopAction, SIGNAL( triggered( bool ) ),this,SLOT(slotStop()) );
-
-  m_breakAction = new KAction(i18n("&Break at Next Statement"), actionCollection(),"breaknext");
-  m_breakAction->setIcon( KIcon( "dbgrunto" ) );
-  connect( m_breakAction, SIGNAL( triggered( bool ) ),this,SLOT(slotBreakNext()) );
+  // Venkman use F12, KDevelop F10
+  KShortcut scNext = KShortcut(KKeySequence(KKey(Qt::Key_F12)));
+  scNext.append(KKeySequence(KKey(Qt::Key_F10)));
+  m_nextAction       = new KAction(i18nc("Next breakpoint","&Next"),"dbgnext",scNext,this,SLOT(slotNext()),
+				   m_actionCollection,"next");
+  m_stepAction       = new KAction(i18n("&Step"),"dbgstep",KShortcut(Qt::Key_F11),this,SLOT(slotStep()),
+				   m_actionCollection,"step");
+  // Venkman use F5, Kdevelop F9
+  KShortcut scCont = KShortcut(KKeySequence(KKey(Qt::Key_F5)));
+  scCont.append(KKeySequence(KKey(Qt::Key_F9)));
+  m_continueAction   = new KAction(i18n("&Continue"),"dbgrun",scCont,this,SLOT(slotContinue()),
+				   m_actionCollection,"cont");
+  m_stopAction       = new KAction(i18n("St&op"),"stop",KShortcut(Qt::Key_F4),this,SLOT(slotStop()),
+				   m_actionCollection,"stop");
+  m_breakAction      = new KAction(i18n("&Break at Next Statement"),"dbgrunto",KShortcut(Qt::Key_F8),this,SLOT(slotBreakNext()),
+				   m_actionCollection,"breaknext");
 
   m_nextAction->setToolTip(i18nc("Next breakpoint","Next"));
   m_stepAction->setToolTip(i18n("Step"));
@@ -755,7 +755,7 @@ bool KJSDebugWin::sourceParsed(KJS::ExecState *exec, int sourceId,
       // but we still know the interpreter
       sourceFile = new SourceFile("(unknown)",source.qstring(),exec->interpreter());
       m_sourceSelFiles.append(sourceFile);
-      m_sourceSel->addItem("???");
+      m_sourceSel->addItem(QString::number(index) += "-???");
     }
   }
   else {
