@@ -56,10 +56,10 @@
 using namespace DOM;
 using namespace khtml;
 
-HTMLElementImpl::HTMLElementImpl(DocumentPtr *doc)
+HTMLElementImpl::HTMLElementImpl(DocumentImpl *doc)
     : ElementImpl( doc )
 {
-    m_htmlCompat = doc && doc->document() ? doc->document()->htmlMode() != DocumentImpl::XHtml : false;
+    m_htmlCompat = doc && doc->htmlMode() != DocumentImpl::XHtml;
 }
 
 HTMLElementImpl::~HTMLElementImpl()
@@ -259,6 +259,10 @@ void HTMLElementImpl::parseAttribute(AttributeImpl *attr)
         setHTMLEventListener(EventImpl::BLUR_EVENT,
             getDocument()->createHTMLEventListener(attr->value().string(), "onblur", this));
         break;
+    case ATTR_ONSCROLL:
+        setHTMLEventListener(EventImpl::SCROLL_EVENT,
+            getDocument()->createHTMLEventListener(attr->value().string(), "onscroll", this));
+        break;
 // other misc attributes
     default:
 #ifdef UNSUPPORTED_ATTR
@@ -450,15 +454,17 @@ void HTMLElementImpl::removeCSSProperty(int id)
 
 DOMString HTMLElementImpl::innerHTML() const
 {
-    DOMString result = "";
-    for (NodeImpl *child = firstChild(); child != NULL; child = child->nextSibling())
-        result += child->toString();
+    QString result; //Use QString to accumulate since DOMString is poor for appends
+    for (NodeImpl *child = firstChild(); child != NULL; child = child->nextSibling()) {
+        DOMString kid = child->toString();
+        result += QConstString(kid.unicode(), kid.length()).string();
+    }
     return result;
 }
 
 DOMString HTMLElementImpl::innerText() const
 {
-    DOMString text = "";
+    QString text = "";
     if(!firstChild())
         return text;
 
@@ -479,7 +485,8 @@ DOMString HTMLElementImpl::innerText() const
             n = next;
         }
         if(n->isTextNode() ) {
-            text += static_cast<const TextImpl *>(n)->data();
+            DOMStringImpl* data = static_cast<const TextImpl *>(n)->string();
+            text += QConstString(data->s, data->l).string();
         }
     }
  end:
@@ -528,28 +535,26 @@ DocumentFragment HTMLElementImpl::createContextualFragment( const DOMString &htm
     // we need to pop <html> and <body> elements and remove <head> to
     // accomadate folks passing complete HTML documents to make the
     // child of an element.
-    Node n;
-
     for ( NodeImpl* node = fragment->firstChild(); node; ) {
         if (node->id() == ID_HTML || node->id() == ID_BODY) {
             NodeImpl* firstChild = node->firstChild();
             NodeImpl* child = firstChild;
             while ( child ) {
                 NodeImpl *nextChild = child->nextSibling();
-                n = fragment->insertBefore(child, node, ignoredExceptionCode);
+                fragment->insertBefore(child, node, ignoredExceptionCode);
                 child = nextChild;
             }
             if ( !firstChild ) {
                 NodeImpl *nextNode = node->nextSibling();
-                n = fragment->removeChild(node, ignoredExceptionCode);
+                fragment->removeChild(node, ignoredExceptionCode);
                 node = nextNode;
             } else {
-                n = fragment->removeChild(node, ignoredExceptionCode);
+                fragment->removeChild(node, ignoredExceptionCode);
                 node = firstChild;
             }
         } else if (node->id() == ID_HEAD) {
             NodeImpl *nextNode = node->nextSibling();
-            n = fragment->removeChild(node, ignoredExceptionCode);
+            fragment->removeChild(node, ignoredExceptionCode);
             node = nextNode;
         } else {
             node = node->nextSibling();
@@ -663,7 +668,7 @@ DOMString HTMLElementImpl::toString() const
 }
 
 // -------------------------------------------------------------------------
-HTMLGenericElementImpl::HTMLGenericElementImpl(DocumentPtr *doc, ushort i)
+HTMLGenericElementImpl::HTMLGenericElementImpl(DocumentImpl *doc, ushort i)
     : HTMLElementImpl(doc)
 {
     _id = i;
