@@ -73,7 +73,7 @@
 using namespace DOM;
 using namespace khtml;
 
-HTMLFormElementImpl::HTMLFormElementImpl(DocumentPtr *doc, bool implicit)
+HTMLFormElementImpl::HTMLFormElementImpl(DocumentImpl *doc, bool implicit)
     : HTMLElementImpl(doc)
 {
     m_implicit = implicit;
@@ -237,7 +237,7 @@ QByteArray HTMLFormElementImpl::formData(bool& ok)
     QString str = m_acceptcharset.string();
     const QChar space(' ');
     const unsigned int strLength = str.length();
-    for(unsigned int i=0; i < strLength; ++i) if(str[i].latin1() == ',') str[i] = space;
+    for(unsigned int i=0; i < strLength; ++i) if(str[i].toLatin1() == ',') str[i] = space;
     const QStringList charsets = str.split(' ');
     QTextCodec* codec = 0;
     KHTMLView *view = getDocument()->view();
@@ -270,7 +270,7 @@ QByteArray HTMLFormElementImpl::formData(bool& ok)
     m_encCharset = codec->name();
     const unsigned int m_encCharsetLength = m_encCharset.length();
     for(unsigned int i=0; i < m_encCharsetLength; ++i)
-        m_encCharset[i] = m_encCharset[i].latin1() == ' ' ? QChar('-') : m_encCharset[i].toLower();
+        m_encCharset[i] = m_encCharset[i].toLatin1() == ' ' ? QChar('-') : m_encCharset[i].toLower();
 
     QStringList fileUploads, fileNotUploads;
 
@@ -331,7 +331,7 @@ QByteArray HTMLFormElementImpl::formData(bool& ok)
                         hstr += fixUpfromUnicode(codec, "; filename=\"" + path.fileName() + "\"");
                         if (path.isValid()) {
                             fileUploads << path.pathOrUrl();
-                            const KMimeType::Ptr ptr = KMimeType::findByURL(path);
+                            const KMimeType::Ptr ptr = KMimeType::findByUrl(path);
                             if (!ptr->name().isEmpty()) {
                                 hstr += "\r\nContent-Type: ";
                                 hstr += ptr->name().toAscii().constData();
@@ -529,7 +529,7 @@ void HTMLFormElementImpl::gatherWalletData()
     m_havePassword = false;
     m_haveTextarea = false;
     const KUrl formUrl(getDocument()->URL());
-    if (!view->nonPasswordStorableSite(formUrl.host())) {
+    if (view && !view->nonPasswordStorableSite(formUrl.host())) {
         for (Q3PtrListIterator<HTMLGenericFormElementImpl> it(formElements); it.current(); ++it) {
             if (it.current()->id() == ID_INPUT)  {
                 HTMLInputElementImpl* const c = static_cast<HTMLInputElementImpl*> (it.current());
@@ -627,9 +627,9 @@ void HTMLFormElementImpl::submit(  )
                 dialog->setObjectName( "questionYesNoCancel" );
                 dialog->setCaption( i18n("Save Login Information") );
                 dialog->setButtons( KDialog::Yes | KDialog::No | KDialog::Cancel );
-                dialog->setButtonGuiItem( KDialog::User1, i18n("Store") );
+                dialog->setButtonGuiItem( KDialog::User1, KGuiItem(i18n("Store")) );
                 dialog->setButtonGuiItem( KDialog::User2, KGuiItem(i18n("Ne&ver for This Site")) );
-                dialog->setButtonGuiItem( KDialog::User3, i18n("Do Not Store") );
+                dialog->setButtonGuiItem( KDialog::User3, KGuiItem(i18n("Do Not Store")) );
                 dialog->setDefaultButton( KDialog::Yes );
                 dialog->setEscapeButton( KDialog::Cancel );
                 dialog->setModal( true );
@@ -811,7 +811,7 @@ void HTMLFormElementImpl::removeImgElement(HTMLImageElementImpl *e)
 
 // -------------------------------------------------------------------------
 
-HTMLGenericFormElementImpl::HTMLGenericFormElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f)
+HTMLGenericFormElementImpl::HTMLGenericFormElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f)
     : HTMLElementImpl(doc)
 {
     m_disabled = m_readOnly = false;
@@ -913,6 +913,11 @@ HTMLFormElementImpl *HTMLGenericFormElementImpl::getForm() const
     {
         if( p->id() == ID_FORM )
             return static_cast<HTMLFormElementImpl *>(p);
+        if( p->parentNode() && p->parentNode()->id() == ID_TABLE && p->previousSibling() )
+        {
+            p = p->previousSibling();
+            continue;
+        }
         p = p->parentNode();
     }
 #ifdef FORMS_DEBUG
@@ -1006,9 +1011,12 @@ void HTMLGenericFormElementImpl::defaultEventHandler(EventImpl *evt)
         case EventImpl::MOUSEMOVE_EVENT:
         case EventImpl::MOUSEOUT_EVENT:
         case EventImpl::MOUSEOVER_EVENT:
+        case EventImpl::KHTML_MOUSEWHEEL_EVENT:
         case EventImpl::KEYDOWN_EVENT:
         case EventImpl::KEYUP_EVENT:
         case EventImpl::KEYPRESS_EVENT:
+        case EventImpl::DOMFOCUSIN_EVENT:
+        case EventImpl::DOMFOCUSOUT_EVENT:
             if (static_cast<RenderWidget*>(renderer())->handleEvent(*evt))
 		evt->setDefaultHandled();
         default:
@@ -1029,6 +1037,8 @@ void HTMLGenericFormElementImpl::defaultEventHandler(EventImpl *evt)
         if (evt->id()==EventImpl::MOUSEDOWN_EVENT || evt->id()==EventImpl::KEYDOWN_EVENT)
         {
             setActive();
+            if (renderer() && renderer()->isWidget())
+                static_cast<RenderWidget*>(renderer())->widget()->setFocus(); // ### mmh..
         }
         else if (evt->id() == EventImpl::MOUSEUP_EVENT || evt->id()==EventImpl::KEYUP_EVENT)
         {
@@ -1082,7 +1092,7 @@ bool HTMLGenericFormElementImpl::isEditable()
 
 // -------------------------------------------------------------------------
 
-HTMLButtonElementImpl::HTMLButtonElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f)
+HTMLButtonElementImpl::HTMLButtonElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f)
     : HTMLGenericFormElementImpl(doc, f)
 {
     m_clicked = false;
@@ -1195,7 +1205,7 @@ void HTMLButtonElementImpl::attach()
 
 // -------------------------------------------------------------------------
 
-HTMLFieldSetElementImpl::HTMLFieldSetElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f)
+HTMLFieldSetElementImpl::HTMLFieldSetElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f)
     : HTMLGenericFormElementImpl(doc, f)
 {
 }
@@ -1232,7 +1242,7 @@ void HTMLFieldSetElementImpl::parseAttribute(AttributeImpl *attr)
 
 // -------------------------------------------------------------------------
 
-HTMLInputElementImpl::HTMLInputElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f)
+HTMLInputElementImpl::HTMLInputElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f)
     : HTMLGenericFormElementImpl(doc, f)
 {
     m_type = TEXT;
@@ -1240,6 +1250,8 @@ HTMLInputElementImpl::HTMLInputElementImpl(DocumentPtr *doc, HTMLFormElementImpl
     m_size = 20;
     m_clicked = false;
     m_checked = false;
+    m_defaultChecked = false;
+    m_useDefaultChecked = true;
     m_indeterminate = false;
 
     m_haveType = false;
@@ -1345,7 +1357,7 @@ QString HTMLInputElementImpl::state( )
         return QLatin1String("."); // empty string, avoid restoring
     case CHECKBOX:
     case RADIO:
-        return QLatin1String(m_checked ? "on" : "off");
+        return QLatin1String(checked() ? "on" : "off");
     case TEXT:
         if (autoComplete() && value() != getAttribute(ATTR_VALUE) && getDocument()->view())
             getDocument()->view()->addFormCompletionItem(name().string(), value().string());
@@ -1408,7 +1420,9 @@ void HTMLInputElementImpl::parseAttribute(AttributeImpl *attr)
         }
         break;
     case ATTR_CHECKED:
-        // WebCore has m_defaultChecked and m_useDefaultChecked code here....
+	m_defaultChecked = attr->val();
+ 	if (m_useDefaultChecked) // We only need to setChanged if the form is looking
+	    setChanged();        // at the default checked state right now.
         break;
     case ATTR_MAXLENGTH:
     {
@@ -1416,7 +1430,7 @@ void HTMLInputElementImpl::parseAttribute(AttributeImpl *attr)
         if (!attr->val()) break;
         bool ok;
         const int ml = attr->val()->toInt(&ok);
-        if (ml > 0 && ml < 1024)
+        if (ml > 0 && ml < 32767)
             m_maxLen = ml;
         else if (ok && ml <= 0)
             m_maxLen = 0;
@@ -1480,7 +1494,7 @@ void HTMLInputElementImpl::attach()
                     nvalue += value[i];
             m_value = nvalue;
         }
-        m_checked = (getAttribute(ATTR_CHECKED) != 0);
+        m_defaultChecked = (getAttribute(ATTR_CHECKED) != 0);
         if ( m_type == IMAGE )
             addHTMLAlignment( getAttribute( ATTR_ALIGN ) );
         m_inited = true;
@@ -1674,8 +1688,18 @@ bool HTMLInputElementImpl::encoding(const QTextCodec* codec, khtml::encodingList
 
 void HTMLInputElementImpl::reset()
 {
-    setValue(getAttribute(ATTR_VALUE));
-    setChecked(getAttribute(ATTR_CHECKED) != 0);
+    if (m_type == FILE) {
+	// set directly to bypass security check. emptying the value
+	// should mean no risk.
+	if (!m_value.isEmpty()) {
+	    m_value = DOMString();
+	    setChanged();
+	}
+    } else {
+	setValue(getAttribute(ATTR_VALUE));
+    }
+    m_useDefaultChecked = true;
+    m_checked = m_defaultChecked;
     setIndeterminate(true);
 }
 
@@ -1684,7 +1708,8 @@ void HTMLInputElementImpl::setChecked(bool _checked)
     if (m_form && m_type == RADIO && _checked && !name().isEmpty())
         m_form->radioClicked(this);
 
-    if (m_checked == _checked) return;
+    if (checked() == _checked) return;
+    m_useDefaultChecked = false;
     m_checked = _checked;
 
 //     setIndeterminate(false);
@@ -1879,7 +1904,7 @@ void HTMLInputElementImpl::setSelectionRange(long start, long end)
 
 // -------------------------------------------------------------------------
 
-HTMLLabelElementImpl::HTMLLabelElementImpl(DocumentPtr *doc)
+HTMLLabelElementImpl::HTMLLabelElementImpl(DocumentImpl *doc)
     : HTMLGenericFormElementImpl(doc)
 {
 }
@@ -1935,7 +1960,7 @@ void HTMLLabelElementImpl::defaultEventHandler(EventImpl *evt)
 
 	if (act) {
 	    NodeImpl* const formNode=getFormElement();
-	    if (formNode) {
+	    if (formNode && evt->target() != formNode) {
 		getDocument()->setFocusNode(formNode);
 		if (formNode->id()==ID_INPUT)
 		    static_cast<DOM::HTMLInputElementImpl*>(formNode)->click();
@@ -1948,7 +1973,7 @@ void HTMLLabelElementImpl::defaultEventHandler(EventImpl *evt)
 
 // -------------------------------------------------------------------------
 
-HTMLLegendElementImpl::HTMLLegendElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f)
+HTMLLegendElementImpl::HTMLLegendElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f)
     : HTMLGenericFormElementImpl(doc, f)
 {
 }
@@ -1990,7 +2015,7 @@ void HTMLLegendElementImpl::parseAttribute(AttributeImpl *attr)
 
 // -------------------------------------------------------------------------
 
-HTMLSelectElementImpl::HTMLSelectElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f)
+HTMLSelectElementImpl::HTMLSelectElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f)
     : HTMLGenericFormElementImpl(doc, f)
 {
     m_multiple = false;
@@ -2234,20 +2259,18 @@ NodeImpl *HTMLSelectElementImpl::insertBefore ( NodeImpl *newChild, NodeImpl *re
     return result;
 }
 
-NodeImpl *HTMLSelectElementImpl::replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, int &exceptioncode )
+void HTMLSelectElementImpl::replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, int &exceptioncode )
 {
-    NodeImpl* const result = HTMLGenericFormElementImpl::replaceChild(newChild,oldChild, exceptioncode);
+    HTMLGenericFormElementImpl::replaceChild(newChild,oldChild, exceptioncode);
     if( !exceptioncode )
         setRecalcListItems();
-    return result;
 }
 
-NodeImpl *HTMLSelectElementImpl::removeChild ( NodeImpl *oldChild, int &exceptioncode )
+void HTMLSelectElementImpl::removeChild ( NodeImpl *oldChild, int &exceptioncode )
 {
-    NodeImpl* const result = HTMLGenericFormElementImpl::removeChild(oldChild, exceptioncode);
+    HTMLGenericFormElementImpl::removeChild(oldChild, exceptioncode);
     if( !exceptioncode )
         setRecalcListItems();
-    return result;
 }
 
 NodeImpl *HTMLSelectElementImpl::appendChild ( NodeImpl *newChild, int &exceptioncode )
@@ -2451,8 +2474,8 @@ void HTMLSelectElementImpl::reset()
                 anySelected = true;
         }
     }
-    // If there is no default selection, jump to first option.
-    if ( !anySelected ) {
+    // If this is a single-row SELECT and there is no default selection, jump to first option.
+    if ( !anySelected && m_size <= 1 ) {
         for (i = 0; i < itemsSize; ++i) {
             if (items[i]->id() == ID_OPTION) {
                 static_cast<HTMLOptionElementImpl*>(items[i])->setSelected(true);
@@ -2485,7 +2508,7 @@ void HTMLSelectElementImpl::notifyOptionSelected(HTMLOptionElementImpl *selected
 
 // -------------------------------------------------------------------------
 
-HTMLKeygenElementImpl::HTMLKeygenElementImpl(DocumentPtr* doc, HTMLFormElementImpl* f)
+HTMLKeygenElementImpl::HTMLKeygenElementImpl(DocumentImpl* doc, HTMLFormElementImpl* f)
     : HTMLSelectElementImpl(doc, f)
 {
     const QStringList keys = KSSLKeyGen::supportedKeySizes();
@@ -2494,7 +2517,7 @@ HTMLKeygenElementImpl::HTMLKeygenElementImpl(DocumentPtr* doc, HTMLFormElementIm
     for ( ; i != iEnd; ++i) {
         HTMLOptionElementImpl* const o = new HTMLOptionElementImpl(doc, form());
         addChild(o);
-        o->addChild(doc->document()->createTextNode(DOMString(*i).implementation()));
+        o->addChild(doc->createTextNode(DOMString(*i).implementation()));
     }
 }
 
@@ -2544,7 +2567,7 @@ NodeImpl::Id HTMLOptGroupElementImpl::id() const
 
 // -------------------------------------------------------------------------
 
-HTMLOptionElementImpl::HTMLOptionElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f)
+HTMLOptionElementImpl::HTMLOptionElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f)
     : HTMLGenericFormElementImpl(doc, f)
 {
     m_selected = false;
@@ -2652,7 +2675,7 @@ HTMLSelectElementImpl *HTMLOptionElementImpl::getSelect() const
 
 // -------------------------------------------------------------------------
 
-HTMLTextAreaElementImpl::HTMLTextAreaElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f)
+HTMLTextAreaElementImpl::HTMLTextAreaElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f)
     : HTMLGenericFormElementImpl(doc, f)
 {
     // DTD requires rows & cols be specified, but we will provide reasonable defaults
@@ -2951,7 +2974,7 @@ long HTMLTextAreaElementImpl::textLength()
 
 // -------------------------------------------------------------------------
 
-HTMLIsIndexElementImpl::HTMLIsIndexElementImpl(DocumentPtr *doc, HTMLFormElementImpl *f)
+HTMLIsIndexElementImpl::HTMLIsIndexElementImpl(DocumentImpl *doc, HTMLFormElementImpl *f)
     : HTMLInputElementImpl(doc, f)
 {
     m_type = TEXT;
