@@ -48,7 +48,7 @@
 using namespace khtml;
 using namespace DOM;
 
-HTMLTableElementImpl::HTMLTableElementImpl(DocumentPtr *doc)
+HTMLTableElementImpl::HTMLTableElementImpl(DocumentImpl *doc)
   : HTMLElementImpl(doc)
 {
     rules = None;
@@ -312,6 +312,7 @@ NodeImpl *HTMLTableElementImpl::appendChild(NodeImpl *child, int &exceptioncode)
 
 void HTMLTableElementImpl::handleChildAdd( NodeImpl *child )
 {
+    if (!child) return;
     switch(child->id()) {
     case ID_CAPTION:
         tCaption.childAdded(this, child);
@@ -330,6 +331,7 @@ void HTMLTableElementImpl::handleChildAdd( NodeImpl *child )
 
 void HTMLTableElementImpl::handleChildAppend( NodeImpl *child )
 {
+    if (!child) return;
     switch(child->id()) {
     case ID_CAPTION:
         tCaption.childAppended(child);
@@ -348,6 +350,7 @@ void HTMLTableElementImpl::handleChildAppend( NodeImpl *child )
 
 void HTMLTableElementImpl::handleChildRemove( NodeImpl *child )
 {
+    if (!child) return;
     switch(child->id()) {
     case ID_CAPTION:
         tCaption.childRemoved(this, child);
@@ -386,21 +389,18 @@ NodeImpl *HTMLTableElementImpl::insertBefore ( NodeImpl *newChild, NodeImpl *ref
     return retval;
 }
 
-NodeImpl *HTMLTableElementImpl::replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, int &exceptioncode )
+void HTMLTableElementImpl::replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, int &exceptioncode )
 {
     handleChildRemove( oldChild ); //Always safe.
-
-    NodeImpl* retval = HTMLElementImpl::replaceChild( newChild, oldChild, exceptioncode );
-    if (retval)
+    HTMLElementImpl::replaceChild( newChild, oldChild, exceptioncode );
+    if ( !exceptioncode )
         handleChildAdd( newChild );
-
-    return retval;
 }
 
-NodeImpl *HTMLTableElementImpl::removeChild ( NodeImpl *oldChild, int &exceptioncode )
+void HTMLTableElementImpl::removeChild ( NodeImpl *oldChild, int &exceptioncode )
 {
     handleChildRemove( oldChild );
-    return HTMLElementImpl::removeChild( oldChild, exceptioncode);
+    HTMLElementImpl::removeChild( oldChild, exceptioncode);
 }
 
 void HTMLTableElementImpl::parseAttribute(AttributeImpl *attr)
@@ -661,7 +661,7 @@ void HTMLTablePartElementImpl::parseAttribute(AttributeImpl *attr)
 
 // -------------------------------------------------------------------------
 
-HTMLTableSectionElementImpl::HTMLTableSectionElementImpl(DocumentPtr *doc,
+HTMLTableSectionElementImpl::HTMLTableSectionElementImpl(DocumentImpl *doc,
                                                          ushort tagid, bool implicit)
     : HTMLTablePartElementImpl(doc)
 {
@@ -833,7 +833,7 @@ void HTMLTableRowElementImpl::deleteCell( long index, int &exceptioncode )
 
 // -------------------------------------------------------------------------
 
-HTMLTableCellElementImpl::HTMLTableCellElementImpl(DocumentPtr *doc, int tag)
+HTMLTableCellElementImpl::HTMLTableCellElementImpl(DocumentImpl *doc, int tag)
   : HTMLTablePartElementImpl(doc)
 {
   _col = -1;
@@ -868,16 +868,18 @@ void HTMLTableCellElementImpl::parseAttribute(AttributeImpl *attr)
         //addCSSLength(CSS_PROP_BORDER_WIDTH, attr->value());
         break;
     case ATTR_ROWSPAN:
-        // ###
         rSpan = attr->val() ? attr->val()->toInt() : 1;
         // limit this to something not causing an overflow with short int
         if(rSpan < 1 || rSpan > 1024) rSpan = 1;
+        if (renderer())
+            renderer()->updateFromElement();                    
         break;
     case ATTR_COLSPAN:
-        // ###
         cSpan = attr->val() ? attr->val()->toInt() : 1;
         // limit this to something not causing an overflow with short int
         if(cSpan < 1 || cSpan > 1024) cSpan = 1;
+        if (renderer())
+            renderer()->updateFromElement();        
         break;
     case ATTR_NOWRAP:
         if (attr->val() != 0)
@@ -930,11 +932,11 @@ void HTMLTableCellElementImpl::attach()
 
 // -------------------------------------------------------------------------
 
-HTMLTableColElementImpl::HTMLTableColElementImpl(DocumentPtr *doc, ushort i)
+HTMLTableColElementImpl::HTMLTableColElementImpl(DocumentImpl *doc, ushort i)
     : HTMLTablePartElementImpl(doc)
 {
     _id = i;
-    _span = (_id == ID_COLGROUP ? 0 : 1);
+    _span = 1;
 }
 
 NodeImpl::Id HTMLTableColElementImpl::id() const
@@ -949,6 +951,7 @@ void HTMLTableColElementImpl::parseAttribute(AttributeImpl *attr)
     {
     case ATTR_SPAN:
         _span = attr->val() ? attr->val()->toInt() : 1;
+        if (_span < 1) _span = 1;
         break;
     case ATTR_WIDTH:
         if (!attr->value().isEmpty())
