@@ -126,7 +126,7 @@ public:
  *
  */
 
-KHTMLParser::KHTMLParser( KHTMLView *_parent, DocumentPtr *doc)
+KHTMLParser::KHTMLParser( KHTMLView *_parent, DocumentImpl *doc)
 {
     //kDebug( 6035 ) << "parser constructor" << endl;
 #if SPEED_DEBUG > 0
@@ -135,7 +135,6 @@ KHTMLParser::KHTMLParser( KHTMLView *_parent, DocumentPtr *doc)
 
     HTMLWidget    = _parent;
     document      = doc;
-    document->ref();
 
     blockStack = 0;
     current = 0;
@@ -146,11 +145,10 @@ KHTMLParser::KHTMLParser( KHTMLView *_parent, DocumentPtr *doc)
     reset();
 }
 
-KHTMLParser::KHTMLParser( DOM::DocumentFragmentImpl *i, DocumentPtr *doc )
+KHTMLParser::KHTMLParser( DOM::DocumentFragmentImpl *i, DocumentImpl *doc )
 {
     HTMLWidget = 0;
     document = doc;
-    document->ref();
 
     forbiddenTag = new ushort[ID_CLOSE_TAG+1];
 
@@ -174,15 +172,13 @@ KHTMLParser::~KHTMLParser()
 
     if (current) current->deref();
 
-    document->deref();
-
     delete [] forbiddenTag;
     delete isindex;
 }
 
 void KHTMLParser::reset()
 {
-    setCurrent ( document->document() );
+    setCurrent ( document );
 
     freeBlock();
 
@@ -231,7 +227,7 @@ void KHTMLParser::parseToken(Token *t)
 
     // holy shit. apparently some sites use </br> instead of <br>
     // be compatible with IE and NS
-    if(t->tid == ID_BR+ID_CLOSE_TAG && document->document()->inCompatMode())
+    if(t->tid == ID_BR+ID_CLOSE_TAG && document->inCompatMode())
         t->tid -= ID_CLOSE_TAG;
 
     if(t->tid > ID_CLOSE_TAG)
@@ -330,6 +326,11 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
 #ifdef PARSER_DEBUG
         kDebug( 6035 ) << "added " << n->nodeName().string() << " to " << tmp->nodeName().string() << ", new current=" << newNode->nodeName().string() << endl;
 #endif
+        // We allow TABLE > FORM in dtd.cpp, but do not allow the form have children in this case
+        if (current->id() == ID_TABLE && id == ID_FORM) {
+            flat = true;
+            static_cast<HTMLFormElementImpl*>(n)->setMalformed(true);
+        }
 
 	// don't push elements without end tag on the stack
         if(tagPriority[id] != 0 && !flat) {
@@ -345,8 +346,8 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
             if(!n->attached() && HTMLWidget)
                 n->attach();
             if (n->maintainsState()) {
-                document->document()->registerMaintainsState(n);
-                QString state(document->document()->nextState());
+                document->registerMaintainsState(n);
+                QString state(document->nextState());
                 if (!state.isNull()) n->restoreState(state);
             }
             n->close();
@@ -354,6 +355,7 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
 	    if(n->isInline()) m_inline = true;
         }
 
+
 #if SPEED_DEBUG < 1
         if(tagPriority[id] == 0 && n->renderer())
             n->renderer()->calcMinMaxWidth();
@@ -367,12 +369,11 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
         HTMLElementImpl *e;
         bool handled = false;
 
-        // first switch on current element for a elements with optional end-tag
+        // first switch on current element for elements with optional end-tag and inline-only content
         switch(current->id())
         {
         case ID_P:
-        case ID_DD:
-        case ID_LI:
+        case ID_DT:
             if(!n->isInline())
             {
                 popBlock(current->id());
@@ -396,13 +397,14 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
                 return insertNode(n);
             }
             break;
-        case ID_COMMENT:
-            break;
         case ID_HEAD:
             // ### allow not having <HTML> in at all, as per HTML spec
             if (!current->isDocumentNode() && current->id() != ID_HTML )
                 return false;
             break;
+        case ID_COMMENT:
+            if( head )
+                break;
         case ID_META:
         case ID_LINK:
         case ID_ISINDEX:
@@ -676,7 +678,11 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
             {
                 NodeImpl *node = current;
                 NodeImpl *parent = node->parentNode();
-
+                // A script may have removed the current node's parent from the DOM
+                // http://bugzilla.opendarwin.org/show_bug.cgi?id=7137
+                // FIXME: we should do real recovery here and re-parent with the correct node.
+                if (!parent)
+                    return false;
                 NodeImpl *parentparent = parent->parentNode();
 
                 if (n->isTextNode() ||
@@ -690,6 +696,8 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
                     node = (node->id() == ID_TABLE) ? node :
                            ((node->id() == ID_TR ) ? parentparent : parent);
                     NodeImpl *parent = node->parentNode();
+                    if (!parent)
+                        return false;
                     int exceptioncode = 0;
 #ifdef PARSER_DEBUG
                     kDebug( 6035 ) << "calling insertBefore(" << n->nodeName().string() << "," << node->nodeName().string() << ")" << endl;
@@ -792,7 +800,7 @@ bool KHTMLParser::insertNode(NodeImpl *n, bool flat)
         default:
             if(current->isDocumentNode())
             {
-                if(current->firstChild() == 0) {
+                if(current->firstChild() == 0 || !current->firstChild()->isHTMLElement()) {
                     e = new HTMLHtmlElementImpl(document);
                     insertNode(e);
                     handled = true;
@@ -876,8 +884,8 @@ NodeImpl *KHTMLParser::getElement(Token* t)
             // we can't implement that behavior now because it could cause too many
             // regressions and the headaches are not worth the work as long as there is
             // no site actually relying on that detail (Dirk)
-            if (static_cast<HTMLDocumentImpl*>(document->document())->body())
-                static_cast<HTMLDocumentImpl*>(document->document())->body()
+            if (static_cast<HTMLDocumentImpl*>(document)->body())
+                static_cast<HTMLDocumentImpl*>(document)->body()
                     ->addCSSProperty(CSS_PROP_DISPLAY, CSS_VAL_NONE);
             inBody = false;
         }
@@ -1190,9 +1198,7 @@ NodeImpl *KHTMLParser::getElement(Token* t)
         n = new TextImpl(document, t->text);
         break;
     case ID_COMMENT:
-#ifdef COMMENTS_IN_DOM
         n = new CommentImpl(document, t->text);
-#endif
         break;
     default:
         kDebug( 6035 ) << "Unknown tag " << t->tid << "!" << endl;
@@ -1306,18 +1312,21 @@ bool KHTMLParser::isAffectedByResidualStyle(int _id)
 
 void KHTMLParser::handleResidualStyleCloseTagAcrossBlocks(HTMLStackElem* elem)
 {
-    // Find the element that crosses over to a higher level.   For now, if there is more than
-    // one, we will just give up and not attempt any sort of correction.  It's highly unlikely that
-    // there will be more than one, since <p> tags aren't allowed to be nested.
+    // Find the element that crosses over to a higher level.
+    // ### For now, if there is more than one, we will only make sure we close the residual style.
     int exceptionCode = 0;
     HTMLStackElem* curr = blockStack;
     HTMLStackElem* maxElem = 0;
+    HTMLStackElem* endElem = 0;
     HTMLStackElem* prev = 0;
     HTMLStackElem* prevMaxElem = 0;
+    bool advancedResidual = false; // ### if set we only close the residual style
     while (curr && curr != elem) {
         if (curr->level > elem->level) {
-            if (maxElem)
-                return;
+            if (!isAffectedByResidualStyle(curr->id)) return;
+            if (maxElem) advancedResidual = true;
+            else
+                endElem = curr;
             maxElem = curr;
             prevMaxElem = prev;
         }
@@ -1326,7 +1335,7 @@ void KHTMLParser::handleResidualStyleCloseTagAcrossBlocks(HTMLStackElem* elem)
         curr = curr->next;
     }
 
-    if (!curr || !maxElem || !isAffectedByResidualStyle(maxElem->id)) return;
+    if (!curr || !maxElem ) return;
 
     NodeImpl* residualElem = prev->node;
     NodeImpl* blockElem = prevMaxElem ? prevMaxElem->node : current;
@@ -1339,7 +1348,7 @@ void KHTMLParser::handleResidualStyleCloseTagAcrossBlocks(HTMLStackElem* elem)
     if (!parentElem->childAllowed(blockElem))
         return;
 
-    if (maxElem->node->parentNode() != elem->node) {
+    if (maxElem->node->parentNode() != elem->node && !advancedResidual) {
         // Walk the stack and remove any elements that aren't residual style tags.  These
         // are basically just being closed up.  Example:
         // <font><span>Moo<p>Goo</font></p>.
@@ -1373,6 +1382,8 @@ void KHTMLParser::handleResidualStyleCloseTagAcrossBlocks(HTMLStackElem* elem)
             if (isResidualStyleTag(currElem->node->id())) {
                 // Create a clone of this element.
                 currNode = currElem->node->cloneNode(false);
+                currElem->node->close();
+                removeForbidden(currElem->id, forbiddenTag);
 
                 // Change the stack element's node to point to the clone.
                 currElem->setNode(currNode);
@@ -1402,8 +1413,10 @@ void KHTMLParser::handleResidualStyleCloseTagAcrossBlocks(HTMLStackElem* elem)
     // The end result will be: <b>...</b><p><b>Foo</b>Goo</p>
     //
     // Step 1: Remove |blockElem| from its parent, doing a batch detach of all the kids.
+    SharedPtr<NodeImpl> guard(blockElem);
     blockElem->parentNode()->removeChild(blockElem, exceptionCode);
 
+    if (!advancedResidual) {
     // Step 2: Clone |residualElem|.
     NodeImpl* newNode = residualElem->cloneNode(false); // Shallow clone. We don't pick up the same kids.
 
@@ -1413,6 +1426,7 @@ void KHTMLParser::handleResidualStyleCloseTagAcrossBlocks(HTMLStackElem* elem)
     NodeImpl* currNode = blockElem->firstChild();
     while (currNode) {
         NodeImpl* nextNode = currNode->nextSibling();
+        SharedPtr<NodeImpl> guard(currNode); //Protect from deletion while moving
         blockElem->removeChild(currNode, exceptionCode);
         newNode->appendChild(currNode, exceptionCode);
         currNode = nextNode;
@@ -1429,6 +1443,7 @@ void KHTMLParser::handleResidualStyleCloseTagAcrossBlocks(HTMLStackElem* elem)
     // Step 4: Place |newNode| under |blockElem|.  |blockElem| is still out of the document, so no
     // attachment can occur yet.
     blockElem->appendChild(newNode, exceptionCode);
+    }
 
     // Step 5: Reparent |blockElem|.  Now the full attachment of the fixed up tree takes place.
     parentElem->appendChild(blockElem, exceptionCode);
@@ -1450,7 +1465,7 @@ void KHTMLParser::handleResidualStyleCloseTagAcrossBlocks(HTMLStackElem* elem)
     // In the above example, Goo should stay italic.
     curr = blockStack;
     HTMLStackElem* residualStyleStack = 0;
-    while (curr && curr != maxElem) {
+    while (curr && curr != endElem) {
         // We will actually schedule this tag for reopening
         // after we complete the close of this entire block.
         NodeImpl* currNode = current;
@@ -1623,9 +1638,9 @@ void KHTMLParser::popOneBlock(bool delBlock)
 
 #if SPEED_DEBUG < 1
     if((Elem->node != current)) {
-        if (current->maintainsState() && document->document()){
-            document->document()->registerMaintainsState(current);
-            QString state(document->document()->nextState());
+        if (current->maintainsState() && document){
+            document->registerMaintainsState(current);
+            QString state(document->nextState());
             if (!state.isNull()) current->restoreState(state);
         }
         current->close();
