@@ -123,7 +123,7 @@ static const char titleEnd [] = "</title";
 #endif
 // ----------------------------------------------------------------------------
 
-HTMLTokenizer::HTMLTokenizer(DOM::DocumentPtr *_doc, KHTMLView *_view)
+HTMLTokenizer::HTMLTokenizer(DOM::DocumentImpl *_doc, KHTMLView *_view)
 {
     view = _view;
     buffer = 0;
@@ -138,7 +138,7 @@ HTMLTokenizer::HTMLTokenizer(DOM::DocumentPtr *_doc, KHTMLView *_view)
     reset();
 }
 
-HTMLTokenizer::HTMLTokenizer(DOM::DocumentPtr *_doc, DOM::DocumentFragmentImpl *i)
+HTMLTokenizer::HTMLTokenizer(DOM::DocumentImpl *_doc, DOM::DocumentFragmentImpl *i)
 {
     view = 0;
     buffer = 0;
@@ -316,7 +316,7 @@ void HTMLTokenizer::parseSpecial(TokenizerString &src)
     while ( !src.isEmpty() ) {
         checkScriptBuffer();
         unsigned char ch = src->toLatin1();
-        if ( !scriptCodeResync && !brokenComments && !textarea && !xmp && !title && ch == '-' && scriptCodeSize >= 3 && !src.escaped() && QString::fromRawData( scriptCode+scriptCodeSize-3, 3 ) == "<!-" ) {
+        if ( !scriptCodeResync && !brokenComments && !textarea && !xmp && ch == '-' && scriptCodeSize >= 3 && !src.escaped() && QString::fromRawData( scriptCode+scriptCodeSize-3, 3 ) == "<!-" ) {
             comment = true;
             scriptCode[ scriptCodeSize++ ] = ch;
             ++src;
@@ -388,13 +388,14 @@ void HTMLTokenizer::scriptHandler()
 
     // Scripts following a frameset element should not be executed or even loaded in the case of extern scripts.
     bool followingFrameset = (parser->doc()->body() && parser->doc()->body()->id() == ID_FRAMESET);
+    bool effectiveScript = !parser->skipMode() && !followingFrameset;
     bool deferredScript = false;
 
-    if ( !parser->skipMode() && !followingFrameset) {
+    if ( effectiveScript ) {
         CachedScript* cs = 0;
 
         // forget what we just got, load from src url instead
-        if ( !currentScriptSrc.isEmpty() &&
+        if ( !currentScriptSrc.isEmpty() && javascript &&
              (cs = parser->doc()->docLoader()->requestScript(currentScriptSrc, scriptSrcCharset) )) {
             cachedScript.enqueue(cs);
         }
@@ -413,13 +414,16 @@ void HTMLTokenizer::scriptHandler()
             setSrc(TokenizerString());
             scriptCodeSize = scriptCodeResync = 0;
             scriptExecution( exScript, QString(), tagStartLineno /*scriptStartLineno*/ );
+        } else {
+            // script was filtered or disallowed
+            effectiveScript = false;
         }
     }
 
     script = false;
     scriptCodeSize = scriptCodeResync = 0;
 
-    if (parser->skipMode() || followingFrameset)
+    if ( !effectiveScript )
         return;
 
     if ( !m_executingScript && cachedScript.isEmpty() ) {
@@ -427,8 +431,8 @@ void HTMLTokenizer::scriptHandler()
     } else if ( cachedScript.isEmpty() ) {
         write( pendingQueue.pop(), false );
     } else if ( !deferredScript && pendingQueue.count() > 1) {
-       TokenizerString t = pendingQueue.pop();
-       pendingQueue.top().prepend( t );
+        TokenizerString t = pendingQueue.pop();
+        pendingQueue.top().prepend( t );
     }
 }
 
@@ -491,17 +495,15 @@ void HTMLTokenizer::parseComment(TokenizerString &src)
 
             if (canClose || handleBrokenComments || scriptEnd ){
                 ++src;
-                if ( !( script || xmp || textarea || style) ) {
-#ifdef COMMENTS_IN_DOM
+                if ( !( title || script || xmp || textarea || style) ) {
                     checkScriptBuffer();
                     scriptCode[ scriptCodeSize ] = 0;
                     scriptCode[ scriptCodeSize + 1 ] = 0;
                     currToken.tid = ID_COMMENT;
-                    processListing(DOMStringIt(scriptCode, scriptCodeSize - 2));
+                    processListing(TokenizerString(scriptCode, scriptCodeSize - 2));
                     processToken();
                     currToken.tid = ID_COMMENT + ID_CLOSE_TAG;
                     processToken();
-#endif
                     scriptCodeSize = 0;
                 }
                 comment = false;
@@ -597,6 +599,7 @@ void HTMLTokenizer::parseEntity(TokenizerString &src, QChar *&dest, bool start)
     if( start )
     {
         cBufferPos = 0;
+        entityLen = 0;
         Entity = SearchEntity;
     }
 
@@ -692,8 +695,8 @@ void HTMLTokenizer::parseEntity(TokenizerString &src, QChar *&dest, bool start)
                 if ( tag == NoTag ) {
                     const entity* e = kde_findEntity(cBuffer, cBufferPos);
                     if ( e && e->code < 256 ) {
-                        Entity = SearchSemicolon;
-                        break;
+                        EntityChar = e->code;
+                        entityLen = cBufferPos;
                     }
                 }
             }
@@ -701,8 +704,12 @@ void HTMLTokenizer::parseEntity(TokenizerString &src, QChar *&dest, bool start)
             if(Entity == SearchSemicolon) {
                 if(cBufferPos > 1) {
                     const entity *e = kde_findEntity(cBuffer, cBufferPos);
-                    if(e && ( e->code < 256 || *src == ';' ))
+                    // IE only accepts unterminated entities < 256,
+                    // Gecko accepts them all, but only outside tags
+                    if(e && ( tag == NoTag || e->code < 256 || *src == ';' )) {
                         EntityChar = e->code;
+                        entityLen = cBufferPos;
+                    }
                 }
             }
             break;
@@ -718,7 +725,10 @@ void HTMLTokenizer::parseEntity(TokenizerString &src, QChar *&dest, bool start)
 
             if ( !EntityChar.isNull() ) {
                 checkBuffer();
-                // Just insert it
+                if (entityLen > 0 && entityLen < cBufferPos) {
+                    int rem = cBufferPos - entityLen;
+                    src.prepend( TokenizerString(QString::fromAscii(cBuffer+entityLen, rem)) );
+                }
                 src.push( EntityChar );
             } else {
 #ifdef TOKEN_DEBUG
@@ -730,7 +740,6 @@ void HTMLTokenizer::parseEntity(TokenizerString &src, QChar *&dest, bool start)
                 for(unsigned int i = 0; i < cBufferPos; i++)
                     dest[i] = cBuffer[i];
                 dest += cBufferPos;
-                Entity = NoEntity;
                 if (pre)
                     prePos += cBufferPos+1;
             }
@@ -900,8 +909,17 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                         unsigned int a;
                         cBuffer[cBufferPos] = '\0';
                         a = khtml::getAttrID(cBuffer, cBufferPos);
-                        if ( !a )
-                            attrName = QLatin1String(QByteArray(cBuffer, cBufferPos+1).data());
+
+                        if ( !a ) {
+                            // did we just get /> or e.g checked/>
+                            if (curchar == '>' && cBufferPos >=1 && cBuffer[cBufferPos-1] == '/') {
+                                currToken.flat = true;
+                                if (cBufferPos>1)
+                                    a = khtml::getAttrID(cBuffer, cBufferPos-1);
+                            }
+                            if (!a)
+                                attrName = QLatin1String(QByteArray(cBuffer, cBufferPos+1).data());
+                        }
 
                         dest = buffer;
                         *dest++ = a;
@@ -911,9 +929,6 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                         else
                             kDebug( 6036 ) << "Known attribute: " << QByteArray(cBuffer, cBufferPos+1).data() << endl;
 #endif
-                        // did we just get />
-                        if (!a && cBufferPos == 1 && *cBuffer == '/' && curchar == '>')
-                            currToken.flat = true;
 
                         tag = SearchEqual;
                         break;
@@ -957,7 +972,7 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                     }
                     else {
                         DOMString v("");
-                        currToken.addAttribute(parser->docPtr()->document(), buffer, attrName, v);
+                        currToken.addAttribute(parser->docPtr(), buffer, attrName, v);
                         dest = buffer;
                         tag = SearchAttribute;
                     }
@@ -1012,7 +1027,7 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                         while(dest > buffer+1 && (*(dest-1) == '\n' || *(dest-1) == '\r'))
                             dest--; // remove trailing newlines
                         DOMString v(buffer+1, dest-buffer-1);
-                        currToken.addAttribute(parser->docPtr()->document(), buffer, attrName, v);
+                        currToken.addAttribute(parser->docPtr(), buffer, attrName, v);
 
                         dest = buffer;
                         tag = SearchAttribute;
@@ -1048,7 +1063,7 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                     if ( curchar <= ' ' || curchar == '>' )
                     {
                         DOMString v(buffer+1, dest-buffer-1);
-                        currToken.addAttribute(parser->docPtr()->document(), buffer, attrName, v);
+                        currToken.addAttribute(parser->docPtr(), buffer, attrName, v);
                         dest = buffer;
                         tag = SearchAttribute;
                         break;
@@ -1144,6 +1159,8 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                         type.compare("text/ecmascript") != 0 &&
                         type.compare("text/livescript") != 0 &&
 			type.compare("application/x-javascript") != 0 &&
+			type.compare("application/x-ecmascript") != 0 &&
+			type.compare("application/javascript") != 0 &&
 			type.compare("application/ecmascript") != 0 )
                         javascript = false;
                 } else if( a ) {
@@ -1317,7 +1334,10 @@ void HTMLTokenizer::write( const TokenizerString &str, bool appendData )
         return;
     }
 
-    setSrc(str);
+    if (!src.isEmpty())
+        src.append(str);
+    else
+        setSrc(str);
     m_abort = false;
 
 //     if (Entity)
@@ -1633,7 +1653,8 @@ void HTMLTokenizer::processToken()
 #endif
         currToken.text = new DOMStringImpl( buffer, dest - buffer );
         currToken.text->ref();
-        currToken.tid = ID_TEXT;
+        if (currToken.tid != ID_COMMENT)
+            currToken.tid = ID_TEXT;
     }
     else if(!currToken.tid) {
         currToken.reset();
