@@ -49,6 +49,7 @@
 #include <qpushbutton.h>
 #include <qlayout.h>
 #include <qregexp.h>
+#include <Q3CString>
 
 #include <stdlib.h>
 #include <assert.h>
@@ -293,12 +294,12 @@ void KJavaAppletServer::setupJava( KJavaProcess *p )
     //check for http proxies...
     if( KProtocolManager::useProxy() )
     {
-        // only proxyForURL honors automatic proxy scripts
+        // only proxyForUrl honors automatic proxy scripts
         // we do not know the applet url here so we just use a dummy url
         // this is a workaround for now
         // FIXME
         const KUrl dummyURL( "http://www.kde.org/" );
-        const QString httpProxy = KProtocolManager::proxyForURL(dummyURL);
+        const QString httpProxy = KProtocolManager::proxyForUrl(dummyURL);
         kDebug(6100) << "httpProxy is " << httpProxy << endl;
 
         const KUrl url( httpProxy );
