@@ -49,14 +49,17 @@
 
 K_EXPORT_COMPONENT_FACTORY (kjavaappletviewer, KJavaAppletViewerFactory)
 
-KInstance *KJavaAppletViewerFactory::s_instance = 0;
+KComponentData *KJavaAppletViewerFactory::s_componentData = 0;
+KIconLoader *KJavaAppletViewerFactory::s_iconLoader = 0;
 
 KJavaAppletViewerFactory::KJavaAppletViewerFactory () {
-    s_instance = new KInstance ("kjava");
+    s_componentData = new KComponentData("kjava");
+    s_iconLoader = new KIconLoader(s_componentData->componentName(), s_componentData->dirs());
 }
 
 KJavaAppletViewerFactory::~KJavaAppletViewerFactory () {
-    delete s_instance;
+    delete s_iconLoader;
+    delete s_componentData;
 }
 
 KParts::Part *KJavaAppletViewerFactory::createPartObject
@@ -158,6 +161,7 @@ AppletParameterDialog::AppletParameterDialog (KJavaAppletWidget * parent)
         table->setItem (count, 1, tit);
     }
     setMainWidget (table);
+    connect(this,SIGNAL(closeClicked()),this,SLOT(slotClose()));
 }
 
 void AppletParameterDialog::slotClose () {
@@ -207,7 +211,7 @@ void CoverWidget::resizeEvent (QResizeEvent * e) {
 class StatusBarIcon : public QLabel {
 public:
     StatusBarIcon (QWidget * parent) : QLabel (parent) {
-        setPixmap (SmallIcon (QString ("java"), KJavaAppletViewerFactory::instance ()));
+        setPixmap ( KJavaAppletViewerFactory::iconLoader()->loadIcon("java", K3Icon::Small) );
     }
 protected:
     void mousePressEvent (QMouseEvent *) {
@@ -311,7 +315,7 @@ KJavaAppletViewer::KJavaAppletViewer (QWidget * wparent,
     // check codebase first
     const KUrl kbaseURL( baseurl );
     const KUrl newURL(kbaseURL, codebase);
-    if (KAuthorized::authorizeURLAction("redirect", KUrl(baseurl), newURL))
+    if (KAuthorized::authorizeUrlAction("redirect", KUrl(baseurl), newURL))
         applet->setCodeBase (newURL.url());
     applet->setAppletClass (classname);
     KJavaAppletContext* const cxt = serverMaintainer->getContext (parent, baseurl);
@@ -350,7 +354,7 @@ KJavaAppletViewer::KJavaAppletViewer (QWidget * wparent,
     if (wparent)
         wparent->topLevelWidget ()->installEventFilter (this);
 
-    setInstance (KJavaAppletViewerFactory::instance ());
+    setComponentData (KJavaAppletViewerFactory::componentData());
     KParts::Part::setWidget (m_view);
 
     connect (applet->getContext(), SIGNAL(appletLoaded()), this, SLOT(appletLoaded()));
@@ -388,7 +392,7 @@ KJavaAppletViewer::~KJavaAppletViewer () {
     }
 }
 
-bool KJavaAppletViewer::openURL (const KUrl & url) {
+bool KJavaAppletViewer::openUrl (const KUrl & url) {
     if (!m_view) return false;
     m_closed = false;
     KJavaAppletWidget* const w = m_view->appletWidget ();
@@ -422,8 +426,8 @@ bool KJavaAppletViewer::openURL (const KUrl & url) {
     return url.isValid ();
 }
 
-bool KJavaAppletViewer::closeURL () {
-    kDebug(6100) << "closeURL" << endl;
+bool KJavaAppletViewer::closeUrl () {
+    kDebug(6100) << "closeUrl" << endl;
     m_closed = true;
     KJavaApplet* const applet = m_view->appletWidget ()->applet ();
     if (applet->isCreated ())
@@ -472,7 +476,7 @@ KJavaAppletViewerBrowserExtension::KJavaAppletViewerBrowserExtension (KJavaApple
 }
 
 void KJavaAppletViewerBrowserExtension::urlChanged (const QString & url) {
-    emit setLocationBarURL (url);
+    emit setLocationBarUrl (url);
 }
 
 void KJavaAppletViewerBrowserExtension::setLoadingProgress (int percentage) {
@@ -524,7 +528,7 @@ void KJavaAppletViewerBrowserExtension::showDocument (const QString & doc,
     const KUrl url (doc);
     KParts::URLArgs args;
     args.frameName = frame;
-    emit openURLRequest (url, args);
+    emit openUrlRequest (url, args);
 }
 
 //-----------------------------------------------------------------------------
