@@ -38,6 +38,7 @@
 #include <qclipboard.h>
 #include <qfileinfo.h>
 #include <qmenu.h>
+#include <qurl.h>
 #include <qmetaobject.h>
 
 #include <kdebug.h>
@@ -58,9 +59,9 @@
 #include <kiconloader.h>
 #include <kdesktopfile.h>
 #include <kinputdialog.h>
-#include <ktempfile.h>
+#include <ktemporaryfile.h>
 #include "khtml_factory.h"
-#include <kstdaction.h>
+#include <kstandardaction.h>
 #include <kactioncollection.h>
 #include <kactionmenu.h>
 
@@ -203,7 +204,7 @@ void KHTMLPartBrowserExtension::cut()
     if ( m_editableFormWidget->inherits( "QLineEdit" ) )
         static_cast<QLineEdit *>( &(*m_editableFormWidget) )->cut();
     else if ( m_editableFormWidget->inherits( "QTextEdit" ) )
-        static_cast<Q3TextEdit *>( &(*m_editableFormWidget) )->cut();
+        static_cast<QTextEdit *>( &(*m_editableFormWidget) )->cut();
 }
 
 void KHTMLPartBrowserExtension::copy()
@@ -253,7 +254,7 @@ void KHTMLPartBrowserExtension::copy()
         if ( m_editableFormWidget->inherits( "QLineEdit" ) )
             static_cast<QLineEdit *>( &(*m_editableFormWidget) )->copy();
         else if ( m_editableFormWidget->inherits( "QTextEdit" ) )
-            static_cast<Q3TextEdit *>( &(*m_editableFormWidget) )->copy();
+            static_cast<QTextEdit *>( &(*m_editableFormWidget) )->copy();
     }
 }
 
@@ -262,21 +263,22 @@ void KHTMLPartBrowserExtension::searchProvider()
     // action name is of form "previewProvider[<searchproviderprefix>:]"
     const QString searchProviderPrefix = QString( sender()->objectName() ).mid( 14 );
 
-    KURIFilterData data;
+    KUriFilterData data;
     QStringList list;
     data.setData( searchProviderPrefix + m_part->selectedText() );
     list << "kurisearchfilter" << "kuriikwsfilter";
 
-    if( !KURIFilter::self()->filterURI(data, list) )
+    if( !KUriFilter::self()->filterUri(data, list) )
     {
         KDesktopFile file("searchproviders/google.desktop", true, "services");
-        data.setData(file.readEntry("Query").replace("\\{@}", m_part->selectedText()));
+        QString encodedSearchTerm = QUrl::toPercentEncoding(m_part->selectedText());
+	data.setData(file.readEntry("Query").replace("\\{@}", encodedSearchTerm));
     }
 
     KParts::URLArgs args;
     args.frameName = "_blank";
 
-    emit m_part->browserExtension()->openURLRequest( data.uri(), args );
+    emit m_part->browserExtension()->openUrlRequest( data.uri(), args );
 }
 
 void KHTMLPartBrowserExtension::openSelection()
@@ -284,7 +286,7 @@ void KHTMLPartBrowserExtension::openSelection()
     KParts::URLArgs args;
     args.frameName = "_blank";
 
-    emit m_part->browserExtension()->openURLRequest( m_part->selectedText(), args );
+    emit m_part->browserExtension()->openUrlRequest( m_part->selectedText(), args );
 }
 
 void KHTMLPartBrowserExtension::paste()
@@ -301,7 +303,7 @@ void KHTMLPartBrowserExtension::paste()
     if ( m_editableFormWidget->inherits( "QLineEdit" ) )
         static_cast<QLineEdit *>( &(*m_editableFormWidget) )->paste();
     else if ( m_editableFormWidget->inherits( "QTextEdit" ) )
-        static_cast<Q3TextEdit *>( &(*m_editableFormWidget) )->paste();
+        static_cast<QTextEdit *>( &(*m_editableFormWidget) )->paste();
 }
 
 void KHTMLPartBrowserExtension::callExtensionProxyMethod( const char *method )
@@ -339,8 +341,8 @@ void KHTMLPartBrowserExtension::updateEditActions()
     if( m_editableFormWidget) {
         if ( qobject_cast<QLineEdit*>(m_editableFormWidget))
             hasSelection = static_cast<QLineEdit *>( &(*m_editableFormWidget) )->hasSelectedText();
-        else if(qobject_cast<Q3TextEdit*>(m_editableFormWidget))
-            hasSelection = static_cast<Q3TextEdit *>( &(*m_editableFormWidget) )->hasSelectedText();
+        else if(qobject_cast<QTextEdit*>(m_editableFormWidget))
+            hasSelection = static_cast<QTextEdit *>( &(*m_editableFormWidget) )->hasSelectedText();
     }
 
     enableAction( "copy", hasSelection );
@@ -377,10 +379,10 @@ void KHTMLPartBrowserExtension::print()
 
 void KHTMLPartBrowserExtension::disableScrolling()
 {
-  Q3ScrollView *scrollView = m_part->view();
-  if (scrollView) {
-    scrollView->setVScrollBarMode(Q3ScrollView::AlwaysOff);
-    scrollView->setHScrollBarMode(Q3ScrollView::AlwaysOff);
+  QScrollArea *scrollArea = m_part->view();
+  if (scrollArea) {
+    scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
   }
 }
 
@@ -402,7 +404,7 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const QString &doc,
   d->m_url = url;
   bool isImage = false;
   bool hasSelection = khtml->hasSelection();
-  setInstance( khtml->instance() );
+  setComponentData( khtml->componentData() );
 
   DOM::Element e;
   e = khtml->nodeUnderMouse();
@@ -422,10 +424,12 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const QString &doc,
 
   if (hasSelection)
   {
-      KAction* copyAction = KStdAction::copy( d->m_khtml->browserExtension(), SLOT( copy() ), actionCollection(), "copy" );
+      QAction* copyAction = actionCollection()->addAction( KStandardAction::Copy, "copy",
+                                                           d->m_khtml->browserExtension(), SLOT( copy() ) );
+
       copyAction->setText(i18n("&Copy Text"));
       copyAction->setEnabled(d->m_khtml->browserExtension()->isActionEnabled( "copy" ));
-      actionCollection()->insert( khtml->actionCollection()->action( "selectAll" ) );
+      actionCollection()->addAction( "selectAll", khtml->actionCollection()->action( "selectAll" ) );
 
 
       // Fill search provider entries
@@ -436,6 +440,7 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const QString &doc,
 
       // search text
       QString selectedText = khtml->selectedText();
+      selectedText.replace("&", "&&");
       if ( selectedText.length()>18 ) {
         selectedText.truncate(15);
         selectedText+="...";
@@ -446,15 +451,15 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const QString &doc,
 
       // search provider icon
       QPixmap icon;
-      KURIFilterData data;
+      KUriFilterData data;
       QStringList list;
       data.setData( QString("some keyword") );
       list << "kurisearchfilter" << "kuriikwsfilter";
 
       QString name;
-      if ( KURIFilter::self()->filterURI(data, list) )
+      if ( KUriFilter::self()->filterUri(data, list) )
       {
-        QString iconPath = KStandardDirs::locate("cache", KMimeType::favIconForURL(data.uri()) + ".png");
+        QString iconPath = KStandardDirs::locate("cache", KMimeType::favIconForUrl(data.uri()) + ".png");
         if ( iconPath.isEmpty() )
           icon = SmallIcon("find");
         else
@@ -467,7 +472,8 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const QString &doc,
         name = "Google";
       }
 
-      KAction *action = new KAction( i18n( "Search '%1' at %2" ,  selectedText ,  name ), actionCollection(), "searchProvider" );
+      KAction *action = new KAction( i18n( "Search for '%1' with %2" ,  selectedText ,  name ), this );
+      actionCollection()->addAction( "searchProvider", action );
       static_cast<QAction*>( action )->setIcon( QIcon( icon ) );
       connect( action, SIGNAL( triggered( bool ) ), d->m_khtml->browserExtension(), SLOT( searchProvider() ) );
 
@@ -477,7 +483,8 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const QString &doc,
       favoriteEngines = config.readEntry("FavoriteSearchEngines", favoriteEngines);
 
       if ( !favoriteEngines.isEmpty()) {
-        KActionMenu* providerList = new KActionMenu( i18n( "Search '%1' At" ,  selectedText ), actionCollection(), "searchProviderList" );
+        KActionMenu* providerList = new KActionMenu( i18n( "Search for '%1' with" ,  selectedText ), this );
+        actionCollection()->addAction( "searchProviderList", providerList );
 
         QStringList::ConstIterator it = favoriteEngines.begin();
         for ( ; it != favoriteEngines.end(); ++it ) {
@@ -489,16 +496,17 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const QString &doc,
           const QString searchProviderPrefix = *(service->property("Keys").toStringList().begin()) + keywordDelimiter;
           data.setData( searchProviderPrefix + "some keyword" );
 
-          if ( KURIFilter::self()->filterURI(data, list) )
+          if ( KUriFilter::self()->filterUri(data, list) )
           {
-            QString iconPath = KStandardDirs::locate("cache", KMimeType::favIconForURL(data.uri()) + ".png");
+            QString iconPath = KStandardDirs::locate("cache", KMimeType::favIconForUrl(data.uri()) + ".png");
             if ( iconPath.isEmpty() )
               icon = SmallIcon("find");
             else
               icon = QPixmap( iconPath );
             name = service->name();
 
-            KAction *action = new KAction( name, actionCollection(), QString( "searchProvider" + searchProviderPrefix ).toLatin1().constData() );
+            KAction *action = new KAction( name, this  );
+            actionCollection()->addAction( QString( "searchProvider" + searchProviderPrefix ).toLatin1().constData(), action );
             static_cast<QAction*>( action )->setIcon( QIcon( icon ) );
             connect( action, SIGNAL( triggered( bool ) ), d->m_khtml->browserExtension(), SLOT( searchProvider() ) );
 
@@ -509,16 +517,18 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const QString &doc,
 
 
       if ( selectedText.contains("://") && KUrl(selectedText).isValid() ) {
-         KAction *action = new KAction( i18n( "Open '%1'" ,  selectedText ), actionCollection(), "openSelection" );
+         KAction *action = new KAction( i18n( "Open '%1'" ,  selectedText ), this );
+         actionCollection()->addAction( "openSelection", action );
          action->setIcon( KIcon( "window_new" ) );
          connect( action, SIGNAL( triggered( bool ) ), d->m_khtml->browserExtension(), SLOT( openSelection() ) );
       }
   }
   else if ( url.isEmpty() && !isImage )
   {
-      actionCollection()->insert( khtml->actionCollection()->action( "security" ) );
-      actionCollection()->insert( khtml->actionCollection()->action( "setEncoding" ) );
-      KAction *action = new KAction( i18n( "Stop Animations" ), actionCollection(), "stopanimations" );
+      actionCollection()->addAction( "security", khtml->actionCollection()->action( "security" ) );
+      actionCollection()->addAction( "setEncoding", khtml->actionCollection()->action( "setEncoding" ) );
+      KAction *action = new KAction( i18n( "Stop Animations" ), this );
+      actionCollection()->addAction( "stopanimations", action );
       connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotStopAnimations() ) );
   }
 
@@ -526,15 +536,18 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const QString &doc,
   {
     if (url.protocol() == "mailto")
     {
-      KAction *action = new KAction( i18n( "Copy Email Address" ), actionCollection(), "copylinklocation" );
+      KAction *action = new KAction( i18n( "Copy Email Address" ), this );
+      actionCollection()->addAction( "copylinklocation", action );
       connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotCopyLinkLocation() ) );
     }
     else
     {
-      KAction *action = new KAction( i18n( "&Save Link As..." ), actionCollection(), "savelinkas" );
+      KAction *action = new KAction( i18n( "&Save Link As..." ), this );
+      actionCollection()->addAction( "savelinkas", action );
       connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotSaveLinkAs() ) );
 
-      action = new KAction( i18n( "Copy &Link Address" ), actionCollection(), "copylinklocation" );
+      action = new KAction( i18n( "Copy &Link Address" ), this );
+      actionCollection()->addAction( "copylinklocation", action );
       connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotCopyLinkLocation() ) );
     }
   }
@@ -544,52 +557,63 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const QString &doc,
   {
     if ( khtml->parentPart() )
     {
-      KAction *action = new KAction( i18n( "Open in New &Window" ), actionCollection(), "frameinwindow" );
+      KAction *action = new KAction( i18n( "Open in New &Window" ), this );
+      actionCollection()->addAction( "frameinwindow", action );
       action->setIcon( KIcon( "window_new" ) );
       connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotFrameInWindow() ) );
 
-      action = new KAction( i18n( "Open in &This Window" ), actionCollection(), "frameintop" );
+      action = new KAction( i18n( "Open in &This Window" ), this );
+      actionCollection()->addAction( "frameintop", action );
       connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotFrameInTop() ) );
 
-      action = new KAction( i18n( "Open in &New Tab" ), actionCollection(), "frameintab" );
+      action = new KAction( i18n( "Open in &New Tab" ), this );
+      actionCollection()->addAction( "frameintab", action );
       action->setIcon( KIcon( "tab_new" ) );
       connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotFrameInTab() ) );
 
-      action = new KAction( i18n( "Reload Frame" ), actionCollection(), "reloadframe" );
+      action = new KAction( i18n( "Reload Frame" ), this );
+      actionCollection()->addAction( "reloadframe", action );
       connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotReloadFrame() ) );
 
       if ( KHTMLFactory::defaultHTMLSettings()->isAdFilterEnabled() ) {
           if ( khtml->d->m_frame->m_type == khtml::ChildFrame::IFrame ) {
-              action = new KAction( i18n( "Block IFrame..." ), actionCollection(), "blockiframe" );
+              action = new KAction( i18n( "Block IFrame..." ), this );
+              actionCollection()->addAction( "blockiframe", action );
               connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotBlockIFrame() ) );
           }
       }
 
-      action = new KAction( i18n( "View Frame Source" ), actionCollection(), "viewFrameSource" );
+      action = new KAction( i18n( "View Frame Source" ), this );
+      actionCollection()->addAction( "viewFrameSource", action );
       connect( action, SIGNAL( triggered( bool ) ), d->m_khtml, SLOT( slotViewDocumentSource() ) );
 
-      action = new KAction( i18n( "View Frame Information" ), actionCollection(), "viewFrameInfo" );
+      action = new KAction( i18n( "View Frame Information" ), this );
+      actionCollection()->addAction( "viewFrameInfo", action );
       connect( action, SIGNAL( triggered( bool ) ), d->m_khtml, SLOT( slotViewPageInfo() ) );
 
       // This one isn't in khtml_popupmenu.rc anymore, because Print isn't either,
       // and because print frame is already in the toolbar and the menu.
       // But leave this here, so that it's easy to read it.
-      action = new KAction( i18n( "Print Frame..." ), actionCollection(), "printFrame" );
+      action = new KAction( i18n( "Print Frame..." ), this );
+      actionCollection()->addAction( "printFrame", action );
       action->setIcon( KIcon( "frameprint" ) );
       connect( action, SIGNAL( triggered( bool ) ), d->m_khtml->browserExtension(), SLOT( print() ) );
-      action = new KAction( i18n( "Save &Frame As..." ), actionCollection(), "saveFrame" );
+      action = new KAction( i18n( "Save &Frame As..." ), this );
+      actionCollection()->addAction( "saveFrame", action );
       connect( action, SIGNAL( triggered( bool ) ), d->m_khtml, SLOT( slotSaveFrame() ) );
 
-      actionCollection()->insert( khtml->parentPart()->actionCollection()->action( "viewDocumentSource" ) );
-      actionCollection()->insert( khtml->parentPart()->actionCollection()->action( "viewPageInfo" ) );
+      actionCollection()->addAction( "viewDocumentSource",
+                                     khtml->parentPart()->actionCollection()->action( "viewDocumentSource" ) );
+      actionCollection()->addAction( "viewPageInfo", khtml->parentPart()->actionCollection()->action( "viewPageInfo" ) );
     } else {
-      actionCollection()->insert( khtml->actionCollection()->action( "viewDocumentSource" ) );
-      actionCollection()->insert( khtml->actionCollection()->action( "viewPageInfo" ) );
+      actionCollection()->addAction( "viewDocumentSource", khtml->actionCollection()->action( "viewDocumentSource" ) );
+      actionCollection()->addAction( "viewPageInfo", khtml->actionCollection()->action( "viewPageInfo" ) );
     }
   } else if (isImage || !url.isEmpty()) {
-    actionCollection()->insert( khtml->actionCollection()->action( "viewDocumentSource" ) );
-    actionCollection()->insert( khtml->actionCollection()->action( "viewPageInfo" ) );
-    KAction *action = new KAction( i18n( "Stop Animations" ), actionCollection(), "stopanimations" );
+    actionCollection()->addAction( "viewDocumentSource", khtml->actionCollection()->action( "viewDocumentSource" ) );
+    actionCollection()->addAction( "viewPageInfo", khtml->actionCollection()->action( "viewPageInfo" ) );
+    KAction *action = new KAction( i18n( "Stop Animations" ), this );
+    actionCollection()->addAction( "stopanimations", action );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotStopAnimations() ) );
   }
 
@@ -608,36 +632,43 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const QString &doc,
     }
     else
       d->m_imageURL = KUrl( static_cast<DOM::HTMLInputElement>( e ).src().string() );
-    KAction *action = new KAction( i18n( "Save Image As..." ), actionCollection(), "saveimageas" );
+    KAction *action = new KAction( i18n( "Save Image As..." ), this );
+    actionCollection()->addAction( "saveimageas", action );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotSaveImageAs() ) );
-    action = new KAction( i18n( "Send Image..." ), actionCollection(), "sendimage" );
+    action = new KAction( i18n( "Send Image..." ), this );
+    actionCollection()->addAction( "sendimage", action );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotSendImage() ) );
 
 #ifndef QT_NO_MIMECLIPBOARD
-    action = new KAction( i18n( "Copy Image" ), actionCollection(), "copyimage" );
+    action = new KAction( i18n( "Copy Image" ), this );
+    actionCollection()->addAction( "copyimage", action );
     action->setEnabled(!d->m_pixmap.isNull());
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotCopyImage() ) );
 #endif
 
     if(d->m_pixmap.isNull()) {    //fallback to image location if still loading the image.  this will always be true if ifdef QT_NO_MIMECLIPBOARD
-      action = new KAction( i18n( "Copy Image Location" ), actionCollection(), "copyimagelocation" );
+      action = new KAction( i18n( "Copy Image Location" ), this );
+      actionCollection()->addAction( "copyimagelocation", action );
       connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotCopyImageLocation() ) );
     }
 
     QString name = KStringHandler::csqueeze(d->m_imageURL.fileName()+d->m_imageURL.query(), 25);
-    action = new KAction( i18n( "View Image (%1)" , d->m_suggestedFilename.isEmpty() ? name.replace("&", "&&") : d->m_suggestedFilename.replace("&", "&&")), actionCollection(), "viewimage" );
+    action = new KAction( i18n( "View Image (%1)" , d->m_suggestedFilename.isEmpty() ? name.replace("&", "&&") : d->m_suggestedFilename.replace("&", "&&")), this );
+    actionCollection()->addAction( "viewimage", action );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotViewImage() ) );
 
 
     if (KHTMLFactory::defaultHTMLSettings()->isAdFilterEnabled())
     {
-      action = new KAction( i18n( "Block Image..." ), actionCollection(), "blockimage" );
+      action = new KAction( i18n( "Block Image..." ), this );
+      actionCollection()->addAction( "blockimage", action );
       connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotBlockImage() ) );
 
       if (!d->m_imageURL.host().isEmpty() &&
           !d->m_imageURL.protocol().isEmpty())
       {
-        action = new KAction( i18n( "Block Images From %1" , d->m_imageURL.host()), actionCollection(), "blockhost" );
+        action = new KAction( i18n( "Block Images From %1" , d->m_imageURL.host()), this );
+        actionCollection()->addAction( "blockhost", action );
         connect( action, SIGNAL( triggered( bool ) ), this, SLOT( slotBlockHost() ) );
       }
     }
@@ -648,7 +679,7 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const QString &doc,
 
   QDomElement menu = domDocument().documentElement().namedItem( "Menu" ).toElement();
 
-  if ( actionCollection()->count() > 0 )
+  if ( actionCollection()->actions().count() > 0 )
     menu.insertBefore( domDocument().createElement( "separator" ), menu.firstChild() );
 }
 
@@ -791,9 +822,9 @@ void KHTMLPopupGUIClient::slotReloadFrame()
   args.reload = true;
   args.metaData()["referrer"] = d->m_khtml->pageReferrer();
   // reload document
-  d->m_khtml->closeURL();
-  d->m_khtml->browserExtension()->setURLArgs( args );
-  d->m_khtml->openURL( d->m_khtml->url() );
+  d->m_khtml->closeUrl();
+  d->m_khtml->browserExtension()->setUrlArgs( args );
+  d->m_khtml->openUrl( d->m_khtml->url() );
 }
 
 void KHTMLPopupGUIClient::slotFrameInWindow()
@@ -809,7 +840,7 @@ void KHTMLPopupGUIClient::slotFrameInTop()
   KParts::URLArgs args( d->m_khtml->browserExtension()->urlArgs() );
   args.metaData()["referrer"] = d->m_khtml->pageReferrer();
   args.frameName = "_top";
-  emit d->m_khtml->browserExtension()->openURLRequest( d->m_khtml->url(), args );
+  emit d->m_khtml->browserExtension()->openUrlRequest( d->m_khtml->url(), args );
 }
 
 void KHTMLPopupGUIClient::slotFrameInTab()
@@ -842,7 +873,7 @@ void KHTMLPopupGUIClient::saveURL( QWidget *parent, const QString &caption,
         QFileInfo info( destURL.path() );
         if( info.exists() ) {
           // TODO: use KIO::RenameDlg (shows more information)
-          query = KMessageBox::warningContinueCancel( parent, i18n( "A file named \"%1\" already exists. " "Are you sure you want to overwrite it?" ,  info.fileName() ), i18n( "Overwrite File?" ), i18n( "Overwrite" ) );
+          query = KMessageBox::warningContinueCancel( parent, i18n( "A file named \"%1\" already exists. " "Are you sure you want to overwrite it?" ,  info.fileName() ), i18n( "Overwrite File?" ), KGuiItem(i18n( "Overwrite" )) );
         }
        }
    } while ( query == KMessageBox::Cancel );
@@ -863,22 +894,23 @@ void KHTMLPopupGUIClient::saveURL( const KUrl &url, const KUrl &destURL,
             if (destURL.isLocalFile())
             {
                 KSaveFile destFile(destURL.path());
-                if (destFile.status() == 0)
+                if (destFile.open())
                 {
-                    KHTMLPageCache::self()->saveData(cacheId, destFile.dataStream());
+                    QDataStream stream ( &destFile );
+                    KHTMLPageCache::self()->saveData(cacheId, &stream);
                     saved = true;
                 }
             }
             else
             {
                 // save to temp file, then move to final destination.
-                KTempFile destFile;
-                if (destFile.status() == 0)
+                KTemporaryFile destFile;
+                if (destFile.open())
                 {
-                    KHTMLPageCache::self()->saveData(cacheId, destFile.dataStream());
-                    destFile.close();
+                    QDataStream stream ( &destFile );
+                    KHTMLPageCache::self()->saveData(cacheId, &stream);
                     KUrl url2 = KUrl();
-                    url2.setPath(destFile.name());
+                    url2.setPath(destFile.fileName());
                     KIO::file_move(url2, destURL, -1, true /*overwrite*/);
                     saved = true;
                 }
@@ -959,17 +991,13 @@ bool KHTMLPartBrowserHostExtension::openURLInFrame( const KUrl &url, const KPart
   return m_part->openURLInFrame( url, urlArgs );
 }
 
-void KHTMLPartBrowserHostExtension::virtual_hook( int id, void *data )
+KParts::BrowserHostExtension* KHTMLPartBrowserHostExtension::findFrameParent( KParts::ReadOnlyPart
+      *callingPart, const QString &frame )
 {
-  if (id == VIRTUAL_FIND_FRAME_PARENT)
-  {
-    FindFrameParentParams *param = static_cast<FindFrameParentParams*>(data);
-    KHTMLPart *parentPart = m_part->findFrameParent(param->callingPart, param->frame);
+    KHTMLPart *parentPart = m_part->findFrameParent(callingPart, frame);
     if (parentPart)
-       param->parent = parentPart->browserHostExtension();
-    return;
-  }
-  BrowserHostExtension::virtual_hook( id, data );
+       return parentPart->browserHostExtension();
+    return 0;
 }
 
 
@@ -977,8 +1005,8 @@ void KHTMLPartBrowserHostExtension::virtual_hook( int id, void *data )
 extern const int KDE_NO_EXPORT fastZoomSizes[];
 extern const int KDE_NO_EXPORT fastZoomSizeCount;
 
-KHTMLZoomFactorAction::KHTMLZoomFactorAction( KHTMLPart *part, bool direction, const QString &icon, const QString &text, KActionCollection *parent, const char *name )
-    : KSelectAction( text, parent, name )
+KHTMLZoomFactorAction::KHTMLZoomFactorAction( KHTMLPart *part, bool direction, const QString &icon, const QString &text, QObject *parent )
+    : KSelectAction( text, parent )
 {
     setIcon( KIcon( icon ) );
 
