@@ -123,7 +123,7 @@ namespace KWallet
  * \code
  * KUrl url = "http://www.kde.org";
  * KHTMLPart *w = new KHTMLPart();
- * w->openURL(url);
+ * w->openUrl(url);
  * w->view()->resize(500, 400);
  * w->show();
  * \endcode
@@ -224,14 +224,16 @@ class KHTML_EXPORT KHTMLPart : public KParts::ReadOnlyPart
   Q_PROPERTY( bool javaEnabled READ javaEnabled WRITE setJavaEnabled )
   Q_PROPERTY( bool dndEnabled READ dndEnabled WRITE setDNDEnabled )
   Q_PROPERTY( bool pluginsEnabled READ pluginsEnabled WRITE setPluginsEnabled )
+  
   /*
-   * don't add setOnlyLocalReferences. its not supposed to be accessible via D-BUS
-   */
+   *
+   * Don't add setOnlyLocalReferences here. It shouldn't be accessible via DBus.
+   *
+   **/
   Q_PROPERTY( bool modified READ isModified )
   Q_PROPERTY( QString encoding READ encoding WRITE setEncoding )
   Q_PROPERTY( QString lastModified READ lastModified )
   Q_PROPERTY( bool metaRefreshEnabled READ metaRefreshEnabled WRITE setMetaRefreshEnabled )
-  Q_PROPERTY( QString url READ url )
 
 public:
   enum GUIProfile { DefaultGUI, BrowserViewGUI /* ... */ };
@@ -261,14 +263,14 @@ public:
   /**
    * Opens the specified URL @p url.
    *
-   * Reimplemented from KParts::ReadOnlyPart::openURL .
+   * Reimplemented from KParts::ReadOnlyPart::openUrl .
    */
-  virtual bool openURL( const KUrl &url );
+  virtual bool openUrl( const KUrl &url );
 
   /**
    * Stops loading the document and kills all data requests (for images, etc.)
    */
-  virtual bool closeURL();
+  virtual bool closeUrl();
 
   /**
    * Called when a certain error situation (i.e. connection timed out) occurred.
@@ -333,7 +335,7 @@ public:
    * One thing people using this method to add things to the interpreter must
    * consider, is that when you start writing new content to the part, the
    * interpreter is cleared. This includes both use of the
-   * begin( const KUrl &, int, int ) method, and the openURL( const KUrl & )
+   * begin( const KUrl &, int, int ) method, and the openUrl( const KUrl & )
    * method. If you want your objects to have a longer lifespan, then you must
    * retain a KJS::Object yourself to ensure that the reference count of your
    * custom objects never reaches 0. You will also need to re-add your
@@ -397,7 +399,7 @@ public:
   void setJavaEnabled( bool enable );
 
   /**
-   * Return true if Java applet support is enabled, false if disabled
+   * Return @p true if Java applet support is enabled, @p false if disabled
    */
   bool javaEnabled() const;
 
@@ -418,7 +420,7 @@ public:
   void setPluginsEnabled( bool enable );
 
   /**
-   * Returns true if plugins are enabled/disabled.
+   * Returns @p true if plugins are enabled, @p false if disabled.
    */
   bool pluginsEnabled() const;
 
@@ -451,13 +453,13 @@ public:
    * Java and Plugins support. This might change in the future if the security model
    * is becoming more sophisticated, so don't rely on this behaviour.
    *
-   * ( default false - everything is loaded unless forbidden by KApplication::authorizeURLAction).
+   * ( default @p false - everything is loaded unless forbidden by KApplication::authorizeURLAction).
    */
   void setOnlyLocalReferences( bool enable );
 
   /**
    * Returns whether only file:/ or data:/ references are allowed
-   * to be loaded ( default false ).  See setOnlyLocalReferences.
+   * to be loaded ( default @p false ).  See setOnlyLocalReferences.
    **/
   bool onlyLocalReferences() const;
 
@@ -486,12 +488,11 @@ public:
 
   /**
    * Enumeration for displaying the caret.
-   * @param Visible caret is displayed
-   * @param Invisible caret is not displayed
-   * @param Blink caret toggles between visible and invisible
    */
   enum CaretDisplayPolicy {
-	CaretVisible, CaretInvisible, CaretBlink
+      CaretVisible, /**< caret is displayed */
+      CaretInvisible, /**<  caret is not displayed */
+      CaretBlink /**< caret toggles between visible and invisible */
   };
 
   /**
@@ -722,15 +723,15 @@ public:
 
   /**
    * Finds the next occurrence of the string or expression.
-   * If isRegExp is true then str is converted to a QRegExp, and caseSensitive is ignored.
+   * If isRegExp is @p true then str is converted to a QRegExp, and caseSensitive is ignored.
    * @deprecated, use findText( str, options, parent, findDialog )
    */
   bool findTextNext( const QString &str, bool forward, bool caseSensitive, bool isRegExp );
 
   /**
    * Finds the next occurrence of a string set by @ref findText()
-   * @param reverse if true, revert seach direction (only if no find dialog is used)
-   * @return true if a new match was found.
+   * @param reverse if @p true, revert seach direction (only if no find dialog is used)
+   * @return @p true if a new match was found.
    */
   bool findTextNext( bool reverse = false );
 
@@ -903,7 +904,7 @@ public:
 
   /**
    * Returns whether a frame with the specified name is exists or not.
-   * In contrary to the findFrame method this one also returns true
+   * In contrary to the findFrame method this one also returns @p true
    * if the frame is defined but no displaying component has been
    * found/loaded, yet.
    */
@@ -1003,7 +1004,7 @@ public:
   /**
    * Checks whether the page contains unsubmitted form changes.
    *
-   * @return true if form changes exist
+   * @return @p true if form changes exist
    */
   bool isModified() const;
 
@@ -1190,7 +1191,7 @@ public Q_SLOTS:
    * Execute the specified snippet of JavaScript code.
    *
    * Returns @p true if JavaScript was enabled, no error occurred
-   * and the code returned true itself or @p false otherwise.
+   * and the code returned @p true itself or @p false otherwise.
    * @deprecated, use executeString( DOM::Node(), script)
    */
   QVariant executeScript( const QString &script );
@@ -1244,6 +1245,23 @@ public Q_SLOTS:
                         const QString& contentType = QString(),
                         const QString& boundary = QString() );
 
+protected Q_SLOTS:
+
+  /**
+   * Called when the job downloading the page is finished.
+   * Can be reimplemented, for instance to get metadata out of the job,
+   * but make sure to call KHTMLPart::slotFinished() too.
+   */
+  virtual void slotFinished( KJob* );
+
+protected:
+  /**
+   * Hook for adding code before a job is started.
+   * This can be used to add metadata, like job->addMetaData("PropagateHttpHeader", "true")
+   * to get the HTTP headers.
+   */
+  virtual void startingJob( KIO::Job * ) {}
+
 private Q_SLOTS:
 
   /**
@@ -1263,10 +1281,6 @@ private Q_SLOTS:
    * @internal
    */
   void slotRestoreData( const QByteArray &data );
-  /**
-   * @internal
-   */
-  void slotFinished( KJob* );
   /**
    * @internal
    */
@@ -1432,7 +1446,7 @@ private Q_SLOTS:
   /*
    * @internal
    */
-  void slotJobSpeed( KIO::Job*, unsigned long );
+  void slotJobSpeed( KJob*, unsigned long );
 
   /**
    * @internal
