@@ -291,11 +291,11 @@ void KHTMLSettings::init()
   KConfig global( "khtmlrc", true, false );
   init( &global, true );
 
-  KConfig *local = KGlobal::config();
+  KSharedConfig::Ptr local = KGlobal::config();
   if ( !local )
     return;
 
-  init( local, false );
+  init( local.data(), false );
 }
 
 void KHTMLSettings::init( KConfig * config, bool reset )
@@ -731,8 +731,9 @@ bool KHTMLSettings::isAdFiltered( const QString &url ) const
     {
         if (!url.startsWith("data:"))
         {
-	  QVector<QRegExp>::iterator it;
-	  for (it=d->adFilters.begin(); it != d->adFilters.end(); ++it)
+            QVector<QRegExp>::const_iterator it(d->adFilters.constBegin());
+            QVector<QRegExp>::const_iterator end(d->adFilters.constEnd());
+            for (; it != end; ++it)
             {
                 if ((*it).indexIn(url) != -1)
                 {
@@ -1032,7 +1033,7 @@ void KHTMLSettings::setJSErrorsEnabled(bool enabled)
 {
   d->m_jsErrorsEnabled = enabled;
   // save it
-  KConfig *config = KGlobal::config();
+  KSharedConfig::Ptr config = KGlobal::config();
   config->setGroup("HTML Settings");
   config->writeEntry("ReportJSErrors", enabled);
   config->sync();
@@ -1057,7 +1058,7 @@ void KHTMLSettings::setJSPopupBlockerPassivePopup(bool enabled)
 {
     d->m_jsPopupBlockerPassivePopup = enabled;
     // save it
-    KConfig *config = KGlobal::config();
+    KSharedConfig::Ptr config = KGlobal::config();
     config->setGroup("Java/JavaScript Settings");
     config->writeEntry("PopupBlockerPassivePopup", enabled);
     config->sync();
