@@ -20,13 +20,13 @@
 #include "kmultipart.h"
 
 
-#include <kinstance.h>
+#include <kcomponentdata.h>
 #include <kmimetype.h>
 #include <klocale.h>
 #include <kjobuidelegate.h>
 #include <kio/job.h>
 #include <qfile.h>
-#include <ktempfile.h>
+#include <ktemporaryfile.h>
 #include <kmessagebox.h>
 #include <kparts/componentfactory.h>
 #include <kparts/genericfactory.h>
@@ -101,14 +101,14 @@ KMultiPart::KMultiPart( QWidget *parentWidget,
 {
     m_filter = 0L;
 
-    setInstance( KMultiPartFactory::instance() );
+    setComponentData( KMultiPartFactory::componentData() );
 
     KVBox *box = new KVBox( parentWidget );
     setWidget( box );
 
     m_extension = new KParts::BrowserExtension( this );
 
-    // We probably need to use m_extension to get the urlArgs in openURL...
+    // We probably need to use m_extension to get the urlArgs in openUrl...
 
     m_part = 0L;
     m_isHTMLPart = false;
@@ -134,7 +134,7 @@ KMultiPart::~KMultiPart()
     delete m_job;
     delete m_lineParser;
     if ( m_tempFile ) {
-        m_tempFile->setAutoDelete( true );
+        m_tempFile->setAutoRemove( true );
         delete m_tempFile;
     }
     delete m_filter;
@@ -153,7 +153,7 @@ void KMultiPart::startHeader()
 }
 
 
-bool KMultiPart::openURL( const KUrl &url )
+bool KMultiPart::openUrl( const KUrl &url )
 {
     m_url = url;
     m_lineParser->reset();
@@ -341,8 +341,8 @@ void KMultiPart::setPart( const QString& mimeType )
         connect( childExtension, SIGNAL( openURLNotify() ),
                  m_extension, SIGNAL( openURLNotify() ) );
 
-        connect( childExtension, SIGNAL( openURLRequestDelayed( const KUrl &, const KParts::URLArgs & ) ),
-                 m_extension, SIGNAL( openURLRequest( const KUrl &, const KParts::URLArgs & ) ) );
+        connect( childExtension, SIGNAL( openUrlRequestDelayed( const KUrl &, const KParts::URLArgs & ) ),
+                 m_extension, SIGNAL( openUrlRequest( const KUrl &, const KParts::URLArgs & ) ) );
 
         connect( childExtension, SIGNAL( createNewWindow( const KUrl &, const KParts::URLArgs & ) ),
                  m_extension, SIGNAL( createNewWindow( const KUrl &, const KParts::URLArgs & ) ) );
@@ -400,7 +400,7 @@ void KMultiPart::setPart( const QString& mimeType )
     // Load the part's plugins too.
     // ###### This is a hack. The bug is that KHTMLPart doesn't load its plugins
     // if className != "Browser/View".
-    loadPlugins( this, m_part, m_part->instance() );
+    loadPlugins( this, m_part, m_part->componentData() );
     // Get the part's GUI to appear
     if ( guiFactory )
         guiFactory->addClient( this );
@@ -429,11 +429,11 @@ void KMultiPart::startOfData()
     // Pass URLArgs (e.g. reload)
     KParts::BrowserExtension* childExtension = KParts::BrowserExtension::childObject( m_part );
     if ( childExtension )
-        childExtension->setURLArgs( m_extension->urlArgs() );
+        childExtension->setUrlArgs( m_extension->urlArgs() );
 
     m_nextMimeType.clear();
     if ( m_tempFile ) {
-        m_tempFile->setAutoDelete( true );
+        m_tempFile->setAutoRemove( true );
         delete m_tempFile;
         m_tempFile = 0;
     }
@@ -445,7 +445,8 @@ void KMultiPart::startOfData()
     else
     {
         // ###### TODO use a QByteArray and a data: URL instead
-        m_tempFile = new KTempFile;
+        m_tempFile = new KTemporaryFile;
+        m_tempFile->open();
     }
 }
 
@@ -470,7 +471,7 @@ void KMultiPart::reallySendData( const QByteArray& line )
     }
     else if ( m_tempFile )
     {
-        m_tempFile->file()->write( line.data(), line.size() );
+        m_tempFile->write( line.data(), line.size() );
     }
 }
 
@@ -490,15 +491,15 @@ void KMultiPart::endOfData()
             // Otherwise we'd keep canceling it, and nothing would ever show up...
             kDebug() << "KMultiPart::endOfData part isn't ready, skipping frame" << endl;
             ++m_numberOfFramesSkipped;
-            m_tempFile->setAutoDelete( true );
+            m_tempFile->setAutoRemove( true );
         }
         else
         {
-            kDebug() << "KMultiPart::endOfData opening " << m_tempFile->name() << endl;
+            kDebug() << "KMultiPart::endOfData opening " << m_tempFile->fileName() << endl;
             KUrl url;
-            url.setPath( m_tempFile->name() );
+            url.setPath( m_tempFile->fileName() );
             m_partIsLoading = true;
-            (void) m_part->openURL( url );
+            (void) m_part->openUrl( url );
         }
         delete m_tempFile;
         m_tempFile = 0L;
@@ -520,11 +521,11 @@ void KMultiPart::slotPartCompleted()
     }
 }
 
-bool KMultiPart::closeURL()
+bool KMultiPart::closeUrl()
 {
     m_timer->stop();
     if ( m_part )
-        return m_part->closeURL();
+        return m_part->closeUrl();
     return true;
 }
 
