@@ -75,7 +75,7 @@
 #ifdef IMAGE_TITLES
 #include <qfile.h>
 #include <kfilemetainfo.h>
-#include <ktempfile.h>
+#include <ktemporaryfile.h>
 #endif
 
 #include "html/html_documentimpl.h"
@@ -398,6 +398,7 @@ CachedImage::CachedImage(DocLoader* dl, const DOMString &url, KIO::CacheControl
     //p = 0;
     //pixPart = 0;
     bg = 0;
+    scaled = 0;
     bgColor = qRgba( 0, 0, 0, 0xFF );
     typeChecked = false;
     isFullyTransparent = false;
@@ -451,9 +452,8 @@ void CachedImage::deref( CachedObjectClient *c )
 #define BGMINWIDTH      32
 #define BGMINHEIGHT     32
 
-const QPixmap &CachedImage::tiled_pixmap(const QColor& newc)
+QPixmap CachedImage::tiled_pixmap(const QColor& newc, int xWidth, int xHeight)
 {
-    QColor color = newc;
 
     // no error indication for background images
     if(m_hadError||m_wasBlocked) return *Cache::nullPixmap;
@@ -467,49 +467,123 @@ const QPixmap &CachedImage::tiled_pixmap(const QColor& newc)
 #endif
 
     static QRgb bgTransparent = qRgba( 0, 0, 0, 0xFF );
-    if ( (bgColor != bgTransparent) && (bgColor != newc.rgba()) ) {
+
+    QSize s(pixmap_size());
+    int w = xWidth;
+    int h = xHeight;
+
+    if (w == -1) xWidth = w = s.width();
+    if (h == -1) xHeight = h = s.height();
+
+    if ( ( (bgColor != bgTransparent) && (bgColor != newc.rgba()) ) ||
+         ( bgSize != QSize(xWidth, xHeight)) )
+    {
         delete bg; bg = 0;
     }
 
     if (bg)
         return *bg;
 
-    bool haveBgColor = newc.isValid();
+    const QPixmap* src; //source for pretiling, if any
 
-    if (!haveBgColor)
-        color = Qt::transparent;
+    const QPixmap &r = pixmap(); //this is expensive
+    if (r.isNull()) return r;
 
-    //See whether/how much we should tile.
-    //### FIXME: restore support for mask-only case.
-    int w = i->size().width();
-    int h = i->size().height();
-    QSize s(i->size());
-    if ( w*h < 8192 )
-    {
-        if ( s.width() < BGMINWIDTH )
-            w = ((BGMINWIDTH  / s.width())+1) * s.width();
-        if ( s.height() < BGMINHEIGHT )
-            h = ((BGMINHEIGHT / s.height())+1) * s.height();
+    //See whether we should scale
+    if (xWidth != s.width() || xHeight != s.height()) {
+        src = &scaled_pixmap(xWidth, xHeight);
+    } else {
+        src = &r;
     }
 
-    QPixmap r (w, h);
-    r.fill(color); //Fill with the appropriate bg color/transparency
+    bgSize = QSize(xWidth, xHeight);
 
-    QPainter paint(&r);
-    ImagePainter pi(i);
-    //Tile as far as needed...
-    for (int x = 0; x < w; x += i->size().width())
-        for (int y = 0; y < h; y += i->size().height())
-            pi.paint(x, y, &paint);
+    //See whether we can - and should - pre-blend
+    // ### this needs serious investigations. Not likely to help with transparent bgColor,
+    // won't work with CSS3 multiple backgrounds. Does it help at all in Qt4? (ref: #114938)
+    if (newc.isValid() && (r.hasAlpha() || r.hasAlphaChannel())) {
+        bg = new QPixmap(xWidth, xHeight);
+        bg->fill(newc);
+        QPainter p(bg);
+        p.drawPixmap(0, 0, *src);
+        bgColor = newc.rgba();
+        src     = bg;
+    } else {
+        bgColor = bgTransparent;
+    }
+
+    //See whether to pre-tile.
+    if ( w*h < 8192 )
+    {
+        if ( r.width() < BGMINWIDTH )
+            w = ((BGMINWIDTH-1) / xWidth + 1) * xWidth;
+        if ( r.height() < BGMINHEIGHT )
+            h = ((BGMINHEIGHT-1) / xHeight + 1) * xHeight;
+    }
+    
+    if ( w != xWidth  || h != xHeight )
+    {
+        // kDebug() << "pre-tiling " << s.width() << "," << s.height() << " to " << w << "," << h << endl;
+        QPixmap* oldbg = bg;
+        bg = new QPixmap(w, h);
+        if (src->hasAlpha() || src->hasAlphaChannel()) {
+            if (newc.isValid() && (bgColor != bgTransparent))
+                bg->fill( bgColor );
+            else
+                bg->fill( Qt::transparent );
+        }
+        
+        QPainter p(bg);
+        p.drawTiledPixmap(0, 0, w, h, *src);
+        p.end();
+
+        if ( src == oldbg )
+            delete oldbg;
+    } else if (src && !bg) {
+        // we were asked for the entire pixmap. Cache it.
+        // ### goes against imload stuff, but it's far too expensive
+        //     to recreate the full pixmap each time it's requested as
+        //     we don't know what portion of it will be used eventually 
+        //     (by e.g. paintBackgroundExtended). It could be a few pixels of
+        //     a huge image. See #140248/#1 for an obvious example.
+        //     Imload probably needs to handle all painting in paintBackgroundExtended.
+        bg = new QPixmap(*src);
+    }
+
+    if (bg)
+        return *bg;
+
+    return *src;
+}
+
+
+QPixmap CachedImage::scaled_pixmap( int xWidth, int xHeight )
+{
+    // no error indication for background images
+    if(m_hadError||m_wasBlocked) return *Cache::nullPixmap;
+
+    // If we don't have size yet, nothing to draw yet
+    if (i->size().width() == 0 || i->size().height() == 0)
+        return *Cache::nullPixmap;
+
+    if (scaled) {
+        if (scaled->width() == xWidth && scaled->height() == xHeight)
+            return *scaled;
+        delete scaled;
+    }
+
+    //### this is quite awful performance-wise
+    QImage im(xWidth, xHeight, QImage::Format_ARGB32_Premultiplied);
+
+    QPainter paint(&im);
+    paint.setCompositionMode(QPainter::CompositionMode_Source);
+    ImagePainter pi(i, QSize(xWidth, xHeight));
+    pi.paint(0, 0, &paint);
     paint.end();
 
-    if (haveBgColor)
-        bgColor = color.rgb();
-    else
-        bgColor = bgTransparent;
+    scaled = new QPixmap(QPixmap::fromImage(im));
 
-    bg = new QPixmap(r);
-    return *bg;
+    return *scaled;
 }
 
 QPixmap CachedImage::pixmap( ) const
@@ -525,11 +599,9 @@ QPixmap CachedImage::pixmap( ) const
     QImage im(w, h, QImage::Format_ARGB32_Premultiplied);
 
     QPainter paint(&im);
+    paint.setCompositionMode(QPainter::CompositionMode_Source);
     ImagePainter pi(i);
-    //Tile as far as needed...
-    for (int x = 0; x < w; x += i->size().width())
-        for (int y = 0; y < h; y += i->size().height())
-            pi.paint(x, y, &paint);
+    pi.paint(0, 0, &paint);
     paint.end();
     return QPixmap::fromImage( im );
 }
@@ -544,7 +616,7 @@ QSize CachedImage::pixmap_size() const
 }
 
 
-void CachedImage::imageHasGeometry(khtmlImLoad::Image* img, int width, int height)
+void CachedImage::imageHasGeometry(khtmlImLoad::Image* /*img*/, int width, int height)
 {
 #ifdef LOADER_DEBUG
     kDebug(6060) << this << " got geometry "<< width << "x" << height << endl;
@@ -552,7 +624,7 @@ void CachedImage::imageHasGeometry(khtmlImLoad::Image* img, int width, int heigh
     do_notify(QRect(0, 0, width, height));
 }
 
-void CachedImage::imageChange     (khtmlImLoad::Image* img, QRect region)
+void CachedImage::imageChange     (khtmlImLoad::Image* /*img*/, QRect region)
 {
 #ifdef LOADER_DEBUG
     kDebug(6060) << "Image " << this << " change " <<
@@ -574,13 +646,13 @@ void CachedImage::doNotifyFinished()
     }
 }
 
-void CachedImage::imageError(khtmlImLoad::Image* img)
+void CachedImage::imageError(khtmlImLoad::Image* /*img*/)
 {
     error(0, 0);
 }
 
 
-void CachedImage::imageDone(khtmlImLoad::Image* img)
+void CachedImage::imageDone(khtmlImLoad::Image* /*img*/)
 {
 #ifdef LOADER_DEBUG
     kDebug(6060)<<"Image is done:" << this << endl;
@@ -707,6 +779,7 @@ void CachedImage::movieStatus(int status)
 
 void CachedImage::setShowAnimations( KHTMLSettings::KAnimationAdvice showAnimations )
 {
+    (void) showAnimations;
 #if 0
     m_showAnimations = showAnimations;
     if ( (m_showAnimations == KHTMLSettings::KAnimationDisabled) && imgSource ) {
@@ -737,14 +810,23 @@ void CachedImage::setShowAnimations( KHTMLSettings::KAnimationAdvice showAnimati
 //     delete m; m = 0;
 // }
 
+void CachedImage::pauseAnimations()
+{
+//    if ( m ) m->setPaused( true );
+}
+
+void CachedImage::resumeAnimations()
+{
+//    if ( m ) m->setPaused( false );
+}
+
 void CachedImage::clear()
 {
     delete i;   i = new khtmlImLoad::Image(this);
+    delete scaled;  scaled = 0;
     bgColor = qRgba( 0, 0, 0, 0xff );
     delete bg;  bg = 0;
-/*    delete p;   p = 0;
-
-    delete pixPart; pixPart = 0; */
+    bgSize = QSize(-1,-1);
 
     formatType = 0;
     typeChecked = false;
@@ -862,6 +944,48 @@ void CachedImage::error( int /*err*/, const char* /*text*/ )
         it()->notifyFinished(this);
 }
 
+// -------------------------------------------------------------------------------------------
+
+CachedSound::CachedSound(DocLoader* dl, const DOMString &url, KIO::CacheControl _cachePolicy, const char*)
+    : CachedObject(url, Sound, _cachePolicy, 0)
+{
+    setAccept( QLatin1String("*/*") ); // should be whatever phonon would accept...
+    Cache::loader()->load(dl, this, false);
+    m_loading = true;
+}
+
+void CachedSound::ref(CachedObjectClient *c)
+{
+    CachedObject::ref(c);
+
+    if(!m_loading) c->notifyFinished(this);
+}
+
+void CachedSound::data( QBuffer &buffer, bool eof )
+{
+    if(!eof) return;
+    buffer.close();
+    setSize(buffer.buffer().size());
+
+    m_sound = buffer.buffer();
+    m_loading = false;
+    checkNotify();
+}
+
+void CachedSound::checkNotify()
+{
+    if(m_loading) return;
+
+    for (Q3PtrDictIterator<CachedObjectClient> it( m_clients); it.current();)
+        it()->notifyFinished(this);
+}
+
+void CachedSound::error( int /*err*/, const char* /*text*/ )
+{
+    m_loading = false;
+    checkNotify();
+}
+
 // ------------------------------------------------------------------------------------------
 
 Request::Request(DocLoader* dl, CachedObject *_object, bool _incremental)
@@ -960,7 +1084,7 @@ bool DocLoader::needReload(CachedObject *existing, const QString& fullURL)
     KUrl fullURL (m_doc->completeURL( url.string() )); \
     if ( !fullURL.isValid() || \
          ( m_part && m_part->onlyLocalReferences() && fullURL.protocol() != "file" && fullURL.protocol() != "data") || \
-         doRedirectCheck && ( kapp && m_doc && !KAuthorized::authorizeURLAction("redirect", m_doc->URL(), fullURL))) \
+         doRedirectCheck && ( kapp && m_doc && !KAuthorized::authorizeUrlAction("redirect", m_doc->URL(), fullURL))) \
          return 0L;
 
 CachedImage *DocLoader::requestImage( const DOM::DOMString &url)
@@ -1000,6 +1124,13 @@ CachedScript *DocLoader::requestScript( const DOM::DOMString &url, const QString
     return s;
 }
 
+CachedSound *DocLoader::requestSound( const DOM::DOMString &url )
+{
+    DOCLOADER_SECCHECK(true);
+    CachedSound* s = Cache::requestObject<CachedSound, CachedObject::Sound>( this, fullURL, 0 );
+    return s;
+}
+
 #undef DOCLOADER_SECCHECK
 
 void DocLoader::setAutoloadImages( bool enable )
@@ -1038,6 +1169,28 @@ void DocLoader::setShowAnimations( KHTMLSettings::KAnimationAdvice showAnimation
         }
 }
 
+void DocLoader::pauseAnimations()
+{
+    for ( Q3PtrDictIterator<CachedObject> it( m_docObjects ); it.current(); ++it )
+        if ( it.current()->type() == CachedObject::Image )
+        {
+            CachedImage *img = const_cast<CachedImage*>( static_cast<const CachedImage *>( it.current() ) );
+
+            img->pauseAnimations();
+        }
+}
+
+void DocLoader::resumeAnimations()
+{
+    for ( Q3PtrDictIterator<CachedObject> it( m_docObjects ); it.current(); ++it )
+        if ( it.current()->type() == CachedObject::Image )
+        {
+            CachedImage *img = const_cast<CachedImage*>( static_cast<const CachedImage *>( it.current() ) );
+
+            img->resumeAnimations();
+        }
+}
+
 // ------------------------------------------------------------------------------------------
 
 Loader::Loader() : QObject()
@@ -1136,11 +1289,11 @@ void Loader::slotFinished( KJob* job )
           static_cast<CachedImage*>( r->object )->setSuggestedFilename(fn);
 #ifdef IMAGE_TITLES
           static_cast<CachedImage*>( r->object )->setSuggestedTitle(fn);
-          KTempFile tf;
-          tf.setAutoDelete(true);
-          tf.file()->write((const char*)r->m_buffer.buffer().data(), r->m_buffer.size());
-          tf.sync();
-          KFileMetaInfo kfmi(tf.name());
+          KTemporaryFile tf;
+          tf.open();
+          tf.write((const char*)r->m_buffer.buffer().data(), r->m_buffer.size());
+          tf.flush();
+          KFileMetaInfo kfmi(tf.fileName());
           if (!kfmi.isEmpty()) {
               KFileMetaInfoItem i = kfmi.item("Name");
               if (i.isValid()) {
@@ -1278,7 +1431,7 @@ void Cache::init()
         nullPixmap = new QPixmap;
 
     if ( !brokenPixmap )
-        brokenPixmap = new QPixmap(KHTMLFactory::instance()->iconLoader()->loadIcon("file_broken", K3Icon::Desktop, 16, K3Icon::DisabledState));
+        brokenPixmap = new QPixmap(KHTMLFactory::iconLoader()->loadIcon("file_broken", K3Icon::Desktop, 16, K3Icon::DisabledState));
 
     if ( !blockedPixmap ) {
         blockedPixmap = new QPixmap();
@@ -1302,10 +1455,34 @@ void Cache::clear()
     cache->setAutoDelete( true );
 
 #ifndef NDEBUG
-    for (Q3DictIterator<CachedObject> it(*cache); it.current(); ++it)
-        assert(it.current()->canDelete());
-    foreach (CachedObject* co, *freeList)
-        assert(co->canDelete());
+    bool crash = false;
+    for (Q3DictIterator<CachedObject> it(*cache); it.current(); ++it) {
+        if (!it.current()->canDelete()) {
+            kDebug( 6060 ) << " Object in cache still linked to" << endl;
+            kDebug( 6060 ) << " -> URL: " << it.current()->url() << endl;
+            kDebug( 6060 ) << " -> #clients: " << it.current()->count() << endl;
+            crash = true;
+//         assert(it.current()->canDelete());
+        }
+    }
+    foreach (CachedObject* co, *freeList) {
+        if (!co->canDelete()) {
+            kDebug( 6060 ) << " Object in freelist still linked to" << endl;
+            kDebug( 6060 ) << " -> URL: " << co->url() << endl;
+            kDebug( 6060 ) << " -> #clients: " << co->count() << endl;
+            crash = true;
+            /*
+            QPtrDictIterator<CachedObjectClient> it(freeList->current()->m_clients);
+            for(;it.current(); ++it) {
+                if (dynamic_cast<RenderObject*>(it.current())) {
+                    kdDebug( 6060 ) << " --> RenderObject" << endl;
+                } else
+                    kdDebug( 6060 ) << " --> Something else" << endl;
+            }*/
+        }
+//         assert(freeList->current()->canDelete());
+    }
+    assert(!crash);
 #endif
 
     delete cache; cache = 0;
@@ -1421,6 +1598,7 @@ void Cache::statistics()
     int images = 0;
     int scripts = 0;
     int stylesheets = 0;
+    int sound = 0;
     Q3DictIterator<CachedObject> it(*cache);
     for(it.toFirst(); it.current(); ++it)
     {
@@ -1428,7 +1606,7 @@ void Cache::statistics()
         switch(o->type()) {
         case CachedObject::Image:
         {
-            CachedImage *im = static_cast<CachedImage *>(o);
+            //CachedImage *im = static_cast<CachedImage *>(o);
             images++;
             /*if(im->m != 0)
             {
@@ -1443,6 +1621,9 @@ void Cache::statistics()
         case CachedObject::Script:
             scripts++;
             break;
+        case CachedObject::Sound:
+            sound++;
+            break;
         }
         size += o->size();
     }
@@ -1454,6 +1635,7 @@ void Cache::statistics()
     kDebug( 6060 ) << "Number of cached movies: " << movie << endl;
     kDebug( 6060 ) << "Number of cached scripts: " << scripts << endl;
     kDebug( 6060 ) << "Number of cached stylesheets: " << stylesheets << endl;
+    kDebug( 6060 ) << "Number of cached sounds: " << sound << endl;
     kDebug( 6060 ) << "pixmaps:   allocated space approx. " << size << " kB" << endl;
     kDebug( 6060 ) << "movies :   allocated space approx. " << msize/1024 << " kB" << endl;
     kDebug( 6060 ) << "--------------------------------------------------------------------" << endl;
