@@ -2,7 +2,7 @@
  * This file is part of the html renderer for KDE.
  *
  * Copyright (C) 2000-2003 Lars Knoll (knoll@kde.org)
- *           (C) 2003-2004 Apple Computer, Inc.
+ *           (C) 2003-2005 Apple Computer, Inc.
  *           (C) 2005 Allan Sandfeld Jensen (kde@carewolf.com)
  *
  * This library is free software; you can redistribute it and/or
@@ -277,7 +277,7 @@ static inline RenderObject *Bidinext(RenderObject *par, RenderObject *current, B
 
         if (!next) break;
 
-        if (next->isText() || next->isBR() || next->isFloating() || next->isReplaced() || next->isPositioned()
+        if (next->isText() || next->isBR() || next->isFloating() || next->isReplaced() || next->isPositioned() || next->isGlyph()
             || ((!skipInlines || !next->firstChild()) // Always return EMPTY inlines.
                 && next->isInlineFlow()))
             break;
@@ -298,7 +298,7 @@ static RenderObject *first( RenderObject *par, BidiState &bidi, bool skipInlines
             return o; // Never skip empty inlines.
     }
 
-    if (o && !o->isText() && !o->isBR() && !o->isReplaced() && !o->isFloating() && !o->isPositioned())
+    if (o && !o->isText() && !o->isBR() && !o->isReplaced() && !o->isFloating() && !o->isPositioned() && !o->isGlyph())
         o = Bidinext( par, o, bidi, skipInlines );
     return o;
 }
@@ -729,6 +729,8 @@ void RenderBlock::computeHorizontalPositionsForLine(InlineFlowBox* lineBox, Bidi
     switch(style()->textAlign()) {
         case LEFT:
         case KHTML_LEFT:
+            if (style()->direction() == RTL && totWidth > availableWidth)
+                x -= (totWidth - availableWidth);
             numSpaces = 0;
             break;
         case JUSTIFY:
@@ -742,7 +744,8 @@ void RenderBlock::computeHorizontalPositionsForLine(InlineFlowBox* lineBox, Bidi
                 break;
         case RIGHT:
         case KHTML_RIGHT:
-            x += availableWidth - totWidth;
+            if (style()->direction() == RTL || totWidth < availableWidth)
+                x += availableWidth - totWidth;
             numSpaces = 0;
             break;
         case CENTER:
@@ -784,7 +787,7 @@ void RenderBlock::computeHorizontalPositionsForLine(InlineFlowBox* lineBox, Bidi
     if (rightPos > m_overflowWidth)
         m_overflowWidth = rightPos; // FIXME: Work for rtl overflow also.
     if (x < 0)
-        m_negativeOverflowWidth = qMax(m_negativeOverflowWidth, -x);
+        m_overflowLeft = qMin(m_overflowLeft, x);
 }
 
 void RenderBlock::computeVerticalPositionsForLine(InlineFlowBox* lineBox)
@@ -1292,15 +1295,6 @@ void RenderBlock::bidiReorderLine(const BidiIterator &start, const BidiIterator
     // reversing is only done up to the lowest odd level
     if( !(levelLow%2) ) levelLow++;
 
-#if BIDI_DEBUG > 0
-    kDebug(6041) << "lineLow = " << (uint)levelLow << ", lineHigh = " << (uint)levelHigh << endl;
-    kDebug(6041) << "logical order is:" << endl;
-    Q3PtrListIterator<BidiRun> it2(runs);
-    BidiRun *r2;
-    for ( ; (r2 = it2.current()); ++it2 )
-        kDebug(6041) << "    " << r2 << "  start=" << r2->start << "  stop=" << r2->stop << "  level=" << (uint)r2->level << endl;
-#endif
-
     int count = sBidiRunCount - 1;
 
     // do not reverse for visually ordered web sites
@@ -1389,12 +1383,20 @@ void RenderBlock::layoutInlineChildren(bool relayoutChildren, int breakBeforeLin
 
     m_height = borderTop() + paddingTop();
     int toAdd = borderBottom() + paddingBottom();
-    if (m_layer && style()->scrollsOverflow() && style()->height().isVariable())
+    if (m_layer && scrollsOverflowX() && style()->height().isVariable())
         toAdd += m_layer->horizontalScrollbarHeight();
 
     // Clear out our line boxes.
     deleteInlineBoxes();
 
+    // Text truncation only kicks in if your overflow isn't visible and your
+    // text-overflow-mode isn't clip.
+    bool hasTextOverflow = style()->textOverflow() && hasOverflowClip();
+
+    // Walk all the lines and delete our ellipsis line boxes if they exist.
+    if (hasTextOverflow)
+         deleteEllipsisLineBoxes();
+
     if (firstChild()) {
         // layout replaced elements
         RenderObject *o = first( this, bidi, false );
@@ -1581,13 +1583,14 @@ redo_linebreak:
             kDebug( 6040 ) << "Widows: " << widows << endl;
             // Check if we have enough orphans after respecting widows count
             int newOrphans = orphans - (style()->widows() - widows);
-            if (newOrphans < style()->orphans() && parent()->canClear(this,PageBreakHarder))
-            {
-                // Relayout to remove incorrect page-break
-                setNeedsPageClear(true);
-                setContainsPageBreak(false);
-                layoutInlineChildren(relayoutChildren, -1);
-                return;
+            if (newOrphans < style()->orphans()) {
+                if (parent()->canClear(this,PageBreakHarder)) {
+                    // Relayout to remove incorrect page-break
+                    setNeedsPageClear(true);
+                    setContainsPageBreak(false);
+                    layoutInlineChildren(relayoutChildren, -1);
+                    return;
+                }
             } else {
                 // Set hint and try again
                 layoutInlineChildren(relayoutChildren, newOrphans+1);
@@ -1606,8 +1609,15 @@ redo_linebreak:
     m_height += toAdd;
 
     // Always make sure this is at least our height.
-    if (m_overflowHeight < m_height)
-        m_overflowHeight = m_height;
+    m_overflowHeight = qMax(m_height, m_overflowHeight);
+
+    // See if any lines spill out of the block.  If so, we need to update our overflow width.
+    checkLinesForOverflow();
+
+    // See if we have any lines that spill out of our block.  If we do, then we will
+    // possibly need to truncate text.
+    if (hasTextOverflow)
+        checkLinesForTextOverflow();
 
 #if BIDI_DEBUG > 1
     kDebug(6041) << " ------- bidi end " << this << " -------" << endl;
@@ -1790,7 +1800,7 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
             KHTMLAssert(!o->firstChild());
             tmpW += o->marginLeft()+o->borderLeft()+o->paddingLeft()+
                     o->marginRight()+o->borderRight()+o->paddingRight();
-        } else if ( o->isReplaced() ) {
+        } else if ( o->isReplaced() || o->isGlyph() ) {
             EWhiteSpace currWS = o->style()->whiteSpace();
             EWhiteSpace lastWS = last->style()->whiteSpace();
 
@@ -1879,9 +1889,9 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
                         // Add the width up to but not including the hyphen.
                         tmpW += t->width(lastSpace, pos - lastSpace, f);
 
-                        // For whitespace normal only, include the hyphen.  We need to ensure it will fit
+                        // For wrapping text only, include the hyphen.  We need to ensure it will fit
                         // on the line if it shows when we break.
-                        if (o->style()->whiteSpace() == NORMAL)
+                        if (o->style()->autoWrap())
                             tmpW += t->width(pos, 1, f);
 
                         BidiIterator startMid(0, o, pos+1);
@@ -1896,23 +1906,26 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
 #ifdef APPLE_CHANGES    // KDE applies wordspacing differently
                 bool applyWordSpacing = false;
 #endif
-                if ( (preserveLF && c == '\n') || (autoWrap && (isBreakable( str, pos, strlen ) || isSoftBreakable)) ) {
-                    if (ignoringSpaces) {
-                        if (!currentCharacterIsSpace) {
-                            // Stop ignoring spaces and begin at this
-                            // new point.
-                            ignoringSpaces = false;
-                            lastSpace = pos; // e.g., "Foo    goo", don't add in any of the ignored spaces.
-                            BidiIterator startMid ( 0, o, pos );
-                            addMidpoint(startMid);
-                        }
-                        else {
-                            // Just keep ignoring these spaces.
-                            pos++;
-                            len--;
-                            continue;
-                        }
+                if (ignoringSpaces) {
+                    // We need to stop ignoring spaces, if we encounter a non-space or
+                    // a run that doesn't collapse spaces.
+                    if (!currentCharacterIsSpace || preserveWS) {
+                        // Stop ignoring spaces and begin at this
+                        // new point.
+                        ignoringSpaces = false;
+                        lastSpace = pos; // e.g., "Foo    goo", don't add in any of the ignored spaces.
+                        BidiIterator startMid ( 0, o, pos );
+                        addMidpoint(startMid);
                     }
+                    else {
+                        // Just keep ignoring these spaces.
+                        pos++;
+                        len--;
+                        continue;
+                    }
+                }
+
+                if ( (preserveLF && c == '\n') || (autoWrap && (isBreakable( str, pos, strlen ) || isSoftBreakable)) ) {
 
                     tmpW += t->width(lastSpace, pos - lastSpace, f);
                     if (!appliedStartWidth) {
@@ -1977,28 +1990,20 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
                     if (applyWordSpacing)
                         w += wordSpacing;
 #endif
-                    if (!ignoringSpaces && !preserveWS) {
-                        // If we encounter a newline, or if we encounter a
-                        // second space, we need to go ahead and break up this
-                        // run and enter a mode where we start collapsing spaces.
-                        if (currentCharacterIsSpace && previousCharacterIsSpace) {
-                            ignoringSpaces = true;
-
-                            // We just entered a mode where we are ignoring
-                            // spaces. Create a midpoint to terminate the run
-                            // before the second space.
-                            addMidpoint(ignoreStart);
-                            lastSpace = pos;
-                        }
-                    }
                 }
-                else if (ignoringSpaces) {
-                    // Stop ignoring spaces and begin at this
-                    // new point.
-                    ignoringSpaces = false;
-                    lastSpace = pos; // e.g., "Foo    goo", don't add in any of the ignored spaces.
-                    BidiIterator startMid ( 0, o, pos );
-                    addMidpoint(startMid);
+
+                if (!ignoringSpaces && !preserveWS) {
+                    // If we encounter a second space, we need to go ahead and break up this run
+                    // and enter a mode where we start collapsing spaces.
+                    if (currentCharacterIsSpace && previousCharacterIsSpace) {
+                        ignoringSpaces = true;
+
+                        // We just entered a mode where we are ignoring
+                        // spaces. Create a midpoint to terminate the run
+                        // before the second space.
+                        addMidpoint(ignoreStart);
+                        lastSpace = pos;
+                    }
                 }
 
                 if (currentCharacterIsSpace && !previousCharacterIsSpace) {
@@ -2028,7 +2033,7 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
         RenderObject* next = Bidinext(start.par, o, bidi);
         bool autoWrap = o->style()->autoWrap();
         bool checkForBreak = autoWrap;
-        if (w && w + tmpW > width && lBreak.obj && !o->style()->preserveLF())
+        if (w && w + tmpW > width && lBreak.obj && !o->style()->preserveLF() && !autoWrap)
             checkForBreak = true;
         else if (next && o->isText() && next->isText() && !next->isBR()) {
             if (autoWrap || next->style()->autoWrap()) {
@@ -2198,6 +2203,52 @@ BidiIterator RenderBlock::findNextLineBreak(BidiIterator &start, BidiState &bidi
     return lBreak;
 }
 
+void RenderBlock::checkLinesForOverflow()
+{
+    for (RootInlineBox* curr = static_cast<khtml::RootInlineBox*>(firstLineBox()); curr; curr = static_cast<khtml::RootInlineBox*>(curr->nextLineBox())) {
+//         m_overflowLeft = min(curr->leftOverflow(), m_overflowLeft);
+        m_overflowTop = qMin(curr->topOverflow(), m_overflowTop);
+//         m_overflowWidth = max(curr->rightOverflow(), m_overflowWidth);
+        m_overflowHeight = qMax(curr->bottomOverflow(), m_overflowHeight);
+    }
+}
+
+void RenderBlock::deleteEllipsisLineBoxes()
+{
+    for (RootInlineBox* curr = firstRootBox(); curr; curr = curr->nextRootBox())
+        curr->clearTruncation();
+}
+
+void RenderBlock::checkLinesForTextOverflow()
+{
+    // Determine the width of the ellipsis using the current font.
+    QChar ellipsis = 0x2026; // FIXME: CSS3 says this is configurable, also need to use 0x002E (FULL STOP) if 0x2026 not renderable
+    static QString ellipsisStr(ellipsis);
+    const Font& firstLineFont = style(true)->htmlFont();
+    const Font& font = style()->htmlFont();
+    int firstLineEllipsisWidth = firstLineFont.width(&ellipsis, 1, 0);
+    int ellipsisWidth = (font == firstLineFont) ? firstLineEllipsisWidth : font.width(&ellipsis, 1, 0);
+
+    // For LTR text truncation, we want to get the right edge of our padding box, and then we want to see
+    // if the right edge of a line box exceeds that.  For RTL, we use the left edge of the padding box and
+    // check the left edge of the line box to see if it is less
+    // Include the scrollbar for overflow blocks, which means we want to use "contentWidth()"
+    bool ltr = style()->direction() == LTR;
+    for (RootInlineBox* curr = firstRootBox(); curr; curr = curr->nextRootBox()) {
+        int blockEdge = ltr ? rightOffset(curr->yPos()) : leftOffset(curr->yPos());
+        int lineBoxEdge = ltr ? curr->xPos() + curr->width() : curr->xPos();
+        if ((ltr && lineBoxEdge > blockEdge) || (!ltr && lineBoxEdge < blockEdge)) {
+            // This line spills out of our box in the appropriate direction.  Now we need to see if the line
+            // can be truncated.  In order for truncation to be possible, the line must have sufficient space to
+            // accommodate our truncation string, and no replaced elements (images, tables) can overlap the ellipsis
+            // space.
+            int width = curr == firstRootBox() ? firstLineEllipsisWidth : ellipsisWidth;
+            if (curr->canAccommodateEllipsis(ltr, blockEdge, lineBoxEdge, width))
+                curr->placeEllipsis(ellipsisStr, ltr, blockEdge, width);
+        }
+    }
+}
+
 // For --enable-final
 #undef BIDI_DEBUG
 #undef DEBUG_LINEBREAKS
