@@ -24,17 +24,22 @@
 
 #include "rendering/render_canvas.h"
 #include "rendering/render_layer.h"
+#include "rendering/render_replaced.h"
 #include "xml/dom_docimpl.h"
 
 #include "khtmlview.h"
 #include "khtml_part.h"
 #include <kdebug.h>
 #include <kglobal.h>
+#include <QScrollBar>
 
 using namespace khtml;
 
 //#define BOX_DEBUG
 //#define SPEED_DEBUG
+#ifdef SPEED_DEBUG
+  #include <QTime>
+#endif
 
 RenderCanvas::RenderCanvas(DOM::NodeImpl* node, KHTMLView *view)
     : RenderBlock(node)
@@ -54,6 +59,7 @@ RenderCanvas::RenderCanvas(DOM::NodeImpl* node, KHTMLView *view)
 
     m_rootWidth = m_rootHeight = 0;
     m_viewportWidth = m_viewportHeight = 0;
+    m_cachedDocWidth = m_cachedDocHeight = -1;
 
     setPositioned(true); // to 0,0 :)
 
@@ -72,7 +78,7 @@ RenderCanvas::RenderCanvas(DOM::NodeImpl* node, KHTMLView *view)
     m_selectionEnd = 0;
     m_selectionStartPos = -1;
     m_selectionEndPos = -1;
-    
+
     m_needsWidgetMasks = false;
 
     // Create a new root layer for our layer hierarchy.
@@ -109,9 +115,7 @@ void RenderCanvas::calcWidth()
         return;
     }
 
-    m_width = m_view ?
-                m_view->frameWidth() + paddingLeft() + paddingRight() + borderLeft() + borderRight()
-                : m_minWidth;
+    m_width = m_view ? m_view->frameWidth() : m_minWidth;
 
     if (style()->marginLeft().isFixed())
         m_marginLeft = style()->marginLeft().value();
@@ -135,8 +139,6 @@ void RenderCanvas::calcMinMaxWidth()
     setMinMaxKnown();
 }
 
-//#define SPEED_DEBUG
-
 void RenderCanvas::layout()
 {
     if (m_pagedMode) {
@@ -151,25 +153,11 @@ void RenderCanvas::layout()
     for(RenderObject* c = firstChild(); c; c = c->nextSibling())
         c->setChildNeedsLayout(true);
 
-#ifdef SPEED_DEBUG
-    QTime qt;
-    qt.start();
-#endif
-    if ( recalcMinMax() )
-	recalcMinMaxWidths();
-
-#ifdef SPEED_DEBUG
-    kDebug() << "RenderCanvas::calcMinMax time used=" << qt.elapsed() << endl;
-    qt.start();
-#endif
-
-#ifdef SPEED_DEBUG
-    kDebug() << "RenderCanvas::layout time used=" << qt.elapsed() << endl;
-    qt.start();
-#endif
     int oldWidth = m_width;
     int oldHeight = m_height;
 
+    m_cachedDocWidth = m_cachedDocHeight = -1;
+
     if (m_pagedMode || !m_view) {
         m_width = m_rootWidth;
         m_height = m_rootHeight;
@@ -180,40 +168,94 @@ void RenderCanvas::layout()
         m_viewportHeight = m_height = m_view->visibleHeight();
     }
 
+#ifdef SPEED_DEBUG
+    QTime qt;
+    qt.start();
+#endif
+
+    if ( recalcMinMax() )
+	recalcMinMaxWidths();
+
+#ifdef SPEED_DEBUG
+    kDebug() << "RenderCanvas::calcMinMax time used=" << qt.elapsed() << endl;
+    qt.start();
+#endif
+
     bool relayoutChildren = (oldWidth != m_width) || (oldHeight != m_height);
 
     RenderBlock::layoutBlock( relayoutChildren );
 
-    int docW = docWidth();
-    int docH = docHeight();
+#ifdef SPEED_DEBUG
+    kDebug() << "RenderCanvas::layout time used=" << qt.elapsed() << endl;
+    qt.start();
+#endif
+
+    updateDocumentSize();
+
+    layer()->updateLayerPositions( layer(), needsFullRepaint(), true );
+
+    if (!m_pagedMode && m_needsWidgetMasks)
+        layer()->updateWidgetMasks(layer());
+
+    scheduleDeferredRepaints();
+    setNeedsLayout(false);
+
+#ifdef SPEED_DEBUG
+    kDebug() << "RenderCanvas::end time used=" << qt.elapsed() << endl;
+#endif
+}
+
+void RenderCanvas::setNeedsWidgetMasks( bool b ) 
+{
+    if (b == m_needsWidgetMasks)
+        return;
+    m_needsWidgetMasks = b;
+    KHTMLWidget* k = dynamic_cast<KHTMLWidget*>(m_view);
+    if (k)
+        k->m_kwp->setIsRedirected(!b);
+}
+
+void RenderCanvas::updateDocumentSize()
+{
+     // update our cached document size
+    int hDocH = m_cachedDocHeight = docHeight();
+    int hDocW = m_cachedDocWidth = docWidth();
+    
+    int zLevel = m_view? m_view->zoomLevel() : 100;
+    hDocW = hDocW*zLevel/100;
+    hDocH = hDocH*zLevel/100;
+
+    if (!m_pagedMode && m_view) {
 
-    if (!m_pagedMode) {
         bool vss = !m_view->verticalScrollBar()->isHidden();
         bool hss = !m_view->horizontalScrollBar()->isHidden();
-        QSize s = m_view->viewportSize(docW, docH);
-
-        // content size, with scrollbar hysteresis
-        int hDocH = docH;
-        int hDocW = docW;
+        QSize s = m_view->maximumViewportSize();
 
+        int zoomedDocWidth = m_cachedDocWidth*zLevel/100;
+        int zoomedDocHeight = m_cachedDocHeight*zLevel/100;
+        if ( zoomedDocWidth > s.width() )
+            s.setWidth( s.width()-m_view->verticalScrollBar()->sizeHint().width() );
+        if ( zoomedDocHeight > s.height() )
+            s.setHeight( s.height()-m_view->horizontalScrollBar()->sizeHint().height() );
+   
         // if we are about to show a scrollbar, and the document is sized to the viewport w or h,
         // then reserve the scrollbar space so that it doesn't trigger the _other_ scrollbar
 
         if (!vss && m_width - m_view->verticalScrollBar()->sizeHint().width() == s.width() &&
-            docW <= m_width)
-            hDocW = qMin( docW, s.width() );
+            zoomedDocWidth <= m_width)
+            hDocW = qMin( zoomedDocWidth, s.width() );
 
         if (!hss && m_height - m_view->horizontalScrollBar()->sizeHint().height() == s.height() &&
-            docH <= m_height)
-            hDocH = qMin( docH, s.height() );
+            zoomedDocHeight <= m_height)
+            hDocH = qMin( zoomedDocHeight, s.height() );
 
         // likewise, if a scrollbar is shown, and we have a cunning plan to turn it off,
         // think again if we are falling downright in the hysteresis zone
 
-        if (vss && s.width() > docW && docW > m_view->visibleWidth())
+        if (vss && s.width() > zoomedDocWidth && zoomedDocWidth > m_view->visibleWidth())
             hDocW = s.width()+1;
 
-        if (hss && s.height() > docH && docH > m_view->visibleHeight())
+        if (hss && s.height() > zoomedDocHeight && zoomedDocHeight > m_view->visibleHeight())
             hDocH = s.height()+1;
 
         m_view->resizeContents(hDocW, hDocH);
@@ -221,21 +263,29 @@ void RenderCanvas::layout()
         setWidth( m_viewportWidth = s.width() );
         setHeight( m_viewportHeight = s.height() );
     }
+    layer()->resize( qMax( m_cachedDocWidth,int( m_width ) ), qMax( m_cachedDocHeight,m_height ) );
+}
 
-#ifdef SPEED_DEBUG
-    kDebug() << "RenderCanvas::end time used=" << qt.elapsed() << endl;
-#endif
-
-//     kDebug(6040) << "RenderCanvas::resize layer to " << qMax( docW,int( m_width ) ) << "x"  << qMax( docH,m_height ) << endl;
-
-    layer()->resize( qMax( docW,int( m_width ) ), qMax( docH,m_height ) );
-    layer()->updateLayerPositions( layer(), needsFullRepaint(), true );
-    
-    if (!m_pagedMode && m_needsWidgetMasks)
-        layer()->updateWidgetMasks(layer());
-
-    scheduleDeferredRepaints();
-    setNeedsLayout(false);
+void RenderCanvas::updateDocSizeAfterLayerTranslation( RenderObject* o, bool posXOffset, bool posYOffset )
+{
+    if (needsLayout())
+        return;
+    int rightmost, lowest;
+    o->absolutePosition( rightmost, lowest );
+    if (posXOffset) {
+        rightmost += o->rightmostPosition(false, true);
+        setCachedDocWidth( qMax(docWidth(), rightmost) );
+    } else {
+        setCachedDocWidth( -1 );
+    }
+    if (posYOffset) {
+        lowest += o->lowestPosition(false, true);
+        setCachedDocHeight( qMax(docHeight(), lowest) );
+    } else {
+        setCachedDocHeight( -1 );
+    }
+//    kDebug() << " posXOffset: " << posXOffset << " posYOffset " << posYOffset << " m_cachedDocWidth  " <<  m_cachedDocWidth << " m_cachedDocHeight  " << m_cachedDocHeight << endl;
+    updateDocumentSize();
 }
 
 bool RenderCanvas::needsFullRepaint() const
@@ -243,13 +293,13 @@ bool RenderCanvas::needsFullRepaint() const
     return m_needsFullRepaint || m_pagedMode;
 }
 
-void RenderCanvas::repaintViewRectangle(int x, int y, int w, int h)
+void RenderCanvas::repaintViewRectangle(int x, int y, int w, int h, bool asap)
 {
   KHTMLAssert( view() );
-  view()->scheduleRepaint( x, y, w, h );
+  view()->scheduleRepaint( x, y, w, h, asap );
 }
 
-bool RenderCanvas::absolutePosition(int &xPos, int &yPos, bool f)
+bool RenderCanvas::absolutePosition(int &xPos, int &yPos, bool f) const
 {
     if ( f && m_pagedMode) {
         xPos = 0;
@@ -306,7 +356,7 @@ void RenderCanvas::paintBoxDecorations(PaintInfo& paintInfo, int /*_tx*/, int /*
     paintInfo.p->fillRect(paintInfo.r, view()->palette().color(QPalette::Active, QPalette::Base));
 }
 
-void RenderCanvas::repaintRectangle(int x, int y, int w, int h, bool immediate, bool f)
+void RenderCanvas::repaintRectangle(int x, int y, int w, int h, Priority p, bool f)
 {
     if (m_staticMode) return;
 //    kDebug( 6040 ) << "updating views contents (" << x << "/" << y << ") (" << w << "/" << h << ")" << endl;
@@ -324,11 +374,10 @@ void RenderCanvas::repaintRectangle(int x, int y, int w, int h, bool immediate,
 
     if (m_view && ur.intersects(vr)) {
 
-        if (immediate)
-	// ### KWQ's updateContents has an additional parameter "now".
-	// It's not clear what the difference between updateContents(...,true)
-	// and repaintContents(...) is. As Qt doesn't have this, I'm leaving it out. (LS)
-            m_view->updateContents(ur/*, true*/);
+        if (p == RealtimePriority)
+            m_view->updateContents(ur);
+        else if (p == HighPriority)
+            m_view->scheduleRepaint(x, y, w, h, true /*asap*/);
         else
             m_view->scheduleRepaint(x, y, w, h);
     }
@@ -350,10 +399,10 @@ void RenderCanvas::scheduleDeferredRepaints()
     m_dirtyChildren.clear();
 }
 
-void RenderCanvas::repaint(bool immediate)
+void RenderCanvas::repaint(Priority p)
 {
     if (m_view && !m_staticMode) {
-        if (immediate) {
+        if (p == RealtimePriority) {
             //m_view->resizeContents(docWidth(), docHeight());
             m_view->unscheduleRepaint();
             if (needsLayout()) {
@@ -362,8 +411,11 @@ void RenderCanvas::repaint(bool immediate)
             }
 	    // ### same as in repaintRectangle
             m_view->updateContents(m_view->contentsX(), m_view->contentsY(),
-                                   m_view->visibleWidth(), m_view->visibleHeight()/*, true*/);
+                                   m_view->visibleWidth(), m_view->visibleHeight());
         }
+        else if (p == HighPriority)
+            m_view->scheduleRepaint(m_view->contentsX(), m_view->contentsY(),
+                                    m_view->visibleWidth(), m_view->visibleHeight(), true /*asap*/);
         else
             m_view->scheduleRepaint(m_view->contentsX(), m_view->contentsY(),
                                     m_view->visibleWidth(), m_view->visibleHeight());
@@ -377,10 +429,11 @@ static QRect enclosingPositionedRect (RenderObject *n)
     if (enclosingParent) {
         int ox, oy;
         enclosingParent->absolutePosition(ox, oy);
-        int off = 0;
-        if (!enclosingParent->hasOverflowClip())
-            off = enclosingParent->negativeOverflowWidth();
-        rect.setX(ox - off);
+        if (!enclosingParent->hasOverflowClip()) {
+            ox += enclosingParent->overflowLeft();
+            oy += enclosingParent->overflowTop();
+        }
+        rect.setX(ox);
         rect.setY(oy);
         rect.setWidth(enclosingParent->effectiveWidth());
         rect.setHeight(enclosingParent->effectiveHeight());
@@ -442,8 +495,8 @@ void RenderCanvas::setSelection(RenderObject *s, int sp, RenderObject *e, int ep
     int oldStartPos = m_selectionStartPos;
     RenderObject *oldEnd = m_selectionEnd;
     int oldEndPos = m_selectionEndPos;
-    Q3PtrList<RenderObject> oldSelectedInside;
-    Q3PtrList<RenderObject> newSelectedInside;
+    QList<RenderObject*> oldSelectedInside;
+    QList<RenderObject*> newSelectedInside;
     RenderObject *os = oldStart;
 
     while (os && os != oldEnd)
@@ -459,7 +512,7 @@ void RenderCanvas::setSelection(RenderObject *s, int sp, RenderObject *e, int ep
                     no = no->nextSibling();
             }
         }
-        if (os->selectionState() == SelectionInside && !oldSelectedInside.containsRef(os))
+        if (os->selectionState() == SelectionInside && !oldSelectedInside.contains(os))
             oldSelectedInside.append(os);
 
         os = no;
@@ -517,7 +570,7 @@ void RenderCanvas::setSelection(RenderObject *s, int sp, RenderObject *e, int ep
                 if (no)
                     no = no->nextSibling();
             }
-        if (o->selectionState() == SelectionInside && !newSelectedInside.containsRef(o))
+        if (o->selectionState() == SelectionInside && !newSelectedInside.contains(o))
             newSelectedInside.append(o);
 
         o=no;
@@ -529,8 +582,11 @@ void RenderCanvas::setSelection(RenderObject *s, int sp, RenderObject *e, int ep
     if (!m_view)
         return;
 
-    newSelectedInside.removeRef(s);
-    newSelectedInside.removeRef(e);
+    int i;
+    i = newSelectedInside.indexOf(s);
+    if (i != -1) newSelectedInside.removeAt(i);
+    i = newSelectedInside.indexOf(e);
+    if (i != -1) newSelectedInside.removeAt(i);
 
     QRect updateRect;
 
@@ -545,16 +601,17 @@ void RenderCanvas::setSelection(RenderObject *s, int sp, RenderObject *e, int ep
     // If so we have to draw them.
     // Could be faster by building list of non-intersecting rectangles rather
     // than unioning rectangles.
-    Q3PtrListIterator<RenderObject> oldIterator(oldSelectedInside);
+    QListIterator<RenderObject*> oldIterator(oldSelectedInside);
     bool firstRect = true;
-    for (; oldIterator.current(); ++oldIterator){
-        if (!newSelectedInside.containsRef(oldIterator.current())){
+    while (oldIterator.hasNext()) {
+        o = oldIterator.next();
+        if (!newSelectedInside.contains(o)){
             if (firstRect){
-                updateRect = enclosingPositionedRect(oldIterator.current());
+                updateRect = enclosingPositionedRect(o);
                 firstRect = false;
             }
             else
-                updateRect = updateRect.unite(enclosingPositionedRect(oldIterator.current()));
+                updateRect = updateRect.unite(enclosingPositionedRect(o));
         }
     }
     if (!firstRect){
@@ -565,16 +622,17 @@ void RenderCanvas::setSelection(RenderObject *s, int sp, RenderObject *e, int ep
     // If so we have to draw them.
     // Could be faster by building list of non-intersecting rectangles rather
     // than unioning rectangles.
-    Q3PtrListIterator<RenderObject> newIterator(newSelectedInside);
+    QListIterator<RenderObject*> newIterator(newSelectedInside);
     firstRect = true;
-    for (; newIterator.current(); ++newIterator){
-        if (!oldSelectedInside.containsRef(newIterator.current())){
-            if (firstRect){
-                updateRect = enclosingPositionedRect(newIterator.current());
+    while (newIterator.hasNext()) {
+        o = newIterator.next();
+        if (!oldSelectedInside.contains(o)) {
+            if (firstRect) {
+                updateRect = enclosingPositionedRect(o);
                 firstRect = false;
             }
             else
-                updateRect = updateRect.unite(enclosingPositionedRect(newIterator.current()));
+                updateRect = updateRect.unite(enclosingPositionedRect(o));
         }
     }
     if (!firstRect) {
@@ -679,6 +737,9 @@ QRect RenderCanvas::viewRect() const
 
 int RenderCanvas::docHeight() const
 {
+    if (m_cachedDocHeight != -1)
+        return m_cachedDocHeight;
+
     int h;
     if (m_pagedMode || !m_view)
         h = m_height;
@@ -707,6 +768,9 @@ int RenderCanvas::docHeight() const
 
 int RenderCanvas::docWidth() const
 {
+    if (m_cachedDocWidth != -1)
+        return m_cachedDocWidth;
+
     int w;
     if (m_pagedMode || !m_view)
         w = m_width;
@@ -716,7 +780,7 @@ int RenderCanvas::docWidth() const
     RenderObject *fc = firstChild();
     if(fc) {
         // ow: like effectiveWidth() but without the negative
-        const int ow = hasOverflowClip() ? width() : overflowWidth();
+        const int ow = fc->hasOverflowClip() ? fc->width() : fc->overflowWidth();
         int dw = ow + fc->marginLeft() + fc->marginRight();
         int rightmostPos = fc->rightmostPosition(false);
 // kDebug(6040) << "w " << w << " rightmostPos " << rightmostPos << " dw " << dw << " fc->rw " << fc->effectiveWidth() << " fc->width() " << fc->width() << endl;
