@@ -5,6 +5,7 @@
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2000 Dirk Mueller (mueller@kde.org)
  *           (C) 2006 Maksim Orlovich (maksim@kde.org)
+ *           (C) 2006 Germain Garand (germain@ebooksfrance.org)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -37,10 +38,11 @@
 #include <k3spell.h>
 #include <kurlcompletion.h>
 #include <kwin.h>
-#include <kstdaction.h>
+#include <kstandardaction.h>
 #include <kactioncollection.h>
 
 #include <QAbstractItemView>
+#include <QAbstractTextDocumentLayout>
 #include <qstyle.h>
 #include <QStyleOptionButton>
 
@@ -56,7 +58,7 @@
 #include "khtml_ext.h"
 #include "xml/dom_docimpl.h"
 
-#include <q3popupmenu.h>
+#include <qmenu.h>
 #include <qbitmap.h>
 
 using namespace khtml;
@@ -143,8 +145,7 @@ short RenderButton::baselinePosition( bool f ) const
 RenderCheckBox::RenderCheckBox(HTMLInputElementImpl *element)
     : RenderButton(element)
 {
-    QCheckBox* b = new QCheckBox(view()->viewport());
-    b->setObjectName( "__khtml" );
+    CheckBoxWidget* b = new CheckBoxWidget(view()->widget());
     //b->setAutoMask(true);
     b->setMouseTracking(true);
     setQWidget(b);
@@ -186,13 +187,27 @@ void RenderCheckBox::slotStateChanged(int state)
     deref();
 }
 
+bool RenderCheckBox::handleEvent(const DOM::EventImpl& ev)
+{
+    switch(ev.id()) {
+      case EventImpl::DOMFOCUSIN_EVENT:
+      case EventImpl::DOMFOCUSOUT_EVENT:
+      case EventImpl::MOUSEMOVE_EVENT:
+      case EventImpl::MOUSEOUT_EVENT:
+      case EventImpl::MOUSEOVER_EVENT:
+          return RenderButton::handleEvent(ev);
+      default:
+          break;
+    }
+    return false;
+}
+
 // -------------------------------------------------------------------------------
 
 RenderRadioButton::RenderRadioButton(HTMLInputElementImpl *element)
     : RenderButton(element)
 {
-    QRadioButton* b = new QRadioButton(view()->viewport());
-    b->setObjectName( "__khtml" );
+    RadioButtonWidget* b = new RadioButtonWidget(view()->widget());
     b->setMouseTracking(true);
     b->setAutoExclusive(false);
     setQWidget(b);
@@ -232,14 +247,28 @@ void RenderRadioButton::slotToggled(bool activated)
     }
 }
 
+bool RenderRadioButton::handleEvent(const DOM::EventImpl& ev)
+{
+    switch(ev.id()) {
+      case EventImpl::DOMFOCUSIN_EVENT:
+      case EventImpl::DOMFOCUSOUT_EVENT:
+      case EventImpl::MOUSEMOVE_EVENT:
+      case EventImpl::MOUSEOUT_EVENT:
+      case EventImpl::MOUSEOVER_EVENT:
+          return RenderButton::handleEvent(ev);
+      default:
+          break;
+    }
+    return false;
+}
+
 // -------------------------------------------------------------------------------
 
 
 RenderSubmitButton::RenderSubmitButton(HTMLInputElementImpl *element)
     : RenderButton(element)
 {
-    QPushButton* p = new QPushButton(view()->viewport());
-    p->setObjectName( "__khtml" );
+    PushButtonWidget* p = new PushButtonWidget(view()->widget());
     setQWidget(p);
     //p->setAutoMask(true);
     p->setMouseTracking(true);
@@ -323,10 +352,10 @@ RenderResetButton::RenderResetButton(HTMLInputElementImpl *element)
 LineEditWidget::LineEditWidget(DOM::HTMLInputElementImpl* input, KHTMLView* view, QWidget* parent)
     : KLineEdit(parent), m_input(input), m_view(view), m_spell(0)
 {
-    setObjectName( "__khtml" );
+    m_kwp->setIsRedirected( true );
     setMouseTracking(true);
     KActionCollection *ac = new KActionCollection(this);
-    m_spellAction = KStdAction::spelling( this, SLOT( slotCheckSpelling() ), ac );
+    m_spellAction = KStandardAction::spelling( this, SLOT( slotCheckSpelling() ), ac );
 }
 
 LineEditWidget::~LineEditWidget()
@@ -390,16 +419,12 @@ void LineEditWidget::contextMenuEvent(QContextMenuEvent *e)
 {
     QMenu* popup = createStandardContextMenu();
 
-/*    menu->exec(e->globalPos());
-            delete menu;*/
-
-    if ( !popup ) {
+    if ( !popup )
         return;
-    }
 
     if (m_input->autoComplete()) {
         popup->addSeparator();
-        QAction* act = popup->addAction( SmallIconSet("history_clear"), i18n("Clear &History"));
+        QAction* act = popup->addAction( KIcon("history_clear"), i18n("Clear &History"));
         act->setEnabled(compObj() && !compObj()->isEmpty());
         connect(act, SIGNAL(triggered()),
                 this, SLOT(clearHistoryActivated()));
@@ -412,6 +437,8 @@ void LineEditWidget::contextMenuEvent(QContextMenuEvent *e)
         popup->addAction( m_spellAction );
         m_spellAction->setEnabled( !text().isEmpty() );
     }
+    popup->exec(e->globalPos());
+    delete popup;
 }
 
 
@@ -476,7 +503,7 @@ void LineEditWidget::mouseMoveEvent(QMouseEvent *e)
 RenderLineEdit::RenderLineEdit(HTMLInputElementImpl *element)
     : RenderFormElement(element)
 {
-    LineEditWidget *edit = new LineEditWidget(element, view(), view()->viewport());
+    LineEditWidget *edit = new LineEditWidget(element, view(), view()->widget());
     connect(edit,SIGNAL(returnPressed()), this, SLOT(slotReturnPressed()));
     connect(edit,SIGNAL(textChanged(const QString &)),this,SLOT(slotTextChanged(const QString &)));
 
@@ -515,7 +542,7 @@ void RenderLineEdit::slotReturnPressed()
     // don't submit the form when return was pressed in a completion-popup
     KCompletionBox *box = widget()->completionBox(false);
 
-    if ( box && box->isVisible() && box->currentItem() != -1 ) {
+    if ( box && box->isVisible() && box->currentRow() != -1 ) {
       box->hide();
       return;
     }
@@ -568,8 +595,8 @@ void RenderLineEdit::calcMinMaxWidth()
 void RenderLineEdit::updateFromElement()
 {
     int ml = element()->maxLength();
-    if ( ml < 0 || ml > 1024 )
-        ml = 1024;
+    if ( ml < 0 )
+        ml = 32767;
 
     if ( widget()->maxLength() != ml )  {
         widget()->setMaxLength( ml );
@@ -699,11 +726,8 @@ void RenderFieldset::paintBoxDecorations(PaintInfo& pI, int _tx, int _ty)
     h -= yOff;
     _ty += yOff - borderTopExtra();
 
-    int my = qMax(_ty,pI.r.y());
-    int end = qMin( pI.r.y() + pI.r.height(),  _ty + h );
-    int mh = end - my;
-
-    paintBackground(pI.p, style()->backgroundColor(), style()->backgroundLayers(), my, mh, _tx, _ty, w, h);
+    QRect cr = QRect(_tx, _ty, w, h).intersected( pI.r );
+    paintBackground(pI.p, style()->backgroundColor(), style()->backgroundLayers(), cr, _tx, _ty, w, h);
 
     if ( style()->hasBorder() )
 	    paintBorderMinusLegend(pI.p, _tx, _ty, w, h, style(), legend->xPos(), legend->width());
@@ -794,14 +818,14 @@ void RenderFieldset::setStyle(RenderStyle* _style)
 RenderFileButton::RenderFileButton(HTMLInputElementImpl *element)
     : RenderFormElement(element)
 {
-    KUrlRequester* w = new KUrlRequester( view()->viewport() );
-    w->setObjectName( "__khtml" );
+    FileButtonWidget* w = new FileButtonWidget( view()->widget() );
 
     w->setMode(KFile::File | KFile::ExistingOnly);
     w->completionObject()->setDir(KGlobalSettings::documentPath());
 
     connect(w->lineEdit(), SIGNAL(returnPressed()), this, SLOT(slotReturnPressed()));
     connect(w->lineEdit(), SIGNAL(textChanged(const QString &)),this,SLOT(slotTextChanged(const QString &)));
+    connect(w, SIGNAL(urlSelected(const QString &)),this,SLOT(slotUrlSelected(const QString &)));
 
     setQWidget(w);
     m_haveFocus = false;
@@ -817,7 +841,7 @@ void RenderFileButton::calcMinMaxWidth()
     int size = element()->size();
 
     int h = fm.lineSpacing();
-    int w = fm.width( 'x' ) * (size > 0 ? size : 17); // "some"
+    int w = fm.width( 'x' ) * (size > 0 ? size+1 : 17); // "some"
     KLineEdit* edit = static_cast<KUrlRequester*>( m_widget )->lineEdit();
 
     QStyleOption opt;
@@ -826,7 +850,7 @@ void RenderFileButton::calcMinMaxWidth()
                                              &opt,
           QSize(w + 2 + 2, qMax(h, 14) + 2 + 2), edit)
         .expandedTo(QApplication::globalStrut());
-    QSize bs = static_cast<KUrlRequester*>( m_widget )->sizeHint();
+    QSize bs = static_cast<KUrlRequester*>( m_widget )->minimumSizeHint() - edit->minimumSizeHint();
 
     setIntrinsicWidth( s.width() + bs.width() );
     setIntrinsicHeight( qMax(s.height(), bs.height()) );
@@ -866,6 +890,11 @@ void RenderFileButton::slotTextChanged(const QString &/*string*/)
    element()->m_value = KUrl( widget()->url() ).pathOrUrl();
 }
 
+void RenderFileButton::slotUrlSelected(const QString &)
+{
+	element()->onChange();
+}
+
 void RenderFileButton::select()
 {
     widget()->lineEdit()->selectAll();
@@ -891,12 +920,32 @@ RenderLegend::RenderLegend(HTMLGenericFormElementImpl *element)
 ComboBoxWidget::ComboBoxWidget(QWidget *parent)
     : KComboBox(false, parent)
 {
-    setObjectName( "__khtml" );
+    m_kwp->setIsRedirected( true );
     //setAutoMask(true);
     if (view()) view()->installEventFilter(this);
     setMouseTracking(true);
 }
 
+void ComboBoxWidget::showPopup()
+{
+    QPoint p = pos();
+    blockSignals(true);
+    move( m_kwp->absolutePos() );
+    blockSignals(false);
+
+    KComboBox::showPopup();
+
+    blockSignals(true);
+    move( p );
+    blockSignals(false);
+}
+
+void ComboBoxWidget::hidePopup()
+{
+    KComboBox::hidePopup();
+}
+
+
 bool ComboBoxWidget::event(QEvent *e)
 {
     if (KComboBox::event(e))
@@ -931,7 +980,7 @@ bool ComboBoxWidget::eventFilter(QObject *dest, QEvent *e)
 	case Qt::Key_Backtab:
 	    // ugly hack. emulate popdownlistbox() (private in QComboBox)
 	    // we re-use ke here to store the reference to the generated event.
-	    ke = new QKeyEvent(QEvent::KeyPress, Qt::Key_Escape, 0, 0);
+	    ke = new QKeyEvent(QEvent::KeyPress, Qt::Key_Escape, Qt::NoModifier);
 	    QApplication::sendEvent(dest,ke);
 	    focusNextPrevChild(forward);
 	    delete ke;
@@ -984,7 +1033,9 @@ void RenderSelect::updateFromElement()
         }
 
         if (m_useListBox && oldMultiple != m_multiple) {
-            static_cast<KListBox*>(m_widget)->setSelectionMode(m_multiple ? Q3ListBox::Extended : Q3ListBox::Single);
+            static_cast<KListWidget*>(m_widget)->setSelectionMode(m_multiple ? 
+                                            QListWidget::ExtendedSelection 
+                                          : QListWidget::SingleSelection);
         }
         m_selectionChanged = true;
         m_optionsChanged = true;
@@ -998,7 +1049,7 @@ void RenderSelect::updateFromElement()
         int listIndex;
 
         if(m_useListBox) {
-            static_cast<KListBox*>(m_widget)->clear();
+            static_cast<KListWidget*>(m_widget)->clear();
         }
 
         else
@@ -1011,10 +1062,10 @@ void RenderSelect::updateFromElement()
                     text = "";
 
                 if(m_useListBox) {
-                    Q3ListBoxText *item = new Q3ListBoxText(QString(text.implementation()->s, text.implementation()->l));
-                    static_cast<KListBox*>(m_widget)
-                        ->insertItem(item, listIndex);
-                    item->setSelectable(false);
+                    QListWidgetItem *item = new QListWidgetItem(QString(text.implementation()->s, text.implementation()->l));
+                    static_cast<KListWidget*>(m_widget)
+                        ->insertItem(listIndex,item);
+                    item->setFlags(item->flags() & ~Qt::ItemIsSelectable);
                 }
                 else {
                     static_cast<KComboBox*>(m_widget)
@@ -1040,11 +1091,12 @@ void RenderSelect::updateFromElement()
                 }
 
                 if(m_useListBox) {
-                    KListBox *l = static_cast<KListBox*>(m_widget);
-                    l->insertItem(text, listIndex);
+                    KListWidget *l = static_cast<KListWidget*>(m_widget);
+                    l->insertItem(listIndex,text);
                     DOMString disabled = optElem->getAttribute(ATTR_DISABLED);
                     if (!disabled.isNull() && l->item( listIndex )) {
-                        l->item( listIndex )->setSelectable( false );
+                        l->item( listIndex )->setFlags( l->item(listIndex)->flags() 
+                                                            & ~Qt::ItemIsSelectable );
                     }
                 }  else
                     static_cast<KComboBox*>(m_widget)->insertItem(listIndex, text);
@@ -1101,15 +1153,15 @@ void RenderSelect::layout( )
 
     // calculate size
     if(m_useListBox) {
-        KListBox* w = static_cast<KListBox*>(m_widget);
+        KListWidget* w = static_cast<KListWidget*>(m_widget);
 
-        Q3ListBoxItem* p = w->firstItem();
         int width = 0;
         int height = 0;
-        while(p) {
-            width = qMax(width, p->width(p->listBox()));
-            height = qMax(height, p->height(p->listBox()));
-            p = p->next();
+
+        for ( int rowIndex = 0 ; rowIndex < w->count() ; rowIndex++ ) {
+            QListWidgetItem* p = w->item(0);
+            width = qMax(width, p->listWidget()->visualItemRect(p).width());
+            height = qMax(height, p->listWidget()->visualItemRect(p).height());
         }
         if ( !height )
             height = w->fontMetrics().height();
@@ -1123,7 +1175,7 @@ void RenderSelect::layout( )
         // the average of that is IMHO qMin(number of elements, 10)
         // so I did that ;-)
         if(size < 1)
-            size = qMin(static_cast<KListBox*>(m_widget)->count(), 10u);
+            size = qMin(static_cast<KListWidget*>(m_widget)->count(), 10);
 
         width += 2*w->frameWidth() + w->verticalScrollBar()->sizeHint().width();
         height = size*height + 2*w->frameWidth();
@@ -1226,7 +1278,7 @@ void RenderSelect::slotSelectionChanged() // emitted by the listbox only
         // again with updateSelection.
         if ( listItems[i]->id() == ID_OPTION )
             static_cast<HTMLOptionElementImpl*>( listItems[i] )
-                ->m_selected = static_cast<KListBox*>( m_widget )->isSelected( i );
+                ->m_selected = static_cast<KListWidget*>( m_widget )->item(i)->isSelected();
 
     ref();
     element()->onChange();
@@ -1238,15 +1290,11 @@ void RenderSelect::setOptionsChanged(bool _optionsChanged)
     m_optionsChanged = _optionsChanged;
 }
 
-KListBox* RenderSelect::createListBox()
+ListBoxWidget* RenderSelect::createListBox()
 {
-    KListBox *lb = new KListBox(view()->viewport());
-    lb->setObjectName( "__khtml" );
-    lb->setSelectionMode(m_multiple ? Q3ListBox::Extended : Q3ListBox::Single);
-    // ### looks broken
-    //lb->setAutoMask(true);
+    ListBoxWidget *lb = new ListBoxWidget(view()->widget());
+    lb->setSelectionMode(m_multiple ? QListWidget::ExtendedSelection : QListWidget::SingleSelection);
     connect( lb, SIGNAL( selectionChanged() ), this, SLOT( slotSelectionChanged() ) );
-//     connect( lb, SIGNAL( clicked( QListBoxItem * ) ), this, SLOT( slotClicked() ) );
     m_ignoreSelectEvents = false;
     lb->setMouseTracking(true);
 
@@ -1255,7 +1303,7 @@ KListBox* RenderSelect::createListBox()
 
 ComboBoxWidget *RenderSelect::createComboBox()
 {
-    ComboBoxWidget *cb = new ComboBoxWidget(view()->viewport());
+    ComboBoxWidget *cb = new ComboBoxWidget(view()->widget());
     connect(cb, SIGNAL(activated(int)), this, SLOT(slotSelected(int)));
     return cb;
 }
@@ -1266,9 +1314,9 @@ void RenderSelect::updateSelection()
     int i;
     if (m_useListBox) {
         // if multi-select, we select only the new selected index
-        KListBox *listBox = static_cast<KListBox*>(m_widget);
+        KListWidget *listBox = static_cast<KListWidget*>(m_widget);
         for (i = 0; i < int(listItems.size()); i++)
-            listBox->setSelected(i,listItems[i]->id() == ID_OPTION &&
+            listBox->item(i)->setSelected(listItems[i]->id() == ID_OPTION &&
                                  static_cast<HTMLOptionElementImpl*>(listItems[i])->selected());
     }
     else {
@@ -1296,30 +1344,35 @@ void RenderSelect::updateSelection()
 // -------------------------------------------------------------------------
 
 TextAreaWidget::TextAreaWidget(int wrap, QWidget* parent)
-    : K3TextEdit(parent), m_findDlg(0), m_find(0), m_repDlg(0), m_replace(0)
+    : KTextEdit(parent), m_findDlg(0), m_find(0), m_repDlg(0), m_replace(0)
 {
-    setObjectName( "__khtml" );
+    m_kwp->setIsRedirected( true );
     if(wrap != DOM::HTMLTextAreaElementImpl::ta_NoWrap) {
-        setWordWrap(Q3TextEdit::WidgetWidth);
-        setHScrollBarMode( AlwaysOff );
-        setVScrollBarMode( AlwaysOn );
+        setLineWrapMode(QTextEdit::WidgetWidth);
+        setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
+        setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOn );
     }
     else {
-        setWordWrap(Q3TextEdit::NoWrap);
-        setHScrollBarMode( Auto );
-        setVScrollBarMode( Auto );
+        setLineWrapMode(QTextEdit::NoWrap);
+        setHorizontalScrollBarPolicy( Qt::ScrollBarAsNeeded );
+        setVerticalScrollBarPolicy( Qt::ScrollBarAsNeeded );
     }
     KCursor::setAutoHideCursor(viewport(), true);
-    setTextFormat(Qt::PlainText);
-    //setAutoMask(true);
+    setAcceptRichText (false);
     setMouseTracking(true);
 
     KActionCollection *ac = new KActionCollection(this);
-    m_findAction = KStdAction::find( this, SLOT( slotFind() ), ac );
-    m_findNextAction = KStdAction::findNext( this, SLOT( slotFindNext() ), ac );
-    m_replaceAction = KStdAction::replace( this, SLOT( slotReplace() ), ac );
+    m_findAction = KStandardAction::find( this, SLOT( slotFind() ), ac );
+    m_findNextAction = KStandardAction::findNext( this, SLOT( slotFindNext() ), ac );
+    m_replaceAction = KStandardAction::replace( this, SLOT( slotReplace() ), ac );
 }
 
+void TextAreaWidget::scrollContentsBy( int dx, int dy )
+{
+    KTextEdit::scrollContentsBy(dx, dy);
+    update();
+    
+}
 
 TextAreaWidget::~TextAreaWidget()
 {
@@ -1333,29 +1386,25 @@ TextAreaWidget::~TextAreaWidget()
     m_findDlg = 0L;
 }
 
-
-Q3PopupMenu *TextAreaWidget::createPopupMenu(const QPoint& pos)
+void TextAreaWidget::contextMenuEvent(QContextMenuEvent * e)
 {
-    Q3PopupMenu *popup = K3TextEdit::createPopupMenu(pos);
-
-    if ( !popup ) {
-        return 0L;
-    }
+    QMenu *popup = createStandardContextMenu();
 
     if (!isReadOnly()) {
         popup->addSeparator();
 
         popup->addAction( m_findAction );
-        m_findAction->setEnabled( !text().isEmpty() );
+        m_findAction->setEnabled( !toPlainText().isEmpty() );
 
         popup->addAction( m_findNextAction );
         m_findNextAction->setEnabled( m_find != 0 );
 
         popup->addAction( m_replaceAction );
-        m_replaceAction->setEnabled( !text().isEmpty() );
+        m_replaceAction->setEnabled( !toPlainText().isEmpty() );
     }
 
-    return popup;
+    popup->exec(e->globalPos());
+    delete popup;
 }
 
 
@@ -1363,13 +1412,10 @@ void TextAreaWidget::slotFindHighlight(const QString& text, int matchingIndex, i
 {
     Q_UNUSED(text)
     //kDebug() << "Highlight: [" << text << "] mi:" << matchingIndex << " ml:" << matchingLength << endl;
-    if (sender() == m_replace) {
-        setSelection(m_repPara, matchingIndex, m_repPara, matchingIndex + matchingLength);
-        setCursorPosition(m_repPara, matchingIndex);
-    } else {
-        setSelection(m_findPara, matchingIndex, m_findPara, matchingIndex + matchingLength);
-        setCursorPosition(m_findPara, matchingIndex);
-    }
+    QTextCursor tc = textCursor();
+    tc.setPosition(matchingIndex);
+    tc.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor, matchingLength);
+    setTextCursor(tc);
     ensureCursorVisible();
 }
 
@@ -1377,9 +1423,12 @@ void TextAreaWidget::slotFindHighlight(const QString& text, int matchingIndex, i
 void TextAreaWidget::slotReplaceText(const QString &text, int replacementIndex, int /*replacedLength*/, int matchedLength) {
     Q_UNUSED(text)
     //kDebug() << "Replace: [" << text << "] ri:" << replacementIndex << " rl:" << replacedLength << " ml:" << matchedLength << endl;
-    setSelection(m_repPara, replacementIndex, m_repPara, replacementIndex + matchedLength);
-    removeSelectedText();
-    insertAt(m_repDlg->replacement(), m_repPara, replacementIndex);
+    QTextCursor tc = textCursor();
+    tc.setPosition(replacementIndex);
+    tc.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor, matchedLength);
+    tc.removeSelectedText();
+    tc.insertText(m_repDlg->replacement());
+    setTextCursor(tc);
     if (m_replace->options() & KReplaceDialog::PromptOnReplace) {
         ensureCursorVisible();
     }
@@ -1395,14 +1444,9 @@ void TextAreaWidget::slotDoReplace()
 
     delete m_replace;
     m_replace = new KReplace(m_repDlg->pattern(), m_repDlg->replacement(), m_repDlg->options(), this);
-    if (m_replace->options() & KFind::FromCursor) {
-        getCursorPosition(&m_repPara, &m_repIndex);
-    } else if (m_replace->options() & KFind::FindBackwards) {
-        m_repPara = paragraphs() - 1;
-        m_repIndex = paragraphLength(m_repPara) - 1;
-    } else {
-        m_repPara = 0;
-        m_repIndex = 0;
+    m_repIndex = 0;
+    if (m_replace->options() & KFind::FromCursor || m_replace->options() & KFind::FindBackwards) {
+        m_repIndex = textCursor().anchor();
     }
 
     // Connect highlight signal to code which handles highlighting
@@ -1420,60 +1464,23 @@ void TextAreaWidget::slotDoReplace()
 
 void TextAreaWidget::slotReplaceNext()
 {
-    if (!m_replace) {
-        // assert?
+    if (!m_replace)
         return;
-    }
 
-    if (!(m_replace->options() & KReplaceDialog::PromptOnReplace)) {
+    if (!(m_replace->options() & KReplaceDialog::PromptOnReplace))
         viewport()->setUpdatesEnabled(false);
-    }
 
     KFind::Result res = KFind::NoMatch;
-    while (res == KFind::NoMatch) {
-        // If we're done.....
-        if (m_replace->options() & KFind::FindBackwards) {
-            if (m_repIndex == 0 && m_repPara == 0) {
-                break;
-            }
-        } else {
-            if (m_repPara == paragraphs() - 1 &&
-                m_repIndex == paragraphLength(m_repPara) - 1) {
-                break;
-            }
-        }
-
-        if (m_replace->needData()) {
-            m_replace->setData(text(m_repPara), m_repIndex);
-        }
-
-        res = m_replace->replace();
-
-        if (res == KFind::NoMatch) {
-            if (m_replace->options() & KFind::FindBackwards) {
-                if (m_repPara == 0) {
-                    m_repIndex = 0;
-                } else {
-                    m_repPara--;
-                    m_repIndex = paragraphLength(m_repPara) - 1;
-                }
-            } else {
-                if (m_repPara == paragraphs() - 1) {
-                    m_repIndex = paragraphLength(m_repPara) - 1;
-                } else {
-                    m_repPara++;
-                    m_repIndex = 0;
-                }
-            }
-        }
-    }
 
+    if (m_replace->needData())
+        m_replace->setData(toPlainText(), m_repIndex);
+    res = m_replace->replace();
     if (!(m_replace->options() & KReplaceDialog::PromptOnReplace)) {
         viewport()->setUpdatesEnabled(true);
-        repaintChanged();
+        viewport()->update();
     }
 
-    if (res == KFind::NoMatch) { // at end
+    if (res == KFind::NoMatch) {
         m_replace->displayFinalDialog();
         delete m_replace;
         m_replace = 0;
@@ -1494,14 +1501,9 @@ void TextAreaWidget::slotDoFind()
 
     delete m_find;
     m_find = new KFind(m_findDlg->pattern(), m_findDlg->options(), this);
-    if (m_find->options() & KFind::FromCursor) {
-        getCursorPosition(&m_findPara, &m_findIndex);
-    } else if (m_find->options() & KFind::FindBackwards) {
-        m_findPara = paragraphs() - 1;
-        m_findIndex = paragraphLength(m_findPara) - 1;
-    } else {
-        m_findPara = 0;
-        m_findIndex = 0;
+    m_findIndex = 0;
+    if (m_find->options() & KFind::FromCursor || m_find->options() & KFind::FindBackwards) {
+        m_findIndex = textCursor().anchor();
     }
 
     // Connect highlight signal to code which handles highlighting
@@ -1518,51 +1520,15 @@ void TextAreaWidget::slotDoFind()
 
 void TextAreaWidget::slotFindNext()
 {
-    if (!m_find) {
-        // assert?
+    if (!m_find)
         return;
-    }
 
     KFind::Result res = KFind::NoMatch;
-    while (res == KFind::NoMatch) {
-        // If we're done.....
-        if (m_find->options() & KFind::FindBackwards) {
-            if (m_findIndex == 0 && m_findPara == 0) {
-                break;
-            }
-        } else {
-            if (m_findPara == paragraphs() - 1 &&
-                m_findIndex == paragraphLength(m_findPara) - 1) {
-                break;
-            }
-        }
+    if (m_find->needData())
+        m_find->setData(toPlainText(), m_findIndex);
+    res = m_find->find();
 
-        if (m_find->needData()) {
-            m_find->setData(text(m_findPara), m_findIndex);
-        }
-
-        res = m_find->find();
-
-        if (res == KFind::NoMatch) {
-            if (m_find->options() & KFind::FindBackwards) {
-                if (m_findPara == 0) {
-                    m_findIndex = 0;
-                } else {
-                    m_findPara--;
-                    m_findIndex = paragraphLength(m_findPara) - 1;
-                }
-            } else {
-                if (m_findPara == paragraphs() - 1) {
-                    m_findIndex = paragraphLength(m_findPara) - 1;
-                } else {
-                    m_findPara++;
-                    m_findIndex = 0;
-                }
-            }
-        }
-    }
-
-    if (res == KFind::NoMatch) { // at end
+    if (res == KFind::NoMatch) {
         m_find->displayFinalDialog();
         delete m_find;
         m_find = 0;
@@ -1575,7 +1541,7 @@ void TextAreaWidget::slotFindNext()
 
 void TextAreaWidget::slotFind()
 {
-    if( text().isEmpty() )  // saves having to track the text changes
+    if( toPlainText().isEmpty() )  // saves having to track the text changes
         return;
 
     if ( m_findDlg ) {
@@ -1594,7 +1560,7 @@ void TextAreaWidget::slotFind()
 
 void TextAreaWidget::slotReplace()
 {
-    if( text().isEmpty() )  // saves having to track the text changes
+    if( toPlainText().isEmpty() )  // saves having to track the text changes
         return;
 
     if ( m_repDlg ) {
@@ -1630,7 +1596,7 @@ bool TextAreaWidget::event( QEvent *e )
             }
         }
     }
-    return K3TextEdit::event( e );
+    return KTextEdit::event( e );
 }
 
 // -------------------------------------------------------------------------
@@ -1681,7 +1647,7 @@ void RenderTextArea::calcMinMaxWidth()
     QSize size( qMax(element()->cols(), 1L)*m.width('x') + w->frameWidth() +
                 w->verticalScrollBar()->sizeHint().width(),
                 qMax(element()->rows(), 1L)*m.lineSpacing() + w->frameWidth()*4 +
-                (w->wordWrap() == Q3TextEdit::NoWrap ?
+                (w->lineWrapMode() == QTextEdit::NoWrap ?
                  w->horizontalScrollBar()->sizeHint().height() : 0)
         );
 
@@ -1729,13 +1695,13 @@ void RenderTextArea::updateFromElement()
     if ( elementText != text() )
     {
         w->blockSignals(true);
-        int line, col;
-        w->getCursorPosition( &line, &col );
-        int cx = w->contentsX();
-        int cy = w->contentsY();
-        w->setText( elementText );
-        w->setCursorPosition( line, col );
-        w->scrollBy( cx, cy );
+        QTextCursor tc = w->textCursor();
+        int cx = w->horizontalScrollBar()->value();
+        int cy = w->verticalScrollBar()->value();
+        w->setPlainText( elementText );
+        w->setTextCursor(tc);
+        w->horizontalScrollBar()->setValue( cx );
+        w->verticalScrollBar()->setValue( cy );
         w->blockSignals(false);
     }
     element()->m_dirtyvalue = false;
@@ -1754,126 +1720,32 @@ QString RenderTextArea::text()
 {
     QString txt;
     TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
-
-    if(element()->wrap() == DOM::HTMLTextAreaElementImpl::ta_Physical) {
-        // yeah, QTextEdit has no accessor for getting the visually wrapped text
-        for (int p=0; p < w->paragraphs(); ++p) {
-            int ll = 0;
-            int lindex = w->lineOfChar(p, 0);
-            QString paragraphText = w->text(p);
-            int pl = w->paragraphLength(p);
-            paragraphText = paragraphText.left(pl); //Snip invented space.
-            for (int l = 0; l < pl; ++l) {
-                if (lindex != w->lineOfChar(p, l)) {
-                    paragraphText.insert(l+ll++, QLatin1String("\n"));
-                    lindex = w->lineOfChar(p, l);
-                }
-            }
-            txt += paragraphText;
-            if (p < w->paragraphs() - 1)
-                txt += QLatin1String("\n");
+#ifdef __GNUC__
+#warning "Physical wrap mode needs testing (also in ::selection*)"
+#endif
+    if (element()->wrap() == DOM::HTMLTextAreaElementImpl::ta_Physical) {
+        QTextCursor tc(w->document());
+        while (!tc.atEnd()) {
+           tc.movePosition(QTextCursor::EndOfLine, QTextCursor::KeepAnchor);
+           txt += tc.selectedText();
+           if (tc.movePosition(QTextCursor::Down)) {
+               txt += QLatin1String("\n");
+               tc.movePosition(QTextCursor::StartOfLine);
+           }
         }
     }
     else
-        txt = w->text();
-
+        txt = w->toPlainText();
     return txt;
 }
 
-int RenderTextArea::queryParagraphInfo(int para, Mode m, int param) {
-    /* We have to be a bit careful here, as we need to match up the positions
-    to what our value returns here*/
-    TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
-    int        length = 0;
-
-    bool physWrap     = element()->wrap() == DOM::HTMLTextAreaElementImpl::ta_Physical;
-
-    QString paragraphText = w->text(para);
-    int pl                = w->paragraphLength(para);
-    int physicalPL        = pl;
-    if (m == ParaPortionLength)
-        pl = param;
-
-    if (physWrap) {
-        //Go through all the chars of paragraph, and count line changes, chars, etc.
-        int lindex = w->lineOfChar(para, 0);
-        for (int c = 0; c < pl; ++c) {
-            ++length;
-            // Is there a change after this char?
-            if (c+1 < physicalPL && lindex != w->lineOfChar(para, c+1)) {
-                lindex =  w->lineOfChar(para, c+1);
-                ++length;
-            }
-            if (m == ParaPortionOffset && length > param)
-                return c;
-        }
-    } else {
-        //Make sure to count the LF, CR as appropriate. ### this is stupid now, simplify
-        for (int c = 0; c < pl; ++c) {
-            ++length;
-            if (m == ParaPortionOffset && length > param)
-                return c;
-        }
-    }
-    if (m == ParaPortionOffset)
-        return pl;
-    if (m == ParaPortionLength)
-        return length;
-    return length + 1;
-}
-
-long RenderTextArea::computeCharOffset(int para, int index) {
-    if (para < 0)
-        return 0;
-
-    long pos = 0;
-    for (int cp = 0; cp < para; ++cp)
-        pos += queryParagraphInfo(cp, ParaLength);
-
-    if (index >= 0)
-        pos += queryParagraphInfo(para, ParaPortionLength, index);
-    return pos;
-}
-
-void RenderTextArea::computeParagraphAndIndex(long offset, int* para, int* index) {
-    TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
-
-    if (!w->paragraphs()) {
-        *para  = -1;
-        *index = -1;
-        return;
-    }
-
-    //Find the paragraph that contains us..
-    int containingPar = 0;
-    long endPos       = 0;
-    long startPos     = 0;
-    for (int p = 0; p < w->paragraphs(); ++p) {
-        int len = queryParagraphInfo(p, ParaLength);
-        endPos += len;
-        if (endPos > offset) {
-            containingPar = p;
-            break;
-        }
-        startPos += len;
-    }
-
-    *para = containingPar;
-
-    //Now, scan within the paragraph to find the position..
-    long localOffset = offset - startPos;
-
-    *index = queryParagraphInfo(containingPar, ParaPortionOffset, localOffset);
-}
-
 void RenderTextArea::highLightWord( unsigned int length, unsigned int pos )
 {
     TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
     if ( w )
-        w->highLightWord( length, pos );
+        w->highlightWord( length, pos );
 }
 
-
 void RenderTextArea::slotTextChanged()
 {
     element()->m_dirtyvalue = true;
@@ -1890,51 +1762,52 @@ void RenderTextArea::select()
 long RenderTextArea::selectionStart()
 {
     TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
-    int para, index, dummy1, dummy2;
-    w->getSelection(&para, &index, &dummy1, &dummy2);
-    if (para == -1 || index == -1)
-        w->getCursorPosition(&para, &index);
-
-    return computeCharOffset(para, index);
+    return w->textCursor().selectionStart();
 }
 
 long RenderTextArea::selectionEnd()
 {
     TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
-    int para, index, dummy1, dummy2;
-    w->getSelection(&dummy1, &dummy2, &para, &index);
-    if (para == -1 || index == -1)
-        w->getCursorPosition(&para, &index);
+    return w->textCursor().selectionEnd();
+}
 
-    return computeCharOffset(para, index);
+static void setPhysWrapPos(QTextCursor& otc, bool selStart, int idx)
+{
+    QTextCursor tc = otc;
+    tc.setPosition(0);
+    tc.movePosition(QTextCursor::EndOfLine);
+    while (!tc.atEnd()) {
+        if (tc.movePosition(QTextCursor::Down) && tc.position()< idx)
+            --idx;
+        if (tc.position() >= idx)
+            break;
+    }
+    otc.setPosition(idx, selStart ? QTextCursor::MoveAnchor : QTextCursor::KeepAnchor );   
 }
 
 void RenderTextArea::setSelectionStart(long offset) {
     TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
-    int fromPara, fromIndex, toPara, toIndex;
-    w->getSelection(&fromPara, &fromIndex, &toPara, &toIndex);
-    computeParagraphAndIndex(offset, &fromPara, &fromIndex);
-    if (toPara == -1 || toIndex == -1) {
-        toPara  = fromPara;
-        toIndex = fromIndex;
-    }
-    w->setSelection(fromPara, fromIndex, toPara, toIndex);
+    QTextCursor tc = w->textCursor();
+    if (element()->wrap() == DOM::HTMLTextAreaElementImpl::ta_Physical)
+        setPhysWrapPos(tc, true /*selStart*/, offset);
+    else
+        tc.setPosition(offset);
+    w->setTextCursor(tc);
 }
 
 void RenderTextArea::setSelectionEnd(long offset) {
     TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
-    int fromPara, fromIndex, toPara, toIndex;
-    w->getSelection(&fromPara, &fromIndex, &toPara, &toIndex);
-    computeParagraphAndIndex(offset, &toPara, &toIndex);
-    w->setSelection(fromPara, fromIndex, toPara, toIndex);
+    QTextCursor tc = w->textCursor();
+    if (element()->wrap() == DOM::HTMLTextAreaElementImpl::ta_Physical)
+        setPhysWrapPos(tc, false /*selStart*/, offset);
+    else
+        tc.setPosition(offset, QTextCursor::KeepAnchor);
+    w->setTextCursor(tc);
 }
 
 void RenderTextArea::setSelectionRange(long start, long end) {
-    TextAreaWidget* w = static_cast<TextAreaWidget*>(m_widget);
-    int fromPara, fromIndex, toPara, toIndex;
-    computeParagraphAndIndex(start, &fromPara, &fromIndex);
-    computeParagraphAndIndex(end,   &toPara,   &toIndex);
-    w->setSelection(fromPara, fromIndex, toPara, toIndex);
+    setSelectionStart(start);
+    setSelectionEnd(end);
 }
 // ---------------------------------------------------------------------------
 
