@@ -656,7 +656,7 @@ short RenderPart::intrinsicWidth() const
 
 int RenderPart::intrinsicHeight() const
 {
-    return 200;
+    return 150;
 }
 
 void RenderPart::slotViewCleared()
@@ -673,18 +673,15 @@ RenderFrame::RenderFrame( DOM::HTMLFrameElementImpl *frame )
 
 void RenderFrame::slotViewCleared()
 {
-#ifdef __GNUC__
-#warning "Adjust this once we're a QAbstractScrollWidget (and in RenderPartObject)!"
-#endif
-    if(m_widget->inherits("Q3ScrollView")) {
+    if(m_widget->inherits("QScrollArea")) {
 #ifdef DEBUG_LAYOUT
         kDebug(6031) << "frame is a scrollview!" << endl;
 #endif
-        Q3ScrollView *view = static_cast<Q3ScrollView *>(m_widget);
+        QScrollArea *view = static_cast<QScrollArea *>(m_widget);
         if(!element()->frameBorder || !((static_cast<HTMLFrameSetElementImpl *>(element()->parentNode()))->frameBorder()))
             view->setFrameStyle(QFrame::NoFrame);
-	    view->setVScrollBarMode(element()->scrolling );
-	    view->setHScrollBarMode(element()->scrolling );
+	    view->setVerticalScrollBarPolicy(element()->scrolling ); // ?? ###
+	    view->setHorizontalScrollBarPolicy(element()->scrolling );
         if(view->inherits("KHTMLView")) {
 #ifdef DEBUG_LAYOUT
             kDebug(6031) << "frame is a KHTMLview!" << endl;
@@ -760,6 +757,7 @@ void RenderPartObject::updateWidget()
       params.append( QString::fromLatin1("__KHTML__PLUGINBASEURL=\"%1\"").arg(element()->getDocument()->baseURL().url()));
 
       HTMLEmbedElementImpl *embed = 0;
+      QString classId;
       QString serviceType = objbase->serviceType;
       if ( element()->id() == ID_EMBED ) {
 
@@ -774,8 +772,9 @@ void RenderPartObject::updateWidget()
                   embed = static_cast<HTMLEmbedElementImpl *>( child );
                   break;
               }
+          classId = objbase->classId;
 
-          params.append( QString::fromLatin1("__KHTML__CLASSID=\"%1\"").arg( objbase->classId ) );
+          params.append( QString::fromLatin1("__KHTML__CLASSID=\"%1\"").arg( classId ) );
           params.append( QString::fromLatin1("__KHTML__CODEBASE=\"%1\"").arg( objbase->getAttribute(ATTR_CODEBASE).string() ) );
           if (!objbase->getAttribute(ATTR_WIDTH).isEmpty())
               params.append( QString::fromLatin1("WIDTH=\"%1\"").arg( objbase->getAttribute(ATTR_WIDTH).string() ) );
@@ -799,14 +798,16 @@ void RenderPartObject::updateWidget()
               serviceType = "application/x-java-applet";
               url = objbase->classId.mid(5);
           }
-          if(serviceType.isEmpty() && !objbase->classId.isEmpty()) {
-
+          if ( (serviceType.isEmpty() ||
+                      serviceType == "application/x-oleobject") &&
+                  !objbase->classId.isEmpty())
+          {
 #if 0
               // We have a clsid, means this is activex (Niko)
               serviceType = "application/x-activex-handler";
 #endif
 
-              if(objbase->classId.indexOf(QLatin1String("D27CDB6E-AE6D-11cf-96B8-444553540000")) >= 0) {
+              if(classId.indexOf(QString::fromLatin1("D27CDB6E-AE6D-11cf-96B8-444553540000")) >= 0) {
                   // It is ActiveX, but the nsplugin system handling
                   // should also work, that's why we don't override the
                   // serviceType with application/x-activex-handler
@@ -815,17 +816,17 @@ void RenderPartObject::updateWidget()
                   // with nspluginviewer (Niko)
                   serviceType = "application/x-shockwave-flash";
               }
-              else if(objbase->classId.indexOf(QLatin1String("CFCDAA03-8BE4-11cf-B84B-0020AFBBCCFA")) >= 0)
+              else if(classId.indexOf(QLatin1String("CFCDAA03-8BE4-11cf-B84B-0020AFBBCCFA")) >= 0)
                   serviceType = "audio/x-pn-realaudio-plugin";
-              else if(objbase->classId.indexOf(QLatin1String("8AD9C840-044E-11D1-B3E9-00805F499D93")) >= 0 ||
-                      objbase->classId.indexOf(QLatin1String("CAFEEFAC-0014-0000-0000-ABCDEFFEDCBA")) >= 0)
+              else if(classId.indexOf(QLatin1String("8AD9C840-044E-11D1-B3E9-00805F499D93")) >= 0 ||
+                      classId.indexOf(QLatin1String("CAFEEFAC-0014-0000-0000-ABCDEFFEDCBA")) >= 0)
                   serviceType = "application/x-java-applet";
               // http://www.apple.com/quicktime/tools_tips/tutorials/activex.html
-              else if(objbase->classId.indexOf(QLatin1String("02BF25D5-8C17-4B23-BC80-D3488ABDDC6B")) >= 0)
+              else if(classId.indexOf(QLatin1String("02BF25D5-8C17-4B23-BC80-D3488ABDDC6B")) >= 0)
                   serviceType = "video/quicktime";
               // http://msdn.microsoft.com/library/en-us/dnwmt/html/adding_windows_media_to_web_pages__etse.asp?frame=true
-              else if(objbase->classId.indexOf(QLatin1String("6BF52A52-394A-11d3-B153-00C04F79FAA6")) >= 0 ||
-                      objbase->classId.indexOf(QLatin1String("22D6f312-B0F6-11D0-94AB-0080C74C7E95")) >= 0)
+              else if(classId.indexOf(QString::fromLatin1("6BF52A52-394A-11d3-B153-00C04F79FAA6")) >= 0 ||
+                      classId.indexOf(QString::fromLatin1("22D6f312-B0F6-11D0-94AB-0080C74C7E95")) >= 0)
                   serviceType = "video/x-msvideo";
 
               else
@@ -834,7 +835,10 @@ void RenderPartObject::updateWidget()
               // TODO: add more plugins here
           }
       }
-      if ((url.isEmpty() && !embed) || !document()->isURLAllowed(url) || !part->requestObject( this, url, serviceType, params ))
+      if ((url.isEmpty() && !embed &&
+                  (serviceType.isEmpty() || classId.isEmpty())) ||
+              !document()->isURLAllowed(url) ||
+              !part->requestObject( this, url, serviceType, params ))
           objbase->renderAlternative();
   }
 }
@@ -938,7 +942,7 @@ void RenderPartObject::slotPartLoadingErrorNotify()
             QString shortURL = pluginPageURL.protocol() == "http" ? pluginPageURL.host() : pluginPageURL.prettyUrl();
             int res = KMessageBox::questionYesNo( m_view,
                                                   i18n("No plugin found for '%1'.\nDo you want to download one from %2?", mimeName, shortURL),
-                                                  i18n("Missing Plugin"), i18n("Download"), i18n("Do Not Download"), QString("plugin-")+serviceType);
+                                                  i18n("Missing Plugin"), KGuiItem(i18n("Download")), KGuiItem(i18n("Do Not Download")), QString("plugin-")+serviceType);
             if ( res == KMessageBox::Yes )
             {
                 // Display vendor download page
@@ -971,13 +975,13 @@ void RenderPartObject::layout( )
 
 void RenderPartObject::slotViewCleared()
 {
-  if(m_widget->inherits("Q3ScrollView") ) {
+  if(m_widget->inherits("QScrollArea") ) {
 #ifdef DEBUG_LAYOUT
       kDebug(6031) << "iframe is a scrollview!" << endl;
 #endif
-      Q3ScrollView *view = static_cast<Q3ScrollView *>(m_widget);
+      QScrollArea *view = static_cast<QScrollArea *>(m_widget);
       int frameStyle = QFrame::NoFrame;
-      Q3ScrollView::ScrollBarMode scroll = Q3ScrollView::Auto;
+      Qt::ScrollBarPolicy scroll = Qt::ScrollBarAsNeeded;
       int marginw = -1;
       int marginh = -1;
       if ( element()->id() == ID_IFRAME) {
@@ -989,8 +993,8 @@ void RenderPartObject::slotViewCleared()
 	  marginh = frame->marginHeight;
       }
       view->setFrameStyle(frameStyle);
-      view->setVScrollBarMode(scroll );
-      view->setHScrollBarMode(scroll );
+      view->setVerticalScrollBarPolicy(scroll );
+      view->setHorizontalScrollBarPolicy(scroll );
       if(view->inherits("KHTMLView")) {
 #ifdef DEBUG_LAYOUT
           kDebug(6031) << "frame is a KHTMLview!" << endl;
