@@ -55,7 +55,20 @@ void RenderInline::setStyle(RenderStyle* _style)
         currCont = currCont->continuation();
     }
 
-    // Update pseudos for ::before and ::after now.
+    if (attached()) {
+        // Update replaced content
+        updateReplacedContent();
+        // Update pseudos for ::before and ::after
+        updatePseudoChildren();
+    }
+}
+
+// Attach handles initial setStyle that requires parent nodes
+void RenderInline::attach()
+{
+    RenderFlow::attach();
+
+    updateReplacedContent();
     updatePseudoChildren();
 }
 
@@ -84,15 +97,6 @@ void RenderInline::addChildToFlow(RenderObject* newChild, RenderObject* beforeCh
         RenderFlow* oldContinuation = continuation();
         setContinuation(newBox);
 
-        // Someone may have put a <p> inside a <q>, causing a split.  When this happens, the :after content
-        // has to move into the inline continuation.  Call updatePseudoChild to ensure that our :after
-        // content gets properly destroyed.
-        bool isLastChild = (beforeChild == lastChild());
-        updatePseudoChild(RenderStyle::AFTER, lastChild());
-        if (isLastChild && beforeChild != lastChild())
-            beforeChild = 0; // We destroyed the last child, so now we need to update our insertion
-                             // point to be 0.  It's just a straight append now.
-
         splitFlow(beforeChild, newBox, newChild, oldContinuation);
         return;
     }
@@ -238,36 +242,14 @@ void RenderInline::splitFlow(RenderObject* beforeChild, RenderBlock* newBlockBox
     post->setPos(0, -500000);
     post->setNeedsLayout(true);
 
+    updatePseudoChildren();
+
     block->setNeedsLayoutAndMinMaxRecalc();
 }
 
-void RenderInline::paint(PaintInfo& i,
-                      int _tx, int _ty)
+void RenderInline::paint(PaintInfo& i, int _tx, int _ty)
 {
-#ifdef DEBUG_LAYOUT
-    //    kDebug( 6040 ) << renderName() << "(RenderInline) " << this << " ::paintObject() w/h = (" << width() << "/" << height() << ")" << endl;
-#endif
-
-    if ( i.phase == PaintActionElementBackground )
-        return;
-
-    // let the children their backgrounds
-    PaintAction oldphase = i.phase;
-    if ( i.phase == PaintActionChildBackgrounds )
-        i.phase = PaintActionChildBackground;
-
-    paintLineBoxBackgroundBorder(i, _tx, _ty);
-
-    for( RenderObject *child = firstChild(); child; child = child->nextSibling())
-        if(!child->layer() && !child->isFloating())
-            child->paint(i, _tx, _ty);
-
-    paintLineBoxDecorations(i, _tx, _ty);
-
-    i.phase = oldphase;
-    if (style()->visibility() == VISIBLE && i.phase == PaintActionOutline) {
-        paintOutlines(i.p, _tx, _ty);
-    }
+    paintLines(i, _tx, _ty);
 }
 
 /**
@@ -429,25 +411,31 @@ static void appendPoint(QVector<QPoint> &pointArray, QPoint &pnt)
  */
 static void collectHorizontalBoxCoordinates(InlineBox *box,
                                             QVector<QPoint> &pointArray,
-                                            bool bottom, int limit = -500000)
+                                            bool bottom, int offset, int limit = -500000)
 {
 //   kDebug(6000) << "collectHorizontalBoxCoordinates: " << endl;
-    int y = box->yPos() + bottom*box->height();
+    offset = bottom ? offset:-offset;
+    int y = box->yPos() + bottom*box->height() + offset;
     if (limit != -500000 && (bottom ? y < limit : y > limit))
         y = limit;
-    int x = box->xPos() + bottom*box->width();
+    int x = box->xPos() + bottom*box->width() + offset;
     QPoint newPnt(x, y);
     // Add intersection point if point-array not empty.
     if (!pointArray.isEmpty()) {
         QPoint lastPnt = pointArray.back();
         QPoint insPnt(newPnt.x(), lastPnt.y());
+
+        if (offset && ((bottom && lastPnt.y() > y) || (!bottom && lastPnt.y() < y))) {
+            insPnt.rx() = lastPnt.x();
+            insPnt.ry() = y;
+        }
 //         kDebug(6040) << "left: " << lastPnt << " == " << insPnt << ": " << (insPnt == lastPnt) << endl;
         appendPoint(pointArray, insPnt);
     }
     // Insert starting point of box
     appendPoint(pointArray, newPnt);
 
-    newPnt.rx() += bottom ? -box->width() : box->width();
+    newPnt.rx() += (bottom ? -box->width() : box->width()) - 2*offset;
 
     if (box->isInlineFlowBox()) {
         InlineFlowBox *flowBox = static_cast<InlineFlowBox *>(box);
@@ -460,7 +448,7 @@ static void collectHorizontalBoxCoordinates(InlineBox *box,
               l2 = y;
             else
               l2 = limit;
-            collectHorizontalBoxCoordinates(b, pointArray, bottom, l2);
+            collectHorizontalBoxCoordinates(b, pointArray, bottom, qAbs(offset), l2);
         }
 
         // Add intersection point if flow box contained any children
@@ -486,11 +474,11 @@ static void collectHorizontalBoxCoordinates(InlineBox *box,
  *	succeeding
  * @return \c true if this and the next box are disjoint
  */
-inline static bool lineBoxesDisjoint(InlineRunBox *line, bool toBegin)
+inline static bool lineBoxesDisjoint(InlineRunBox *line, int offset, bool toBegin)
 {
   InlineRunBox *next = toBegin ? line->prevLineBox() : line->nextLineBox();
-  return !next || next->xPos() + next->width() < line->xPos()
-               || next->xPos() > line->xPos() + line->width();
+  return !next || next->xPos() + next->width() + 2*offset < line->xPos()
+               || next->xPos() > line->xPos() + line->width() + 2*offset;
 }
 
 /**
@@ -504,25 +492,26 @@ inline static bool lineBoxesDisjoint(InlineRunBox *line, bool toBegin)
  */
 static void collectVerticalBoxCoordinates(InlineRunBox *line,
                                           QVector<QPoint> &pointArray,
-                                          bool left, InlineRunBox **lastline = 0)
+                                          bool left, int offset, InlineRunBox **lastline = 0)
 {
     InlineRunBox *last = 0;
+    offset = left ? -offset:offset;
     for (InlineRunBox* curr = line; curr && !last; curr = left ? curr->prevLineBox() : curr->nextLineBox()) {
         InlineBox *root = curr;
 
-        bool isLast = lineBoxesDisjoint(curr, left);
+        bool isLast = lineBoxesDisjoint(curr, qAbs(offset), left);
         if (isLast) last = curr;
 
         if (root != line && !isLast)
             while (root->parent()) root = root->parent();
-        QPoint newPnt(curr->xPos() + !left*curr->width(),
-                      left ? root->topOverflow() : root->bottomOverflow());
+        QPoint newPnt(curr->xPos() + !left*curr->width() + offset,
+                      (left ? root->topOverflow() : root->bottomOverflow()) + offset);
         if (!pointArray.isEmpty()) {
             QPoint lastPnt = pointArray.back();
             if (newPnt.x()>lastPnt.x() && !left)
-                pointArray.back().setY( qMin(lastPnt.y(), root->topOverflow()) );
+                pointArray.back().setY( qMin(lastPnt.y(), root->topOverflow()-offset) );
             else if (newPnt.x()<lastPnt.x() && left)
-                pointArray.back().setY( qMax(lastPnt.y(), root->bottomOverflow()) );
+                pointArray.back().setY( qMax(lastPnt.y(), root->bottomOverflow()+offset) );
             QPoint insPnt(newPnt.x(), pointArray.back().y());
 //         kDebug(6040) << "left: " << lastPnt << " == " << insPnt << ": " << (insPnt == lastPnt) << endl;
             appendPoint(pointArray, insPnt);
@@ -546,7 +535,7 @@ static void collectVerticalBoxCoordinates(InlineRunBox *line,
 static QPoint *linkEndToBegin(QVector<QPoint> &pointArray)
 {
     uint index = 0;
-    Q_ASSERT(pointArray.size() >= 3);
+    assert(pointArray.size() >= 3);
 
     // if first and last points match, ignore the last one.
     bool linkup = false; QPoint linkupPnt;
@@ -577,6 +566,7 @@ void RenderInline::paintOutlines(QPainter *p, int _tx, int _ty)
 {
     if (style()->outlineWidth() == 0 || style()->outlineStyle() <= BHIDDEN)
         return;
+    int offset = style()->outlineOffset();
 
     // We may have to draw more than one outline path as they may be
     // disjoint.
@@ -584,13 +574,15 @@ void RenderInline::paintOutlines(QPainter *p, int _tx, int _ty)
         QVector<QPoint> path;
 
         // collect topmost outline
-        collectHorizontalBoxCoordinates(curr, path, false);
+        collectHorizontalBoxCoordinates(curr, path, false, offset);
         // collect right outline
-        collectVerticalBoxCoordinates(curr, path, false, &curr);
+        collectVerticalBoxCoordinates(curr, path, false, offset, &curr);
         // collect bottommost outline
-        collectHorizontalBoxCoordinates(curr, path, true);
+        collectHorizontalBoxCoordinates(curr, path, true, offset);
         // collect left outline
-        collectVerticalBoxCoordinates(curr, path, true);
+        collectVerticalBoxCoordinates(curr, path, true, offset);
+
+        if (path.size() < 3) continue;
 
         const QPoint *begin = linkEndToBegin(path);
 
@@ -671,13 +663,11 @@ static void paintOutlineSegment(RenderObject *o, QPainter *p, int tx, int ty,
     int ow = o->style()->outlineWidth();
     EBorderStyle os = o->style()->outlineStyle();
     QColor oc = o->style()->outlineColor();
-    // ### outline-offset is not this simple to merge anymore
-    int offset = 0; // o->style()->outlineOffset();
 
-    int x1 = tx + p1.x() - offset;
-    int y1 = ty + p1.y() - offset;
-    int x2 = tx + p2.x() + offset;
-    int y2 = ty + p2.y() + offset;
+    int x1 = tx + p1.x();
+    int y1 = ty + p1.y();
+    int x2 = tx + p2.x();
+    int y2 = ty + p2.y();
     if (x1 > x2) {
         kSwap(x1, x2);
         if (bsOrientation(curBS) == BSHorizontal) kSwap(prevBS, nextBS);
@@ -833,23 +823,17 @@ const char *RenderInline::renderName() const
 
 bool RenderInline::nodeAtPoint(NodeInfo& info, int _x, int _y, int _tx, int _ty, HitTestAction hitTestAction, bool inside)
 {
+/*
     if ( hitTestAction != HitTestSelfOnly ) {
         for (RenderObject* child = lastChild(); child; child = child->previousSibling())
             if (!child->layer() && !child->isFloating() && child->nodeAtPoint(info, _x, _y, _tx, _ty, HitTestAll))
                 inside = true;
     }
-
-
+*/
     // Check our line boxes if we're still not inside.
-    if (hitTestAction != HitTestChildrenOnly && !inside && style()->visibility() != HIDDEN) {
+    if (/*hitTestAction != HitTestChildrenOnly &&*/ !inside && style()->visibility() != HIDDEN) {
         // See if we're inside one of our line boxes.
-        for (InlineRunBox* curr = firstLineBox(); curr; curr = curr->nextLineBox()) {
-            if((_y >=_ty + curr->m_y) && (_y < _ty + curr->m_y + curr->m_height) &&
-               (_x >= _tx + curr->m_x) && (_x <_tx + curr->m_x + curr->m_width) ) {
-                inside = true;
-                break;
-            }
-        }
+        inside = hitTestLines(info, _x, _y, _tx, _ty, hitTestAction);
     }
 
     if (inside && element()) {
