@@ -52,22 +52,19 @@
 #include "render_canvas.h"
 #include "render_arena.h"
 #include "render_replaced.h"
+#include "render_form.h"
 #include "xml/dom_docimpl.h"
 #include "xml/dom2_eventsimpl.h"
 #include "misc/htmltags.h"
 #include "html/html_blockimpl.h"
 #include "xml/dom_restyler.h"
 
-#include <qscrollbar.h>
-#include <q3ptrvector.h>
 #include <qstyle.h>
 
 using namespace DOM;
 using namespace khtml;
 
-#ifdef APPLE_CHANGES
-QScrollBar* RenderLayer::gScrollBar = 0;
-#endif
+ScrollBarWidget* RenderLayer::gScrollBar = 0;
 
 #ifndef NDEBUG
 static bool inRenderLayerDetach;
@@ -97,7 +94,10 @@ m_vBar( 0 ),
 m_scrollMediator( 0 ),
 m_posZOrderList( 0 ),
 m_negZOrderList( 0 ),
+m_overflowList(0),
 m_zOrderListsDirty( true ),
+m_overflowListDirty(true),
+m_isOverflowOnly( shouldBeOverflowOnly() ),
 m_markedForRepaint( false ),
 m_hasOverlaidWidgets( false ),
 m_marquee( 0 )
@@ -113,6 +113,7 @@ RenderLayer::~RenderLayer()
     delete m_scrollMediator;
     delete m_posZOrderList;
     delete m_negZOrderList;
+    delete m_overflowList;
     delete m_marquee;
 }
 
@@ -125,7 +126,7 @@ void RenderLayer::updateLayerPosition()
         return;
 
     int x = m_object->xPos();
-    int y = m_object->yPos();
+    int y = m_object->yPos() - m_object->borderTopExtra();
 
     if (!m_object->isPositioned()) {
         // We must adjust our position by walking up the render tree looking for the
@@ -136,6 +137,8 @@ void RenderLayer::updateLayerPosition()
             y += curr->yPos();
             curr = curr->parent();
         }
+	if (curr)
+	  y += curr->borderTopExtra();
     }
 
     if (m_object->isRelPositioned())
@@ -172,7 +175,7 @@ QRegion RenderLayer::paintedRegion(RenderLayer* rootLayer)
         convertToLayerCoords(rootLayer,x,y);
         QRect cr(x,y,width(),height());
         if ( s->backgroundImage() || s->backgroundColor().isValid() || s->hasBorder() || 
-             s->scrollsOverflow() || renderer()->isReplaced() ) {
+             renderer()->scrollsOverflow() || renderer()->isReplaced() ) {
             r += cr;
         } else {
             r += renderer()->visibleFlowRegion(x, y);
@@ -189,17 +192,17 @@ QRegion RenderLayer::paintedRegion(RenderLayer* rootLayer)
     return r;
 }
 
-void RenderLayer::repaint( bool markForRepaint )
+void RenderLayer::repaint( Priority p, bool markForRepaint )
 {
     if (markForRepaint && m_markedForRepaint)
         return;
     for (RenderLayer* child = firstChild(); child; child = child->nextSibling())
-        child->repaint( markForRepaint );
+        child->repaint( p, markForRepaint );
     QRect layerBounds, damageRect, fgrect;
     calculateRects(renderer()->canvas()->layer(), renderer()->viewRect(), layerBounds, damageRect, fgrect);
     m_visibleRect = damageRect.intersect( layerBounds );
     if (m_visibleRect.isValid())
-        renderer()->canvas()->repaintViewRectangle( m_visibleRect.x(), m_visibleRect.y(), m_visibleRect.width(), m_visibleRect.height() );
+        renderer()->canvas()->repaintViewRectangle( m_visibleRect.x(), m_visibleRect.y(), m_visibleRect.width(), m_visibleRect.height(), (p > NormalPriority) );
     if (markForRepaint)
         m_markedForRepaint = true;
 }
@@ -256,8 +259,8 @@ void RenderLayer::updateWidgetMasks(RenderLayer* rootLayer)
         uint count = m_posZOrderList ? m_posZOrderList->count() : 0;
         bool needUpdate = (count || !m_region.isEmpty());
         if (count) {
-            Q3ScrollView* sv = m_object->element()->getDocument()->view();
-            m_region = QRect(0,0,sv->contentsWidth(),sv->contentsHeight());
+            KHTMLView* sa = m_object->element()->getDocument()->view();
+            m_region = QRect(0,0,sa->contentsWidth(),sa->contentsHeight());
 
             for (uint i = 0; i < count; i++) {
                 RenderLayer* child = m_posZOrderList->at(i);
@@ -278,15 +281,15 @@ void RenderLayer::updateWidgetMasks(RenderLayer* rootLayer)
 short RenderLayer::width() const
 {
     int w = m_object->width();
-    if (!m_object->style()->hidesOverflow())
+    if (!m_object->hasOverflowClip())
         w = qMax(m_object->overflowWidth(), w);
     return w;
 }
 
 int RenderLayer::height() const
 {
-    int h = m_object->height();
-    if (!m_object->style()->hidesOverflow())
+    int h = m_object->height() + m_object->borderTopExtra() + m_object->borderBottomExtra();
+    if (!m_object->hasOverflowClip())
         h = qMax(m_object->overflowHeight(), h);
     return h;
 }
@@ -311,19 +314,17 @@ RenderLayer* RenderLayer::enclosingPositionedAncestor() const
     return curr;
 }
 
-#ifdef APPLE_CHANGES
-bool RenderLayer::isTransparent()
+bool RenderLayer::isTransparent() const
 {
     return m_object->style()->opacity() < 1.0f;
 }
 
-RenderLayer* RenderLayer::transparentAncestor()
+RenderLayer* RenderLayer::transparentAncestor() const
 {
     RenderLayer* curr = parent();
     for ( ; curr && curr->m_object->style()->opacity() == 1.0f; curr = curr->parent());
     return curr;
 }
-#endif
 
 void* RenderLayer::operator new(size_t sz, RenderArena* renderArena) throw()
 {
@@ -371,12 +372,16 @@ void RenderLayer::addChild(RenderLayer *child, RenderLayer* beforeChild)
 
     child->setParent(this);
 
-    // Dirty the z-order list in which we are contained.  The stackingContext() can be null in the
-    // case where we're building up generated content layers.  This is ok, since the lists will start
-    // off dirty in that case anyway.
-    RenderLayer* stackingContext = child->stackingContext();
-    if (stackingContext)
-        stackingContext->dirtyZOrderLists();
+    if (child->isOverflowOnly())
+        dirtyOverflowList();
+    else {
+        // Dirty the z-order list in which we are contained.  The stackingContext() can be null in the
+        // case where we're building up generated content layers.  This is ok, since the lists will start
+        // off dirty in that case anyway.
+        RenderLayer* stackingContext = child->stackingContext();
+        if (stackingContext)
+            stackingContext->dirtyZOrderLists();
+    }
 }
 
 RenderLayer* RenderLayer::removeChild(RenderLayer* oldChild)
@@ -392,12 +397,16 @@ RenderLayer* RenderLayer::removeChild(RenderLayer* oldChild)
     if (m_last == oldChild)
         m_last = oldChild->previousSibling();
 
-    // Dirty the z-order list in which we are contained.  When called via the
-    // reattachment process in removeOnlyThisLayer, the layer may already be disconnected
-    // from the main layer tree, so we need to null-check the |stackingContext| value.
-    RenderLayer* stackingContext = oldChild->stackingContext();
-    if (stackingContext)
-        oldChild->stackingContext()->dirtyZOrderLists();
+    if (oldChild->isOverflowOnly())
+        dirtyOverflowList();
+    else {
+        // Dirty the z-order list in which we are contained.  When called via the
+        // reattachment process in removeOnlyThisLayer, the layer may already be disconnected
+        // from the main layer tree, so we need to null-check the |stackingContext| value.
+        RenderLayer* stackingContext = oldChild->stackingContext();
+        if (stackingContext)
+            stackingContext->dirtyZOrderLists();
+    }
 
     oldChild->setPreviousSibling(0);
     oldChild->setNextSibling(0);
@@ -525,7 +534,7 @@ void RenderLayer::checkInlineRelOffset(const RenderObject* o, int& x, int& y)
 
 void RenderLayer::scrollToOffset(int x, int y, bool updateScrollbars, bool repaint)
 {
-    if (renderer()->style()->overflow() != OMARQUEE) {
+    if (renderer()->style()->overflowX() != OMARQUEE || !renderer()->hasOverflowClip()) {
         if (x < 0) x = 0;
         if (y < 0) y = 0;
 
@@ -557,7 +566,7 @@ void RenderLayer::scrollToOffset(int x, int y, bool updateScrollbars, bool repai
 
     // Just schedule a full repaint of our object.
     if (repaint)
-        m_object->repaint(true);
+        m_object->repaint(RealtimePriority);
 
     if (updateScrollbars) {
         if (m_hBar)
@@ -592,13 +601,12 @@ void RenderLayer::updateScrollPositionFromScrollbars()
 void
 RenderLayer::showScrollbar(Qt::Orientation o, bool show)
 {
-    QScrollBar *sb = (o == Qt::Horizontal) ? m_hBar : m_vBar;
+    ScrollBarWidget *sb = (o == Qt::Horizontal) ? m_hBar : m_vBar;
 
     if (show && !sb) {
-        Q3ScrollView* scrollView = m_object->element()->getDocument()->view();
-        sb = new QScrollBar(o, scrollView );
-        sb->setObjectName("__khtml");
-        scrollView->addChild(sb, 0, -50000);
+        KHTMLView* view = m_object->element()->getDocument()->view();
+        sb = new ScrollBarWidget(o, view->widget());
+        sb->move(0, -50000);
         sb->setAttribute(Qt::WA_NoSystemBackground);
         sb->show();
         if (!m_scrollMediator)
@@ -642,20 +650,11 @@ int RenderLayer::horizontalScrollbarHeight()
 
 }
 
-void RenderLayer::moveScrollbarsAside()
-{
-    Q3ScrollView* scrollView = m_object->element()->getDocument()->view();
-  if (m_hBar)
-        scrollView->addChild(m_hBar, 0, -50000);
-    if (m_vBar)
-        scrollView->addChild(m_vBar, 0, -50000);
-}
-
 void RenderLayer::positionScrollbars(const QRect& absBounds)
 {
 #ifdef APPLE_CHANGES
     if (m_vBar) {
-        scrollView->addChild(m_vBar, absBounds.x()+absBounds.width()-m_object->borderRight()-m_vBar->width(),
+        view->addChild(m_vBar, absBounds.x()+absBounds.width()-m_object->borderRight()-m_vBar->width(),
                      absBounds.y()+m_object->borderTop());
         m_vBar->resize(m_vBar->width(), absBounds.height() -
                        (m_object->borderTop()+m_object->borderBottom()) -
@@ -663,7 +662,7 @@ void RenderLayer::positionScrollbars(const QRect& absBounds)
     }
 
     if (m_hBar) {
-        scrollView->addChild(m_hBar, absBounds.x()+m_object->borderLeft(),
+        view->addChild(m_hBar, absBounds.x()+m_object->borderLeft(),
                      absBounds.y()+absBounds.height()-m_object->borderBottom()-m_hBar->height());
         m_hBar->resize(absBounds.width() - (m_object->borderLeft()+m_object->borderRight()) -
                        (m_vBar ? m_vBar->width()-1 : 0), m_hBar->height());
@@ -679,26 +678,26 @@ void RenderLayer::positionScrollbars(const QRect& absBounds)
     if (w <= 0 || h <= 0 || (!m_vBar && !m_hBar))
 	return;
 
-    Q3ScrollView* scrollView = m_object->element()->getDocument()->view();
+    KHTMLView* view = m_object->element()->getDocument()->view();
 
     tx += bl;
     ty += bt;
 
-    QScrollBar *b = m_hBar;
+    ScrollBarWidget *b = m_hBar;
     if (!m_hBar)
 	b = m_vBar;
     int sw = b->style()->pixelMetric(QStyle::PM_ScrollBarExtent);
 
     if (m_vBar) {
-	QRect vBarRect = QRect(tx + w - sw + 1, ty, sw, h - (m_hBar ? sw : 0) + 1);
+	QRect vBarRect = QRect(tx + w - sw, ty, sw, h - (m_hBar ? sw : 0));
         m_vBar->resize(vBarRect.width(), vBarRect.height());
-        scrollView->addChild(m_vBar, vBarRect.x(), vBarRect.y());
+        m_vBar->m_kwp->setPos(QPoint(vBarRect.x(), vBarRect.y()));
     }
 
     if (m_hBar) {
-	QRect hBarRect = QRect(tx, ty + h - sw + 1, w - (m_vBar ? sw : 0) + 1, sw);
+	QRect hBarRect = QRect(tx, ty + h - sw, w - (m_vBar ? sw : 0), sw);
         m_hBar->resize(hBarRect.width(), hBarRect.height());
-        scrollView->addChild(m_hBar, hBarRect.x(), hBarRect.y());
+        m_hBar->m_kwp->setPos(QPoint(hBarRect.x(), hBarRect.y()));
     }
 #endif
 }
@@ -708,8 +707,13 @@ void RenderLayer::positionScrollbars(const QRect& absBounds)
 
 void RenderLayer::checkScrollbarsAfterLayout()
 {
-    int rightPos = m_object->overflowWidth();
-    int bottomPos = m_object->overflowHeight();
+    int rightPos = m_object->rightmostPosition(true);
+    int bottomPos = m_object->lowestPosition(true);
+
+/*  TODO
+    m_scrollLeft = m_object->leftmostPosition(true);
+    m_scrollTop = m_object->highestPosition(true);
+*/
 
     int clientWidth = m_object->clientWidth();
     int clientHeight = m_object->clientHeight();
@@ -724,21 +728,31 @@ void RenderLayer::checkScrollbarsAfterLayout()
     bool needHorizontalBar = rightPos > width();
     bool needVerticalBar = bottomPos > height();
 
-    bool haveHorizontalBar = m_hBar;
-    bool haveVerticalBar = m_vBar;
+    bool haveHorizontalBar = m_hBar && m_hBar->isEnabled();
+    bool haveVerticalBar = m_vBar && m_vBar->isEnabled();
+    
+    bool hasOvf = m_object->hasOverflowClip();
 
     // overflow:scroll should just enable/disable.
-    if (m_object->style()->overflow() == OSCROLL) {
+    if (hasOvf && m_object->style()->overflowX() == OSCROLL)
         m_hBar->setEnabled(needHorizontalBar);
+    if (hasOvf && m_object->style()->overflowY() == OSCROLL)
         m_vBar->setEnabled(needVerticalBar);
-    }
 
     // overflow:auto may need to lay out again if scrollbars got added/removed.
-    bool scrollbarsChanged = (m_object->style()->overflow() == OAUTO) &&
-        (haveHorizontalBar != needHorizontalBar || haveVerticalBar != needVerticalBar);
+    bool scrollbarsChanged = (hasOvf && m_object->style()->overflowX() == OAUTO && haveHorizontalBar != needHorizontalBar)
+                          || (hasOvf && m_object->style()->overflowY() == OAUTO && haveVerticalBar != needVerticalBar);
     if (scrollbarsChanged) {
-        showScrollbar(Qt::Horizontal, needHorizontalBar);
-        showScrollbar(Qt::Vertical, needVerticalBar);
+        if (m_object->style()->overflowX() == OAUTO) {
+            showScrollbar(Qt::Horizontal, needHorizontalBar);
+            if (m_hBar) 
+                m_hBar->setEnabled(true);
+        }
+        if (m_object->style()->overflowY() == OAUTO) {
+            showScrollbar(Qt::Vertical, needVerticalBar);
+            if (m_vBar)
+                m_vBar->setEnabled(true);
+        }
 
         m_object->setNeedsLayout(true);
 	if (m_object->isRenderBlock())
@@ -784,18 +798,14 @@ void RenderLayer::paintScrollbars(RenderObject::PaintInfo& pI)
     if (!m_object->element())
        return;
 
-    Q3ScrollView* scrollView = m_object->element()->getDocument()->view();
+    KHTMLView* view = m_object->element()->getDocument()->view();
     if (m_hBar) {
-	int x = m_hBar->x();
-	int y = m_hBar->y();
-	scrollView->viewportToContents(x, y, x, y);
-	RenderWidget::paintWidget(pI, m_hBar, x, y);
+	QPoint p = m_hBar->m_kwp->absolutePos();
+	RenderWidget::paintWidget(pI, m_hBar, p.x(), p.y());
     }
     if (m_vBar) {
-	int x = m_vBar->x();
-	int y = m_vBar->y();
-	scrollView->viewportToContents(x, y, x, y);
-	RenderWidget::paintWidget(pI, m_vBar, x, y);
+        QPoint p = m_vBar->m_kwp->absolutePos();
+	RenderWidget::paintWidget(pI, m_vBar, p.x(), p.y());
     }
 #endif
 }
@@ -814,7 +824,7 @@ static void setClip(QPainter* p, const QRect& paintDirtyRect, const QRect& clipR
 #ifdef APPLE_CHANGES
     p->addClip(clipRect);
 #else
-    p->setClipRegion(clipRect, Qt::IntersectClip);
+    p->setClipRect(clipRect, Qt::IntersectClip);
 #endif
 
 }
@@ -835,14 +845,14 @@ void RenderLayer::paintLayer(RenderLayer* rootLayer, QPainter *p,
     int x = layerBounds.x();
     int y = layerBounds.y();
 
-    // Ensure our z-order lists are up-to-date.
+    // Ensure our lists are up-to-date.
     updateZOrderLists();
+    updateOverflowList();
 
-#ifdef APPLE_CHANGES
     // Set our transparency if we need to.
+    float previousOpacity = p->opacity();
     if (isTransparent())
-        p->beginTransparencyLayer(renderer()->style()->opacity());
-#endif
+        p->setOpacity(renderer()->style()->opacity());
 
     // We want to paint our layer, but only if we intersect the damage rect.
     bool shouldPaint = intersectsDamageRect(layerBounds, damageRect);
@@ -855,7 +865,7 @@ void RenderLayer::paintLayer(RenderLayer* rootLayer, QPainter *p,
             // Paint the background.
             RenderObject::PaintInfo paintInfo(p, damageRect, PaintActionElementBackground);
             renderer()->paint(paintInfo,
-                              x - renderer()->xPos(), y - renderer()->yPos());
+                              x - renderer()->xPos(), y - renderer()->yPos() + renderer()->borderTopExtra());
 
             // Position our scrollbars.
             positionScrollbars(layerBounds);
@@ -886,21 +896,26 @@ void RenderLayer::paintLayer(RenderLayer* rootLayer, QPainter *p,
 
         RenderObject::PaintInfo paintInfo(p, clipRectToApply, PaintActionSelection);
 
+        int tx = x - renderer()->xPos();
+        int ty = y - renderer()->yPos() + renderer()->borderTopExtra();
+
         if (selectionOnly)
-            renderer()->paint(paintInfo, x - renderer()->xPos(), y - renderer()->yPos());
+            renderer()->paint(paintInfo, tx, ty);
         else {
             paintInfo.phase = PaintActionChildBackgrounds;
-            renderer()->paint(paintInfo, x - renderer()->xPos(), y - renderer()->yPos());
+            renderer()->paint(paintInfo, tx, ty);
             paintInfo.phase = PaintActionFloat;
-            renderer()->paint(paintInfo, x - renderer()->xPos(), y - renderer()->yPos());
+            renderer()->paint(paintInfo, tx, ty);
             paintInfo.phase = PaintActionForeground;
-            renderer()->paint(paintInfo, x - renderer()->xPos(), y - renderer()->yPos());
-            paintInfo.phase = PaintActionOutline;
-            renderer()->paint(paintInfo, x - renderer()->xPos(), y - renderer()->yPos());
+            renderer()->paint(paintInfo, tx, ty);
             RenderCanvas *rc = static_cast<RenderCanvas*>(renderer()->document()->renderer());
+            if (rc->maximalOutlineSize()) {
+                paintInfo.phase = PaintActionOutline;
+                renderer()->paint(paintInfo, tx, ty);
+            }
             if (rc->selectionStart() && rc->selectionEnd()) {
                 paintInfo.phase = PaintActionSelection;
-                renderer()->paint(paintInfo, x - renderer()->xPos(), y - renderer()->yPos());
+                renderer()->paint(paintInfo, tx, ty);
             }
         }
 
@@ -908,6 +923,11 @@ void RenderLayer::paintLayer(RenderLayer* rootLayer, QPainter *p,
         restoreClip(p, paintDirtyRect, clipRectToApply);
     }
 
+    // Paint any child layers that have overflow.
+    if (m_overflowList)
+        foreach (RenderLayer* layer, *m_overflowList)
+            layer->paintLayer(rootLayer, p, paintDirtyRect, selectionOnly);
+
     // Now walk the sorted list of children with positive z-indices.
     if (m_posZOrderList) {
         uint count = m_posZOrderList->count();
@@ -928,19 +948,15 @@ void RenderLayer::paintLayer(RenderLayer* rootLayer, QPainter *p,
     }
 #endif
 
-#ifdef APPLE_CHANGES
     // End our transparency layer
     if (isTransparent())
-        p->endTransparencyLayer();
-#endif
+        p->setOpacity(previousOpacity);
 }
 
 bool RenderLayer::nodeAtPoint(RenderObject::NodeInfo& info, int x, int y)
 {
-#ifdef APPLE_CHANGES
     // Clear our our scrollbar variable
     RenderLayer::gScrollBar = 0;
-#endif
 
     int stx = m_x;
     int sty = m_y;
@@ -979,8 +995,9 @@ RenderLayer* RenderLayer::nodeAtPointForLayer(RenderLayer* rootLayer, RenderObje
     QRect layerBounds, bgRect, fgRect;
     calculateRects(rootLayer, hitTestRect, layerBounds, bgRect, fgRect);
 
-    // Ensure our z-order lists are up-to-date.
+    // Ensure our lists are up-to-date.
     updateZOrderLists();
+    updateOverflowList();
 
     // This variable tracks which layer the mouse ends up being inside.  The minute we find an insideLayer,
     // we are done and can return it.
@@ -998,12 +1015,22 @@ RenderLayer* RenderLayer::nodeAtPointForLayer(RenderLayer* rootLayer, RenderObje
         }
     }
 
-    // Next we want to see if the mouse pos is inside the child RenderObjects of the layer.
+    // Now check our overflow objects.
+    if (m_overflowList) {
+        QVector<RenderLayer*>::iterator it = m_overflowList->end();
+        while (it != m_overflowList->begin()) {
+            --it;
+            insideLayer = (*it)->nodeAtPointForLayer(rootLayer, info,  xMousePos, yMousePos, hitTestRect);
+            if (insideLayer)
+                return insideLayer;
+        }
+    }
 
+    // Next we want to see if the mouse pos is inside the child RenderObjects of the layer.
     if (containsPoint(xMousePos, yMousePos, fgRect) &&
         renderer()->nodeAtPoint(info, xMousePos, yMousePos,
-                                layerBounds.x() - renderer()->xPos(),
-                                layerBounds.y() - renderer()->yPos(),
+                            layerBounds.x() - renderer()->xPos(),
+                            layerBounds.y() - renderer()->yPos() + m_object->borderTopExtra(),
                                 HitTestChildrenOnly)) {
 	if (info.innerNode() != m_object->element())
 	    return this;
@@ -1024,7 +1051,7 @@ RenderLayer* RenderLayer::nodeAtPointForLayer(RenderLayer* rootLayer, RenderObje
     if (containsPoint(xMousePos, yMousePos, bgRect) &&
         renderer()->nodeAtPoint(info, xMousePos, yMousePos,
                                 layerBounds.x() - renderer()->xPos(),
-                                layerBounds.y() - renderer()->yPos(),
+                                layerBounds.y() - renderer()->yPos() + m_object->borderTopExtra(),
                                 HitTestSelfOnly))
         return this;
 
@@ -1124,9 +1151,7 @@ bool RenderLayer::intersectsDamageRect(const QRect& layerBounds, const QRect& da
 
 bool RenderLayer::containsPoint(int x, int y, const QRect& damageRect) const
 {
-    return (renderer()->isCanvas() || renderer()->isRoot() || renderer()->isBody() ||
-            renderer()->hasOverhangingFloats() ||
-            (renderer()->isInline() && !renderer()->isReplaced()) ||
+    return (renderer()->isCanvas() || renderer()->isRoot() || renderer()->isInlineFlow() ||
             damageRect.contains(x, y));
 }
 
@@ -1281,6 +1306,13 @@ void RenderLayer::dirtyZOrderLists()
     m_zOrderListsDirty = true;
 }
 
+void RenderLayer::dirtyOverflowList()
+{
+    if (m_overflowList)
+        m_overflowList->clear();
+    m_overflowListDirty = true;
+}
+
 void RenderLayer::updateZOrderLists()
 {
     if (!isStackingContext() || !m_zOrderListsDirty)
@@ -1302,6 +1334,22 @@ void RenderLayer::updateZOrderLists()
     m_zOrderListsDirty = false;
 }
 
+void RenderLayer::updateOverflowList()
+{
+    if (!m_overflowListDirty)
+        return;
+
+    for (RenderLayer* child = firstChild(); child; child = child->nextSibling()) {
+        if (child->isOverflowOnly()) {
+            if (!m_overflowList)
+                m_overflowList = new QVector<RenderLayer*>;
+            m_overflowList->append(child);
+        }
+    }
+
+    m_overflowListDirty = false;
+}
+
 void RenderLayer::collectLayers(QVector<RenderLayer*>*& posBuffer, QVector<RenderLayer*>*& negBuffer)
 {
     // FIXME: A child render object or layer could override visibility.  Don't remove this
@@ -1310,15 +1358,18 @@ void RenderLayer::collectLayers(QVector<RenderLayer*>*& posBuffer, QVector<Rende
     if (renderer()->style()->visibility() != VISIBLE)
         return;
 
-    // Determine which buffer the child should be in.
-    QVector<RenderLayer*>*& buffer = (zIndex() >= 0) ? posBuffer : negBuffer;
+    // Overflow layers are just painted by their enclosing layers, so they don't get put in zorder lists.
+    if (!isOverflowOnly()) {
+        // Determine which buffer the child should be in.
+        QVector<RenderLayer*>*& buffer = (zIndex() >= 0) ? posBuffer : negBuffer;
 
-    // Create the buffer if it doesn't exist yet.
-    if (!buffer)
-        buffer = new QVector<RenderLayer*>();
+        // Create the buffer if it doesn't exist yet.
+        if (!buffer)
+            buffer = new QVector<RenderLayer*>();
 
-    // Append ourselves at the end of the appropriate buffer.
-    buffer->append(this);
+        // Append ourselves at the end of the appropriate buffer.
+        buffer->append(this);
+    }
 
     // Recur into our children to collect more layers, but only if we don't establish
     // a stacking context.
@@ -1382,11 +1433,13 @@ static void writeLayers(QTextStream &ts, const RenderLayer* rootLayer, RenderLay
     QRect layerBounds, damageRect, clipRectToApply;
     l->calculateRects(rootLayer, paintDirtyRect, layerBounds, damageRect, clipRectToApply);
 
-    // Ensure our z-order lists are up-to-date.
+    // Ensure our lists are up-to-date.
     l->updateZOrderLists();
+    l->updateOverflowList();
 
     bool shouldPaint = l->intersectsDamageRect(layerBounds, damageRect);
     QVector<RenderLayer*>* negList = l->negZOrderList();
+    QVector<RenderLayer*>* ovfList = l->overflowList();
     if (shouldPaint && negList && negList->count() > 0)
         write(ts, *l, layerBounds, damageRect, clipRectToApply, -1, indent);
 
@@ -1398,6 +1451,11 @@ static void writeLayers(QTextStream &ts, const RenderLayer* rootLayer, RenderLay
     if (shouldPaint)
         write(ts, *l, layerBounds, damageRect, clipRectToApply, negList && negList->count() > 0, indent);
 
+    if (ovfList) {
+        for (QVector<RenderLayer*>::iterator it = ovfList->begin(); it != ovfList->end(); ++it)
+            writeLayers(ts, rootLayer, *it, paintDirtyRect, indent);
+    }
+
     QVector<RenderLayer*>* posList = l->posZOrderList();
     if (posList) {
         for (int i = 0; i != posList->count(); ++i)
@@ -1416,9 +1474,27 @@ void RenderLayer::dump(QTextStream &ts, const QString &ind)
 
 #endif
 
+bool RenderLayer::shouldBeOverflowOnly() const
+{
+    return renderer()->style() && renderer()->hasOverflowClip() &&
+           !renderer()->isPositioned() && !renderer()->isRelPositioned() && !isTransparent();
+}
+
 void RenderLayer::styleChanged()
 {
-    if (m_object->style()->overflow() == OMARQUEE && m_object->style()->marqueeBehavior() != MNONE) {
+    bool isOverflowOnly = shouldBeOverflowOnly();
+    if (isOverflowOnly != m_isOverflowOnly) {
+        m_isOverflowOnly = isOverflowOnly;
+        RenderLayer* p = parent();
+        RenderLayer* sc = stackingContext();
+        if (p)
+            p->dirtyOverflowList();
+        if (sc)
+            sc->dirtyZOrderLists();
+    }
+
+    if (m_object->hasOverflowClip() && 
+         m_object->style()->overflowX() == OMARQUEE && m_object->style()->marqueeBehavior() != MNONE) {
         if (!m_marquee)
             m_marquee = new Marquee(this);
         m_marquee->updateMarqueeStyle();
