@@ -286,7 +286,7 @@ void RenderListMarker::paint(PaintInfo& paintInfo, int _tx, int _ty)
     if (offset%2)
         bulletWidth++;
     if (!listPositionInside()) {
-        if (listItem->style()->direction() == LTR)
+        if (listItem && listItem->style()->direction() == LTR)
             xoff = -cMarkerPadding - offset;
         else
             xoff = cMarkerPadding + (haveImage ? 0 : (offset - bulletWidth));
@@ -555,13 +555,7 @@ void RenderListMarker::calcMinMaxWidth()
     case TRAD_CHINESE_INFORMAL:
         m_item = toTradChineseInformal( value );
         break;
-// Quotes:
-    case OPEN_QUOTE:
-        m_item = style()->openQuote( value );
-        break;
-    case CLOSE_QUOTE:
-        m_item = style()->closeQuote( value );
-        break;
+// special:
     case LNONE:
         break;
     default:
