@@ -2,9 +2,9 @@
  * This file is part of the HTML widget for KDE.
  *
  * Copyright (C) 1999-2003 Lars Knoll (knoll@kde.org)
- * Copyright (C) 2000-2003 Dirk Mueller (mueller@kde.org)
- * Copyright (C) 2003 Apple Computer, Inc.
- * Copyright (C) 2004 Germain Garand (germain@ebooksfrance.org)
+ *           (C) 2000-2003 Dirk Mueller (mueller@kde.org)
+ *           (C) 2003 Apple Computer, Inc.
+ *           (C) 2004-2006 Germain Garand (germain@ebooksfrance.org)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -36,16 +36,22 @@
 #include <qapplication.h>
 #include <qlineedit.h>
 #include <kglobalsettings.h>
+#include <kurlrequester.h>
 #include <qobject.h>
 #include <QVector>
 #include <QMatrix>
 #include <qpaintengine.h>
+#include <Q3ListBox>
+#include <Q3ScrollView>
 
 #include "khtml_ext.h"
 #include "khtmlview.h"
 #include "xml/dom2_eventsimpl.h"
 #include "khtml_part.h"
 #include "xml/dom_docimpl.h"
+#include "misc/helper.h"
+#include "misc/paintbuffer.h"
+#include "css/cssvalues.h"
 #include <kdebug.h>
 
 bool khtml::allowWidgetPaintEvents = false;
@@ -60,7 +66,7 @@ RenderReplaced::RenderReplaced(DOM::NodeImpl* node)
     // init RenderObject attributes
     setReplaced(true);
 
-    m_intrinsicWidth = 200;
+    m_intrinsicWidth = 300;
     m_intrinsicHeight = 150;
 }
 
@@ -87,11 +93,48 @@ void RenderReplaced::calcMinMaxWidth()
     setMinMaxKnown();
 }
 
+
 void RenderReplaced::position(InlineBox* box, int /*from*/, int /*len*/, bool /*reverse*/)
 {
     setPos( box->xPos(), box->yPos() );
 }
 
+FindSelectionResult RenderReplaced::checkSelectionPoint(int _x, int _y, int _tx, int _ty, DOM::NodeImpl*& node, int &offset, SelPointState &)
+{
+#if 0
+    kDebug(6040) << "RenderReplaced::checkSelectionPoint(_x="<<_x<<",_y="<<_y<<",_tx="<<_tx<<",_ty="<<_ty<<")" << endl
+                    << "xPos: " << xPos() << " yPos: " << yPos() << " width: " << width() << " height: " << height() << endl
+                << "_ty + yPos: " << (_ty + yPos()) << " + height: " << (_ty + yPos() + height()) << "; _tx + xPos: " << (_tx + xPos()) << " + width: " << (_tx + xPos() + width()) << endl;
+#endif
+    node = element();
+    offset = 0;
+
+    if ( _y < _ty + yPos() )
+        return SelectionPointBefore; // above -> before
+
+    if ( _y > _ty + yPos() + height() ) {
+        // below -> after
+        // Set the offset to the max
+        offset = 1;
+        return SelectionPointAfter;
+    }
+    if ( _x > _tx + xPos() + width() ) {
+        // to the right
+        // ### how to regard bidi in replaced elements? (LS)
+        offset = 1;
+        return SelectionPointAfterInLine;
+    }
+
+    // The Y matches, check if we're on the left
+    if ( _x < _tx + xPos() ) {
+        // ### how to regard bidi in replaced elements? (LS)
+        return SelectionPointBeforeInLine;
+    }
+
+    offset = _x > _tx + xPos() + width()/2;
+    return SelectionPointInside;
+}
+
 // -----------------------------------------------------------------------------
 
 RenderWidget::RenderWidget(DOM::NodeImpl* node)
@@ -104,7 +147,6 @@ RenderWidget::RenderWidget(DOM::NodeImpl* node)
     m_arena.reset(renderArena());
     m_resizePending = false;
     m_discardResizes = false;
-    m_isKHTMLWidget = false;
     m_needsMask = false;
 
     // this is no real reference counting, its just there
@@ -115,19 +157,27 @@ RenderWidget::RenderWidget(DOM::NodeImpl* node)
 
 void RenderWidget::detach()
 {
+    // warning: keep in sync with RenderObject::detach
+    
+    detachCounters();
     remove();
     deleteInlineBoxes();
 
     if ( m_widget ) {
         if ( m_view ) {
             m_view->setWidgetVisible(this, false);
-            m_view->removeChild( m_widget );
         }
-
+        KHTMLWidget* k = dynamic_cast<KHTMLWidget*>(m_widget);
+        if (k)
+            k->m_kwp->setRenderWidget(0);
         m_widget->removeEventFilter( this );
         m_widget->setMouseTracking( false );
     }
 
+    // make sure our DOM-node don't think we exist
+    if ( node() && node()->renderer() == this)
+        node()->setRenderer(0);
+
     deref();
 }
 
@@ -159,7 +209,7 @@ void  RenderWidget::resizeWidget( int w, int h )
     w = qMin( w, 2000 );
 
     if (m_widget->width() != w || m_widget->height() != h) {
-        m_resizePending = isKHTMLWidget();
+        m_resizePending = isRedirectedWidget();
         ref();
         element()->ref();
         QApplication::postEvent( this, new QWidgetResizeEvent( w, h ) );
@@ -198,6 +248,12 @@ void RenderWidget::flushWidgetResizes() //static
     QApplication::sendPostedEvents( 0, QWidgetResizeEvent::Type );
 }
 
+bool RenderWidget::isRedirectedWidget() const
+{
+    KHTMLWidget* k = dynamic_cast<KHTMLWidget*>(m_widget);
+    return k ? k->m_kwp->isRedirected() : false;   
+}
+
 void RenderWidget::setQWidget(QWidget *widget)
 {
     if (widget != m_widget)
@@ -211,10 +267,14 @@ void RenderWidget::setQWidget(QWidget *widget)
         }
         m_widget = widget;
         if (m_widget) {
+            m_widget->setParent(m_view->widget());
+            KHTMLWidget* k = dynamic_cast<KHTMLWidget*>(m_widget);
+            if (k)
+                k->m_kwp->setRenderWidget(this);
             connect( m_widget, SIGNAL( destroyed()), this, SLOT( slotWidgetDestructed()));
             m_widget->installEventFilter(this);
 
-            if ( (m_isKHTMLWidget = !strcmp(m_widget->objectName(), "__khtml")) && !qobject_cast<QFrame*>(m_widget))
+            if ( isRedirectedWidget() && !qobject_cast<QFrame*>(m_widget))
                 m_widget->setAttribute( Qt::WA_NoSystemBackground );
 
             if (m_widget->focusPolicy() > Qt::StrongFocus)
@@ -230,8 +290,10 @@ void RenderWidget::setQWidget(QWidget *widget)
                 setPos(xPos(), -500000);
         }
         m_view->setWidgetVisible(this, false);
-        m_view->addChild( m_widget, 0, -500000);
-        if ( m_widget ) m_widget->hide();
+        if ( m_widget ) {        
+            m_widget->move(0, -500000);
+            m_widget->hide();
+        }
         m_resizePending = false;
     }
 }
@@ -243,7 +305,7 @@ void RenderWidget::layout( )
     if ( m_widget ) {
         resizeWidget( m_width-borderLeft()-borderRight()-paddingLeft()-paddingRight(),
                       m_height-borderTop()-borderBottom()-paddingTop()-paddingBottom() );
-        if (!isKHTMLWidget() && !isFrame() && !m_needsMask) {
+        if (!isRedirectedWidget() && !isFrame() && !m_needsMask) {
             m_needsMask = true;
             RenderLayer* rl = enclosingStackingContext();
             RenderLayer* el = enclosingLayer();
@@ -255,7 +317,7 @@ void RenderWidget::layout( )
                 el = el->parent();
             }
             if (m_needsMask) {
-                rl->setHasOverlaidWidgets();
+	        if (rl) rl->setHasOverlaidWidgets();
                 canvas()->setNeedsWidgetMasks();
             }
         }
@@ -271,28 +333,38 @@ void RenderWidget::updateFromElement()
         QColor color = style()->color();
         QColor backgroundColor = style()->backgroundColor();
 
-        if ( color.isValid() || backgroundColor.isValid() ) {
-            QPalette pal(QApplication::palette(m_widget));
+        if (!backgroundColor.isValid() && !style()->htmlHacks())
+            backgroundColor = QColor(0,0,0,0);
 
+        // check if we have to paint our background and let it show through
+        bool trans = ( isRedirectedWidget() && style()->backgroundLayers() && 
+                       style()->backgroundLayers()->hasImage() && !qobject_cast<KUrlRequester*>(m_widget) );
+
+        QPalette pal(QApplication::palette(m_widget));
+        if (color.isValid() || backgroundColor.isValid() || trans) {
             int contrast_ = KGlobalSettings::contrast();
             int highlightVal = 100 + (2*contrast_+4)*16/10;
             int lowlightVal = 100 + (2*contrast_+4)*10;
+            bool shouldChangeBgPal = true;
 
-            if (backgroundColor.isValid()) {
-                if (!isKHTMLWidget()) {
-                    QPalette palette;
-                    palette.setColor(widget()->backgroundRole(), backgroundColor);
-                    widget()->setPalette(palette);
-                }
+            if (!backgroundColor.isValid()) 
+                backgroundColor = pal.color( widget()->backgroundRole() );
+            else
+                shouldChangeBgPal = !( (backgroundColor == colorForCSSValue(CSS_VAL_WINDOW)) ||
+                                       (backgroundColor == colorForCSSValue(CSS_VAL_BUTTONFACE)) );
+            if (shouldChangeBgPal || trans) {
+                pal.setColor(widget()->backgroundRole(), trans ? QColor(0,0,0,0) : backgroundColor);
                 for ( int i = 0; i < QPalette::NColorGroups; ++i ) {
-                    pal.setColor( (QPalette::ColorGroup)i, QPalette::Background, backgroundColor );
-                    pal.setColor( (QPalette::ColorGroup)i, QPalette::Light, backgroundColor.light(highlightVal) );
-                    pal.setColor( (QPalette::ColorGroup)i, QPalette::Dark, backgroundColor.dark(lowlightVal) );
-                    pal.setColor( (QPalette::ColorGroup)i, QPalette::Mid, backgroundColor.dark(120) );
-                    pal.setColor( (QPalette::ColorGroup)i, QPalette::Midlight, backgroundColor.light(110) );
-                    pal.setColor( (QPalette::ColorGroup)i, QPalette::Button, backgroundColor );
-                    pal.setColor( (QPalette::ColorGroup)i, QPalette::Base, backgroundColor );
-            }
+                    if (shouldChangeBgPal) {
+                        pal.setColor( (QPalette::ColorGroup)i, QPalette::Window, backgroundColor );
+                        pal.setColor( (QPalette::ColorGroup)i, QPalette::Light, backgroundColor.light(highlightVal) );
+                        pal.setColor( (QPalette::ColorGroup)i, QPalette::Dark, backgroundColor.dark(lowlightVal) );
+                        pal.setColor( (QPalette::ColorGroup)i, QPalette::Mid, backgroundColor.dark(120) );
+                        pal.setColor( (QPalette::ColorGroup)i, QPalette::Midlight, backgroundColor.light(110) );
+                    }
+                    pal.setColor( (QPalette::ColorGroup)i, QPalette::Button, trans ? QColor(0,0,0,0):backgroundColor );
+                    pal.setColor( (QPalette::ColorGroup)i, QPalette::Base, trans ? QColor(0,0,0,0):backgroundColor );
+                }
             }
             if ( color.isValid() ) {
                 struct ColorSet {
@@ -329,13 +401,11 @@ void RenderWidget::updateFromElement()
                     disfg = Qt::darkGray;
                 pal.setColor(QPalette::Disabled,QPalette::Foreground,disfg);
             }
-
-            m_widget->setPalette(pal);
         }
-        else
-            m_widget->setPalette(QPalette());
+        m_widget->setPalette(pal);    
+
         // Border:
-        if (shouldPaintBackgroundOrBorder())
+        if (shouldPaintBorder())
         {
             if (QFrame* frame = qobject_cast<QFrame*>(m_widget))
                 frame->setFrameShape(QFrame::NoFrame);
@@ -348,6 +418,33 @@ void RenderWidget::updateFromElement()
     RenderReplaced::updateFromElement();
 }
 
+void RenderWidget::paintBackground(QPainter *p, const QColor& c, const BackgroundLayer* bgLayer, QRect clipr, int _tx, int _ty, int w, int height)
+{
+    bool fudge = !shouldPaintBorder();
+    paintBackgroundExtended(p, c, bgLayer, clipr, _tx, _ty, w, height,
+                                fudge ? 1 : borderLeft() , fudge ? 1 : borderRight(), paddingLeft(), paddingRight(),
+                                fudge ? 1 : borderTop(), fudge ? 1 : borderBottom(), paddingTop(), paddingBottom());
+}
+                             
+
+void RenderWidget::paintBoxDecorations(PaintInfo& paintInfo, int _tx, int _ty)
+{
+    QRect r = QRect(_tx, _ty, width(), height());
+    QRect cr = r.intersected(paintInfo.r);
+
+    if (qobject_cast<QAbstractScrollArea*>(m_widget) || (isRedirectedWidget() && 
+            style()->backgroundLayers() && style()->backgroundLayers()->hasImage()))
+    {
+        paintBackgrounds(paintInfo.p, style()->backgroundColor(), style()->backgroundLayers(), 
+            cr, r.x(), r.y(), r.width(), r.height());
+    }
+
+    if (shouldPaintBorder() && style()->hasBorder())
+    {
+        paintBorder(paintInfo.p, _tx, _ty, width(), height(), style());
+    }
+}
+
 void RenderWidget::slotWidgetDestructed()
 {
     if (m_view)
@@ -367,17 +464,6 @@ void RenderWidget::setStyle(RenderStyle *_style)
             m_widget->hide();
         }
     }
-
-    // Don't paint borders if the border-style is native
-    // or borders are not supported on this widget
-    if (!canHaveBorder() ||
-        (style()->borderLeftStyle()   == BNATIVE &&
-         style()->borderRightStyle()  == BNATIVE &&
-         style()->borderTopStyle()    == BNATIVE &&
-         style()->borderBottomStyle() == BNATIVE))
-    {
-        setShouldPaintBackgroundOrBorder(false);
-    }
 }
 
 void RenderWidget::paint(PaintInfo& paintInfo, int _tx, int _ty)
@@ -385,7 +471,7 @@ void RenderWidget::paint(PaintInfo& paintInfo, int _tx, int _ty)
     _tx += m_x;
     _ty += m_y;
 
-    if (shouldPaintBackgroundOrBorder() &&
+    if (shouldPaintBackgroundOrBorder() && !qobject_cast<KUrlRequester*>(m_widget) &&
           (paintInfo.phase == PaintActionChildBackground || paintInfo.phase == PaintActionChildBackgrounds))
         paintBoxDecorations(paintInfo, _tx, _ty);
 
@@ -403,7 +489,7 @@ void RenderWidget::paint(PaintInfo& paintInfo, int _tx, int _ty)
     int xPos = _tx+borderLeft()+paddingLeft();
     int yPos = _ty+borderTop()+paddingTop();
 
-    bool khtmlw = isKHTMLWidget();
+    bool khtmlw = isRedirectedWidget();
     int childw = m_widget->width();
     int childh = m_widget->height();
     if ( (childw == 2000 || childh == 3072) && m_widget->inherits( "KHTMLView" ) ) {
@@ -412,8 +498,8 @@ void RenderWidget::paint(PaintInfo& paintInfo, int _tx, int _ty)
         int ch = m_view->visibleHeight();
 
 
-        int childx = m_view->childX( m_widget );
-        int childy = m_view->childY( m_widget );
+        int childx = m_widget->pos().x();
+        int childy = m_widget->pos().y();
 
         int xNew = xPos;
         int yNew = childy;
@@ -438,97 +524,25 @@ void RenderWidget::paint(PaintInfo& paintInfo, int _tx, int _ty)
         yPos = yNew;
     }
     m_view->setWidgetVisible(this, true);
-    m_view->addChild(m_widget, xPos, yPos );
+    if (!khtmlw)
+        m_view->addChild( m_widget, xPos, yPos );
+    else
+        m_widget->move( xPos, -500000 +yPos);
     m_widget->show();
     if (khtmlw)
         paintWidget(paintInfo, m_widget, xPos, yPos);
 }
 
-// The PaintBuffer class provides a shared buffer for widget painting.
-//
-// It will grow to encompass the biggest widget encountered, in order to avoid
-// constantly resizing.
-// When it grows over maxPixelBuffering, it periodically checks if such a size
-// is still needed.  If not, it shrinks down to the biggest size < maxPixelBuffering
-// that was requested during the overflow lapse.
-
-class PaintBuffer: public QObject
-{
-public:
-    static const int maxPixelBuffering = 320*200;
-    static const int leaseTime = 20*1000;
-
-    static QPixmap *grab( QSize s = QSize() ) {
-        if (!m_inst)
-            m_inst = new PaintBuffer;
-        return m_inst->getBuf( s );
-    }
-    static void release() { m_inst->m_grabbed = false; }
-protected:
-    PaintBuffer(): m_overflow(false), m_grabbed(false),
-                   m_timer(0), m_resetWidth(0), m_resetHeight(0) {};
-    void timerEvent(QTimerEvent* e) {
-        assert( m_timer == e->timerId() );
-        if (m_grabbed)
-            return;
-        m_buf = QPixmap(m_resetWidth, m_resetHeight);
-        m_resetWidth = m_resetHeight = 0;
-        killTimer( m_timer );
-        m_timer = 0;
-    }
-
-    QPixmap *getBuf( QSize s ) {
-        assert( !m_grabbed );
-        if (s.isEmpty())
-            return 0;
-
-        m_grabbed = true;
-        bool cur_overflow = false;
-
-        int nw = qMax(m_buf.width(), s.width());
-        int nh = qMax(m_buf.height(), s.height());
-
-        if (!m_overflow && (nw*nh > maxPixelBuffering))
-            cur_overflow = true;
-
-        if (nw != m_buf.width() || nh != m_buf.height())
-            m_buf = QPixmap(nw, nh);
-
-        if (cur_overflow) {
-            m_overflow = true;
-            m_timer = startTimer( leaseTime );
-        } else if (m_overflow) {
-            if( s.width()*s.height() > maxPixelBuffering ) {
-                killTimer( m_timer );
-                m_timer = startTimer( leaseTime );
-            } else {
-                if (s.width() > m_resetWidth)
-                    m_resetWidth = s.width();
-                if (s.height() > m_resetHeight)
-                    m_resetHeight = s.height();
-            }
-        }
-        return &m_buf;
-    }
-private:
-    static PaintBuffer* m_inst;
-    QPixmap m_buf;
-    bool m_overflow;
-    bool m_grabbed;
-    int m_timer;
-    int m_resetWidth;
-    int m_resetHeight;
-};
-
-PaintBuffer *PaintBuffer::m_inst = 0;
-
-static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int ty)
+static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int ty, bool buffered = false)
 {
     if (r.isNull() || r.isEmpty() )
         return;
-    QRegion blit(r);
+
     QVector<QWidget*> cw;
     QVector<QRect> cr;
+    QPoint thePoint(tx, ty);
+    QMatrix m = p->worldMatrix();
+    thePoint = thePoint * m;
 
     if (!widget->children().isEmpty()) {
         // build region
@@ -537,7 +551,6 @@ static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int
 	    QWidget* w = qobject_cast<QWidget* >(it);
 	    if ( w && !w->isTopLevel() && !w->isHidden()) {
 	        QRect r2 = w->geometry();
-		blit.subtract( r2 );
 		r2 = r2.intersect( r );
 		r2.translate(-w->x(), -w->y());
 		cr.append(r2);
@@ -545,51 +558,37 @@ static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int
 	    }
         }
     }
-    QVector<QRect> br = blit.rects();
-
-    const int cnt = br.size();
-    const bool external = p->device()->paintEngine()->type() >= QPaintEngine::PostScript;
-    QPixmap* const pm = PaintBuffer::grab( widget->size() );
-    if (!pm)
-    {
-        kWarning(6040) << "Rendering widget [ " << widget->metaObject()->className() << " ] failed due to invalid size." << endl;
-        return;
-    }
-
-    // fill background
-    if ( external ) {
-	// even hackier!
-        QPainter pt( pm );
-        const QColor c = widget->palette().color(QPalette::Base);
-        for (int i = 0; i < cnt; ++i)
-            pt.fillRect( br[i], c );
-    } else {
-        QRect dr;
-        for (int i = 0; i < cnt; ++i ) {
-            dr = br[i];
-            dr.translate( tx, ty );
-            dr = p->matrix().mapRect(dr);
-            bitBlt(pm, br[i].topLeft(), p->device(), dr);
-        }
-    }
 
     // send paint event
-    QPainter::setRedirected(widget, pm);
+    QPaintDevice *d = p->device();
+    if (buffered) {
+        if (!widget->size().isValid())
+            return;
+        QPixmap* pm = PaintBuffer::grab(widget->size());
+        QPainter pp(pm);
+        pp.setCompositionMode(QPainter::CompositionMode_Clear);
+        pp.eraseRect(r);
+        d = pm;
+    }
+    QPainter::setRedirected(widget, d, buffered ? QPoint(0,0) : -thePoint);
     QPaintEvent e( r );
     QApplication::sendEvent(widget, &e);
     QPainter::restoreRedirected(widget);
+    p->setWorldMatrix( m );
 
-    // transfer result
-    for ( int i = 0; i < cnt; ++i )
-        p->drawPixmap(QPoint(tx+br[i].x(), ty+br[i].y()), *pm, br[i]);
+    if (buffered) {
+        // transfer results
+        p->drawPixmap(QPoint(tx, ty), static_cast<QPixmap&>(*d), r);
+        PaintBuffer::release();
+    }
+
+    widget->setAttribute(Qt::WA_WState_InPaintEvent); // ### horrible hack - FIXME
 
-    // cleanup and recurse
-    PaintBuffer::release();
     QVector<QWidget*>::iterator cwit = cw.begin();
     QVector<QWidget*>::iterator cwitEnd = cw.end();
     QVector<QRect>::const_iterator crit = cr.begin();
     for (; cwit != cwitEnd; ++cwit, ++crit)
-        copyWidget(*crit, p, *cwit, tx+(*cwit)->x(), ty+(*cwit)->y());
+        copyWidget(*crit, p, *cwit, tx+(*cwit)->x(), ty+(*cwit)->y(), buffered);
 }
 
 void RenderWidget::paintWidget(PaintInfo& pI, QWidget *widget, int tx, int ty)
@@ -597,10 +596,12 @@ void RenderWidget::paintWidget(PaintInfo& pI, QWidget *widget, int tx, int ty)
     QPainter* const p = pI.p;
     allowWidgetPaintEvents = true;
 
+    bool buffered = p->combinedMatrix().m22() != 1.0 && !qobject_cast<KHTMLView*>(widget);
+
     QRect rr = pI.r;
     rr.translate(-tx, -ty);
     const QRect r = widget->rect().intersect( rr );
-    copyWidget(r, p, widget, tx, ty);
+    copyWidget(r, p, widget, tx, ty, buffered);
 
     allowWidgetPaintEvents = false;
 }
@@ -612,6 +613,14 @@ bool RenderWidget::eventFilter(QObject* /*o*/, QEvent* e)
         return false;
     if ( !element() ) return true;
 
+
+    static bool directToWidget = false;
+    if (directToWidget) {
+      //We're trying to get the event to the widget 
+      //promptly. So get out of here..
+      return false;
+    }
+
     ref();
     element()->ref();
 
@@ -620,12 +629,25 @@ bool RenderWidget::eventFilter(QObject* /*o*/, QEvent* e)
     //kDebug() << "RenderWidget::eventFilter type=" << e->type() << endl;
     switch(e->type()) {
     case QEvent::FocusOut:
+        // First, forward it to the widget, so that Qt gets a precise
+        // state of the focus before pesky JS can try changing it..
+        directToWidget = true;
+        QApplication::sendEvent(m_widget, e);
+        directToWidget = false;
+        filtered       = true; //We already delivered it!
+        
         // Don't count popup as a valid reason for losing the focus
         // (example: opening the options of a select combobox shouldn't emit onblur)
         if ( static_cast<QFocusEvent*>(e)->reason() != Qt::PopupFocusReason )
             handleFocusOut();
         break;
     case QEvent::FocusIn:
+        //As above, forward to the widget first...
+        directToWidget = true;
+        QApplication::sendEvent(m_widget, e);
+        directToWidget = false;
+        filtered       = true; //We already delivered it!
+
         //kDebug(6000) << "RenderWidget::eventFilter captures FocusIn" << endl;
         element()->getDocument()->setFocusNode(element());
 //         if ( isEditable() ) {
@@ -642,7 +664,7 @@ bool RenderWidget::eventFilter(QObject* /*o*/, QEvent* e)
         break;
 
     case QEvent::Wheel:
-        if (widget()->parentWidget() == view()->viewport()) {
+        if (widget()->parentWidget() == view()->widget()) {
             // don't allow the widget to react to wheel event unless its
             // currently focused. this avoids accidentally changing a select box
             // or something while wheeling a webpage.
@@ -688,6 +710,40 @@ void RenderWidget::EventPropagator::sendEvent(QEvent *e) {
     case QEvent::KeyRelease:
         keyReleaseEvent(static_cast<QKeyEvent *>(e));
         break;
+    case QEvent::FocusIn:
+        focusInEvent(static_cast<QFocusEvent *>(e));
+        break;
+    case QEvent::FocusOut:
+        focusOutEvent(static_cast<QFocusEvent *>(e));
+        break;
+    case QEvent::ContextMenu:
+        contextMenuEvent(static_cast<QContextMenuEvent*>(e));
+        break;
+    default:
+        break;
+    }
+}
+
+void RenderWidget::ScrollViewEventPropagator::sendEvent(QEvent *e) {
+    switch(e->type()) {
+    case QEvent::MouseButtonPress:
+        viewportMousePressEvent(static_cast<QMouseEvent *>(e));
+        break;
+    case QEvent::MouseButtonRelease:
+        viewportMouseReleaseEvent(static_cast<QMouseEvent *>(e));
+        break;
+    case QEvent::MouseButtonDblClick:
+        viewportMouseDoubleClickEvent(static_cast<QMouseEvent *>(e));
+        break;
+    case QEvent::MouseMove:
+        viewportMouseMoveEvent(static_cast<QMouseEvent *>(e));
+        break;
+    case QEvent::KeyPress:
+        keyPressEvent(static_cast<QKeyEvent *>(e));
+        break;
+    case QEvent::KeyRelease:
+        keyReleaseEvent(static_cast<QKeyEvent *>(e));
+        break;
     default:
         break;
     }
@@ -697,41 +753,33 @@ bool RenderWidget::handleEvent(const DOM::EventImpl& ev)
 {
     bool ret = false;
     switch(ev.id()) {
+    case EventImpl::DOMFOCUSIN_EVENT: 
+    case EventImpl::DOMFOCUSOUT_EVENT: {
+          QFocusEvent e(ev.id() == EventImpl::DOMFOCUSIN_EVENT ? QEvent::FocusIn : QEvent::FocusOut);
+          static_cast<EventPropagator *>(m_widget)->sendEvent(&e);
+          ret = e.isAccepted();
+          break;
+    }
+    case EventImpl::KHTML_MOUSEWHEEL_EVENT:
     case EventImpl::MOUSEDOWN_EVENT:
     case EventImpl::MOUSEUP_EVENT:
     case EventImpl::MOUSEMOVE_EVENT: {
         if (!ev.isMouseEvent()) break;
+
         const MouseEventImpl &me = static_cast<const MouseEventImpl &>(ev);
         QMouseEvent* const qme = me.qEvent();
-
-        int absx = 0;
-        int absy = 0;
-
-        absolutePosition(absx, absy);
-
-        const QPoint p(me.clientX() - absx + m_view->contentsX(),
-                 me.clientY() - absy + m_view->contentsY());
         QMouseEvent::Type type;
         Qt::MouseButton button = Qt::NoButton;
+        Qt::MouseButtons buttons = Qt::NoButton;
         Qt::KeyboardModifiers state = 0;
+        Qt::Orientation orient = Qt::Vertical;
 
         if (qme) {
+            buttons = qme->buttons();
             button = qme->button();
             state = qme->modifiers();
             type = qme->type();
         } else {
-            switch(me.id())  {
-            case EventImpl::MOUSEDOWN_EVENT:
-                type = QMouseEvent::MouseButtonPress;
-                break;
-            case EventImpl::MOUSEUP_EVENT:
-                type = QMouseEvent::MouseButtonRelease;
-                break;
-            case EventImpl::MOUSEMOVE_EVENT:
-            default:
-                type = QMouseEvent::MouseMove;
-                break;
-            }
             switch (me.button()) {
             case 0:
                 button = Qt::LeftButton;
@@ -745,6 +793,27 @@ bool RenderWidget::handleEvent(const DOM::EventImpl& ev)
             default:
                 break;
             }
+
+            buttons = button;
+
+            switch(me.id())  {
+            case EventImpl::MOUSEDOWN_EVENT:
+                type = QMouseEvent::MouseButtonPress;
+                break;
+            case EventImpl::MOUSEUP_EVENT:
+                type = QMouseEvent::MouseButtonRelease;
+                break;
+            case EventImpl::KHTML_MOUSEWHEEL_EVENT:
+            case EventImpl::MOUSEMOVE_EVENT:
+            default:
+                type = QMouseEvent::MouseMove;
+                button = Qt::NoButton;
+                break;
+            }
+            
+            if (me.orientation() == MouseEventImpl::OHorizontal)
+                orient = Qt::Horizontal;
+
             if (me.ctrlKey())
                 state |= Qt::ControlModifier;
             if (me.altKey())
@@ -755,12 +824,57 @@ bool RenderWidget::handleEvent(const DOM::EventImpl& ev)
                 state |= Qt::MetaModifier;
         }
 
-//     kDebug(6000) << "sending event to widget "
-//                   << " pos=" << p << " type=" << type
-//                   << " button=" << button << " state=" << state << endl;
-        QMouseEvent e(type, p, button, button, state);
-        static_cast<EventPropagator *>(m_widget)->sendEvent(&e);
-        ret = e.isAccepted();
+        int absx = 0;
+        int absy = 0;
+        absolutePosition(absx, absy);
+        absx += borderLeft()+paddingLeft();
+        absy += borderTop()+paddingTop();
+        QPoint p(me.clientX() - absx + m_view->contentsX(),
+                 me.clientY() - absy + m_view->contentsY());
+
+        QWidget* target = 0;
+        if (ev.id() == EventImpl::MOUSEDOWN_EVENT) {
+            target = m_widget->childAt(p);
+            if (!target || !::qobject_cast<QScrollBar*>(target))
+                target = m_widget;
+            view()->setMouseEventsTarget( target );
+        } else {
+            target = view()->mouseEventsTarget();
+            if (target) {
+                QWidget * parent = target;
+                while (parent && parent != m_widget)
+                    parent = parent->parentWidget();
+                if (!parent) return false;
+            } else {
+                target = m_widget;
+            }
+        }
+        p = target->mapFrom(m_widget, p);
+
+        bool needContextMenuEvent = (type == QMouseEvent::MouseButtonPress && button == Qt::RightButton);
+
+        QEvent *e = (ev.id() == EventImpl::KHTML_MOUSEWHEEL_EVENT) ?
+                    static_cast<QEvent*>(new QWheelEvent(p, -me.detail()*40, buttons, state, orient)) :
+                    static_cast<QEvent*>(new QMouseEvent(type,    p, button, buttons, state));
+        Q3ScrollView * sc = ::qobject_cast<Q3ScrollView*>(target);
+        if (sc && !::qobject_cast<Q3ListBox*>(m_widget))
+            static_cast<ScrollViewEventPropagator *>(sc)->sendEvent(e);
+        else
+            static_cast<EventPropagator *>(target)->sendEvent(e);
+
+        ret = e->isAccepted();
+
+        if (needContextMenuEvent) {
+            QContextMenuEvent cme(QContextMenuEvent::Mouse, p);
+            static_cast<EventPropagator *>(target)->sendEvent(&cme);
+        } else if (type == QEvent::MouseMove && m_widget->testAttribute(Qt::WA_Hover)) {
+            QHoverEvent he( QEvent::HoverMove, p, p );
+            QApplication::sendEvent(m_widget, &he);
+        }
+        if (ev.id() == EventImpl::MOUSEUP_EVENT && button == Qt::LeftButton) {
+            view()->setMouseEventsTarget( 0 );
+        }
+        delete e;
         break;
     }
     case EventImpl::KEYDOWN_EVENT:
@@ -809,11 +923,19 @@ bool RenderWidget::handleEvent(const DOM::EventImpl& ev)
     case EventImpl::MOUSEOUT_EVENT: {
 	QEvent moe( QEvent::Leave );
 	QApplication::sendEvent(m_widget, &moe);
+	if (m_widget->testAttribute(Qt::WA_Hover)) {
+            QHoverEvent he( QEvent::HoverLeave, QPoint(-1,-1), QPoint(0,0) );
+            QApplication::sendEvent(m_widget, &he);
+        }
 	break;
     }
     case EventImpl::MOUSEOVER_EVENT: {
 	QEvent moe( QEvent::Enter );
 	QApplication::sendEvent(m_widget, &moe);
+        if (m_widget->testAttribute(Qt::WA_Hover)) {
+            QHoverEvent he( QEvent::HoverEnter, QPoint(0,0), QPoint(-1,-1) );
+            QApplication::sendEvent(m_widget, &he);
+        }
 	view()->part()->resetHoverText();
 	break;
     }
@@ -834,42 +956,6 @@ void RenderWidget::deref()
     }
 }
 
-FindSelectionResult RenderReplaced::checkSelectionPoint(int _x, int _y, int _tx, int _ty, DOM::NodeImpl*& node, int &offset, SelPointState &)
-{
-#if 0
-    kDebug(6040) << "RenderReplaced::checkSelectionPoint(_x="<<_x<<",_y="<<_y<<",_tx="<<_tx<<",_ty="<<_ty<<")" << endl
-                    << "xPos: " << xPos() << " yPos: " << yPos() << " width: " << width() << " height: " << height() << endl
-                << "_ty + yPos: " << (_ty + yPos()) << " + height: " << (_ty + yPos() + height()) << "; _tx + xPos: " << (_tx + xPos()) << " + width: " << (_tx + xPos() + width()) << endl;
-#endif
-    node = element();
-    offset = 0;
-
-    if ( _y < _ty + yPos() )
-        return SelectionPointBefore; // above -> before
-
-    if ( _y > _ty + yPos() + height() ) {
-        // below -> after
-        // Set the offset to the max
-        offset = 1;
-        return SelectionPointAfter;
-    }
-    if ( _x > _tx + xPos() + width() ) {
-        // to the right
-        // ### how to regard bidi in replaced elements? (LS)
-        offset = 1;
-        return SelectionPointAfterInLine;
-    }
-
-    // The Y matches, check if we're on the left
-    if ( _x < _tx + xPos() ) {
-        // ### how to regard bidi in replaced elements? (LS)
-        return SelectionPointBeforeInLine;
-    }
-
-    offset = _x > _tx + xPos() + width()/2;
-    return SelectionPointInside;
-}
-
 #ifdef ENABLE_DUMP
 void RenderWidget::dump(QTextStream &stream, const QString &ind) const
 {
@@ -882,5 +968,26 @@ void RenderWidget::dump(QTextStream &stream, const QString &ind) const
 }
 #endif
 
+// -----------------------------------------------------------------------------
+
+QPoint KHTMLWidgetPrivate::absolutePos()
+{
+        if (!m_rw)
+            return m_pos;
+        int x, y;
+        m_rw->absolutePosition(x, y);
+        x += m_rw->borderLeft()+m_rw->paddingLeft();
+        y += m_rw->borderTop()+m_rw->paddingTop();
+        return QPoint(x, y);
+}
+
+// -----------------------------------------------------------------------------
+
+KHTMLWidget::KHTMLWidget() 
+    : m_kwp(new KHTMLWidgetPrivate()) {}
+
+KHTMLWidget::~KHTMLWidget() 
+    { delete m_kwp; }
+
 #include "render_replaced.moc"
 
