@@ -106,10 +106,14 @@ StyleVisualData::StyleVisualData(const StyleVisualData& o )
 BackgroundLayer::BackgroundLayer()
 :m_image(RenderStyle::initialBackgroundImage()),
  m_bgAttachment(RenderStyle::initialBackgroundAttachment()),
+ m_bgClip(RenderStyle::initialBackgroundClip()),
+ m_bgOrigin(RenderStyle::initialBackgroundOrigin()),
  m_bgRepeat(RenderStyle::initialBackgroundRepeat()),
+ m_backgroundSize(RenderStyle::initialBackgroundSize()),
  m_next(0)
 {
-    m_imageSet = m_attachmentSet = m_repeatSet = m_xPosSet = m_yPosSet = false;
+    m_imageSet = m_attachmentSet = m_clipSet = m_originSet =
+            m_repeatSet = m_xPosSet = m_yPosSet = m_backgroundSizeSet = false;
 }
 
 BackgroundLayer::BackgroundLayer(const BackgroundLayer& o)
@@ -119,12 +123,18 @@ BackgroundLayer::BackgroundLayer(const BackgroundLayer& o)
     m_xPosition = o.m_xPosition;
     m_yPosition = o.m_yPosition;
     m_bgAttachment = o.m_bgAttachment;
+    m_bgClip = o.m_bgClip;
+    m_bgOrigin = o.m_bgOrigin;
     m_bgRepeat = o.m_bgRepeat;
+    m_backgroundSize = o.m_backgroundSize;
     m_imageSet = o.m_imageSet;
     m_attachmentSet = o.m_attachmentSet;
+    m_clipSet = o.m_clipSet;
+    m_originSet = o.m_originSet;
     m_repeatSet = o.m_repeatSet;
     m_xPosSet = o.m_xPosSet;
     m_yPosSet = o.m_yPosSet;
+    m_backgroundSizeSet = o.m_backgroundSizeSet;
 }
 
 BackgroundLayer::~BackgroundLayer()
@@ -142,22 +152,28 @@ BackgroundLayer& BackgroundLayer::operator=(const BackgroundLayer& o) {
     m_xPosition = o.m_xPosition;
     m_yPosition = o.m_yPosition;
     m_bgAttachment = o.m_bgAttachment;
+    m_bgClip = o.m_bgClip;
+    m_bgOrigin = o.m_bgOrigin;
     m_bgRepeat = o.m_bgRepeat;
+    m_backgroundSize = o.m_backgroundSize;
 
     m_imageSet = o.m_imageSet;
     m_attachmentSet = o.m_attachmentSet;
+    m_originSet = o.m_originSet;
     m_repeatSet = o.m_repeatSet;
     m_xPosSet = o.m_xPosSet;
     m_yPosSet = o.m_yPosSet;
+    m_backgroundSizeSet = o.m_backgroundSizeSet;
 
     return *this;
 }
 
 bool BackgroundLayer::operator==(const BackgroundLayer& o) const {
     return m_image == o.m_image && m_xPosition == o.m_xPosition && m_yPosition == o.m_yPosition &&
-           m_bgAttachment == o.m_bgAttachment && m_bgRepeat == o.m_bgRepeat &&
+           m_bgAttachment == o.m_bgAttachment && m_bgClip == o.m_bgClip && m_bgOrigin == o.m_bgOrigin && m_bgRepeat == o.m_bgRepeat &&
+           m_backgroundSize.width == o.m_backgroundSize.width && m_backgroundSize.height == o.m_backgroundSize.height &&
            m_imageSet == o.m_imageSet && m_attachmentSet == o.m_attachmentSet && m_repeatSet == o.m_repeatSet &&
-           m_xPosSet == o.m_xPosSet && m_yPosSet == o.m_yPosSet &&
+           m_xPosSet == o.m_xPosSet && m_yPosSet == o.m_yPosSet && m_backgroundSizeSet == o.m_backgroundSizeSet &&
            ((m_next && o.m_next) ? *m_next == *o.m_next : m_next == o.m_next);
 }
 
@@ -208,6 +224,28 @@ void BackgroundLayer::fillUnsetProperties()
         }
     }
 
+    for (curr = this; curr && curr->isBackgroundClipSet(); curr = curr->next());
+    if (curr && curr != this) {
+        // We need to fill in the remaining values with the pattern specified.
+        for (BackgroundLayer* pattern = this; curr; curr = curr->next()) {
+            curr->m_bgClip = pattern->m_bgClip;
+            pattern = pattern->next();
+            if (pattern == curr || !pattern)
+                pattern = this;
+        }
+    }
+
+    for (curr = this; curr && curr->isBackgroundOriginSet(); curr = curr->next());
+    if (curr && curr != this) {
+        // We need to fill in the remaining values with the pattern specified.
+        for (BackgroundLayer* pattern = this; curr; curr = curr->next()) {
+            curr->m_bgOrigin = pattern->m_bgOrigin;
+            pattern = pattern->next();
+            if (pattern == curr || !pattern)
+                pattern = this;
+        }
+    }
+
     for (curr = this; curr && curr->isBackgroundRepeatSet(); curr = curr->next());
     if (curr && curr != this) {
         // We need to fill in the remaining values with the pattern specified.
@@ -218,6 +256,17 @@ void BackgroundLayer::fillUnsetProperties()
                 pattern = this;
         }
     }
+
+    for (curr = this; curr && curr->isBackgroundSizeSet(); curr = curr->next());
+    if (curr && curr != this) {
+        // We need to fill in the remaining values with the pattern specified.
+        for (BackgroundLayer* pattern = this; curr; curr = curr->next()) {
+            curr->m_backgroundSize = pattern->m_backgroundSize;
+            pattern = pattern->next();
+            if (pattern == curr || !pattern)
+                pattern = this;
+        }
+    }
 }
 
 void BackgroundLayer::cullEmptyLayers()
@@ -227,7 +276,9 @@ void BackgroundLayer::cullEmptyLayers()
         next = p->m_next;
         if (next && !next->isBackgroundImageSet() &&
             !next->isBackgroundXPositionSet() && !next->isBackgroundYPositionSet() &&
-            !next->isBackgroundAttachmentSet() && !next->isBackgroundRepeatSet()) {
+            !next->isBackgroundAttachmentSet() && !next->isBackgroundClipSet() &&
+            !next->isBackgroundOriginSet() && !next->isBackgroundRepeatSet() &&
+            !next->isBackgroundSizeSet()) {
             delete next;
             p->m_next = 0;
             break;
@@ -249,7 +300,7 @@ bool StyleBackgroundData::operator==(const StyleBackgroundData& o) const
 
 StyleGeneratedData::StyleGeneratedData() : Shared<StyleGeneratedData>(), content(0), counter_reset(0), counter_increment(0) {}
 
-StyleGeneratedData::~StyleGeneratedData() 
+StyleGeneratedData::~StyleGeneratedData()
 {
     if (counter_reset) counter_reset->deref();
     if (counter_increment) counter_increment->deref();
@@ -322,7 +373,7 @@ static bool compareCounterActList(const CSSValueListImpl* ca, const CSSValueList
 
 bool StyleGeneratedData::counterDataEquivalent(const StyleGeneratedData* otherStyle) const
 {
-    return compareCounterActList(counter_reset, otherStyle->counter_reset) && 
+    return compareCounterActList(counter_reset, otherStyle->counter_reset) &&
            compareCounterActList(counter_increment, otherStyle->counter_increment);
 }
 
@@ -558,8 +609,11 @@ bool RenderStyle::operator==(const RenderStyle& o) const
             inherited == o.inherited);
 }
 
-enum EPseudoBit { NO_BIT = 0x0, BEFORE_BIT = 0x1, AFTER_BIT = 0x2, FIRST_LINE_BIT = 0x4,
-                  FIRST_LETTER_BIT = 0x8, SELECTION_BIT = 0x10 };
+enum EPseudoBit { NO_BIT = 0x0,
+                  FIRST_LINE_BIT = 0x1, FIRST_LETTER_BIT = 0x2, SELECTION_BIT = 0x4,
+                  BEFORE_BIT = 0x8, AFTER_BIT = 0x10, MARKER_BIT = 0x20,
+                  REPLACED_BIT = 0x40
+                  };
 
 static int pseudoBit(RenderStyle::PseudoId pseudo)
 {
@@ -568,6 +622,10 @@ static int pseudoBit(RenderStyle::PseudoId pseudo)
             return BEFORE_BIT;
         case RenderStyle::AFTER:
             return AFTER_BIT;
+        case RenderStyle::MARKER:
+            return MARKER_BIT;
+        case RenderStyle::REPLACED:
+            return REPLACED_BIT;
         case RenderStyle::FIRST_LINE:
             return FIRST_LINE_BIT;
         case RenderStyle::FIRST_LETTER:
@@ -600,7 +658,7 @@ RenderStyle* RenderStyle::getPseudoStyle(PseudoId pid) const
     if (noninherited_flags.f._styleType==NOPSEUDO)
         for (ps = pseudoStyle; ps; ps = ps->pseudoStyle)
             if (ps->noninherited_flags.f._styleType==pid)
-		break;
+                break;
     return ps;
 }
 
@@ -697,7 +755,8 @@ RenderStyle::Diff RenderStyle::diff( const RenderStyle *other ) const
 
     if ( *box.get() != *other->box.get() ||
          *visual.get() != *other->visual.get() ||
-         *surround.get() != *other->surround.get() ||
+         (*surround.get() != *other->surround.get()
+           && (other->position() == STATIC || other->position() != position())) ||
          !(inherited->indent == other->inherited->indent) ||
          !(inherited->line_height == other->inherited->line_height) ||
          !(inherited->style_image == other->inherited->style_image) ||
@@ -705,7 +764,8 @@ RenderStyle::Diff RenderStyle::diff( const RenderStyle *other ) const
          !(inherited->border_hspacing == other->inherited->border_hspacing) ||
          !(inherited->border_vspacing == other->inherited->border_vspacing) ||
          !(inherited_flags.f._visuallyOrdered == other->inherited_flags.f._visuallyOrdered) ||
-         !(inherited_flags.f._htmlHacks == other->inherited_flags.f._htmlHacks) )
+         !(inherited_flags.f._htmlHacks == other->inherited_flags.f._htmlHacks) ||
+         !(noninherited_flags.f._textOverflow == other->noninherited_flags.f._textOverflow) )
         return CbLayout;
 
     // changes causing Layout changes:
@@ -752,6 +812,11 @@ RenderStyle::Diff RenderStyle::diff( const RenderStyle *other ) const
 	)
         return Layout;
 
+    // Overflow returns a layout hint.
+    if (noninherited_flags.f._overflowX != other->noninherited_flags.f._overflowX ||
+        noninherited_flags.f._overflowY != other->noninherited_flags.f._overflowY)
+        return Layout;
+
 // only for inline:
 //     EVerticalAlign _vertical_align : 4;
 
@@ -759,14 +824,23 @@ RenderStyle::Diff RenderStyle::diff( const RenderStyle *other ) const
          !(noninherited_flags.f._vertical_align == other->noninherited_flags.f._vertical_align) )
 	    return Layout;
 
+    if (*surround.get() != *other->surround.get()) {
+        assert( other->position() != STATIC );                      // this style is positioned or relatively positioned
+        if ( surround->hasSamePBMData(*other->surround.get()) &&    // padding/border/margin are identical
+             (other->position() == RELATIVE ||
+               !(other->left().isVariable() && other->right().isVariable()) &&  // X isn't static
+               !(other->top().isVariable() && other->bottom().isVariable()) ))   // neither is Y
+           // therefore only the offset is different
+           return Position;
+        return Layout;
+    }
+
     // Visible:
 // 	EVisibility _visibility : 2;
-//     EOverflow _overflow : 4 ;
 // 	int _text_decorations : 4;
 //     DataRef<StyleBackgroundData> background;
     if (inherited->color != other->inherited->color ||
         !(inherited_flags.f._visibility == other->inherited_flags.f._visibility) ||
-        !(noninherited_flags.f._overflow == other->noninherited_flags.f._overflow) ||
         !(inherited_flags.f._text_decorations == other->inherited_flags.f._text_decorations) ||
         !(noninherited_flags.f._hasClip == other->noninherited_flags.f._hasClip) ||
         visual->textDecoration != other->visual->textDecoration ||
@@ -961,8 +1035,10 @@ void RenderStyle::addContent(EQuoteContent q)
 
 // content: normal is the same as having no content at all
 void RenderStyle::setContentNormal() {
-    delete generated->content;
-    generated.access()->content = 0;
+    if (generated->content != 0) {
+        delete generated->content;
+        generated.access()->content = 0;
+    }
 }
 
 // content: none, add an empty content node
@@ -971,6 +1047,15 @@ void RenderStyle::setContentNone() {
     generated.access()->content = new ContentData;
 }
 
+void RenderStyle::setContentData(ContentData *data) {
+    if (data != generated->content) {
+        if (data)
+            generated.access()->content = new ContentData(*data);
+        else
+            generated.access()->content = 0;
+    }
+}
+
 ContentData::ContentData(const ContentData& o) : _contentType(o._contentType)
 {
     switch (_contentType) {
@@ -1083,29 +1168,36 @@ bool RenderStyle::hasCounterIncrement(const DOM::DOMString& c) const
         return false;
 }
 
-static short readCounter(const DOM::DOMString& c, CSSValueListImpl *l)
-{
-    int len = l->length();
-    for(int i=0; i<len; i++) {
-        CounterActImpl* ca = static_cast<CounterActImpl*>(l->item(i));
-        Q_ASSERT(ca != 0);
-        if (ca->m_counter == c) return ca->m_value;
-    }
-    return 0;
-}
-
 short RenderStyle::counterReset(const DOM::DOMString& c) const
 {
-    if (generated->counter_reset)
-        return readCounter(c, generated->counter_reset);
+    if (generated->counter_reset) {
+        int len = generated->counter_reset->length();
+        int value = 0;
+        // Return the last matching counter-reset
+        for(int i=0; i<len; i++) {
+            CounterActImpl* ca = static_cast<CounterActImpl*>(generated->counter_reset->item(i));
+            Q_ASSERT(ca != 0);
+            if (ca->m_counter == c) value = ca->m_value;
+        }
+        return value;
+    }
     else
         return 0;
 }
 
 short RenderStyle::counterIncrement(const DOM::DOMString& c) const
 {
-    if (generated->counter_increment)
-        return readCounter(c, generated->counter_increment);
+    if (generated->counter_increment) {
+        int len = generated->counter_increment->length();
+        int value = 0;
+        // Return the sum of matching counter-increments
+        for(int i=0; i<len; i++) {
+            CounterActImpl* ca = static_cast<CounterActImpl*>(generated->counter_increment->item(i));
+            Q_ASSERT(ca != 0);
+            if (ca->m_counter == c) value += ca->m_value;
+        }
+        return value;
+    }
     else
         return 0;
 }
