@@ -4,7 +4,7 @@
  * Copyright (C) 2000-2003 Lars Knoll (knoll@kde.org)
  *           (C) 2000 Antti Koivisto (koivisto@kde.org)
  *           (C) 2000-2003 Dirk Mueller (mueller@kde.org)
- *           (C) 2002-2004 Apple Computer, Inc.
+ *           (C) 2003-2005 Apple Computer, Inc.
  *           (C) 2004-2006 Allan Sandfeld Jensen (kde@carewolf.com)
  *
  * This library is free software; you can redistribute it and/or
@@ -211,14 +211,12 @@ public:
     unsigned short width : 12;
     EBorderStyle style : 6;
 
-    bool nonZero() const
-    {
-      // rikkus: workaround for gcc 2.95.3
-      return width!=0 && !(style==BNONE);
+    bool nonZero(bool checkStyle = true) const {
+        return width != 0 && !(checkStyle && style == BNONE);
     }
 
     bool isTransparent() const {
-        return color.isValid() && qAlpha(color.rgba()) == 0;
+        return color.isValid() && color.alpha() == 0;
     }
 
     bool operator==(const BorderValue& o) const
@@ -292,6 +290,30 @@ public:
     	return left.nonZero() || right.nonZero() || top.nonZero() || bottom.nonZero();
     }
 
+    unsigned short borderLeftWidth() const {
+        if (left.style == BNONE || left.style == BHIDDEN || left.style == BNATIVE)
+            return 0;
+        return left.width;
+    }
+
+    unsigned short borderRightWidth() const {
+        if (right.style == BNONE || right.style == BHIDDEN || right.style == BNATIVE)
+            return 0;
+        return right.width;
+    }
+
+    unsigned short borderTopWidth() const {
+        if (top.style == BNONE || top.style == BHIDDEN || top.style == BNATIVE)
+            return 0;
+        return top.width;
+    }
+
+    unsigned short borderBottomWidth() const {
+        if (bottom.style == BNONE || bottom.style == BHIDDEN || bottom.style == BNATIVE)
+            return 0;
+        return bottom.width;
+    }
+
     bool operator==(const BorderData& o) const
     {
     	return left==o.left && right==o.right && top==o.top && bottom==o.bottom;
@@ -309,6 +331,9 @@ public:
     bool operator!=(const StyleSurroundData& o) const {
         return !(*this == o);
     }
+    bool hasSamePBMData(const StyleSurroundData& o) const {
+        return (margin == o.margin) && (padding == o.padding) && (border == o.border);
+    }
 
     LengthBox offset;
     LengthBox margin;
@@ -407,10 +432,19 @@ public:
 };
 
 //------------------------------------------------
+enum EBackgroundBox {
+    BGBORDER, BGPADDING, BGCONTENT
+};
+
 enum EBackgroundRepeat {
     REPEAT, REPEAT_X, REPEAT_Y, NO_REPEAT
 };
 
+struct LengthSize {
+    Length width;
+    Length height;
+};
+
 struct BackgroundLayer {
 public:
     BackgroundLayer();
@@ -420,7 +454,11 @@ public:
     Length backgroundXPosition() const { return m_xPosition; }
     Length backgroundYPosition() const { return m_yPosition; }
     bool backgroundAttachment() const { return m_bgAttachment; }
+    EBackgroundBox backgroundClip() const { return m_bgClip; }
+    EBackgroundBox backgroundOrigin() const { return m_bgOrigin; }
     EBackgroundRepeat backgroundRepeat() const { return m_bgRepeat; }
+    LengthSize backgroundSize() const { return m_backgroundSize; }
+
     BackgroundLayer* next() const { return m_next; }
     BackgroundLayer* next() { return m_next; }
 
@@ -428,19 +466,28 @@ public:
     bool isBackgroundXPositionSet() const { return m_xPosSet; }
     bool isBackgroundYPositionSet() const { return m_yPosSet; }
     bool isBackgroundAttachmentSet() const { return m_attachmentSet; }
+    bool isBackgroundClipSet() const { return m_clipSet; }
+    bool isBackgroundOriginSet() const { return m_originSet; }
     bool isBackgroundRepeatSet() const { return m_repeatSet; }
+    bool isBackgroundSizeSet() const { return m_backgroundSizeSet; }
 
     void setBackgroundImage(CachedImage* i) { m_image = i; m_imageSet = true; }
     void setBackgroundXPosition(const Length& l) { m_xPosition = l; m_xPosSet = true; }
     void setBackgroundYPosition(const Length& l) { m_yPosition = l; m_yPosSet = true; }
     void setBackgroundAttachment(bool b) { m_bgAttachment = b; m_attachmentSet = true; }
+    void setBackgroundClip(EBackgroundBox b) { m_bgClip = b; m_clipSet = true; }
+    void setBackgroundOrigin(EBackgroundBox b) { m_bgOrigin = b; m_originSet = true; }
     void setBackgroundRepeat(EBackgroundRepeat r) { m_bgRepeat = r; m_repeatSet = true; }
+    void setBackgroundSize(const LengthSize& b) { m_backgroundSize = b; m_backgroundSizeSet = true; }
 
     void clearBackgroundImage() { m_imageSet = false; }
     void clearBackgroundXPosition() { m_xPosSet = false; }
     void clearBackgroundYPosition() { m_yPosSet = false; }
     void clearBackgroundAttachment() { m_attachmentSet = false; }
+    void clearBackgroundClip() { m_clipSet = false; }
+    void clearBackgroundOrigin() { m_originSet = false; }
     void clearBackgroundRepeat() { m_repeatSet = false; }
+    void clearBackgroundSize() { m_backgroundSizeSet = false; }
 
     void setNext(BackgroundLayer* n) { if (m_next != n) { delete m_next; m_next = n; } }
 
@@ -474,13 +521,20 @@ public:
     Length m_yPosition;
 
     bool m_bgAttachment : 1;
+    EBackgroundBox m_bgClip : 2;
+    EBackgroundBox m_bgOrigin : 2;
     EBackgroundRepeat m_bgRepeat : 2;
 
+    LengthSize m_backgroundSize;
+
     bool m_imageSet : 1;
     bool m_attachmentSet : 1;
+    bool m_clipSet : 1;
+    bool m_originSet : 1;
     bool m_repeatSet : 1;
     bool m_xPosSet : 1;
     bool m_yPosSet : 1;
+    bool m_backgroundSizeSet : 1;
 
     BackgroundLayer* m_next;
 };
@@ -507,7 +561,7 @@ enum EQuoteContent {
 };
 
 enum ContentType {
-    CONTENT_NONE = 0, CONTENT_NORMAL, CONTENT_OBJECT, 
+    CONTENT_NONE = 0, CONTENT_NORMAL, CONTENT_OBJECT,
     CONTENT_TEXT, CONTENT_COUNTER, CONTENT_QUOTE
 };
 
@@ -745,6 +799,17 @@ enum EListStyleType {
      LNONE
 };
 
+inline bool isListStyleCounted(EListStyleType type)
+{
+    switch(type) {
+    case LDISC: case LCIRCLE: case LSQUARE: case LBOX: case LDIAMOND:
+    case LNONE:
+        return false;
+    default:
+        return true;
+    }
+}
+
 enum EListStylePosition { OUTSIDE, INSIDE };
 
 enum EVisibility { VISIBLE, HIDDEN, COLLAPSE };
@@ -775,8 +840,11 @@ class RenderStyle : public Shared<RenderStyle>
 public:
     KHTML_EXPORT static void cleanup();
 
-    // static pseudo styles. Dynamic ones are produced on the fly.
-    enum PseudoId { NOPSEUDO, FIRST_LINE, FIRST_LETTER, BEFORE, AFTER, SELECTION };
+    // pseudo elements
+    enum PseudoId {
+        NOPSEUDO, FIRST_LINE, FIRST_LETTER, SELECTION,
+        BEFORE, AFTER, REPLACED, MARKER
+    };
 
 protected:
 
@@ -831,7 +899,8 @@ protected:
             struct {
                 EDisplay _display : 5;
                 EDisplay _originalDisplay: 5;
-                EOverflow _overflow : 4 ;
+                EOverflow _overflowX : 4 ;
+                EOverflow _overflowY : 4 ;
                 EVerticalAlign _vertical_align : 4;
                 EClear _clear : 2;
                 EPosition _position : 2;
@@ -844,10 +913,15 @@ protected:
 
                 PseudoId _styleType : 4;
                 bool _hasClip : 1;
-                unsigned _pseudoBits : 6;
+                unsigned _pseudoBits : 8;
                 EUnicodeBidi _unicodeBidi : 2;
 
-                unsigned int unused : 18;
+                // non CSS2 non-inherited
+                bool _textOverflow : 1; // Whether or not lines that spill out should be truncated with "..."
+
+                bool _inherited_noninherited : 1;
+
+                unsigned int unused : 10;
             } f;
             quint64 _niflags;
         };
@@ -901,7 +975,8 @@ protected:
 	                                  // makes use of uninitialised bits according to valgrind
 
         noninherited_flags.f._display = noninherited_flags.f._originalDisplay = initialDisplay();
-	noninherited_flags.f._overflow = initialOverflow();
+	noninherited_flags.f._overflowX = initialOverflowX();
+	noninherited_flags.f._overflowY = initialOverflowY();
 	noninherited_flags.f._vertical_align = initialVerticalAlign();
 	noninherited_flags.f._clear = initialClear();
 	noninherited_flags.f._position = initialPosition();
@@ -914,6 +989,8 @@ protected:
 	noninherited_flags.f._hasClip = false;
         noninherited_flags.f._pseudoBits = 0;
 	noninherited_flags.f._unicodeBidi = initialUnicodeBidi();
+	noninherited_flags.f._textOverflow = initialTextOverflow();
+        noninherited_flags.f._inherited_noninherited = false;
         noninherited_flags.f.unused = 0;
     }
 
@@ -928,7 +1005,14 @@ public:
 
     void inheritFrom(const RenderStyle* inheritParent);
 
-    PseudoId styleType() { return  noninherited_flags.f._styleType; }
+    PseudoId styleType() const { return  noninherited_flags.f._styleType; }
+    void setStyleType(PseudoId pi) { noninherited_flags.f._styleType = pi; }
+    bool isGenerated() const {
+        if (styleType() == AFTER || styleType() == BEFORE || styleType() == MARKER || styleType() == REPLACED)
+            return true;
+        else
+            return false;
+    }
 
     bool hasPseudoStyle(PseudoId pi) const;
     void setHasPseudoStyle(PseudoId pi, bool b=true);
@@ -943,7 +1027,7 @@ public:
     bool        hasOffset() const { return surround->offset.nonZero(); }
 
     bool hasBackground() const {
-        if (backgroundColor().isValid() && qAlpha(backgroundColor().rgb()) > 0)
+        if (backgroundColor().isValid() && backgroundColor().alpha() > 0)
             return true;
         else
             return background->m_background.hasImage();
@@ -973,30 +1057,27 @@ public:
     Length  	minHeight() const { return box->min_height; }
     Length  	maxHeight() const { return box->max_height; }
 
+    const BorderData& border() const { return surround->border; }
     const BorderValue& borderLeft() const { return surround->border.left; }
     const BorderValue& borderRight() const { return surround->border.right; }
     const BorderValue& borderTop() const { return surround->border.top; }
     const BorderValue& borderBottom() const { return surround->border.bottom; }
 
-    unsigned short  borderLeftWidth() const
-    { if( surround->border.left.style == BNONE || surround->border.left.style == BNATIVE) return 0; return surround->border.left.width; }
+    unsigned short  borderLeftWidth() const { return surround->border.borderLeftWidth(); }
     EBorderStyle    borderLeftStyle() const { return surround->border.left.style; }
-    const QColor &  borderLeftColor() const { return surround->border.left.color; }
+    const QColor&  borderLeftColor() const { return surround->border.left.color; }
     bool borderLeftIsTransparent() const { return surround->border.left.isTransparent(); }
-    unsigned short  borderRightWidth() const
-    { if (surround->border.right.style == BNONE || surround->border.left.style == BNATIVE) return 0; return surround->border.right.width; }
+    unsigned short  borderRightWidth() const { return surround->border.borderRightWidth(); }
     EBorderStyle    borderRightStyle() const {  return surround->border.right.style; }
-    const QColor &  	    borderRightColor() const {  return surround->border.right.color; }
+    const QColor&   borderRightColor() const {  return surround->border.right.color; }
     bool borderRightIsTransparent() const { return surround->border.right.isTransparent(); }
-    unsigned short  borderTopWidth() const
-    { if(surround->border.top.style == BNONE || surround->border.left.style == BNATIVE) return 0; return surround->border.top.width; }
-    EBorderStyle    borderTopStyle() const {return surround->border.top.style; }
-    const QColor &  borderTopColor() const {  return surround->border.top.color; }
+    unsigned short  borderTopWidth() const { return surround->border.borderTopWidth(); }
+    EBorderStyle    borderTopStyle() const { return surround->border.top.style; }
+    const QColor&  borderTopColor() const {  return surround->border.top.color; }
     bool borderTopIsTransparent() const { return surround->border.top.isTransparent(); }
-    unsigned short  borderBottomWidth() const
-    { if(surround->border.bottom.style == BNONE || surround->border.left.style == BNATIVE) return 0; return surround->border.bottom.width; }
+    unsigned short  borderBottomWidth() const { return surround->border.borderBottomWidth(); }
     EBorderStyle    borderBottomStyle() const {  return surround->border.bottom.style; }
-    const QColor &  	    borderBottomColor() const {  return surround->border.bottom.color; }
+    const QColor&   borderBottomColor() const {  return surround->border.bottom.color; }
     bool borderBottomIsTransparent() const { return surround->border.bottom.isTransparent(); }
 
     unsigned short  outlineSize() const { return outlineWidth() + outlineOffset(); }
@@ -1007,9 +1088,12 @@ public:
     bool outlineStyleIsAuto() const { return background->m_outline._auto; }
     const QColor &  outlineColor() const {  return background->m_outline.color; }
 
-    EOverflow overflow() const { return  noninherited_flags.f._overflow; }
-    bool hidesOverflow() const { return overflow() != OVISIBLE; }
-    bool scrollsOverflow() const { return overflow() == OSCROLL || overflow() == OAUTO; }
+    EOverflow overflowX() const { return  noninherited_flags.f._overflowX; }
+    EOverflow overflowY() const { return  noninherited_flags.f._overflowY; }
+    bool hidesOverflow() const {
+        // either both overflow are visible or none are
+        return overflowX() != OVISIBLE;
+    }
 
     EVisibility visibility() const { return inherited_flags.f._visibility; }
     EVerticalAlign verticalAlign() const { return  noninherited_flags.f._vertical_align; }
@@ -1019,6 +1103,7 @@ public:
     Length clipRight() const { return visual->clip.right; }
     Length clipTop() const { return visual->clip.top; }
     Length clipBottom() const { return visual->clip.bottom; }
+    LengthBox clip() const { return visual->clip; }
     bool hasClip() const { return noninherited_flags.f._hasClip; }
 
     EUnicodeBidi unicodeBidi() const { return noninherited_flags.f._unicodeBidi; }
@@ -1120,6 +1205,7 @@ public:
     int marqueeLoopCount() { return css3NonInheritedData->marquee->loops; }
     EMarqueeBehavior marqueeBehavior() { return css3NonInheritedData->marquee->behavior; }
     EMarqueeDirection marqueeDirection() { return css3NonInheritedData->marquee->direction; }
+    bool textOverflow() const { return noninherited_flags.f._textOverflow; }
     // End CSS3 Getters
 
 // attribute setter methods
@@ -1170,7 +1256,8 @@ public:
     }
     void setOutlineColor(const QColor & v) {  SET_VAR(background,m_outline.color,v) }
 
-    void setOverflow(EOverflow v) {  noninherited_flags.f._overflow = v; }
+    void setOverflowX(EOverflow v) {  noninherited_flags.f._overflowX = v; }
+    void setOverflowY(EOverflow v) {  noninherited_flags.f._overflowY = v; }
     void setVisibility(EVisibility v) { inherited_flags.f._visibility = v; }
     void setVerticalAlign(EVerticalAlign v) { noninherited_flags.f._vertical_align = v; }
     void setVerticalAlignLength(Length l) { SET_VAR(box, vertical_align, l ) }
@@ -1270,6 +1357,7 @@ public:
     void setMarqueeDirection(EMarqueeDirection d) { SET_VAR(css3NonInheritedData.access()->marquee, direction, d); }
     void setMarqueeBehavior(EMarqueeBehavior b) { SET_VAR(css3NonInheritedData.access()->marquee, behavior, b); }
     void setMarqueeLoopCount(int i) { SET_VAR(css3NonInheritedData.access()->marquee, loops, i); }
+    void setTextOverflow(bool b) { noninherited_flags.f._textOverflow = b; }
     // End CSS3 Setters
 
     QPalette palette() const { return visual->palette; }
@@ -1279,9 +1367,10 @@ public:
         const_cast<StyleVisualData *>(visual.get())->palette = QApplication::palette();
     }
 
+    bool useNormalContent() const { return generated->content == 0; }
     ContentData* contentData() const { return generated->content; }
-    bool contentDataEquivalent(const RenderStyle* otherStyle) const 
-    { 
+    bool contentDataEquivalent(const RenderStyle* otherStyle) const
+    {
         return generated->contentDataEquivalent(otherStyle->generated.get());
     }
     void addContent(DOM::DOMStringImpl* s);
@@ -1290,6 +1379,7 @@ public:
     void addContent(EQuoteContent q);
     void setContentNone();
     void setContentNormal();
+    void setContentData(ContentData* content);
 
     DOM::CSSValueListImpl* counterReset() const { return generated->counter_reset; }
     DOM::CSSValueListImpl* counterIncrement() const { return generated->counter_increment; }
@@ -1300,7 +1390,6 @@ public:
     short counterReset(const DOM::DOMString& c) const;
     short counterIncrement(const DOM::DOMString& c) const;
 
-
     bool inheritedNotEqual( RenderStyle *other ) const;
 
     enum Diff { Equal, NonVisible = Equal, Visible, Position, Layout, CbLayout };
@@ -1317,6 +1406,8 @@ public:
                /*originalDisplay() == INLINE_BOX ||*/ originalDisplay() == INLINE_TABLE;
     }
 
+    bool inheritedNoninherited() const { return noninherited_flags.f._inherited_noninherited; }
+    void setInheritedNoninherited(bool b) { noninherited_flags.f._inherited_noninherited = b; }
 
 #ifdef ENABLE_DUMP
     QString createDiff( const RenderStyle &parent ) const;
@@ -1324,7 +1415,10 @@ public:
 
     // Initial values for all the properties
     static bool initialBackgroundAttachment() { return true; }
+    static EBackgroundBox initialBackgroundClip() { return BGBORDER; }
+    static EBackgroundBox initialBackgroundOrigin() { return BGPADDING; }
     static EBackgroundRepeat initialBackgroundRepeat() { return REPEAT; }
+    static LengthSize initialBackgroundSize() { return LengthSize(); }
     static bool initialBorderCollapse() { return false; }
     static EBorderStyle initialBorderStyle() { return BNONE; }
     static ECaptionSide initialCaptionSide() { return CAPTOP; }
@@ -1335,8 +1429,10 @@ public:
     static EFloat initialFloating() { return FNONE; }
     static EListStylePosition initialListStylePosition() { return OUTSIDE; }
     static EListStyleType initialListStyleType() { return LDISC; }
-    static EOverflow initialOverflow() { return OVISIBLE; }
+    static EOverflow initialOverflowX() { return OVISIBLE; }
+    static EOverflow initialOverflowY() { return OVISIBLE; }
     static EPageBreak initialPageBreak() { return PBAUTO; }
+    static bool initialPageBreakInside() { return true; }
     static EPosition initialPosition() { return STATIC; }
     static ETableLayout initialTableLayout() { return TAUTO; }
     static EUnicodeBidi initialUnicodeBidi() { return UBNormal; }
@@ -1376,6 +1472,7 @@ public:
     static Length initialMarqueeIncrement() { return Length(6, Fixed); }
     static EMarqueeBehavior initialMarqueeBehavior() { return MSCROLL; }
     static EMarqueeDirection initialMarqueeDirection() { return MAUTO; }
+    static bool initialTextOverflow() { return false; }
 };
 
 class RenderPageStyle {
