@@ -96,6 +96,102 @@ void InlineTextBox::operator delete(void* ptr, size_t sz)
     *(size_t *)ptr = sz;
 }
 
+void InlineTextBox::selectionStartEnd(int& sPos, int& ePos)
+{
+    int startPos, endPos;
+    if (object()->selectionState() == RenderObject::SelectionInside) {
+        startPos = 0;
+        endPos = renderText()->string()->l;
+    } else {
+        renderText()->selectionStartEnd(startPos, endPos);
+        if (object()->selectionState() == RenderObject::SelectionStart)
+            endPos = renderText()->string()->l;
+        else if (object()->selectionState() == RenderObject::SelectionEnd)
+            startPos = 0;
+    }
+
+    sPos = qMax(startPos - m_start, 0);
+    ePos = qMin(endPos - m_start, (int)m_len);
+}
+
+RenderObject::SelectionState InlineTextBox::selectionState()
+{
+    RenderObject::SelectionState state = object()->selectionState();
+    if (state == RenderObject::SelectionStart || state == RenderObject::SelectionEnd ||
+        state == RenderObject::SelectionBoth) {
+        int startPos, endPos;
+        renderText()->selectionStartEnd(startPos, endPos);
+        
+        bool start = (state != RenderObject::SelectionEnd && startPos >= m_start && startPos < m_start + m_len);
+        bool end = (state != RenderObject::SelectionStart && endPos > m_start && endPos <= m_start + m_len);
+        if (start && end)
+            state = RenderObject::SelectionBoth;
+        else if (start)
+            state = RenderObject::SelectionStart;
+        else if (end)
+            state = RenderObject::SelectionEnd;
+        else if ((state == RenderObject::SelectionEnd || startPos < m_start) &&
+                 (state == RenderObject::SelectionStart || endPos > m_start + m_len))
+            state = RenderObject::SelectionInside;
+    }
+    return state;
+}
+
+void InlineTextBox::paint(RenderObject::PaintInfo& i, int tx, int ty)
+{
+    if (object()->isBR() || object()->style()->visibility() != VISIBLE ||
+        m_truncation == cFullTruncation || i.phase == PaintActionOutline)
+        return;
+
+    if (i.phase == PaintActionSelection && object()->selectionState() == RenderObject::SelectionNone)
+        // When only painting the selection, don't bother to paint if there is none.
+        return;
+
+    int xPos = tx + m_x;
+    int w = width();
+    if ((xPos >= i.r.x() + i.r.width()) || (xPos + w <= i.r.x()))
+        return;
+
+    // Set our font.
+    RenderStyle* styleToUse = object()->style(m_firstLine);
+    int d = styleToUse->textDecorationsInEffect();
+    if (styleToUse->font() != i.p->font())
+        i.p->setFont(styleToUse->font());
+    const Font *font = &styleToUse->htmlFont();
+    bool haveSelection = selectionState() != RenderObject::SelectionNone;
+    
+    // Now calculate startPos and endPos, for painting selection.
+    // We paint selection while endPos > 0
+    int ePos = 0, sPos = 0;
+    if (haveSelection && !object()->canvas()->staticMode()) {
+        selectionStartEnd(sPos, ePos);
+    }
+    i.p->setPen(styleToUse->color());
+
+    if (m_len > 0 && i.phase != PaintActionSelection) {
+        int endPoint = m_len;
+        if (m_truncation != cNoTruncation)
+            endPoint = m_truncation - m_start;
+        if (styleToUse->textShadow())
+            paintShadow(i.p, font, tx, ty, styleToUse->textShadow());
+        if (!haveSelection || sPos != 0 || ePos != m_len) {
+            font->drawText(i.p, m_x + tx, m_y + ty + m_baseline, renderText()->string()->s, renderText()->string()->l, m_start, endPoint,
+                           m_toAdd, m_reversed ? Qt::RightToLeft : Qt::LeftToRight);
+        }
+    }
+
+    if (d != TDNONE && i.phase != PaintActionSelection && styleToUse->htmlHacks()) {
+        i.p->setPen(styleToUse->color());
+        paintDecoration(i.p, font, tx, ty, d);
+    }
+
+    if (haveSelection && i.phase == PaintActionSelection) {
+        //kdDebug(6040) << this << " paintSelection with startPos=" << sPos << " endPos=" << ePos << endl;
+        if ( sPos < ePos )
+	    paintSelection(font, renderText(), i.p, styleToUse, tx, ty, sPos, ePos, d);
+    }
+}
+
 /** returns the proper ::selection pseudo style for the given element
  * @return the style or 0 if no ::selection pseudo applies.
  */
@@ -133,11 +229,10 @@ void InlineTextBox::paintSelection(const Font *f, RenderText *text, QPainter *p,
 	// ### should be at most retrieved once per render text
 	QColor bg = khtml::retrieveBackgroundColor(text);
 	// It may happen that the contrast is -- well -- virtually non existent.
-	// In this case, simply invert the colors
-	if (!khtml::hasSufficientContrast(hbg, bg)) {
-	    hc = QColor(0xff-hc.red(),0xff-hc.green(),0xff-hc.blue());
-	    hbg = QColor(0xff-hbg.red(),0xff-hbg.green(),0xff-hbg.blue());
-	}/*end if*/
+	// In this case, simply swap the colors, thus in compliance with
+	// NN4 (win32 only), IE, and Mozilla.
+	if (!khtml::hasSufficientContrast(hbg, bg))
+	    qSwap(hc, hbg);
     }
 
     p->setPen(hc);
@@ -177,7 +272,13 @@ void InlineTextBox::paintDecoration( QPainter *pt, const Font *f, int _tx, int _
     _tx += m_x;
     _ty += m_y;
 
+    if (m_truncation == cFullTruncation)
+        return;
+
     int width = m_width - 1;
+    if (m_truncation != cNoTruncation) {
+        width = static_cast<RenderText*>(m_object)->width(m_start, m_truncation - m_start, m_firstLine);
+    }
 
     RenderObject *p = object();
 
@@ -483,6 +584,50 @@ long InlineTextBox::maxOffset() const
   return m_start + m_len;
 }
 
+int InlineTextBox::placeEllipsisBox(bool ltr, int blockEdge, int ellipsisWidth, bool& foundBox)
+{
+    if (foundBox) {
+        m_truncation = cFullTruncation;
+        return -1;
+    }
+
+    int ellipsisX = ltr ? blockEdge - ellipsisWidth : blockEdge + ellipsisWidth;
+
+    // For LTR, if the left edge of the ellipsis is to the left of our text run, then we are the run that will get truncated.
+    if (ltr) {
+        if (ellipsisX <= m_x) {
+            // Too far.  Just set full truncation, but return -1 and let the ellipsis just be placed at the edge of the box.
+            m_truncation = cFullTruncation;
+            foundBox = true;
+            return -1;
+        }
+
+        if (ellipsisX < m_x + m_width) {
+            if (m_reversed)
+                return -1; // FIXME: Support LTR truncation when the last run is RTL someday.
+
+            foundBox = true;
+
+            int ax;
+            int offset = offsetForPoint(ellipsisX, ax) - 1;
+            if (offset <= m_start) {
+                // No characters should be rendered.  Set ourselves to full truncation and place the ellipsis at the min of our start
+                // and the ellipsis edge.
+                m_truncation = cFullTruncation;
+                return qMin(ellipsisX, (int)m_x);
+            }
+
+            // Set the truncation index on the text run.  The ellipsis needs to be placed just after the last visible character.
+            m_truncation = offset;
+            return widthFromStart(offset - m_start);
+        }
+    }
+    else {
+        // FIXME: Support RTL truncation someday, including both modes (when the leftmost run on the line is either RTL or LTR)
+    }
+    return -1;
+}
+
 // -----------------------------------------------------------------------------
 
 // remove this once QVector::bsearch is fixed
@@ -555,7 +700,7 @@ void RenderText::setStyle(RenderStyle *_style)
         RenderObject::setStyle( _style );
         m_lineHeight = RenderObject::lineHeight(false);
 
-        if (changedText) {
+        if (!isBR() && changedText) {
             DOM::DOMStringImpl* textToTransform = originalString();
             if (textToTransform)
                 setText(textToTransform, true);
@@ -825,17 +970,9 @@ long RenderText::maxOffset() const
   return m_lines[count - 1]->m_start + m_lines[count - 1]->m_len;
 }
 
-bool RenderText::absolutePosition(int &xPos, int &yPos, bool)
+bool RenderText::absolutePosition(int &xPos, int &yPos, bool) const
 {
     return RenderObject::absolutePosition(xPos, yPos, false);
-
-    if(parent() && parent()->absolutePosition(xPos, yPos, false)) {
-        xPos -= paddingLeft() + borderLeft();
-        yPos -= borderTop() + paddingTop();
-        return true;
-    }
-    xPos = yPos = 0;
-    return false;
 }
 
 bool RenderText::posOfChar(int chr, int &x, int &y)
@@ -857,127 +994,9 @@ bool RenderText::posOfChar(int chr, int &x, int &y)
     return false;
 }
 
-void RenderText::paint( PaintInfo& pI, int tx, int ty)
+void RenderText::paint( PaintInfo& /*pI*/, int /*tx*/, int /*ty*/)
 {
-    if (pI.phase == PaintActionSelection && selectionState() == SelectionNone)
-        // When only painting the selection, don't bother to paint if there is none.
-        return;
-    if (pI.phase != PaintActionForeground && pI.phase != PaintActionSelection
-        || style()->visibility() != VISIBLE)
-        return;
-
-    int s = m_lines.count() - 1;
-    if ( s < 0 ) return;
-
-    if ( ty + m_lines[0]->m_y > pI.r.bottom() ) return;
-    if ( ty + m_lines[s]->m_y + m_lines[s]->height() < pI.r.top() ) return;
-
-    int ow = style()->outlineWidth();
-    RenderStyle* pseudoStyle = hasFirstLine() ? style()->getPseudoStyle(RenderStyle::FIRST_LINE) : 0;
-    InlineTextBox f(0, pI.r.top()-ty);
-    int si = RenderText::findFirstMatching(&m_lines, &f);
-    // something matching found, find the first one to paint
-    bool isStatic = canvas()->staticMode();
-    if (isStatic && pI.phase == PaintActionSelection) return;
-
-    if(si >= 0)
-    {
-        // Move up until out of area to be painted
-        while(si > 0 && m_lines[si-1]->checkVerticalPoint(pI.r.y(), ty, pI.r.height(), m_lineHeight))
-            si--;
-
-        // Now calculate startPos and endPos, for painting selection.
-        // We paint selection while endPos > 0
-        int endPos = 0, startPos = 0;
-        if (!isStatic && (selectionState() != SelectionNone)) {
-            if (selectionState() == SelectionInside) {
-                //kDebug(6040) << this << " SelectionInside -> 0 to end" << endl;
-                startPos = 0;
-                endPos = str->l;
-            } else {
-                selectionStartEnd(startPos, endPos);
-                if(selectionState() == SelectionStart)
-                    endPos = str->l;
-                else if(selectionState() == SelectionEnd)
-                    startPos = 0;
-            }
-            //kDebug(6040) << this << " Selection from " << startPos << " to " << endPos << endl;
-        }
-
-        InlineTextBox* s;
-	const Font *font = &style()->htmlFont();
-
-        bool haveSelection = !isStatic && selectionState() != SelectionNone && startPos != endPos;
-        if (!haveSelection && pI.phase == PaintActionSelection)
-            // When only painting the selection, don't bother to paint if there
-            return;
-
-        // run until we find one that is outside the range, then we
-        // know we can stop
-        do {
-            s = m_lines[si];
-
-            RenderStyle* _style = pseudoStyle && s->m_firstLine ? pseudoStyle : style();
-            int d = _style->textDecorationsInEffect();
-
-            if(_style->font() != pI.p->font())
-                pI.p->setFont(_style->font());
-
-	    //has to be outside the above if because of small caps.
-	    font = &_style->htmlFont();
-
-            pI.p->setPen(_style->color());
-
-#ifdef APPLE_CHANGES
-            // Set a text shadow if we have one.
-            bool setShadow = false;
-            if (_style->textShadow()) {
-                p->setShadow(_style->textShadow()->x, _style->textShadow()->y,
-                             _style->textShadow()->blur, _style->textShadow()->color);
-                setShadow = true;
-            }
-#endif
-
-            const int offset = s->m_start;
-            const int sPos = qMax( startPos - offset, 0 );
-            const int ePos = qMin( endPos - offset, int( s->m_len ) );
-            if (s->m_len > 0 && pI.phase != PaintActionSelection) {
-                //kDebug( 6040 ) << "RenderObject::paintObject(" << QString::fromRawData(str->s + s->m_start, s->m_len) << ") at(" << s->m_x+tx << "/" << s->m_y+ty << ")" << endl;
-                if (_style->textShadow())
-                    s->paintShadow(pI.p, font, tx, ty, _style->textShadow());
-// kDebug(6040) << QString::fromRawData(str->s + s->m_start, s->m_len).left(40) << endl;
-                if (!haveSelection || sPos != 0 || ePos != s->m_len) {
-		    font->drawText(pI.p, s->m_x + tx, s->m_y + ty + s->m_baseline, str->s, str->l, s->m_start, s->m_len,
-				   s->m_toAdd, s->m_reversed ? Qt::RightToLeft : Qt::LeftToRight);
-	        }
-	    }
-
-            if (d != TDNONE && pI.phase == PaintActionForeground
-                && style()->htmlHacks()) {
-                pI.p->setPen(_style->color());
-                s->paintDecoration(pI.p, font, tx, ty, d);
-            }
-
-            if (haveSelection && pI.phase == PaintActionSelection) {
-                //kDebug(6040) << this << " paintSelection with startPos=" << sPos << " endPos=" << ePos << endl;
-		if ( sPos < ePos )
-		    s->paintSelection(font, this, pI.p, _style, tx, ty, sPos, ePos, d);
-
-            }
-#ifdef BIDI_DEBUG
-            {
-                int h = lineHeight( false ) + paddingTop() + paddingBottom() + borderTop() + borderBottom();
-                QColor c2 = QColor("#0000ff");
-                drawBorder(pI.p, tx + s->m_x, ty + s->m_y, tx + s->m_x + 1, ty + s->m_y + h,
-                              RenderObject::BSLeft, c2, c2, SOLID, 1, 1);
-                drawBorder(pI.p, tx + s->m_x + s->m_width, ty + s->m_y, tx + s->m_x + s->m_width + 1, ty + s->m_y + h,
-                              RenderObject::BSRight, c2, c2, SOLID, 1, 1);
-            }
-#endif
-
-        } while (++si < (int)m_lines.count() && m_lines[si]->checkVerticalPoint(pI.r.y()-ow, ty, pI.r.height(), m_lineHeight));
-
-    }
+    KHTMLAssert( false );
 }
 
 void RenderText::calcMinMaxWidth()
@@ -1139,12 +1158,43 @@ void RenderText::setText(DOMStringImpl *text, bool force)
     if ( str && style() ) {
         oldstr = str;
         switch(style()->textTransform()) {
-	case CAPITALIZE:   str = str->capitalize();  break;
-	case UPPERCASE:   str = str->upper();       break;
-	case LOWERCASE:  str = str->lower();       break;
-	case NONE:
-	default:;
-	}
+        case CAPITALIZE:
+        {
+            RenderObject *o;
+            bool runOnString = false;
+
+            // find previous non-empty text renderer if one exists
+            for (o = previousRenderer(); o; o = o->previousRenderer()) {
+                if (!o->isInlineFlow()) {
+                    if (!o->isText())
+                        break;
+
+                    DOMStringImpl *prevStr = static_cast<RenderText*>(o)->string();
+                    // !prevStr can happen with css like "content:open-quote;"
+                    if (!prevStr)
+                        break;
+
+                    if (prevStr->length() == 0)
+                        continue;
+                    QChar c = (*prevStr)[prevStr->length() - 1];
+                    if (!c.isSpace())
+                        runOnString = true;
+
+                    break;
+                }
+            }
+
+            str = str->capitalize(runOnString);
+        }
+        break;
+
+		
+		
+        case UPPERCASE: str = str->upper();       break;
+        case LOWERCASE: str = str->lower();       break;
+        case NONE:
+        default:;
+    }
         str->ref();
         oldstr->deref();
     }
@@ -1240,7 +1290,7 @@ unsigned int RenderText::width(unsigned int from, unsigned int len, const Font *
 
     if ( f == &style()->htmlFont() && from == 0 && len == str->l )
  	 return m_maxWidth;
-
+    
     int w = f->width(str->s, str->l, from, len );
 
     //kDebug( 6040 ) << "RenderText::width(" << from << ", " << len << ") = " << w << endl;
@@ -1267,11 +1317,11 @@ short RenderText::width() const
     return w;
 }
 
-void RenderText::repaint(bool immediate)
+void RenderText::repaint(Priority p)
 {
     RenderObject *cb = containingBlock();
     if(cb)
-        cb->repaint(immediate);
+        cb->repaint(p);
 }
 
 bool RenderText::isFixedWidthFont() const
