@@ -32,10 +32,10 @@
 
 #include <kapplication.h>
 #include <kacceleratormanager.h>
+#include <kstandarddirs.h>
 #include <qimage.h>
 #include <qfile.h>
 #include "test_regression.h"
-#include <unistd.h>
 #include <stdio.h>
 
 #include "css/cssstyleselector.h"
@@ -43,6 +43,7 @@
 #include "rendering/render_style.h"
 #include "rendering/render_layer.h"
 #include "khtmldefaults.h"
+#include <QProcess>
 #include <QWindowsStyle>
 #include <QStyleOptionSlider>
 
@@ -186,7 +187,7 @@ public:
         switch (type)
         {
         case CT_PushButton:
-            return QSize(size.width(), size.height() - 1); 
+            return QSize(size.width(), size.height() - 1);
         case CT_LineEdit:
 	    if (widget && widget->parentWidget() && !qstricmp(widget->parentWidget()->className(), "KUrlRequester"))
 		return QSize(size.width() + 1, size.height());
@@ -231,7 +232,6 @@ public:
 #include <qfileinfo.h>
 #include <qtimer.h>
 #include <kstatusbar.h>
-#include <qfileinfo.h>
 
 #include "misc/decoder.h"
 #include "dom/dom2_range.h"
@@ -251,15 +251,14 @@ public:
 #include "ecma/kjs_binding.h"
 #include "ecma/kjs_dom.h"
 #include "ecma/kjs_window.h"
-#include "ecma/kjs_binding.h"
 #include "ecma/kjs_proxy.h"
 
 using namespace khtml;
 using namespace DOM;
 using namespace KJS;
 
-bool visual = false;
-pid_t xvfb;
+static bool visual = false;
+static pid_t xvfb;
 
 // -------------------------------------------------------------------------
 
@@ -329,7 +328,7 @@ void PartMonitor::partCompleted()
     disconnect(m_part,SIGNAL(completed()),this,SLOT(partCompleted()));
 }
 
-void signal_handler( int )
+static void signal_handler( int )
 {
     printf( "timeout\n" );
     abort();
@@ -388,7 +387,6 @@ ValueImp* RegTestFunction::callAsFunction(ExecState *exec, ObjectImp* /*thisObj*
             if ( docimpl && docimpl->view() && docimpl->renderer() )
             {
                 docimpl->updateRendering();
-                docimpl->view()->layout();
             }
             QString filename = args[0]->toString(exec).qstring();
             filename = RegressionTest::curr->m_currentCategory+"/"+filename;
@@ -490,9 +488,9 @@ ValueImp* KHTMLPartFunction::callAsFunction(ExecState *exec, ObjectImp*/*thisObj
             url.setProtocol("file");
             url.setPath(fullFilename);
             PartMonitor pm(m_part);
-            m_part->openURL(url);
+            m_part->openUrl(url);
             pm.waitForCompletion();
-	    kapp->processEvents(QEventLoop::AllEvents, 60000);
+	    kapp->processEvents(QEventLoop::AllEvents);
             break;
         }
 	case OpenPageAsUrl: {
@@ -529,7 +527,7 @@ ValueImp* KHTMLPartFunction::callAsFunction(ExecState *exec, ObjectImp*/*thisObj
 		m_part->end();
 		pm.waitForCompletion();
 	    }
-	    kapp->processEvents(QEventLoop::AllEvents, 60000);
+	    kapp->processEvents(QEventLoop::AllEvents);
 	    break;
 	}
 	case Begin: {
@@ -544,7 +542,7 @@ ValueImp* KHTMLPartFunction::callAsFunction(ExecState *exec, ObjectImp*/*thisObj
         }
         case End: {
             m_part->end();
-	    kapp->processEvents(QEventLoop::AllEvents, 60000);
+	    kapp->processEvents(QEventLoop::AllEvents);
             break;
         }
 	case ExecuteScript: {
@@ -554,11 +552,11 @@ ValueImp* KHTMLPartFunction::callAsFunction(ExecState *exec, ObjectImp*/*thisObj
 	    proxy->evaluate("",0,code,0,&comp);
 	    if (comp.complType() == Throw)
 		exec->setException(comp.value());
-	    kapp->processEvents(QEventLoop::AllEvents, 60000);
+	    kapp->processEvents(QEventLoop::AllEvents);
 	    break;
 	}
 	case ProcessEvents: {
-	    kapp->processEvents(QEventLoop::AllEvents, 60000);
+	    kapp->processEvents(QEventLoop::AllEvents);
 	    break;
 	}
     }
@@ -605,8 +603,16 @@ int main(int argc, char *argv[])
     setenv( "LC_ALL", "C", 1 );
     setenv( "LANG", "C", 1 );
 
+    // We want KIO to be in the slave-forking mode since
+    // then it'll ask KProtocolInfo::exec for the binary to run,
+    // and we intercept that, limiting the I/O to file://
+    // and the magic data://. See Slave::createSlave in KIO's slave.cpp
+    setenv( "KDE_FORK_SLAVES", "true", 1);
     signal( SIGALRM, signal_handler );
 
+    // workaround various Qt crashes by always enforcing a TrueColor visual
+    QApplication::setColorSpec( QApplication::ManyColor );
+
     KCmdLineArgs::init(argc, argv, "testregression", "TestRegression",
                        "Regression tester for khtml", "1.0");
     KCmdLineArgs::addCmdLineOptions(options);
@@ -637,16 +643,33 @@ int main(int argc, char *argv[])
 
     if (args->isSet("xvfb"))
     {
-    if ( ::access( "/usr/X11R6/bin/Xvfb", X_OK ) ) {
-        fprintf( stderr, "ERROR: We need /usr/X11R6/bin/Xvfb to be installed for reliable results\n" );
-        exit( 1 );
-    }
+        QString xvfbPath = KStandardDirs::findExe("Xvfb");
+        if ( xvfbPath.isEmpty() ) {
+            fprintf( stderr, "ERROR: We need Xvfb to be installed for reliable results\n" );
+            exit( 1 );
+        }
+
+        QByteArray xvfbPath8 = QFile::encodeName(xvfbPath);
+        QStringList fpaths;
+        fpaths.append(baseDir+"/resources");
+
+        const char* const fontdirs[] = { "75dpi", "misc", "Type1" };
+        const char* const fontpaths[] =  {"/usr/share/fonts/", "/usr/X11/lib/X11/fonts/",
+            "/usr/lib/X11/fonts/", "/usr/share/fonts/X11/" };
+
+        for (size_t fp=0; fp < sizeof(fontpaths)/sizeof(*fontpaths); ++fp)
+            for (size_t fd=0; fd < sizeof(fontdirs)/sizeof(*fontdirs); ++fd)
+                if (QFile::exists(QLatin1String(fontpaths[fp])+QLatin1String(fontdirs[fd])))
+                    if (strcmp(fontdirs[fd] , "Type1"))
+                        fpaths.append(QLatin1String(fontpaths[fp])+QLatin1String(fontdirs[fd])+":unscaled");
+                    else
+                        fpaths.append(QLatin1String(fontpaths[fp])+QLatin1String(fontdirs[fd]));
 
         xvfb = fork();
         if ( !xvfb ) {
-            char buffer[1000];
-            sprintf( buffer, "%s/resources,/usr/X11R6/lib/X11/fonts/75dpi:unscaled,/usr/X11R6/lib/X11/fonts/misc:unscaled,/usr/X11R6/lib/X11/fonts/Type1,/usr/share/fonts/X11/misc,/usr/share/fonts/X11/75dpi:unscaled,/usr/share/fonts/X11/Type1", (const char *)baseDir );
-            execl( "/usr/X11R6/bin/Xvfb", "/usr/X11R6/bin/Xvfb", "-once", "-dpi", "100", "-screen", "0", "1024x768x16", "-ac", "-fp", buffer, ":47", (char*)NULL );
+            QByteArray buffer = fpaths.join(",").toLatin1();
+            execl( xvfbPath8.data(), xvfbPath8.data(), "-once", "-dpi", "100", "-screen", "0",
+                    "1024x768x16", "-ac", "-fp", buffer.data(), ":47", (char*)NULL );
         }
 
         setenv( "DISPLAY", ":47", 1 );
@@ -658,10 +681,11 @@ int main(int argc, char *argv[])
     KSimpleConfig sc1( "cryptodefaults" );
     sc1.setGroup( "Warnings" );
     sc1.writeEntry( "OnUnencrypted",  false );
-    KGlobal::instance()->config()->setGroup( "Notification Messages" );
-    KGlobal::instance()->config()->writeEntry( "kjscupguard_alarmhandler", true );
-    KGlobal::instance()->config()->setGroup("HTML Settings");
-    KGlobal::instance()->config()->writeEntry("ReportJSErrors", false);
+    KSharedConfigPtr config = KGlobal::mainComponent().config();
+    config->setGroup( "Notification Messages" );
+    config->writeEntry( "kjscupguard_alarmhandler", true );
+    config->setGroup("HTML Settings");
+    config->writeEntry("ReportJSErrors", false);
     KConfig cfg( "khtmlrc" );
     cfg.setGroup("HTML Settings");
     cfg.writeEntry( "StandardFont", HTML_DEFAULT_VIEW_SANSSERIF_FONT );
@@ -678,7 +702,7 @@ int main(int argc, char *argv[])
     cfg.writeEntry( "WindowOpenPolicy", (int)KHTMLSettings::KJSWindowOpenAllow);
 
     cfg.sync();
-    
+
     KJS::ScriptInterpreter::turnOffCPUGuard();
 
     QPalette pal = a.palette();
@@ -718,7 +742,8 @@ int main(int argc, char *argv[])
     part->setJScriptEnabled(true);
 
     part->executeScript(DOM::Node(), ""); // force the part to create an interpreter
-//    part->setJavaEnabled(true);
+    part->setJavaEnabled(false);
+    part->setPluginsEnabled(false);
 
     if (args->isSet("show"))
 	visual = true;
@@ -746,7 +771,7 @@ int main(int argc, char *argv[])
                                                         args->isSet("genoutput"),
                                                         !args->isSet( "html" ),
                                                         !args->isSet( "js" ));
-    QObject::connect(part->browserExtension(), SIGNAL(openURLRequest(const KUrl &, const KParts::URLArgs &)),
+    QObject::connect(part->browserExtension(), SIGNAL(openUrlRequest(const KUrl &, const KParts::URLArgs &)),
 		     regressionTest, SLOT(slotOpenURL(const KUrl&, const KParts::URLArgs &)));
     QObject::connect(part->browserExtension(), SIGNAL(resizeTopLevelWidget( int, int )),
 		     regressionTest, SLOT(resizeTopLevelWidget( int, int )));
@@ -1272,6 +1297,17 @@ static QString makeRelativePath(const QString &base, const QString &path)
     return rel;
 }
 
+static QString getDiff(QString cmdLine)
+{
+    QProcess p;
+    p.start(cmdLine, QIODevice::ReadOnly);
+    p.waitForFinished();
+    QString text = QString::fromLocal8Bit(p.readAllStandardOutput());
+    text = text.replace( '<', "&lt;" );
+    text = text.replace( '>', "&gt;" );
+    return text;
+}
+
 void RegressionTest::doFailureReport( const QString& test, int failures )
 {
     if ( failures == NoFailure ) {
@@ -1301,33 +1337,15 @@ void RegressionTest::doFailureReport( const QString& test, int failures )
 
     if ( failures & RenderFailure ) {
         renderDiff += "<pre>";
-        FILE *pipe = popen( QString::fromLatin1( "diff -u baseline/%1-render %3/%2-render" )
-                            .arg ( test, test, relOutputDir ).toLatin1(), "r" );
-        QTextIStream *is = new QTextIStream( pipe );
-        for ( int line = 0; line < 100 && !is->atEnd(); ++line ) {
-            QString line = is->readLine();
-            line = line.replace( '<', "&lt;" );
-            line = line.replace( '>', "&gt;" );
-            renderDiff += line + "\n";
-        }
-        delete is;
-        pclose( pipe );
+        renderDiff += getDiff( QString::fromLatin1( "diff -u baseline/%1-render %3/%2-render" )
+                            .arg ( test, test, relOutputDir ) );
         renderDiff += "</pre>";
     }
 
     if ( failures & DomFailure ) {
         domDiff += "<pre>";
-        FILE *pipe = popen( QString::fromLatin1( "diff -u baseline/%1-dom %3/%2-dom" )
-                            .arg ( test, test, relOutputDir ).toLatin1(), "r" );
-        QTextIStream *is = new QTextIStream( pipe );
-        for ( int line = 0; line < 100 && !is->atEnd(); ++line ) {
-            QString line = is->readLine();
-            line = line.replace( '<', "&lt;" );
-            line = line.replace( '>', "&gt;" );
-            domDiff += line  + "\n";
-        }
-        delete is;
-        pclose( pipe );
+        domDiff += getDiff( QString::fromLatin1( "diff -u baseline/%1-dom %3/%2-dom" )
+                            .arg ( test, test, relOutputDir ) );
         domDiff += "</pre>";
     }
 
@@ -1424,15 +1442,15 @@ void RegressionTest::testStaticFile(const QString & filename)
     if (filename.endsWith(".html") || filename.endsWith(".htm")) args.serviceType = "text/html";
     else if (filename.endsWith(".xhtml")) args.serviceType = "application/xhtml+xml";
     else if (filename.endsWith(".xml")) args.serviceType = "text/xml";
-    m_part->browserExtension()->setURLArgs(args);
+    m_part->browserExtension()->setUrlArgs(args);
     // load page
     KUrl url;
     url.setProtocol("file");
     url.setPath(QFileInfo(m_baseDir + "/tests/"+filename).absoluteFilePath());
     PartMonitor pm(m_part);
-    m_part->openURL(url);
+    m_part->openUrl(url);
     pm.waitForCompletion();
-    m_part->closeURL();
+    m_part->closeUrl();
 
     if ( filename.startsWith( "domts/" ) ) {
         QString functionname;
@@ -1778,10 +1796,10 @@ void RegressionTest::createMissingDirs(const QString & filename)
 
 void RegressionTest::slotOpenURL(const KUrl &url, const KParts::URLArgs &args)
 {
-    m_part->browserExtension()->setURLArgs( args );
+    m_part->browserExtension()->setUrlArgs( args );
 
     PartMonitor pm(m_part);
-    m_part->openURL(url);
+    m_part->openUrl(url);
     pm.waitForCompletion();
 }
 
