@@ -417,7 +417,7 @@ void TestRegressionWindow::directoryListingFinished(KJob *)
 			// Detect whether the tests has no corresponding baseline items...
 			if(!ignore && !failure)
 			{
-				if(!dom && !dump && !render && !cacheName.endsWith(".js") && !cacheName.startsWith("domts"))
+				if(!dom[0] && !dump && !render && !cacheName.endsWith(".js") && !cacheName.startsWith("domts"))
 				{
 					// See if parent directory is completely ignored...
 					if(!ignoreParent)
@@ -703,7 +703,7 @@ void TestRegressionWindow::loadOutputHTML() const
 	QFileInfo indexHtml(fileName);
 	if(indexHtml.exists())
 	{
-		m_browserPart->openURL(KUrl::fromPath(fileName));
+		m_browserPart->openUrl(KUrl::fromPath(fileName));
 		m_ui.tabWidget->setTabEnabled(1, true);
 	}
 	else
