@@ -238,7 +238,9 @@ DOMImplementationImpl *DOMImplementationImpl::instance()
 // ------------------------------------------------------------------------
 
 ElementMappingCache::ElementMappingCache():m_dict(257)
-{}
+{
+    m_dict.setAutoDelete(true);
+}
 
 void ElementMappingCache::add(const QString& id, ElementImpl* nd)
 {
@@ -302,10 +304,12 @@ Q3PtrList<DocumentImpl> * DocumentImpl::changedDocuments;
 
 // KHTMLView might be 0
 DocumentImpl::DocumentImpl(DOMImplementationImpl *_implementation, KHTMLView *v)
-    : NodeBaseImpl( new DocumentPtr() ), m_domtree_version(0), m_counterDict(257),
+    : NodeBaseImpl( 0 ), m_domtree_version(0), m_counterDict(257),
       m_imageLoadEventTimer(0)
 {
-    document->doc = this;
+    m_document.resetSkippingRef(this); //Make getDocument return us..
+    m_selfOnlyRefCount = 0;
+
     m_paintDeviceMetrics = 0;
     m_paintDevice = 0;
     //m_decoderMibEnum = 0;
@@ -331,7 +335,7 @@ DocumentImpl::DocumentImpl(DOMImplementationImpl *_implementation, KHTMLView *v)
 
     // ### this should be created during parsing a <!DOCTYPE>
     // not during construction. Not sure who added that and why (Dirk)
-    m_doctype = new DocumentTypeImpl(_implementation, document,
+    m_doctype = new DocumentTypeImpl(_implementation, getDocument(),
                                      DOMString() /* qualifiedName */,
                                      DOMString() /* publicId */,
                                      DOMString() /* systemId */);
@@ -377,8 +381,60 @@ DocumentImpl::DocumentImpl(DOMImplementationImpl *_implementation, KHTMLView *v)
     m_dynamicDomRestyler = new khtml::DynamicDomRestyler();
 }
 
+void DocumentImpl::removedLastRef()
+{
+    if (m_selfOnlyRefCount) {
+        /* In this case, the only references to us are from children,
+           so we have a cycle. We'll try to break it by disconnecting the
+           children from us; this sucks/is wrong, but it's pretty much 
+           the best we can do without tracing. 
+
+           Of course, if dumping the children causes the refcount from them to 
+           drop to 0 we can get killed right here, so better hold
+           a temporary reference, too
+        */
+        DocPtr<DocumentImpl> guard(this);
+
+        // we must make sure not to be retaining any of our children through
+        // these extra pointers or we will create a reference cycle
+        if (m_doctype) {
+            m_doctype->deref(); 
+            m_doctype = 0;
+        }
+        
+        if (m_cssTarget) {
+            m_cssTarget->deref();
+            m_cssTarget = 0;
+        }
+
+        if (m_focusNode) {
+            m_focusNode->deref();
+            m_focusNode = 0;
+        }
+
+        if (m_hoverNode) {
+            m_hoverNode->deref();
+            m_hoverNode = 0;
+        }
+        
+        if (m_activeNode) {
+            m_activeNode->deref();
+            m_activeNode = 0;
+        }
+
+        removeChildren();
+
+        delete m_tokenizer;
+        m_tokenizer = 0;
+    } else {
+        delete this;
+    }
+}
+
 DocumentImpl::~DocumentImpl()
 {
+    //Important: if you need to remove stuff here,
+    //you may also have to fix removedLastRef() above - M.O.
     assert( !m_render );
 
     Q3IntDictIterator<NodeListImpl::Cache> it(m_nodeListCache);
@@ -390,7 +446,7 @@ DocumentImpl::~DocumentImpl()
     if (changedDocuments && m_docChanged)
         changedDocuments->remove(this);
     delete m_tokenizer;
-    document->doc = 0;
+    m_document.resetSkippingRef(0);
     delete m_styleSelector;
     delete m_docLoader;
     if (m_elemSheet )  m_elemSheet->deref();
@@ -412,6 +468,8 @@ DocumentImpl::~DocumentImpl()
         m_focusNode->deref();
     if ( m_hoverNode )
         m_hoverNode->deref();
+    if (m_activeNode)
+        m_activeNode->deref();
 
     m_renderArena.reset();
 
@@ -445,7 +503,7 @@ ElementImpl *DocumentImpl::createElement( const DOMString &name, int* pException
     if ( pExceptioncode && *pExceptioncode )
         return 0;
 
-    XMLElementImpl* e = new XMLElementImpl( document, id );
+    XMLElementImpl* e = new XMLElementImpl( getDocument(), id );
     e->setHTMLCompat( htmlMode() != XHtml ); // Not a real HTML element, but inside an html-compat doc all tags are uppercase.
     return e;
 }
@@ -456,7 +514,7 @@ AttrImpl *DocumentImpl::createAttribute( const DOMString &tagName, int* pExcepti
                   false /* allocate */, isHTMLDocument(), pExceptioncode);
     if ( pExceptioncode && *pExceptioncode )
         return 0;
-    AttrImpl* attr = new AttrImpl( 0, document, id, DOMString("").implementation());
+    AttrImpl* attr = new AttrImpl( 0, getDocument(), id, DOMString("").implementation());
     attr->setHTMLCompat( htmlMode() != XHtml );
     return attr;
 }
@@ -557,8 +615,12 @@ NodeImpl *DocumentImpl::importNode(NodeImpl *importedNode, bool deep, int &excep
 	result = createComment(static_cast<CommentImpl*>(importedNode)->string());
 	deep = false;
     }
+    else if (importedNode->nodeType() == Node::DOCUMENT_FRAGMENT_NODE)
+	result = createDocumentFragment();
     else
 	exceptioncode = DOMException::NOT_SUPPORTED_ERR;
+	
+    //### FIXME: This should handle Attributes, and a few other things
 
     if(deep && result)
     {
@@ -599,7 +661,7 @@ ElementImpl *DocumentImpl::createElementNS( const DOMString &_namespaceURI, cons
     if (!e) {
         Id id = getId(NodeImpl::ElementId, _namespaceURI.implementation(), prefix.implementation(),
                       localName.implementation(), false, false /*HTML already looked up*/);
-        e = new XMLElementImpl( document, id, prefix.implementation() );
+        e = new XMLElementImpl( getDocument(), id, prefix.implementation() );
     }
 
     return e;
@@ -618,7 +680,7 @@ AttrImpl *DocumentImpl::createAttributeNS( const DOMString &_namespaceURI,
     splitPrefixLocalName(_qualifiedName.implementation(), prefix, localName, colonPos);
     Id id = getId(NodeImpl::AttributeId, _namespaceURI.implementation(), prefix.implementation(),
                   localName.implementation(), false, true /*lookupHTML*/);
-    AttrImpl* attr = new AttrImpl(0, document, id, DOMString("").implementation(),
+    AttrImpl* attr = new AttrImpl(0, getDocument(), id, DOMString("").implementation(),
                          prefix.implementation());
     attr->setHTMLCompat( _namespaceURI.isNull() && htmlMode() != XHtml );
     return attr;
@@ -838,6 +900,8 @@ ElementImpl *DocumentImpl::createHTMLElement( const DOMString &name )
     case ID_HR:
         n = new HTMLHRElementImpl(docPtr());
         break;
+    case ID_PLAINTEXT:
+    case ID_XMP:
     case ID_PRE:
         n = new HTMLPreElementImpl(docPtr(), id);
         break;
@@ -961,16 +1025,14 @@ ElementImpl *DocumentImpl::createHTMLElement( const DOMString &name )
     case ID_SPAN:
     case ID_NOBR:
     case ID_WBR:
+    case ID_BDO:
+    case ID_NOFRAMES:
         n = new HTMLGenericElementImpl(docPtr(), id);
         break;
 
     case ID_MARQUEE:
         n = new HTMLMarqueeElementImpl(docPtr());
         break;
-
-    case ID_BDO:
-        break;
-
 // text
     case ID_TEXT:
         kDebug( 6020 ) << "Use document->createTextNode()" << endl;
@@ -1716,42 +1778,34 @@ bool DocumentImpl::prepareMouseEvent( bool readonly, int _x, int _y, MouseEvent
     return false;
 }
 
+
 // DOM Section 1.1.1
-bool DocumentImpl::childAllowed( NodeImpl *newChild )
-{
-    // Documents may contain a maximum of one Element child
-    if (newChild->nodeType() == Node::ELEMENT_NODE) {
-        NodeImpl *c;
-        for (c = firstChild(); c; c = c->nextSibling()) {
-            if (c->nodeType() == Node::ELEMENT_NODE)
-                return false;
-        }
-    }
-
-    // Documents may contain a maximum of one DocumentType child
-    if (newChild->nodeType() == Node::DOCUMENT_TYPE_NODE) {
-        NodeImpl *c;
-        for (c = firstChild(); c; c = c->nextSibling()) {
-            if (c->nodeType() == Node::DOCUMENT_TYPE_NODE)
-                return false;
-        }
-    }
-
-    return childTypeAllowed(newChild->nodeType());
-}
-
 bool DocumentImpl::childTypeAllowed( unsigned short type )
 {
     switch (type) {
-        case Node::ELEMENT_NODE:
-        case Node::PROCESSING_INSTRUCTION_NODE:
-        case Node::COMMENT_NODE:
-        case Node::DOCUMENT_TYPE_NODE:
-            return true;
-            break;
-        default:
+        case Node::ATTRIBUTE_NODE:
+        case Node::CDATA_SECTION_NODE:
+        case Node::DOCUMENT_FRAGMENT_NODE:
+        case Node::DOCUMENT_NODE:
+        case Node::ENTITY_NODE:
+        case Node::ENTITY_REFERENCE_NODE:
+        case Node::NOTATION_NODE:
+        case Node::TEXT_NODE:
+//        case Node::XPATH_NAMESPACE_NODE:
             return false;
+        case Node::COMMENT_NODE:
+        case Node::PROCESSING_INSTRUCTION_NODE:
+            return true;
+        case Node::DOCUMENT_TYPE_NODE:
+        case Node::ELEMENT_NODE:
+            // Documents may contain no more than one of each of these.
+            // (One Element and one DocumentType.)
+            for (NodeImpl* c = firstChild(); c; c = c->nextSibling())
+                if (c->nodeType() == type)
+                    return false;
+            return true;
     }
+    return false;
 }
 
 NodeImpl *DocumentImpl::cloneNode ( bool /*deep*/ )
@@ -1990,7 +2044,7 @@ void DocumentImpl::removeStyleSheet(StyleSheetImpl *sheet, int *exceptioncode)
     if (exceptioncode) *exceptioncode = excode;
 }
 
-void DocumentImpl::updateStyleSelector()
+void DocumentImpl::updateStyleSelector(bool shallow)
 {
 //    kDebug() << "PENDING " << m_pendingStylesheets << endl;
 
@@ -1998,7 +2052,10 @@ void DocumentImpl::updateStyleSelector()
     if (m_pendingStylesheets > 0)
         return;
 
-    recalcStyleSelector();
+    if (shallow)
+        rebuildStyleSelector();
+    else
+        recalcStyleSelector();
     recalcStyle(Force);
 #if 0
 
@@ -2148,6 +2205,11 @@ void DocumentImpl::recalcStyleSelector()
     for (; it.current(); ++it)
 	it.current()->deref();
 
+    rebuildStyleSelector();
+}
+
+void DocumentImpl::rebuildStyleSelector()
+{
     // Create a new style selector
     delete m_styleSelector;
     QString usersheet = m_usersheet;
@@ -2179,6 +2241,13 @@ void DocumentImpl::setFocusNode(NodeImpl *newFocusNode)
 {
     // don't process focus changes while detaching
     if( !m_render ) return;
+    
+    // We do want to blur if a widget is being detached,
+    // but we don't want to emit events since that 
+    // triggers updateLayout() and may recurse detach()
+    bool widgetDetach = m_focusNode && m_focusNode != this &&
+              m_focusNode->renderer() && !m_focusNode->renderer()->parent();
+      
     // Make sure newFocusNode is actually in this document
     if (newFocusNode && (newFocusNode->getDocument() != this))
         return;
@@ -2193,8 +2262,11 @@ void DocumentImpl::setFocusNode(NodeImpl *newFocusNode)
                 oldFocusNode->setActive(false);
 
             oldFocusNode->setFocus(false);
-	    oldFocusNode->dispatchHTMLEvent(EventImpl::BLUR_EVENT,false,false);
-	    oldFocusNode->dispatchUIEvent(EventImpl::DOMFOCUSOUT_EVENT);
+            
+            if (!widgetDetach) {
+                oldFocusNode->dispatchHTMLEvent(EventImpl::BLUR_EVENT,false,false);
+                oldFocusNode->dispatchUIEvent(EventImpl::DOMFOCUSOUT_EVENT);
+            }
             if ((oldFocusNode == this) && oldFocusNode->hasOneRef()) {
                 oldFocusNode->deref(); // deletes this
                 return;
@@ -2224,9 +2296,14 @@ void DocumentImpl::setFocusNode(NodeImpl *newFocusNode)
                         static_cast<RenderWidget*>(m_focusNode->renderer())->widget()->setFocus();
                 }
             }
+        } else {
+            //We're blurring. Better clear the Qt focus/give it to the view...
+            if (view())
+                view()->setFocus();
         }
 
-        updateRendering();
+        if (!widgetDetach)
+            updateRendering();
     }
 }
 
@@ -2278,7 +2355,7 @@ bool DocumentImpl::isURLAllowed(const QString& url) const
         return false;
 
     // do we allow this suburl ?
-    if ( !kapp || (newURL.protocol() != "javascript" && !KAuthorized::authorizeURLAction("redirect", thisPart->url(), newURL)) )
+    if ( !kapp || (newURL.protocol() != "javascript" && !KAuthorized::authorizeUrlAction("redirect", thisPart->url(), newURL)) )
         return false;
 
     // We allow one level of self-reference because some sites depend on that.
@@ -2652,7 +2729,7 @@ void DOM::DocumentImpl::releaseCachedNodeListInfo(NodeListImpl::Cache* entry)
 
 // ----------------------------------------------------------------------------
 
-DocumentFragmentImpl::DocumentFragmentImpl(DocumentPtr *doc) : NodeBaseImpl(doc)
+DocumentFragmentImpl::DocumentFragmentImpl(DocumentImpl *doc) : NodeBaseImpl(doc)
 {
 }
 
@@ -2693,6 +2770,8 @@ DOMString DocumentFragmentImpl::toString() const
     DOMString result;
 
     for (NodeImpl *child = firstChild(); child != NULL; child = child->nextSibling()) {
+        if (child->nodeType() == Node::COMMENT_NODE || child->nodeType() == Node::PROCESSING_INSTRUCTION_NODE)
+            continue;
 	result += child->toString();
     }
 
@@ -2710,7 +2789,7 @@ NodeImpl *DocumentFragmentImpl::cloneNode ( bool deep )
 
 // ----------------------------------------------------------------------------
 
-DocumentTypeImpl::DocumentTypeImpl(DOMImplementationImpl *implementation, DocumentPtr *doc,
+DocumentTypeImpl::DocumentTypeImpl(DOMImplementationImpl *implementation, DocumentImpl *doc,
                                    const DOMString &qualifiedName, const DOMString &publicId,
                                    const DOMString &systemId)
     : NodeImpl(doc), m_implementation(implementation),
