@@ -58,7 +58,7 @@
 using namespace DOM;
 using namespace khtml;
 
-AttrImpl::AttrImpl(ElementImpl* element, DocumentPtr* docPtr, NodeImpl::Id attrId,
+AttrImpl::AttrImpl(ElementImpl* element, DocumentImpl* docPtr, NodeImpl::Id attrId,
 		   DOMStringImpl *value, DOMStringImpl *prefix)
     : NodeBaseImpl(docPtr),
       m_element(element),
@@ -176,8 +176,10 @@ void AttrImpl::setValue( const DOMString &v, int &exceptioncode )
     m_value = v.implementation();
     m_value->ref();
 
-    if (m_element)
+    if (m_element) {
         m_element->parseAttribute(m_attrId,m_value);
+        m_element->attributeChanged(m_attrId);
+    }
 }
 
 void AttrImpl::setNodeValue( const DOMString &v, int &exceptioncode )
@@ -264,8 +266,10 @@ void AttributeImpl::setValue(DOMStringImpl *value, ElementImpl *element)
 	m_data.value = value;
 	m_data.value->ref();
 
-	if (element)
+	if (element) {
 	    element->parseAttribute(this);
+	    element->attributeChanged(m_attrId);
+        }
     }
     else {
 	int exceptioncode = 0;
@@ -274,12 +278,12 @@ void AttributeImpl::setValue(DOMStringImpl *value, ElementImpl *element)
     }
 }
 
-AttrImpl *AttributeImpl::createAttr(ElementImpl *element, DocumentPtr *docPtr)
+AttrImpl *AttributeImpl::createAttr(ElementImpl *element, DocumentImpl *docPtr)
 {
     if (m_attrId) {
 	AttrImpl *attr = new AttrImpl(element,docPtr,m_attrId,m_data.value);
         if (!attr) return 0;
-        attr->setHTMLCompat( docPtr->document()->htmlMode() != DocumentImpl::XHtml );
+        attr->setHTMLCompat( docPtr->htmlMode() != DocumentImpl::XHtml );
 	m_data.value->deref();
 	m_data.attr = attr;
 	m_data.attr->ref();
@@ -302,7 +306,7 @@ void AttributeImpl::free()
 
 // -------------------------------------------------------------------------
 
-ElementImpl::ElementImpl(DocumentPtr *doc)
+ElementImpl::ElementImpl(DocumentImpl *doc)
     : NodeBaseImpl(doc)
 {
     namedAttrMap = 0;
@@ -363,7 +367,6 @@ void ElementImpl::setAttribute(NodeImpl::Id id, const DOMString &value, const DO
         return;
     }
     attributes()->setValue(id, value.implementation(), (qName.isEmpty() ? 0: qName.implementation()));
-    attributeChanged(id);
 }
 
 void ElementImpl::setAttributeNS( const DOMString &namespaceURI, const DOMString &qualifiedName,
@@ -385,14 +388,12 @@ void ElementImpl::setAttributeNS( const DOMString &namespaceURI, const DOMString
                             prefix.implementation(), localName.implementation(), false, true /*lookupHTML*/);
     attributes()->setValue(id, value.implementation(), 0, prefix.implementation(),
                            true /*nsAware*/, !namespaceURI.isNull() /*hasNS*/);
-    attributeChanged(id);
 }
 
 void ElementImpl::setAttribute(NodeImpl::Id id, const DOMString &value)
 {
     int exceptioncode = 0;
     setAttribute(id,value,DOMString(),exceptioncode);
-    attributeChanged(id);
 }
 
 void ElementImpl::setAttributeMap( NamedAttrMapImpl* list )
@@ -418,8 +419,10 @@ void ElementImpl::setAttributeMap( NamedAttrMapImpl* list )
         assert(namedAttrMap->m_element == 0);
         namedAttrMap->setElement(this);
         unsigned long len = namedAttrMap->length();
-        for (unsigned long i = 0; i < len; i++)
+        for (unsigned long i = 0; i < len; i++) {
             parseAttribute(&namedAttrMap->m_attrs[i]);
+            attributeChanged(namedAttrMap->m_attrs[i].id());
+        }
     }
 }
 
@@ -623,6 +626,25 @@ void ElementImpl::setPrefix( const DOMString &_prefix, int &exceptioncode )
         m_prefix->ref();
 }
 
+void ElementImpl::defaultEventHandler(EventImpl *e)
+{
+    if (m_render && m_render->scrollsOverflow()) {
+        switch( e->id() ) {
+          case EventImpl::KEYDOWN_EVENT:
+          case EventImpl::KEYUP_EVENT:
+          case EventImpl::KEYPRESS_EVENT:
+            if (!focused() || e->target() != this)
+              break;
+          // fall through
+          case EventImpl::KHTML_MOUSEWHEEL_EVENT:
+            if (m_render->handleEvent(*e))
+                e->setDefaultHandled();
+          default:
+            break;
+        }
+    }
+}
+
 void ElementImpl::createAttributeMap() const
 {
     namedAttrMap = new NamedAttrMapImpl(const_cast<ElementImpl*>(this));
@@ -740,7 +762,7 @@ void ElementImpl::recalcStyle( StyleChange change )
     }
     qDebug("recalcStyle(%d: %s, changed: %d)[%p: %s]", change, debug, changed(), this, tagName().string().toLatin1().constData());
 #endif
-    if ( hasParentRenderer && (change >= Inherit || changed()) ) {
+    if ( hasParentRenderer && (change >= Inherit || changed() || (change==NoInherit && affectedByNoInherit())) ) {
         RenderStyle *newStyle = getDocument()->styleSelector()->styleForElement(this);
         newStyle->ref();
         StyleChange ch = diff( _style, newStyle );
@@ -772,8 +794,9 @@ void ElementImpl::recalcStyle( StyleChange change )
 
     NodeImpl *n;
     for (n = _first; n; n = n->nextSibling()) {
-        if ( change >= Inherit || n->isTextNode() ||
-             n->hasChangedChild() || n->changed() ) {
+        if ( change >= Inherit || n->hasChangedChild() || n->changed() ||
+             ( change == NoInherit && n->affectedByNoInherit() )
+           ) {
 	    //qDebug("    (%p) calling recalcStyle on child %p/%s, change=%d", this, n, n->isElementNode() ? ((ElementImpl *)n)->tagName().string().toLatin1().constData() : n->isTextNode() ? "text" : "unknown", change );
             n->recalcStyle( change );
         }
@@ -785,6 +808,9 @@ void ElementImpl::recalcStyle( StyleChange change )
 
 bool ElementImpl::isFocusable() const
 {
+    if (m_render && m_render->scrollsOverflow())
+        return true;
+
     // Only make editable elements selectable if its parent element
     // is not editable. FIXME: this is not 100% right as non-editable elements
     // within editable elements are focusable too.
@@ -823,6 +849,12 @@ bool ElementImpl::childTypeAllowed( unsigned short type )
     }
 }
 
+void ElementImpl::scrollIntoView(bool /*alignToTop*/)
+{
+    // ###
+    kdWarning() << "non-standard scrollIntoView() not implemented" << endl;
+}
+
 void ElementImpl::createDecl( )
 {
     m_styleDecls = new CSSStyleDeclarationImpl(0);
@@ -966,18 +998,22 @@ DOMString ElementImpl::selectionToString(NodeImpl *selectionStart, NodeImpl *sel
 
 DOMString ElementImpl::toString() const
 {
-    DOMString result = openTagStartToString();
+    QString result = openTagStartToString().string(); //Accumulate in QString, since DOMString can't append well.
 
     if (hasChildNodes()) {
 	result += ">";
 
 	for (NodeImpl *child = firstChild(); child != NULL; child = child->nextSibling()) {
-	    result += child->toString();
+	    DOMString kid = child->toString();
+	    result += QConstString(kid.unicode(), kid.length()).string();
 	}
 
 	result += "</";
-	result += tagName();
+	result += tagName().string();
 	result += ">";
+    } else if (result.length() == 1) {
+	// ensure we dont get results like < /> can happen when serialize document
+        result = "";
     } else {
 	result += " />";
     }
@@ -1018,14 +1054,14 @@ void ElementImpl::setContentEditable(bool enabled) {
 
 // -------------------------------------------------------------------------
 
-XMLElementImpl::XMLElementImpl(DocumentPtr *doc, NodeImpl::Id id)
+XMLElementImpl::XMLElementImpl(DocumentImpl *doc, NodeImpl::Id id)
     : ElementImpl(doc)
 {
     // Called from createElement(). In this case localName, prefix, and namespaceURI all need to be null.
     m_id = id;
 }
 
-XMLElementImpl::XMLElementImpl(DocumentPtr *doc, NodeImpl::Id id, DOMStringImpl *_prefix)
+XMLElementImpl::XMLElementImpl(DocumentImpl *doc, NodeImpl::Id id, DOMStringImpl *_prefix)
     : ElementImpl(doc)
 {
     // Called from createElementNS()
@@ -1132,6 +1168,7 @@ Node NamedAttrMapImpl::removeNamedItem ( NodeImpl::Id id, bool nsAware, DOMStrin
 	    m_attrCount--;
 	    m_attrs = (AttributeImpl*)realloc(m_attrs,m_attrCount*sizeof(AttributeImpl));
 	    m_element->parseAttribute(id,0);
+	    m_element->attributeChanged(id);
 	    return removed;
 	}
     }
@@ -1200,6 +1237,7 @@ Node NamedAttrMapImpl::setNamedItem ( NodeImpl* arg, bool nsAware, DOMStringImpl
 	    m_attrs[i].m_data.attr->ref();
 	    attr->setElement(m_element);
 	    m_element->parseAttribute(&m_attrs[i]);
+	    m_element->attributeChanged(m_attrs[i].id());
 	    // ### dispatch mutation events
 	    return replaced;
 	}
@@ -1215,6 +1253,7 @@ Node NamedAttrMapImpl::setNamedItem ( NodeImpl* arg, bool nsAware, DOMStringImpl
     if (id == ATTR_ID)
         m_element->updateId(0, attr->val());
     m_element->parseAttribute(&m_attrs[m_attrCount-1]);
+    m_element->attributeChanged(m_attrs[m_attrCount-1].id());
     // ### dispatch mutation events
 
     return 0;
@@ -1322,6 +1361,7 @@ void NamedAttrMapImpl::setValue(NodeImpl::Id id, DOMStringImpl *value, DOMString
 	if (id == ATTR_ID)
 	    m_element->updateId(0, value);
 	m_element->parseAttribute(&m_attrs[m_attrCount-1]);
+	m_element->attributeChanged(m_attrs[m_attrCount-1].id());
     }
     // ### dispatch mutation events
 }
@@ -1339,6 +1379,7 @@ Attr NamedAttrMapImpl::removeAttr(AttrImpl *attr)
 	    m_attrCount--;
 	    m_attrs = (AttributeImpl*)realloc(m_attrs,m_attrCount*sizeof(AttributeImpl));
 	    m_element->parseAttribute(id,0);
+	    m_element->attributeChanged(id);
 	    // ### dispatch mutation events
 	    return removed;
 	}
@@ -1382,6 +1423,7 @@ void NamedAttrMapImpl::copyAttributes(NamedAttrMapImpl *other)
 	if (m_attrs[i].id() == ATTR_ID)
 	   m_element->updateId(0, m_attrs[i].val());
 	m_element->parseAttribute(&m_attrs[i]);
+	m_element->attributeChanged(m_attrs[i].id());
     }
 }
 
