@@ -1,4 +1,4 @@
-/**
+/*
  * This file is part of the DOM implementation for KDE.
  *
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
@@ -59,8 +59,8 @@ void /*KDE_NO_EXPORT*/ mapDOMPosToRenderPos(DOM::NodeImpl *node, long offset,
 using namespace DOM;
 using namespace khtml;
 
-NodeImpl::NodeImpl(DocumentPtr *doc)
-    : document(doc),
+NodeImpl::NodeImpl(DocumentImpl *doc)
+    : m_document(doc),
       m_previous(0),
       m_next(0),
       m_render(0),
@@ -82,16 +82,12 @@ NodeImpl::NodeImpl(DocumentPtr *doc)
       m_hasClassList( false ),
       m_hasClass( false )
 {
-    if (document)
-        document->ref();
 }
 
 NodeImpl::~NodeImpl()
 {
     if (m_render)
         detach();
-    if (document)
-        document->deref();
     if (m_previous)
         m_previous->setNextSibling(0);
     if (m_next)
@@ -142,16 +138,14 @@ NodeImpl *NodeImpl::insertBefore( NodeImpl *, NodeImpl *, int &exceptioncode )
     return 0;
 }
 
-NodeImpl *NodeImpl::replaceChild( NodeImpl *, NodeImpl *, int &exceptioncode )
+void NodeImpl::replaceChild( NodeImpl *, NodeImpl *, int &exceptioncode )
 {
   exceptioncode = DOMException::HIERARCHY_REQUEST_ERR;
-  return 0;
 }
 
-NodeImpl *NodeImpl::removeChild( NodeImpl *, int &exceptioncode )
+void NodeImpl::removeChild( NodeImpl *, int &exceptioncode )
 {
   exceptioncode = DOMException::NOT_FOUND_ERR;
-  return 0;
 }
 
 NodeImpl *NodeImpl::appendChild( NodeImpl *, int &exceptioncode )
@@ -260,7 +254,7 @@ QRect NodeImpl::getRect() const
     int _x, _y;
     if(m_render && m_render->absolutePosition(_x, _y))
         return QRect( _x + m_render->inlineXPos(), _y + m_render->inlineYPos(),
-        		m_render->width(), m_render->height() );
+        		m_render->width(), m_render->height() + renderer()->borderTopExtra() + renderer()->borderBottomExtra() );
 
     return QRect();
 }
@@ -348,7 +342,7 @@ void NodeImpl::dispatchEvent(EventImpl *evt, int &exceptioncode, bool tempEvent)
     evt->setTarget(this);
 
     // Since event handling code could cause this object to be deleted, grab a reference to the view now
-    KHTMLView *view = document->document()->view();
+    KHTMLView *view = getDocument()->view();
 
     dispatchGenericEvent( evt, exceptioncode );
 
@@ -414,7 +408,7 @@ void NodeImpl::dispatchGenericEvent( EventImpl *evt, int &/*exceptioncode */)
     }
 
     // copy this over into a local variable, as the following deref() calls might cause this to be deleted.
-    DocumentPtr *doc = document;
+    DocumentImpl *doc = m_document.get();
     doc->ref();
 
     // deref all nodes in chain
@@ -443,20 +437,20 @@ void NodeImpl::dispatchWindowEvent(int _id, bool canBubbleArg, bool cancelableAr
     EventImpl* const evt = new EventImpl(static_cast<EventImpl::EventId>(_id),canBubbleArg,cancelableArg);
     evt->setTarget( 0 );
     evt->ref();
-    DocumentPtr *doc = document;
+    DocumentImpl *doc = getDocument();
     doc->ref();
     dispatchGenericEvent( evt, exceptioncode );
-    if (!evt->defaultPrevented() && doc->document())
-	doc->document()->defaultEventHandler(evt);
+    if (!evt->defaultPrevented())
+	doc->defaultEventHandler(evt);
 
-    if (_id == EventImpl::LOAD_EVENT && !evt->propagationStopped() && doc->document()) {
+    if (_id == EventImpl::LOAD_EVENT && !evt->propagationStopped()) {
         // For onload events, send them to the enclosing frame only.
         // This is a DOM extension and is independent of bubbling/capturing rules of
         // the DOM.  You send the event only to the enclosing frame.  It does not
         // bubble through the parent document.
-        DOM::ElementImpl* elt = doc->document()->ownerElement();
+        DOM::ElementImpl* elt = doc->ownerElement();
         if (elt && (elt->getDocument()->domain().isNull() ||
-                    elt->getDocument()->domain() == doc->document()->domain())) {
+                    elt->getDocument()->domain() == doc->domain())) {
             // We also do a security check, since we don't want to allow the enclosing
             // iframe to see loads of child documents in other domains.
             evt->setCurrentTarget(elt);
@@ -509,10 +503,12 @@ void NodeImpl::dispatchMouseEvent(QMouseEvent *_mouse, int overrideId, int overr
     int exceptioncode = 0;
     int pageX = _mouse->x();
     int pageY = _mouse->y();
+    if ( getDocument()->view() )
+        getDocument()->view()->revertTransforms( pageX, pageY );
     int clientX = pageX;
     int clientY = pageY;
     if ( getDocument()->view() )
-        getDocument()->view()->viewportToContents( clientX, clientY, pageX, pageY );
+        getDocument()->view()->contentsToViewport( pageX, pageY, pageX, pageY );
 
     int screenX = _mouse->globalX();
     int screenY = _mouse->globalY();
@@ -629,7 +625,7 @@ void NodeImpl::handleLocalEvents(EventImpl *evt, bool useCapture)
     }
 }
 
-void NodeImpl::defaultEventHandler(EventImpl *)
+void NodeImpl::defaultEventHandler(EventImpl *e)
 {
 }
 
@@ -793,6 +789,8 @@ NodeImpl::StyleChange NodeImpl::diff( khtml::RenderStyle *s1, khtml::RenderStyle
 	ch = Inherit;
     else if ( *s1 == *s2 )
  	ch = NoChange;
+    else if (s1->useNormalContent() != s2->useNormalContent())
+        ch = Detach; // when we add generated content all children must be detached
     else if ( s1->inheritedNotEqual( s2 ) )
 	ch = Inherit;
 
@@ -805,6 +803,8 @@ NodeImpl::StyleChange NodeImpl::diff( khtml::RenderStyle *s1, khtml::RenderStyle
         ch = NoInherit;
     if (ch == NoChange && pseudoDiff(s1, s2, khtml::RenderStyle::AFTER))
         ch = NoInherit;
+    if (ch == NoChange && pseudoDiff(s1, s2, khtml::RenderStyle::MARKER))
+        ch = NoInherit;
     if (ch == NoChange && pseudoDiff(s1, s2, khtml::RenderStyle::SELECTION))
         ch = NoInherit;
     if (ch == NoChange && pseudoDiff(s1, s2, khtml::RenderStyle::FIRST_LINE))
@@ -831,6 +831,14 @@ bool NodeImpl::pseudoDiff( khtml::RenderStyle *s1, khtml::RenderStyle *s2, unsig
         return true;
 }
 
+bool NodeImpl::affectedByNoInherit() const
+{
+    if (m_render && m_render->style())
+        return m_render->style()->inheritedNoninherited();
+    else
+        return false;
+}
+
 void NodeImpl::close()
 {
     if (m_render) m_render->close();
@@ -1093,6 +1101,10 @@ NodeImpl *NodeBaseImpl::insertBefore ( NodeImpl *newChild, NodeImpl *refChild, i
 
         // If child is already present in the tree, first remove it
         NodeImpl *newParent = child->parentNode();
+
+        //...guard it in case we need to move it..
+        SharedPtr<NodeImpl> guard(child);
+
         if(newParent)
             newParent->removeChild( child, exceptioncode );
         if ( exceptioncode )
@@ -1127,22 +1139,22 @@ NodeImpl *NodeBaseImpl::insertBefore ( NodeImpl *newChild, NodeImpl *refChild, i
     return newChild;
 }
 
-NodeImpl *NodeBaseImpl::replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, int &exceptioncode )
+void NodeBaseImpl::replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, int &exceptioncode )
 {
     exceptioncode = 0;
 
     if ( oldChild == newChild ) // nothing to do
-	return oldChild;
+	return;
 
     // Make sure adding the new child is ok
     checkAddChild(newChild, exceptioncode);
     if (exceptioncode)
-        return 0;
+        return;
 
     // NOT_FOUND_ERR: Raised if oldChild is not a child of this node.
     if (!oldChild || oldChild->parentNode() != this) {
         exceptioncode = DOMException::NOT_FOUND_ERR;
-        return 0;
+        return;
     }
 
     bool isFragment = newChild->nodeType() == Node::DOCUMENT_FRAGMENT_NODE;
@@ -1156,7 +1168,7 @@ NodeImpl *NodeBaseImpl::replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, i
 
     removeChild(oldChild, exceptioncode);
     if (exceptioncode)
-        return 0;
+        return;
 
     // Add the new child(ren)
     while (child) {
@@ -1168,10 +1180,12 @@ NodeImpl *NodeBaseImpl::replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, i
 	    next = child->nextSibling();
 	if ( child == prev )
 	    prev = child->previousSibling();
+        //...guard it in case we need to move it..
+        SharedPtr<NodeImpl> guard(child);
         if(newParent)
             newParent->removeChild( child, exceptioncode );
         if (exceptioncode)
-            return 0;
+            return;
 
         // Add child in the correct position
         if (prev) prev->setNextSibling(child);
@@ -1198,28 +1212,30 @@ NodeImpl *NodeBaseImpl::replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, i
 
     // ### set style in case it's attached
     dispatchSubtreeModifiedEvent();
-    return oldChild;
+    return;
 }
 
-NodeImpl *NodeBaseImpl::removeChild ( NodeImpl *oldChild, int &exceptioncode )
+void NodeBaseImpl::removeChild ( NodeImpl *oldChild, int &exceptioncode )
 {
     exceptioncode = 0;
 
     // NO_MODIFICATION_ALLOWED_ERR: Raised if this node is readonly.
     if (isReadOnly()) {
         exceptioncode = DOMException::NO_MODIFICATION_ALLOWED_ERR;
-        return 0;
+        return;
     }
 
     // NOT_FOUND_ERR: Raised if oldChild is not a child of this node.
     if (!oldChild || oldChild->parentNode() != this) {
         exceptioncode = DOMException::NOT_FOUND_ERR;
-        return 0;
+        return;
     }
 
     dispatchChildRemovalEvents(oldChild,exceptioncode);
     if (exceptioncode)
-        return 0;
+        return;
+
+    SharedPtr<NodeImpl> memManage(oldChild); //Make sure to free if needed
 
     // Remove from rendering tree
     if (oldChild->attached())
@@ -1251,8 +1267,6 @@ NodeImpl *NodeBaseImpl::removeChild ( NodeImpl *oldChild, int &exceptioncode )
 	for (NodeImpl *c = oldChild; c; c = c->traverseNextNode(oldChild))
 	    c->removedFromDocument();
     }
-
-    return oldChild;
 }
 
 void NodeBaseImpl::removeChildren()
@@ -1307,6 +1321,7 @@ NodeImpl *NodeBaseImpl::appendChild ( NodeImpl *newChild, int &exceptioncode )
 
         // If child is already present in the tree, first remove it
         NodeImpl *oldParent = child->parentNode();
+        SharedPtr<NodeImpl> guard(child); //Guard in case we move it
         if(oldParent) {
             oldParent->removeChild( child, exceptioncode );
             if (exceptioncode)
@@ -1506,7 +1521,7 @@ bool NodeBaseImpl::getLowerRightCorner(int &xPos, int &yPos) const
     {
         o->absolutePosition( xPos, yPos );
         xPos += o->width();
-        yPos += o->height();
+        yPos += o->height() + o->borderTopExtra() + o->borderBottomExtra();
         return true;
     }
     // find the last text/image child, to get a position
@@ -1934,10 +1949,10 @@ Node NamedNodeMapImpl::removeNamedItemNS( const DOMString &namespaceURI, const D
 
 // ----------------------------------------------------------------------------
 
-GenericRONamedNodeMapImpl::GenericRONamedNodeMapImpl(DocumentPtr* doc)
+GenericRONamedNodeMapImpl::GenericRONamedNodeMapImpl(DocumentImpl* doc)
     : NamedNodeMapImpl()
 {
-    m_doc = doc->document();
+    m_doc = doc;
     m_contents = new Q3PtrList<NodeImpl>;
 }
 
