@@ -58,18 +58,6 @@ class CSSStyleDeclarationImpl;
 class RegisteredEventListener;
 class EventImpl;
 
-class DocumentPtr : public khtml::Shared<DocumentPtr>
-{
-public:
-    DocumentImpl *document() const { return doc; }
-private:
-    DocumentPtr() { doc = 0; }
-    friend class DocumentImpl;
-    friend class DOMImplementationImpl;
-
-    DocumentImpl *doc;
-};
-
 struct RegisteredListenerList {
     RegisteredListenerList() : listeners(0)
     {}
@@ -114,7 +102,7 @@ class NodeImpl : public khtml::TreeShared<NodeImpl>
 {
     friend class DocumentImpl;
 public:
-    NodeImpl(DocumentPtr *doc);
+    NodeImpl(DocumentImpl *doc);
     virtual ~NodeImpl();
 
     // DOM methods & attributes for Node
@@ -139,8 +127,10 @@ public:
     // insertBefore, replaceChild and appendChild also close newChild
     // unlike the speed optimized addChild (which is used by the parser)
     virtual NodeImpl *insertBefore ( NodeImpl *newChild, NodeImpl *refChild, int &exceptioncode );
-    virtual NodeImpl *replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, int &exceptioncode );
-    virtual NodeImpl *removeChild ( NodeImpl *oldChild, int &exceptioncode );
+    
+    /* These two methods may delete the old node, so make sure to reference it if you need it */
+    virtual void replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, int &exceptioncode );
+    virtual void removeChild ( NodeImpl *oldChild, int &exceptioncode );
     virtual NodeImpl *appendChild ( NodeImpl *newChild, int &exceptioncode );
     virtual bool hasChildNodes (  ) const;
     virtual NodeImpl *cloneNode ( bool deep ) = 0;
@@ -202,7 +192,8 @@ public:
         MouseRelease,
         MouseClick,
         MouseDblClick,
-        MouseMove
+        MouseMove,
+        MouseWheel
     };
 
     struct MouseEvent
@@ -276,10 +267,12 @@ public:
     static StyleChange diff( khtml::RenderStyle *s1, khtml::RenderStyle *s2 );
     static bool pseudoDiff( khtml::RenderStyle *s1, khtml::RenderStyle *s2, unsigned int pid);
 
+    virtual bool affectedByNoInherit() const;
+
     unsigned long nodeIndex() const;
     // Returns the document that this node is associated with. This is guaranteed to always be non-null, as opposed to
     // DOM's ownerDocument() which is null for Document nodes (and sometimes DocumentType nodes).
-    DocumentImpl* getDocument() const { return document->document(); }
+    DocumentImpl* getDocument() const { return m_document.get(); }
 
     void addEventListener(int id, EventListener *listener, const bool useCapture);
     void removeEventListener(int id, EventListener *listener, bool useCapture);
@@ -330,7 +323,7 @@ public:
      */
     NodeImpl *traversePreviousNode() const;
 
-    DocumentPtr *docPtr() const { return document; }
+    DocumentImpl *docPtr() const { return m_document.get(); }
 
     khtml::RenderObject *renderer() const { return m_render; }
     khtml::RenderObject *nextRenderer();
@@ -454,10 +447,21 @@ public:
      * @param found          When this is set to true, don't print anymore but closing tags.
      * @return An html formatted string for this node and its children between the selectionStart and selectionEnd.
      */
-    virtual DOMString selectionToString(NodeImpl * /*selectionStart*/, NodeImpl * /*selectionEnd*/, int /*startOffset*/, int /*endOffset*/, bool &/*found*/) const { return toString(); }
+    virtual DOMString selectionToString(NodeImpl * selectionStart, 
+                                        NodeImpl * selectionEnd, 
+                                        int startOffset, 
+                                        int endOffset, 
+                                        bool &found) const 
+    { Q_UNUSED(selectionStart);
+      Q_UNUSED(selectionEnd);
+      Q_UNUSED(startOffset);
+      Q_UNUSED(endOffset);
+      Q_UNUSED(found);
+      return toString(); 
+    }
 
 private: // members
-    DocumentPtr *document;
+    khtml::DocPtr<DocumentImpl> m_document;
     NodeImpl *m_previous;
     NodeImpl *m_next;
 protected:
@@ -490,7 +494,7 @@ protected:
 class NodeBaseImpl : public NodeImpl
 {
 public:
-    NodeBaseImpl(DocumentPtr *doc)
+    NodeBaseImpl(DocumentImpl *doc)
         : NodeImpl(doc), _first(0), _last(0) {}
     virtual ~NodeBaseImpl();
 
@@ -498,8 +502,8 @@ public:
     virtual NodeImpl *firstChild() const;
     virtual NodeImpl *lastChild() const;
     virtual NodeImpl *insertBefore ( NodeImpl *newChild, NodeImpl *refChild, int &exceptioncode );
-    virtual NodeImpl *replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, int &exceptioncode );
-    virtual NodeImpl *removeChild ( NodeImpl *oldChild, int &exceptioncode );
+    virtual void replaceChild ( NodeImpl *newChild, NodeImpl *oldChild, int &exceptioncode );
+    virtual void removeChild ( NodeImpl *oldChild, int &exceptioncode );
     virtual NodeImpl *appendChild ( NodeImpl *newChild, int &exceptioncode );
     virtual bool hasChildNodes (  ) const;
 
@@ -714,7 +718,7 @@ public:
 class GenericRONamedNodeMapImpl : public NamedNodeMapImpl
 {
 public:
-    GenericRONamedNodeMapImpl(DocumentPtr* doc);
+    GenericRONamedNodeMapImpl(DocumentImpl* doc);
     virtual ~GenericRONamedNodeMapImpl();
 
     // DOM methods & attributes for NamedNodeMap
