@@ -16,9 +16,8 @@
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 */
-#ifdef HAVE_CONFIG_H
+
 #include <config.h>
-#endif
 
 #include <stdio.h>
 #include <unistd.h>
@@ -37,7 +36,7 @@
 #include <kprotocolinfo.h>
 #include <krun.h>
 #include <kstandarddirs.h>
-#include <ktempfile.h>
+#include <ktemporaryfile.h>
 #include <kurl.h>
 
 #include "kio/global.h"
@@ -156,7 +155,7 @@ IdleSlave::age(time_t now)
 }
 
 KLauncher::KLauncher(int _kdeinitSocket)
-  : KApplication( false ), // No GUI
+  : QObject(0),
     kdeinitSocket(_kdeinitSocket), dontBlockReading(false)
 {
 #ifdef Q_WS_X11
@@ -164,26 +163,29 @@ KLauncher::KLauncher(int _kdeinitSocket)
 #endif
    mAutoTimer.setSingleShot(true);
    new KLauncherAdaptor(this);
-   QDBus::sessionBus().registerObject("/KLauncher", this); // same as ktoolinvocation.cpp
+   QDBusConnection::sessionBus().registerObject("/KLauncher", this); // same as ktoolinvocation.cpp
 
    connect(&mAutoTimer, SIGNAL(timeout()), this, SLOT(slotAutoStart()));
-   connect(QDBus::sessionBus().interface(),
+   connect(QDBusConnection::sessionBus().interface(),
            SIGNAL(serviceOwnerChanged(QString,QString,QString)),
            SLOT(slotNameOwnerChanged(QString,QString,QString)));
 
    QString prefix = KStandardDirs::locateLocal("socket", "klauncher");
-   KTempFile domainname(prefix, QLatin1String(".slave-socket"));
-   if (domainname.status() != 0)
+   KTemporaryFile *domainname = new KTemporaryFile();
+   domainname->setPrefix(prefix);
+   domainname->setSuffix(QLatin1String(".slave-socket"));
+   domainname->setAutoRemove(false);
+   if (!domainname->open())
    {
       // Sever error!
       qDebug("KLauncher: Fatal error, can't create tempfile!");
       ::_exit(1);
    }
-   mPoolSocketName = domainname.name();
+   mPoolSocketName = domainname->fileName();
 #ifdef __CYGWIN__
-   domainname.close();
-   domainname.unlink();
+   domainname->setAutoRemove(true);
 #endif
+   delete domainname;
 
    mPoolSocket.setAddress(QFile::encodeName(mPoolSocketName));
    mPoolSocket.setAcceptBuffered(false); // we use KStreamSockets
@@ -238,8 +240,8 @@ void KLauncher::close()
 void
 KLauncher::destruct(int exit_code)
 {
-   if (kapp) ((KLauncher*)kapp)->close();
-   // We don't delete kapp here, that's intentional.
+   if (qApp) ((KLauncher*)qApp)->close();
+   // We don't delete qApp here, that's intentional.
    ::_exit(exit_code);
 }
 
@@ -327,31 +329,16 @@ KLauncher::slotKDEInitData(int)
      lastRequest->pid = (pid_t) (*request_data);
      kDebug(7016) << lastRequest->name << " (pid " << lastRequest->pid <<
         ") up and running." << endl;
-     switch(lastRequest->dcop_service_type)
+     switch(lastRequest->dbus_startup_type)
      {
-       case KService::DCOP_None:
-       {
+       case KService::DBUS_None:
          lastRequest->status = KLaunchRequest::Running;
          break;
-       }
-
-       case KService::DCOP_Unique:
-       {
+       case KService::DBUS_Unique:
+       case KService::DBUS_Wait:
+       case KService::DBUS_Multi:
          lastRequest->status = KLaunchRequest::Launching;
          break;
-       }
-
-       case KService::DCOP_Wait:
-       {
-         lastRequest->status = KLaunchRequest::Launching;
-         break;
-       }
-
-       case KService::DCOP_Multi:
-       {
-         lastRequest->status = KLaunchRequest::Launching;
-         break;
-       }
      }
      lastRequest = 0;
      return;
@@ -376,10 +363,9 @@ KLauncher::processDied(pid_t pid, long /* exitStatus */)
    {
       if (request->pid == pid)
       {
-         if (request->dcop_service_type == KService::DCOP_Wait)
+         if (request->dbus_startup_type == KService::DBUS_Wait)
             request->status = KLaunchRequest::Done;
-         else if ((request->dcop_service_type == KService::DCOP_Unique) &&
-                  QDBus::sessionBus().interface()->isServiceRegistered(request->dcop_name))
+         else if ((request->dbus_startup_type == KService::DBUS_Unique) && QDBusConnection::sessionBus().interface()->isServiceRegistered(request->dbus_name))
             request->status = KLaunchRequest::Running;
          else
             request->status = KLaunchRequest::Error;
@@ -403,16 +389,16 @@ KLauncher::slotNameOwnerChanged(const QString &appId, const QString &oldOwner,
          continue;
 
       // For unique services check the requested service name first
-      if ((request->dcop_service_type == KService::DCOP_Unique) &&
-          ((appId == request->dcop_name) ||
-           QDBus::sessionBus().interface()->isServiceRegistered(request->dcop_name)))
+      if ((request->dbus_startup_type == KService::DBUS_Unique) &&
+          ((appId == request->dbus_name) ||
+           QDBusConnection::sessionBus().interface()->isServiceRegistered(request->dbus_name)))
       {
          request->status = KLaunchRequest::Running;
          requestDone(request);
          continue;
       }
 
-      const QString rAppId = request->dcop_name;
+      const QString rAppId = request->dbus_name;
       if (rAppId.isEmpty())
           return;
 
@@ -422,7 +408,7 @@ KLauncher::slotNameOwnerChanged(const QString &appId, const QString &oldOwner,
       if (appId.startsWith(rAppId) && ((appId.length() == rAppId.length()) ||
                   (c == QLatin1Char('-'))))
       {
-         request->dcop_name = appId;
+         request->dbus_name = appId;
          request->status = KLaunchRequest::Running;
          requestDone(request);
          continue;
@@ -482,16 +468,15 @@ KLauncher::requestDone(KLaunchRequest *request)
        (request->status == KLaunchRequest::Done))
    {
       requestResult.result = 0;
-      requestResult.dcopName = request->dcop_name;
+      requestResult.dbusName = request->dbus_name;
       requestResult.error = "";
       requestResult.pid = request->pid;
    }
    else
    {
       requestResult.result = 1;
-      requestResult.dcopName = "";
-      requestResult.error = i18n("KDEInit could not launch '%1'.",
-	  request->name);
+      requestResult.dbusName = "";
+      requestResult.error = i18n("KDEInit could not launch '%1'.", request->name);
       if (!request->errorMsg.isEmpty())
          requestResult.error += ":\n" + request->errorMsg;
       requestResult.pid = 0;
@@ -525,13 +510,13 @@ KLauncher::requestDone(KLaunchRequest *request)
 
    if (request->transaction.type() != QDBusMessage::InvalidMessage)
    {
-      if ( requestResult.dcopName.isNull() ) // null strings can't be sent
-          requestResult.dcopName = "";
+      if ( requestResult.dbusName.isNull() ) // null strings can't be sent
+          requestResult.dbusName = "";
       Q_ASSERT( !requestResult.error.isNull() );
-      request->transaction.sendReply(QVariantList() << requestResult.result
-                                     << requestResult.dcopName
+      QDBusConnection::sessionBus().send(request->transaction.createReply(QVariantList() << requestResult.result
+                                     << requestResult.dbusName
                                      << requestResult.error
-                                     << requestResult.pid);
+                                     << requestResult.pid));
    }
    requestList.removeAll( request );
    delete request;
@@ -595,7 +580,7 @@ void KLauncher::exec_blind(const QString &name, const QStringList &arg_list, con
    request->autoStart = false;
    request->name = name;
    request->arg_list =  arg_list;
-   request->dcop_service_type = KService::DCOP_None;
+   request->dbus_startup_type = KService::DBUS_None;
    request->pid = 0;
    request->status = KLaunchRequest::Launching;
    request->envs = envs;
@@ -720,20 +705,18 @@ KLauncher::start_service(KService::Ptr service, const QStringList &_urls,
       return false;
    }
 
-   request->name = request->arg_list.first();
-   request->arg_list.removeFirst(); //(request->arg_list.begin());
+   request->name = request->arg_list.takeFirst();
+   request->dbus_startup_type =  service->DBUSStartupType();
 
-   request->dcop_service_type =  service->DCOPServiceType();
-
-   if ((request->dcop_service_type == KService::DCOP_Unique) ||
-       (request->dcop_service_type == KService::DCOP_Multi))
+   if ((request->dbus_startup_type == KService::DBUS_Unique) ||
+       (request->dbus_startup_type == KService::DBUS_Multi))
    {
-      QVariant v = service->property("X-DCOP-ServiceName");
+      QVariant v = service->property("X-DBUS-ServiceName");
       if (v.isValid())
-         request->dcop_name = v.toString().toUtf8();
-      if (request->dcop_name.isEmpty())
+         request->dbus_name = v.toString().toUtf8();
+      if (request->dbus_name.isEmpty())
       {
-         request->dcop_name = QFile::encodeName(KRun::binaryName(service->exec(), true));
+         request->dbus_name = "org.kde." + QFile::encodeName(KRun::binaryName(service->exec(), true));
       }
    }
 
@@ -854,9 +837,9 @@ KLauncher::kdeinit_exec(const QString &app, const QStringList &args,
    request->name = app.toLocal8Bit();
 
    if (wait)
-      request->dcop_service_type = KService::DCOP_Wait;
+      request->dbus_startup_type = KService::DBUS_Wait;
    else
-      request->dcop_service_type = KService::DCOP_None;
+      request->dbus_startup_type = KService::DBUS_None;
    request->pid = 0;
 #ifdef Q_WS_X11
    request->startup_id = startup_id;
@@ -1016,7 +999,7 @@ KLauncher::requestSlave(const QString &protocol,
     request->autoStart = false;
     request->name = name;
     request->arg_list =  arg_list;
-    request->dcop_service_type = KService::DCOP_None;
+    request->dbus_startup_type = KService::DBUS_None;
     request->pid = 0;
 #ifdef Q_WS_X11
     request->startup_id = "0";
@@ -1077,7 +1060,7 @@ KLauncher::slotSlaveStatus(IdleSlave *slave)
        SlaveWaitRequest *waitRequest = it.next();
        if (waitRequest->pid == slave->pid())
        {
-          waitRequest->transaction.sendReply();
+           QDBusConnection::sessionBus().send(waitRequest->transaction.createReply());
           it.remove();
           delete waitRequest;
        }
@@ -1119,4 +1102,9 @@ void KLauncher::reparseConfiguration()
       slave->reparseConfiguration();
 }
 
+void 
+KLauncher::slotGotOutput()
+{
+}
+
 #include "klauncher.moc"
