@@ -24,12 +24,11 @@
 #include "kbookmarkmenu_p.h"
 #include "kbookmarkimporter.h"
 #include <kdebug.h>
-#include <krun.h>
 #include <kstandarddirs.h>
 #include <ksavefile.h>
 #include <qregexp.h>
 #include <kmessagebox.h>
-#include <kprocess.h>
+#include <qprocess.h>
 #include <klocale.h>
 #include <kapplication.h>
 #include <qdatetime.h>
@@ -37,6 +36,7 @@
 #include <qfileinfo.h>
 #include <qtextstream.h>
 #include <QtDBus/QtDBus>
+#include <QTextStream>
 #include <kstaticdeleter.h>
 #include "kbookmarkmanageradaptor_p.h"
 
@@ -118,7 +118,7 @@ KBookmarkManager* KBookmarkManager::createTempManager()
 #define PI_DATA "version=\"1.0\" encoding=\"UTF-8\""
 
 KBookmarkManager::KBookmarkManager( const QString & bookmarksFile, const QString & dbusObjectName, bool bImportDesktopFiles )
-    : m_doc("xbel"), m_docIsLoaded(false)
+    : m_doc("xbel"), m_docIsLoaded(false), m_dbusObjectName(dbusObjectName)
 {
     init( "/KBookmarkManager/"+dbusObjectName );
 
@@ -156,15 +156,13 @@ void KBookmarkManager::init( const QString& dbusPath )
     if ( dbusPath != "/KBookmarkManager/" )
     {
         new KBookmarkManagerAdaptor(this);
-        QDBus::sessionBus().registerObject( dbusPath, this );
+        QDBusConnection::sessionBus().registerObject( dbusPath, this );
 
-        QDBus::sessionBus().connect(QString(), dbusPath, "org.kde.KIO.KBookmarkManager",
+        QDBusConnection::sessionBus().connect(QString(), dbusPath, "org.kde.KIO.KBookmarkManager",
                                     "bookmarksChanged", this, SLOT(notifyChanged(QString,QDBusMessage)));
-        QDBus::sessionBus().connect(QString(), dbusPath, "org.kde.KIO.KBookmarkManager",
+        QDBusConnection::sessionBus().connect(QString(), dbusPath, "org.kde.KIO.KBookmarkManager",
                                     "bookmarkConfigChanged", this, SLOT(notifyConfigChanged()));
     }
-
-    m_showNSBookmarks = true;
 }
 
 KBookmarkManager::~KBookmarkManager()
@@ -212,7 +210,7 @@ void KBookmarkManager::parse() const
         {
             kWarning() << "Old style bookmarks found. Calling convertToXBEL." << endl;
             docElem.setTagName("xbel");
-            if ( docElem.hasAttribute( "HIDE_NSBK" ) && m_showNSBookmarks ) // non standard either, but we need it
+            if ( docElem.hasAttribute( "HIDE_NSBK" ) ) // non standard either, but we need it
             {
                 docElem.setAttribute( "hide_nsbk", docElem.attribute( "HIDE_NSBK" ) == "1" ? "yes" : "no" );
                 docElem.removeAttribute( "HIDE_NSBK" );
@@ -327,14 +325,14 @@ bool KBookmarkManager::saveAs( const QString & filename, bool toolbarCache ) con
     if(toolbarCache && !root().isToolbarGroup())
     {
         KSaveFile cacheFile( cacheFilename );
-        if ( cacheFile.status() == 0 )
+        if ( cacheFile.open() )
         {
             QString str;
             QTextStream stream(&str, QIODevice::WriteOnly);
             stream << root().findToolbar();
             const QByteArray cstr = str.toUtf8();
-            cacheFile.file()->write( cstr.data(), cstr.length() );
-            cacheFile.close();
+            cacheFile.write( cstr.data(), cstr.length() );
+            cacheFile.finalize();
         }
     }
     else // remove any (now) stale cache
@@ -343,12 +341,13 @@ bool KBookmarkManager::saveAs( const QString & filename, bool toolbarCache ) con
     }
 
     KSaveFile file( filename );
-    if ( file.status() == 0 )
+    if ( file.open() )
     {
-        file.simpleBackupFile( file.name(), QString(), ".bak" );
-        QByteArray cstr = internalDocument().toByteArray(); // is in UTF8
-        file.file()->write( cstr.data(), cstr.length() );
-        if ( file.close() )
+        file.simpleBackupFile( file.fileName(), QString(), ".bak" );
+        QTextStream stream(&file);
+        stream << internalDocument().toString();
+        stream.flush();
+        if ( file.finalize() )
             return true;
     }
 
@@ -359,7 +358,7 @@ bool KBookmarkManager::saveAs( const QString & filename, bool toolbarCache ) con
                              "This error message will only be shown once. The cause "
                              "of the error needs to be fixed as quickly as possible, "
                              "which is most likely a full hard drive.",
-                         filename, strerror(file.status()));
+                         filename, file.errorString());
         if (qApp->type() != QApplication::Tty)
             KMessageBox::error( 0L, error );
         else
@@ -547,7 +546,7 @@ void KBookmarkManager::notifyCompleteChange( QString caller ) // DBUS call
 {
     if (!m_update) return;
 
-    //kDebug(7043) << "KBookmarkManager::notifyCompleteChange" << endl;
+    kDebug(7043) << "KBookmarkManager::notifyCompleteChange" << endl;
     // The bk editor tells us we should reload everything
     // Reparse
     parse();
@@ -565,11 +564,12 @@ void KBookmarkManager::notifyConfigChanged() // DBUS call
 
 void KBookmarkManager::notifyChanged( QString groupAddress, const QDBusMessage &msg ) // DBUS call
 {
+    kDebug() << "KBookmarkManager::notifyChanged ( "<<groupAddress<<")"<<endl;
     if (!m_update) return;
 
     // Reparse (the whole file, no other choice)
     // if someone else notified us
-    if (msg.service() != QDBus::sessionBus().baseService())
+    if (msg.service() != QDBusConnection::sessionBus().baseService())
        parse();
 
     //kDebug(7043) << "KBookmarkManager::notifyChanged " << groupAddress << endl;
@@ -578,22 +578,6 @@ void KBookmarkManager::notifyChanged( QString groupAddress, const QDBusMessage &
     emit changed( groupAddress, QString() );
 }
 
-bool KBookmarkManager::showNSBookmarks() const
-{
-    return KBookmarkMenu::showDynamicBookmarks("netscape").show;
-}
-
-void KBookmarkManager::setShowNSBookmarks( bool show )
-{
-    m_showNSBookmarks = show;
-    if (this != userBookmarksManager())
-       return;
-    KBookmarkMenu::DynMenuInfo info
-       = KBookmarkMenu::showDynamicBookmarks("netscape");
-    info.show = show;
-    KBookmarkMenu::setDynamicBookmarks("netscape", info);
-}
-
 void KBookmarkManager::setEditorOptions( const QString& caption, bool browser )
 {
     m_editorCaption = caption;
@@ -602,36 +586,27 @@ void KBookmarkManager::setEditorOptions( const QString& caption, bool browser )
 
 void KBookmarkManager::slotEditBookmarks()
 {
-    KProcess proc;
-    proc << QLatin1String("keditbookmarks");
-    if (!m_editorCaption.isNull())
-       proc << QLatin1String("--customcaption") << m_editorCaption;
-    if (!m_browserEditor)
-       proc << QLatin1String("--nobrowser");
-    proc << m_bookmarksFile;
-    proc.start(KProcess::DontCare);
+    QStringList args;
+    if ( !m_editorCaption.isEmpty() )
+       args << QLatin1String("--customcaption") << m_editorCaption;
+    if ( !m_browserEditor )
+       args << QLatin1String("--nobrowser");
+    if( !m_dbusObjectName.isEmpty() )
+      args << QLatin1String("--dbusObjectName") << m_dbusObjectName;
+    args << m_bookmarksFile;
+    QProcess::startDetached("keditbookmarks", args);
 }
 
 void KBookmarkManager::slotEditBookmarksAtAddress( const QString& address )
 {
-    KProcess proc;
+    QStringList args;
     //TODO shouldn't we pass --customcaption and --nobrowser too?
-    proc << QLatin1String("keditbookmarks")
-         << QLatin1String("--address") << address
+    args << QLatin1String("--address") << address
          << m_bookmarksFile;
-    proc.start(KProcess::DontCare);
+    QProcess::startDetached("keditbookmarks", args);
 }
 
 ///////
-
-void KBookmarkOwner::openBookmarkURL( const QString& url )
-{
-  (void) new KRun(KUrl( url ),(QWidget*)0);
-}
-
-void KBookmarkOwner::virtual_hook( int, void* )
-{ /*BASE::virtual_hook( id, data );*/ }
-
 bool KBookmarkManager::updateAccessMetadata( const QString & url, bool emitSignal )
 {
     if (!s_bk_map) {
@@ -681,7 +656,7 @@ void KBookmarkManager::updateFavicon( const QString &url, const QString &favicon
 KBookmarkManager* KBookmarkManager::userBookmarksManager()
 {
    QString bookmarksFile = KStandardDirs::locateLocal("data", QLatin1String("konqueror/bookmarks.xml"));
-   return KBookmarkManager::managerForFile( bookmarksFile, "konqueror" );
+   return KBookmarkManager::managerForFile( bookmarksFile, "konqueror", true );
 }
 
 KBookmarkSettings* KBookmarkSettings::s_self = 0;
@@ -709,4 +684,11 @@ KBookmarkSettings *KBookmarkSettings::self()
    return s_self;
 }
 
+void KBookmarkOwner::openBookmark(const KBookmark & bm, Qt::MouseButtons mb, Qt::KeyboardModifiers km)
+{
+   Q_UNUSED(bm)
+   Q_UNUSED(mb)
+   Q_UNUSED(km)
+}
+
 #include "kbookmarkmanager.moc"
