@@ -2,6 +2,7 @@
 // vim: set ts=2 sts=2 sw=2 et:
 /* This file is part of the KDE project
    Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
+   Copyright (C) 2006 Daniel Teske <teske@squorn.de>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -21,9 +22,6 @@
 
 #include "kbookmarkmenu.h"
 #include "kbookmarkmenu_p.h"
-#include "kbookmarkimporter.h"
-#include "kbookmarkimporter_opera.h"
-#include "kbookmarkimporter_ie.h"
 
 #include <kapplication.h>
 #include <kauthorized.h>
@@ -34,9 +32,12 @@
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kmenu.h>
-#include <kstdaccel.h>
-#include <kstdaction.h>
+#include <kstandardshortcut.h>
+#include <kstandardaction.h>
 #include <kstringhandler.h>
+#include <krun.h>
+#include <kstringhandler.h>
+#include <kactioncollection.h>
 
 #include <qclipboard.h>
 #include <qfile.h>
@@ -76,63 +77,80 @@ static QString makeTextNodeMod(KBookmark bk, const QString &m_nodename, const QS
 
 KBookmarkMenu::KBookmarkMenu( KBookmarkManager* mgr,
                               KBookmarkOwner * _owner, KMenu * _parentMenu,
-                              KActionCollection *collec, bool _isRoot, bool _add,
-                              const QString & parentAddress )
+                              KActionCollection * actionCollection)
   : QObject(),
-    m_bIsRoot(_isRoot), m_bAddBookmark(_add),
-    m_bAddShortcuts(true),
+    m_bIsRoot(true),
     m_pManager(mgr), m_pOwner(_owner),
     m_parentMenu( _parentMenu ),
-    m_actionCollection( collec ),
-    m_parentAddress( parentAddress ),
-    m_rmb(0)
+    m_actionCollection( actionCollection ),
+    m_parentAddress( QString("") ) //TODO KBookmarkAdress::root
 {
   m_parentMenu->setKeyboardShortcutsEnabled( true );
 
+  //kDebug(7043) << "KBookmarkMenu::KBookmarkMenu " << this << " address : " << m_parentAddress << endl;
 
-  connect( m_parentMenu, SIGNAL( hovered( QAction * ) ),
-           this, SLOT( slotActionHoovered( QAction * ) ) );
+  connect( _parentMenu, SIGNAL( aboutToShow() ),
+            SLOT( slotAboutToShow() ) );
 
-  m_bNSBookmark = m_parentAddress.isNull();
-  if ( !m_bNSBookmark ) // not for the netscape bookmark
+  if ( KBookmarkSettings::self()->m_contextmenu )
   {
-    //kDebug(7043) << "KBookmarkMenu::KBookmarkMenu " << this << " address : " << m_parentAddress << endl;
-
-    connect( _parentMenu, SIGNAL( aboutToShow() ),
-             SLOT( slotAboutToShow() ) );
-
-    if ( KBookmarkSettings::self()->m_contextmenu )
-    {
-      m_parentMenu->setContextMenuPolicy(Qt::CustomContextMenu);
-      connect(m_parentMenu, SIGNAL(customContextMenuRequested(const QPoint &)), this, SLOT(contextMenu(const QPoint &)));
-    }
-
-    if ( m_bIsRoot )
-    {
-      connect( m_pManager, SIGNAL( changed(const QString &, const QString &) ),
-               SLOT( slotBookmarksChanged(const QString &) ) );
-    }
+    m_parentMenu->setContextMenuPolicy(Qt::CustomContextMenu);
+    connect(m_parentMenu, SIGNAL(customContextMenuRequested(const QPoint &)), this, SLOT(contextMenu(const QPoint &)));
   }
 
+  connect( m_pManager, SIGNAL( changed(const QString &, const QString &) ),
+              SLOT( slotBookmarksChanged(const QString &) ) );
+
+  m_bDirty = true;
+
   // add entries that possibly have a shortcut, so they are available _before_ first popup
+  addAddBookmark();
+  addAddBookmarksList();
+  addEditBookmarks();
+}
+
+void KBookmarkMenu::addActions()
+{
   if ( m_bIsRoot )
   {
-    if ( m_bAddBookmark )
-    {
-      addAddBookmark();
-      if ( extOwner() )
-        addAddBookmarksList(); // FIXME
-    }
-
+    addAddBookmark();
+    addAddBookmarksList();
     addEditBookmarks();
+    addNewFolder();
   }
+  else
+  {
+    if ( m_parentMenu->actions().count() > 0 )
+      m_parentMenu->addSeparator();
 
+    addAddBookmark();
+    addAddBookmarksList();
+    addNewFolder();
+  }
+}
+
+KBookmarkMenu::KBookmarkMenu( KBookmarkManager* mgr,
+                              KBookmarkOwner * _owner, KMenu * _parentMenu,
+                              const QString & parentAddress)
+  : QObject(),
+    m_bIsRoot(false),
+    m_pManager(mgr), m_pOwner(_owner),
+    m_parentMenu( _parentMenu ),
+    m_actionCollection( 0 ),
+    m_parentAddress( parentAddress )
+{
+  m_parentMenu->setKeyboardShortcutsEnabled( true );
+  connect( _parentMenu, SIGNAL( aboutToShow() ), SLOT( slotAboutToShow() ) );
+  if ( KBookmarkSettings::self()->m_contextmenu )
+  {
+    m_parentMenu->setContextMenuPolicy(Qt::CustomContextMenu);
+    connect(m_parentMenu, SIGNAL(customContextMenuRequested(const QPoint &)), this, SLOT(contextMenu(const QPoint &)));
+  }
   m_bDirty = true;
 }
 
 KBookmarkMenu::~KBookmarkMenu()
 {
-  //kDebug(7043) << "KBookmarkMenu::~KBookmarkMenu() " << this << endl;
   qDeleteAll( m_lstSubMenus );
   qDeleteAll( m_actions );
 }
@@ -148,113 +166,126 @@ void KBookmarkMenu::slotAboutToShow()
   if ( m_bDirty )
   {
     m_bDirty = false;
+    clear();
     refill();
+    m_parentMenu->adjustSize();
   }
 }
 
-QString KBookmarkMenu::s_highlightedImportType;
-QString KBookmarkMenu::s_highlightedImportLocation;
-
-void KBookmarkMenu::slotActionHoovered( QAction* action )
+void KBookmarkMenu::contextMenu( const QPoint & pos )
 {
-  if (qobject_cast<KBookmarkActionMenu*>(action) ||
-      qobject_cast<KBookmarkAction*>(action))
-  {
-    m_highlightedAddress = action->property("address").toString();
-  }
-  else if (qobject_cast<KImportedBookmarksActionMenu*>(action))
-  {
-    s_highlightedImportType = action->property("type").toString();
-    s_highlightedImportLocation = action->property("location").toString();
-  }
+  QAction * action = m_parentMenu->actionAt(pos);
+  KBookmarkActionInterface* act = dynamic_cast<KBookmarkActionInterface *>(action);
+  if (!act)
+      return;
+  act->contextMenu(m_parentMenu->mapToGlobal(pos), m_pManager, m_pOwner);
+}
+
+/********************************************************************/
+/********************************************************************/
+/********************************************************************/
+
+KBookmarkActionInterface::KBookmarkActionInterface(KBookmark bk)
+: bm(bk)
+{}
+
+KBookmarkActionInterface::~KBookmarkActionInterface()
+{
+}
+
+void KBookmarkActionInterface::contextMenu(QPoint, KBookmarkManager*, KBookmarkOwner*)
+{
+}
+
+const KBookmark KBookmarkActionInterface::bookmark() const
+{
+  return bm;
+}
+
+/********************************************************************/
+/********************************************************************/
+/********************************************************************/
+
+KBookmarkActionContextMenu & KBookmarkActionContextMenu::self()
+{
+  static KBookmarkActionContextMenu s;
+  return s;
+}
+
+void KBookmarkActionContextMenu::contextMenu(QPoint pos, QString highlightedAddress,
+	KBookmarkManager *pManager, KBookmarkOwner *pOwner)
+{
+  m_parentAddress = KBookmark::parentAddress(highlightedAddress) ;
+  m_highlightedAddress = highlightedAddress;
+  m_pManager = pManager;
+  m_pOwner = pOwner;
+  delete m_contextMenu;
+  m_contextMenu = new QMenu();
+
+  KBookmark bookmark = atAddress(m_highlightedAddress);
+  addBookmark();
+
+  if (bookmark.isGroup())
+    addFolderActions();
   else
-  {
-    m_highlightedAddress.clear();
-    s_highlightedImportType.clear();
-    s_highlightedImportLocation.clear();
-  }
+    addBookmarkActions();
+
+  addProperties();
+
+  if(!m_highlightedAddress.isNull())
+    m_contextMenu->popup(pos);
 }
 
-/********************************************************************/
-/********************************************************************/
-/********************************************************************/
-
-RMB::RMB(QString parentAddress, QString highlightedAddress,
-         KBookmarkManager *pManager, KBookmarkOwner *pOwner, QWidget *parentMenu)
-: m_parentAddress(parentAddress), m_highlightedAddress(highlightedAddress),
-  m_pManager(pManager), m_pOwner(pOwner), m_parentMenu(parentMenu)
+void KBookmarkActionContextMenu::addBookmark()
 {
-    m_contextMenu = new QMenu();
+  if( m_pOwner && m_pOwner->addBookmarkEntry() )
+    m_contextMenu->addAction( SmallIcon("bookmark_add"), i18n( "Add Bookmark Here" ), this, SLOT(slotInsert()) );
 }
 
-RMB::~RMB()
+void KBookmarkActionContextMenu::addFolderActions()
+{
+  m_contextMenu->addAction( i18n( "Open Folder in Bookmark Editor" ), this, SLOT(slotEditAt()) );
+  m_contextMenu->addSeparator();
+  m_contextMenu->addAction( SmallIcon("editdelete"), i18n( "Delete Folder" ), this, SLOT(slotRemove()) );
+  m_contextMenu->addSeparator();
+}
+
+
+void KBookmarkActionContextMenu::addProperties()
+{
+  m_contextMenu->addAction( i18n( "Properties" ), this, SLOT(slotProperties()) );
+}
+
+void KBookmarkActionContextMenu::addBookmarkActions()
+{
+  m_contextMenu->addAction( i18n( "Copy Link Address" ), this, SLOT(slotCopyLocation()) );
+  m_contextMenu->addSeparator();
+  m_contextMenu->addAction( SmallIcon("editdelete"), i18n( "Delete Bookmark" ), this, SLOT(slotRemove()) );
+  m_contextMenu->addSeparator();
+}
+
+KBookmarkActionContextMenu::~KBookmarkActionContextMenu()
 {
     delete m_contextMenu;
 }
 
-void RMB::popup(const QPoint & pos)
-{
-    if(!m_highlightedAddress.isNull())
-        m_contextMenu->popup(pos);
-}
-
-QMenu * RMB::contextMenu()
-{
-    return m_contextMenu;
-}
-
-KBookmark RMB::atAddress(const QString & address)
+KBookmark KBookmarkActionContextMenu::atAddress(const QString & address)
 {
   KBookmark bookmark = m_pManager->findByAddress( address );
   Q_ASSERT(!bookmark.isNull());
   return bookmark;
 }
 
-void KBookmarkMenu::contextMenu( const QPoint & pos )
+void KBookmarkActionContextMenu::slotEditAt()
 {
-  showContextMenu(m_highlightedAddress, pos);
-}
-
-void RMB::fillContextMenu( const QString & address)
-{
-  m_contextMenu->clear();
-  KBookmark bookmark = atAddress(address);
-  m_contextMenu->addAction( SmallIcon("bookmark_add"), i18n( "Add Bookmark Here" ), this, SLOT(slotInsert()) );
-}
-
-void RMB::fillContextMenu2( const QString & address)
-{
-  KBookmark bookmark = atAddress(address);
-
-  if (bookmark.isGroup()) {
-    m_contextMenu->addAction( i18n( "Open Folder in Bookmark Editor" ), this, SLOT(slotEditAt()) );
-    m_contextMenu->addSeparator();
-    m_contextMenu->addAction( SmallIcon("editdelete"), i18n( "Delete Folder" ), this, SLOT(slotRemove()) );
-    m_contextMenu->addSeparator();
-    m_contextMenu->addAction( i18n( "Properties" ), this, SLOT(slotProperties()) );
-  }
-  else
-  {
-    m_contextMenu->addAction( i18n( "Copy Link Address" ), this, SLOT(slotCopyLocation()) );
-    m_contextMenu->addSeparator();
-    m_contextMenu->addAction( SmallIcon("editdelete"), i18n( "Delete Bookmark" ), this, SLOT(slotRemove()) );
-    m_contextMenu->addSeparator();
-    m_contextMenu->addAction( i18n( "Properties" ), this, SLOT(slotProperties()) );
-  }
-}
-
-void RMB::slotEditAt()
-{
-  kDebug(7043) << "KBookmarkMenu::slotEditAt" << m_highlightedAddress << endl;
-
+  //kDebug(7043) << "KBookmarkMenu::slotEditAt" << m_highlightedAddress << endl;
   KBookmark bookmark = atAddress(m_highlightedAddress);
-
   m_pManager->slotEditBookmarksAtAddress( m_highlightedAddress );
 }
 
-void RMB::slotProperties()
+void KBookmarkActionContextMenu::slotProperties()
 {
-  kDebug(7043) << "KBookmarkMenu::slotProperties" << m_highlightedAddress << endl;
+  //kDebug(7043) << "KBookmarkMenu::slotProperties" << m_highlightedAddress << endl;
 
   KBookmark bookmark = atAddress(m_highlightedAddress);
 
@@ -276,11 +307,11 @@ void RMB::slotProperties()
   m_pManager->emitChanged( parentBookmark );
 }
 
-void RMB::slotInsert()
+void KBookmarkActionContextMenu::slotInsert()
 {
-  kDebug(7043) << "KBookmarkMenu::slotInsert" << m_highlightedAddress << endl;
+  //kDebug(7043) << "KBookmarkMenu::slotInsert" << m_highlightedAddress << endl;
 
-  QString url = m_pOwner->currentURL();
+  QString url = m_pOwner->currentUrl();
   if (url.isEmpty())
   {
     KMessageBox::error( 0L, i18n("Cannot add bookmark with empty URL."));
@@ -311,7 +342,7 @@ void RMB::slotInsert()
   }
 }
 
-void RMB::slotRemove()
+void KBookmarkActionContextMenu::slotRemove()
 {
   //kDebug(7043) << "KBookmarkMenu::slotRemove" << m_highlightedAddress << endl;
 
@@ -324,7 +355,7 @@ void RMB::slotRemove()
                  : i18n("Are you sure you wish to remove the bookmark\n\"%1\"?", bookmark.text()),
           folder ? i18n("Bookmark Folder Deletion")
                  : i18n("Bookmark Deletion"),
-          KStdGuiItem::del())
+          KStandardGuiItem::del())
         != KMessageBox::Continue
      )
     return;
@@ -336,7 +367,7 @@ void RMB::slotRemove()
     m_parentMenu->hide();
 }
 
-void RMB::slotCopyLocation()
+void KBookmarkActionContextMenu::slotCopyLocation()
 {
   //kDebug(7043) << "KBookmarkMenu::slotCopyLocation" << m_highlightedAddress << endl;
 
@@ -356,23 +387,9 @@ void RMB::slotCopyLocation()
 /********************************************************************/
 /********************************************************************/
 /********************************************************************/
-
-void KBookmarkMenu::showContextMenu( const QString & address, const QPoint & pos)
-{
-
-  delete m_rmb;
-  m_rmb = new RMB(m_parentAddress, m_highlightedAddress, m_pManager, m_pOwner, m_parentMenu);
-  m_rmb->fillContextMenu(address);
-  emit aboutToShowContextMenu( m_rmb->atAddress(address), m_rmb->contextMenu());
-  m_rmb->fillContextMenu2(address);
-  m_rmb->popup( m_parentMenu->mapToGlobal(pos) );
-}
-
 void KBookmarkMenu::slotBookmarksChanged( const QString & groupAddress )
 {
-  if (m_bNSBookmark)
-    return;
-
+  kDebug(7043)<<"KBookmarkMenu::slotBookmarksChanged( "<<groupAddress<<endl;
   if ( groupAddress == m_parentAddress )
   {
     //kDebug(7043) << "KBookmarkMenu::slotBookmarksChanged -> setting m_bDirty on " << groupAddress << endl;
@@ -381,7 +398,6 @@ void KBookmarkMenu::slotBookmarksChanged( const QString & groupAddress )
   else
   {
     // Iterate recursively into child menus
-
     for ( QList<KBookmarkMenu *>::iterator it = m_lstSubMenus.begin(), end = m_lstSubMenus.end() ;
           it != end ; ++it ) {
       (*it)->slotBookmarksChanged( groupAddress );
@@ -389,13 +405,12 @@ void KBookmarkMenu::slotBookmarksChanged( const QString & groupAddress )
   }
 }
 
-void KBookmarkMenu::refill()
+void KBookmarkMenu::clear()
 {
-  //kDebug(7043) << "KBookmarkMenu::refill()" << endl;
   qDeleteAll( m_lstSubMenus );
   m_lstSubMenus.clear();
 
-  for ( QList<KAction *>::iterator it = m_actions.begin(), end = m_actions.end() ;
+  for ( QList<QAction *>::iterator it = m_actions.begin(), end = m_actions.end() ;
         it != end ; ++it )
   {
         m_parentMenu->removeAction(*it);
@@ -403,19 +418,27 @@ void KBookmarkMenu::refill()
 
   m_parentMenu->clear();
   m_actions.clear();
+}
 
-  fillBookmarkMenu();
-  m_parentMenu->adjustSize();
+void KBookmarkMenu::refill()
+{
+  //kDebug(7043) << "KBookmarkMenu::refill()" << endl;
+  if(m_bIsRoot)
+    addActions();
+  fillBookmarks();
+  if(!m_bIsRoot)
+    addActions();
 }
 
 void KBookmarkMenu::addAddBookmarksList()
 {
-  if (!KAuthorized::authorizeKAction("bookmarks"))
-     return;
+  if( !m_pOwner || !m_pOwner->addBookmarkEntry() || !m_pOwner->supportsTabs() || !KAuthorized::authorizeKAction("bookmarks") )
+    return;
 
   QString title = i18n( "Bookmark Tabs as Folder..." );
 
-  KAction * paAddBookmarksList = new KAction( title, m_actionCollection, m_bIsRoot ? "add_bookmarks_list" : 0 );
+  KAction * paAddBookmarksList = new KAction( title, this );
+  m_actionCollection->addAction( m_bIsRoot ? "add_bookmarks_list" : 0, paAddBookmarksList );
   paAddBookmarksList->setIcon( KIcon( "bookmarks_list_add" ) );
   paAddBookmarksList->setToolTip( i18n( "Add a folder of bookmarks for all open tabs." ) );
   connect( paAddBookmarksList, SIGNAL( triggered( bool ) ), this, SLOT( slotAddBookmarksList() ) );
@@ -426,14 +449,15 @@ void KBookmarkMenu::addAddBookmarksList()
 
 void KBookmarkMenu::addAddBookmark()
 {
-  if (!KAuthorized::authorizeKAction("bookmarks"))
-     return;
+  if( !m_pOwner || !m_pOwner->addBookmarkEntry() || !KAuthorized::authorizeKAction("bookmarks") )
+    return;
 
   QString title = i18n( "Add Bookmark" );
 
-  KAction * paAddBookmarks = new KAction( title, m_actionCollection, m_bIsRoot ? "add_bookmark" : 0 );
+  KAction * paAddBookmarks = new KAction( title, this );
+  m_actionCollection->addAction( m_bIsRoot ? "add_bookmark" : 0, paAddBookmarks );
   paAddBookmarks->setIcon( KIcon( "bookmark_add" ) );
-  paAddBookmarks->setShortcut( m_bIsRoot && m_bAddShortcuts ? KStdAccel::addBookmark() : KShortcut() );
+  paAddBookmarks->setShortcut( m_bIsRoot ? KStandardShortcut::addBookmark() : KShortcut() );
   paAddBookmarks->setToolTip( i18n( "Add a bookmark for the current document" ) );
   connect( paAddBookmarks, SIGNAL( triggered( bool ) ), this, SLOT( slotAddBookmark() ) );
 
@@ -443,11 +467,11 @@ void KBookmarkMenu::addAddBookmark()
 
 void KBookmarkMenu::addEditBookmarks()
 {
-  if (!KAuthorized::authorizeKAction("bookmarks"))
-     return;
+  if( m_pOwner && !m_pOwner->editBookmarkEntry() || !KAuthorized::authorizeKAction("bookmarks") )
+    return;
 
-  KAction * m_paEditBookmarks = KStdAction::editBookmarks( m_pManager, SLOT( slotEditBookmarks() ),
-                                                             m_actionCollection, "edit_bookmarks" );
+  QAction * m_paEditBookmarks = m_actionCollection->addAction( KStandardAction::EditBookmarks, "edit_bookmarks",
+                                                               m_pManager, SLOT( slotEditBookmarks() ) );
   m_parentMenu->addAction(m_paEditBookmarks);
   m_paEditBookmarks->setToolTip( i18n( "Edit your bookmark collection in a separate window" ) );
   m_actions.append( m_paEditBookmarks );
@@ -455,13 +479,14 @@ void KBookmarkMenu::addEditBookmarks()
 
 void KBookmarkMenu::addNewFolder()
 {
-  if (!KAuthorized::authorizeKAction("bookmarks"))
-     return;
+  if( !m_pOwner || !m_pOwner->addBookmarkEntry() || !KAuthorized::authorizeKAction("bookmarks"))
+    return;
 
   QString title = i18n( "&New Bookmark Folder..." );
   title.remove( QChar( '&' ) ); //FIXME Hmm, why?
 
-  KAction * paNewFolder = new KAction( title, m_actionCollection,"dummyname" );
+  KAction * paNewFolder = new KAction( title, this );
+  m_actionCollection->addAction( "dummyname", paNewFolder );
   paNewFolder->setIcon( KIcon( "folder_new" ) );
   paNewFolder->setToolTip( i18n( "Create a new bookmark folder in this menu" ) );
   connect( paNewFolder, SIGNAL( triggered( bool ) ), this, SLOT( slotNewFolder() ) );
@@ -470,148 +495,57 @@ void KBookmarkMenu::addNewFolder()
   m_actions.append( paNewFolder );
 }
 
-void KBookmarkMenu::fillBookmarkMenu()
+void KBookmarkMenu::fillBookmarks()
 {
-  if (!KAuthorized::authorizeKAction("bookmarks"))
-     return;
-
-  if ( m_bIsRoot )
-  {
-    if ( m_bAddBookmark )
-    {
-      addAddBookmark();
-      if ( extOwner() )
-        addAddBookmarksList(); // FIXME
-    }
-
-    addEditBookmarks();
-
-    if ( m_bAddBookmark && !KBookmarkSettings::self()->m_advancedaddbookmark )
-      addNewFolder();
-  }
-
-  if ( m_bIsRoot
-    && KBookmarkManager::userBookmarksManager()->path() == m_pManager->path() )
-  {
-    bool haveSep = false;
-
-    const QStringList keys = KBookmarkMenu::dynamicBookmarksList();
-    for ( QStringList::const_iterator it = keys.begin(); it != keys.end(); ++it )
-    {
-       DynMenuInfo info;
-       info = showDynamicBookmarks((*it));
-
-       if ( !info.show || !QFile::exists( info.location ) )
-          continue;
-
-       if (!haveSep)
-       {
-          m_parentMenu->addSeparator();
-          haveSep = true;
-       }
-
-       KActionMenu * actionMenu;
-       actionMenu = new KImportedBookmarksActionMenu(
-                              KIcon(info.type), info.name,
-                              m_actionCollection, "kbookmarkmenu" );
-
-       actionMenu->setProperty( "type", info.type );
-       actionMenu->setProperty( "location", info.location );
-
-       m_parentMenu->addAction(actionMenu);
-       m_actions.append( actionMenu );
-
-       KBookmarkMenu *subMenu =
-          new KBookmarkMenu( m_pManager, m_pOwner, actionMenu->menu(),
-                             m_actionCollection, false,
-                             m_bAddBookmark, QString() );
-       connect( subMenu, SIGNAL( openBookmark( const QString &, Qt::MouseButtons, Qt::KeyboardModifiers ) ),
-                this, SIGNAL( openBookmark( const QString &, Qt::MouseButtons, Qt::KeyboardModifiers ) ));
-       m_lstSubMenus.append( subMenu );
-
-       connect(actionMenu->menu(), SIGNAL(aboutToShow()), subMenu, SLOT(slotNSLoad()));
-    }
-  }
-
   KBookmarkGroup parentBookmark = m_pManager->findByAddress( m_parentAddress ).toGroup();
   Q_ASSERT(!parentBookmark.isNull());
-  bool separatorInserted = false;
-  for ( KBookmark bm = parentBookmark.first(); !bm.isNull();  bm = parentBookmark.next(bm) )
+
+  if ( m_bIsRoot && !parentBookmark.first().isNull() ) // at least one bookmark
   {
-    QString text = KStringHandler::csqueeze(bm.fullText(), 60);
-    text.replace( '&', "&&" );
-    if ( !separatorInserted && m_bIsRoot) {
-      // inserted before the first konq bookmark, to avoid the separator if no konq bookmark
       m_parentMenu->addSeparator();
-      separatorInserted = true;
-    }
-    if ( !bm.isGroup() )
-    {
-      if ( bm.isSeparator() )
-      {
-        m_parentMenu->addSeparator();
-      }
-      else
-      {
-        //kDebug(7043) << "Creating URL bookmark menu item for " << bm.text() << endl;
-        KAction * action = new KBookmarkAction( text, bm.icon(), 0, m_actionCollection, 0 );
-        connect(action, SIGNAL( triggered(Qt::MouseButtons, Qt::KeyboardModifiers) ), SLOT( slotBookmarkSelected(Qt::MouseButtons, Qt::KeyboardModifiers) ));
-
-        action->setProperty( "url", bm.url().url() );
-        action->setProperty( "address", bm.address() );
-
-        action->setToolTip( bm.url().pathOrUrl() );
-
-        m_parentMenu->addAction(action);
-        m_actions.append( action );
-      }
-    }
-    else
-    {
-      //kDebug(7043) << "Creating bookmark submenu named " << bm.text() << endl;
-      KActionMenu * actionMenu = new KBookmarkActionMenu( KIcon(bm.icon()), text,
-                                                          m_actionCollection,
-                                                          "kbookmarkmenu" );
-      actionMenu->setProperty( "address", bm.address() );
-      m_parentMenu->addAction(actionMenu);
-      m_actions.append( actionMenu );
-
-      KBookmarkMenu *subMenu = new KBookmarkMenu( m_pManager, m_pOwner, actionMenu->menu(),
-                                                  m_actionCollection, false,
-                                                  m_bAddBookmark,
-                                                  bm.address() );
-
-      connect(subMenu, SIGNAL( aboutToShowContextMenu( const KBookmark &, QMenu * ) ),
-                 this, SIGNAL( aboutToShowContextMenu( const KBookmark &, QMenu * ) ));
-      connect(subMenu, SIGNAL( openBookmark( const QString &, Qt::MouseButtons, Qt::KeyboardModifiers ) ),
-                this, SIGNAL( openBookmark( const QString &, Qt::MouseButtons, Qt::KeyboardModifiers ) ));
-      m_lstSubMenus.append( subMenu );
-    }
   }
 
-  if ( !m_bIsRoot && m_bAddBookmark )
+  for ( KBookmark bm = parentBookmark.first(); !bm.isNull();  bm = parentBookmark.next(bm) )
   {
-    if ( m_parentMenu->actions().count() > 0 )
-      m_parentMenu->addSeparator();
+    m_parentMenu->addAction(actionForBookmark(bm));
+  }
+}
 
-    addAddBookmark();
-    if ( extOwner() )
-      addAddBookmarksList(); // FIXME
-    addNewFolder();
+QAction* KBookmarkMenu::actionForBookmark(KBookmark bm)
+{
+  if ( bm.isGroup() )
+  {
+    //kDebug(7043) << "Creating bookmark submenu named " << bm.text() << endl;
+    KActionMenu * actionMenu = new KBookmarkActionMenu( bm, this );
+    m_actionCollection->addAction( "kbookmarkmenu", actionMenu );
+    m_actions.append( actionMenu );
+    KBookmarkMenu *subMenu = new KBookmarkMenu( m_pManager, m_pOwner, actionMenu->menu(), bm.address() );
+    m_lstSubMenus.append( subMenu );
+    return actionMenu;
+  }
+  else if ( bm.isSeparator() )
+  {
+    QAction *sa = new QAction(this);
+    sa->setSeparator(true);
+    m_actions.append(sa);
+    return sa;
+  }
+  else
+  {
+    //kDebug(7043) << "Creating bookmark menu item for " << bm.text() << endl;
+    KAction * action = new KBookmarkAction( bm, m_pOwner, this );
+    m_actionCollection->addAction(action->objectName(), action);
+    m_actions.append( action );
+    return action;
   }
 }
 
 void KBookmarkMenu::slotAddBookmarksList()
 {
-  KExtendedBookmarkOwner *extOwner = dynamic_cast<KExtendedBookmarkOwner*>(m_pOwner);
-  if (!extOwner)
-  {
-    kWarning() << "erm, sorry ;-)" << endl;
+  if( !m_pOwner || !m_pOwner->supportsTabs())
     return;
-  }
 
-  KExtendedBookmarkOwner::QStringPairList list;
-  extOwner->fillBookmarksList( list );
+  const QList<QPair<QString, QString> > & list = m_pOwner->currentBookmarkList();
 
   KBookmarkGroup parentBookmark = m_pManager->findByAddress( m_parentAddress ).toGroup();
   Q_ASSERT(!parentBookmark.isNull());
@@ -619,7 +553,7 @@ void KBookmarkMenu::slotAddBookmarksList()
   if ( group.isNull() )
     return; // user canceled i guess
 
-  KExtendedBookmarkOwner::QStringPairList::const_iterator it;
+  QList<QPair<QString, QString> >::const_iterator it;
   for ( it = list.begin(); it != list.end(); ++it )
     group.addBookmark( m_pManager, (*it).first, KUrl((*it).second) );
 
@@ -629,10 +563,13 @@ void KBookmarkMenu::slotAddBookmarksList()
 
 void KBookmarkMenu::slotAddBookmark()
 {
+  if( !m_pOwner ) return;
   KBookmarkGroup parentBookmark;
-  parentBookmark = m_pManager->addBookmarkDialog(m_pOwner->currentURL(), m_pOwner->currentTitle(), m_parentAddress);
+  parentBookmark = m_pManager->addBookmarkDialog(m_pOwner->currentUrl(), m_pOwner->currentTitle(), m_parentAddress);
   if (!parentBookmark.isNull())
+  {
     m_pManager->emitChanged( parentBookmark );
+  }
 }
 
 void KBookmarkMenu::slotNewFolder()
@@ -648,32 +585,45 @@ void KBookmarkMenu::slotNewFolder()
   }
 }
 
-void KBookmarkMenu::slotBookmarkSelected(Qt::MouseButtons mb, Qt::KeyboardModifiers km)
-{
-  kDebug(7043) << "KBookmarkMenu::slotBookmarkSelected()" << endl;
-  if ( !m_pOwner ) return; // this view doesn't handle bookmarks...
-
-  if (qobject_cast<const KAction*>(sender()))
-  {
-      const QString& url = sender()->property("url").toString();
-      m_pOwner->openBookmarkURL( url );
-      emit openBookmark( url, mb, km );
-  }
-}
-
-KExtendedBookmarkOwner* KBookmarkMenu::extOwner()
-{
-  return dynamic_cast<KExtendedBookmarkOwner*>(m_pOwner);
-}
-
-void KBookmarkMenu::slotNSLoad()
+void KImportedBookmarkMenu::slotNSLoad()
 {
+  kDebug(7043)<<"**** slotNSLoad  ****"<<m_type<<"  "<<m_location<<endl;
   // only fill menu once
   m_parentMenu->disconnect(SIGNAL(aboutToShow()));
 
   // not NSImporter, but kept old name for BC reasons
   KBookmarkMenuNSImporter importer( m_pManager, this, m_actionCollection );
-  importer.openBookmarks(s_highlightedImportLocation, s_highlightedImportType);
+  importer.openBookmarks(m_location, m_type);
+}
+
+KImportedBookmarkMenu::KImportedBookmarkMenu( KBookmarkManager* mgr,
+                 KBookmarkOwner * owner, KMenu * parentMenu,
+                 const QString & type, const QString & location )
+    :KBookmarkMenu(mgr, owner, parentMenu, QString()), m_type(type), m_location(location)
+{
+    connect(parentMenu, SIGNAL(aboutToShow()), this, SLOT(slotNSLoad()));
+}
+
+KImportedBookmarkMenu::KImportedBookmarkMenu( KBookmarkManager* mgr,
+                 KBookmarkOwner * owner, KMenu * parentMenu)
+    :KBookmarkMenu(mgr, owner, parentMenu, QString()), m_type(QString()), m_location(QString())
+{
+
+}
+
+KImportedBookmarkMenu::~KImportedBookmarkMenu()
+{
+
+}
+
+void KImportedBookmarkMenu::refill()
+{
+
+}
+
+void KImportedBookmarkMenu::clear()
+{
+
 }
 
 /********************************************************************/
@@ -766,7 +716,7 @@ KBookmarkEditDialog::KBookmarkEditDialog(const QString& title, const QString& ur
     setButtonGuiItem( KDialog::Ok,  KGuiItem( i18n( "&Add" ), "bookmark_add") );
   } else {
     setButtons( Ok | Cancel );
-    setButtonGuiItem( KDialog::Ok, i18n( "&Update" ) );
+    setButtonGuiItem( KDialog::Ok, KGuiItem(i18n( "&Update" )) );
   }
 
   setDefaultButton( KDialog::Ok );
@@ -810,6 +760,8 @@ KBookmarkEditDialog::KBookmarkEditDialog(const QString& title, const QString& ur
              this,         SLOT( slotDoubleClicked(QTreeWidgetItem *) ) );
     connect( this, SIGNAL( user1Clicked() ), SLOT( slotUser1() ) );
   }
+  connect(this,SIGNAL(okClicked()),this,SLOT(slotOk()));
+  connect(this,SIGNAL(cancelClicked()),this,SLOT(slotCancel()));
 }
 
 void KBookmarkEditDialog::slotDoubleClicked( QTreeWidgetItem * item )
@@ -922,29 +874,21 @@ void KBookmarkMenuNSImporter::connectToImporter(const QObject &importer)
 
 void KBookmarkMenuNSImporter::newBookmark( const QString & text, const QString & url, const QString & )
 {
-  QString _text = KStringHandler::csqueeze(text);
-  _text.replace( '&', "&&" );
-  KAction * action = new KBookmarkAction(_text, "html",0,m_actionCollection, 0);
-  connect(action, SIGNAL( triggered(Qt::MouseButtons, Qt::KeyboardModifiers) ),
-          m_menu, SLOT( slotBookmarkSelected(Qt::MouseButtons, Qt::KeyboardModifiers) ));
-  action->setProperty( "url", url );
-  action->setToolTip( url );
+  KBookmark bm = KBookmark::standaloneBookmark(text, url, QString("html"));
+  KAction * action = new KImportedBookmarkAction(bm, mstack.top()->m_pOwner, this);
+  m_actionCollection->addAction(action->objectName(), action);
   mstack.top()->m_parentMenu->addAction(action);
   mstack.top()->m_actions.append( action );
 }
 
 void KBookmarkMenuNSImporter::newFolder( const QString & text, bool, const QString & )
 {
-  QString _text = KStringHandler::csqueeze(text);
-  _text.replace( '&', "&&" );
-  KActionMenu * actionMenu = new KActionMenu( KIcon("folder"), _text, m_actionCollection, 0L );
+  QString _text = KStringHandler::csqueeze(text).replace( '&', "&&" );
+  KActionMenu * actionMenu = new KImportedBookmarkActionMenu( KIcon("folder"), _text, this );
+  m_actionCollection->addAction( actionMenu->objectName(), actionMenu );
   mstack.top()->m_parentMenu->addAction(actionMenu);
   mstack.top()->m_actions.append( actionMenu );
-  KBookmarkMenu *subMenu = new KBookmarkMenu( m_pManager, m_menu->m_pOwner, actionMenu->menu(),
-                                              m_actionCollection, false,
-                                              m_menu->m_bAddBookmark, QString() );
-  connect( subMenu, SIGNAL( openBookmark( const QString &, Qt::MouseButtons, Qt::KeyboardModifiers ) ),
-           m_menu, SIGNAL( openBookmark( const QString &, Qt::MouseButtons, Qt::KeyboardModifiers ) ));
+  KImportedBookmarkMenu *subMenu = new KImportedBookmarkMenu( m_pManager, m_menu->m_pOwner, actionMenu->menu());
   mstack.top()->m_lstSubMenus.append( subMenu );
 
   mstack.push(subMenu);
@@ -964,90 +908,81 @@ void KBookmarkMenuNSImporter::endFolder()
 /********************************************************************/
 /********************************************************************/
 
-KBookmarkMenu::DynMenuInfo KBookmarkMenu::showDynamicBookmarks( const QString &id )
+
+KBookmarkAction::KBookmarkAction(KBookmark bk, KBookmarkOwner* owner, QObject *parent )
+  : KAction( KStringHandler::csqueeze(bk.text()).replace('&', "&&"), parent),
+    KBookmarkActionInterface(bk),
+    m_pOwner(owner)
 {
-  KConfig config("kbookmarkrc", false, false);
-  config.setGroup("Bookmarks");
-
-  DynMenuInfo info;
-  info.show = false;
-
-  if (!config.hasKey("DynamicMenus")) {
-    // upgrade path
-    if (id == "netscape") {
-      KBookmarkManager *manager = KBookmarkManager::userBookmarksManager();
-      info.show = manager->root().internalElement().attribute("hide_nsbk") != "yes";
-      info.location = KNSBookmarkImporter::netscapeBookmarksFile();
-      info.type = "netscape";
-      info.name = i18n("Netscape Bookmarks");
-    } // else, no show
-
-  } else {
-    // have new version config
-    if (config.hasGroup("DynamicMenu-" + id)) {
-      config.setGroup("DynamicMenu-" + id);
-      info.show = config.readEntry("Show", false);
-      info.location = config.readPathEntry("Location");
-      info.type = config.readEntry("Type");
-      info.name = config.readEntry("Name");
-    } // else, no show
-  }
-
-  return info;
+  setIcon(KIcon(bookmark().icon()));
+  setToolTip( bookmark().url().pathOrUrl() );
+  connect(this, SIGNAL( triggered(Qt::MouseButtons, Qt::KeyboardModifiers) ),
+     SLOT( slotSelected(Qt::MouseButtons, Qt::KeyboardModifiers) ));
 }
 
-QStringList KBookmarkMenu::dynamicBookmarksList()
+KBookmarkAction::~KBookmarkAction()
 {
-  KConfig config("kbookmarkrc", false, false);
-  config.setGroup("Bookmarks");
+}
 
-  QStringList mlist;
-  if (config.hasKey("DynamicMenus"))
-    mlist = config.readEntry("DynamicMenus", QStringList());
+void KBookmarkAction::contextMenu(QPoint pos, KBookmarkManager* m_pManager, KBookmarkOwner* m_pOwner)
+{
+  KBookmarkActionContextMenu::self().contextMenu(pos, bookmark().address(), m_pManager, m_pOwner);
+}
+
+void KBookmarkAction::slotSelected(Qt::MouseButtons mb, Qt::KeyboardModifiers km)
+{
+  if( !m_pOwner )
+    new KRun( bookmark().url() ,(QWidget*)0);
   else
-    mlist << "netscape";
-
-  return mlist;
+    m_pOwner->openBookmark( bookmark(), mb, km );
 }
 
-void KBookmarkMenu::setDynamicBookmarks(const QString &id, const DynMenuInfo &newMenu)
+KBookmarkActionMenu::KBookmarkActionMenu(KBookmark bm, QObject *parent)
+  : KActionMenu(KIcon(bm.icon()), KStringHandler::csqueeze(bm.text()).replace('&', "&&"), parent),
+    KBookmarkActionInterface(bm)
 {
-  KConfig config("kbookmarkrc", false, false);
+}
 
-  // add group unconditionally
-  config.setGroup("DynamicMenu-" + id);
-  config.writeEntry("Show", newMenu.show);
-  config.writePathEntry("Location", newMenu.location);
-  config.writeEntry("Type", newMenu.type);
-  config.writeEntry("Name", newMenu.name);
+KBookmarkActionMenu::KBookmarkActionMenu(KBookmark bm, const QString & text, QObject *parent)
+  : KActionMenu(text, parent),
+    KBookmarkActionInterface(bm)
+{
+}
 
-  QStringList elist;
+KBookmarkActionMenu::~KBookmarkActionMenu()
+{
+}
 
-  config.setGroup("Bookmarks");
-  if (!config.hasKey("DynamicMenus")) {
-    if (newMenu.type != "netscape") {
-      // update from old xbel method to new rc method
-      // though only if not writing the netscape setting
-      config.setGroup("DynamicMenu-" "netscape");
-      DynMenuInfo xbelSetting;
-      xbelSetting = showDynamicBookmarks("netscape");
-      config.writeEntry("Show", xbelSetting.show);
-      config.writePathEntry("Location", xbelSetting.location);
-      config.writeEntry("Type", xbelSetting.type);
-      config.writeEntry("Name", xbelSetting.name);
-    }
-  } else {
-    elist = config.readEntry("DynamicMenus", QStringList());
-  }
+void KBookmarkActionMenu::contextMenu(QPoint pos, KBookmarkManager* m_pManager, KBookmarkOwner* m_pOwner)
+{
+  KBookmarkActionContextMenu::self().contextMenu(pos, bookmark().address(), m_pManager, m_pOwner );
+}
 
-  // make sure list includes type
-  config.setGroup("Bookmarks");
-  if (!elist.contains(id)) {
-    elist << id;
-    config.writeEntry("DynamicMenus", elist);
-  }
+KImportedBookmarkAction::KImportedBookmarkAction(KBookmark bk, KBookmarkOwner* owner, QObject *parent )
+: KAction( KStringHandler::csqueeze(bk.text()).replace('&', "&&"), parent),
+  KBookmarkActionInterface(bk),
+  m_pOwner(owner)
+{
+  setIcon(KIcon(bookmark().icon()));
+  setToolTip( bookmark().url().pathOrUrl() );
+  connect(this, SIGNAL( triggered(Qt::MouseButtons, Qt::KeyboardModifiers) ),
+          SLOT( slotSelected(Qt::MouseButtons, Qt::KeyboardModifiers) ));
+}
 
-  config.sync();
+KImportedBookmarkAction::~KImportedBookmarkAction()
+{
+}
+
+void KImportedBookmarkAction::contextMenu(QPoint pos, KBookmarkManager* m_pManager, KBookmarkOwner* m_pOwner)
+{
+}
+
+void KImportedBookmarkAction::slotSelected(Qt::MouseButtons mb, Qt::KeyboardModifiers km)
+{
+  if( !m_pOwner )
+    new KRun( bookmark().url() ,(QWidget*)0);
+  else
+    m_pOwner->openBookmark( bookmark(), mb, km );
 }
 
 #include "kbookmarkmenu.moc"
