@@ -40,21 +40,23 @@
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kmenu.h>
-#include <kstdaction.h>
+#include <kstandardaction.h>
 #include <kio/job.h>
+#include <kio/deletejob.h>
+#include <kio/copyjob.h>
 #include <kio/jobuidelegate.h>
 #include <kio/jobclasses.h>
 #include <kio/netaccess.h>
 #include <kio/previewjob.h>
-#include <kio/renamedlg.h>
+#include <kio/renamedialog.h>
 #include <kpropertiesdialog.h>
 #include <kmimetypefactory.h>
-#include <kstdaccel.h>
+#include <kstandardshortcut.h>
 #include <kde_file.h>
 #include <kactioncollection.h>
 #include <ktoggleaction.h>
-#include <kseparatoraction.h>
 #include <kactionmenu.h>
+#include <kconfiggroup.h>
 
 #include "config-kfile.h"
 #include "kcombiview.h"
@@ -78,7 +80,8 @@ public:
         dirHighlighting = false;
         dropOptions = 0;
         configGroup=0;
-        viewActionSeparator = new KSeparatorAction();
+        viewActionSeparator = new QAction(0);
+        viewActionSeparator->setSeparator(true);
     }
 
     ~KDirOperatorPrivate() {
@@ -90,7 +93,7 @@ public:
     QString lastURL; // used for highlighting a directory on cdUp
     bool onlyDoubleClickSelectsFiles;
     QTimer *progressDelayTimer;
-    KSeparatorAction *viewActionSeparator;
+    QAction *viewActionSeparator;
     int dropOptions;
 
     KConfigGroup *configGroup;
@@ -122,7 +125,7 @@ KDirOperator::KDirOperator(const KUrl& _url, QWidget *parent)
         currUrl.addPath("/"); // make sure we have a trailing slash!
     }
 
-    setDirLister( new KDirLister( true ) );
+    setDirLister( new KDirLister() );
 
     connect(&myCompletion, SIGNAL(match(const QString&)),
             SLOT(slotCompletionMatch(const QString&)));
@@ -223,7 +226,7 @@ void KDirOperator::insertViewDependentActions()
          // first insert the normal actions, then the grouped ones
          QList<QActionGroup*> groups = viewActionCollection->actionGroups();
 
-         foreach (KAction* action, viewActionCollection->actionsWithoutGroup())
+         foreach (QAction* action, viewActionCollection->actionsWithoutGroup())
             viewActionMenu->addAction( action );
 
          foreach (QActionGroup* group, groups)
@@ -391,8 +394,12 @@ void KDirOperator::mkdir()
     bool ok;
     QString where = url().pathOrUrl();
     QString name = i18n( "New Folder" );
+#ifdef Q_WS_WIN
+    if ( url().isLocalFile() && QFileInfo( url().toLocalFile() + name ).exists() )
+#else
     if ( url().isLocalFile() && QFileInfo( url().path(KUrl::AddTrailingSlash) + name ).exists() )
-         name = KIO::RenameDlg::suggestName( url(), name );
+#endif
+         name = KIO::RenameDialog::suggestName( url(), name );
 
     QString dir = KInputDialog::getText( i18n( "New Folder" ),
                                          i18n( "Create new folder in:\n%1" ,  where ),
@@ -431,7 +438,7 @@ bool KDirOperator::mkdir( const QString& directory, bool enterDirectory )
                                               "create that folder." ));
     }
     else if ( enterDirectory ) {
-        setURL( url, true );
+        setUrl( url, true );
     }
 
     return writeOk;
@@ -467,14 +474,14 @@ KIO::DeleteJob * KDirOperator::del( const KFileItemList& items,
                 i18n( "<qt>Do you really want to delete\n <b>'%1'</b>?</qt>" ,
                   files.first() ),
                                                       i18n("Delete File"),
-                                                      KStdGuiItem::del(), "AskForDelete" );
+                                                      KStandardGuiItem::del(), "AskForDelete" );
         }
         else
             ret = KMessageBox::warningContinueCancelList( parent,
-                i18np("Do you really want to delete this item?", "Do you really want to delete these %n items?", items.count() ),
+                i18np("Do you really want to delete this item?", "Do you really want to delete these %1 items?", items.count() ),
                                                     files,
                                                     i18n("Delete Files"),
-                                                    KStdGuiItem::del(), "AskForDelete" );
+                                                    KStandardGuiItem::del(), "AskForDelete" );
         doIt = (ret == KMessageBox::Continue);
     }
 
@@ -526,7 +533,7 @@ KIO::CopyJob * KDirOperator::trash( const KFileItemList& items,
         }
         else
             ret = KMessageBox::warningContinueCancelList( parent,
-                i18np("translators: not called for n == 1", "Do you really want to trash these %n items?", items.count() ),
+                i18np("translators: not called for n == 1", "Do you really want to trash these %1 items?", items.count() ),
                                                     files,
                                                     i18n("Trash Files"),
                                                     KGuiItem(i18nc("to trash", "&Trash"),"edittrash"), "AskForTrash" );
@@ -596,9 +603,9 @@ void KDirOperator::checkPath(const QString &, bool /*takeFiles*/) // SLOT
 
     if (u.isLocalFile()) {
         // the empty path is kind of a hack
-        KFileItem i("", u.path());
+        KFileItem i("", u.toLocalFile());
         if (i.isDir())
-            setURL(text, true);
+            setUrl(text, true);
         else {
             if (takeFiles)
                 if (acceptOnlyExisting && !i.isFile())
@@ -607,7 +614,7 @@ void KDirOperator::checkPath(const QString &, bool /*takeFiles*/) // SLOT
                     filenameEntered = true;
         }
     } else
-        setURL(text, true);
+        setUrl(text, true);
 
     if (filenameEntered) {
         filename_ = u.url();
@@ -621,7 +628,7 @@ void KDirOperator::checkPath(const QString &, bool /*takeFiles*/) // SLOT
     kDebug(kfile_area) << "TODO KDirOperator::checkPath()" << endl;
 }
 
-void KDirOperator::setURL(const KUrl& _newurl, bool clearforward)
+void KDirOperator::setUrl(const KUrl& _newurl, bool clearforward)
 {
     KUrl newurl;
 
@@ -630,7 +637,11 @@ void KDirOperator::setURL(const KUrl& _newurl, bool clearforward)
     else
         newurl = _newurl;
 
+#ifdef Q_WS_WIN
+    QString pathstr = newurl.toLocalFile();
+#else
     QString pathstr = newurl.path(KUrl::AddTrailingSlash);
+#endif
     newurl.setPath(pathstr);
 
     // already set
@@ -652,6 +663,7 @@ void KDirOperator::setURL(const KUrl& _newurl, bool clearforward)
     if (clearforward) {
         // autodelete should remove this one
         backStack.push(new KUrl(currUrl));
+        qDeleteAll( forwardStack );
         forwardStack.clear();
     }
 
@@ -666,7 +678,7 @@ void KDirOperator::setURL(const KUrl& _newurl, bool clearforward)
     backAction->setEnabled( !backStack.isEmpty() );
     upAction->setEnabled( !isRoot() );
 
-    openURL( newurl );
+    openUrl( newurl );
 }
 
 void KDirOperator::updateDir()
@@ -679,13 +691,13 @@ void KDirOperator::updateDir()
 void KDirOperator::rereadDir()
 {
     pathChanged();
-    openURL( currUrl, false, true );
+    openUrl( currUrl, false, true );
 }
 
 
-bool KDirOperator::openURL( const KUrl& url, bool keep, bool reload )
+bool KDirOperator::openUrl( const KUrl& url, bool keep, bool reload )
 {
-    bool result = dir->openURL( url, keep, reload );
+    bool result = dir->openUrl( url, keep, reload );
     if ( !result ) // in that case, neither completed() nor canceled() will be emitted by KDL
         slotCanceled();
 
@@ -740,7 +752,7 @@ void KDirOperator::back()
 
     KUrl *s = backStack.pop();
 
-    setURL(*s, false);
+    setUrl(*s, false);
     delete s;
 }
 
@@ -753,7 +765,7 @@ void KDirOperator::forward()
     backStack.push(new KUrl(currUrl));
 
     KUrl *s = forwardStack.pop();
-    setURL(*s, false);
+    setUrl(*s, false);
     delete s;
 }
 
@@ -766,14 +778,14 @@ void KDirOperator::cdUp()
 {
     KUrl tmp(currUrl);
     tmp.cd(QLatin1String(".."));
-    setURL(tmp, true);
+    setUrl(tmp, true);
 }
 
 void KDirOperator::home()
 {
     KUrl u;
     u.setPath( QDir::homePath() );
-    setURL(u, true);
+    setUrl(u, true);
 }
 
 void KDirOperator::clearFilter()
@@ -1062,18 +1074,18 @@ void KDirOperator::connectView(KFileView *view)
 
     if ( listDir ) {
         QApplication::setOverrideCursor( Qt::WaitCursor );
-        openURL( currUrl );
+        openUrl( currUrl );
     }
     else
         view->listingCompleted();
 }
 
-KFile::Mode KDirOperator::mode() const
+KFile::Modes KDirOperator::mode() const
 {
     return myMode;
 }
 
-void KDirOperator::setMode(KFile::Mode m)
+void KDirOperator::setMode(KFile::Modes m)
 {
     if (myMode == m)
         return;
@@ -1109,6 +1121,7 @@ void KDirOperator::setDirLister( KDirLister *lister )
     dir = lister;
 
     dir->setAutoUpdate( true );
+    dir->setDelayedMimeTypes(true);
 
     QWidget* mainWidget = topLevelWidget();
     dir->setMainWindow (mainWidget);
@@ -1156,7 +1169,7 @@ void KDirOperator::insertNewFiles(const KFileItemList &newone)
 
 void KDirOperator::selectDir(const KFileItem *item)
 {
-    setURL(item->url(), true);
+    setUrl(item->url(), true);
 }
 
 void KDirOperator::itemDeleted(KFileItem *item)
@@ -1246,51 +1259,61 @@ void KDirOperator::setupActions()
     myActionCollection->setObjectName( "KDirOperator::myActionCollection" );
     myActionCollection->setAssociatedWidget( topLevelWidget() );
 
-    actionMenu = new KActionMenu( i18n("Menu"), myActionCollection, "popupMenu" );
+    actionMenu = new KActionMenu( i18n("Menu"), this );
+    myActionCollection->addAction( "popupMenu", actionMenu );
 
-    upAction = KStdAction::up( this, SLOT( cdUp() ), myActionCollection, "up" );
+    upAction = myActionCollection->addAction( KStandardAction::Up, "up", this, SLOT( cdUp() ) );
     upAction->setText( i18n("Parent Folder") );
 
-    backAction = KStdAction::back( this, SLOT( back() ), myActionCollection, "back" );
+    backAction = myActionCollection->addAction( KStandardAction::Back, "back", this, SLOT( back() ) );
 
-    forwardAction = KStdAction::forward( this, SLOT(forward()), myActionCollection, "forward" );
+    forwardAction = myActionCollection->addAction( KStandardAction::Forward, "forward", this, SLOT( forward() ) );
 
-    homeAction = KStdAction::home( this, SLOT( home() ), myActionCollection, "home" );
+    homeAction = myActionCollection->addAction( KStandardAction::Home, "home", this, SLOT( home() ) );
     homeAction->setText(i18n("Home Folder"));
 
-    reloadAction = KStdAction::redisplay( this, SLOT(rereadDir()), myActionCollection, "reload" );
+    reloadAction = myActionCollection->addAction( KStandardAction::Redisplay, "reload", this, SLOT( rereadDir() ) );
     reloadAction->setText( i18n("Reload") );
-    reloadAction->setShortcut( KStdAccel::shortcut( KStdAccel::Reload ));
+    reloadAction->setShortcuts( KStandardShortcut::shortcut( KStandardShortcut::Reload ));
 
-    actionSeparator = new KSeparatorAction( myActionCollection, "separator" );
+    actionSeparator = new QAction( this );
+    myActionCollection->addAction( "separator", actionSeparator );
 
-    mkdirAction = new KAction( i18n("New Folder..."), myActionCollection, "mkdir" );
+    mkdirAction = new KAction( i18n("New Folder..."), this );
+    myActionCollection->addAction( "mkdir", mkdirAction );
     mkdirAction->setIcon( KIcon( QLatin1String("folder_new") ) );
     connect( mkdirAction, SIGNAL( triggered( bool ) ), this, SLOT( mkdir() ) );
 
-    KAction* trash = new KAction( i18n( "Move to Trash" ), myActionCollection, "trash" );
+    KAction* trash = new KAction( i18n( "Move to Trash" ), this );
+    myActionCollection->addAction( "trash", trash );
     trash->setIcon( KIcon( "edittrash" ) );
-    trash->setShortcut( Qt::Key_Delete );
+    trash->setShortcuts( KShortcut(Qt::Key_Delete) );
     connect( trash, SIGNAL( triggered(bool) ), SLOT( trashSelected() ) );
 
-    KAction* action = new KAction( i18n( "Delete" ), myActionCollection, "delete" );
+    KAction* action = new KAction( i18n( "Delete" ), this );
+    myActionCollection->addAction( "delete", action );
     action->setIcon( KIcon( "editdelete" ) );
-    action->setShortcut( Qt::SHIFT+Qt::Key_Delete );
+    action->setShortcuts( KShortcut(Qt::SHIFT+Qt::Key_Delete) );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT( deleteSelected() ) );
 
     // the sort menu actions
-    sortActionMenu = new KActionMenu( i18n("Sorting"), myActionCollection, "sorting menu");
+    sortActionMenu = new KActionMenu( i18n("Sorting"), this );
+    myActionCollection->addAction( "sorting menu",  sortActionMenu );
 
-    byNameAction = new KAction( i18n("By Name"), myActionCollection, "by name" );
+    byNameAction = new KAction( i18n("By Name"), this );
+    myActionCollection->addAction( "by name", byNameAction );
     connect( byNameAction, SIGNAL( triggered( bool ) ), this, SLOT( slotSortByName() ) );
 
-    byDateAction = new KAction( i18n("By Date"), myActionCollection, "by date" );
+    byDateAction = new KAction( i18n("By Date"), this );
+    myActionCollection->addAction( "by date", byDateAction );
     connect( byDateAction, SIGNAL( triggered( bool ) ), this, SLOT( slotSortByDate() ) );
 
-    bySizeAction = new KAction( i18n("By Size"), myActionCollection, "by size" );
+    bySizeAction = new KAction( i18n("By Size"), this );
+    myActionCollection->addAction( "by size", bySizeAction );
     connect( bySizeAction, SIGNAL( triggered( bool ) ), this, SLOT( slotSortBySize() ) );
 
-    reverseAction = new KToggleAction( i18n("Reverse"), myActionCollection, "reversed" );
+    reverseAction = new KToggleAction( i18n("Reverse"), this );
+    myActionCollection->addAction( "reversed", reverseAction );
     connect( reverseAction, SIGNAL( triggered( bool ) ), this, SLOT( slotSortReversed() ) );
 
     QActionGroup* sortGroup = new QActionGroup(this);
@@ -1298,33 +1321,41 @@ void KDirOperator::setupActions()
     byDateAction->setActionGroup(sortGroup);
     bySizeAction->setActionGroup(sortGroup);
 
-    dirsFirstAction = new KToggleAction( i18n("Folders First"), myActionCollection, "dirs first");
+    dirsFirstAction = new KToggleAction( i18n("Folders First"), this );
+    myActionCollection->addAction( "dirs first", dirsFirstAction );
     connect( dirsFirstAction, SIGNAL( toggled( bool ) ), SLOT( slotToggleDirsFirst() ));
 
-    caseInsensitiveAction = new KToggleAction(i18n("Case Insensitive"), myActionCollection, "case insensitive" );
+    caseInsensitiveAction = new KToggleAction(i18n("Case Insensitive"), this );
+    myActionCollection->addAction( "case insensitive", caseInsensitiveAction );
     connect( caseInsensitiveAction, SIGNAL( toggled( bool ) ), SLOT( slotToggleIgnoreCase() ));
 
     // the view menu actions
-    viewActionMenu = new KActionMenu( i18n("&View"), myActionCollection, "view menu" );
+    viewActionMenu = new KActionMenu( i18n("&View"), this );
+    myActionCollection->addAction( "view menu", viewActionMenu );
     connect( viewActionMenu->menu(), SIGNAL( aboutToShow() ), SLOT( insertViewDependentActions() ));
 
-    shortAction = new KAction(i18n("Short View"), myActionCollection, "short view");
+    shortAction = new KAction(i18n("Short View"), this );
+    myActionCollection->addAction( "short view",  shortAction );
     shortAction->setIcon( KIcon( QLatin1String("view_multicolumn") ) );
     connect( shortAction, SIGNAL( activated() ), SLOT( slotSimpleView() ));
 
-    detailedAction = new KAction( i18n("Detailed View"), myActionCollection, "detailed view");
+    detailedAction = new KAction( i18n("Detailed View"), this );
+    myActionCollection->addAction( "detailed view", detailedAction );
     detailedAction->setIcon( KIcon( QLatin1String("view_detailed") ) );
     connect( detailedAction, SIGNAL( activated() ), SLOT( slotDetailedView() ));
 
-    showHiddenAction = new KToggleAction( i18n("Show Hidden Files"), myActionCollection, "show hidden" );
+    showHiddenAction = new KToggleAction( i18n("Show Hidden Files"), this );
+    myActionCollection->addAction( "show hidden", showHiddenAction );
     connect( showHiddenAction, SIGNAL( toggled( bool ) ), SLOT( slotToggleHidden( bool ) ));
 
-    separateDirsAction = new KToggleAction( i18n("Separate Folders"), myActionCollection, "separate dirs" );
+    separateDirsAction = new KToggleAction( i18n("Separate Folders"), this );
+    myActionCollection->addAction( "separate dirs", separateDirsAction );
     connect( separateDirsAction, SIGNAL( triggered( bool ) ), this, SLOT(slotSeparateDirs()) );
 
-    KToggleAction *previewAction = new KToggleAction(i18n("Show Preview"), myActionCollection, "preview" );
+    KToggleAction *previewAction = new KToggleAction(i18n("Show Preview"), this );
+    myActionCollection->addAction( "preview", previewAction );
     previewAction->setIcon( KIcon( "thumbnail" ) );
-    previewAction->setCheckedState(i18n("Hide Preview"));
+    previewAction->setCheckedState(KGuiItem(i18n("Hide Preview")));
     connect( previewAction, SIGNAL( toggled( bool )),
              SLOT( togglePreview( bool )));
 
@@ -1333,7 +1364,8 @@ void KDirOperator::setupActions()
     shortAction->setActionGroup( detailGroup );
     detailedAction->setActionGroup( detailGroup );
 
-    action = new KAction( i18n("Properties"), myActionCollection, "properties" );
+    action = new KAction( i18n("Properties"), this );
+    myActionCollection->addAction( "properties", action );
     action->setShortcut( KShortcut(Qt::ALT+Qt::Key_Return) );
     connect( action, SIGNAL( triggered( bool ) ), this, SLOT(slotProperties()) );
 }
@@ -1601,8 +1633,10 @@ QProgressBar * KDirOperator::progressBar() const
 
 void KDirOperator::clearHistory()
 {
+    qDeleteAll( backStack );
     backStack.clear();
     backAction->setEnabled( false );
+    qDeleteAll( forwardStack );
     forwardStack.clear();
     forwardAction->setEnabled( false );
 }
@@ -1643,8 +1677,9 @@ void KDirOperator::slotProperties()
 {
     if ( m_fileView ) {
         const KFileItemList *list = m_fileView->selectedItems();
-        if ( !list->isEmpty() )
-            (void) new KPropertiesDialog( *list, this, "props dlg", true);
+        if ( !list->isEmpty() ) {
+            KPropertiesDialog::showDialog( *list, this );
+        }
     }
 }
 
@@ -1662,7 +1697,11 @@ bool KDirOperator::isReadable( const KUrl& url )
         return true; // what else can we say?
 
     KDE_struct_stat buf;
+#ifdef Q_WS_WIN
+    QString ts = url.toLocalFile();
+#else
     QString ts = url.path(KUrl::AddTrailingSlash);
+#endif
     bool readable = ( KDE_stat( QFile::encodeName( ts ), &buf) == 0 );
     if (readable) { // further checks
         DIR *test;
@@ -1713,7 +1752,4 @@ bool KDirOperator::showHiddenFiles( ) const
     return showHiddenAction->isChecked();
 }
 
-void KDirOperator::virtual_hook( int, void* )
-{ /*BASE::virtual_hook( id, data );*/ }
-
 #include "kdiroperator.moc"
