@@ -33,7 +33,9 @@
 #include <QLayout>
 #include <QLineEdit>
 #include <QFileDialog>
-
+#ifdef Q_WS_WIN
+#include <QFSFileEngine>
+#endif
 
 #include <kaction.h>
 #include <kapplication.h>
@@ -62,7 +64,7 @@
 #include <krecentdirs.h>
 #include <kshell.h>
 #include <kstandarddirs.h>
-#include <kstdguiitem.h>
+#include <kstandardguiitem.h>
 #include <kstaticdeleter.h>
 #include <ktoolbar.h>
 #include <kurl.h>
@@ -71,6 +73,7 @@
 #include <kuser.h>
 #include <kactioncollection.h>
 #include <kactionmenu.h>
+#include <kconfiggroup.h>
 
 #include "config-kfile.h"
 #include "kpreviewwidgetbase.h"
@@ -168,7 +171,11 @@ static KStaticDeleter<KUrl> ldd;
 
 KFileDialog::KFileDialog( const KUrl& startDir, const QString& filter,
                           QWidget *parent, QWidget* widget)
+#ifdef Q_WS_WIN
+    : KDialog( parent , Qt::WindowMinMaxButtonsHint)
+#else
     : KDialog( parent )
+#endif
 {
     init( startDir, filter, widget );
 }
@@ -178,7 +185,7 @@ KFileDialog::~KFileDialog()
 {
     hide();
 
-    KConfig *config = KGlobal::config();
+    KSharedConfig::Ptr config = KGlobal::config();
 
     if (d->urlBar)
         d->urlBar->save( config );
@@ -280,7 +287,7 @@ void KFileDialog::setPreviewWidget(const KPreviewWidgetBase *w) {
     d->hasView = true;
 }
 
-KUrl KFileDialog::getCompleteURL(const QString &_url)
+KUrl KFileDialog::getCompleteUrl(const QString &_url)
 {
     QString url = KShell::tildeExpand(_url);
     KUrl u;
@@ -305,7 +312,7 @@ KUrl KFileDialog::getCompleteURL(const QString &_url)
 // FIXME: check for "existing" flag here?
 void KFileDialog::slotOk()
 {
-    kDebug(kfile_area) << "slotOK\n";
+    kDebug(kfile_area) << "slotOk\n";
 
     // a list of all selected files/directories (if any)
     // can only be used if the user didn't type any filenames/urls himself
@@ -371,7 +378,7 @@ void KFileDialog::slotOk()
             }
         }
 
-        KUrl url = KIO::NetAccess::mostLocalURL(d->url,topLevelWidget());
+        KUrl url = KIO::NetAccess::mostLocalUrl(d->url,topLevelWidget());
         if ( (mode() & KFile::LocalOnly) == KFile::LocalOnly &&
              !url.isLocalFile() ) {
 // ### after message freeze, add message for directories!
@@ -404,7 +411,7 @@ void KFileDialog::slotOk()
     }
 
     else {
-        selectedUrl = getCompleteURL(locationEdit->currentText());
+        selectedUrl = getCompleteUrl(locationEdit->currentText());
 
         // appendExtension() may change selectedUrl
         appendExtension (selectedUrl);
@@ -415,7 +422,7 @@ void KFileDialog::slotOk()
        return;
     }
 
-    KUrl url = KIO::NetAccess::mostLocalURL(selectedUrl,topLevelWidget());
+    KUrl url = KIO::NetAccess::mostLocalUrl(selectedUrl,topLevelWidget());
     if ( (mode() & KFile::LocalOnly) == KFile::LocalOnly &&
          !url.isLocalFile() ) {
         KMessageBox::sorry( d->mainWidget,
@@ -433,7 +440,7 @@ void KFileDialog::slotOk()
         bool done = true;
         if ( d->url.isLocalFile() ) {
             if ( locationEdit->currentText().trimmed().isEmpty() ) {
-                QFileInfo info( d->url.path() );
+                QFileInfo info( d->url.toLocalFile() );
                 if ( info.isDir() ) {
                     d->filenames.clear();
                     d->urlList.clear();
@@ -461,7 +468,7 @@ void KFileDialog::slotOk()
                     if ( ops->dirOnlyMode() )
                     {
                         KUrl fullURL(d->url, locationEdit->currentText());
-                        if ( QFile::exists( fullURL.path() ) )
+                        if ( QFile::exists( fullURL.toLocalFile() ) )
                         {
                             d->url = fullURL;
                             d->filenames.clear();
@@ -495,7 +502,7 @@ void KFileDialog::slotOk()
             return;
     }
 
-    if (!KAuthorized::authorizeURLAction("open", KUrl(), d->url))
+    if (!KAuthorized::authorizeUrlAction("open", KUrl(), d->url))
     {
         QString msg = KIO::buildErrorString(KIO::ERR_ACCESS_DENIED, d->url.prettyUrl());
         KMessageBox::error( d->mainWidget, msg);
@@ -513,7 +520,7 @@ void KFileDialog::slotOk()
         for ( KUrl::List::ConstIterator it = list.begin();
               it != list.end(); ++it )
         {
-            if (!KAuthorized::authorizeURLAction("open", KUrl(), *it))
+            if (!KAuthorized::authorizeUrlAction("open", KUrl(), *it))
             {
                 QString msg = KIO::buildErrorString(KIO::ERR_ACCESS_DENIED, (*it).prettyUrl());
                 KMessageBox::error( d->mainWidget, msg);
@@ -622,13 +629,13 @@ void KFileDialog::accept()
         locationEdit->insertItem( 1,file);
     }
 
-    KConfig *config = KGlobal::config();
+    KSharedConfig::Ptr config = KGlobal::config();
     config->setForceGlobal( true );
     KConfigGroup grp(config,ConfigGroup);
     writeConfig(&grp);
     config->setForceGlobal( false );
 
-    saveRecentFiles( config );
+    saveRecentFiles(config.data());
     config->sync();
 
     KDialog::accept();
@@ -775,9 +782,9 @@ void KFileDialog::init( const KUrl& startDir, const QString& filter, QWidget* wi
     d->mainWidget = new QWidget( this );
     d->mainWidget->setObjectName( "KFileDialog::mainWidget" );
     setMainWidget( d->mainWidget );
-    d->okButton = new KPushButton( KStdGuiItem::ok(), d->mainWidget );
+    d->okButton = new KPushButton( KStandardGuiItem::ok(), d->mainWidget );
     d->okButton->setDefault( true );
-    d->cancelButton = new KPushButton(KStdGuiItem::cancel(), d->mainWidget);
+    d->cancelButton = new KPushButton(KStandardGuiItem::cancel(), d->mainWidget);
     connect( d->okButton, SIGNAL( clicked() ), SLOT( slotOk() ));
     connect( d->cancelButton, SIGNAL( clicked() ), SLOT( slotCancel() ));
     d->customWidget = widget;
@@ -788,6 +795,7 @@ void KFileDialog::init( const KUrl& startDir, const QString& filter, QWidget* wi
     QtMsgHandler oldHandler = qInstallMsgHandler( silenceQToolBar );
     toolbar = new KToolBar( d->mainWidget, "KFileDialog::toolbar", true);
     toolbar->setMovable(false);
+
     qInstallMsgHandler( oldHandler );
 
     d->pathCombo = new KUrlComboBox( KUrlComboBox::Directories, true,
@@ -797,17 +805,28 @@ void KFileDialog::init( const KUrl& startDir, const QString& filter, QWidget* wi
                                                  "The drop-down list also lists commonly used locations. "
                                                  "This includes standard locations, such as your home folder, as well as "
                                                  "locations that have been visited recently.") + i18n (autocompletionWhatsThisText));
-
     KUrl u;
-    u.setPath( QDir::rootPath() );
-    QString text = i18n("Root Folder: %1",  u.path() );
-    d->pathCombo->addDefaultURL( u,
-                                 KIO::pixmapForURL( u, 0, K3Icon::Small ),
+    QString text;
+#ifdef Q_WS_WIN
+    foreach( const QFileInfo &drive,QFSFileEngine::drives() ) 
+    {
+        u.setPath( drive.filePath() );
+        text = i18n("Drive: %1",  u.toLocalFile() );
+        d->pathCombo->addDefaultUrl( u,
+                                 KIO::pixmapForUrl( u, 0, K3Icon::Small ),
                                  text );
+    }
+#else
+    u.setPath( QDir::rootPath() );
+    text = i18n("Root Folder: %1",  u.toLocalFile() );
+    d->pathCombo->addDefaultUrl( u,
+                                 KIO::pixmapForUrl( u, 0, K3Icon::Small ),
+                                 text );
+#endif
 
     u.setPath( QDir::homePath() );
     text = i18n("Home Folder: %1",  u.path( KUrl::AddTrailingSlash ) );
-    d->pathCombo->addDefaultURL( u, KIO::pixmapForURL( u, 0, K3Icon::Small ),
+    d->pathCombo->addDefaultUrl( u, KIO::pixmapForUrl( u, 0, K3Icon::Small ),
                                  text );
 
     KUrl docPath;
@@ -816,31 +835,31 @@ void KFileDialog::init( const KUrl& startDir, const QString& filter, QWidget* wi
           QDir(docPath.path(KUrl::AddTrailingSlash)).exists() )
     {
       text = i18n("Documents: %1",  docPath.path( KUrl::AddTrailingSlash ) );
-        d->pathCombo->addDefaultURL( docPath,
-                                     KIO::pixmapForURL( docPath, 0, K3Icon::Small ),
+        d->pathCombo->addDefaultUrl( docPath,
+                                     KIO::pixmapForUrl( docPath, 0, K3Icon::Small ),
                                      text );
     }
 
     u.setPath( KGlobalSettings::desktopPath() );
     text = i18n("Desktop: %1",  u.path( KUrl::AddTrailingSlash ) );
-    d->pathCombo->addDefaultURL( u,
-                                 KIO::pixmapForURL( u, 0, K3Icon::Small ),
+    d->pathCombo->addDefaultUrl( u,
+                                 KIO::pixmapForUrl( u, 0, K3Icon::Small ),
                                  text );
 
-    d->url = getStartURL( startDir, d->fileClass );
+    d->url = getStartUrl( startDir, d->fileClass );
     d->selection = d->url.url();
 
     // If local, check it exists. If not, go up until it exists.
     if ( d->url.isLocalFile() )
     {
-        if ( !QFile::exists( d->url.path() ) )
+        if ( !QFile::exists( d->url.toLocalFile() ) )
         {
             d->url = d->url.upUrl();
-            QDir dir( d->url.path() );
+            QDir dir( d->url.toLocalFile() );
             while ( !dir.exists() )
             {
                 d->url = d->url.upUrl();
-                dir.setPath( d->url.path() );
+                dir.setPath( d->url.toLocalFile() );
             }
         }
     }
@@ -873,24 +892,27 @@ void KFileDialog::init( const KUrl& startDir, const QString& filter, QWidget* wi
     coll->action( "forward" )->setWhatsThis(i18n("Click this button to move forward one step in the browsing history."));
     toolbar->addAction( coll->action( "reload" ) );
     coll->action( "reload" )->setWhatsThis(i18n("Click this button to reload the contents of the current location."));
-    coll->action( "mkdir" )->setShortcut(Qt::Key_F10);
+    coll->action( "mkdir" )->setShortcut( QKeySequence(Qt::Key_F10) );
     toolbar->addAction( coll->action( "mkdir" ) );
     coll->action( "mkdir" )->setWhatsThis(i18n("Click this button to create a new folder."));
 
     KToggleAction *showSidebarAction =
-        new KToggleAction(i18n("Show Quick Access Navigation Panel"), coll,"toggleSpeedbar");
-    showSidebarAction->setShortcut( Qt::Key_F9 );
-    showSidebarAction->setCheckedState(i18n("Hide Quick Access Navigation Panel"));
+        new KToggleAction(i18n("Show Quick Access Navigation Panel"), this);
+    coll->addAction("toggleSpeedbar", showSidebarAction);
+    showSidebarAction->setShortcut( QKeySequence(Qt::Key_F9) );
+    showSidebarAction->setCheckedState(KGuiItem(i18n("Hide Quick Access Navigation Panel")));
     connect( showSidebarAction, SIGNAL( toggled( bool ) ),
              SLOT( toggleSpeedbar( bool )) );
 
     KToggleAction *showBookmarksAction =
-            new KToggleAction(i18n("Show Bookmarks"), coll, "toggleBookmarks");
-    showBookmarksAction->setCheckedState(i18n("Hide Bookmarks"));
+        new KToggleAction(i18n("Show Bookmarks"), this);
+    coll->addAction("toggleBookmarks", showBookmarksAction);
+    showBookmarksAction->setCheckedState(KGuiItem(i18n("Hide Bookmarks")));
     connect( showBookmarksAction, SIGNAL( toggled( bool ) ),
              SLOT( toggleBookmarks( bool )) );
 
-    KActionMenu *menu = new KActionMenu( KIcon("configure"), i18n("Configure"), coll, "extra menu" );
+    KActionMenu *menu = new KActionMenu( KIcon("configure"), i18n("Configure"), this);
+    coll->addAction("extra menu", menu);
     menu->setWhatsThis(i18n("<qt>This is the configuration menu for the file dialog. "
                             "Various options can be accessed from this menu including: <ul>"
                             "<li>how files are sorted in the list</li>"
@@ -901,18 +923,18 @@ void KFileDialog::init( const KUrl& startDir, const QString& filter, QWidget* wi
                             "<li>separating folders from files</li></ul></qt>"));
     menu->addAction( coll->action( "sorting menu" ));
     menu->addSeparator();
-    coll->action( "short view" )->setShortcut(Qt::Key_F6);
+    coll->action( "short view" )->setShortcut( QKeySequence(Qt::Key_F6) );
     menu->addAction( coll->action( "short view" ));
-    coll->action( "detailed view" )->setShortcut(Qt::Key_F7);
+    coll->action( "detailed view" )->setShortcut( QKeySequence(Qt::Key_F7) );
     menu->addAction( coll->action( "detailed view" ));
     menu->addSeparator();
-    coll->action( "show hidden" )->setShortcut(Qt::Key_F8);
+    coll->action( "show hidden" )->setShortcut( QKeySequence(Qt::Key_F8) );
     menu->addAction( coll->action( "show hidden" ));
     menu->addAction( showSidebarAction );
     menu->addAction( showBookmarksAction );
-    coll->action( "preview" )->setShortcut(Qt::Key_F11);
+    coll->action( "preview" )->setShortcut( QKeySequence(Qt::Key_F11) );
     menu->addAction( coll->action( "preview" ));
-    coll->action( "separate dirs" )->setShortcut(Qt::Key_F12);
+    coll->action( "separate dirs" )->setShortcut( QKeySequence(Qt::Key_F12) );
     menu->addAction( coll->action( "separate dirs" ));
 
     menu->setDelayed( false );
@@ -988,8 +1010,8 @@ void KFileDialog::init( const KUrl& startDir, const QString& filter, QWidget* wi
 
     initGUI(); // activate GM
 
-    KConfig* config = KGlobal::config();
-    readRecentFiles( config );
+    KSharedConfig::Ptr config = KGlobal::config();
+    readRecentFiles(config.data());
 
     adjustSize();
 
@@ -1002,6 +1024,8 @@ void KFileDialog::init( const KUrl& startDir, const QString& filter, QWidget* wi
 void KFileDialog::initSpeedbar()
 {
     d->urlBar = new KFileSpeedBar( d->mainWidget );
+    d->urlBar->setFrameStyle( QFrame::Box | QFrame::Plain );
+
     d->urlBar->setObjectName( QLatin1String( "url bar" ) );
     connect( d->urlBar, SIGNAL( activated( const KUrl& )),
              SLOT( enterUrl( const KUrl& )) );
@@ -1028,10 +1052,11 @@ void KFileDialog::initGUI()
     d->urlBarLayout = new QHBoxLayout();
     d->boxLayout->addItem(d->urlBarLayout ); // needed for the urlBar that may appear
     QVBoxLayout *vbox = new QVBoxLayout();
+    vbox->setMargin(KDialog::marginHint());
     d->urlBarLayout->addItem(vbox);
 
     vbox->addWidget(ops, 4);
-    vbox->addSpacing(3);
+    vbox->addSpacing(KDialog::spacingHint());
 
     QGridLayout* lafBox= new QGridLayout();
 
@@ -1045,10 +1070,9 @@ void KFileDialog::initGUI()
     lafBox->addWidget(d->cancelButton, 1, 2, Qt::AlignVCenter);
 
     lafBox->setColumnStretch(1, 4);
-    lafBox->setMargin(5);
 
-    vbox->addLayout(lafBox, 0);
-    vbox->addSpacing(3);
+    vbox->addLayout(lafBox);
+    vbox->addSpacing(KDialog::spacingHint());
 
     // add the Automatically Select Extension checkbox
     vbox->addWidget (d->autoSelectExtCheckBox);
@@ -1113,7 +1137,7 @@ void KFileDialog::slotFilterChanged()
 void KFileDialog::setUrl(const KUrl& url, bool clearforward)
 {
     d->selection.clear();
-    ops->setURL( url, clearforward);
+    ops->setUrl( url, clearforward);
 }
 
 // Protected
@@ -1123,7 +1147,7 @@ void KFileDialog::urlEntered(const KUrl& url)
     d->selection.clear();
 
     if ( d->pathCombo->count() != 0 ) { // little hack
-        d->pathCombo->setURL( url );
+        d->pathCombo->setUrl( url );
     }
 
     locationEdit->blockSignals( true );
@@ -1172,19 +1196,22 @@ void KFileDialog::setSelection(const QString& url)
         return;
     }
 
-    KUrl u = getCompleteURL(url);
+    KUrl u = getCompleteUrl(url);
     if (!u.isValid()) { // if it still is
         kWarning() << url << " is not a correct argument for setSelection!" << endl;
         return;
     }
 
-//     #warning FIXME: http URLs, e.g. from KUrlCombo
+    // Honor protocols that do not support directory listing
+    if (!KProtocolManager::supportsListing(u))
+        return;
 
     /* we strip the first / from the path to avoid file://usr which means
      *  / on host usr
      */
     KFileItem i(KFileItem::Unknown, KFileItem::Unknown, u, true );
     //    KFileItem i(u.path());
+    kDebug(kfile_area) << "KFileItem " << u.path() << " " << i.isDir() << " " << u.isLocalFile() << " " << QFile::exists( u.path() ) << endl;
     if ( i.isDir() && u.isLocalFile() && QFile::exists( u.path() ) ) {
         // trust isDir() only if the file is
         // local (we cannot stat non-local urls) and if it exists!
@@ -1196,12 +1223,10 @@ void KFileDialog::setSelection(const QString& url)
         QString filename = u.url();
         int sep = filename.lastIndexOf('/');
         if (sep >= 0) { // there is a / in it
-            if ( KProtocolManager::supportsListing( u )) {
-                KUrl dir(u);
-                dir.setQuery( QString() );
-                dir.setFileName( QString() );
-                setUrl(dir, true );
-            }
+            KUrl dir(u);
+            dir.setQuery( QString() );
+            dir.setFileName( QString() );
+            setUrl(dir, true );
 
             // filename must be decoded, or "name with space" would become
             // "name%20with%20space", so we use KUrl::fileName()
@@ -1346,7 +1371,7 @@ QString KFileDialog::getExistingDirectory(const KUrl& startDir,
 {
 #ifdef Q_WS_WIN
     return QFileDialog::getExistingDirectory(parent, caption,
-                                             startDir.toString(), QFileDialog::ShowDirsOnly);
+                                             startDir.path(), QFileDialog::ShowDirsOnly);
 #else
     KUrl url = KDirSelectDialog::selectDirectory(startDir, true, parent,
                                                  caption);
@@ -1480,7 +1505,7 @@ QString KFileDialog::selectedFile() const
 {
     if ( result() == QDialog::Accepted )
     {
-      KUrl url = KIO::NetAccess::mostLocalURL(d->url,topLevelWidget());
+      KUrl url = KIO::NetAccess::mostLocalUrl(d->url,topLevelWidget());
        if (url.isLocalFile())
            return url.path();
        else {
@@ -1502,7 +1527,7 @@ QStringList KFileDialog::selectedFiles() const
             KUrl::List urls = parseSelectedUrls();
             QList<KUrl>::const_iterator it = urls.begin();
             while ( it != urls.end() ) {
-                url = KIO::NetAccess::mostLocalURL(*it,topLevelWidget());
+                url = KIO::NetAccess::mostLocalUrl(*it,topLevelWidget());
                 if ( url.isLocalFile() )
                     list.append( url.path() );
                 ++it;
@@ -1529,8 +1554,11 @@ QString KFileDialog::getSaveFileName(const KUrl& dir, const QString& filter,
 {
     bool specialDir = (!dir.isEmpty()) && (dir.protocol() == "kfiledialog");
     KFileDialog dlg( specialDir ? dir : KUrl(), filter, parent);
-    if ( !specialDir )
-        dlg.setSelection( dir.toString() ); // may also be a filename
+    if ( !specialDir ) {
+        if (!dir.isLocalFile())
+            kWarning() << "KFileDialog::getSaveFileName called with non-local start dir " << dir << endl;
+        dlg.setSelection( dir.path() ); // may also be a filename
+    }
 
     dlg.setOperationMode( Saving );
     dlg.setMode( KFile::File );
@@ -1559,8 +1587,11 @@ QString KFileDialog::getSaveFileNameWId(const KUrl& dir, const QString& filter,
     // TODO
 #endif
 
-    if ( !specialDir )
-        dlg.setSelection( dir.toString() ); // may also be a filename
+    if ( !specialDir ) {
+        if (!dir.isLocalFile())
+            kWarning() << "KFileDialog::getSaveFileNameWId called with non-local start dir " << dir << endl;
+        dlg.setSelection( dir.path() ); // may also be a filename
+    }
 
     dlg.setOperationMode( KFileDialog::Saving);
     dlg.setMode( KFile::File );
@@ -1581,7 +1612,7 @@ KUrl KFileDialog::getSaveUrl(const KUrl& dir, const QString& filter,
     bool specialDir = (!dir.isEmpty()) && (dir.protocol() == "kfiledialog");
     KFileDialog dlg(specialDir ? dir : KUrl(), filter, parent);
     if ( !specialDir )
-    dlg.setSelection( dir.toString() ); // may also be a filename
+        dlg.setSelection( dir.url() ); // may also be a filename
 
     dlg.setWindowTitle(caption.isNull() ? i18n("Save As") : caption);
     dlg.setOperationMode( Saving );
@@ -1596,7 +1627,7 @@ KUrl KFileDialog::getSaveUrl(const KUrl& dir, const QString& filter,
     return url;
 }
 
-void KFileDialog::show()
+void KFileDialog::showEvent(QShowEvent* event)
 {
     if ( !d->hasView ) { // delayed view-creation
         ops->setView(KFile::Default);
@@ -1604,10 +1635,10 @@ void KFileDialog::show()
         d->hasView = true;
     }
 
-    KDialog::show();
+    KDialog::showEvent(event);
 }
 
-void KFileDialog::setMode( KFile::Mode m )
+void KFileDialog::setMode( KFile::Modes m )
 {
     ops->setMode(m);
     if ( ops->dirOnlyMode() ) {
@@ -1620,12 +1651,7 @@ void KFileDialog::setMode( KFile::Mode m )
     updateAutoSelectExtension ();
 }
 
-void KFileDialog::setMode( unsigned int m )
-{
-    setMode(static_cast<KFile::Mode>( m ));
-}
-
-KFile::Mode KFileDialog::mode() const
+KFile::Modes KFileDialog::mode() const
 {
     return ops->mode();
 }
@@ -1639,10 +1665,10 @@ void KFileDialog::readConfig( KConfigGroup *configGroup)
     ops->readConfig(configGroup);
 
     KUrlComboBox *combo = d->pathCombo;
-    combo->setURLs( configGroup->readPathListEntry( RecentURLs ), KUrlComboBox::RemoveTop );
+    combo->setUrls( configGroup->readPathListEntry( RecentURLs ), KUrlComboBox::RemoveTop );
     combo->setMaxItems( configGroup->readEntry( RecentURLsNumber,
                                        DefaultRecentURLsNumber ) );
-    combo->setURL( ops->url() );
+    combo->setUrl( ops->url() );
     autoDirectoryFollowing = configGroup->readEntry( AutoDirectoryFollowing,
                                             DefaultDirectoryFollowing );
 
@@ -1700,7 +1726,7 @@ void KFileDialog::readRecentFiles( KConfig *kc )
 
     locationEdit->setMaxItems( kc->readEntry( RecentFilesNumber,
                                               DefaultRecentURLsNumber ) );
-    locationEdit->setURLs( kc->readPathListEntry( RecentFiles ),
+    locationEdit->setUrls( kc->readPathListEntry( RecentFiles ),
                            KUrlComboBox::RemoveBottom );
     locationEdit->insertItem(0, QString()); // dummy item without pixmap
     locationEdit->setCurrentIndex( 0 );
@@ -1738,7 +1764,7 @@ void KFileDialog::slotCancel()
     ops->close();
     reject(); //KDialog::slotCancel();
 
-    KConfig *config = KGlobal::config();
+    KSharedConfig::Ptr config = KGlobal::config();
     config->setForceGlobal( true );
     KConfigGroup grp(config,ConfigGroup);
     writeConfig(&grp);
@@ -1761,13 +1787,13 @@ void KFileDialog::setOperationMode( OperationMode mode )
     d->keepLocation = (mode == Saving);
     filterWidget->setEditable( !d->hasDefaultFilter || mode != Saving );
     if ( mode == Opening )
-       d->okButton->setGuiItem( KStdGuiItem::open() );
+       d->okButton->setGuiItem( KStandardGuiItem::open() );
     else if ( mode == Saving ) {
-       d->okButton->setGuiItem( KStdGuiItem::save() );
+       d->okButton->setGuiItem( KStandardGuiItem::save() );
        setNonExtSelection();
     }
     else
-       d->okButton->setGuiItem( KStdGuiItem::ok() );
+       d->okButton->setGuiItem( KStandardGuiItem::ok() );
     updateLocationWhatsThis ();
     updateAutoSelectExtension ();
 }
@@ -1977,7 +2003,7 @@ void KFileDialog::updateLocationEditExtension (const QString &lastExtension)
     if (urlStr.isEmpty ())
         return;
 
-    KUrl url = getCompleteURL (urlStr);
+    KUrl url = getCompleteUrl (urlStr);
     kDebug (kfile_area) << "updateLocationEditExtension (" << url << ")" << endl;
 
     const int fileNameOffset = urlStr.lastIndexOf ('/') + 1;
@@ -2149,18 +2175,16 @@ void KFileDialog::toggleSpeedbar( bool show )
         d->urlBar->show();
 
         // check to see if they have a home item defined, if not show the home button
-        KUrlBarItem *urlItem = static_cast<KUrlBarItem*>( d->urlBar->listBox()->firstItem() );
         KUrl homeURL;
         homeURL.setPath( QDir::homePath() );
-        while ( urlItem )
+        for ( int rowIndex = 0 ; rowIndex < d->urlBar->listBox()->count() ; rowIndex++ )
         {
-          if ( homeURL.equals( urlItem->url(), KUrl::CompareWithoutTrailingSlash ) )
-            {
+            KUrlBarItem *urlItem = static_cast<KUrlBarItem*>( d->urlBar->listBox()->item(rowIndex) );
+          
+            if ( homeURL.equals( urlItem->url(), KUrl::CompareWithoutTrailingSlash ) ) {
                 toolbar->removeAction( ops->actionCollection()->action( "home" ) );
                 break;
             }
-
-            urlItem = static_cast<KUrlBarItem*>( urlItem->next() );
         }
     }
     else
@@ -2168,8 +2192,8 @@ void KFileDialog::toggleSpeedbar( bool show )
         if (d->urlBar)
             d->urlBar->hide();
 
-        KAction* homeAction = ops->actionCollection()->action( "home" );
-        KAction* reloadAction = ops->actionCollection()->action( "reload" );
+        QAction* homeAction = ops->actionCollection()->action( "home" );
+        QAction* reloadAction = ops->actionCollection()->action( "reload" );
         if ( !toolbar->actions().contains(homeAction) )
             toolbar->insertAction( reloadAction, homeAction );
     }
@@ -2187,10 +2211,11 @@ void KFileDialog::toggleBookmarks(bool show)
         }
 
         d->bookmarkHandler = new KFileBookmarkHandler( this );
-        connect( d->bookmarkHandler, SIGNAL( openURL( const QString& )),
+        connect( d->bookmarkHandler, SIGNAL( openUrl( const QString& )),
                     SLOT( enterUrl( const QString& )));
 
-        d->bookmarkButton = new KActionMenu(i18n("Bookmarks"), actionCollection(), "bookmark");
+        d->bookmarkButton = new KActionMenu(KIcon("bookmark"),i18n("Bookmarks"), this);
+        actionCollection()->addAction("bookmark", d->bookmarkButton);
         d->bookmarkButton->setMenu(d->bookmarkHandler->menu());
         d->bookmarkButton->setWhatsThis(i18n("<qt>This button allows you to bookmark specific locations. "
                                 "Click on this button to open the bookmark menu where you may add, "
@@ -2225,7 +2250,7 @@ void KFileDialog::initStatic()
 }
 
 // static
-KUrl KFileDialog::getStartURL( const KUrl& startDir,
+KUrl KFileDialog::getStartUrl( const KUrl& startDir,
                                QString& recentDirClass )
 {
     initStatic();
