@@ -588,13 +588,6 @@ public:
                                  QWidget *parent = 0,
                                  const QString& caption = QString() );
 
-    virtual void show();
-
-    /**
-     * Convenient overload of the other setMode(unsigned int) method.
-     */
-    void setMode( KFile::Mode m );
-
     /**
      * Sets the mode of the dialog.
      *
@@ -610,19 +603,19 @@ public:
      * \endcode
      * You can OR the values, e.g.
      * \code
-     * KFile::Mode mode = static_cast<KFile::Mode>( KFile::Files |
-     *                                              KFile::ExistingOnly |
-     *                                              KFile::LocalOnly );
+     * KFile::Modes mode = KFile::Files |
+     *                     KFile::ExistingOnly |
+     *                     KFile::LocalOnly );
      * setMode( mode );
      * \endcode
      */
-    void setMode( unsigned int m );
+    void setMode( KFile::Modes m );
 
     /**
      * Returns the mode of the filedialog.
      * @see setMode()
      */
-    KFile::Mode mode() const;
+    KFile::Modes mode() const;
 
     /**
      * Sets the text to be displayed in front of the selection.
@@ -699,7 +692,7 @@ public:
      * @return The URL that should be listed by default (e.g. by KFileDialog or
      *         KDirSelectDialog).
      */
-    static KUrl getStartURL( const KUrl& startDir, QString& recentDirClass );
+    static KUrl getStartUrl( const KUrl& startDir, QString& recentDirClass );
 
     /**
      * @internal
@@ -713,7 +706,7 @@ Q_SIGNALS:
       * selection mode. The best way to get notified about selected file(s)
       * is to connect to the okClicked() signal inherited from KDialog
       * and call selectedFile(), selectedFiles(),
-      * selectedURL() or selectedURLs().
+      * selectedUrl() or selectedUrls().
       */
     void fileSelected(const QString&);
 
@@ -753,6 +746,8 @@ protected:
 
     KFileFilterCombo *filterWidget;
 
+    virtual void showEvent(QShowEvent* event);
+
     /**
      * Reimplemented to animate the cancel button.
      */
@@ -808,7 +803,7 @@ protected:
     /**
      * Returns the absolute version of the URL specified in locationEdit.
      */
-    KUrl getCompleteURL(const QString&);
+    KUrl getCompleteUrl(const QString&);
 
     /**
      * Returns the filename extension associated with the currentFilter().
