@@ -19,6 +19,7 @@
    Boston, MA 02110-1301, USA.
 */
 
+#include <qapplication.h>
 #include <qdesktopwidget.h>
 #include <qevent.h>
 #include <qfontmetrics.h>
@@ -31,16 +32,16 @@
 #include <qtooltip.h>
 
 #include <kaction.h>
-#include <kapplication.h>
 #include <klocale.h>
 #include <kfileitem.h>
 #include <kiconeffect.h>
 #include <kglobalsettings.h>
 #include <kio/previewjob.h>
 #include <ktoggleaction.h>
-#include <kstdaction.h>
+#include <kstandardaction.h>
 #include <kactioncollection.h>
 #include <kactionmenu.h>
+#include <kconfiggroup.h>
 
 #include "kfileiconview.h"
 #include "config-kfile.h"
@@ -64,10 +65,12 @@ public:
 
         noArrangement = false;
         ignoreMaximumSize = false;
-        smallColumns = new KAction( i18n("Small Icons"), parent->actionCollection(), "small columns" );
+        smallColumns = new KAction( i18n("Small Icons"), parent );
+        parent->actionCollection()->addAction( "small columns", smallColumns );
         connect( smallColumns, SIGNAL( triggered( bool ) ), parent, SLOT( slotSmallColumns() ) );
 
-        largeRows = new KAction( i18n("Large Icons"), parent->actionCollection(), "large rows" );
+        largeRows = new KAction( i18n("Large Icons"), parent );
+        parent->actionCollection()->addAction( "large rows", largeRows );
         connect( largeRows, SIGNAL( triggered( bool ) ), parent, SLOT( slotLargeRows() ) );
 
 
@@ -75,13 +78,10 @@ public:
         smallColumns->setActionGroup(sizeGroup);
         largeRows->setActionGroup(sizeGroup);
 
-        previews = new KToggleAction( i18n("Thumbnail Previews"),
-                                      parent->actionCollection(),
-                                      "show previews" );
-        zoomIn = KStdAction::zoomIn( parent, SLOT( zoomIn() ),
-                                     parent->actionCollection(), "zoomIn" );
-        zoomOut = KStdAction::zoomOut( parent, SLOT( zoomOut() ),
-                                     parent->actionCollection(), "zoomOut" );
+        previews = new KToggleAction( i18n("Thumbnail Previews"), parent );
+        parent->actionCollection()->addAction( "show previews", previews );
+        zoomIn = parent->actionCollection()->addAction( KStandardAction::ZoomIn, "zoomIn", parent, SLOT( zoomIn() ) );
+        zoomOut = parent->actionCollection()->addAction( KStandardAction::ZoomOut, "zoomOut", parent, SLOT( zoomOut() ) );
 
         connect( previews, SIGNAL( toggled( bool )),
                  parent, SLOT( slotPreviewsToggled( bool )));
@@ -97,8 +97,8 @@ public:
             job->kill();
     }
 
-    KAction *smallColumns, *largeRows;
-    KAction *zoomIn, *zoomOut;
+    QAction *smallColumns, *largeRows;
+    QAction *zoomIn, *zoomOut;
     KToggleAction *previews;
     KIO::PreviewJob *job;
     KFileIconViewItem *dropItem;
@@ -169,7 +169,7 @@ KFileIconView::KFileIconView(QWidget *parent, const char *name)
 		 SLOT( highlighted( Q3IconViewItem * )));
 
     // for mimetype resolving
-    m_resolver = new KMimeTypeResolver<KFileIconViewItem,KFileIconView>(this);
+    m_resolver = new K3MimeTypeResolver<KFileIconViewItem,KFileIconView>(this);
 }
 
 KFileIconView::~KFileIconView()
@@ -480,7 +480,7 @@ void KFileIconView::slotSelectionChanged()
 
 void KFileIconView::slotSmallColumns()
 {
-    kDebug() << "slotSmallColumns " << kBacktrace() << endl;
+    //kDebug(250) << "slotSmallColumns " << kBacktrace() << endl;
 
     // setItemTextPos(), setArrangement(), setWordWrapIconText() and
     // setIconSize() all call arrangeItemsInGrid() :( Prevent this.
@@ -505,12 +505,12 @@ void KFileIconView::slotSmallColumns()
 
 void KFileIconView::slotLargeRows()
 {
-    kDebug() << "slotLargeRows " << kBacktrace() << endl;
+    // kDebug(250) << "slotLargeRows " << kBacktrace() << endl;
     // setItemTextPos(), setArrangement(), setWordWrapIconText() and
     // setIconSize() all call arrangeItemsInGrid() :( Prevent this.
     d->noArrangement = true; // stop arrangeItemsInGrid()!
 
-    setGridX( KGlobal::iconLoader()->currentSize( K3Icon::Desktop ) + 50 );
+    setGridX( KIconLoader::global()->currentSize( K3Icon::Desktop ) + 50 );
     setItemTextPos( Bottom );
     setArrangement( LeftToRight );
     setWordWrapIconText( true );
