@@ -22,6 +22,7 @@
 #include <kurlbar.h>
 
 class KConfig;
+class KSharedConfigPtr;
 
 class KIO_EXPORT KFileSpeedBar : public KUrlBar
 {
@@ -30,8 +31,7 @@ public:
     KFileSpeedBar( QWidget *parent = 0 );
     ~KFileSpeedBar();
 
-    virtual void save( KConfig *config );
-    virtual QSize sizeHint() const;
+    virtual void save(KSharedConfigPtr config);
 
 private:
     bool m_initializeSpeedbar : 1;
