@@ -253,7 +253,7 @@ Q3DragObject * KFileTreeView::dragObject()
 
    KUrl::List urls;
    const QList<Q3ListViewItem *> fileList = selectedItems();
-   for (int i = 0; i < fileList.size(); ++i) 
+   for (int i = 0; i < fileList.size(); ++i)
    {
       urls.append( static_cast<KFileTreeViewItem*>(fileList.at(i))->url() );
    }
@@ -358,7 +358,8 @@ void KFileTreeView::slotSelectionChanged()
 KFileTreeBranch* KFileTreeView::addBranch( const KUrl &path, const QString& name,
                               bool showHidden )
 {
-    const QPixmap& folderPix = KMimeType::mimeType("inode/directory")->pixmap( K3Icon::Desktop,K3Icon::SizeSmall );
+    const QPixmap& folderPix = KIconLoader::global()->loadMimeTypeIcon( KMimeType::mimeType("inode/directory")->iconName(),
+                                                                        K3Icon::Desktop, K3Icon::SizeSmall );
 
     return addBranch( path, name, folderPix, showHidden);
 }
@@ -589,7 +590,7 @@ KFileTreeViewItem * KFileTreeView::currentKFileTreeViewItem() const
    return static_cast<KFileTreeViewItem *>( selectedItem() );
 }
 
-KUrl KFileTreeView::currentURL() const
+KUrl KFileTreeView::currentUrl() const
 {
     KFileTreeViewItem *item = currentKFileTreeViewItem();
     if ( item )
@@ -641,7 +642,7 @@ KFileTreeViewItem *KFileTreeView::findItem( KFileTreeBranch* brnch, const QStrin
 
          kDebug(250) << "assembled complete dir string " << url.prettyUrl() << endl;
 
-         KFileItem *fi = brnch->findByURL( url );
+         KFileItem *fi = brnch->findByUrl( url );
          if( fi )
          {
             ret = static_cast<KFileTreeViewItem*>( fi->extraData( brnch ));
