@@ -14,6 +14,7 @@
 #include "kicondialog.h"
 #include <kbuttongroup.h>
 #include <k3iconviewsearchline.h>
+#include <assert.h>
 
 #include <kapplication.h>
 #include <klocale.h>
@@ -38,8 +39,8 @@
 #include <qfileinfo.h>
 #include <qtoolbutton.h>
 #include <qprogressbar.h>
+#include <ksvgrenderer.h>
 
-#include <QSvgRenderer>
 #include <QImage>
 #include <QPainter>
 
@@ -150,11 +151,12 @@ void KIconCanvas::slotLoadFiles()
 	else {
             // Special stuff for SVG icons
             img = QImage(60, 60, QImage::Format_ARGB32_Premultiplied);
-            QPainter p(&img);
-            QSvgRenderer renderer(*it);
-            if (renderer.isValid())
+            img.fill(0);
+            KSvgRenderer renderer(*it);
+            if (renderer.isValid()) {
+                QPainter p(&img);
                 renderer.render(&p);
-            p.end();
+            }
         }
 
 	if (img.isNull())
@@ -235,7 +237,7 @@ KIconDialog::KIconDialog(QWidget *parent)
     setButtons( Ok | Cancel );
     setDefaultButton( Ok );
 
-    mpLoader = KGlobal::iconLoader();
+    mpLoader = KIconLoader::global();
     init();
 }
 
@@ -262,18 +264,18 @@ void KIconDialog::init()
     setMainWidget(main);
 
     QVBoxLayout *top = new QVBoxLayout(main);
-    top->setSpacing( spacingHint() );
 
-    QGroupBox *bgroup = new QGroupBox(main );
-    bgroup->setTitle(  i18n("Icon Source"));
+    QGroupBox *bgroup = new QGroupBox(main);
+    bgroup->setTitle(i18n("Icon Source"));
+
     QVBoxLayout *vbox = new QVBoxLayout;
-    vbox->setSpacing(KDialog::spacingHint());
-    vbox->setMargin(KDialog::marginHint());
     bgroup->setLayout( vbox );
     top->addWidget(bgroup);
+
     QGridLayout *grid = new QGridLayout();
+    grid->setSpacing(KDialog::spacingHint());
     bgroup->layout()->addItem(grid);
-    grid->addItem(new QSpacerItem(0, 15), 0, 0);
+
     mpRb1 = new QRadioButton(i18n("S&ystem icons:"), bgroup);
     connect( mpRb1, SIGNAL( clicked() ), SLOT( slotSystemIconClicked() ) );
     grid->addWidget(mpRb1, 1, 0);
@@ -295,12 +297,6 @@ void KIconDialog::init()
     searchLayout->setSpacing(KDialog::spacingHint());
     top->addLayout(searchLayout);
 
-    QToolButton *clearSearch = new QToolButton(main);
-    clearSearch->setText(i18n("Clear Search"));
-    clearSearch->setToolTip(i18n("Clear Search"));
-    clearSearch->setIcon(SmallIconSet(QApplication::isRightToLeft() ? "clear_left" :"locationbar_erase"));
-    searchLayout->addWidget(clearSearch);
-
     QLabel *searchLabel = new QLabel(i18n("&Search:"), main);
     searchLayout->addWidget(searchLabel);
 
@@ -308,10 +304,6 @@ void KIconDialog::init()
     searchLayout->addWidget(d->searchLine);
     searchLabel->setBuddy(d->searchLine);
 
-
-    // signals and slots connections
-    connect(clearSearch, SIGNAL(clicked()), d->searchLine, SLOT(clear()));
-
     QString wtstr = i18n("Search interactively for icon names (e.g. folder).");
     searchLabel->setWhatsThis(wtstr);
     d->searchLine->setWhatsThis(wtstr);
@@ -333,13 +325,49 @@ void KIconDialog::init()
     // When pressing Ok or Cancel, stop loading icons
     connect(this, SIGNAL(hidden()), mpCanvas, SLOT(stopLoading()));
 
-    // The order must match the context definitions in K3Icon.
-    mpCombo->addItem(i18n("Actions"));
-    mpCombo->addItem(i18n("Applications"));
-    mpCombo->addItem(i18n("Devices"));
-    mpCombo->addItem(i18n("Filesystems"));
-    mpCombo->addItem(i18n("Mimetypes"));
+    static const char* const context_text[] = {
+        I18N_NOOP( "Actions" ),
+        I18N_NOOP( "Animations" ),
+        I18N_NOOP( "Applications" ),
+        I18N_NOOP( "Categories" ),
+        I18N_NOOP( "Devices" ),
+        I18N_NOOP( "Emblems" ),
+        I18N_NOOP( "Emotes" ),
+        I18N_NOOP( "Filesystems" ),
+        I18N_NOOP( "International" ),
+        I18N_NOOP( "Mimetypes" ),
+        I18N_NOOP( "Places" ),
+        I18N_NOOP( "Status" ) };
+    static const K3Icon::Context context_id[] = {
+        K3Icon::Action,
+        K3Icon::Animation,
+        K3Icon::Application,
+        K3Icon::Category,
+        K3Icon::Device,
+        K3Icon::Emblem,
+        K3Icon::Emote,
+        K3Icon::FileSystem,
+        K3Icon::International,
+        K3Icon::MimeType,
+        K3Icon::Place,
+        K3Icon::StatusIcon };
+    mNumContext = 0;
+    int cnt = sizeof( context_text ) / sizeof( context_text[ 0 ] );
+    // check all 3 arrays have same sizes
+    assert( cnt == sizeof( context_id ) / sizeof( context_id[ 0 ] )
+            && cnt == sizeof( mContextMap ) / sizeof( mContextMap[ 0 ] ));
+    for( int i = 0;
+         i < cnt;
+         ++i )
+    {
+        if( mpLoader->hasContext( context_id[ i ] ))
+        {
+            mpCombo->addItem(i18n( context_text[ i ] ));
+            mContextMap[ mNumContext++ ] = context_id[ i ];
+        }
+    }
     mpCombo->setFixedSize(mpCombo->sizeHint());
+
     mpBrowseBut->setFixedWidth(mpCombo->width());
 
     // Make the dialog a little taller
@@ -424,7 +452,7 @@ void KIconDialog::setup(K3Icon::Group group, K3Icon::Context context,
     mpCombo->setEnabled(!user);
     mpBrowseBut->setEnabled(user);
     mContext = context;
-    mpCombo->setCurrentIndex(mContext-1);
+    setContext( context );
 }
 
 void KIconDialog::setup(K3Icon::Group group, K3Icon::Context context,
@@ -441,8 +469,20 @@ void KIconDialog::setup(K3Icon::Group group, K3Icon::Context context,
     mpRb2->setEnabled( !lockUser || user );
     mpCombo->setEnabled(!user);
     mpBrowseBut->setEnabled( user && !lockCustomDir );
+    setContext( context );
+}
+
+void KIconDialog::setContext( K3Icon::Context context )
+{
     mContext = context;
-    mpCombo->setCurrentIndex(mContext-1);
+    for( int i = 0;
+         i < mNumContext;
+         ++i )
+        if( mContextMap[ i ] == context )
+        {
+            mpCombo->setCurrentIndex( i );
+            return;
+        }
 }
 
 void KIconDialog::setCustomLocation( const QString& location )
@@ -550,7 +590,7 @@ void KIconDialog::slotOtherIconClicked()
 
 void KIconDialog::slotContext(int id)
 {
-    mContext = static_cast<K3Icon::Context>(id+1);
+    mContext = static_cast<K3Icon::Context>( mContextMap[ id ] );
     showIcons();
 }
 
@@ -599,7 +639,7 @@ class KIconButton::KIconButtonPrivate
 KIconButton::KIconButton(QWidget *parent)
     : QPushButton(parent),d(new KIconButtonPrivate)
 {
-    init( KGlobal::iconLoader() );
+    init( KIconLoader::global() );
 }
 
 KIconButton::KIconButton(KIconLoader *loader, QWidget *parent)
