@@ -25,8 +25,8 @@
  * display a properties dialog.
  */
 
-#ifndef __propsdlg_h
-#define __propsdlg_h
+#ifndef __propsdialog_h
+#define __propsdialog_h
 
 #include <qstring.h>
 #include <q3ptrlist.h>
@@ -44,11 +44,8 @@ class KLineEdit;
 class KUrlRequester;
 class QPushButton;
 class KIconButton;
-class KPropsDlgPlugin;
+class KPropertiesDialogPlugin;
 class QComboBox;
-class Q3ListBox;
-
-#define KPropsPage KPropsDlgPlugin
 
 class KJob;
 namespace KIO { class Job; }
@@ -64,8 +61,9 @@ namespace KIO { class Job; }
  *
  * \image html kpropertiesdialog.png "Typical KProperties Dialog"
  *
- * This class must be created with (void)new KPropertiesDialog(...)
- * It will take care of deleting itself.
+ * The best way to display the properties dialog is to use showDialog().
+ * Otherwise, you should use (void)new KPropertiesDialog(...)
+ * It will take care of deleting itself when closed.
  *
  * If you are looking for more flexibility, see KFileMetaInfo and
  * KFileMetaInfoWidget.
@@ -82,7 +80,7 @@ public:
    * @param _items the list of items to check.
    * @return true if there are any property pages, otherwise false.
    */
-  static bool canDisplay( KFileItemList _items );
+  static bool canDisplay( const KFileItemList& _items );
 
   /**
    * Brings up a Properties dialog, as shown above.
@@ -94,12 +92,9 @@ public:
    * @param item file item whose properties should be displayed.
    * @param parent is the parent of the dialog widget.
    * @param name is the internal name.
-   * @param modal tells the dialog whether it should be modal.
-   * @param autoShow tells the dialog whether it should show itself automatically.
    */
   KPropertiesDialog( KFileItem * item,
-                     QWidget* parent = 0L, const char* name = 0L,
-                     bool modal = false, bool autoShow = true);
+                     QWidget* parent = 0 );
 
   /**
    * \overload
@@ -112,28 +107,9 @@ public:
    * @param _items list of file items whose properties should be displayed.
    * @param parent is the parent of the dialog widget.
    * @param name is the internal name.
-   * @param modal tells the dialog whether it should be modal.
-   * @param autoShow tells the dialog whether it should show itself automatically.
    */
-  KPropertiesDialog( KFileItemList _items,
-                     QWidget *parent = 0L, const char* name = 0L,
-                     bool modal = false, bool autoShow = true);
-
-#ifndef KDE_NO_COMPAT
-  /**
-   * @deprecated  You should use the following constructor instead of this one.
-   * The only change that is required is to delete the _mode argument.
-   *
-   * @param _url the URL whose properties should be displayed
-   * @param _mode unused.
-   * @param parent is the parent of the dialog widget.
-   * @param name is the internal name.
-   * @param modal tells the dialog whether it should be modal.
-   * @param autoShow tells the dialog whether it should show itself automatically.  */
-  KDE_CONSTRUCTOR_DEPRECATED KPropertiesDialog( const KUrl& _url, mode_t _mode,
-                     QWidget* parent = 0L, const char* name = 0L,
-                     bool modal = false, bool autoShow = true);
-#endif
+  KPropertiesDialog( const KFileItemList& _items,
+                     QWidget *parent = 0 );
 
   /**
    * Brings up a Properties dialog. Convenience constructor for
@@ -143,18 +119,16 @@ public:
    * @param _url the URL whose properties should be displayed
    * @param parent is the parent of the dialog widget.
    * @param name is the internal name.
-   * @param modal tells the dialog whether it should be modal.
-   * IMPORTANT: This constructor, together with modal=true, leads to a grave
+   *
+   * IMPORTANT: This constructor, together with exec(), leads to a grave
    * display bug (due to KIO::stat() being run before the dialog has all the
    * necessary information). Do not use this combination for now.
+   * TODO: Check if the above is still true with Qt4.
    * For local files with a known mimetype, simply create a KFileItem and pass
    * it to the other constructor.
-   *
-   * @param autoShow tells the dialog whethr it should show itself automatically.
    */
   KPropertiesDialog( const KUrl& _url,
-                     QWidget* parent = 0L, const char* name = 0L,
-                     bool modal = false, bool autoShow = true);
+                     QWidget* parent = 0 );
 
   /**
    * Creates a properties dialog for a new .desktop file (whose name
@@ -167,13 +141,10 @@ public:
    * like mimetype.desktop
    * @param parent is the parent of the dialog widget.
    * @param name is the internal name.
-   * @param modal tells the dialog whether it should be modal.
-   * @param autoShow tells the dialog whethr it should show itself automatically.
    */
   KPropertiesDialog( const KUrl& _tempUrl, const KUrl& _currentDir,
                      const QString& _defaultName,
-                     QWidget* parent = 0L, const char* name = 0L,
-                     bool modal = false, bool autoShow = true);
+                     QWidget* parent = 0 );
 
   /**
    * Creates an empty properties dialog (for applications that want use
@@ -184,8 +155,8 @@ public:
    * @param name is the internal name.
    * @param modal tells the dialog whether it should be modal.
    */
-  KPropertiesDialog (const QString& title,
-                     QWidget* parent = 0L, const char* name = 0L, bool modal = false);
+  KPropertiesDialog(const QString& title,
+                    QWidget* parent = 0);
 
   /**
    * Cleans up the properties dialog and frees any associated resources,
@@ -200,10 +171,10 @@ public:
    * On MS Windows, if @p item points to a local file, native (non modal) property
    * dialog is displayed (@p parent and @p modal are ignored in this case).
    *
-   * @return true on succesfull dialog displaying (can be false on win32).
+   * @return true on successful dialog displaying (can be false on win32).
    */
   static bool showDialog(KFileItem* item, QWidget* parent = 0,
-                         const char* name = 0, bool modal = false);
+                         bool modal = true);
 
   /**
    * Immediately displays a Properties dialog using constructor with
@@ -211,10 +182,10 @@ public:
    * On MS Windows, if @p _url points to a local file, native (non modal) property
    * dialog is displayed (@p parent and @p modal are ignored in this case).
    *
-   * @return true on succesfull dialog displaying (can be false on win32).
+   * @return true on successful dialog displaying (can be false on win32).
    */
   static bool showDialog(const KUrl& _url, QWidget* parent = 0,
-                         const char* name = 0, bool modal = false);
+                         bool modal = true);
 
   /**
    * Immediately displays a Properties dialog using constructor with
@@ -223,25 +194,25 @@ public:
    * to a local file, native (non modal) property dialog is displayed
    * (@p parent and @p modal are ignored in this case).
    *
-   * @return true on succesfull dialog displaying (can be false on win32).
+   * @return true on successful dialog displaying (can be false on win32).
    */
   static bool showDialog(const KFileItemList& _items, QWidget* parent = 0,
-                         const char* name = 0, bool modal = false);
+                         bool modal = true);
 
   /**
    * Adds a "3rd party" properties plugin to the dialog.  Useful
    * for extending the properties mechanism.
    *
-   * To create a new plugin type, inherit from the base class KPropsDlgPlugin
+   * To create a new plugin type, inherit from the base class KPropertiesDialogPlugin
    * and implement all the methods. If you define a service .desktop file
    * for your plugin, you do not need to call insertPlugin().
    *
-   * @param plugin is a pointer to the KPropsDlgPlugin. The Properties
-   *        dialog will do destruction for you. The KPropsDlgPlugin \b must
+   * @param plugin is a pointer to the KPropertiesDialogPlugin. The Properties
+   *        dialog will do destruction for you. The KPropertiesDialogPlugin \b must
    *        have been created with the KPropertiesDialog as its parent.
-   * @see KPropsDlgPlugin
+   * @see KPropertiesDialogPlugin
    */
-  void insertPlugin (KPropsDlgPlugin *plugin);
+  void insertPlugin (KPropertiesDialogPlugin *plugin);
 
   /**
    * The URL of the file that has its properties being displayed.
@@ -295,7 +266,7 @@ public:
    * Renames the item to the specified name. This can only be called if
    * the dialog applies to a single file or URL.
    * @param _name new filename, encoded.
-   * \see FilePropsDlgPlugin::applyChanges
+   * \see FilePropsDialogPlugin::applyChanges
    */
   void rename( const QString& _name );
 
@@ -369,7 +340,7 @@ private:
   /**
    * Common initialization for all constructors
    */
-  void init (bool modal = false, bool autoShow = true);
+  void init();
 
   /**
    * Inserts all pages in the dialog.
@@ -395,7 +366,7 @@ private:
   /**
    * List of all plugins inserted ( first one first )
    */
-  Q3PtrList<KPropsDlgPlugin> m_pageList;
+  Q3PtrList<KPropertiesDialogPlugin> m_pageList;
 
 private Q_SLOTS:
   void slotStatResult( KJob * ); // No longer used
@@ -413,14 +384,14 @@ private:
  * A plugin in itself is just a library containing code, not a dialog's page.
  * It's up to the plugin to insert pages into the parent dialog.
  *
- * To make a plugin available, define a service that implements the KPropsDlg/Plugin
+ * To make a plugin available, define a service that implements the KPropertiesDialog/Plugin
  * servicetype, as well as the mimetypes for which the plugin should be created.
- * For instance, ServiceTypes=KPropsDlg/Plugin,text/html,application/x-mymimetype.
+ * For instance, ServiceTypes=KPropertiesDialog/Plugin,text/html,application/x-mymimetype.
  *
  * You can also include X-KDE-Protocol=file if you want that plugin
  * to be loaded only for local files, for instance.
  */
-class KIO_EXPORT KPropsDlgPlugin : public QObject
+class KIO_EXPORT KPropertiesDialogPlugin : public QObject
 {
   Q_OBJECT
 public:
@@ -429,8 +400,8 @@ public:
    * To insert tabs into the properties dialog, use the add methods provided by
    * KPageDialog (the properties dialog is a KPageDialog).
    */
-  KPropsDlgPlugin( KPropertiesDialog *_props );
-  virtual ~KPropsDlgPlugin();
+  KPropertiesDialogPlugin( KPropertiesDialog *_props );
+  virtual ~KPropertiesDialogPlugin();
 
   /**
    * Applies all changes to the file.
@@ -467,8 +438,8 @@ protected:
 
   int fontHeight;
 private:
-  class KPropsDlgPluginPrivate;
-  KPropsDlgPluginPrivate* const d;
+  class KPropertiesDialogPluginPrivate;
+  KPropertiesDialogPluginPrivate* const d;
 };
 
 /**
@@ -476,7 +447,7 @@ private:
  *  This plugin displays the name of the file, its size and access times.
  * @internal
  */
-class KIO_EXPORT KFilePropsPlugin : public KPropsDlgPlugin
+class KIO_EXPORT KFilePropsPlugin : public KPropertiesDialogPlugin
 {
   Q_OBJECT
 public:
@@ -496,7 +467,7 @@ public:
   /**
    * Tests whether the files specified by _items need a 'General' plugin.
    */
-  static bool supports( KFileItemList _items );
+  static bool supports( const KFileItemList& _items );
 
   /**
    * Called after all plugins applied their changes
@@ -554,7 +525,7 @@ private:
  * the owner of a file.
  * @internal
  */
-class KIO_EXPORT KFilePermissionsPropsPlugin : public KPropsDlgPlugin
+class KIO_EXPORT KFilePermissionsPropsPlugin : public KPropertiesDialogPlugin
 {
   Q_OBJECT
 public:
@@ -582,7 +553,7 @@ public:
   /**
    * Tests whether the file specified by _items needs a 'Permissions' plugin.
    */
-  static bool supports( KFileItemList _items );
+  static bool supports( const KFileItemList& _items );
 
 private Q_SLOTS:
 
@@ -643,7 +614,7 @@ private:
  * Such files are used to represent a program in kicker and konqueror.
  * @internal
  */
-class KIO_EXPORT KUrlPropsPlugin : public KPropsDlgPlugin
+class KIO_EXPORT KUrlPropsPlugin : public KPropertiesDialogPlugin
 {
   Q_OBJECT
 public:
@@ -655,7 +626,7 @@ public:
 
   virtual void applyChanges();
 
-  static bool supports( KFileItemList _items );
+  static bool supports( const KFileItemList& _items );
 
 private:
   KUrlRequester *URLEdit;
@@ -678,7 +649,7 @@ private:
  * Type=MimeType
  * @internal
  */
-class KIO_EXPORT KBindingPropsPlugin : public KPropsDlgPlugin
+class KIO_EXPORT KBindingPropsPlugin : public KPropertiesDialogPlugin
 {
   Q_OBJECT
 public:
@@ -689,7 +660,7 @@ public:
   virtual ~KBindingPropsPlugin();
 
   virtual void applyChanges();
-  static bool supports( KFileItemList _items );
+  static bool supports( const KFileItemList& _items );
 
 private:
 
@@ -708,7 +679,7 @@ private:
  * Properties plugin for device .desktop files
  * @internal
  */
-class KIO_EXPORT KDevicePropsPlugin : public KPropsDlgPlugin
+class KIO_EXPORT KDevicePropsPlugin : public KPropertiesDialogPlugin
 {
   Q_OBJECT
 public:
@@ -717,7 +688,7 @@ public:
 
   virtual void applyChanges();
 
-  static bool supports( KFileItemList _items );
+  static bool supports( const KFileItemList& _items );
 
 private Q_SLOTS:
   void slotActivated( int );
@@ -760,7 +731,7 @@ class Ui_KPropertiesDesktopBase;
  * Such files are used to represent a program in kicker and konqueror.
  * @internal
  */
-class KIO_EXPORT KDesktopPropsPlugin : public KPropsDlgPlugin
+class KIO_EXPORT KDesktopPropsPlugin : public KPropertiesDialogPlugin
 {
   Q_OBJECT
 public:
@@ -772,14 +743,13 @@ public:
 
   virtual void applyChanges();
 
-  static bool supports( KFileItemList _items );
+  static bool supports( const KFileItemList& _items );
 
 public Q_SLOTS:
   void slotAddFiletype();
   void slotDelFiletype();
   void slotBrowseExec();
   void slotAdvanced();
-  void slotSelectMimetype();
 
 private:
   void checkCommandChanged();
@@ -791,7 +761,7 @@ private:
   QString m_origCommandStr;
   QString m_terminalOptionStr;
   QString m_suidUserStr;
-  QString m_dcopServiceType;
+  QString m_dbusStartusType;
   bool m_terminalBool;
   bool m_terminalCloseBool;
   bool m_suidBool;
@@ -802,107 +772,5 @@ private:
   KDesktopPropsPluginPrivate* d;
 };
 
-/**
- * Used to edit the files containing
- * [Desktop Entry]
- * Type=Application
- *
- * Such files are used to represent a program in kicker and konqueror.
- * @internal
- * @deprecated replaced with KDesktopPropsPlugin
- */
- /// Remove in KDE4
-class KIO_EXPORT_DEPRECATED KExecPropsPlugin : public KPropsDlgPlugin
-{
-  Q_OBJECT
-public:
-  /**
-   * Constructor
-   */
-  KExecPropsPlugin( KPropertiesDialog *_props );
-  virtual ~KExecPropsPlugin();
-
-  virtual void applyChanges();
-
-  static bool supports( KFileItemList _items );
-
-public Q_SLOTS:
-  void slotBrowseExec();
-
-private Q_SLOTS:
-  void enableCheckedEdit();
-  void enableSuidEdit();
-
-private:
-
-    QLabel *terminalLabel;
-    QLabel *suidLabel;
-    KLineEdit *execEdit;
-    QCheckBox *terminalCheck;
-    QCheckBox *suidCheck;
-    KLineEdit *terminalEdit;
-    KLineEdit *suidEdit;
-    KLineEdit *swallowExecEdit;
-    KLineEdit *swallowTitleEdit;
-    QPushButton *execBrowse;
-
-    QString execStr;
-    QString swallowExecStr;
-    QString swallowTitleStr;
-    QString termOptionsStr;
-    bool termBool;
-    bool suidBool;
-    QString suidUserStr;
-
-    class KExecPropsPluginPrivate;
-    KExecPropsPluginPrivate* const d;
-};
-
-/**
- * Used to edit the files containing
- * [Desktop Entry]
- * Type=Application
- *
- * Such files are used to represent a program in kicker and konqueror.
- * @internal
- * @deprecated replaced with KDesktopPropsPlugin
- */
- /// Remove in KDE4
-class KIO_EXPORT_DEPRECATED KApplicationPropsPlugin : public KPropsDlgPlugin
-{
-  Q_OBJECT
-public:
-  /**
-   * Constructor
-   */
-  KApplicationPropsPlugin( KPropertiesDialog *_props );
-  virtual ~KApplicationPropsPlugin();
-
-  virtual void applyChanges();
-
-  static bool supports( KFileItemList _items );
-
-public Q_SLOTS:
-  void slotDelExtension();
-  void slotAddExtension();
-
-private Q_SLOTS:
-  void updateButton();
-
-private:
-  void addMimeType( const QString & name );
-
-  QLineEdit *commentEdit;
-  QLineEdit *genNameEdit;
-  QLineEdit *nameEdit;
-  Q3ListBox  *extensionsList;
-  Q3ListBox  *availableExtensionsList;
-  QPushButton *addExtensionButton;
-  QPushButton *delExtensionButton;
-
-  class KApplicationPropsPluginPrivate;
-  KApplicationPropsPluginPrivate* const d;
-};
-
 #endif
 
