@@ -24,7 +24,7 @@
 #include <kglobal.h>
 #include <kicondialog.h>
 #include <kiconloader.h>
-#include <kinstance.h>
+#include <kcomponentdata.h>
 #include <klineedit.h>
 #include <klocale.h>
 #include <kmimetype.h>
@@ -43,10 +43,20 @@
 #include <qpainter.h>
 #include <qmenu.h>
 #include <qstyle.h>
-
+#include <qitemdelegate.h>
 
 #include <unistd.h>
 #include <kvbox.h>
+#include <kconfiggroup.h>
+
+class KUrlBarItemDelegate : public QItemDelegate
+{
+public:
+    KUrlBarItemDelegate(QObject* parent) : QItemDelegate(parent) {}
+
+    virtual void paint(QPainter* painter,const QStyleOptionViewItem& option,const QModelIndex& index) const;
+    virtual QSize sizeHint(const QStyleOptionViewItem& option , const QModelIndex& index) const;
+};
 
 ///////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////
@@ -80,7 +90,7 @@ public:
 KUrlBarItem::KUrlBarItem( KUrlBar *parent,
                           const KUrl& url, bool persistent, const QString& description,
                           const QString& icon, K3Icon::Group group )
-    : Q3ListBoxPixmap( KIconLoader::unknown() /*, parent->listBox()*/ ),
+    : QListWidgetItem(),
       m_url( url ),
       m_pixmap( ),
       m_parent( parent ),
@@ -92,7 +102,7 @@ KUrlBarItem::KUrlBarItem( KUrlBar *parent,
 KUrlBarItem::KUrlBarItem( KUrlBar *parent,
                           const KUrl& url, const QString& description,
                           const QString& icon, K3Icon::Group group )
-    : Q3ListBoxPixmap( KIconLoader::unknown() /*, parent->listBox()*/ ),
+    : QListWidgetItem(),
       m_url( url ),
       m_pixmap(  ),
       m_parent( parent ),
@@ -104,9 +114,11 @@ KUrlBarItem::KUrlBarItem( KUrlBar *parent,
 void KUrlBarItem::init( const QString& icon, K3Icon::Group group,
                         const QString& description, bool persistent )
 {
+
+    qDebug() << __FUNCTION__ << "created noew KUrlBarItem with description" << description;
+
     d->isPersistent = persistent;
 
-    setCustomHighlighting( true );
     setIcon( icon, group );
     setDescription( description );
 }
@@ -116,7 +128,7 @@ KUrlBarItem::~KUrlBarItem()
     delete d;
 }
 
-void KUrlBarItem::setURL( const KUrl& url )
+void KUrlBarItem::setUrl( const KUrl& url )
 {
     m_url = url;
     if ( m_description.isEmpty() )
@@ -129,10 +141,10 @@ void KUrlBarItem::setIcon( const QString& icon, K3Icon::Group group )
     m_group = group;
 
     if ( icon.isEmpty() )
-        m_pixmap = KIO::pixmapForURL( m_url, 0, group, iconSize() );
+        QListWidgetItem::setIcon( KIO::pixmapForUrl( m_url, 0, group, iconSize() ) );
     else
-        m_pixmap = KGlobal::iconLoader()->loadIcon( icon, group, iconSize(),
-                                                    K3Icon::DefaultState );
+        QListWidgetItem::setIcon( KIconLoader::global()->loadIcon( icon, group, iconSize(),
+                                                    K3Icon::DefaultState ) ); 
 }
 
 void KUrlBarItem::setDescription( const QString& desc )
@@ -167,130 +179,59 @@ int KUrlBarItem::iconSize() const
     return m_parent->iconSize();
 }
 
-void KUrlBarItem::paint( QPainter *p )
+QSize KUrlBarItemDelegate::sizeHint(const QStyleOptionViewItem& option , const QModelIndex& index) const
 {
-    Q3ListBox *box = listBox();
-    int w = width( box );
-    static const int margin = KDialog::spacingHint();
+    int width = 0;
+    int height = 0;
 
-    // draw sunken selection
-    if ( isCurrent() || isSelected() ) {
-        int h = height( box );
+    QString text = index.data(Qt::DisplayRole).toString();
 
-        QBrush brush = box->palette().brush( QPalette::Highlight );
-        p->fillRect( 0, 0, w, h, brush );
-        QPen pen = p->pen();
-        QPen oldPen = pen;
-        pen.setColor( box->palette().color( QPalette::Mid ) );
-        p->setPen( pen );
+    QFontMetrics metrics(option.font);
 
-        p->drawPoint( 0, 0 );
-        p->drawPoint( 0, h - 1 );
-        p->drawPoint( w - 1, 0 );
-        p->drawPoint( w - 1, h - 1 );
+    width = metrics.width(text) + option.decorationSize.width() + 
+                   KDialog::spacingHint() * 2;
+    
+    height = qMax( metrics.height() , option.decorationSize.height() ) +
+                    KDialog::spacingHint() * 2;
 
-        p->setPen( oldPen );
-    }
-
-    if ( m_parent->iconSize() < K3Icon::SizeMedium ) {
-        // small icon -> draw icon next to text
-
-        // ### mostly cut & paste of QListBoxPixmap::paint() until Qt 3.1
-        // (where it will properly use pixmap() instead of the internal pixmap)
-        const QPixmap *pm = pixmap();
-        int yPos = qMax( 0, (height(box) - pm->height())/2 );
-
-        p->drawPixmap( margin, yPos, *pm );
-        if ( !text().isEmpty() ) {
-            QFontMetrics fm = p->fontMetrics();
-            if ( pm->height() < fm.height() )
-                yPos = fm.ascent() + fm.leading()/2;
-            else
-                yPos = pm->height()/2 - fm.height()/2 + fm.ascent();
-
-            yPos += margin;
-            int stringWidth = box->width() - pm->width() - 2 - (margin * 2);
-            QString visibleText = fm.elidedText( text(), Qt::ElideRight, stringWidth );
-            int xPos = pm->width() + margin + 2;
-
-            if ( isCurrent() || isSelected() ) {
-                p->setPen( box->palette().color( QPalette::Highlight ).dark(115) );
-                p->drawText( xPos + ( QApplication::isRightToLeft() ? -1 : 1),
-                             yPos + 1, visibleText );
-                p->setPen( box->palette().color( QPalette::HighlightedText ) );
-            }
-
-            p->drawText( xPos, yPos, visibleText );
-        }
-        // end cut & paste (modulo pixmap centering)
-    }
-
-    else {
-        // big icons -> draw text below icon
-        int y = margin;
-        const QPixmap *pm = pixmap();
-
-        if ( !pm->isNull() ) {
-            int x = (w - pm->width()) / 2;
-            x = qMax( x, margin );
-            p->drawPixmap( x, y, *pm );
-        }
-
-        if ( !text().isEmpty() ) {
-            QFontMetrics fm = p->fontMetrics();
-            y += pm->height() + fm.height() - fm.descent();
-
-            int stringWidth = box->width() - (margin * 2);
-            QString visibleText = fm.elidedText( text(), Qt::ElideRight, stringWidth );
-            int x = (w - fm.width( visibleText )) / 2;
-            x = qMax( x, margin );
-
-            if ( isCurrent() || isSelected() ) {
-                p->setPen( box->palette().color( QPalette::Highlight ).dark(115) );
-                p->drawText( x + ( QApplication::isRightToLeft() ? -1 : 1),
-                             y + 1, visibleText );
-                p->setPen( box->palette().color( QPalette::HighlightedText ) );
-            }
-
-            p->drawText( x, y, visibleText );
-        }
-    }
+    return QSize( width , height );
 }
 
-QSize KUrlBarItem::sizeHint() const
+void KUrlBarItemDelegate::paint( QPainter *painter , const QStyleOptionViewItem& option , 
+                                 const QModelIndex& index) const
 {
-    int wmin = 0;
-    int hmin = 0;
-    const KUrlBarListBox *lb =static_cast<const KUrlBarListBox*>(listBox());
+    const QString text = index.data(Qt::DisplayRole).toString();
+    const QIcon icon = index.data(Qt::DecorationRole).value<QIcon>();
 
-    if ( m_parent->iconSize() < K3Icon::SizeMedium ) {
-        wmin = Q3ListBoxPixmap::width( lb ) + KDialog::spacingHint() * 2;
-        hmin = Q3ListBoxPixmap::height( lb ) + KDialog::spacingHint() * 2;
-    }
-    else {
-        wmin = qMax(lb->fontMetrics().width(text()), pixmap()->width()) + KDialog::spacingHint() * 2;
-        hmin = lb->fontMetrics().lineSpacing() + pixmap()->height() + KDialog::spacingHint() * 2;
-    }
+    QRect decorationRect = option.rect;
+    QRect textRect = option.rect;
 
-    if ( lb->isVertical() )
-        wmin = qMin( wmin, lb->viewport()->sizeHint().width() );
-    else
-        hmin = qMin( hmin, lb->viewport()->sizeHint().height() );
+    const int decorationMargin = (option.rect.height() -
+                                  option.decorationSize.height()) / 2;
 
-    return QSize( wmin, hmin );
+    decorationRect.setTop( decorationRect.top() + decorationMargin );
+    decorationRect.setBottom( decorationRect.bottom() - decorationMargin );
+    decorationRect.setLeft( KDialog::spacingHint() );
+    decorationRect.setWidth( option.decorationSize.width() );
+    
+    textRect.setLeft(   decorationRect.right()  
+                      + KDialog::spacingHint() );
+
+    painter->drawPixmap( decorationRect , icon.pixmap(decorationRect.size()) );
+    painter->drawText( textRect , Qt::AlignVCenter , text );
 }
 
-int KUrlBarItem::width( const Q3ListBox *lb ) const
+int KUrlBarItem::width( const QListWidget *lb ) const
 {
-    if ( static_cast<const KUrlBarListBox *>( lb )->isVertical() )
+    if ( static_cast<const KUrlBarListWidget *>( lb )->isVertical() )
         return qMax( sizeHint().width(), lb->viewport()->width() );
     else
         return sizeHint().width();
 }
 
-int KUrlBarItem::height( const Q3ListBox *lb ) const
+int KUrlBarItem::height( const QListWidget *lb ) const
 {
-    if ( static_cast<const KUrlBarListBox *>( lb )->isVertical() )
+    if ( static_cast<const KUrlBarListWidget *>( lb )->isVertical() )
         return sizeHint().height();
     else
         return qMax( sizeHint().height(), lb->viewport()->height() );
@@ -372,7 +313,7 @@ KUrlBarItem * KUrlBar::insertItem(const KUrl& url, const QString& description,
 {
     KUrlBarItem *item = new KUrlBarItem(this, url, description, icon, group);
     item->setApplicationLocal( applicationLocal );
-    m_listBox->insertItem( item );
+    m_listBox->addItem( item );
     return item;
 }
 
@@ -380,7 +321,7 @@ KUrlBarItem * KUrlBar::insertDynamicItem(const KUrl& url, const QString& descrip
                                          const QString& icon, K3Icon::Group group )
 {
     KUrlBarItem *item = new KUrlBarItem(this, url, false, description, icon, group);
-    m_listBox->insertItem( item );
+    m_listBox->addItem( item );
     return item;
 }
 
@@ -400,12 +341,13 @@ Qt::Orientation KUrlBar::orientation() const
     return m_listBox->orientation();
 }
 
-void KUrlBar::setListBox( KUrlBarListBox *view )
+void KUrlBar::setListBox( KUrlBarListWidget *view )
 {
     delete m_listBox;
 
     if ( !view ) {
-        m_listBox = new KUrlBarListBox( this, "urlbar listbox" );
+        m_listBox = new KUrlBarListWidget( this );
+        m_listBox->setObjectName("urlbar listbox" );
         setOrientation( Qt::Vertical );
     }
     else {
@@ -415,19 +357,34 @@ void KUrlBar::setListBox( KUrlBarListBox *view )
         m_listBox->resize( width(), height() );
     }
 
-    m_listBox->setSelectionMode( KListBox::Single );
+    m_listBox->setSelectionMode( QAbstractItemView::SingleSelection );
     paletteChange( palette() );
     m_listBox->setFocusPolicy( Qt::TabFocus );
 
-    connect( m_listBox, SIGNAL( mouseButtonClicked( int, Q3ListBoxItem *, const QPoint & ) ),
-             SLOT( slotSelected( int, Q3ListBoxItem * )));
+    m_listBox->setFrameStyle( QFrame::Plain | QFrame::Box );
+
+    connect( m_listBox, SIGNAL( itemClicked(QListWidgetItem *) ),
+             SLOT( slotSelected(QListWidgetItem *)));
+
+
+#ifdef __GNUC__
+#warning "KDE4 - Port the drag/drop handling."
+#endif
+
+    // These signals refer to the Q3ListBox class and are not
+    // available in QListWidget.  They need to be ported
+
+    m_listBox->setContextMenuPolicy(Qt::CustomContextMenu);
+    
     connect( m_listBox, SIGNAL( dropped( QDropEvent * )),
              this, SLOT( slotDropped( QDropEvent * )));
-    connect( m_listBox, SIGNAL( contextMenuRequested( Q3ListBoxItem *,
-                                                      const QPoint& )),
-             SLOT( slotContextMenuRequested( Q3ListBoxItem *, const QPoint& )));
-    connect( m_listBox, SIGNAL( returnPressed( Q3ListBoxItem * ) ),
-             SLOT( slotSelected( Q3ListBoxItem * ) ));
+    connect( m_listBox, SIGNAL( customContextMenuRequested(const QPoint &)),
+             SLOT( slotContextMenuRequested( const QPoint& )));
+    
+    connect( m_listBox, SIGNAL( returnPressed( QListWidgetItem * ) ),
+             SLOT( slotSelected( QListWidgetItem * ) ));
+
+    // --  
 }
 
 void KUrlBar::setIconSize( int size )
@@ -438,10 +395,9 @@ void KUrlBar::setIconSize( int size )
     m_iconSize = size;
 
     // reload the icons with the new size
-    KUrlBarItem *item = static_cast<KUrlBarItem*>( m_listBox->firstItem() );
-    while ( item ) {
+    for ( int rowIndex = 0 ; rowIndex < m_listBox->count() ; rowIndex++ ) {
+        KUrlBarItem *item = static_cast<KUrlBarItem*>( m_listBox->item(rowIndex) );
         item->setIcon( item->icon(), item->iconGroup() );
-        item = static_cast<KUrlBarItem*>( item->next() );
     }
 
     resize( sizeHint() );
@@ -460,9 +416,9 @@ void KUrlBar::resizeEvent( QResizeEvent *e )
 }
 
 
-void KUrlBar::paletteChange( const QPalette & )
+void KUrlBar::paletteChange( const QPalette & pal )
 {
-    QPalette pal = palette();
+    /*QPalette pal = palette();
     QColor selectedTextColor = pal.color( QPalette::Normal, QPalette::BrightText );
     QColor foreground = pal.color( QPalette::Normal, QPalette::Foreground );
     pal.setColor( QPalette::Normal,   QPalette::Base, Qt::gray );
@@ -471,7 +427,7 @@ void KUrlBar::paletteChange( const QPalette & )
     pal.setColor( QPalette::Inactive, QPalette::Base, Qt::gray );
     pal.setColor( QPalette::Inactive, QPalette::HighlightedText, selectedTextColor );
     pal.setColor( QPalette::Inactive, QPalette::Text, foreground );
-
+*/
     setPalette( pal );
 }
 
@@ -526,15 +482,7 @@ QSize KUrlBar::minimumSizeHint() const
     return QSize( w, h );
 }
 
-void KUrlBar::slotSelected( int button, Q3ListBoxItem *item )
-{
-    if ( button != Qt::LeftButton )
-        return;
-
-    slotSelected( item );
-}
-
-void KUrlBar::slotSelected( Q3ListBoxItem *item )
+void KUrlBar::slotSelected( QListWidgetItem *item )
 {
     if ( item && item != m_activeItem )
         m_activeItem = static_cast<KUrlBarItem*>( item );
@@ -555,16 +503,18 @@ void KUrlBar::setCurrentItem( const KUrl& url )
         return;
 
     bool hasURL = false;
-    Q3ListBoxItem *item = m_listBox->firstItem();
-    while ( item ) {
+    
+    for (int rowIndex = 0 ; rowIndex < m_listBox->count() ; rowIndex++ )
+    {
+        QListWidgetItem *item = m_listBox->item(rowIndex);
+    
       if ( static_cast<KUrlBarItem*>( item )->url().url(KUrl::RemoveTrailingSlash) == u ) {
             m_activeItem = static_cast<KUrlBarItem*>( item );
             m_listBox->setCurrentItem( item );
-            m_listBox->setSelected( item, true );
+            item->setSelected(true);
             hasURL = true;
             break;
         }
-        item = item->next();
     }
 
     if ( !hasURL ) {
@@ -575,13 +525,13 @@ void KUrlBar::setCurrentItem( const KUrl& url )
 
 KUrlBarItem * KUrlBar::currentItem() const
 {
-    Q3ListBoxItem *item = m_listBox->item( m_listBox->currentItem() );
+    QListWidgetItem *item = m_listBox->item( m_listBox->currentRow() );
     if ( item )
         return static_cast<KUrlBarItem *>( item );
     return 0L;
 }
 
-KUrl KUrlBar::currentURL() const
+KUrl KUrlBar::currentUrl() const
 {
     KUrlBarItem *item = currentItem();
     return item ? item->url() : KUrl();
@@ -612,7 +562,7 @@ void KUrlBar::readConfig( KConfig *appConfig, const QString& itemGroup )
 void KUrlBar::readItem( int i, KConfigBase *config, bool applicationLocal )
 {
     QString number = QString::number( i );
-    KUrl url( config->readPathEntry( QString("URL_") + number ));
+    KUrl url( config->readPathEntry( QString(QLatin1String("URL_") + number).toUtf8().constData() ));
     if ( !url.isValid() || !KProtocolInfo::isKnownProtocol( url ))
         return; // nothing we could do.
 
@@ -636,21 +586,21 @@ void KUrlBar::writeConfig( KConfig *config, const QString& itemGroup )
 
     int i = 0;
     int numLocal = 0;
-    KUrlBarItem *item = static_cast<KUrlBarItem*>( m_listBox->firstItem() );
-
-    while ( item )
+    
+    for ( int rowIndex = 0 ; rowIndex < m_listBox->count() ; rowIndex++ )
     {
+        KUrlBarItem *item = static_cast<KUrlBarItem*>( m_listBox->item(rowIndex) );
+
         if ( item->isPersistent() ) // we only save persistent items
         {
             if ( item->applicationLocal() )
             {
-                writeItem( item, numLocal, config, false );
+                writeItem( item, numLocal, &cg, false );
                 numLocal++;
             }
 
             i++;
         }
-        item = static_cast<KUrlBarItem*>( item->next() );
     }
     cg.writeEntry("Number of Entries", numLocal);
 
@@ -661,20 +611,20 @@ void KUrlBar::writeConfig( KConfig *config, const QString& itemGroup )
         config->setGroup( itemGroup + " (Global)" );
 
         int numGlobals = 0;
-        item = static_cast<KUrlBarItem*>( m_listBox->firstItem() );
 
-        while ( item )
+        for ( int rowIndex = 0 ; rowIndex < m_listBox->count() ; rowIndex++ )
         {
-            if ( item->isPersistent() ) // we only save persistent items
+            KUrlBarItem* urlBarItem = static_cast<KUrlBarItem*>( m_listBox->item(rowIndex) );
+
+            if ( urlBarItem->isPersistent() ) // we only save persistent items
             {
-                if ( !item->applicationLocal() )
+                if ( !urlBarItem->applicationLocal() )
                 {
-                    writeItem( item, numGlobals, config, true );
+                    writeItem( urlBarItem, numGlobals, config, true );
                     numGlobals++;
                 }
             }
 
-            item = static_cast<KUrlBarItem*>( item->next() );
         }
         config->writeEntry("Number of Entries", numGlobals,
                            KConfigBase::Normal|KConfigBase::Global);
@@ -683,7 +633,7 @@ void KUrlBar::writeConfig( KConfig *config, const QString& itemGroup )
     m_isModified = false;
 }
 
-void KUrlBar::writeItem( KUrlBarItem *item, int i, KConfig *config,
+void KUrlBar::writeItem( KUrlBarItem *item, int i, KConfigBase *config,
                          bool global )
 {
     if ( !item->isPersistent() )
@@ -698,10 +648,10 @@ void KUrlBar::writeItem( KUrlBarItem *item, int i, KConfig *config,
     KConfigBase::WriteConfigFlags flags = KConfigBase::Normal;
     if ( global )
         flags |= KConfigBase::Global;
-    config->writePathEntry( URL + number, item->url().prettyUrl(), flags );
-    config->writeEntry( Description + number, item->description(), flags );
-    config->writeEntry( Icon + number, item->icon(), flags );
-    config->writeEntry( IconGroup + number, item->iconGroup(), flags );
+    config->writePathEntry( QString(URL + number).toUtf8().constData(), item->url().prettyUrl(), flags );
+    config->writeEntry( QString(Description + number).toUtf8(), item->description(), flags );
+    config->writeEntry( QString(Icon + number).toUtf8(), item->icon(), flags );
+    config->writeEntry( QString(IconGroup + number).toUtf8().data(), item->iconGroup(), flags );
 }
 
 
@@ -723,11 +673,13 @@ void KUrlBar::slotDropped( QDropEvent *e )
     }
 }
 
-void KUrlBar::slotContextMenuRequested( Q3ListBoxItem *_item, const QPoint& pos )
+void KUrlBar::slotContextMenuRequested( const QPoint& pos )
 {
     if (m_isImmutable)
         return;
 
+    QListWidgetItem *_item = m_listBox->itemAt(pos);
+
     KUrlBarItem *item = dynamic_cast<KUrlBarItem*>( _item );
 
     KUrl lastURL = m_activeItem ? m_activeItem->url() : KUrl();
@@ -741,20 +693,19 @@ void KUrlBar::slotContextMenuRequested( Q3ListBoxItem *_item, const QPoint& pos
     QAction* EditItem = 0L;
     if (item != 0L && item->isPersistent())
     {
-        EditItem = popup->addAction(SmallIconSet("edit"), i18n("&Edit Entry..."));
+        EditItem = popup->addAction(KIcon("edit"), i18n("&Edit Entry..."));
         popup->addSeparator();
     }
 
-    QAction* AddItem = popup->addAction(SmallIconSet("filenew"), i18n("&Add Entry..."));
+    QAction* AddItem = popup->addAction(KIcon("filenew"), i18n("&Add Entry..."));
 
     QAction* RemoveItem = 0L;
     if (item != 0L && item->isPersistent())
-        RemoveItem = popup->addAction( SmallIconSet("editdelete"), i18n("&Remove Entry"));
+        RemoveItem = popup->addAction( KIcon("editdelete"), i18n("&Remove Entry"));
 
-    QAction* result = popup->exec( pos );
+    QAction* result = popup->exec( m_listBox->mapToGlobal(pos) );
     if (result == IconSize) {
         setIconSize( smallIcons ? K3Icon::SizeMedium : K3Icon::SizeSmallMedium );
-        m_listBox->triggerUpdate( true );
 
     } else if (result == AddItem) {
         addNewItem();
@@ -777,7 +728,7 @@ bool KUrlBar::addNewItem()
     KUrlBarItem *item = new KUrlBarItem( this, d->currentURL,
                                          i18n("Enter a description") );
     if ( editItem( item ) ) {
-        m_listBox->insertItem( item );
+        m_listBox->addItem( item );
         return true;
     }
 
@@ -800,11 +751,10 @@ bool KUrlBar::editItem( KUrlBarItem *item )
                                             icon, appLocal,
                                             m_iconSize, this ))
     {
-        item->setURL( url );
+        item->setUrl( url );
         item->setDescription( description );
         item->setIcon( icon );
         item->setApplicationLocal( appLocal );
-        m_listBox->triggerUpdate( true );
         m_isModified = true;
         updateGeometry();
         return true;
@@ -817,33 +767,36 @@ bool KUrlBar::editItem( KUrlBarItem *item )
 ///////////////////////////////////////////////////////////////////
 
 
-KUrlBarListBox::KUrlBarListBox( QWidget *parent, const char *name )
-    : KListBox( parent, name )
+KUrlBarListWidget::KUrlBarListWidget( QWidget *parent )
+    : KListWidget( parent )
 {
+    setItemDelegate( new KUrlBarItemDelegate(this) );
+    setContextMenuPolicy(Qt::CustomContextMenu);
     setAcceptDrops( true );
     viewport()->setAcceptDrops( true );
 }
 
-KUrlBarListBox::~KUrlBarListBox()
+KUrlBarListWidget::~KUrlBarListWidget()
 {
 }
 
-void KUrlBarListBox::paintEvent( QPaintEvent* )
+/*void KUrlBarListWidget::paintEvent( QPaintEvent* )
 {
     QPainter p(this);
     p.setPen( palette().color( QPalette::Mid ) );
     p.drawRect( 0, 0, width() - 1, height() - 1 );
-}
+}*/
 
-void KUrlBarListBox::populateMimeData( QMimeData* mimeData )
+void KUrlBarListWidget::populateMimeData( QMimeData* mimeData )
 {
     KUrl::List urls;
-    KUrlBarItem *item = static_cast<KUrlBarItem*>( firstItem() );
+    
+    for (int rowIndex = 0 ; rowIndex < count() ; rowIndex++)
+    {
+        KUrlBarItem *urlBarItem = static_cast<KUrlBarItem*>( item(rowIndex) );
 
-    while ( item ) {
-        if ( item->isSelected() )
-            urls.append( item->url() );
-        item = static_cast<KUrlBarItem*>( item->next() );
+        if ( urlBarItem->isSelected() )
+            urls.append( urlBarItem->url() );
     }
 
     if ( !urls.isEmpty() ) {
@@ -852,45 +805,43 @@ void KUrlBarListBox::populateMimeData( QMimeData* mimeData )
     }
 }
 
-void KUrlBarListBox::contentsDragEnterEvent( QDragEnterEvent *e )
+void KUrlBarListWidget::contentsDragEnterEvent( QDragEnterEvent *e )
 {
     e->setAccepted( KUrl::List::canDecode( e->mimeData() ) );
 }
 
-void KUrlBarListBox::contentsDropEvent( QDropEvent *e )
+void KUrlBarListWidget::contentsDropEvent( QDropEvent *e )
 {
     emit dropped( e );
 }
 
-void KUrlBarListBox::contextMenuEvent( QContextMenuEvent *e )
+/*void KUrlBarListWidget::contextMenuEvent( QContextMenuEvent *e )
 {
     if (e)
     {
-        emit contextMenuRequested( itemAt( e->globalPos() ), e->globalPos() );
+        emit contextMenuRequested( item );
         e->accept(); // Consume the event to avoid multiple contextMenuEvent calls...
     }
-}
+}*/
 
-void KUrlBarListBox::setOrientation( Qt::Orientation orient )
+void KUrlBarListWidget::setOrientation( Qt::Orientation orient )
 {
     if ( orient == Qt::Vertical ) {
-        setColumnMode( 1 );
-        setRowMode( Variable );
+        //TODO KDE4 - Port me: setColumnMode( 1 );
     }
     else {
-        setRowMode( 1 );
-        setColumnMode( Variable );
+        //TODO KDE4 - Port me: setRowMode( 1 );
     }
 
     m_orientation = orient;
 }
 
-bool KUrlBarListBox::event( QEvent* e )
+bool KUrlBarListWidget::event( QEvent* e )
 {
     if ( e->type() == QEvent::ToolTip )
     {
         QHelpEvent* he = static_cast<QHelpEvent*>( e );
-        Q3ListBoxItem *item = itemAt( he->pos() );
+        QListWidgetItem *item = itemAt( he->pos() );
         if ( item ) {
             QString text = static_cast<KUrlBarItem*>( item )->toolTip();
             if ( !text.isEmpty() )
@@ -900,7 +851,7 @@ bool KUrlBarListBox::event( QEvent* e )
         return true;
     }
 
-    return KListBox::event(e);
+    return KListWidget::event(e);
 }
 
 
@@ -981,7 +932,7 @@ KUrlBarItemDialog::KUrlBarItemDialog( bool allowGlobal, const KUrl& url,
     m_iconButton->setObjectName( QLatin1String( "icon button" ) );
     m_iconButton->setIconSize( iconSize );
     if ( icon.isEmpty() )
-        icon = KMimeType::iconNameForURL( url );
+        icon = KMimeType::iconNameForUrl( url );
     m_iconButton->setIcon( icon );
     label->setBuddy( m_iconButton );
     label->setWhatsThis(whatsThisText );
@@ -989,10 +940,10 @@ KUrlBarItemDialog::KUrlBarItemDialog( bool allowGlobal, const KUrl& url,
 
     if ( allowGlobal ) {
         QString appName;
-        if ( KGlobal::instance()->aboutData() )
-            appName = KGlobal::instance()->aboutData()->programName();
+        if ( KGlobal::mainComponent().aboutData() )
+            appName = KGlobal::mainComponent().aboutData()->programName();
         if ( appName.isEmpty() )
-            appName = QLatin1String( KGlobal::instance()->instanceName() );
+            appName = QLatin1String( KGlobal::mainComponent().componentName() );
         m_appLocal = new QCheckBox( i18n("&Only show when using this application (%1)",  appName ), box );
         m_appLocal->setChecked( appLocal );
         m_appLocal->setWhatsThis(i18n("<qt>Select this setting if you want this "
