@@ -20,11 +20,7 @@
 */
 
 #include "copyjob.h"
-
-#include <assert.h>
-
-#include <qtimer.h>
-#include <qfile.h>
+#include "deletejob.h"
 
 #include <klocale.h>
 #include <ksimpleconfig.h>
@@ -37,13 +33,20 @@
 #include "kprotocolmanager.h"
 
 #include "kio/observer.h"
+#include "kio/jobuidelegate.h"
 
 #include <kdirnotify.h>
-#include <ktempfile.h>
+#include <ktemporaryfile.h>
 
 #ifdef Q_OS_UNIX
 #include <utime.h>
 #endif
+#include <assert.h>
+
+#include <qtimer.h>
+#include <qfile.h>
+#include <sys/stat.h> // mode_t
+#include <QPointer>
 
 using namespace KIO;
 
@@ -52,6 +55,7 @@ using namespace KIO;
 
 #define KIO_ARGS QByteArray packedArgs; QDataStream stream( &packedArgs, QIODevice::WriteOnly ); stream
 
+/** @internal */
 class CopyJob::CopyJobPrivate
 {
 public:
@@ -89,11 +93,11 @@ CopyJob::CopyJob( const KUrl::List& src, const KUrl& dest, CopyMode mode, bool a
     d->m_globalDestinationState = destinationState;
 
     if ( showProgressInfo ) {
-        connect( this, SIGNAL( totalFiles( KIO::Job*, unsigned long ) ),
-                 Observer::self(), SLOT( slotTotalFiles( KIO::Job*, unsigned long ) ) );
+        connect( this, SIGNAL( totalFiles( KJob*, unsigned long ) ),
+                 Observer::self(), SLOT( slotTotalFiles( KJob*, unsigned long ) ) );
 
-        connect( this, SIGNAL( totalDirs( KIO::Job*, unsigned long ) ),
-                 Observer::self(), SLOT( slotTotalDirs( KIO::Job*, unsigned long ) ) );
+        connect( this, SIGNAL( totalDirs( KJob*, unsigned long ) ),
+                 Observer::self(), SLOT( slotTotalDirs( KJob*, unsigned long ) ) );
     }
     QTimer::singleShot(0, this, SLOT(slotStart()));
     /**
@@ -406,7 +410,7 @@ void CopyJob::slotEntries(KIO::Job* job, const UDSEntryList& list)
             {
                 QString destFileName;
                 if ( hasCustomURL &&
-                     KProtocolManager::fileNameUsedForCopying( url ) == KProtocolInfo::FromURL ) {
+                     KProtocolManager::fileNameUsedForCopying( url ) == KProtocolInfo::FromUrl ) {
                     //destFileName = url.fileName(); // Doesn't work for recursive listing
                     // Count the number of prefixes used by the recursive listjob
                     int numberOfSlashes = displayName.count( '/' ); // don't make this a find()!
@@ -516,7 +520,7 @@ void CopyJob::statCurrentSrc()
         }
         else if ( m_mode == Move && (
                 // Don't go renaming right away if we need a stat() to find out the destination filename
-                KProtocolManager::fileNameUsedForCopying( m_currentSrcURL ) == KProtocolInfo::FromURL ||
+                KProtocolManager::fileNameUsedForCopying( m_currentSrcURL ) == KProtocolInfo::FromUrl ||
                 destinationState != DEST_IS_DIR || m_asMethod )
             )
         {
@@ -672,7 +676,7 @@ void CopyJob::slotResultCreatingDirs( KJob * job )
                 // Did the user choose to overwrite already?
                 const QString destFile = (*it).uDest.path();
                 if ( shouldOverwrite( destFile ) ) { // overwrite => just skip
-                    emit copyingDone( this, ( *it ).uSource, ( *it ).uDest, true /* directory */, false /* renamed */ );
+                    emit copyingDone( this, (*it).uSource, (*it).uDest, (*it).mtime, true /* directory */, false /* renamed */ );
                     dirs.erase( it ); // Move on to next dir
                 } else {
                     if ( !isInteractive() ) {
@@ -705,7 +709,7 @@ void CopyJob::slotResultCreatingDirs( KJob * job )
     else // no error : remove from list, to move on to next dir
     {
         //this is required for the undo feature
-        emit copyingDone( this, (*it).uSource, (*it).uDest, true, false );
+        emit copyingDone( this, (*it).uSource, (*it).uDest, (*it).mtime, true, false );
         d->m_directoriesCopied.append( *it );
         dirs.erase( it );
     }
@@ -737,23 +741,23 @@ void CopyJob::slotResultConflictCreatingDirs( KJob * job )
     assert ( !hasSubjobs() ); // We should have only one job at a time ...
 
     // Always multi and skip (since there are files after that)
-    RenameDlg_Mode mode = (RenameDlg_Mode)( M_MULTI | M_SKIP );
+    RenameDialog_Mode mode = (RenameDialog_Mode)( M_MULTI | M_SKIP );
     // Overwrite only if the existing thing is a dir (no chance with a file)
     if ( m_conflictError == ERR_DIR_ALREADY_EXIST )
     {
         if( (*it).uSource == (*it).uDest ||
             ((*it).uSource.protocol() == (*it).uDest.protocol() &&
               (*it).uSource.path( KUrl::RemoveTrailingSlash ) == linkDest) )
-          mode = (RenameDlg_Mode)( mode | M_OVERWRITE_ITSELF);
+          mode = (RenameDialog_Mode)( mode | M_OVERWRITE_ITSELF);
         else
-          mode = (RenameDlg_Mode)( mode | M_OVERWRITE );
+          mode = (RenameDialog_Mode)( mode | M_OVERWRITE );
     }
 
     QString existingDest = (*it).uDest.path();
     QString newPath;
     if (m_reportTimer)
         m_reportTimer->stop();
-    RenameDlg_Result r = Observer::self()->open_RenameDlg( this, i18n("Folder Already Exists"),
+    RenameDialog_Result r = ui()->askFileRename( this, i18n("Folder Already Exists"),
                                          (*it).uSource.url(),
                                          (*it).uDest.url(),
                                          mode, newPath,
@@ -824,14 +828,14 @@ void CopyJob::slotResultConflictCreatingDirs( KJob * job )
             break;
         case R_OVERWRITE:
             m_overwriteList.append( existingDest );
-            emit copyingDone( this, ( *it ).uSource, ( *it ).uDest, true /* directory */, false /* renamed */ );
+            emit copyingDone( this, (*it).uSource, (*it).uDest, (*it).mtime, true /* directory */, false /* renamed */ );
             // Move on to next dir
             dirs.erase( it );
             m_processedDirs++;
             break;
         case R_OVERWRITE_ALL:
             m_bOverwriteAll = true;
-            emit copyingDone( this, ( *it ).uSource, ( *it ).uDest, true /* directory */, false /* renamed */ );
+            emit copyingDone( this, (*it).uSource, (*it).uDest, (*it).mtime, true /* directory */, false /* renamed */ );
             // Move on to next dir
             dirs.erase( it );
             m_processedDirs++;
@@ -962,7 +966,7 @@ void CopyJob::slotResultCopyingFiles( KJob * job )
         }
         else
             //required for the undo feature
-            emit copyingDone( this, (*it).uSource, (*it).uDest, false, false );
+            emit copyingDone( this, (*it).uSource, (*it).uDest, (*it).mtime, false, false );
         // remove from list, to move on to next file
         files.erase( it );
     }
@@ -985,7 +989,7 @@ void CopyJob::slotResultConflictCopyingFiles( KJob * job )
     // The file we were trying to create:
     QList<CopyInfo>::Iterator it = files.begin();
 
-    RenameDlg_Result res;
+    RenameDialog_Result res;
     QString newPath;
 
     if (m_reportTimer)
@@ -1005,11 +1009,11 @@ void CopyJob::slotResultConflictCopyingFiles( KJob * job )
 
         // Offer overwrite only if the existing thing is a file
         // If src==dest, use "overwrite-itself"
-        RenameDlg_Mode mode;
+        RenameDialog_Mode mode;
         bool isDir = true;
 
         if( m_conflictError == ERR_DIR_ALREADY_EXIST )
-            mode = (RenameDlg_Mode) 0;
+            mode = (RenameDialog_Mode) 0;
         else
         {
             if ( (*it).uSource == (*it).uDest  ||
@@ -1022,18 +1026,18 @@ void CopyJob::slotResultConflictCopyingFiles( KJob * job )
         }
 
         if ( m_bSingleFileCopy )
-            mode = (RenameDlg_Mode) ( mode | M_SINGLE );
+            mode = (RenameDialog_Mode) ( mode | M_SINGLE );
         else
-            mode = (RenameDlg_Mode) ( mode | M_MULTI | M_SKIP );
+            mode = (RenameDialog_Mode) ( mode | M_MULTI | M_SKIP );
 
-        res = Observer::self()->open_RenameDlg( this, !isDir ?
-                                i18n("File Already Exists") : i18n("Already Exists as Folder"),
-                                (*it).uSource.url(),
-                                (*it).uDest.url(),
-                                mode, newPath,
-                              (*it).size, destsize,
-                              (*it).ctime, destctime,
-                              (*it).mtime, destmtime );
+        res = ui()->askFileRename( this, !isDir ?
+                                   i18n("File Already Exists") : i18n("Already Exists as Folder"),
+                                   (*it).uSource.url(),
+                                   (*it).uDest.url(),
+                                   mode, newPath,
+                                   (*it).size, destsize,
+                                   (*it).ctime, destctime,
+                                   (*it).mtime, destmtime );
 
     }
     else
@@ -1046,10 +1050,10 @@ void CopyJob::slotResultConflictCopyingFiles( KJob * job )
         }
         else
         {
-            SkipDlg_Result skipResult = Observer::self()->open_SkipDlg( this, files.count() > 1,
-                                                                        job->errorString() );
+            SkipDialog_Result skipResult = ui()->askSkip( this, files.count() > 1,
+                                                          job->errorString() );
 
-            // Convert the return code from SkipDlg into a RenameDlg code
+            // Convert the return code from SkipDialog into a RenameDialog code
             res = ( skipResult == S_SKIP ) ? R_SKIP :
                          ( skipResult == S_AUTO_SKIP ) ? R_AUTO_SKIP :
                                         R_CANCEL;
@@ -1170,20 +1174,20 @@ void CopyJob::copyNextFile()
                         config.setDesktopGroup();
                         KUrl url = (*it).uSource;
                         url.setPass( "" );
-                        config.writePathEntry( QString::fromLatin1("URL"), url.url() );
-                        config.writeEntry( QString::fromLatin1("Name"), url.url() );
-                        config.writeEntry( QString::fromLatin1("Type"), QString::fromLatin1("Link") );
+                        config.writePathEntry( "URL", url.url() );
+                        config.writeEntry( "Name", url.url() );
+                        config.writeEntry( "Type", QString::fromLatin1("Link") );
                         QString protocol = (*it).uSource.protocol();
-                        if ( protocol == QString::fromLatin1("ftp") )
-                            config.writeEntry( QString::fromLatin1("Icon"), QString::fromLatin1("ftp") );
-                        else if ( protocol == QString::fromLatin1("http") )
-                            config.writeEntry( QString::fromLatin1("Icon"), QString::fromLatin1("www") );
-                        else if ( protocol == QString::fromLatin1("info") )
-                            config.writeEntry( QString::fromLatin1("Icon"), QString::fromLatin1("info") );
-                        else if ( protocol == QString::fromLatin1("mailto") )   // sven:
-                            config.writeEntry( QString::fromLatin1("Icon"), QString::fromLatin1("kmail") ); // added mailto: support
+                        if ( protocol == QLatin1String("ftp") )
+                            config.writeEntry( "Icon", QString::fromLatin1("ftp") );
+                        else if ( protocol == QLatin1String("http") )
+                            config.writeEntry( "Icon", QString::fromLatin1("www") );
+                        else if ( protocol == QLatin1String("info") )
+                            config.writeEntry( "Icon", QString::fromLatin1("info") );
+                        else if ( protocol == QLatin1String("mailto") )   // sven:
+                            config.writeEntry( "Icon", QString::fromLatin1("kmail") ); // added mailto: support
                         else
-                            config.writeEntry( QString::fromLatin1("Icon"), QString::fromLatin1("unknown") );
+                            config.writeEntry( "Icon", QString::fromLatin1("unknown") );
                         config.sync();
                         files.erase( it );
                         m_processedFiles++;
@@ -1250,7 +1254,10 @@ void CopyJob::copyNextFile()
             KIO::FileCopyJob * copyJob = KIO::file_copy( (*it).uSource, (*it).uDest, permissions, bOverwrite, false, false/*no GUI*/ );
             copyJob->setParentJob( this ); // in case of rename dialog
             copyJob->setSourceSize( (*it).size );
-            copyJob->setModificationTime( (*it).mtime );
+            if ((*it).mtime != -1) {
+                QDateTime dt; dt.setTime_t( (*it).mtime );
+                copyJob->setModificationTime( dt );
+            }
             newjob = copyJob;
             //kDebug(7007) << "CopyJob::copyNextFile : Copying " << (*it).uSource << " to " << (*it).uDest << endl;
             m_currentSrcURL=(*it).uSource;
@@ -1403,6 +1410,7 @@ void CopyJob::slotResultSettingDirAttributes( KJob * job )
 }
 #endif
 
+// We were trying to do a direct renaming, before even stat'ing
 void CopyJob::slotResultRenaming( KJob* job )
 {
     int err = job->error();
@@ -1427,10 +1435,12 @@ void CopyJob::slotResultRenaming( KJob* job )
             kDebug(7007) << "Couldn't rename directly, dest already exists. Detected special case of lower/uppercase renaming in same dir, try with 2 rename calls" << endl;
             QByteArray _src( QFile::encodeName(m_currentSrcURL.path()) );
             QByteArray _dest( QFile::encodeName(dest.path()) );
-            KTempFile tmpFile( m_currentSrcURL.directory(KUrl::ObeyTrailingSlash) );
-            QByteArray _tmp( QFile::encodeName(tmpFile.name()) );
-            kDebug(7007) << "CopyJob::slotResult KTempFile status:" << tmpFile.status() << " using " << _tmp << " as intermediary" << endl;
-            tmpFile.unlink();
+            KTemporaryFile tmpFile;
+            tmpFile.setPrefix(m_currentSrcURL.directory(KUrl::ObeyTrailingSlash));
+            tmpFile.setAutoRemove(false);
+            tmpFile.open();
+            QByteArray _tmp( QFile::encodeName(tmpFile.fileName()) );
+            kDebug(7007) << "CopyJob::slotResult KTemporaryFile using " << _tmp << " as intermediary" << endl;
             if ( ::rename( _src, _tmp ) == 0 )
             {
                 if ( !QFile::exists( _dest ) && ::rename( _tmp, _dest ) == 0 )
@@ -1442,7 +1452,7 @@ void CopyJob::slotResultRenaming( KJob* job )
                 {
                     // Revert back to original name!
                     if ( ::rename( _tmp, _src ) != 0 ) {
-                        kError(7007) << "Couldn't rename " << tmpFile.name() << " back to " << _src << " !" << endl;
+                        kError(7007) << "Couldn't rename " << tmpFile.fileName() << " back to " << _src << " !" << endl;
                         // Severe error, abort
                         Job::slotResult( job ); // will set the error and emit result(this)
                         return;
@@ -1481,13 +1491,13 @@ void CopyJob::slotResultRenaming( KJob* job )
             } else {
                 QString newPath;
                 // If src==dest, use "overwrite-itself"
-                RenameDlg_Mode mode = (RenameDlg_Mode)
+                RenameDialog_Mode mode = (RenameDialog_Mode)
                                       ( ( m_currentSrcURL == dest ) ? M_OVERWRITE_ITSELF : M_OVERWRITE );
 
                 if ( m_srcList.count() > 1 )
-                    mode = (RenameDlg_Mode) ( mode | M_MULTI | M_SKIP );
+                    mode = (RenameDialog_Mode) ( mode | M_MULTI | M_SKIP );
                 else
-                    mode = (RenameDlg_Mode) ( mode | M_SINGLE );
+                    mode = (RenameDialog_Mode) ( mode | M_SINGLE );
 
                 // we lack mtime info for both the src (not stated)
                 // and the dest (stated but this info wasn't stored)
@@ -1513,7 +1523,7 @@ void CopyJob::slotResultRenaming( KJob* job )
                     mtimeDest = stat_buf.st_mtime;
                 }
 
-                RenameDlg_Result r = Observer::self()->open_RenameDlg(
+                RenameDialog_Result r = ui()->askFileRename(
                     this,
                     err != ERR_DIR_ALREADY_EXIST ? i18n("File Already Exists") : i18n("Already Exists as Folder"),
                     m_currentSrcURL.url(),
@@ -1585,7 +1595,7 @@ void CopyJob::slotResultRenaming( KJob* job )
     else
     {
         //kDebug(7007) << "Renaming succeeded, move on" << endl;
-        emit copyingDone( this, *m_currentStatSrc, dest, true, true );
+        emit copyingDone( this, *m_currentStatSrc, dest, -1 /*mtime unknown, and not needed*/, true, true );
         statNextSrc();
     }
 }
