@@ -88,7 +88,7 @@ KIO_EXPORT QString KIO::convertSeconds( unsigned int seconds )
   const QTime time(hours, mins, seconds);
   const QString timeStr( KGlobal::locale()->formatTime(time, true /*with seconds*/, true /*duration*/) );
   if ( days > 0 )
-    return i18np("1 day %1", "%n days %1", days, timeStr);
+    return i18np("1 day %2", "%1 days %2", days, timeStr);
   else
     return timeStr;
 }
@@ -118,17 +118,14 @@ KIO_EXPORT QTime KIO::calculateRemaining( KIO::filesize_t totalSize, KIO::filesi
 
 KIO_EXPORT QString KIO::itemsSummaryString(uint items, uint files, uint dirs, KIO::filesize_t size, bool showSize)
 {
-    QString text = items == 0 ? i18n( "No Items" ) : i18np( "One Item", "%n Items", items );
-    text += " - ";
-    text += files == 0 ? i18n( "No Files" ) : i18np( "One File", "%n Files", files );
-    if ( showSize && files > 0 )
-    {
-        text += ' ';
-        text += i18n("(%1 Total)", KIO::convertSize( size ) );
+    const QString itemsText = items == 0 ? i18n( "No Items" ) : i18np( "One Item", "%1 Items", items );
+    const QString filesText = files == 0 ? i18n( "No Files" ) : i18np( "One File", "%1 Files", files );
+    const QString foldersText = dirs == 0 ? i18n( "No Folders" ) : i18np("One Folder", "%1 Folders", dirs);
+    if ( showSize && files > 0 ) {
+        const QString sizeText = i18n("(%1 Total)", KIO::convertSize( size ) );
+        return i18nc("Items (Folders, Files), Size", "%1 (%2, %3), %4", itemsText, foldersText, filesText, sizeText);
     }
-    text += " - ";
-    text += dirs == 0 ? i18n( "No Folders" ) : i18np("One Folder", "%n Folders", dirs);
-    return text;
+    return i18nc("Items (Folders, Files)", "%1 (%2, %3)", itemsText, foldersText, filesText);
 }
 
 KIO_EXPORT QString KIO::encodeFileName( const QString & _str )
@@ -419,6 +416,8 @@ KIO_EXPORT QString KIO::unsupportedActionErrorString(const QString &protocol, in
       return i18n("Using sub-URLs with %1 is not supported.", protocol);
     case CMD_MULTI_GET:
       return i18n("Multiple get is not supported with protocol %1.", protocol);
+    case CMD_OPEN:
+      return i18n("Opening files is not supported with protocol %1.", protocol);
     default:
       return i18n("Protocol %1 does not support action %2.", protocol, cmd);
   }/*end switch*/
@@ -1974,9 +1973,9 @@ QString KIO::getCacheControlString(KIO::CacheControl cacheControl)
     return QString();
 }
 
-QPixmap KIO::pixmapForURL( const KUrl & _url, mode_t _mode, K3Icon::Group _group,
+QPixmap KIO::pixmapForUrl( const KUrl & _url, mode_t _mode, K3Icon::Group _group,
                            int _force_size, int _state, QString * _path )
 {
-    const QString iconName = KMimeType::iconNameForURL( _url, _mode );
-    return KGlobal::iconLoader()->loadMimeTypeIcon( iconName, _group, _force_size, _state, _path );
+    const QString iconName = KMimeType::iconNameForUrl( _url, _mode );
+    return KIconLoader::global()->loadMimeTypeIcon( iconName, _group, _force_size, _state, _path );
 }
