@@ -26,8 +26,8 @@
 #include <qmap.h>
 #include <QList>
 
-#include <kicontheme.h> // K3Icon (for pixmapForURL)
-#include <QtGui/QPixmap> // for pixmapForURL
+#include <kicontheme.h> // K3Icon (for pixmapForUrl)
+#include <QtGui/QPixmap> // for pixmapForUrl
 
 #include <sys/stat.h> // S_ISDIR
 
@@ -157,7 +157,7 @@ namespace KIO
     CMD_DEL = 'K', // 75
     CMD_CHMOD = 'L', // 76
     CMD_SPECIAL = 'M', // 77
-    CMD_USERPASS = 'N', // 78
+    // 'N', // 78  - currently unused
     CMD_REPARSECONFIGURATION = 'O', // 79
     CMD_META_DATA = 'P', // 80
     CMD_SYMLINK = 'Q', // 81
@@ -166,11 +166,22 @@ namespace KIO
     CMD_RESUMEANSWER = 'T', // 84
     CMD_CONFIG = 'U', // 85
     CMD_MULTI_GET = 'V', // 86
-    CMD_SETLINKDEST = 'W' // 87
+    CMD_SETLINKDEST = 'W', // 87
+    CMD_OPEN = 'X' // 88
     // Add new ones here once a release is done, to avoid breaking binary compatibility.
     // Note that protocol-specific commands shouldn't be added here, but should use special.
   };
 
+  /**
+   * Commands that can be invoked on a slave-file.
+   */
+  enum FileCommand {
+    CMD_READ  = 90,
+    CMD_WRITE = 91,
+    CMD_SEEK  = 92,
+    CMD_CLOSE = 93
+  };
+
   /**
    * Error codes that can be emitted by KIO.
    */
@@ -247,8 +258,9 @@ namespace KIO
     ERR_UPGRADE_REQUIRED = KJob::UserDefinedError + 64, // A transport upgrade is required to access this
                                                         // object.  For instance, TLS is demanded by
                                                         // the server in order to continue.
-    ERR_POST_DENIED = KJob::UserDefinedError + 65 // Issued when trying to POST data to a certain Ports
+    ERR_POST_DENIED = KJob::UserDefinedError + 65, // Issued when trying to POST data to a certain Ports
                                                   // see job.cpp
+    ERR_COULD_NOT_SEEK = KJob::UserDefinedError + 66
   };
 
   /**
@@ -408,7 +420,7 @@ namespace KIO
    *              Ignored if 0
    * @return the pixmap of the URL, can be a default icon if not found
    */
-  KIO_EXPORT QPixmap pixmapForURL( const KUrl & _url, mode_t _mode = 0, K3Icon::Group _group = K3Icon::Desktop,
+  KIO_EXPORT QPixmap pixmapForUrl( const KUrl & _url, mode_t _mode = 0, K3Icon::Group _group = K3Icon::Desktop,
                                    int _force_size = 0, int _state = 0, QString * _path = 0 );
 
   /**
@@ -589,7 +601,7 @@ public:
  *   for (; it != end; ++it) {
  *     const KIO::UDSEntry& entry = *it;
  *     QString name = entry.stringValue( KIO::UDS_NAME );
- *     bool isDir = S_ISDIR( entry.numberValue( KIO::UDS_FILE_TYPE ) );
+ *     bool isDir = entry.isDir();
  *     KIO::filesize_t size = entry.numberValue( KIO::UDS_SIZE, -1 );
  *     ...
  *   }
