@@ -61,13 +61,14 @@ extern "C" {
 #include "kmimemagic.h"
 #include "kprotocolinfo.h"
 #include "kprotocolmanager.h"
+#include "filejob.h"
 
 #include "kio/observer.h"
 
 #include "kssl/ksslcsessioncache.h"
 
 #include <kdirnotify.h>
-#include <ktempfile.h>
+#include <ktemporaryfile.h>
 
 #ifdef Q_OS_UNIX
 #include <utime.h>
@@ -80,13 +81,13 @@ using namespace KIO;
 
 #define KIO_ARGS QByteArray packedArgs; QDataStream stream( &packedArgs, QIODevice::WriteOnly ); stream
 
+/** @internal */
 class Job::JobPrivate
 {
 public:
-    JobPrivate() : m_interactive( true ), m_suspended( false ), m_parentJob( 0L ), m_extraFlags(0)
+    JobPrivate() : m_suspended( false ), m_parentJob( 0L ), m_extraFlags(0)
                    {}
 
-    bool m_interactive;
     bool m_suspended;
     // Maybe we could use the QObject parent/child mechanism instead
     // (requires a new ctor, and moving the ctor code to some init()).
@@ -96,6 +97,7 @@ public:
 
 Job::Job(bool showProgressInfo) : KCompositeJob(0), m_speedTimer(0), d( new JobPrivate )
 {
+    setCapabilities( KJob::Killable | KJob::Pausable );
     setUiDelegate( new JobUiDelegate( showProgressInfo ) );
 }
 
@@ -122,13 +124,16 @@ void Job::addSubjob(Job *job, bool inheritMetaData)
     KCompositeJob::addSubjob( job );
 
     // Forward information from that subjob.
-    connect( job, SIGNAL(speed( KIO::Job*, unsigned long )),
-             SLOT(slotSpeed(KIO::Job*, unsigned long)) );
+    connect( job, SIGNAL(speed( KJob*, unsigned long )),
+             SLOT(slotSpeed(KJob*, unsigned long)) );
 
     if (inheritMetaData)
        job->mergeMetaData(m_outgoingMetaData);
 
-    job->ui()->setWindow( m_window );
+    if (ui()) {
+        job->ui()->setWindow( ui()->window() );
+        job->ui()->updateUserTimestamp( ui()->userTimestamp() );
+    }
 }
 
 void Job::removeSubjob( KJob *jobBase, bool mergeMetaData )
@@ -202,7 +207,7 @@ bool Job::isSuspended() const
   return d->m_suspended;
 }
 
-void Job::slotSpeed( KIO::Job*, unsigned long speed )
+void Job::slotSpeed( KJob*, unsigned long speed )
 {
   //kDebug(7007) << "Job::slotSpeed " << speed << endl;
   emitSpeed( speed );
@@ -366,7 +371,7 @@ SimpleJob::~SimpleJob()
 {
     if (m_slave) // was running
     {
-        kDebug(7007) << "SimpleJob::~SimpleJob: Killing running job in destructor!"  << endl;
+        kDebug(7007) << "SimpleJob::~SimpleJob: Killing running job in destructor!"  << kBacktrace() << endl;
 #if 0
         m_slave->kill();
         Scheduler::jobFinished( this, m_slave ); // deletes the slave
@@ -419,7 +424,13 @@ void SimpleJob::start(Slave *slave)
        addMetaData("window-id", id.setNum((ulong)ui()->window()->winId()));
     }
 
-    QString sslSession = KSSLCSessionCache::getSessionForURL(m_url);
+    if (ui() && ui()->userTimestamp())
+    {
+       QString id;
+       addMetaData("user-timestamp", id.setNum((ulong)ui()->userTimestamp()));
+    }
+
+    QString sslSession = KSSLCSessionCache::getSessionForUrl(m_url);
     if ( !sslSession.isNull() )
     {
         addMetaData("ssl_session_id", sslSession);
@@ -509,7 +520,7 @@ void SimpleJob::slotConnected()
 void SimpleJob::slotNeedProgressId()
 {
     if ( !progressId() )
-        setProgressId( Observer::self()->newJob( this, false ) );
+        setProgressId( Observer::self()->newJob( this, Observer::JobHidden ) );
     m_slave->setProgressId( progressId() );
 }
 
@@ -549,7 +560,7 @@ void SimpleJob::storeSSLSessionFromJob(const KUrl &m_redirectionURL) {
 
     if ( !sslSession.isNull() ) {
 	    const KUrl &queryURL = m_redirectionURL.isEmpty()?m_url:m_redirectionURL;
-	    KSSLCSessionCache::putSessionForURL(queryURL, sslSession);
+	    KSSLCSessionCache::putSessionForUrl(queryURL, sslSession);
     }
 }
 
@@ -572,7 +583,7 @@ void MkdirJob::start(Slave *slave)
 void MkdirJob::slotRedirection( const KUrl &url)
 {
      kDebug(7007) << "MkdirJob::slotRedirection(" << url << ")" << endl;
-     if (!KAuthorized::authorizeURLAction("redirect", m_url, url))
+     if (!KAuthorized::authorizeUrlAction("redirect", m_url, url))
      {
        kWarning(7007) << "MkdirJob: Redirection from " << m_url << " to " << url << " REJECTED!" << endl;
        setError( ERR_ACCESS_DENIED );
@@ -707,7 +718,7 @@ void StatJob::slotStatEntry( const KIO::UDSEntry & entry )
 void StatJob::slotRedirection( const KUrl &url)
 {
      kDebug(7007) << "StatJob::slotRedirection(" << url << ")" << endl;
-     if (!KAuthorized::authorizeURLAction("redirect", m_url, url))
+     if (!KAuthorized::authorizeUrlAction("redirect", m_url, url))
      {
        kWarning(7007) << "StatJob: Redirection from " << m_url << " to " << url << " REJECTED!" << endl;
        setError( ERR_ACCESS_DENIED );
@@ -802,7 +813,7 @@ void TransferJob::slotData( const QByteArray &_data)
 void TransferJob::slotRedirection( const KUrl &url)
 {
      kDebug(7007) << "TransferJob::slotRedirection(" << url << ")" << endl;
-     if (!KAuthorized::authorizeURLAction("redirect", m_url, url))
+     if (!KAuthorized::authorizeUrlAction("redirect", m_url, url))
      {
        kWarning(7007) << "TransferJob: Redirection from " << m_url << " to " << url << " REJECTED!" << endl;
        return;
@@ -1016,8 +1027,8 @@ void TransferJob::start(Slave *slave)
     connect( slave, SIGNAL(errorPage() ),
              SLOT( slotErrorPage() ) );
 
-    connect( slave, SIGNAL( needSubURLData() ),
-             SLOT( slotNeedSubURLData() ) );
+    connect( slave, SIGNAL( needSubUrlData() ),
+             SLOT( slotNeedSubUrlData() ) );
 
     connect( slave, SIGNAL(canResume( KIO::filesize_t ) ),
              SLOT( slotCanResume( KIO::filesize_t ) ) );
@@ -1034,17 +1045,17 @@ void TransferJob::start(Slave *slave)
        slave->suspend();
 }
 
-void TransferJob::slotNeedSubURLData()
+void TransferJob::slotNeedSubUrlData()
 {
     // Job needs data from subURL.
     m_subJob = KIO::get( m_subUrl, false, false);
     internalSuspend(); // Put job on hold until we have some data.
     connect(m_subJob, SIGNAL( data(KIO::Job*,const QByteArray &)),
-            SLOT( slotSubURLData(KIO::Job*,const QByteArray &)));
+            SLOT( slotSubUrlData(KIO::Job*,const QByteArray &)));
     addSubjob(m_subJob);
 }
 
-void TransferJob::slotSubURLData(KIO::Job*, const QByteArray &data)
+void TransferJob::slotSubUrlData(KIO::Job*, const QByteArray &data)
 {
     // The Alternating Bitburg protocol in action again.
     staticData = data;
@@ -1081,6 +1092,11 @@ void TransferJob::slotResult( KJob *job)
    }
 }
 
+void TransferJob::setModificationTime( const QDateTime& mtime )
+{
+    addMetaData( "modified", mtime.toString( Qt::ISODate ) );
+}
+
 TransferJob *KIO::get( const KUrl& url, bool reload, bool showProgressInfo )
 {
     // Send decoded path and encoded query
@@ -1091,6 +1107,14 @@ TransferJob *KIO::get( const KUrl& url, bool reload, bool showProgressInfo )
     return job;
 }
 
+FileJob *KIO::open( const KUrl& url, int access )
+{
+    // Send decoded path and encoded query
+    KIO_ARGS << url << access;
+    FileJob * job = new FileJob( url, packedArgs );
+    return job;
+}
+
 class PostErrorJob : public TransferJob
 {
 public:
@@ -1208,7 +1232,7 @@ TransferJob *KIO::http_post( const KUrl& url, const QByteArray &postData, bool s
       _url.setPath("/");
     }
 
-    if (!_error && !KAuthorized::authorizeURLAction("open", KUrl(), _url))
+    if (!_error && !KAuthorized::authorizeUrlAction("open", KUrl(), _url))
         _error = KIO::ERR_ACCESS_DENIED;
 
     // if request is not valid, return an invalid transfer job
@@ -1401,12 +1425,12 @@ void DirectCopyJob::slotCanResume( KIO::filesize_t offset )
 
 //////////////////////////
 
-
+/** @internal */
 class FileCopyJob::FileCopyJobPrivate
 {
 public:
     KIO::filesize_t m_sourceSize;
-    time_t m_modificationTime;
+    QDateTime m_modificationTime;
     SimpleJob *m_delJob;
 };
 
@@ -1435,7 +1459,6 @@ FileCopyJob::FileCopyJob( const KUrl& src, const KUrl& dest, int permissions,
     m_putJob = 0;
     d->m_delJob = 0;
     d->m_sourceSize = (KIO::filesize_t) -1;
-    d->m_modificationTime = static_cast<time_t>( -1 );
     QTimer::singleShot(0, this, SLOT(slotStart()));
 }
 
@@ -1507,7 +1530,7 @@ void FileCopyJob::setSourceSize( KIO::filesize_t size )
        m_totalSize = size;
 }
 
-void FileCopyJob::setModificationTime( time_t mtime )
+void FileCopyJob::setModificationTime( const QDateTime& mtime )
 {
     d->m_modificationTime = mtime;
 }
@@ -1586,9 +1609,8 @@ void FileCopyJob::startDataPump()
     m_getJob = 0L; // for now
     m_putJob = put( m_dest, m_permissions, m_overwrite, m_resume, false /* no GUI */);
     //kDebug(7007) << "FileCopyJob: m_putJob = " << m_putJob << " m_dest=" << m_dest << endl;
-    if ( d->m_modificationTime != static_cast<time_t>( -1 ) ) {
-        QDateTime dt; dt.setTime_t( d->m_modificationTime );
-        m_putJob->addMetaData( "modified", dt.toString( Qt::ISODate ) );
+    if ( d->m_modificationTime.isValid() ) {
+        m_putJob->setModificationTime( d->m_modificationTime );
     }
 
     // The first thing the put job will tell us is whether we can
@@ -1607,18 +1629,18 @@ void FileCopyJob::slotCanResume( KIO::Job* job, KIO::filesize_t offset )
         //kDebug(7007) << "FileCopyJob::slotCanResume from PUT job. offset=" << KIO::number(offset) << endl;
         if (offset)
         {
-            RenameDlg_Result res = R_RESUME;
+            RenameDialog_Result res = R_RESUME;
 
             if (!KProtocolManager::autoResume() && !m_overwrite)
             {
                 QString newPath;
                 KIO::Job* job = ( !progressId() && parentJob() ) ? parentJob() : this;
                 // Ask confirmation about resuming previous transfer
-                res = Observer::self()->open_RenameDlg(
+                res = ui()->askFileRename(
                       job, i18n("File Already Exists"),
                       m_src.url(),
                       m_dest.url(),
-                      (RenameDlg_Mode) (M_OVERWRITE | M_RESUME | M_NORENAME), newPath,
+                      (RenameDialog_Mode) (M_OVERWRITE | M_RESUME | M_NORENAME), newPath,
                       d->m_sourceSize, offset );
             }
 
@@ -1665,8 +1687,10 @@ void FileCopyJob::slotCanResume( KIO::Job* job, KIO::filesize_t offset )
             connectSubjob( m_getJob ); // Progress info depends on get
             m_getJob->internalResume(); // Order a beer
 
-            connect( m_getJob, SIGNAL(data(KIO::Job *, const QByteArray&)),
-                     SLOT( slotData(KIO::Job *, const QByteArray&)));
+            connect( m_getJob, SIGNAL(data(KIO::Job*,const QByteArray&)),
+                     SLOT( slotData(KIO::Job*,const QByteArray&)) );
+            connect( m_getJob, SIGNAL(mimetype(KIO::Job*,const QString&) ),
+                     SLOT(slotMimetype(KIO::Job*,const QString&)) );
         }
         else // copyjob
         {
@@ -1727,6 +1751,11 @@ void FileCopyJob::slotDataReq( KIO::Job * , QByteArray &data)
    m_buffer = QByteArray();
 }
 
+void FileCopyJob::slotMimetype( KIO::Job*, const QString& type )
+{
+    emit mimetype( this, type );
+}
+
 void FileCopyJob::slotResult( KJob *job)
 {
    //kDebug(7007) << "FileCopyJob this=" << this << " ::slotResult(" << job << ")" << endl;
@@ -1934,7 +1963,7 @@ void ListJob::slotResult( KJob * job )
 
 void ListJob::slotRedirection( const KUrl & url )
 {
-     if (!KAuthorized::authorizeURLAction("redirect", m_url, url))
+     if (!KAuthorized::authorizeUrlAction("redirect", m_url, url))
      {
        kWarning(7007) << "ListJob: Redirection from " << m_url << " to " << url << " REJECTED!" << endl;
        return;
@@ -1949,10 +1978,10 @@ void ListJob::slotFinished()
 {
     // Support for listing archives as directories
     if ( error() == KIO::ERR_IS_FILE && m_url.isLocalFile() ) {
-        KMimeType::Ptr ptr = KMimeType::findByURL( m_url, 0, true, true );
+        KMimeType::Ptr ptr = KMimeType::findByUrl( m_url, 0, true, true );
         if ( ptr ) {
             QString proto = ptr->property("X-KDE-LocalProtocol").toString();
-            if ( !proto.isEmpty() ) {
+            if ( !proto.isEmpty() && KProtocolInfo::isKnownProtocol( proto) ) {
                 m_redirectionURL = m_url;
                 m_redirectionURL.setProtocol( proto );
                 setError( 0 );
@@ -2007,7 +2036,7 @@ void ListJob::setUnrestricted(bool unrestricted)
 
 void ListJob::start(Slave *slave)
 {
-    if (!KAuthorized::authorizeURLAction("list", m_url, m_url) && !(extraFlags() & EF_ListJobUnrestricted))
+    if (!KAuthorized::authorizeUrlAction("list", m_url, m_url) && !(extraFlags() & EF_ListJobUnrestricted))
     {
         setError( ERR_ACCESS_DENIED );
         setErrorText( m_url.url() );
@@ -2135,7 +2164,7 @@ bool MultiGetJob::findCurrentEntry()
 void MultiGetJob::slotRedirection( const KUrl &url)
 {
   if (!findCurrentEntry()) return; // Error
-  if (!KAuthorized::authorizeURLAction("redirect", m_url, url))
+  if (!KAuthorized::authorizeUrlAction("redirect", m_url, url))
   {
      kWarning(7007) << "MultiGetJob: Redirection from " << m_currentEntry.url << " to " << url << " REJECTED!" << endl;
      return;
