@@ -25,10 +25,6 @@
 #include <qobject.h>
 #include <qlinkedlist.h>
 #include <qstringlist.h>
-#include <qpointer.h>
-
-#include <sys/types.h>
-#include <sys/stat.h>
 
 #include <kurl.h>
 #include <kio/global.h>
@@ -86,15 +82,11 @@ namespace KIO {
         /**
          * Abort this job.
          * This kills all subjobs and deletes the job.
-	 *
-         * @param quietly if false, Job will emit signal result
-         * and ask kio_uiserver to close the progress window.
-         * @p quietly is set to true for subjobs. Whether applications
-         * should call with true or false depends on whether they rely
-         * on result being emitted or not.
+         *
          */
         virtual bool doKill();
 
+public Q_SLOTS:
         /**
          * Suspend this job
          * @see resume
@@ -107,6 +99,7 @@ namespace KIO {
          */
         virtual void resume();
 
+public:
         /**
          * @return true if this job was suspended by suspend()
          */
@@ -166,7 +159,7 @@ namespace KIO {
 
         /**
          * Set the parent Job.
-         * One example use of this is when FileCopyJob calls open_RenameDlg,
+         * One example use of this is when FileCopyJob calls open_RenameDialog,
          * it must pass the correct progress ID of the parent CopyJob
          * (to hide the progress dialog).
 	 * You can set the parent job only once. By default a job does not
@@ -262,7 +255,7 @@ namespace KIO {
 	 * @param job the job that emitted this signal
 	 * @param speed the speed in bytes/s
          */
-        void speed( KIO::Job *job, unsigned long speed );
+        void speed( KJob *job, unsigned long speed );
 
     protected Q_SLOTS:
         /**
@@ -271,7 +264,7 @@ namespace KIO {
 	 * @param speed the speed in bytes/s
 	 * @see speed()
          */
-        void slotSpeed( KIO::Job *job, unsigned long speed );
+        void slotSpeed( KJob *job, unsigned long speed );
 
         /**
          * Remove speed information.
@@ -325,7 +318,6 @@ namespace KIO {
 
     private:
         QTimer *m_speedTimer;
-        QPointer<QWidget> m_window;
 
         class JobPrivate;
         JobPrivate *d;
@@ -707,6 +699,12 @@ namespace KIO {
                     const QByteArray &_staticData,
                     bool showProgressInfo);
 
+        /**
+         * Sets the modification time of the file to be created (by KIO::put)
+         * Note that some kioslaves might ignore this.
+         */
+        void setModificationTime( const QDateTime& mtime );
+
         /**
 	 * @internal
          * Called by the scheduler when a @p slave gets to
@@ -775,6 +773,13 @@ namespace KIO {
          */
         bool reportDataSent();
 
+        /**
+         * Call this in the slot connected to result,
+         * and only after making sure no error happened.
+	 * @return the mimetype of the URL
+         */
+         QString mimetype() const { return m_mimetype; }
+
     Q_SIGNALS:
         /**
          * Data from the slave has arrived.
@@ -841,8 +846,8 @@ namespace KIO {
         virtual void slotData( const QByteArray &data);
         virtual void slotDataReq();
         virtual void slotMimetype( const QString &mimetype );
-        virtual void slotNeedSubURLData();
-        virtual void slotSubURLData(KIO::Job*, const QByteArray &);
+        virtual void slotNeedSubUrlData();
+        virtual void slotSubUrlData(KIO::Job*, const QByteArray &);
         virtual void slotMetaData( const KIO::MetaData &_metaData);
         void slotErrorPage();
         void slotCanResume( KIO::filesize_t offset );
@@ -1029,13 +1034,6 @@ namespace KIO {
 	*/
         MimetypeJob(const KUrl& url, int command, const QByteArray &packedArgs, bool showProgressInfo);
 
-        /**
-         * Call this in the slot connected to result,
-         * and only after making sure no error happened.
-	 * @return the mimetype of the URL
-         */
-         QString mimetype() const { return m_mimetype; }
-
         /**
 	 * @internal
          * Called by the scheduler when a slave gets to
@@ -1088,24 +1086,38 @@ namespace KIO {
          * in which case the mtime of the source is applied to the destination (if the protocol
          * supports the concept).
          */
-        void setModificationTime( time_t mtime );
+        void setModificationTime( const QDateTime& mtime );
 
 	/**
 	 * Returns the source URL.
 	 * @return the source URL
 	 */
-        KUrl srcURL() const { return m_src; }
+        KUrl srcUrl() const { return m_src; }
 
 	/**
 	 * Returns the destination URL.
 	 * @return the destination URL
 	 */
-        KUrl destURL() const { return m_dest; }
+        KUrl destUrl() const { return m_dest; }
+
+    Q_SIGNALS:
+        /**
+         * Mimetype determined during a file copy.
+         * This is never emitted during a move, and might not be emitted during
+         * a file copy, depending on the slave. But when a get and a put are
+         * being used (which is the common case), this signal forwards the
+         * mimetype information from the get job.
+         *
+         * @param job the job that emitted this signal
+         * @param type the mime type
+         */
+        void mimetype( KIO::Job *job, const QString &type );
 
     public Q_SLOTS:
         void slotStart();
         void slotData( KIO::Job *, const QByteArray &data);
         void slotDataReq( KIO::Job *, QByteArray &data);
+        void slotMimetype( KIO::Job*, const QString& type );
 
     protected Q_SLOTS:
         /**
@@ -1207,7 +1219,7 @@ namespace KIO {
          * was no redirection.
          * @return the redirection url
          */
-        const KUrl& redirectionURL() const { return m_redirectionURL; }
+        const KUrl& redirectionUrl() const { return m_redirectionURL; }
 
         /**
          * Do not apply any KIOSK restrictions to this job.
@@ -1263,9 +1275,4 @@ namespace KIO {
     };
 }
 
-// Source compat includes
-
-#include <kio/copyjob.h>
-#include <kio/deletejob.h>
-
 #endif
