@@ -70,7 +70,7 @@ KDirListerCache::KDirListerCache( int maxCount )
   connect( kdirwatch, SIGNAL( deleted( const QString& ) ),
            this, SLOT( slotFileDeleted( const QString& ) ) );
 
-  kdirnotify = new org::kde::KDirNotify(QString(), QString(), QDBus::sessionBus(), this);
+  kdirnotify = new org::kde::KDirNotify(QString(), QString(), QDBusConnection::sessionBus(), this);
   connect(kdirnotify, SIGNAL(FileRenamed(QString,QString)), SLOT(slotFileRenamed(QString,QString)));
   connect(kdirnotify, SIGNAL(FilesAdded(QString)), SLOT(slotFilesAdded(QString)));
   connect(kdirnotify, SIGNAL(FilesChanged(QStringList)), SLOT(slotFilesChanged(QStringList)));
@@ -102,7 +102,7 @@ bool KDirListerCache::listDir( KDirLister *lister, const KUrl& _u,
   _url.adjustPath(KUrl::RemoveTrailingSlash);
   QString urlStr = _url.url();
 
-  if ( !validURL( lister, _url ) )
+  if ( !validUrl( lister, _url ) )
     return false;
 
 #ifdef DEBUG_CACHE
@@ -286,7 +286,7 @@ bool KDirListerCache::listDir( KDirLister *lister, const KUrl& _u,
   return true;
 }
 
-bool KDirListerCache::validURL( const KDirLister *lister, const KUrl& url ) const
+bool KDirListerCache::validUrl( const KDirLister *lister, const KUrl& url ) const
 {
   if ( !url.isValid() )
   {
@@ -648,9 +648,9 @@ bool KDirListerCache::checkUpdate( const QString& _dir )
     return true;
 }
 
-KFileItem *KDirListerCache::itemForURL( const KUrl& url ) const
+KFileItem *KDirListerCache::itemForUrl( const KUrl& url ) const
 {
-  return findByURL( 0, url );
+  return findByUrl( 0, url );
 }
 
 KFileItemList *KDirListerCache::itemsForDir( const KUrl& _dir ) const
@@ -677,7 +677,7 @@ KFileItem *KDirListerCache::findByName( const KDirLister *lister, const QString&
   return 0;
 }
 
-KFileItem *KDirListerCache::findByURL( const KDirLister *lister, const KUrl& _u ) const
+KFileItem *KDirListerCache::findByUrl( const KDirLister *lister, const KUrl& _u ) const
 {
   KUrl _url = _u;
   _url.adjustPath(KUrl::RemoveTrailingSlash);
@@ -691,7 +691,7 @@ KFileItem *KDirListerCache::findByURL( const KDirLister *lister, const KUrl& _u
 
   const KFileItemList *itemList = itemsForDir( parentDir );
   if ( itemList )
-    return itemList->findByURL( _url );
+    return itemList->findByUrl( _url );
   return 0;
 }
 
@@ -758,7 +758,7 @@ void KDirListerCache::slotFilesChanged( const QStringList &fileList ) // from KD
     if ( url.isLocalFile() )
     {
       kDebug(7004) << "KDirListerCache::slotFilesChanged " << url << endl;
-      KFileItem *fileitem = findByURL( 0, url );
+      KFileItem *fileitem = findByUrl( 0, url );
       if ( fileitem )
       {
           // we need to refresh the item, because e.g. the permissions can have changed.
@@ -801,7 +801,7 @@ void KDirListerCache::slotFileRenamed( const QString &_src, const QString &_dst
   // Now update the KFileItem representing that file or dir (not exclusive with the above!)
   KUrl oldurl( src );
   oldurl.adjustPath( KUrl::RemoveTrailingSlash );
-  KFileItem *fileitem = findByURL( 0, oldurl );
+  KFileItem *fileitem = findByUrl( 0, oldurl );
   if ( fileitem )
   {
     if ( !fileitem->isLocalFile() && !fileitem->localPath().isEmpty() ) // it uses UDS_LOCAL_PATH? ouch, needs an update then
@@ -809,7 +809,7 @@ void KDirListerCache::slotFileRenamed( const QString &_src, const QString &_dst
     else
     {
         aboutToRefreshItem( fileitem );
-        fileitem->setURL( dst );
+        fileitem->setUrl( dst );
         fileitem->refreshMimeType();
         emitRefreshItem( fileitem );
     }
@@ -911,7 +911,7 @@ void KDirListerCache::slotFileDirtyDelayed()
 
   KUrl u;
   u.setPath( file );
-  KFileItem *item = findByURL( 0, u ); // search all items
+  KFileItem *item = findByUrl( 0, u ); // search all items
   if ( item )
   {
     // we need to refresh the item, because e.g. the permissions can have changed.
@@ -1363,7 +1363,7 @@ void KDirListerCache::renameDir( const KUrl &oldUrl, const KUrl &newUrl )
         newItemUrl.setPath( newDirUrl.path() );
         newItemUrl.addPath( oldItemUrl.fileName() );
         kDebug(7004) << "KDirListerCache::renameDir renaming " << oldItemUrlStr << " to " << newItemUrl.url() << endl;
-        (*kit)->setURL( newItemUrl );
+        (*kit)->setUrl( newItemUrl );
       }
       emitRedirections( oldDirUrl, newDirUrl );
     }
@@ -1526,17 +1526,16 @@ void KDirListerCache::slotUpdateResult( KJob * j )
   for ( kdl = listers->first(); kdl; kdl = listers->next() )
     delayedMimeTypes &= kdl->d->delayedMimeTypes;
 
-  // should be enough to get reasonable speed in most cases
-  Q3Dict<KFileItem> fileItems( 9973 );
+  QHash<QString, KFileItem *> fileItems; // fileName -> KFileItem*
 
   // Unmark all items in url
   for ( KFileItemList::iterator kit = dir->lstItems.begin(), kend = dir->lstItems.end() ; kit != kend ; ++kit )
   {
     (*kit)->unmark();
-    fileItems.insert( (*kit)->url().url(), *kit );
+    fileItems.insert( (*kit)->name(), *kit );
   }
 
-  KFileItem *item = 0, *tmp;
+  KFileItem *tmp;
 
   KIO::UDSEntryList buf = jobs.value( job );
   KIO::UDSEntryList::ConstIterator it = buf.begin();
@@ -1544,12 +1543,9 @@ void KDirListerCache::slotUpdateResult( KJob * j )
   for ( ; it != end; ++it )
   {
     // Form the complete url
-    if ( !item )
-      item = new KFileItem( *it, jobUrl, delayedMimeTypes, true );
-    else
-      item->setUDSEntry( *it, jobUrl, delayedMimeTypes, true );
+    KFileItem item( *it, jobUrl, delayedMimeTypes, true );
 
-    const QString name = item->name();
+    const QString name = item.name();
     Q_ASSERT( !name.isEmpty() );
 
     // we duplicate the check for dotdot here, to avoid iterating over
@@ -1563,53 +1559,45 @@ void KDirListerCache::slotUpdateResult( KJob * j )
       // there is no root item yet
       if ( !dir->rootItem )
       {
-        dir->rootItem = item;
-        item = 0;
+        dir->rootItem = new KFileItem(item);
 
         for ( KDirLister *kdl = listers->first(); kdl; kdl = listers->next() )
           if ( !kdl->d->rootFileItem && kdl->d->url == jobUrl )
             kdl->d->rootFileItem = dir->rootItem;
       }
-
       continue;
     }
 
     // Find this item
-    if ( (tmp = fileItems[item->url().url()]) )
+    if ( (tmp = fileItems.value(item.name())) )
     {
-      tmp->mark();
-
       // check if something changed for this file
-      if ( !tmp->cmp( *item ) )
+      if ( !tmp->cmp( item ) )
       {
         for ( kdl = listers->first(); kdl; kdl = listers->next() )
           kdl->aboutToRefreshItem( tmp );
 
         //kDebug(7004) << "slotUpdateResult: file changed: " << tmp->name() << endl;
-        tmp->assign( *item );
+        *tmp = item;
 
         for ( kdl = listers->first(); kdl; kdl = listers->next() )
           kdl->addRefreshItem( tmp );
       }
+      tmp->mark();
     }
     else // this is a new file
     {
       //kDebug(7004) << "slotUpdateResult: new file: " << name << endl;
 
-      item->mark();
-      dir->lstItems.append( item );
+      KFileItem* pitem = new KFileItem(item); // we're not using kfileitem by value yet
+      pitem->mark();
+      dir->lstItems.append( pitem );
 
       for ( kdl = listers->first(); kdl; kdl = listers->next() )
-        kdl->addNewItem( item );
-
-      // item used, we need a new one for the next iteration
-      item = 0;
+        kdl->addNewItem( pitem );
     }
   }
 
-  if ( item )
-    delete item;
-
   jobs.remove( job );
 
   deleteUnmarkedItems( listers, dir->lstItems );
@@ -1651,8 +1639,8 @@ KIO::ListJob *KDirListerCache::jobForUrl( const QString& url, KIO::ListJob *not_
 
 const KUrl& KDirListerCache::joburl( KIO::ListJob *job )
 {
-  if ( job->redirectionURL().isValid() )
-     return job->redirectionURL();
+  if ( job->redirectionUrl().isValid() )
+     return job->redirectionUrl();
   else
      return job->url();
 }
@@ -1822,14 +1810,12 @@ void KDirListerCache::printDebug()
 /*********************** -- The new KDirLister -- ************************/
 
 
-KDirLister::KDirLister( bool _delayedMimeTypes )
-	:d(new KDirListerPrivate)
+KDirLister::KDirLister( QObject* parent )
+    : QObject(parent), d(new KDirListerPrivate)
 {
   kDebug(7003) << "+KDirLister" << endl;
 
-
   d->complete = true;
-  d->delayedMimeTypes = _delayedMimeTypes;
 
   setAutoUpdate( true );
   setDirOnlyMode( false );
@@ -1849,7 +1835,7 @@ KDirLister::~KDirLister()
   delete d;
 }
 
-bool KDirLister::openURL( const KUrl& _url, bool _keep, bool _reload )
+bool KDirLister::openUrl( const KUrl& _url, bool _keep, bool _reload )
 {
   kDebug(7003) << k_funcinfo << _url.prettyUrl()
                 << " keep=" << _keep << " reload=" << _reload << endl;
@@ -2042,9 +2028,9 @@ KFileItem *KDirLister::rootItem() const
   return d->rootFileItem;
 }
 
-KFileItem *KDirLister::findByURL( const KUrl& _url ) const
+KFileItem *KDirLister::findByUrl( const KUrl& _url ) const
 {
-  return s_pCache->findByURL( this, _url );
+  return s_pCache->findByUrl( this, _url );
 }
 
 KFileItem *KDirLister::findByName( const QString& _name ) const
@@ -2392,7 +2378,7 @@ void KDirLister::slotProcessedSize( KJob *job, qulonglong size )
   emit processedSize( result );
 }
 
-void KDirLister::slotSpeed( KIO::Job *job, unsigned long spd )
+void KDirLister::slotSpeed( KJob *job, unsigned long spd )
 {
   d->jobData[static_cast<KIO::ListJob *>(job)].speed = spd;
 
@@ -2439,8 +2425,8 @@ void KDirLister::connectJob( KIO::ListJob *job )
            this, SLOT(slotTotalSize( KJob *, qulonglong )) );
   connect( job, SIGNAL(processedSize( KJob *, qulonglong )),
            this, SLOT(slotProcessedSize( KJob *, qulonglong )) );
-  connect( job, SIGNAL(speed( KIO::Job *, unsigned long )),
-           this, SLOT(slotSpeed( KIO::Job *, unsigned long )) );
+  connect( job, SIGNAL(speed( KJob *, unsigned long )),
+           this, SLOT(slotSpeed( KJob *, unsigned long )) );
 }
 
 void KDirLister::setMainWindow( QWidget *window )
@@ -2460,15 +2446,15 @@ KFileItemList KDirLister::items( WhichItems which ) const
 
 KFileItemList KDirLister::itemsForDir( const KUrl& dir, WhichItems which ) const
 {
-    KFileItemList result;
     KFileItemList *allItems = s_pCache->itemsForDir( dir );
     if ( !allItems )
-        return result;
+        return KFileItemList();
 
     if ( which == AllItems )
-        result = *allItems; // shallow copy
+        return *allItems;
     else // only items passing the filters
     {
+        KFileItemList result;
         KFileItemList::const_iterator kit = allItems->begin();
         const KFileItemList::const_iterator kend = allItems->end();
         for ( ; kit != kend; ++kit )
@@ -2478,15 +2464,19 @@ KFileItemList KDirLister::itemsForDir( const KUrl& dir, WhichItems which ) const
             if ( !isExcluded && matchesMimeFilter( item ) )
                 result.append( item );
         }
+        return result;
     }
-
-    return result;
 }
 
-// to keep BC changes
+bool KDirLister::delayedMimeTypes() const
+{
+    return d->delayedMimeTypes;
+}
 
-void KDirLister::virtual_hook( int, void * )
-{ /*BASE::virtual_hook( id, data );*/ }
+void KDirLister::setDelayedMimeTypes( bool delayedMimeTypes )
+{
+    d->delayedMimeTypes = delayedMimeTypes;
+}
 
 #include "kdirlister.moc"
 #include "kdirlister_p.moc"
