@@ -49,6 +49,7 @@
 #include <kglobal.h>
 #include <kstaticdeleter.h>
 #include <kde_file.h>
+#include <kconfiggroup.h>
 
 // debug
 #include <sys/ioctl.h>
@@ -258,7 +259,7 @@ void KDirWatchPrivate::slotActivated()
           if ( event->mask & (IN_CREATE|IN_MOVED_TO) ) {
             Entry* sub_entry = 0;
             Q_FOREACH(sub_entry, e->m_entries)
-              if (sub_entry->path == e->path + "/" + path) break;
+              if (sub_entry->path == e->path + '/' + path) break;
 
             if (sub_entry /*&& sub_entry->isDir*/) {
               removeEntry(0,e->path, sub_entry);
@@ -715,7 +716,7 @@ void KDirWatchPrivate::removeEntries( KDirWatch* instance )
       minfreq = (*it).freq;
   }
 
-  foreach(QString path, pathList)
+  foreach(const QString &path, pathList)
     removeEntry(instance, path, 0);
 
   if (minfreq > freq) {
