@@ -313,7 +313,7 @@ void KFileMetaInfo::init( const KUrl& url, const QString& mimeType,
 
     QString mT;
     if (mimeType.isEmpty())
-        mT = KMimeType::findByURL(url)->name();
+        mT = KMimeType::findByUrl(url)->name();
     else
         mT = mimeType;
 
@@ -329,11 +329,15 @@ void KFileMetaInfo::init( const KUrl& url, const QString& mimeType,
         KFilePlugin *p = plugin();
         Q_ASSERT( p );
         if ( p && !p->readInfo( item, what) )
+        {
+            deref();
             d = Data::makeNull();
+        }
     }
     else
     {
 //        kDebug(7033) << "No mimetype info for " << mimeType << endl;
+        deref();
         d = Data::makeNull();
     }
 }
@@ -536,7 +540,7 @@ bool KFileMetaInfo::applyChanges( const QString& path )
         else
         {
             QStringList keys = group.keys();
-            foreach (QString key, keys)
+            foreach (const QString &key, keys)
             {
                 if ( group[key].isModified() )
                 {
@@ -568,7 +572,7 @@ bool KFileMetaInfo::applyChanges( const QString& path )
     return ret;
 }
 
-KFilePlugin * const KFileMetaInfo::plugin() const
+KFilePlugin * KFileMetaInfo::plugin() const
 {
     assert(isValid());
     KFileMetaInfoProvider* prov = KFileMetaInfoProvider::self();
@@ -782,10 +786,6 @@ KFileMimeTypeInfo * KFilePlugin::addMimeTypeInfo( const QString& mimeType )
     return KFileMetaInfoProvider::self()->addMimeTypeInfo( mimeType );
 }
 
-void KFilePlugin::virtual_hook( int, void* )
-{ /*BASE::virtual_hook( id, data );*/ }
-
-
 KFileMimeTypeInfo::GroupInfo*  KFilePlugin::addGroupInfo(KFileMimeTypeInfo* info,
                   const QString& key, const QString& translatedKey) const
 {
