@@ -24,7 +24,7 @@
    m_unit is a define in <sys/sysmacros.h> */
 #define m_unit outouftheway_m_unit
 
-#include <QHash>
+#include <QtCore/QHash>
 #include <qvariant.h>
 #include <qobject.h>
 #include <qstring.h>
@@ -1226,7 +1226,7 @@ protected:
      * @return a pointer to the plugin that belogs to this object's mimetype.
      *         It will be auto-loaded if it's currently not loaded
      **/
-    KFilePlugin * const plugin() const;
+    KFilePlugin * plugin() const;
 
     void ref();
     void deref();
@@ -1605,15 +1605,6 @@ protected:
     QStringList m_preferredKeys;
     QStringList m_preferredGroups;
 
-protected:
-    /**
-     * Helper method to allow binary compatible extensions when needing
-     * "new virtual methods"
-     *
-     * @param id the identifier of the new "virtual" method
-     * @param data any parameter data the new "virtual" method needs
-     */
-    virtual void virtual_hook( int id, void* data );
 private:
     class KFilePluginPrivate;
     KFilePluginPrivate *d;
