@@ -56,7 +56,6 @@ KMimeMagic* KMimeMagic::self()
 void KMimeMagic::initStatic()
 {
   s_pSelf = kmimemagicsd.setObject( s_pSelf, new KMimeMagic() );
-  s_pSelf->setFollowLinks( true );
 }
 
 #include <stdio.h>
@@ -64,7 +63,6 @@ void KMimeMagic::initStatic()
 #include <stdlib.h>
 #include <sys/wait.h>
 #include <sys/types.h>
-#include <sys/stat.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <ctype.h>
@@ -97,7 +95,7 @@ void KMimeMagic::initStatic()
 #define MIME_INODE_CDEV        "inode/chardevice"
 #define MIME_INODE_BDEV        "inode/blockdevice"
 #define MIME_INODE_FIFO        "inode/fifo"
-#define MIME_INODE_LINK        "inode/link"
+#define MIME_INODE_LINK        "inode/symlink"
 #define MIME_INODE_SOCK        "inode/socket"
 // Following should go in magic-file - Fritz
 #define MIME_APPL_TROFF        "application/x-troff"
@@ -580,7 +578,6 @@ public:
 
 /* current config */
 struct config_rec {
-    bool followLinks;
     QString resultBuf;
     int accuracy;
 
@@ -1522,10 +1519,8 @@ fsmagic(struct config_rec* conf, const char *fn, KDE_struct_stat *sb)
             } else
                 strcpy(buf, tmp);
         }
-        if (conf->followLinks)
-            process( conf, QFile::decodeName( buf ) );
-        else
-            conf->resultBuf = MIME_INODE_LINK;
+        // Follow symlinks
+        process( conf, QFile::decodeName( buf ) );
         return 1;
     }
     return 1;
@@ -2126,7 +2121,8 @@ KMimeMagic::KMimeMagic()
     QString mimefile = KStandardDirs::locate( "mime", "magic" );
     init( mimefile );
     // Add snippets from share/config/magic/*
-    QStringList snippets = KGlobal::dirs()->findAllResources( "config", "magic/*.magic", true );
+    QStringList snippets = KGlobal::dirs()->findAllResources( "config", "magic/*.magic", 
+                                                              KStandardDirs::Recursive );
     for ( QStringList::Iterator it = snippets.begin() ; it != snippets.end() ; ++it )
         if ( !mergeConfig( *it ) )
             kWarning() << k_funcinfo << "Failed to parse " << *it << endl;
@@ -2145,7 +2141,6 @@ void KMimeMagic::init( const QString& _configfile )
 	/* set up the magic list (empty) */
 	conf->magic = conf->last = NULL;
 	magicResult = NULL;
-	conf->followLinks = false;
 
         conf->utimeConf = 0L; // created on demand
 	/* on the first time through we read the magic file */
@@ -2212,12 +2207,6 @@ KMimeMagic::mergeBufConfig(char * _configbuf)
 	return false;
 }
 
-void
-KMimeMagic::setFollowLinks( bool _enable )
-{
-	conf->followLinks = _enable;
-}
-
 KMimeMagicResult *
 KMimeMagic::findBufferType(const QByteArray &array)
 {
