@@ -1,6 +1,6 @@
 /*  This file is part of the KDE libraries
  *  Copyright (C) 1999 Waldo Bastian <bastian@kde.org>
- *                     David Faure   <faure@kde.org>
+ *                2000-2007 David Faure <faure@kde.org>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -25,6 +25,7 @@
 #include "kprotocolmanager.h"
 #include <kde_file.h>
 
+#include <kconfiggroup.h>
 #include <kmessageboxwrapper.h>
 
 #include <kdebug.h>
@@ -50,48 +51,77 @@
 #include <stddef.h>
 #include <unistd.h>
 #include <stdlib.h>
-#include "kmimetypefactory.h"
 
 template class KSharedPtr<KMimeType>;
 
-KMimeType::Ptr KMimeType::s_pDefaultType;
-bool KMimeType::s_bChecked = false;
+static KMimeType::Ptr s_pDefaultMimeType;
 
+static void errorMissingMimeType( const QString& _type )
+{
+    KMessageBoxWrapper::sorry(
+        0, i18n( "Could not find mime type\n%1", _type ) );
+}
+
+// KDE4 TODO: when we are in kdecore, share d pointer with KServiceType
+class KMimeType::Private
+{
+public:
+    QStringList m_lstPatterns;
+    QString m_strIcon;
+    void loadInternal( QDataStream& _str );
+};
+
+void KMimeType::Private::loadInternal( QDataStream& _str )
+{
+    // kDebug(7009) << "KMimeType::load( QDataStream& ) : loading list of patterns" << endl;
+    _str >> m_lstPatterns >> m_strIcon;
+}
+
+/**
+ * This function makes sure that the default mime type exists.
+ * Not file-static because it needs access to the private KMimeType constructor...
+ */
 void KMimeType::buildDefaultType()
 {
-  assert ( !s_pDefaultType );
-  // Try to find the default type
-  KMimeType::Ptr mime = KMimeTypeFactory::self()->
-        findMimeTypeByName( defaultMimeType() );
+    assert ( !s_pDefaultMimeType );
+    // Try to find the default type
+    KMimeType::Ptr mime = KMimeTypeFactory::self()->
+                          findMimeTypeByName( KMimeType::defaultMimeType() );
 
-  if (mime)
-  {
-      s_pDefaultType = mime;
-  }
-  else
-  {
-     errorMissingMimeType( defaultMimeType() );
-     KStandardDirs stdDirs;
-     QString sDefaultMimeType = stdDirs.resourceDirs("mime").first()+defaultMimeType()+".desktop";
-     s_pDefaultType = new KMimeType( sDefaultMimeType, defaultMimeType(),
-                                     "unknown", "mime", QStringList() );
-  }
+    if (mime)
+    {
+        s_pDefaultMimeType = mime;
+    }
+    else
+    {
+        QString defaultMimeType = KMimeType::defaultMimeType();
+        errorMissingMimeType( defaultMimeType );
+        // TODO: port to xdg mime
+        KStandardDirs stdDirs;
+        QString sDefaultMimeType = stdDirs.resourceDirs("mime").first()+defaultMimeType+".desktop";
+        s_pDefaultMimeType = new KMimeType( sDefaultMimeType, defaultMimeType,
+                                            "unknown", "mime" );
+    }
 }
 
 KMimeType::Ptr KMimeType::defaultMimeTypePtr()
 {
-  if ( !s_pDefaultType ) // we need a default type first
-    buildDefaultType();
-  return s_pDefaultType;
+    if ( !s_pDefaultMimeType ) // we need a default type first
+        buildDefaultType();
+    return s_pDefaultMimeType;
 }
 
-// Check for essential mimetypes
+/**
+ * This function makes sure that vital mime types are installed.
+ */
 void KMimeType::checkEssentialMimeTypes()
 {
-  if ( s_bChecked ) // already done
-    return;
-  if ( !s_pDefaultType ) // we need a default type first
-    buildDefaultType();
+    static bool s_bChecked = false;
+
+    if ( s_bChecked ) // already done
+        return;
+    if ( !s_pDefaultMimeType ) // we need a default type first
+        KMimeType::buildDefaultType();
 
   s_bChecked = true; // must be done before building mimetypes
 
@@ -123,13 +153,6 @@ void KMimeType::checkEssentialMimeTypes()
     errorMissingMimeType( "application/x-desktop" );
 }
 
-void KMimeType::errorMissingMimeType( const QString& _type )
-{
-  QString tmp = i18n( "Could not find mime type\n%1" ,  _type );
-
-  KMessageBoxWrapper::sorry( 0, tmp );
-}
-
 KMimeType::Ptr KMimeType::mimeType( const QString& _name )
 {
   KMimeType::Ptr mime = KMimeTypeFactory::self()->findMimeTypeByName( _name );
@@ -139,13 +162,14 @@ KMimeType::Ptr KMimeType::mimeType( const QString& _name )
   // is used instead.
   if ( !mime || !mime->isType( KST_KMimeType ) )
   {
-    if ( !s_pDefaultType )
+    if ( !s_pDefaultMimeType )
       buildDefaultType();
-    return s_pDefaultType;
+    return s_pDefaultMimeType;
   }
 
   // We got a mimetype
 #endif
+
   return mime;
 }
 
@@ -154,20 +178,25 @@ KMimeType::List KMimeType::allMimeTypes()
   return KMimeTypeFactory::self()->allMimeTypes();
 }
 
-KMimeType::Ptr KMimeType::findByURL( const KUrl& _url, mode_t _mode,
-                                     bool _is_local_file, bool _fast_mode )
+KMimeType::Ptr KMimeType::findByUrlHelper( const KUrl& _url, mode_t _mode,
+                                           bool _is_local_file, bool _fast_mode,
+                                           bool& needsMagic )
 {
   checkEssentialMimeTypes();
   QString path = _url.path();
+  needsMagic = false;
 
-  if ( !_fast_mode && !_is_local_file && _url.isLocalFile() )
-    _is_local_file = true;
-
-  if ( !_fast_mode && _is_local_file && (_mode == 0 || _mode == (mode_t)-1) )
+  if ( !_fast_mode )
   {
-    KDE_struct_stat buff;
-    if ( KDE_stat( QFile::encodeName(path), &buff ) != -1 )
-      _mode = buff.st_mode;
+      if ( !_is_local_file && _url.isLocalFile() )
+          _is_local_file = true;
+
+      if ( _is_local_file && (_mode == 0 || _mode == (mode_t)-1) )
+      {
+          KDE_struct_stat buff;
+          if ( KDE_stat( QFile::encodeName(path), &buff ) != -1 )
+              _mode = buff.st_mode;
+      }
   }
 
   // Look at mode_t first
@@ -207,17 +236,23 @@ KMimeType::Ptr KMimeType::findByURL( const KUrl& _url, mode_t _mode,
         if ( _is_local_file || _url.hasSubUrl() || // Explicitly trust suburls
              KProtocolInfo::determineMimetypeFromExtension( _url.protocol() ) )
         {
-            if ( _is_local_file && !_fast_mode ) {
-                if ( mime->patternsAccuracy()<100 )
-                {
+            if ( mime->patternsAccuracy() < 100 ) {
+                if ( _is_local_file && !_fast_mode ) {
                     KMimeMagicResult* result =
-                            KMimeMagic::self()->findFileType( path );
+                        KMimeMagic::self()->findFileType( path );
 
-                    if ( result && result->isValid() )
-                        return mimeType( result->mimeType() );
+                    if ( result && result->isValid() && result->accuracy() > 0 ) {
+                        mime = mimeType( result->mimeType() );
+                        if (!mime) {
+                            kWarning() << "KMimeMagic returned unknown mimetype " << result->mimeType() << endl;
+                            return defaultMimeTypePtr();
+                        }
+                        return mime;
+                    }
+                } else {
+                    needsMagic = true;
                 }
             }
-
             return mime;
         }
       }
@@ -268,7 +303,7 @@ KMimeType::Ptr KMimeType::findByURL( const KUrl& _url, mode_t _mode,
   }
 
   // Do some magic for local files
-  //kDebug(7009) << QString("Mime Type finding for '%1'").arg(path) << endl;
+  kDebug(7009) << QString("Mime Type finding for '%1'").arg(path) << endl;
   KMimeMagicResult* result = KMimeMagic::self()->findFileType( path );
 
   // If we still did not find it, we must assume the default mime type
@@ -276,15 +311,43 @@ KMimeType::Ptr KMimeType::findByURL( const KUrl& _url, mode_t _mode,
     return defaultMimeTypePtr();
 
   // The mimemagic stuff was successful
-  return mimeType( result->mimeType() );
+  KMimeType::Ptr mime = mimeType( result->mimeType() );
+  if (!mime) {
+      kWarning() << "KMimeMagic returned unknown mimetype " << result->mimeType() << endl;
+      return defaultMimeTypePtr();
+  }
+  return mime;
 }
 
-KMimeType::Ptr KMimeType::findByURL( const KUrl& _url, mode_t _mode,
+KMimeType::Ptr KMimeType::findByUrl( const KUrl& _url, mode_t _mode,
                                      bool _is_local_file, bool _fast_mode,
                                      bool *accurate )
 {
-    KMimeType::Ptr mime = findByURL(_url, _mode, _is_local_file, _fast_mode);
-    if (accurate) *accurate = !(_fast_mode) || ((mime->patternsAccuracy() == 100) && mime != defaultMimeTypePtr());
+    bool needsMagic;
+    KMimeType::Ptr mime = findByUrlHelper(_url, _mode, _is_local_file, _fast_mode, needsMagic);
+    if (accurate) {
+        if (_is_local_file && !_fast_mode)
+            *accurate = true; // we used KMimeMagic, so the result is as good as can be
+        else
+            // we didn't use mimemagic; the result is accurate only if the patterns were not marked as unreliable
+            *accurate = !needsMagic && mime != defaultMimeTypePtr();
+    }
+    return mime;
+}
+
+KMimeType::Ptr KMimeType::findByNameAndContent( const QString& name, const QByteArray& data, mode_t _mode )
+{
+    // First look at mode and name
+    bool needsMagic;
+    KMimeType::Ptr mime = findByUrlHelper(name, _mode, false, true, needsMagic);
+
+    if ( needsMagic || !mime || mime->name() == KMimeType::defaultMimeType() ) {
+        // findByUrl found nothing conclusive, look at the data
+        KMimeMagicResult * result = KMimeMagic::self()->findBufferFileType(data, name);
+        if ( result->mimeType() != KMimeType::defaultMimeType() )
+          mime = KMimeType::mimeType( result->mimeType() );
+    }
+
     return mime;
 }
 
@@ -301,7 +364,7 @@ KMimeType::Ptr KMimeType::findByPath( const QString& path, mode_t mode, bool fas
 {
     KUrl u;
     u.setPath(path);
-    return findByURL( u, mode, true, fast_mode );
+    return findByUrl( u, mode, true, fast_mode );
 }
 
 KMimeType::Ptr KMimeType::findByContent( const QByteArray &data, int *accuracy )
@@ -349,24 +412,15 @@ KMimeType::Format KMimeType::findFormatByFileContent( const QString &fileName )
   return result;
 }
 
-// Used to create the default mimetype, in particular
+// Used only to create the default mimetype
 KMimeType::KMimeType( const QString & _fullpath, const QString& _type, const QString& _icon,
-                      const QString& _comment, const QStringList& _patterns )
-  : KServiceType( _fullpath, _type, _comment ), m_strIcon( _icon ), d( 0 )
+                      const QString& _comment )
+  : KServiceType( _fullpath, _type, _comment ), d(new Private)
 {
-  m_lstPatterns = _patterns;
+    d->m_strIcon = _icon;
 }
 
-KMimeType::KMimeType( const QString & _fullpath ) : KServiceType( _fullpath ), d( 0 )
-{
-  KDesktopFile _cfg( _fullpath, true );
-  init ( &_cfg );
-
-  if ( !isValid() )
-    kWarning(7009) << "mimetype not valid '" << name() << "' (missing entry in the file ?)" << endl;
-}
-
-KMimeType::KMimeType( KDesktopFile *config ) : KServiceType( config ), d( 0 )
+KMimeType::KMimeType( KDesktopFile *config ) : KServiceType( config ), d(new Private)
 {
   init( config );
 
@@ -377,8 +431,8 @@ KMimeType::KMimeType( KDesktopFile *config ) : KServiceType( config ), d( 0 )
 void KMimeType::init( KDesktopFile * config )
 {
   config->setDesktopGroup();
-  m_strIcon = config->readIcon();
-  m_lstPatterns = config->readEntry( "Patterns", QStringList(), ';' );
+  d->m_strIcon = config->readIcon();
+  d->m_lstPatterns = config->readEntry( "Patterns", QStringList(), ';' );
 
   // Read the X-KDE-AutoEmbed setting and store it in the properties map
   QString XKDEAutoEmbed = QLatin1String("X-KDE-AutoEmbed");
@@ -403,21 +457,15 @@ void KMimeType::init( KDesktopFile * config )
     m_mapProps.insert( XKDEPatternsAccuracy, config->readEntry( XKDEPatternsAccuracy, QString() ) );
 }
 
-KMimeType::KMimeType( QDataStream& _str, int offset ) : KServiceType( _str, offset ), d( 0 )
+KMimeType::KMimeType( QDataStream& _str, int offset ) : KServiceType( _str, offset ), d(new Private)
 {
-  loadInternal( _str ); // load our specific stuff
+  d->loadInternal( _str ); // load our specific stuff
 }
 
 void KMimeType::load( QDataStream& _str )
 {
   KServiceType::load( _str );
-  loadInternal( _str );
-}
-
-void KMimeType::loadInternal( QDataStream& _str )
-{
-  // kDebug(7009) << "KMimeType::load( QDataStream& ) : loading list of patterns" << endl;
-  _str >> m_lstPatterns >> m_strIcon;
+  d->loadInternal( _str );
 }
 
 void KMimeType::save( QDataStream& _str )
@@ -425,15 +473,15 @@ void KMimeType::save( QDataStream& _str )
   KServiceType::save( _str );
   // Warning adding/removing fields here involves a binary incompatible change - update version
   // number in ksycoca.h
-  _str << m_lstPatterns << m_strIcon;
+  _str << d->m_lstPatterns << d->m_strIcon;
 }
 
 QVariant KMimeType::property( const QString& _name ) const
 {
   if ( _name == "Patterns" )
-    return QVariant( m_lstPatterns );
+    return QVariant( d->m_lstPatterns );
   if ( _name == "Icon" )
-    return QVariant( m_strIcon );
+    return QVariant( d->m_strIcon );
 
   return KServiceType::property( _name );
 }
@@ -451,29 +499,12 @@ KMimeType::~KMimeType()
 {
 }
 
-QPixmap KMimeType::pixmap( K3Icon::Group _group, int _force_size, int _state,
-                           QString * _path ) const
+QString KMimeType::iconNameForUrl( const KUrl & _url, mode_t _mode )
 {
-  return KGlobal::iconLoader()->loadMimeTypeIcon( icon(), _group, _force_size, _state, _path );
-}
-
-QPixmap KMimeType::pixmap( const KUrl& _url, K3Icon::Group _group, int _force_size,
-                           int _state, QString * _path ) const
-{
-  return KGlobal::iconLoader()->loadMimeTypeIcon( icon( _url ), _group, _force_size, _state, _path );
-}
-
-QString KMimeType::iconForURL( const KUrl & _url, mode_t _mode )
-{
-  return iconNameForURL(_url,_mode);
-}
-
-QString KMimeType::iconNameForURL( const KUrl & _url, mode_t _mode )
-{
-    const KMimeType::Ptr mt = findByURL( _url, _mode, _url.isLocalFile(),
+    const KMimeType::Ptr mt = findByUrl( _url, _mode, _url.isLocalFile(),
                                          false /*HACK*/);
     static const QString& unknown = KGlobal::staticQString("unknown");
-    const QString mimeTypeIcon = mt->icon( _url );
+    const QString mimeTypeIcon = mt->iconName( _url );
     QString i = mimeTypeIcon;
 
     // if we don't find an icon, maybe we can use the one for the protocol
@@ -481,7 +512,7 @@ QString KMimeType::iconNameForURL( const KUrl & _url, mode_t _mode )
         // and for the root of the protocol (e.g. trash:/) the protocol icon has priority over the mimetype icon
         || _url.path().length() <= 1 )
     {
-        i = favIconForURL( _url ); // maybe there is a favicon?
+        i = favIconForUrl( _url ); // maybe there is a favicon?
 
         if ( i.isEmpty() )
             i = KProtocolInfo::icon( _url.protocol() );
@@ -490,10 +521,10 @@ QString KMimeType::iconNameForURL( const KUrl & _url, mode_t _mode )
         if ( _url.path().length() <= 1 && ( i == unknown || i.isEmpty() ) )
             i = mimeTypeIcon;
     }
-    return i;
+    return !i.isEmpty() ? i : unknown;
 }
 
-QString KMimeType::favIconForURL( const KUrl& url )
+QString KMimeType::favIconForUrl( const KUrl& url )
 {
     // this method will be called quite often, so better not read the config
     // again and again.
@@ -510,7 +541,7 @@ QString KMimeType::favIconForURL( const KUrl& url )
         return QString();
 
     QDBusInterface kded( "org.kde.kded", "/modules/favicons", "org.kde.FavIcon" );
-    QDBusReply<QString> result = kded.call( "iconForURL", url.url() );
+    QDBusReply<QString> result = kded.call( "iconForUrl", url.url() );
     return result;              // default is QString()
 }
 
@@ -565,7 +596,7 @@ QString KFolderType::icon( const QString& _url, bool _is_local ) const
 QString KFolderType::icon( const KUrl& _url ) const
 {
   if ( _url.isEmpty() || !_url.isLocalFile() )
-    return KMimeType::icon( _url );
+    return KMimeType::iconName( _url );
 
   KUrl u( _url );
   u.addPath( ".directory" );
@@ -614,7 +645,7 @@ QString KFolderType::icon( const KUrl& _url ) const
   }
 
   if ( icon.isEmpty() )
-    return KMimeType::icon( _url );
+    return KMimeType::iconName( _url );
 
   if ( icon.startsWith( "./" ) ) {
     // path is relative with respect to the location
@@ -660,11 +691,26 @@ const QString & KMimeType::defaultMimeType()
     return s_strDefaultMimeType;
 }
 
+QString KMimeType::iconName() const
+{
+    return d->m_strIcon;
+}
+
+QString KMimeType::iconName( const KUrl& ) const
+{
+    return d->m_strIcon;
+}
+
+const QStringList& KMimeType::patterns() const
+{
+    return d->m_lstPatterns;
+}
+
 void KMimeType::virtual_hook( int id, void* data )
 { KServiceType::virtual_hook( id, data ); }
 
 void KFolderType::virtual_hook( int id, void* data )
 { KMimeType::virtual_hook( id, data ); }
 
-#include "kmimetyperesolver.moc"
+#include "k3mimetyperesolver.moc"
 
