@@ -234,7 +234,7 @@ QString KProtocolManager::proxyFor( const QString& protocol )
   return cfg->readEntry( scheme + "Proxy", QString() );
 }
 
-QString KProtocolManager::proxyForURL( const KUrl &url )
+QString KProtocolManager::proxyForUrl( const KUrl &url )
 {
   QString proxy;
   ProxyType pt = proxyType();
@@ -351,7 +351,7 @@ QString KProtocolManager::slaveProtocol(const KUrl &url, QString &proxy)
 
   if (useProxy())
   {
-     proxy = proxyForURL(url);
+     proxy = proxyForUrl(url);
      if ((proxy != "DIRECT") && (!proxy.isEmpty()))
      {
         bool isRevMatch = false;
@@ -655,6 +655,15 @@ bool KProtocolManager::supportsMoving( const KUrl &url )
   return prot->m_supportsMoving;
 }
 
+bool KProtocolManager::supportsOpening( const KUrl &url )
+{
+  KProtocolInfo::Ptr prot = findProtocol(url);
+  if ( !prot )
+    return false;
+
+  return prot->m_supportsOpening;
+}
+
 bool KProtocolManager::canCopyFromFile( const KUrl &url )
 {
   KProtocolInfo::Ptr prot = findProtocol(url);
@@ -706,7 +715,7 @@ KProtocolInfo::FileNameUsedForCopying KProtocolManager::fileNameUsedForCopying(
 {
   KProtocolInfo::Ptr prot = findProtocol(url);
   if ( !prot )
-    return KProtocolInfo::FromURL;
+    return KProtocolInfo::FromUrl;
 
   return prot->fileNameUsedForCopying();
 }
