@@ -217,7 +217,7 @@ public:
    * @returns the proxy server address if one is available
    *          or QString() otherwise
    */
-  static QString proxyForURL( const KUrl& url );
+  static QString proxyForUrl( const KUrl& url );
 
   /**
    * Marks this proxy as bad (down). It will not be used for the
@@ -406,6 +406,17 @@ public:
    */
   static bool supportsMoving( const KUrl &url );
 
+  /**
+   * Returns whether the protocol can be opened using KIO::open(const KUrl&).
+   *
+   * This corresponds to the "opening=" field in the protocol description file.
+   * Valid values for this field are "true" or "false" (default).
+   *
+   * @param url the url to check
+   * @return true if the protocol supports opening
+   */
+  static bool supportsOpening( const KUrl &url );
+
   /**
    * Returns whether the protocol can copy files/objects directly from the
    * filesystem itself. If not, the application will read files from the
