@@ -39,7 +39,7 @@
 #include "kio/global.h"
 #include "kio/scheduler.h"
 #include "kio/netaccess.h"
-#include "kfile/kopenwith.h"
+#include "kfile/kopenwithdialog.h"
 #include "kfile/krecentdocument.h"
 #include "kdedesktopmimetype.h"
 
@@ -64,6 +64,7 @@
 #include <kshell.h>
 #include <QTextDocument>
 #include <kde_file.h>
+#include <kconfiggroup.h>
 
 #ifdef Q_WS_X11
 #include <kwin.h>
@@ -166,7 +167,7 @@ pid_t KRun::runUrl( const KUrl& u, const QString& _mimetype, QWidget* window, bo
   {
     // Open-with dialog
     // TODO : pass the mimetype as a parameter, to show it (comment field) in the dialog !
-    // Hmm, in fact KOpenWithDlg::setServiceType already guesses the mimetype from the first URL of the list...
+    // Hmm, in fact KOpenWithDialog::setServiceType already guesses the mimetype from the first URL of the list...
     return displayOpenWithDialog( lst, window, tempFile, suggestedFileName );
   }
 
@@ -181,7 +182,7 @@ bool KRun::displayOpenWithDialog( const KUrl::List& lst, QWidget* window, bool t
        return false;
     }
 
-    KOpenWithDlg l( lst, i18n("Open with:"), QString(), window );
+    KOpenWithDialog l( lst, i18n("Open with:"), QString(), window );
     if ( l.exec() )
     {
       KService::Ptr service = l.service();
@@ -275,7 +276,7 @@ KRunMX2::subst( int option, const KUrl &url, QStringList &ret )
 {
    switch( option ) {
    case 'u':
-      ret << (url.isLocalFile() ? url.path() : url.url());
+      ret << url.pathOrUrl();
       break;
    case 'd':
       ret << url.directory();
@@ -539,7 +540,7 @@ bool KRun::checkStartupNotify( const QString& /*binName*/, const KService* servi
   {
       if( service )
       {
-          if( service->type() == "Application" )
+          if( service->isApplication() )
               wmclass = "0"; // doesn't have .desktop entries needed, start as non-compliant
           else
               return false; // no startup notification at all
@@ -634,7 +635,7 @@ static KUrl::List resolveURLs( const KUrl::List& _urls, const KService& _service
       if ( !supported && KProtocolInfo::protocolClass(url.protocol()) == ":local" )
       {
         // Maybe we can resolve to a local URL?
-        KUrl localURL = KIO::NetAccess::mostLocalURL( url, 0 );
+        KUrl localURL = KIO::NetAccess::mostLocalUrl( url, 0 );
         if ( localURL != url ) {
           *it = localURL;
           kDebug(7010) << "Changed to " << localURL << endl;
@@ -771,7 +772,7 @@ void KRun::init()
     m_timer.start( 0 );
     return;
   }
-  if ( !KAuthorized::authorizeURLAction( "open", KUrl(), m_strURL))
+  if ( !KAuthorized::authorizeUrlAction( "open", KUrl(), m_strURL))
   {
     QString msg = KIO::buildErrorString(KIO::ERR_ACCESS_DENIED, m_strURL.prettyUrl());
     d->m_showingError = true;
@@ -810,7 +811,7 @@ void KRun::init()
       m_mode = buff.st_mode;
     }
 
-    KMimeType::Ptr mime = KMimeType::findByURL( m_strURL, m_mode, m_bIsLocalFile );
+    KMimeType::Ptr mime = KMimeType::findByUrl( m_strURL, m_mode, m_bIsLocalFile );
     assert( mime );
     kDebug(7010) << "MIME TYPE is " << mime->name() << endl;
     foundMimeType( mime->name() );
@@ -904,7 +905,7 @@ void KRun::scanFile()
   // Not when there is a query in the URL, in any case.
   if ( m_strURL.query().isEmpty() )
   {
-    KMimeType::Ptr mime = KMimeType::findByURL( m_strURL );
+    KMimeType::Ptr mime = KMimeType::findByUrl( m_strURL );
     assert( mime );
     if ( mime->name() != "application/octet-stream" || m_bIsLocalFile )
     {
@@ -1316,7 +1317,4 @@ KProcessRunner::slotProcessExited(KProcess * p)
   deleteLater();
 }
 
-void KRun::virtual_hook( int, void* )
-{ /*BASE::virtual_hook( id, data );*/ }
-
 #include "krun.moc"
