@@ -25,7 +25,7 @@
 #include <qfile.h>
 #include <kdebug.h>
 #include <kmimetype.h>
-#include <ktempfile.h>
+#include <ktemporaryfile.h>
 
 #include <kfilterdev.h>
 #include <kfilterbase.h>
@@ -43,7 +43,7 @@ public:
     KTarPrivate() : tarEnd( 0 ), tmpFile( 0 ) {}
     QStringList dirList;
     qint64 tarEnd;
-    KTempFile* tmpFile;
+    KTemporaryFile* tmpFile;
     QString mimetype;
     QByteArray origFileName;
 
@@ -127,11 +127,14 @@ bool KTar::createDevice( QIODevice::OpenMode mode )
         // has to walk through the decompression filter each time.
         // Which is in fact nearly as slow as a complete decompression for each file.
 
-        d->tmpFile = new KTempFile( KStandardDirs::locateLocal("tmp", "ktar-"),".tar");
-        kDebug( 7041 ) << "KTar::createDevice creating TempFile: " << d->tmpFile->name() << endl;
-        d->tmpFile->setAutoDelete(true);
+        Q_ASSERT(!d->tmpFile);
+        d->tmpFile = new KTemporaryFile();
+        d->tmpFile->setPrefix("ktar-");
+        d->tmpFile->setSuffix(".tar");
+        d->tmpFile->open();
+        kDebug( 7041 ) << "KTar::createDevice creating TempFile: " << d->tmpFile->fileName() << endl;
 
-        setDevice( d->tmpFile->file() );
+        setDevice( d->tmpFile );
         return true;
     }
 }
@@ -271,13 +274,10 @@ bool KTar::KTarPrivate::fillTempFile( const QString & fileName) {
     QIODevice *filterDev = KFilterDev::deviceForFile( fileName, mimetype, forced );
 
     if( filterDev ) {
-        QFile* file = tmpFile->file();
-        file->close();
-        if ( ! file->open( QIODevice::WriteOnly ) )
-        {
-            delete filterDev;
-            return false;
-        }
+        QFile* file = tmpFile;
+        Q_ASSERT(file->isOpen());
+        Q_ASSERT(file->openMode() & QIODevice::WriteOnly);
+        file->seek(0);
         QByteArray buffer;
         buffer.resize(8*1024);
         if ( ! filterDev->open( QIODevice::ReadOnly ) )
@@ -300,9 +300,10 @@ bool KTar::KTarPrivate::fillTempFile( const QString & fileName) {
         filterDev->close();
         delete filterDev;
 
-        file->close();
-        if ( ! file->open( QIODevice::ReadOnly ) )
-            return false;
+        file->flush();
+        file->seek(0);
+        Q_ASSERT(file->isOpen());
+        Q_ASSERT(file->openMode() & QIODevice::ReadOnly);
     }
     else
         kDebug( 7041 ) << "KTar::openArchive: no filterdevice found!" << endl;
@@ -415,8 +416,8 @@ bool KTar::openArchive( QIODevice::OpenMode mode ) {
                     // Let's hack around hard links. Our classes don't support that, so make them symlinks
                     if ( typeflag == '1' )
                     {
-                        size = nm.length(); // in any case, we don't want to skip the real size, hence this resetting of size
-                        kDebug(7041) << "HARD LINK, setting size to " << size << endl;
+                        kDebug(7041) << "HARD LINK, setting size to 0 instead of " << size << endl;
+                        size = 0; // no contents
                     }
 
                     //kDebug(7041) << "KTar::openArchive file " << nm << " size=" << size << endl;
@@ -485,9 +486,8 @@ bool KTar::KTarPrivate::writeBackTempFile( const QString & fileName ) {
 
     QIODevice *dev = KFilterDev::deviceForFile( fileName, mimetype, forced );
     if( dev ) {
-        QFile* file = tmpFile->file();
-        file->close();
-        if ( ! file->open(QIODevice::ReadOnly) || ! dev->open(QIODevice::WriteOnly) )
+        QFile* file = tmpFile;
+        if ( !dev->open(QIODevice::WriteOnly) )
         {
             file->close();
             delete dev;
@@ -495,6 +495,7 @@ bool KTar::KTarPrivate::writeBackTempFile( const QString & fileName ) {
         }
         if ( forced )
             static_cast<KFilterDev *>(dev)->setOrigFileName( origFileName );
+        file->seek(0);
         QByteArray buffer;
         buffer.resize(8*1024);
         qint64 len;
@@ -514,19 +515,25 @@ bool KTar::KTarPrivate::writeBackTempFile( const QString & fileName ) {
 bool KTar::closeArchive() {
     d->dirList.clear();
 
+    bool ok = true;
+
     // If we are in write mode and had created
     // a temporary tar file, we have to write
     // back the changes to the original file
-    if( mode() == QIODevice::WriteOnly)
-        return d->writeBackTempFile( fileName() );
+    if( mode() == QIODevice::WriteOnly) {
+        ok = d->writeBackTempFile( fileName() );
+        delete d->tmpFile;
+        d->tmpFile = 0;
+        setDevice(0);
+    }
 
-    return true;
+    return ok;
 }
 
 bool KTar::doFinishWriting( qint64 size ) {
     // Write alignment
     int rest = size % 0x200;
-    if ( mode() & QIODevice::ReadWrite )
+    if ( ( mode() & QIODevice::ReadWrite ) == QIODevice::ReadWrite )
         d->tarEnd = device()->pos() + (rest ? 0x200 - rest : 0); // Record our new end of archive
     if ( rest )
     {
@@ -682,7 +689,8 @@ bool KTar::doPrepareWriting(const QString &name, const QString &user,
 
     char buffer[ 0x201 ];
     memset( buffer, 0, 0x200 );
-    if ( mode() & QIODevice::ReadWrite ) device()->seek(d->tarEnd); // Go to end of archive as might have moved with a read
+    if ( ( mode() & QIODevice::ReadWrite ) == QIODevice::ReadWrite )
+        device()->seek(d->tarEnd); // Go to end of archive as might have moved with a read
 
     // provide converted stuff we need later on
     const QByteArray encodedFileName = QFile::encodeName(fileName);
@@ -734,7 +742,8 @@ bool KTar::doWriteDir(const QString &name, const QString &user,
 
     char buffer[ 0x201 ];
     memset( buffer, 0, 0x200 );
-    if ( mode() & QIODevice::ReadWrite ) device()->seek(d->tarEnd); // Go to end of archive as might have moved with a read
+    if ( ( mode() & QIODevice::ReadWrite ) == QIODevice::ReadWrite )
+        device()->seek(d->tarEnd); // Go to end of archive as might have moved with a read
 
     // provide converted stuff we need lateron
     QByteArray encodedDirname = QFile::encodeName(dirName);
@@ -757,7 +766,7 @@ bool KTar::doWriteDir(const QString &name, const QString &user,
 
     // Write header
     device()->write( buffer, 0x200 );
-    if ( mode() & QIODevice::ReadWrite )
+    if ( ( mode() & QIODevice::ReadWrite ) == QIODevice::ReadWrite )
         d->tarEnd = device()->pos();
 
     d->dirList.append( dirName ); // contains trailing slash
@@ -784,7 +793,8 @@ bool KTar::doWriteSymLink(const QString &name, const QString &target,
 
     char buffer[ 0x201 ];
     memset( buffer, 0, 0x200 );
-    if ( mode() & QIODevice::ReadWrite ) device()->seek(d->tarEnd); // Go to end of archive as might have moved with a read
+    if ( ( mode() & QIODevice::ReadWrite ) == QIODevice::ReadWrite )
+        device()->seek(d->tarEnd); // Go to end of archive as might have moved with a read
 
     // provide converted stuff we need lateron
     QByteArray encodedFileName = QFile::encodeName(fileName);
@@ -813,7 +823,7 @@ bool KTar::doWriteSymLink(const QString &name, const QString &target,
 
     // Write header
     bool retval = device()->write( buffer, 0x200 ) == 0x200;
-    if ( mode() & QIODevice::ReadWrite )
+    if ( ( mode() & QIODevice::ReadWrite ) == QIODevice::ReadWrite )
         d->tarEnd = device()->pos();
     return retval;
 }
