@@ -127,8 +127,8 @@ Scheduler::Scheduler()
 
     const QString dbusPath = "/KIO/Scheduler";
     const QString dbusInterface = "org.kde.KIO.Scheduler";
-    QDBusConnection dbus = QDBus::sessionBus();
-    dbus.registerObject("/KIO/Scheduler", this, QDBusConnection::ExportSlots);
+    QDBusConnection dbus = QDBusConnection::sessionBus();
+    dbus.registerObject("/KIO/Scheduler", this, QDBusConnection::ExportScriptableSlots);
     dbus.connect(QString(), dbusPath, dbusInterface, "reparseSlaveConfiguration",
                  this, SLOT(slotReparseSlaveConfiguration(QString)));
 #ifdef __GNUC__
@@ -880,9 +880,4 @@ Scheduler* Scheduler::self() {
     return instance;
 }
 
-void Scheduler::virtual_hook( int, void* )
-{ /* BASE::virtual_hook( id, data ); */ }
-
-
-
 #include "scheduler.moc"
