@@ -40,6 +40,7 @@
 #include <qfile.h>
 #include <qlist.h>
 #include <QtDBus/QtDBus>
+#include "uiserveriface.h"
 
 #include <kapplication.h>
 #include <kcrash.h>
@@ -63,6 +64,11 @@
 #endif
 #endif
 
+extern "C" {
+    static void sigsegv_handler(int sig);
+    static void sigpipe_handler(int sig);
+}
+
 using namespace KIO;
 
 typedef QList<QByteArray> AuthKeysList;
@@ -114,6 +120,7 @@ public:
     bool needSendCanResume:1;
     bool onHold:1;
     bool wasKilled:1;
+    bool exit_loop:1;
     MetaData configData;
     SlaveBaseConfig *config;
     KUrl onHoldUrl;
@@ -226,6 +233,7 @@ SlaveBase::SlaveBase( const QByteArray &protocol,
     connectSlave(mAppSocket);
 
     d->remotefile = 0;
+    d->exit_loop = false;
 }
 
 SlaveBase::~SlaveBase()
@@ -236,11 +244,10 @@ SlaveBase::~SlaveBase()
 
 void SlaveBase::dispatchLoop()
 {
-#ifdef Q_OS_UNIX //TODO: WIN32
     fd_set rfds;
     int retval;
 
-    while (true)
+    while (!d->exit_loop)
     {
        if (d->timeout && (d->timeout < time(0)))
        {
@@ -269,14 +276,19 @@ void SlaveBase::dispatchLoop()
        { // dispatch application messages
           int cmd;
           QByteArray data;
-          if ( appconn->read(&cmd, data) != -1 )
+          int ret = appconn->read(&cmd, data);
+#ifdef Q_WS_WIN
+          if ( ret == -2 ) //win32: WSAEWOULDBLOCK condition
+          	continue;
+#endif
+          if ( ret != -1 )
           {
              dispatch(cmd, data);
           }
           else // some error occurred, perhaps no more application
           {
              // When the app exits, should the slave be put back in the pool ?
-             if (mConnectedToApp && !mPoolSocket.isEmpty())
+             if (!d->exit_loop && mConnectedToApp && !mPoolSocket.isEmpty())
              {
                 disconnectSlave();
                 mConnectedToApp = false;
@@ -303,13 +315,16 @@ void SlaveBase::dispatchLoop()
           return;
        }
     }
-#endif
 }
 
 void SlaveBase::connectSlave(const QString& path)
 {
-#ifdef Q_OS_UNIX //TODO: KSocket not yet available on WIN32
+#ifdef Q_WS_WIN
+    // @TODO: localhost does not work yet
+    KNetwork::KStreamSocket *sock = new KNetwork::KStreamSocket(getenv("COMPUTERNAME"),
+#else
     KNetwork::KStreamSocket *sock = new KNetwork::KStreamSocket(QString(),
+#endif
 						QFile::encodeName(path));
     appconn->init(sock);
 
@@ -318,10 +333,10 @@ void SlaveBase::connectSlave(const QString& path)
         kDebug(7019) << "SlaveBase: failed to connect to " << path << endl
 		      << "Reason: " << sock->errorString() << endl;
         exit();
+        return;
     }
 
     setConnection(appconn);
-#endif
 }
 
 void SlaveBase::disconnectSlave()
@@ -361,9 +376,7 @@ void SlaveBase::sendMetaData()
 {
    KIO_DATA << mOutgoingMetaData;
 
-   slaveWriteError = false;
-   m_pConnection->send( INF_META_DATA, data );
-   if (slaveWriteError) exit();
+   send( INF_META_DATA, data );
    mOutgoingMetaData.clear(); // Clear
 }
 
@@ -380,9 +393,7 @@ void SlaveBase::data( const QByteArray &data )
 {
    if (!mOutgoingMetaData.isEmpty())
       sendMetaData();
-   slaveWriteError = false;
-   m_pConnection->send( MSG_DATA, data );
-   if (slaveWriteError) exit();
+   send( MSG_DATA, data );
 }
 
 void SlaveBase::dataReq( )
@@ -393,7 +404,14 @@ void SlaveBase::dataReq( )
 */
    if (d->needSendCanResume)
       canResume(0);
-   m_pConnection->send( MSG_DATA_REQ );
+   send( MSG_DATA_REQ );
+}
+
+void SlaveBase::opened()
+{
+   if (!mOutgoingMetaData.isEmpty())
+      sendMetaData();
+   send( MSG_OPENED );
 }
 
 void SlaveBase::error( int _errid, const QString &_text )
@@ -402,7 +420,7 @@ void SlaveBase::error( int _errid, const QString &_text )
     mOutgoingMetaData.clear();
     KIO_DATA << (qint32) _errid << _text;
 
-    m_pConnection->send( MSG_ERROR, data );
+    send( MSG_ERROR, data );
     //reset
     listEntryCurrentSize = 100;
     d->sentListEntries=0;
@@ -411,9 +429,7 @@ void SlaveBase::error( int _errid, const QString &_text )
 
 void SlaveBase::connected()
 {
-    slaveWriteError = false;
-    m_pConnection->send( MSG_CONNECTED );
-    if (slaveWriteError) exit();
+    send( MSG_CONNECTED );
 }
 
 void SlaveBase::finished()
@@ -421,7 +437,7 @@ void SlaveBase::finished()
     mIncomingMetaData.clear(); // Clear meta data
     if (!mOutgoingMetaData.isEmpty())
        sendMetaData();
-    m_pConnection->send( MSG_FINISHED );
+    send( MSG_FINISHED );
 
     // reset
     listEntryCurrentSize = 100;
@@ -429,9 +445,9 @@ void SlaveBase::finished()
     d->totalSize=0;
 }
 
-void SlaveBase::needSubURLData()
+void SlaveBase::needSubUrlData()
 {
-    m_pConnection->send( MSG_NEED_SUBURL_DATA );
+    send( MSG_NEED_SUBURL_DATA );
 }
 
 void SlaveBase::slaveStatus( const QString &host, bool connected )
@@ -441,20 +457,18 @@ void SlaveBase::slaveStatus( const QString &host, bool connected )
     KIO_DATA << pid << mProtocol << host << b;
     if (d->onHold)
        stream << d->onHoldUrl;
-    m_pConnection->send( MSG_SLAVE_STATUS, data );
+    send( MSG_SLAVE_STATUS, data );
 }
 
 void SlaveBase::canResume()
 {
-    m_pConnection->send( MSG_CANRESUME );
+    send( MSG_CANRESUME );
 }
 
 void SlaveBase::totalSize( KIO::filesize_t _bytes )
 {
     KIO_DATA << KIO_FILESIZE_T(_bytes);
-    slaveWriteError = false;
-    m_pConnection->send( INF_TOTAL_SIZE, data );
-    if (slaveWriteError) exit();
+    send( INF_TOTAL_SIZE, data );
 
     //this one is usually called before the first item is listed in listDir()
     struct timeval tp;
@@ -490,9 +504,7 @@ void SlaveBase::processedSize( KIO::filesize_t _bytes )
 
     if( emitSignal ) {
         KIO_DATA << KIO_FILESIZE_T(_bytes);
-        slaveWriteError = false;
-        m_pConnection->send( INF_PROCESSED_SIZE, data );
-            if (slaveWriteError) exit();
+        send( INF_PROCESSED_SIZE, data );
         if ( gettimeofday_res == 0 ) {
             d->last_tv.tv_sec = tv.tv_sec;
             d->last_tv.tv_usec = tv.tv_usec;
@@ -501,6 +513,18 @@ void SlaveBase::processedSize( KIO::filesize_t _bytes )
 //    d->processed_size = _bytes;
 }
 
+void SlaveBase::written( KIO::filesize_t _bytes )
+{
+    KIO_DATA << KIO_FILESIZE_T(_bytes);
+    send( MSG_WRITTEN, data );
+}
+
+void SlaveBase::position( KIO::filesize_t _pos )
+{
+    KIO_DATA << KIO_FILESIZE_T(_pos);
+    send( INF_POSITION, data );
+}
+
 void SlaveBase::processedPercent( float /* percent */ )
 {
   kDebug(7019) << "SlaveBase::processedPercent: STUB" << endl;
@@ -510,20 +534,18 @@ void SlaveBase::processedPercent( float /* percent */ )
 void SlaveBase::speed( unsigned long _bytes_per_second )
 {
     KIO_DATA << (quint32) _bytes_per_second;
-    slaveWriteError = false;
-    m_pConnection->send( INF_SPEED, data );
-    if (slaveWriteError) exit();
+    send( INF_SPEED, data );
 }
 
 void SlaveBase::redirection( const KUrl& _url )
 {
     KIO_DATA << _url;
-    m_pConnection->send( INF_REDIRECTION, data );
+    send( INF_REDIRECTION, data );
 }
 
 void SlaveBase::errorPage()
 {
-    m_pConnection->send( INF_ERROR_PAGE );
+    send( INF_ERROR_PAGE );
 }
 
 static bool isSubCommand(int cmd)
@@ -549,17 +571,21 @@ void SlaveBase::mimeType( const QString &_type)
     {
       // kDebug(7019) << "(" << getpid() << ") mimeType: emitting meta data" << endl;
       KIO_DATA << mOutgoingMetaData;
-      m_pConnection->send( INF_META_DATA, data );
+      send( INF_META_DATA, data );
     }
     KIO_DATA << _type;
-    m_pConnection->send( INF_MIME_TYPE, data );
+    send( INF_MIME_TYPE, data );
     while(true)
     {
        cmd = 0;
-       if ( m_pConnection->read( &cmd, data ) == -1 ) {
+       int ret = m_pConnection->read( &cmd, data );
+       if ( ret == -1 ) {
            kDebug(7019) << "SlaveBase: mimetype: read error" << endl;
            exit();
+           return;
        }
+       if ( ret == -2 ) // win32: WSAEWOULDBLOCK condition
+          continue;
        // kDebug(7019) << "(" << getpid() << ") Slavebase: mimetype got " << cmd << endl;
        if ( cmd == CMD_HOST) // Ignore.
           continue;
@@ -577,26 +603,30 @@ void SlaveBase::mimeType( const QString &_type)
 
 void SlaveBase::exit()
 {
-    this->~SlaveBase();
+    d->exit_loop = true;
+    // Using ::exit() here is too much (crashes in qdbus's qglobalstatic object),
+    // so let's cleanly exit dispatchLoop() instead.
+    // Update: we do need to call exit(), otherwise a long download (get()) would
+    // keep going until it ends, even though the application exited.
     ::exit(255);
 }
 
 void SlaveBase::warning( const QString &_msg)
 {
     KIO_DATA << _msg;
-    m_pConnection->send( INF_WARNING, data );
+    send( INF_WARNING, data );
 }
 
 void SlaveBase::infoMessage( const QString &_msg)
 {
     KIO_DATA << _msg;
-    m_pConnection->send( INF_INFOMESSAGE, data );
+    send( INF_INFOMESSAGE, data );
 }
 
 bool SlaveBase::requestNetwork(const QString& host)
 {
     KIO_DATA << host << d->slaveid;
-    m_pConnection->send( MSG_NET_REQUEST, data );
+    send( MSG_NET_REQUEST, data );
 
     if ( waitForAnswer( INF_NETWORK_STATUS, 0, data ) != -1 )
     {
@@ -611,15 +641,13 @@ bool SlaveBase::requestNetwork(const QString& host)
 void SlaveBase::dropNetwork(const QString& host)
 {
     KIO_DATA << host << d->slaveid;
-    m_pConnection->send( MSG_NET_DROP, data );
+    send( MSG_NET_DROP, data );
 }
 
 void SlaveBase::statEntry( const UDSEntry& entry )
 {
     KIO_DATA << entry;
-    slaveWriteError = false;
-    m_pConnection->send( MSG_STAT_ENTRY, data );
-    if (slaveWriteError) exit();
+    send( MSG_STAT_ENTRY, data );
 }
 
 void SlaveBase::listEntry( const UDSEntry& entry, bool _ready )
@@ -671,13 +699,11 @@ void SlaveBase::listEntries( const UDSEntryList& list )
     const UDSEntryList::ConstIterator end = list.end();
     for (; it != end; ++it)
       stream << *it;
-    slaveWriteError = false;
-    m_pConnection->send( MSG_LIST_ENTRIES, data);
-    if (slaveWriteError) exit();
+    send( MSG_LIST_ENTRIES, data);
     d->sentListEntries+=(uint)list.count();
 }
 
-void SlaveBase::sigsegv_handler(int sig)
+static void sigsegv_handler(int sig)
 {
 #ifdef Q_OS_UNIX
     signal(sig,SIG_DFL); // Next one kills
@@ -703,7 +729,7 @@ void SlaveBase::sigsegv_handler(int sig)
 #endif
 }
 
-void SlaveBase::sigpipe_handler (int)
+static void sigpipe_handler (int)
 {
     // We ignore a SIGPIPE in slaves.
     // A SIGPIPE can happen in two cases:
@@ -732,6 +758,8 @@ void SlaveBase::listDir(KUrl const &)
 { error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_LISTDIR)); }
 void SlaveBase::get(KUrl const & )
 { error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_GET)); }
+void SlaveBase::open(KUrl const &, int )
+{ error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_OPEN)); }
 void SlaveBase::mimetype(KUrl const &url)
 { get(url); }
 void SlaveBase::rename(KUrl const &, KUrl const &, bool)
@@ -748,7 +776,7 @@ void SlaveBase::mkdir(KUrl const &, int)
 { error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_MKDIR)); }
 void SlaveBase::chmod(KUrl const &, int)
 { error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_CHMOD)); }
-void SlaveBase::setSubURL(KUrl const &)
+void SlaveBase::setSubUrl(KUrl const &)
 { error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_SUBURL)); }
 void SlaveBase::multiGet(const QByteArray &)
 { error(  ERR_UNSUPPORTED_ACTION, unsupportedActionErrorString(mProtocol, CMD_MULTI_GET)); }
@@ -767,22 +795,31 @@ bool SlaveBase::dispatch()
 
     int cmd;
     QByteArray data;
-    if ( m_pConnection->read( &cmd, data ) == -1 )
+    int ret = m_pConnection->read( &cmd, data );
+    if ( ret == -1 )
     {
         kDebug(7019) << "SlaveBase::dispatch() has read error." << endl;
         return false;
     }
+    if ( ret == -2 ) // win32: WSAEWOULDBLOCK condition
+      return true;
 
     dispatch( cmd, data );
     return true;
 }
 
-bool SlaveBase::openPassDlg( AuthInfo& info, const QString &errorMsg )
+bool SlaveBase::openPasswordDialog( AuthInfo& info, const QString &errorMsg )
 {
     AuthInfo authResult;
-    long windowId = metaData("window-id").toLong();
+    const long windowId = metaData("window-id").toLong();
+    const unsigned long userTimestamp = metaData("user-timestamp").toULong();
 
-    kDebug(7019) << "SlaveBase::openPassDlg window-id=" << windowId << endl;
+    org::kde::KIO::UIServer uiserver("org.kde.kuiserver", "/UIServer", QDBusConnection::sessionBus());
+    const long progressId = metaData("progress-id").toLong();
+    if (progressId)
+        uiserver.setJobVisible(progressId, false);
+
+    kDebug(7019) << "SlaveBase::openPasswordDialog window-id=" << windowId << endl;
 
     QDBusInterface kps( "org.kde.kded", "/modules/kpasswdserver", "org.kde.KPasswdServer" );
 
@@ -795,29 +832,32 @@ bool SlaveBase::openPassDlg( AuthInfo& info, const QString &errorMsg )
 
     if (metaData("no-auth-prompt").toLower() == "true")
        reply = kps.call("queryAuthInfo", data, QString(QLatin1String("<NoAuthPrompt>")),
-                         qlonglong(windowId), s_seqNr);
+                         qlonglong(windowId), s_seqNr, qlonglong(userTimestamp));
     else
-       reply = kps.call("queryAuthInfo", data, errorMsg, qlonglong(windowId), s_seqNr);
+       reply = kps.call("queryAuthInfo", data, errorMsg, qlonglong(windowId), s_seqNr, qlonglong(userTimestamp));
 
+    bool callOK = reply.type() == QDBusMessage::ReplyMessage;
+    //if (progressId)
+    //    uiserver.setJobVisible(progressId, true);
 
-    if ( reply.type() != QDBusMessage::ReplyMessage )
+    if (!callOK)
     {
        kWarning(7019) << "Can't communicate with kded_kpasswdserver (for queryAuthInfo)!" << endl;
-       kDebug(7019) << reply.at(0).toString() << endl;
+       kDebug(7019) << reply.arguments().at(0).toString() << endl;
        return false;
     }
 
-    QDataStream stream2( reply.at(1).toByteArray() );
+    QDataStream stream2( reply.arguments().at(1).toByteArray() );
     stream2 >> authResult;
-    s_seqNr = reply.at(1).toLongLong();
+    s_seqNr = reply.arguments().at(1).toLongLong();
 
     if (!authResult.isModified())
        return false;
 
     info = authResult;
 
-    kDebug(7019) << "SlaveBase::openPassDlg: username=" << info.username << endl;
-    kDebug(7019) << "SlaveBase::openPassDlg: password=[hidden]" << endl;
+    kDebug(7019) << "SlaveBase::openPasswordDialog: username=" << info.username << endl;
+    kDebug(7019) << "SlaveBase::openPasswordDialog: password=[hidden]" << endl;
 
     return true;
 }
@@ -833,7 +873,7 @@ int SlaveBase::messageBox( const QString &text, MessageBoxType type, const QStri
 {
     kDebug(7019) << "messageBox " << type << " " << text << " - " << caption << buttonYes << buttonNo << endl;
     KIO_DATA << (qint32)type << text << caption << buttonYes << buttonNo << dontAskAgainName;
-    m_pConnection->send( INF_MESSAGEBOX, data );
+    send( INF_MESSAGEBOX, data );
     if ( waitForAnswer( CMD_MESSAGEBOXANSWER, 0, data ) != -1 )
     {
         QDataStream stream( data );
@@ -850,7 +890,7 @@ bool SlaveBase::canResume( KIO::filesize_t offset )
     kDebug(7019) << "SlaveBase::canResume offset=" << KIO::number(offset) << endl;
     d->needSendCanResume = false;
     KIO_DATA << KIO_FILESIZE_T(offset);
-    m_pConnection->send( MSG_RESUME, data );
+    send( MSG_RESUME, data );
     if ( offset )
     {
         int cmd;
@@ -878,6 +918,9 @@ int SlaveBase::waitForAnswer( int expected1, int expected2, QByteArray & data, i
             kDebug(7019) << "SlaveBase::waitForAnswer has read error." << endl;
             return -1;
         }
+        if ( result == -2 ) // win32: WSAEWOULDBLOCK condition
+          continue;
+
         if ( cmd == expected1 || cmd == expected2 )
         {
             if ( pCmd ) *pCmd = cmd;
@@ -979,6 +1022,11 @@ void SlaveBase::dispatch( int command, const QByteArray &data )
         stream >> url;
         get( url );
     } break;
+    case CMD_OPEN:
+    {
+        stream >> url >> i;
+        open( url, i );
+    } break;
     case CMD_PUT:
     {
         int permissions;
@@ -1054,7 +1102,7 @@ void SlaveBase::dispatch( int command, const QByteArray &data )
         break;
     case CMD_SUBURL:
         stream >> url;
-        setSubURL(url);
+        setSubUrl(url);
         break;
     case CMD_NONE:
         fprintf(stderr, "Got unexpected CMD_NONE!\n");
@@ -1073,6 +1121,7 @@ bool SlaveBase::checkCachedAuthentication( AuthInfo& info )
 {
     AuthInfo authResult;
     long windowId = metaData("window-id").toLong();
+    unsigned long userTimestamp = metaData("user-timestamp").toULong();
 
     kDebug(7019) << "SlaveBase::checkCachedAuthInfo window = " << windowId << " url = " << info.url.url() << endl;
 
@@ -1083,7 +1132,7 @@ bool SlaveBase::checkCachedAuthentication( AuthInfo& info )
        QDataStream stream(&data, QIODevice::WriteOnly);
        stream << info;
     }
-    QDBusReply<QByteArray> reply = kps.call("checkAuthInfo", data, qlonglong(windowId));
+    QDBusReply<QByteArray> reply = kps.call("checkAuthInfo", data, qlonglong(windowId), qlonglong(userTimestamp));
 
     if ( !reply.isValid() )
     {
@@ -1172,6 +1221,14 @@ void SlaveBase::setKillFlag()
    d->wasKilled=true;
 }
 
+void SlaveBase::send(int cmd, const QByteArray& arr )
+{
+   slaveWriteError = false;
+   if (!m_pConnection->send(cmd, arr))
+       // Note that slaveWriteError can also be set by sigpipe_handler
+       slaveWriteError = true;
+   if (slaveWriteError) exit();
+}
+
 void SlaveBase::virtual_hook( int, void* )
 { /*BASE::virtual_hook( id, data );*/ }
-
