@@ -90,6 +90,12 @@ public:
      */
     void dataReq( );
 
+    /**
+     * open succedes
+     * @see open
+     */
+    void opened();
+
     /**
      * Call to signal an error.
      * This also finishes the job, no need to call finished.
@@ -123,7 +129,7 @@ public:
     /**
      * Call to signal that data from the sub-URL is needed
      */
-    void needSubURLData();
+    void needSubUrlData();
 
     /**
      * Used to report the status of the slave.
@@ -180,6 +186,10 @@ public:
      */
     void processedSize( KIO::filesize_t _bytes );
 
+    void position( KIO::filesize_t _pos );
+
+    void written( KIO::filesize_t _bytes );
+
     /**
      * Only use this if you can't know in advance the size of the
      * copied data. For example, if you're doing variable bitrate
@@ -332,7 +342,7 @@ public:
     /**
      * Prepare slave for streaming operation
      */
-    virtual void setSubURL(const KUrl&url);
+    virtual void setSubUrl(const KUrl&url);
 
     /**
      * Opens the connection (forced)
@@ -363,6 +373,14 @@ public:
      */
     virtual void get( const KUrl& url );
 
+    /**
+     * open.
+     * @param url the full url for this request. Host, port and user of the URL
+     *        can be assumed to be the same as in the last setHost() call.
+     * @param access 1 - rdonly, 2 - wronly or 3 - rdwr
+     */
+    virtual void open( const KUrl& url, int access );
+
     /**
      * put, i.e. write data into a file.
      *
@@ -378,7 +396,7 @@ public:
      *
      * @see canResume()
      */
-    virtual void put( const KUrl& url, int permissions, bool overwrite, bool /*resume*/ );
+    virtual void put( const KUrl& url, int permissions, bool overwrite, bool resume );
 
     /**
      * Finds all details for one file or directory.
@@ -548,15 +566,6 @@ public:
      */
     void setTimeoutSpecialCommand(int timeout, const QByteArray &data=QByteArray());
 
-    /**
-     * @internal
-     */
-    static void sigsegv_handler(int);
-    /**
-     * @internal
-     */
-    static void sigpipe_handler(int);
-
     /////////////////
     // Dispatching (internal)
     ////////////////
@@ -610,7 +619,7 @@ public:
      *
      * \code
      * KIO::AuthInfo authInfo;
-     * if ( openPassDlg( authInfo ) )
+     * if ( openPasswordDialog( authInfo ) )
      * {
      *    kDebug() << QLatin1String("User: ")
      *              << authInfo.username << endl;
@@ -627,7 +636,7 @@ public:
      * authInfo.caption= "Acme Password Dialog";
      * authInfo.username= "Wile E. Coyote";
      * QString errorMsg = "You entered an incorrect password.";
-     * if ( openPassDlg( authInfo, errorMsg ) )
+     * if ( openPasswordDialog( authInfo, errorMsg ) )
      * {
      *    kDebug() << QLatin1String("User: ")
      *              << authInfo.username << endl;
@@ -648,7 +657,7 @@ public:
      * @param errorMsg Error message to show
      * @return      @p true if user clicks on "OK", @p false otherwsie.
      */
-    bool openPassDlg( KIO::AuthInfo& info, const QString &errorMsg = QString() );
+    bool openPasswordDialog( KIO::AuthInfo& info, const QString &errorMsg = QString() );
 
     /**
      * Checks for cached authentication based on parameters
@@ -669,7 +678,7 @@ public:
      * info.verifyPath = true;
      * if ( !checkCachedAuthentication( info ) )
      * {
-     *    if ( !openPassDlg(info) )
+     *    if ( !openPasswordDialog(info) )
      *     ....
      * }
      * \endcode
@@ -680,10 +689,10 @@ public:
     bool checkCachedAuthentication( AuthInfo& info );
 
     /**
-     * Explicitly store authentication information. openPassDlg already
+     * Explicitly store authentication information. openPasswordDialog already
      * stores password information automatically, you only need to call
      * this function if you want to store authentication information that
-     * is different from the information returned by openPassDlg.
+     * is different from the information returned by openPasswordDialog.
      */
     bool cacheAuthentication( const AuthInfo& info );
 
@@ -767,6 +776,7 @@ protected:
     virtual void virtual_hook( int id, void* data );
 
 private:
+    void send(int cmd, const QByteArray& arr = QByteArray());
     SlaveBasePrivate* const d;
 };
 
