@@ -40,7 +40,7 @@
 #include <ksocks.h>
 #include <kdebug.h>
 #include <ksslall.h>
-#include <ksslcertdlg.h>
+#include <ksslcertdialog.h>
 #include <kmessagebox.h>
 #ifndef Q_WS_WIN //temporary
 #include <kresolver.h>
@@ -60,9 +60,12 @@
 
 using namespace KIO;
 
+#ifndef KDE_USE_FINAL
 typedef QMap<QString, QString> StringStringMap;
 Q_DECLARE_METATYPE(StringStringMap)
+#endif
 
+/** @internal */
 class TCPSlaveBase::TcpSlaveBasePrivate
 {
 public:
@@ -157,10 +160,8 @@ ssize_t TCPSlaveBase::write(const char *data, ssize_t len)
             (void) doSSLHandShake( true );
         return d->kssl->write(data, len);
     }
-    return d->socket.write(data, len);
-#else
-    return 0;
 #endif
+    return d->socket.write(data, len);
 }
 
 ssize_t TCPSlaveBase::read(char* data, ssize_t len)
@@ -172,10 +173,8 @@ ssize_t TCPSlaveBase::read(char* data, ssize_t len)
             (void) doSSLHandShake( true );
         return d->kssl->read(data, len);
     }
-    return d->socket.read(data, len);
-#else
-    return 0;
 #endif
+    return d->socket.read(data, len);
 }
 
 
@@ -258,11 +257,6 @@ if ((m_bIsSSL || d->usingTLS) && !d->useSSLTunneling) {       // SSL CASE
     }
   }
 } else {                                                      // NON SSL CASE
-
-#ifndef Q_OS_UNIX
-  return -1;
-#endif
-
   while (true) {
     rc = d->socket.bytesAvailable();
 
@@ -325,8 +319,8 @@ bool TCPSlaveBase::connectToHost( const QString &host,
                                   const QString &service,
                                   bool sendError )
 {
-#ifdef Q_OS_UNIX
     d->userAborted = false;
+#ifdef Q_OS_UNIX
 
     //  - leaving SSL - warn before we even connect
     if (metaData("main_frame_request") == "TRUE" &&
@@ -363,7 +357,7 @@ bool TCPSlaveBase::connectToHost( const QString &host,
           }
        }
     }
-
+#endif
     d->status = -1;
     d->host = host;
     d->needSSLHandShake = m_bIsSSL;
@@ -403,9 +397,6 @@ bool TCPSlaveBase::connectToHost( const QString &host,
         setMetaData("ssl_in_use", "FALSE");
 
     return true;
-#else //!Q_OS_UNIX
-    return false;
-#endif //Q_OS_UNIX
 }
 
 void TCPSlaveBase::closeDescriptor()
@@ -626,8 +617,8 @@ KSSLCertificateHome::KSSLAuthAction aa;
 
     if (certs.isEmpty()) return;  // we had nothing else, and prompt failed
 
-    if (!QDBus::sessionBus().interface()->isServiceRegistered("org.kde.kio.uiserver")) {
-        KToolInvocation::startServiceByDesktopPath("kio_uiserver.desktop",
+    if (!QDBusConnection::sessionBus().interface()->isServiceRegistered("org.kde.kio.uiserver")) {
+        KToolInvocation::startServiceByDesktopPath("kuiserver.desktop",
                                                    QStringList() );
     }
 
@@ -635,10 +626,10 @@ KSSLCertificateHome::KSSLAuthAction aa;
 
     QDBusMessage retVal = uis.call("showSSLCertDialog", ourHost, certs, metaData("window-id").toLongLong());
     if (retVal.type() == QDBusMessage::ReplyMessage) {
-        if (retVal.at(0).toBool()) {
-           send = retVal.at(1).toBool();
-           save = retVal.at(2).toBool();
-           certname = retVal.at(3).toString();
+        if (retVal.arguments().at(0).toBool()) {
+           send = retVal.arguments().at(1).toBool();
+           save = retVal.arguments().at(2).toBool();
+           certname = retVal.arguments().at(3).toString();
         }
      }
   }
@@ -672,7 +663,7 @@ KSSLCertificateHome::KSSLAuthAction aa;
         else
            showprompt = true;
         if (showprompt) {
-           if (!openPassDlg(ai, first ? QString() :
+           if (!openPasswordDialog(ai, first ? QString() :
                    i18n("Unable to open the certificate. Try a new password?")))
               break;
         }
@@ -795,7 +786,7 @@ int TCPSlaveBase::verifyCertificate()
        Q3PtrList<KSSLCertificate> chain = pc.chain().getChain();
        for (KSSLCertificate *c = chain.first(); c; c = chain.next()) {
           theChain += c->toString();
-          theChain += "\n";
+          theChain += '\n';
        }
        setMetaData("ssl_peer_chain", theChain);
     } else setMetaData("ssl_peer_chain", "");
@@ -868,8 +859,8 @@ int TCPSlaveBase::verifyCertificate()
                 }
 
                 if (result == KMessageBox::Yes) {
-                  if (!QDBus::sessionBus().interface()->isServiceRegistered("org.kde.kio.uiserver"))
-                      KToolInvocation::startServiceByDesktopPath("kio_uiserver.desktop",
+                  if (!QDBusConnection::sessionBus().interface()->isServiceRegistered("org.kde.kio.uiserver"))
+                      KToolInvocation::startServiceByDesktopPath("kuiserver.desktop",
                                                                  QStringList() );
 
                   QDBusInterface uis("org.kde.kio.uiserver", "/UIServer", "org.kde.KIO.UIServer");
@@ -988,8 +979,8 @@ int TCPSlaveBase::verifyCertificate()
                                  i18n("&Details"),
                                  i18n("Co&nnect"));
                 if (result == KMessageBox::Yes) {
-                  if (!QDBus::sessionBus().interface()->isServiceRegistered("org.kde.kio.uiserver"))
-                      KToolInvocation::startServiceByDesktopPath("kio_uiserver.desktop",
+                  if (!QDBusConnection::sessionBus().interface()->isServiceRegistered("org.kde.kio.uiserver"))
+                      KToolInvocation::startServiceByDesktopPath("kuiserver.desktop",
                                                                  QStringList() );
 
                   QDBusInterface uis("org.kde.kio.uiserver", "/UIServer", "org.kde.KIO.UIServer");
@@ -1070,8 +1061,8 @@ int TCPSlaveBase::verifyCertificate()
 
       if ( result == KMessageBox::Yes )
       {
-          if (!QDBus::sessionBus().interface()->isServiceRegistered("org.kde.kio.uiserver"))
-              KToolInvocation::startServiceByDesktopPath("kio_uiserver.desktop",
+          if (!QDBusConnection::sessionBus().interface()->isServiceRegistered("org.kde.kio.uiserver"))
+              KToolInvocation::startServiceByDesktopPath("kuiserver.desktop",
                                                          QStringList() );
 
           QDBusInterface uis("org.kde.kio.uiserver", "/UIServer", "org.kde.KIO.UIServer");
@@ -1117,9 +1108,8 @@ bool TCPSlaveBase::isConnectionValid()
       return false;
 
     qint64 retval = -1;
-#ifdef Q_OS_UNIX
     retval = d->socket.bytesAvailable();
-#endif
+
     // retval == -1 ==> Error
     // retval ==  0 ==> Connection Idle
     // retval ==  1 ==> either Active or Closed
