@@ -32,7 +32,7 @@
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kdebug.h>
-#include <kio/passdlg.h>
+#include <kpassworddialog.h>
 #include <kwallet.h>
 
 #include "config.h"
@@ -176,7 +176,7 @@ static bool readFromWallet( KWallet::Wallet* wallet, const QString& key, const Q
 }
 
 QByteArray
-KPasswdServer::checkAuthInfo(const QByteArray &data, qlonglong windowId, const QDBusMessage &msg)
+KPasswdServer::checkAuthInfo(const QByteArray &data, qlonglong windowId, qlonglong usertime, const QDBusMessage &msg)
 {
     KIO::AuthInfo info;
     {
@@ -186,6 +186,8 @@ KPasswdServer::checkAuthInfo(const QByteArray &data, qlonglong windowId, const Q
     }
     kDebug(130) << "KPasswdServer::checkAuthInfo: User= " << info.username
               << ", WindowId = " << windowId << endl;
+    if( usertime != 0 )
+        kapp->updateUserTimestamp( usertime );
 
     QString key = createCacheKey(info);
 
@@ -250,7 +252,7 @@ KPasswdServer::checkAuthInfo(const QByteArray &data, qlonglong windowId, const Q
 
 QByteArray
 KPasswdServer::queryAuthInfo(const QByteArray &data, const QString &errorMsg, qlonglong windowId,
-                             qlonglong seqNr, const QDBusMessage &msg)
+                             qlonglong seqNr, qlonglong usertime, const QDBusMessage &msg)
 {
     KIO::AuthInfo info;
     {
@@ -262,6 +264,8 @@ KPasswdServer::queryAuthInfo(const QByteArray &data, const QString &errorMsg, ql
               << ", Message= " << info.prompt << ", WindowId = " << windowId << endl;
     if ( !info.password.isEmpty() ) // should we really allow the caller to pre-fill the password?
         kDebug(130) <<  "password was set by caller" << endl;
+    if( usertime != 0 )
+        kapp->updateUserTimestamp( usertime );
 
     QString key = createCacheKey(info);
     Request *request = new Request;
@@ -356,7 +360,7 @@ KPasswdServer::processRequest()
            QString prompt = request->errorMsg;
            prompt += i18n("  Do you want to retry?");
            int dlgResult = KMessageBox::warningContinueCancel(0, prompt,
-                           i18n("Authentication"), i18n("Retry"));
+                           i18n("Authentication"), KGuiItem(i18n("Retry")));
            if (dlgResult != KMessageBox::Continue)
               askPw = false;
         }
@@ -377,7 +381,9 @@ KPasswdServer::processRequest()
                     hasWalletData = readFromWallet( m_wallet, request->key, info.realmValue, username, password, info.readOnly, knownLogins );
             }
 
-            KIO::PasswordDialog dlg( info.prompt, username, info.keepPassword );
+            KPasswordDialog dlg( 0l,  info.keepPassword ? ( KPasswordDialog::ShowUsernameLine |  KPasswordDialog::ShowKeepPassword) : KPasswordDialog::ShowUsernameLine ) ;
+            dlg.setPrompt(info.prompt);
+            dlg.setUsername(username);
             if (info.caption.isEmpty())
                dlg.setPlainCaption( i18n("Authorization Dialog") );
             else
@@ -390,7 +396,7 @@ KPasswdServer::processRequest()
                dlg.setPassword( password );
 
             if (info.readOnly)
-               dlg.setUserReadOnly( true );
+               dlg.setUsernameReadOnly( true );
             else
                dlg.setKnownLogins( knownLogins );
 
@@ -438,7 +444,7 @@ KPasswdServer::processRequest()
 
     QDataStream stream2(&replyData, QIODevice::WriteOnly);
     stream2 << info;
-    request->transaction.sendReply(QVariantList() << replyData << m_seqNr);
+    QDBusConnection::sessionBus().send(request->transaction.createReply(QVariantList() << replyData << m_seqNr));
 
     m_authPending.remove((unsigned int) 0);
 
@@ -489,7 +495,7 @@ KPasswdServer::processRequest()
                stream2 << info;
            }
 
-           waitRequest->transaction.sendReply();
+           QDBusConnection::sessionBus().send(waitRequest->transaction.createReply());
 
            m_authWait.remove();
            waitRequest = m_authWait.current();
