@@ -58,7 +58,7 @@ static const char* (*K_RAND_file_name) (char *, size_t) = 0L;
 static int (*K_RAND_load_file)  (const char *, long) = 0L;
 static int (*K_RAND_write_file) (const char *) = 0L;
 static SSL_METHOD * (*K_TLSv1_client_method) () = 0L;
-static SSL_METHOD * (*K_SSLv3_client_method) () = 0L;
+static SSL_METHOD * (*K_SSLv23_client_method) () = 0L;
 static X509 * (*K_SSL_get_peer_certificate) (SSL *) = 0L;
 static int (*K_SSL_CIPHER_get_bits) (SSL_CIPHER *,int *) = 0L;
 static char * (*K_SSL_CIPHER_get_version) (SSL_CIPHER *) = 0L;
@@ -281,7 +281,7 @@ KConfig *cfg;
    {
    QString libname = findMostRecentLib("/usr/lib" KDELIBSUFF, "crypto");
    if (!libname.isNull())
-         _cryptoLib = ll->globalLibrary(libname.latin1());
+         _cryptoLib = ll->globalLibrary(libname.toLatin1());
    }
 #elif defined(__CYGWIN__)
    libpaths << "/usr/bin/"
@@ -499,7 +499,7 @@ KConfig *cfg;
    {
    QString libname = findMostRecentLib("/usr/lib", "ssl");
    if (!libname.isNull())
-         _sslLib = ll->globalLibrary(libname.latin1());
+         _sslLib = ll->globalLibrary(libname.toLatin1());
    }
 #else
    for (QStringList::Iterator it = libpaths.begin();
@@ -547,7 +547,7 @@ KConfig *cfg;
       K_SSL_ctrl = (long (*)(SSL * ,int, long, char *))
                                   _sslLib->symbol("SSL_ctrl");
       K_TLSv1_client_method = (SSL_METHOD *(*)()) _sslLib->symbol("TLSv1_client_method");
-      K_SSLv3_client_method = (SSL_METHOD *(*)()) _sslLib->symbol("SSLv3_client_method");
+      K_SSLv23_client_method = (SSL_METHOD *(*)()) _sslLib->symbol("SSLv23_client_method");
       K_SSL_get_peer_certificate = (X509 *(*)(SSL *)) _sslLib->symbol("SSL_get_peer_certificate");
       K_SSL_CIPHER_get_bits = (int (*)(SSL_CIPHER *,int *)) _sslLib->symbol("SSL_CIPHER_get_bits");
       K_SSL_CIPHER_get_version = (char * (*)(SSL_CIPHER *)) _sslLib->symbol("SSL_CIPHER_get_version");
@@ -749,8 +749,8 @@ SSL_METHOD *KOpenSSLProxy::TLSv1_client_method() {
 }
 
 
-SSL_METHOD *KOpenSSLProxy::SSLv3_client_method() {
-   if (K_SSLv3_client_method) return (K_SSLv3_client_method)();
+SSL_METHOD *KOpenSSLProxy::SSLv23_client_method() {
+   if (K_SSLv23_client_method) return (K_SSLv23_client_method)();
    return 0L;
 }
 
