@@ -25,6 +25,7 @@
 #endif
 
 #include <qtimer.h>
+#include <Q3CString>
 
 #include "kssld.h"
 #include <kconfig.h>
@@ -41,7 +42,6 @@
 #include <pwd.h>
 #include <unistd.h>
 #include <qfile.h>
-#include <q3sortedlist.h>
 #include <kglobal.h>
 #include <kstandarddirs.h>
 #include <kdebug.h>
@@ -684,7 +684,8 @@ void KSSLD::caVerifyUpdate() {
 		return;
 
 	cfg->setGroup(QString());
-	quint32 newStamp = KGlobal::dirs()->calcResourceHash("config", "ksslcalist", true);
+	quint32 newStamp = KGlobal::dirs()->calcResourceHash("config", "ksslcalist",
+                                                             KStandardDirs::Recursive);
 	quint32 oldStamp = cfg->readEntry("ksslcalistStamp", 0);
 	if (oldStamp != newStamp)
 	{
