@@ -35,6 +35,7 @@
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kpassworddialog.h>
+#include <knewpassworddialog.h>
 #include <kstandarddirs.h>
 #include <kwalletentry.h>
 #include <kwin.h>
@@ -48,11 +49,6 @@
 
 #include <assert.h>
 
-#ifdef Q_WS_X11
-#include <QX11Info>
-#include <X11/Xlib.h>
-#endif
-
 #include "kwalletdadaptor.h"
 
 extern "C" {
@@ -76,6 +72,7 @@ class KWalletTransaction {
 		QString appid;
 		qlonglong wId;
 		QString wallet;
+		bool modal;
 };
 
 KWalletD::KWalletD()
@@ -83,19 +80,20 @@ KWalletD::KWalletD()
 	srand(time(0));
 	_showingFailureNotify = false;
 	_transactions.setAutoDelete(true);
-	_timeouts = new KTimeout(17);
+	_timeouts = new KTimeout();
 	_closeIdle = false;
 	_idleTime = 0;
 	connect(_timeouts, SIGNAL(timedOut(int)), this, SLOT(timedOut(int)));
 
 	(void)new KWalletDAdaptor(this);
 	// register another name
-	QDBus::sessionBus().registerService("org.kde.kwalletd");
-	kdesktop = new QDBusInterface("org.kde.kdesktop", "/KScreensaver", "org.kde.KScreensaverIface");
-
+	QDBusConnection::sessionBus().registerService("org.kde.kwalletd");
+#ifdef Q_WS_X11
+	kdesktop = new QDBusInterface("org.kde.kdesktop", "/Screensaver", "org.kde.kdesktop.Screensaver");
+#endif
 	reconfigure();
 	KGlobal::dirs()->addResourceType("kwallet", "share/apps/kwallet");
-		connect(QDBus::sessionBus().interface(), SIGNAL(serviceUnregistered(QString)),
+		connect(QDBusConnection::sessionBus().interface(), SIGNAL(serviceUnregistered(QString)),
 				SLOT(slotServiceUnregistered(QString)));
 	_dw = new KDirWatch(this );
 		_dw->setObjectName( "KWallet Directory Watcher" );
@@ -108,10 +106,10 @@ KWalletD::KWalletD()
 KWalletD::~KWalletD() {
 	delete _timeouts;
 	_timeouts = 0;
-
+#ifdef Q_WS_X11
 	delete kdesktop;
 	kdesktop = 0;
-
+#endif
 	closeAllWallets();
 	_transactions.clear();
 }
@@ -147,7 +145,7 @@ void KWalletD::processTransactions() {
 
 		switch (xact->tType) {
 			case KWalletTransaction::Open:
-				res = doTransactionOpen(xact->appid, xact->wallet, xact->wId, xact->msg);
+				res = doTransactionOpen(xact->appid, xact->wallet, xact->wId, xact->modal);
 
 				// multiple requests from the same client
 				// should not produce multiple password
@@ -173,7 +171,7 @@ void KWalletD::processTransactions() {
 				res = -1;
 				break;
 			case KWalletTransaction::ChangePassword:
-				doTransactionChangePassword(xact->appid, xact->wallet, xact->wId, xact->msg);
+				doTransactionChangePassword(xact->appid, xact->wallet, xact->wId);
 				// fall through - no return
 			default:
 				_transactions.removeRef(xact);
@@ -181,7 +179,7 @@ void KWalletD::processTransactions() {
 		}
 
 		if (xact->tType != KWalletTransaction::ChangePassword) {
-			xact->msg.sendReply(res);
+                    QDBusConnection::sessionBus().send(xact->msg.createReply(res));
 		}
 		_transactions.removeRef(xact);
 	}
@@ -190,28 +188,24 @@ void KWalletD::processTransactions() {
 }
 
 #if 0
-void KWalletD::openAsynchronous(const QString& wallet, const QByteArray& returnObject, uint wId) {
+void KWalletD::openAsynchronous(const QString& wallet, const QByteArray& returnObject, uint wId, const QString& appid) {
 	DCOPClient *dc = callingDcopClient();
 	if (!dc) {
 		return;
 	}
 
-	QByteArray appid = dc->senderId();
 	if (!_enabled ||
 		!QRegExp("^[A-Za-z0-9]+[A-Za-z0-9\\s\\-_]*$").exactMatch(wallet)) {
 		DCOPRef(appid, returnObject).send("walletOpenResult", -1);
 		return;
 	}
 
-	QByteArray peerName = friendlyDCOPPeerName();
-
 	KWalletTransaction *xact = new KWalletTransaction;
 
-	xact->appid = peerName;
-	xact->rawappid = appid;
-	xact->client = callingDcopClient();
+	xact->appid = appid;
 	xact->wallet = wallet;
 	xact->wId = wId;
+	xact->modal = false;
 	xact->tType = KWalletTransaction::Open;
 	xact->returnObject = returnObject;
 	_transactions.append(xact);
@@ -222,18 +216,18 @@ void KWalletD::openAsynchronous(const QString& wallet, const QByteArray& returnO
 }
 #endif
 
-int KWalletD::openPath(const QString& path, qlonglong wId, const QDBusMessage& msg) {
+int KWalletD::openPath(const QString& path, qlonglong wId, const QString& appid) {
 	if (!_enabled) { // guard
 		return -1;
 	}
 
 	// FIXME: setup transaction
-	int rc = internalOpen(msg.service(), path, true, wId, msg);
+	int rc = internalOpen(appid, path, true, wId, false);
 	return rc;
 }
 
 
-int KWalletD::open(const QString& wallet, qlonglong wId, const QDBusMessage &msg) {
+int KWalletD::open(const QString& wallet, qlonglong wId, const QString& appid, const QDBusMessage &msg) {
 	if (!_enabled) { // guard
 		return -1;
 	}
@@ -242,7 +236,6 @@ int KWalletD::open(const QString& wallet, qlonglong wId, const QDBusMessage &msg
 		return -1;
 	}
 
-	QString appid = msg.service();
 	KWalletTransaction *xact = new KWalletTransaction;
 	_transactions.append(xact);
 
@@ -251,19 +244,54 @@ int KWalletD::open(const QString& wallet, qlonglong wId, const QDBusMessage &msg
 	xact->appid = appid;
 	xact->wallet = wallet;
 	xact->wId = wId;
+	xact->modal = true; // mark dialogs as modal, the app has blocking wait
 	xact->tType = KWalletTransaction::Open;
 	QTimer::singleShot(0, this, SLOT(processTransactions()));
+	checkActiveDialog();
 	return 0; // process later
 }
 
+// Sets up a dialog that will be shown by kwallet.
+void KWalletD::setupDialog( QWidget* dialog, WId wId, const QString& appid, bool modal ) {
+	if( wId != 0 )
+		KWin::setMainWindow( dialog, wId ); // correct, set dialog parent
+	else {
+		if( appid.isEmpty())
+			kWarning() << "Using kwallet without parent window!" << endl;
+		else
+			kWarning() << "Application '" << appid << "' using kwallet without parent window!" << endl;
+		// allow dialog activation even if it interrupts, better than trying hacks
+		// with keeping the dialog on top or on all desktops
+		kapp->updateUserTimestamp();
+	}
+	if( modal )
+		KWin::setState( dialog->winId(), NET::Modal );
+	else
+		KWin::clearState( dialog->winId(), NET::Modal );
+	activeDialog = dialog;
+}
 
-int KWalletD::doTransactionOpen(const QString& appid, const QString& wallet, qlonglong wId, const QDBusMessage& msg) {
+// If there's a dialog already open and another application tries some operation that'd lead to
+// opening a dialog, that application will be blocked by this dialog. A proper solution would
+// be to set the second application's window also as a parent for the active dialog, so that
+// KWin properly handles focus changes and so on, but there's currently no support for multiple
+// dialog parents. Hopefully to be done in KDE4, for now just use all kinds of bad hacks to make
+//  sure the user doesn't overlook the active dialog.
+void KWalletD::checkActiveDialog() {
+	if( !activeDialog || activeDialog->isHidden())
+		return;
+	kapp->updateUserTimestamp();
+	KWin::setState( activeDialog->winId(), NET::KeepAbove );
+	KWin::setOnAllDesktops( activeDialog->winId(), true );
+	KWin::forceActiveWindow( activeDialog->winId());
+}
+
+
+int KWalletD::doTransactionOpen(const QString& appid, const QString& wallet, qlonglong wId, bool modal) {
 	if (_firstUse && !wallets().contains(KWallet::Wallet::LocalWallet())) {
 		// First use wizard
 		KWalletWizard *wiz = new KWalletWizard(0);
-#ifdef Q_WS_X11
-		XSetTransientForHint(QX11Info::display(), wiz->winId(), wId);
-#endif
+		setupDialog( wiz, wId, appid, modal );
 		int rc = wiz->exec();
 		if (rc == QDialog::Accepted) {
 			KConfig cfg("kwalletrc");
@@ -302,12 +330,12 @@ int KWalletD::doTransactionOpen(const QString& appid, const QString& wallet, qlo
 		cfg.sync();
 	}
 
-	int rc = internalOpen(appid, wallet, false, WId(wId), msg);
+	int rc = internalOpen(appid, wallet, false, WId(wId), modal);
 	return rc;
 }
 
 
-int KWalletD::internalOpen(const QString& appid, const QString& wallet, bool isPath, WId w, const QDBusMessage& msg) {
+int KWalletD::internalOpen(const QString& appid, const QString& wallet, bool isPath, WId w, bool modal) {
 	int rc = -1;
 	bool brandNew = false;
 
@@ -336,7 +364,7 @@ int KWalletD::internalOpen(const QString& appid, const QString& wallet, bool isP
 		}
 
 		KWallet::Backend *b = new KWallet::Backend(wallet, isPath);
-		KPasswordDialog *kpd = 0L;
+		QString password;
 		bool emptyPass = false;
 		if ((isPath && QFile::exists(wallet)) || (!isPath && KWallet::Backend::exists(wallet))) {
 			int pwless = b->open(QByteArray());
@@ -346,73 +374,76 @@ int KWalletD::internalOpen(const QString& appid, const QString& wallet, bool isP
 					delete b;
 					b = new KWallet::Backend(wallet, isPath);
 				}
-				kpd = new KPasswordDialog(KPasswordDialog::Password, false, 0);
+				KPasswordDialog *kpd = new KPasswordDialog();
 				if (appid.isEmpty()) {
 					kpd->setPrompt(i18n("<qt>KDE has requested to open the wallet '<b>%1</b>'. Please enter the password for this wallet below.", Qt::escape(wallet)));
 				} else {
 					kpd->setPrompt(i18n("<qt>The application '<b>%1</b>' has requested to open the wallet '<b>%2</b>'. Please enter the password for this wallet below.", Qt::escape(appid), Qt::escape(wallet)));
 				}
 				brandNew = false;
-				kpd->setButtonGuiItem(KDialog::Ok,KStdGuiItem::open());
+				kpd->setButtonGuiItem(KDialog::Ok,KStandardGuiItem::open());
+				kpd->setCaption(i18n("KDE Wallet Service"));
+				while (!b->isOpen()) {
+					setupDialog( kpd, w, appid, modal );
+					if (kpd->exec() == KDialog::Accepted) {
+						password = kpd->password();
+						int rc = b->open(password.toUtf8());
+						if (!b->isOpen()) {
+							kpd->setPrompt(i18n("<qt>Error opening the wallet '<b>%1</b>'. Please try again.<br>(Error code %2: %3)", Qt::escape(wallet), rc, KWallet::Backend::openRCToString(rc)));
+						}
+					} else {
+						break;
+					}
+				}
+				delete kpd;
 			} else {
 				emptyPass = true;
 			}
-		} else if (wallet == KWallet::Wallet::LocalWallet() ||
-				wallet == KWallet::Wallet::NetworkWallet()) {
-			// Auto create these wallets.
-			kpd = new KPasswordDialog(KPasswordDialog::NewPassword, false, 0);
-			if (appid.isEmpty()) {
-				kpd->setPrompt(i18n("KDE has requested to open the wallet. This is used to store sensitive data in a secure fashion. Please enter a password to use with this wallet or click cancel to deny the application's request."));
-			} else {
-				kpd->setPrompt(i18n("<qt>The application '<b>%1</b>' has requested to open the KDE wallet. This is used to store sensitive data in a secure fashion. Please enter a password to use with this wallet or click cancel to deny the application's request.", Qt::escape(appid)));
-			}
-			brandNew = true;
-			kpd->setButtonGuiItem(KDialog::Ok,KStdGuiItem::open());
 		} else {
-			kpd = new KPasswordDialog(KPasswordDialog::NewPassword, false, 0);
-			if (appid.length() == 0) {
-				kpd->setPrompt(i18n("<qt>KDE has requested to create a new wallet named '<b>%1</b>'. Please choose a password for this wallet, or cancel to deny the application's request.", Qt::escape(wallet)));
-			} else {
-				kpd->setPrompt(i18n("<qt>The application '<b>%1</b>' has requested to create a new wallet named '<b>%2</b>'. Please choose a password for this wallet, or cancel to deny the application's request.", Qt::escape(appid), Qt::escape(wallet)));
-			}
-			brandNew = true;
-			kpd->setButtonGuiItem(KDialog::Ok,KGuiItem(i18n("C&reate"),"filenew"));
-		}
-
-		if (kpd) {
-			kpd->setCaption(i18n("KDE Wallet Service"));
-			kpd->setAllowEmptyPasswords(true);
-		}
-
-		const char *p = 0L;
-		while (!b->isOpen()) {
-			assert(kpd); // kpd can't be null if isOpen() is false
-#ifdef Q_WS_X11
-			XSetTransientForHint(QX11Info::display(), kpd->winId(), w);
-#endif
-			KWin::setState( kpd->winId(), NET::KeepAbove );
-			KWin::setOnAllDesktops(kpd->winId(), true);
-			if (kpd->exec() == KDialog::Accepted) {
-				p = kpd->password();
-				int rc = b->open(QByteArray(p, strlen(p)));
-				if (!b->isOpen()) {
-					kpd->setPrompt(i18n("<qt>Error opening the wallet '<b>%1</b>'. Please try again.<br>(Error code %2: %3)", Qt::escape(wallet), rc, KWallet::Backend::openRCToString(rc)));
-					kpd->clearPassword();
+			KNewPasswordDialog *kpd = new KNewPasswordDialog();
+			if (wallet == KWallet::Wallet::LocalWallet() ||
+						 wallet == KWallet::Wallet::NetworkWallet()) 
+			{
+				// Auto create these wallets.
+				if (appid.isEmpty()) {
+					kpd->setPrompt(i18n("KDE has requested to open the wallet. This is used to store sensitive data in a secure fashion. Please enter a password to use with this wallet or click cancel to deny the application's request."));
+				} else {
+					kpd->setPrompt(i18n("<qt>The application '<b>%1</b>' has requested to open the KDE wallet. This is used to store sensitive data in a secure fashion. Please enter a password to use with this wallet or click cancel to deny the application's request.", Qt::escape(appid)));
 				}
 			} else {
-				break;
+				if (appid.length() == 0) {
+					kpd->setPrompt(i18n("<qt>KDE has requested to create a new wallet named '<b>%1</b>'. Please choose a password for this wallet, or cancel to deny the application's request.", Qt::escape(wallet)));
+				} else {
+					kpd->setPrompt(i18n("<qt>The application '<b>%1</b>' has requested to create a new wallet named '<b>%2</b>'. Please choose a password for this wallet, or cancel to deny the application's request.", Qt::escape(appid), Qt::escape(wallet)));
+				}
 			}
+			brandNew = true;
+			kpd->setCaption(i18n("KDE Wallet Service"));			
+			kpd->setButtonGuiItem(KDialog::Ok,KGuiItem(i18n("C&reate"),"filenew"));
+			while (!b->isOpen()) {
+				setupDialog( kpd, w, appid, modal );
+				if (kpd->exec() == KDialog::Accepted) {
+					password = kpd->password();
+					int rc = b->open(password.toUtf8());
+					if (!b->isOpen()) {
+						kpd->setPrompt(i18n("<qt>Error opening the wallet '<b>%1</b>'. Please try again.<br>(Error code %2: %3)", Qt::escape(wallet), rc, KWallet::Backend::openRCToString(rc)));
+					}
+				} else {
+					break;
+				}
+			}
+			delete kpd;
 		}
 
-		if (!emptyPass && (!p || !b->isOpen())) {
+
+
+		if (!emptyPass && (password.isNull() || !b->isOpen())) {
 			delete b;
-			delete kpd;
 			return -1;
 		}
 
 		if (emptyPass && _openPrompt && !isAuthorizedApp(appid, wallet, w)) {
 			delete b;
-			delete kpd;
 			return -1;
 		}
 
@@ -420,15 +451,13 @@ int KWalletD::internalOpen(const QString& appid, const QString& wallet, bool isP
 		if (emptyPass) {
 			_passwords[wallet] = "";
 		} else {
-			_passwords[wallet] = p;
+			_passwords[wallet] = password.toUtf8();
 		}
 		_handles[appid].append(rc);
 
-		delete kpd; // don't refactor this!!  Argh I hate KPassDlg
-
 		if (brandNew) {
-			createFolder(rc, KWallet::Wallet::PasswordFolder(), msg);
-			createFolder(rc, KWallet::Wallet::FormDataFolder(), msg);
+			createFolder(rc, KWallet::Wallet::PasswordFolder(), appid);
+			createFolder(rc, KWallet::Wallet::FormDataFolder(), appid);
 		}
 
 		b->ref();
@@ -464,20 +493,19 @@ bool KWalletD::isAuthorizedApp(const QString& appid, const QString& wallet, WId
 	}
 
 	if (!implicitAllow(wallet, thisApp)) {
-		KBetterThanKDialog *dialog = new KBetterThanKDialog;
-		if (appid.isEmpty()) {
+		KConfig cfg("kwalletrc");
+		cfg.setGroup("Auto Allow");
+		if (!cfg.entryIsImmutable(wallet)) {
+		    KBetterThanKDialog *dialog = new KBetterThanKDialog;
+		    if (appid.isEmpty()) {
 			dialog->setLabel(i18n("<qt>KDE has requested access to the open wallet '<b>%1</b>'.", Qt::escape(wallet)));
-		} else {
+		    } else {
 			dialog->setLabel(i18n("<qt>The application '<b>%1</b>' has requested access to the open wallet '<b>%2</b>'.", Qt::escape(QString(appid)), Qt::escape(wallet)));
+		    }
+		    setupDialog( dialog, w, appid, false );
+		    response = dialog->exec();
+		    delete dialog;
 		}
-#ifdef Q_WS_X11
-		XSetTransientForHint(QX11Info::display(), dialog->winId(), w);
-#endif
-		KWin::setState(dialog->winId(), NET::KeepAbove);
-		KWin::setOnAllDesktops(dialog->winId(), true);
-
-		response = dialog->exec();
-		delete dialog;
 	}
 
 	if (response == 0 || response == 1) {
@@ -486,6 +514,9 @@ bool KWalletD::isAuthorizedApp(const QString& appid, const QString& wallet, WId
 			cfg.setGroup("Auto Allow");
 			QStringList apps = cfg.readEntry(wallet, QStringList());
 			if (!apps.contains(thisApp)) {
+				if (cfg.entryIsImmutable(wallet)) {
+					return false;
+				}
 				apps += thisApp;
 				_implicitAllowMap[wallet] += thisApp;
 				cfg.writeEntry(wallet, apps);
@@ -524,9 +555,7 @@ int KWalletD::deleteWallet(const QString& wallet) {
 }
 
 
-void KWalletD::changePassword(const QString& wallet, qlonglong wId, const QDBusMessage& msg) {
-	QString appid = msg.service();
-
+void KWalletD::changePassword(const QString& wallet, qlonglong wId, const QString& appid, const QDBusMessage& msg) {
 	KWalletTransaction *xact = new KWalletTransaction;
 
 	//msg.setDelayedReply(true);
@@ -534,15 +563,18 @@ void KWalletD::changePassword(const QString& wallet, qlonglong wId, const QDBusM
 	xact->appid = appid;
 	xact->wallet = wallet;
 	xact->wId = wId;
+	xact->modal = false;
 	xact->tType = KWalletTransaction::ChangePassword;
 
 	_transactions.append(xact);
 
 	QTimer::singleShot(0, this, SLOT(processTransactions()));
+	checkActiveDialog();
+	checkActiveDialog();
 }
 
 
-void KWalletD::doTransactionChangePassword(const QString& appid, const QString& wallet, qlonglong wId, const QDBusMessage& msg) {
+void KWalletD::doTransactionChangePassword(const QString& appid, const QString& wallet, qlonglong wId) {
 	Q3IntDictIterator<KWallet::Backend> it(_wallets);
 	KWallet::Backend *w = 0L;
 	int handle = -1;
@@ -555,7 +587,7 @@ void KWalletD::doTransactionChangePassword(const QString& appid, const QString&
 	}
 
 	if (!it.current()) {
-		handle = doTransactionOpen(appid, wallet, wId, msg);
+		handle = doTransactionOpen(appid, wallet, wId, false);
 		if (-1 == handle) {
 			KMessageBox::sorryWId(wId, i18n("Unable to open wallet. The wallet must be opened in order to change the password."), i18n("KDE Wallet Service"));
 			return;
@@ -570,25 +602,22 @@ void KWalletD::doTransactionChangePassword(const QString& appid, const QString&
 
 	assert(w);
 
-	KPasswordDialog *kpd;
-	kpd = new KPasswordDialog(KPasswordDialog::NewPassword, false, 0);
+	KNewPasswordDialog *kpd;
+	kpd = new KNewPasswordDialog();
 	kpd->setPrompt(i18n("<qt>Please choose a new password for the wallet '<b>%1</b>'.", Qt::escape(wallet)));
 	kpd->setCaption(i18n("KDE Wallet Service"));
 	kpd->setAllowEmptyPasswords(true);
-#ifdef Q_WS_X11
-	XSetTransientForHint(QX11Info::display(), kpd->winId(), wId);
-#endif
+	setupDialog( kpd, wId, appid, false );
 	if (kpd->exec() == KDialog::Accepted) {
-		const char *p = kpd->password();
-		if (p) {
-			_passwords[wallet] = p;
-			QByteArray pa(p, strlen(p));
-			int rc = w->close(pa);
+		QString p = kpd->password();
+		if (!p.isNull()) {
+			_passwords[wallet] = p.toUtf8();
+			int rc = w->close(p.toUtf8());
 			if (rc < 0) {
 				KMessageBox::sorryWId(wId, i18n("Error re-encrypting the wallet. Password was not changed."), i18n("KDE Wallet Service"));
 				reclose = true;
 			} else {
-				rc = w->open(pa);
+				rc = w->open(p.toUtf8());
 				if (rc < 0) {
 					KMessageBox::sorryWId(wId, i18n("Error reopening the wallet. Data may be lost."), i18n("KDE Wallet Service"));
 					reclose = true;
@@ -600,7 +629,7 @@ void KWalletD::doTransactionChangePassword(const QString& appid, const QString&
 	delete kpd;
 
 	if (reclose) {
-		close(handle, true, msg);
+		close(handle, true, appid);
 	}
 }
 
@@ -649,8 +678,7 @@ int KWalletD::closeWallet(KWallet::Backend *w, int handle, bool force) {
 }
 
 
-int KWalletD::close(int handle, bool force, const QDBusMessage& msg) {
-	QString appid = msg.service();
+int KWalletD::close(int handle, bool force, const QString& appid) {
 	KWallet::Backend *w = _wallets.find(handle);
 	bool contains = false;
 
@@ -728,7 +756,7 @@ QStringList KWalletD::wallets() const {
 
 	dir.setFilter(QDir::Files | QDir::NoSymLinks);
 
-	foreach (QFileInfo fi, dir.entryInfoList()) {
+	foreach (const QFileInfo &fi, dir.entryInfoList()) {
 		QString fn = fi.fileName();
 		if (fn.endsWith(".kwl")) {
 			fn.truncate(fn.length()-4);
@@ -739,10 +767,10 @@ QStringList KWalletD::wallets() const {
 }
 
 
-void KWalletD::sync(int handle, const QDBusMessage &msg) {
+void KWalletD::sync(int handle, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		QByteArray p;
 		QString wallet = b->walletName();
 		p = QByteArray(_passwords[wallet].data(), _passwords[wallet].length());
@@ -752,10 +780,10 @@ void KWalletD::sync(int handle, const QDBusMessage &msg) {
 }
 
 
-QStringList KWalletD::folderList(int handle, const QDBusMessage& msg) {
+QStringList KWalletD::folderList(int handle, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		return b->folderList();
 	}
 
@@ -763,10 +791,10 @@ QStringList KWalletD::folderList(int handle, const QDBusMessage& msg) {
 }
 
 
-bool KWalletD::hasFolder(int handle, const QString& f, const QDBusMessage& msg) {
+bool KWalletD::hasFolder(int handle, const QString& f, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		return b->hasFolder(f);
 	}
 
@@ -774,10 +802,10 @@ bool KWalletD::hasFolder(int handle, const QString& f, const QDBusMessage& msg)
 }
 
 
-bool KWalletD::removeFolder(int handle, const QString& f, const QDBusMessage& msg) {
+bool KWalletD::removeFolder(int handle, const QString& f, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		bool rc = b->removeFolder(f);
 		emit folderListUpdated(b->walletName());
 		return rc;
@@ -787,10 +815,10 @@ bool KWalletD::removeFolder(int handle, const QString& f, const QDBusMessage& ms
 }
 
 
-bool KWalletD::createFolder(int handle, const QString& f, const QDBusMessage& msg) {
+bool KWalletD::createFolder(int handle, const QString& f, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		bool rc = b->createFolder(f);
 		emit folderListUpdated(b->walletName());
 		return rc;
@@ -800,10 +828,10 @@ bool KWalletD::createFolder(int handle, const QString& f, const QDBusMessage& ms
 }
 
 
-QByteArray KWalletD::readMap(int handle, const QString& folder, const QString& key, const QDBusMessage& msg) {
+QByteArray KWalletD::readMap(int handle, const QString& folder, const QString& key, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		b->setFolder(folder);
 		KWallet::Entry *e = b->readEntry(key);
 		if (e && e->type() == KWallet::Wallet::Map) {
@@ -815,10 +843,10 @@ QByteArray KWalletD::readMap(int handle, const QString& folder, const QString& k
 }
 
 
-QVariantMap KWalletD::readMapList(int handle, const QString& folder, const QString& key, const QDBusMessage& msg) {
+QVariantMap KWalletD::readMapList(int handle, const QString& folder, const QString& key, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		b->setFolder(folder);
 		QVariantMap rc;
 		foreach (KWallet::Entry *entry, b->readEntryList(key)) {
@@ -833,10 +861,10 @@ QVariantMap KWalletD::readMapList(int handle, const QString& folder, const QStri
 }
 
 
-QByteArray KWalletD::readEntry(int handle, const QString& folder, const QString& key, const QDBusMessage& msg) {
+QByteArray KWalletD::readEntry(int handle, const QString& folder, const QString& key, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		b->setFolder(folder);
 		KWallet::Entry *e = b->readEntry(key);
 		if (e) {
@@ -848,10 +876,10 @@ QByteArray KWalletD::readEntry(int handle, const QString& folder, const QString&
 }
 
 
-QVariantMap KWalletD::readEntryList(int handle, const QString& folder, const QString& key, const QDBusMessage& msg) {
+QVariantMap KWalletD::readEntryList(int handle, const QString& folder, const QString& key, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		b->setFolder(folder);
 		QVariantMap rc;
 		foreach (KWallet::Entry *entry, b->readEntryList(key)) {
@@ -864,10 +892,10 @@ QVariantMap KWalletD::readEntryList(int handle, const QString& folder, const QSt
 }
 
 
-QStringList KWalletD::entryList(int handle, const QString& folder, const QDBusMessage& msg) {
+QStringList KWalletD::entryList(int handle, const QString& folder, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		b->setFolder(folder);
 		return b->entryList();
 	}
@@ -876,10 +904,10 @@ QStringList KWalletD::entryList(int handle, const QString& folder, const QDBusMe
 }
 
 
-QString KWalletD::readPassword(int handle, const QString& folder, const QString& key, const QDBusMessage& msg) {
+QString KWalletD::readPassword(int handle, const QString& folder, const QString& key, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		b->setFolder(folder);
 		KWallet::Entry *e = b->readEntry(key);
 		if (e && e->type() == KWallet::Wallet::Password) {
@@ -891,10 +919,10 @@ QString KWalletD::readPassword(int handle, const QString& folder, const QString&
 }
 
 
-QVariantMap KWalletD::readPasswordList(int handle, const QString& folder, const QString& key, const QDBusMessage& msg) {
+QVariantMap KWalletD::readPasswordList(int handle, const QString& folder, const QString& key, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		b->setFolder(folder);
 		QVariantMap rc;
 		foreach (KWallet::Entry *entry, b->readEntryList(key)) {
@@ -909,10 +937,10 @@ QVariantMap KWalletD::readPasswordList(int handle, const QString& folder, const
 }
 
 
-int KWalletD::writeMap(int handle, const QString& folder, const QString& key, const QByteArray& value, const QDBusMessage& msg) {
+int KWalletD::writeMap(int handle, const QString& folder, const QString& key, const QByteArray& value, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		b->setFolder(folder);
 		KWallet::Entry e;
 		e.setKey(key);
@@ -927,10 +955,10 @@ int KWalletD::writeMap(int handle, const QString& folder, const QString& key, co
 }
 
 
-int KWalletD::writeEntry(int handle, const QString& folder, const QString& key, const QByteArray& value, int entryType, const QDBusMessage& msg) {
+int KWalletD::writeEntry(int handle, const QString& folder, const QString& key, const QByteArray& value, int entryType, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		b->setFolder(folder);
 		KWallet::Entry e;
 		e.setKey(key);
@@ -945,10 +973,10 @@ int KWalletD::writeEntry(int handle, const QString& folder, const QString& key,
 }
 
 
-int KWalletD::writeEntry(int handle, const QString& folder, const QString& key, const QByteArray& value, const QDBusMessage& msg) {
+int KWalletD::writeEntry(int handle, const QString& folder, const QString& key, const QByteArray& value, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		b->setFolder(folder);
 		KWallet::Entry e;
 		e.setKey(key);
@@ -963,10 +991,10 @@ int KWalletD::writeEntry(int handle, const QString& folder, const QString& key,
 }
 
 
-int KWalletD::writePassword(int handle, const QString& folder, const QString& key, const QString& value, const QDBusMessage& msg) {
+int KWalletD::writePassword(int handle, const QString& folder, const QString& key, const QString& value, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		b->setFolder(folder);
 		KWallet::Entry e;
 		e.setKey(key);
@@ -981,10 +1009,10 @@ int KWalletD::writePassword(int handle, const QString& folder, const QString& ke
 }
 
 
-int KWalletD::entryType(int handle, const QString& folder, const QString& key, const QDBusMessage& msg) {
+int KWalletD::entryType(int handle, const QString& folder, const QString& key, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		if (!b->hasFolder(folder)) {
 			return KWallet::Wallet::Unknown;
 		}
@@ -998,10 +1026,10 @@ int KWalletD::entryType(int handle, const QString& folder, const QString& key, c
 }
 
 
-bool KWalletD::hasEntry(int handle, const QString& folder, const QString& key, const QDBusMessage& msg) {
+bool KWalletD::hasEntry(int handle, const QString& folder, const QString& key, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		if (!b->hasFolder(folder)) {
 			return false;
 		}
@@ -1013,10 +1041,10 @@ bool KWalletD::hasEntry(int handle, const QString& folder, const QString& key, c
 }
 
 
-int KWalletD::removeEntry(int handle, const QString& folder, const QString& key, const QDBusMessage& msg) {
+int KWalletD::removeEntry(int handle, const QString& folder, const QString& key, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		if (!b->hasFolder(folder)) {
 			return 0;
 		}
@@ -1101,10 +1129,10 @@ void KWalletD::doCloseSignals(int handle, const QString& wallet) {
 }
 
 
-int KWalletD::renameEntry(int handle, const QString& folder, const QString& oldName, const QString& newName, const QDBusMessage& msg) {
+int KWalletD::renameEntry(int handle, const QString& folder, const QString& oldName, const QString& newName, const QString& appid) {
 	KWallet::Backend *b;
 
-	if ((b = getWallet(msg.service(), handle))) {
+	if ((b = getWallet(appid, handle))) {
 		b->setFolder(folder);
 		int rc = b->renameEntry(oldName, newName);
 		emitFolderUpdated(b->walletName(), folder);
@@ -1184,15 +1212,15 @@ void KWalletD::reconfigure() {
 	int timeSave = _idleTime;
 	// in minutes!
 	_idleTime = cfg.readEntry("Idle Timeout", 10) * 60 * 1000;
-
+#ifdef Q_WS_X11
 	if ( kdesktop->isValid() ) {
 		if (cfg.readEntry("Close on Screensaver", false)) {
-			connect(kdesktop, SIGNAL(KDE_start_screensaver()), SLOT(closeAllWallets()));
+			connect(kdesktop, SIGNAL(screenSaverStarted()), SLOT(closeAllWallets()));
 		} else {
-			kdesktop->disconnect(SIGNAL(KDE_start_screensaver()), this, SLOT(closeAllWallets()));
+			kdesktop->disconnect(SIGNAL(screenSaverStarted()), this, SLOT(closeAllWallets()));
 		}
 	}
-
+#endif
 	// Handle idle changes
 	if (_closeIdle) {
 		if (_idleTime != timeSave) { // Timer length changed
