@@ -24,7 +24,6 @@
 #include <config.h>
 
 #include <kurl.h>
-#include <kapplication.h>
 #include <kde_file.h>
 #include <kio/netaccess.h>
 #include <kdebug.h>
@@ -34,7 +33,6 @@
 #include <qfileinfo.h>
 #include <qeventloop.h>
 #include <qdir.h>
-#include <qfileinfo.h>
 #include <qhash.h>
 #include <qvariant.h>
 
@@ -47,28 +45,39 @@
 #include <sys/types.h>
 #include <utime.h>
 #include <kprotocolinfo.h>
+#include <kio/scheduler.h>
+#include <kio/directorysizejob.h>
+#include <kio/copyjob.h>
 
 QTEST_KDEMAIN( JobTest, GUI )
 
 // The code comes partly from kdebase/kioslave/trash/testtrash.cpp
 
-QString JobTest::homeTmpDir() const
+static QString homeTmpDir()
 {
     return QFile::decodeName( getenv( "KDEHOME" ) ) + "/jobtest/";
 }
 
-QString JobTest::otherTmpDir() const
+static QString otherTmpDir()
 {
+#ifdef Q_WS_WIN
+    return QDir::tempPath() + "/jobtest/";
+#else
     // This one needs to be on another partition
     return "/tmp/jobtest/";
+#endif
 }
 
-KUrl JobTest::systemTmpDir() const
+static KUrl systemTmpDir()
 {
+#ifdef Q_WS_WIN
+    return KUrl( "system:" + QDir::homePath() + "/.kde-unit-test/jobtest-system/" );
+#else
     return KUrl( "system:/home/.kde-unit-test/jobtest-system/" );
+#endif
 }
 
-QString JobTest::realSystemPath() const
+static QString realSystemPath()
 {
     return QFile::decodeName( getenv( "KDEHOME" ) ) + "/jobtest-system/";
 }
@@ -118,7 +127,7 @@ static void setTimeStamp( const QString& path )
     utbuf.actime = tp.tv_sec - 30; // 30 seconds ago
     utbuf.modtime = tp.tv_sec - 60; // 60 second ago
     utime( QFile::encodeName( path ), &utbuf );
-    qDebug( "Time changed for %s", qPrintable( path ) );
+    kDebug() << "Time changed for " << path << endl;
 #endif
 }
 
@@ -156,8 +165,10 @@ static void createTestDirectory( const QString& path )
     if ( !ok && !dir.exists() )
         kFatal() << "couldn't create " << path << endl;
     createTestFile( path + "/testfile" );
+#ifndef Q_WS_WIN
     createTestSymlink( path + "/testlink" );
     QVERIFY( QFileInfo( path + "/testlink" ).isSymLink() );
+#endif
     setTimeStamp( path );
 }
 
@@ -174,7 +185,7 @@ void JobTest::get()
     kDebug() << k_funcinfo << endl;
     const QString filePath = homeTmpDir() + "fileFromHome";
     createTestFile( filePath );
-    KUrl u; u.setPath( filePath );
+    KUrl u( filePath );
     m_result = -1;
     KIO::StoredTransferJob* job = KIO::storedGet( u );
     job->setUiDelegate( 0 );
@@ -193,14 +204,59 @@ void JobTest::slotGetResult( KJob* job )
     emit exitLoop();
 }
 
+void JobTest::put()
+{
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    KUrl u(filePath);
+    KIO::TransferJob* job = KIO::put( u, 0600, true /*overwrite*/, false /*no resume*/ );
+    QDateTime mtime = QDateTime::currentDateTime().addSecs( -30 ); // 30 seconds ago
+    mtime.setTime_t(mtime.toTime_t()); // hack for losing the milliseconds
+    job->setModificationTime(mtime);
+    job->setUiDelegate( 0 );
+    connect( job, SIGNAL( result(KJob*) ),
+            this, SLOT( slotResult(KJob*) ) );
+    connect( job, SIGNAL(dataReq(KIO::Job*, QByteArray&)),
+             this, SLOT(slotDataReq(KIO::Job*, QByteArray&)) );
+    m_result = -1;
+    m_dataReqCount = 0;
+    enterLoop();
+    QVERIFY( m_result == 0 ); // no error
+
+    QFileInfo fileInfo(filePath);
+    QVERIFY(fileInfo.exists());
+    QCOMPARE(fileInfo.size(), 30LL); // "This is a test for KIO::put()\n"
+    QCOMPARE((int)fileInfo.permissions(), (int)(QFile::ReadOwner | QFile::WriteOwner | QFile::ReadUser | QFile::WriteUser ));
+    QCOMPARE(fileInfo.lastModified(), mtime);
+}
+
+void JobTest::slotDataReq( KIO::Job*, QByteArray& data )
+{
+    // Really not the way you'd write a slotDataReq usually :)
+    switch(m_dataReqCount++) {
+    case 0:
+        data = "This is a test for ";
+        break;
+    case 1:
+        data = "KIO::put()\n";
+        break;
+    case 2:
+        data = QByteArray();
+        break;
+    }
+}
+
+void JobTest::slotResult( KJob* job )
+{
+    m_result = job->error();
+    emit exitLoop();
+}
+
 ////
 
 void JobTest::copyLocalFile( const QString& src, const QString& dest )
 {
-    KUrl u;
-    u.setPath( src );
-    KUrl d;
-    d.setPath( dest );
+    const KUrl u( src );
+    const KUrl d( dest );
 
     // copy the file with file_copy
     bool ok = KIO::NetAccess::file_copy( u, d );
@@ -214,7 +270,13 @@ void JobTest::copyLocalFile( const QString& src, const QString& dest )
         // The datapump solution ignores mtime, the app has to call FileCopyJob::setModificationTime()
         QFileInfo srcInfo( src );
         QFileInfo destInfo( dest );
+#ifdef Q_WS_WIN
+        // win32 time may differs in msec part
+        QCOMPARE( srcInfo.lastModified().toString("dd.MM.yyyy hh:mm"),
+                  destInfo.lastModified().toString("dd.MM.yyyy hh:mm") );
+#else
         QCOMPARE( srcInfo.lastModified(), destInfo.lastModified() );
+#endif
     }
 
     // cleanup and retry with KIO::copy()
@@ -227,7 +289,13 @@ void JobTest::copyLocalFile( const QString& src, const QString& dest )
         // check that the timestamp is the same (#24443)
         QFileInfo srcInfo( src );
         QFileInfo destInfo( dest );
+#ifdef Q_WS_WIN
+        // win32 time may differs in msec part
+        QCOMPARE( srcInfo.lastModified().toString("dd.MM.yyyy hh:mm"),
+                  destInfo.lastModified().toString("dd.MM.yyyy hh:mm") );
+#else
         QCOMPARE( srcInfo.lastModified(), destInfo.lastModified() );
+#endif
     }
 }
 
@@ -261,7 +329,13 @@ void JobTest::copyLocalDirectory( const QString& src, const QString& _dest, int
         // check that the timestamp is the same (#24443)
         QFileInfo srcInfo( src );
         QFileInfo destInfo( dest );
+#ifdef Q_WS_WIN
+        // win32 time may differs in msec part
+        QCOMPARE( srcInfo.lastModified().toString("dd.MM.yyyy hh:mm"),
+                  destInfo.lastModified().toString("dd.MM.yyyy hh:mm") );
+#else
         QCOMPARE( srcInfo.lastModified(), destInfo.lastModified() );
+#endif
     }
 }
 
@@ -366,7 +440,9 @@ void JobTest::moveLocalDirectory( const QString& src, const QString& dest )
     QVERIFY( QFile::exists( src ) );
     QVERIFY( QFileInfo( src ).isDir() );
     QVERIFY( QFileInfo( src + "/testfile" ).isFile() );
+#ifndef Q_WS_WIN
     QVERIFY( QFileInfo( src + "/testlink" ).isSymLink() );
+#endif
     KUrl u;
     u.setPath( src );
     KUrl d;
@@ -378,8 +454,9 @@ void JobTest::moveLocalDirectory( const QString& src, const QString& dest )
     QVERIFY( QFileInfo( dest ).isDir() );
     QVERIFY( QFileInfo( dest + "/testfile" ).isFile() );
     QVERIFY( !QFile::exists( src ) ); // not there anymore
-
+#ifndef Q_WS_WIN
     QVERIFY( QFileInfo( dest + "/testlink" ).isSymLink() );
+#endif
 }
 
 void JobTest::moveFileToSamePartition()
@@ -411,25 +488,32 @@ void JobTest::moveFileToOtherPartition()
 
 void JobTest::moveSymlinkToOtherPartition()
 {
+#ifndef Q_WS_WIN
     kDebug() << k_funcinfo << endl;
     const QString filePath = homeTmpDir() + "testlink";
     const QString dest = otherTmpDir() + "testlink_moved";
     createTestSymlink( filePath );
     moveLocalSymlink( filePath, dest );
+#endif
 }
 
 void JobTest::moveDirectoryToOtherPartition()
 {
     kDebug() << k_funcinfo << endl;
+#ifndef Q_WS_WIN
     const QString src = homeTmpDir() + "dirFromHome";
     const QString dest = otherTmpDir() + "dirFromHome_moved";
     createTestDirectory( src );
     moveLocalDirectory( src, dest );
+#endif
 }
 
 void JobTest::moveFileNoPermissions()
 {
     kDebug() << k_funcinfo << endl;
+#ifdef Q_WS_WIN
+    kDebug() << "port to win32" << endl;
+#else
     const QString src = "/etc/passwd";
     const QString dest = homeTmpDir() + "passwd";
     QVERIFY( QFile::exists( src ) );
@@ -452,11 +536,15 @@ void JobTest::moveFileNoPermissions()
     // In fact we assume /home is a separate partition, in this test, so:
     QVERIFY( QFile::exists( dest ) );
     QVERIFY( QFile::exists( src ) );
+#endif
 }
 
 void JobTest::moveDirectoryNoPermissions()
 {
     kDebug() << k_funcinfo << endl;
+#ifdef Q_WS_WIN
+    kDebug() << "port to win32" << endl;
+#else
 #if 1
     QString src = "/etc/rc.d";
     if ( !QFile::exists( src ) )
@@ -464,7 +552,7 @@ void JobTest::moveDirectoryNoPermissions()
 #else
     QString src = "/etc";
 #endif
-    const QString dest = homeTmpDir() + "mdnp";
+   const QString dest = homeTmpDir() + "mdnp";
     QVERIFY( QFile::exists( src ) );
     QVERIFY( QFileInfo( src ).isDir() );
     KUrl u;
@@ -481,30 +569,79 @@ void JobTest::moveDirectoryNoPermissions()
     QVERIFY( KIO::NetAccess::lastError() == KIO::ERR_ACCESS_DENIED );
     QVERIFY( QFile::exists( dest ) ); // see moveFileNoPermissions
     QVERIFY( QFile::exists( src ) );
+#endif
 }
 
 void JobTest::listRecursive()
 {
+    // Note: many other tests must have been run before since we rely on the files they created
+
     const QString src = homeTmpDir();
+#ifndef Q_WS_WIN
     // Add a symlink to a dir, to make sure we don't recurse into those
     bool symlinkOk = symlink( "dirFromHome", QFile::encodeName( src + "/dirFromHome_link" ) ) == 0;
     QVERIFY( symlinkOk );
-
-    KUrl u;
-    u.setPath( src );
-    KIO::ListJob* job = KIO::listRecursive( u );
+#endif
+    KIO::ListJob* job = KIO::listRecursive( KUrl(src) );
     job->setUiDelegate( 0 );
     connect( job, SIGNAL( entries( KIO::Job*, const KIO::UDSEntryList& ) ),
              SLOT( slotEntries( KIO::Job*, const KIO::UDSEntryList& ) ) );
     bool ok = KIO::NetAccess::synchronousRun( job, 0 );
     QVERIFY( ok );
     m_names.sort();
+    QByteArray ref_names = QByteArray( ".,..,"
+            "dirFromHome,dirFromHome/testfile,"
+#ifndef Q_WS_WIN
+            "dirFromHome/testlink,"
+#endif
+            "dirFromHome_copied,"
+            "dirFromHome_copied/dirFromHome,dirFromHome_copied/dirFromHome/testfile,"
+#ifndef Q_WS_WIN
+            "dirFromHome_copied/dirFromHome/testlink,"
+#endif
+            "dirFromHome_copied/testfile,"
+#ifndef Q_WS_WIN
+            "dirFromHome_copied/testlink,dirFromHome_link,"
+#endif
+            "fileFromHome,fileFromHome_copied");
+
     qDebug( "%s", qPrintable( m_names.join( "," ) ) );
-    QCOMPARE( m_names.join( "," ).toLatin1(), QByteArray( ".,..,"
-            "dirFromHome,dirFromHome/testfile,dirFromHome/testlink,dirFromHome_copied,"
-            "dirFromHome_copied/dirFromHome,dirFromHome_copied/dirFromHome/testfile,dirFromHome_copied/dirFromHome/testlink,"
-            "dirFromHome_copied/testfile,dirFromHome_copied/testlink,dirFromHome_link,"
-            "fileFromHome,fileFromHome_copied" ) );
+    qDebug( "%s", ref_names.data() );
+    QCOMPARE( m_names.join( "," ).toLatin1(), ref_names );
+}
+
+void JobTest::directorySize()
+{
+    // Note: many other tests must have been run before since we rely on the files they created
+
+    const QString src = homeTmpDir();
+
+    KIO::DirectorySizeJob* job = KIO::directorySize( KUrl(src) );
+    job->setUiDelegate( 0 );
+    bool ok = KIO::NetAccess::synchronousRun( job, 0 );
+    QVERIFY( ok );
+    kDebug() << "totalSize: " << job->totalSize() << endl;
+    kDebug() << "totalFiles: " << job->totalFiles() << endl;
+    kDebug() << "totalSubdirs: " << job->totalSubdirs() << endl;
+#ifdef Q_WS_WIN
+    QCOMPARE(job->totalFiles(), 5ULL); // see expected result in listRecursive() above
+    QCOMPARE(job->totalSubdirs(), 3ULL); // see expected result in listRecursive() above
+    QVERIFY(job->totalSize() > 54);
+#else
+    QCOMPARE(job->totalFiles(), 8ULL); // see expected result in listRecursive() above
+    QCOMPARE(job->totalSubdirs(), 4ULL); // see expected result in listRecursive() above
+    QVERIFY(job->totalSize() > 512);
+#endif
+
+    // TODO test error case
+}
+
+void JobTest::directorySizeError()
+{
+    KIO::DirectorySizeJob* job = KIO::directorySize( KUrl("/I/Dont/Exist") );
+    job->setUiDelegate( 0 );
+    bool ok = KIO::NetAccess::synchronousRun( job, 0 );
+    QVERIFY( !ok );
 }
 
 void JobTest::slotEntries( KIO::Job*, const KIO::UDSEntryList& lst )
@@ -885,7 +1022,11 @@ void JobTest::copyFileToSystem( bool resolve_local_urls )
     kDebug() << "copying " << u << " to " << d << endl;
 
     // copy the file with file_copy
-    bool ok = KIO::NetAccess::file_copy( u, d );
+    m_mimetype.clear();
+    KIO::FileCopyJob* job = KIO::file_copy( u, d );
+    connect( job, SIGNAL(mimetype(KIO::Job*,const QString&)),
+             this, SLOT(slotMimetype(KIO::Job*,const QString&)) );
+    bool ok = KIO::NetAccess::synchronousRun( job, 0 );
     QVERIFY( ok );
 
     QString dest = realSystemPath() + "fileFromHome_copied";
@@ -899,6 +1040,9 @@ void JobTest::copyFileToSystem( bool resolve_local_urls )
         // unless we use setModificationTime in the app code.
     }
 
+    // Check mimetype
+    QCOMPARE(m_mimetype, QString("text/plain"));
+
     // cleanup and retry with KIO::copy()
     QFile::remove( dest );
     ok = KIO::NetAccess::dircopy( u, d, 0 );
@@ -916,4 +1060,24 @@ void JobTest::copyFileToSystem( bool resolve_local_urls )
     kio_resolve_local_urls = true;
 }
 
+void JobTest::getInvalidUrl()
+{
+    KUrl url("file://\"\"");
+    QVERIFY(!url.isValid());
+
+    KIO::SimpleJob* job = KIO::get(url);
+    QVERIFY(job != 0);
+
+    KIO::Scheduler::scheduleJob(job); // shouldn't crash (#135456)
+
+    bool ok = KIO::NetAccess::synchronousRun( job, 0 );
+    QVERIFY( !ok ); // it should fail :)
+}
+
+void JobTest::slotMimetype(KIO::Job* job, const QString& type)
+{
+    QVERIFY( job != 0 );
+    m_mimetype = type;
+}
+
 #include "jobtest.moc"
