@@ -1,20 +1,22 @@
-/*
- *  Copyright (C) 2006 David Faure   <faure@kde.org>
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Library General Public
- *  License version 2 as published by the Free Software Foundation;
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
- */
+/* This file is part of the KDE project
+   Copyright (C) 2006 David Faure <faure@kde.org>
+
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Library General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
+
+   This library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Library General Public License for more details.
+
+   You should have received a copy of the GNU Library General Public License
+   along with this library; see the file COPYING.LIB.  If not, write to
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
+*/
+
 
 #include "karchivetest.h"
 #include "karchivetest.moc"
@@ -25,6 +27,7 @@
 #include <qfileinfo.h>
 #include <kdebug.h>
 #include <kfilterdev.h>
+#include <ktempdir.h>
 
 QTEST_KDEMAIN( KArchiveTest, NoGUI )
 
@@ -54,7 +57,7 @@ static void writeTestFilesToArchive( KArchive* archive )
     localFile.close();
     ok = archive->addLocalFile( "test3", "mydir/test3" );
     QVERIFY( ok );
-    ok = archive->writeFile( "my/dir/test3", "dfaure", "hackers", "I don't speak German (David)", 29 );
+    ok = archive->writeFile( "my/dir/test3", "dfaure", "hackers", "I do not speak German\nDavid.", 29 );
     QVERIFY( ok );
 
     // Now a medium file : 100 null bytes
@@ -122,10 +125,19 @@ static void testFileData( KArchive* archive )
     QCOMPARE( arr.size(), 13 );
     QCOMPARE( arr, QByteArray( "Noch so einer" ) );
 
-    // Now test using device()
-    QIODevice *dev = f->device();
+    // Now test using createDevice()
+    QIODevice *dev = f->createDevice();
     QByteArray contents = dev->readAll();
     QCOMPARE( contents, arr );
+    delete dev;
+
+    dev = f->createDevice();
+    contents = dev->read(5); // test reading in two chunks
+    QCOMPARE(contents.size(), 5);
+    contents += dev->read(50);
+    QCOMPARE(contents.size(), 13);
+    QCOMPARE( QString::fromLatin1(contents), QString::fromLatin1(arr) );
+    delete dev;
 
     e = dir->entry( "mediumfile" );
     QVERIFY( e && e->isFile() );
@@ -137,10 +149,52 @@ static void testFileData( KArchive* archive )
     f = (KArchiveFile*)e;
     QCOMPARE( f->data().size(), 20000   );
 
+    e = dir->entry( "my/dir/test3" );
+    QVERIFY( e && e->isFile() );
+    f = (KArchiveFile*)e;
+    dev = f->createDevice();
+    QByteArray firstLine = dev->readLine();
+    QCOMPARE(QString::fromLatin1(firstLine), QString::fromLatin1("I do not speak German\n"));
+    QByteArray secondLine = dev->read(100);
+    QCOMPARE(QString::fromLatin1(secondLine), QString::fromLatin1("David."));
+    delete dev;
 }
 
-static const char* s_tarFileName = "karchivetest.tar.gz";
-static const char* s_tarMaxLengthFileName = "karchivetest-maxlength.tar.gz";
+static void testCopyTo( KArchive* archive )
+{
+    const KArchiveDirectory* dir = archive->directory();
+    KTempDir tmpDir;
+    const QString dirName = tmpDir.name();
+
+    dir->copyTo( dirName );
+
+    QVERIFY(QFile::exists(dirName+"dir"));
+    QVERIFY(QFileInfo(dirName+"dir").isDir());
+
+    QFileInfo fileInfo1(dirName+"dir/subdir/mediumfile2");
+    QVERIFY(fileInfo1.exists());
+    QVERIFY(fileInfo1.isFile());
+    QCOMPARE(fileInfo1.size(), Q_INT64_C(100));
+
+    QFileInfo fileInfo2(dirName+"hugefile");
+    QVERIFY(fileInfo2.exists());
+    QVERIFY(fileInfo2.isFile());
+    QCOMPARE(fileInfo2.size(), Q_INT64_C(20000));
+
+    QFileInfo fileInfo3(dirName+"mediumfile");
+    QVERIFY(fileInfo3.exists());
+    QVERIFY(fileInfo3.isFile());
+    QCOMPARE(fileInfo3.size(), Q_INT64_C(100));
+
+    QFileInfo fileInfo4(dirName+"my/dir/test3");
+    QVERIFY(fileInfo4.exists());
+    QVERIFY(fileInfo4.isFile());
+    QCOMPARE(fileInfo4.size(), Q_INT64_C(29));
+}
+
+static const char* s_tarFileName = "karchivetest.tar";
+static const char* s_tarGzFileName = "karchivetest.tar.gz";
+static const char* s_tarGzMaxLengthFileName = "karchivetest-maxlength.tar.gz";
 static const char* s_zipFileName = "karchivetest.zip";
 static const char* s_zipMaxLengthFileName = "karchivetest-maxlength.zip";
 
@@ -159,13 +213,31 @@ void KArchiveTest::testCreateTar()
     QFileInfo fileInfo( QFile::encodeName( s_tarFileName ) );
     QVERIFY( fileInfo.exists() );
     // We can't check for an exact size because of the addLocalFile, whose data is system-dependent
+    QVERIFY( fileInfo.size() > 450 );
+}
+
+void KArchiveTest::testCreateTarGz()
+{
+    KTar tar( s_tarGzFileName );
+
+    bool ok = tar.open( QIODevice::WriteOnly );
+    QVERIFY( ok );
+
+    writeTestFilesToArchive( &tar );
+
+    ok = tar.close();
+    QVERIFY( ok );
+
+    QFileInfo fileInfo( QFile::encodeName( s_tarGzFileName ) );
+    QVERIFY( fileInfo.exists() );
+    // We can't check for an exact size because of the addLocalFile, whose data is system-dependent
     QVERIFY( fileInfo.size() > 350 );
 }
 
 void KArchiveTest::testReadTar()
 {
     // testCreateTar must have been run first.
-    KTar tar( s_tarFileName );
+    KTar tar( s_tarGzFileName );
 
     bool ok = tar.open( QIODevice::ReadOnly );
     QVERIFY( ok );
@@ -205,8 +277,8 @@ void KArchiveTest::testReadTar()
 void KArchiveTest::testUncompress()
 {
     // testCreateTar must have been run first.
-    QVERIFY( QFile::exists( s_tarFileName ) );
-    QIODevice *filterDev = KFilterDev::deviceForFile( s_tarFileName, "application/x-gzip", true );
+    QVERIFY( QFile::exists( s_tarGzFileName ) );
+    QIODevice *filterDev = KFilterDev::deviceForFile( s_tarGzFileName, "application/x-gzip", true );
     QVERIFY( filterDev );
     QByteArray buffer;
     buffer.resize(8*1024);
@@ -230,7 +302,7 @@ void KArchiveTest::testUncompress()
 void KArchiveTest::testTarFileData()
 {
     // testCreateTar must have been run first.
-    KTar tar( s_tarFileName );
+    KTar tar( s_tarGzFileName );
     bool ok = tar.open( QIODevice::ReadOnly );
     QVERIFY( ok );
 
@@ -240,9 +312,22 @@ void KArchiveTest::testTarFileData()
     QVERIFY( ok );
 }
 
+void KArchiveTest::testTarCopyTo()
+{
+    // testCreateTar must have been run first.
+    KTar tar( s_tarGzFileName );
+    bool ok = tar.open( QIODevice::ReadOnly );
+    QVERIFY( ok );
+
+    testCopyTo( &tar );
+
+    ok = tar.close();
+    QVERIFY( ok );
+}
+
 void KArchiveTest::testTarMaxLength()
 {
-    KTar tar( s_tarMaxLengthFileName );
+    KTar tar( s_tarGzMaxLengthFileName );
 
     bool ok = tar.open( QIODevice::WriteOnly );
     QVERIFY( ok );
@@ -326,6 +411,21 @@ void KArchiveTest::testCreateZip()
     QCOMPARE( arr, zipMimeType );
 }
 
+void KArchiveTest::testCreateZipError()
+{
+    // Giving a directory name to kzip must give an error case in close(), see #136630.
+    // Otherwise we just lose data.
+    KZip zip( QDir::currentPath() );
+
+    bool ok = zip.open( QIODevice::WriteOnly );
+    QVERIFY( ok );
+
+    writeTestFilesToArchive( &zip );
+
+    ok = zip.close();
+    QVERIFY( !ok );
+}
+
 void KArchiveTest::testReadZip()
 {
     // testCreateZip must have been run first.
@@ -376,6 +476,19 @@ void KArchiveTest::testZipFileData()
     QVERIFY( ok );
 }
 
+void KArchiveTest::testZipCopyTo()
+{
+    // testCreateZip must have been run first.
+    KZip zip( s_zipFileName );
+    bool ok = zip.open( QIODevice::ReadOnly );
+    QVERIFY( ok );
+
+    testCopyTo( &zip );
+
+    ok = zip.close();
+    QVERIFY( ok );
+}
+
 void KArchiveTest::testZipMaxLength()
 {
     KZip zip( s_zipMaxLengthFileName );
@@ -415,9 +528,8 @@ void KArchiveTest::testZipMaxLength()
 
 void KArchiveTest::cleanupTestCase()
 {
-    QVERIFY( QFile::remove("karchivetest-maxlength.tar.gz") );
-    QVERIFY( QFile::remove("karchivetest-maxlength.zip") );
-    QVERIFY( QFile::remove("karchivetest.tar.gz") );
-    QVERIFY( QFile::remove("karchivetest.zip") );
+    QFile::remove("karchivetest-maxlength.tar.gz");
+    QFile::remove("karchivetest-maxlength.zip");
+    QFile::remove("karchivetest.tar.gz");
+    QFile::remove("karchivetest.zip");
 }
-  
