@@ -10,8 +10,9 @@ private Q_SLOTS:
     void initTestCase();
     void testByName();
     void testIcons();
-    void testFindByURL();
+    void testFindByUrl();
     void testFindByPath();
+    void testFindByNameAndContent();
     void testAllMimeTypes();
     void testMimeTypeParent();
     void testMimeTypeTraderForTextPlain();
