@@ -2,6 +2,7 @@
    Copyright (C) 2000-2002 Stephan Kulow <coolo@kde.org>
    Copyright (C) 2000-2002 David Faure <faure@kde.org>
    Copyright (C) 2000-2002 Waldo Bastian <bastian@kde.org>
+   Copyright (C) 2006 Allan Sandfeld Jensen <sandfeld@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -62,14 +63,21 @@
 #include <string.h>
 #endif
 
-#include <qdatetime.h>
+#include <QByteArray>
+#include <QDateTime>
+#include <QVarLengthArray>
+#include <QCoreApplication>
 #include <qregexp.h>
+#ifdef Q_WS_WIN
+#include <QDir>
+#include <QFileInfo>
+#endif
 
 #include <kdebug.h>
 #include <kurl.h>
-#include <kinstance.h>
+#include <kcomponentdata.h>
 #include <ksimpleconfig.h>
-#include <ktempfile.h>
+#include <ktemporaryfile.h>
 #include <klocale.h>
 #include <qfile.h>
 #include "file.h"
@@ -84,6 +92,7 @@
 #endif
 
 #include <kstandarddirs.h>
+#include <kio/connection.h>
 #include <kio/ioslave_defaults.h>
 #include <kde_file.h>
 #include <kglobal.h>
@@ -103,7 +112,9 @@ static void appendACLAtoms( const QByteArray & path, UDSEntry& entry,
 extern "C" int KDE_EXPORT kdemain( int argc, char **argv )
 {
   KLocale::setMainCatalog("kdelibs");
-  KInstance instance( "kio_file" );
+
+  QCoreApplication app( argc, argv ); // needed for QSocketNotifier
+  KComponentData componentData( "kio_file" );
   ( void ) KGlobal::locale();
 
   kDebug(7101) << "Starting " << getpid() << endl;
@@ -176,7 +187,7 @@ int FileProtocol::setACL( const char *path, mode_t perm, bool directoryDefault )
 
 void FileProtocol::chmod( const KUrl& url, int permissions )
 {
-    QByteArray _path( QFile::encodeName(url.path()) );
+    QByteArray _path( QFile::encodeName(url.toLocalFile()) );
     /* FIXME: Should be atomic */
     if ( ::chmod( _path.data(), permissions ) == -1 ||
         ( setACL( _path.data(), permissions, false ) == -1 ) ||
@@ -186,7 +197,7 @@ void FileProtocol::chmod( const KUrl& url, int permissions )
         switch (errno) {
             case EPERM:
             case EACCES:
-                error( KIO::ERR_ACCESS_DENIED, url.path() );
+                error( KIO::ERR_ACCESS_DENIED, _path );
                 break;
 #if defined(ENOTSUP)
             case ENOTSUP: // from setACL since chmod can't return ENOTSUP
@@ -194,10 +205,10 @@ void FileProtocol::chmod( const KUrl& url, int permissions )
                 break;
 #endif
             case ENOSPC:
-                error( KIO::ERR_DISK_FULL, url.path() );
+                error( KIO::ERR_DISK_FULL, _path );
                 break;
             default:
-        error( KIO::ERR_CANNOT_CHMOD, url.path() );
+        error( KIO::ERR_CANNOT_CHMOD, _path );
         }
     } else
         finished();
@@ -205,7 +216,7 @@ void FileProtocol::chmod( const KUrl& url, int permissions )
 
 void FileProtocol::mkdir( const KUrl& url, int permissions )
 {
-    QByteArray _path( QFile::encodeName(url.path()));
+    QByteArray _path( QFile::encodeName(url.toLocalFile()));
 
     kDebug(7101) << "mkdir(): " << _path << ", permission = " << permissions << endl;
 
@@ -213,13 +224,13 @@ void FileProtocol::mkdir( const KUrl& url, int permissions )
     if ( KDE_stat( _path.data(), &buff ) == -1 ) {
         if ( KDE_mkdir( _path.data(), 0777 /*umask will be applied*/ ) != 0 ) {
             if ( errno == EACCES ) {
-          error( KIO::ERR_ACCESS_DENIED, url.path() );
+          error( KIO::ERR_ACCESS_DENIED, _path );
           return;
             } else if ( errno == ENOSPC ) {
-          error( KIO::ERR_DISK_FULL, url.path() );
+          error( KIO::ERR_DISK_FULL, _path );
           return;
             } else {
-          error( KIO::ERR_COULD_NOT_MKDIR, url.path() );
+          error( KIO::ERR_COULD_NOT_MKDIR, _path );
           return;
             }
         } else {
@@ -233,15 +244,16 @@ void FileProtocol::mkdir( const KUrl& url, int permissions )
 
     if ( S_ISDIR( buff.st_mode ) ) {
         kDebug(7101) << "ERR_DIR_ALREADY_EXIST" << endl;
-        error( KIO::ERR_DIR_ALREADY_EXIST, url.path() );
+        error( KIO::ERR_DIR_ALREADY_EXIST, _path );
         return;
     }
-    error( KIO::ERR_FILE_ALREADY_EXIST, url.path() );
+    error( KIO::ERR_FILE_ALREADY_EXIST, _path );
     return;
 }
 
 void FileProtocol::get( const KUrl& url )
 {
+    kDebug( 7101 ) << "File::get" << endl;
     if (!url.isLocalFile()) {
         KUrl redir(url);
 	redir.setProtocol(config()->readEntry("DefaultRemoteProtocol", "smb"));
@@ -250,28 +262,28 @@ void FileProtocol::get( const KUrl& url )
 	return;
     }
 
-    QByteArray _path( QFile::encodeName(url.path()));
+    QByteArray _path( QFile::encodeName(url.toLocalFile()));
     KDE_struct_stat buff;
     if ( KDE_stat( _path.data(), &buff ) == -1 ) {
         if ( errno == EACCES )
-           error( KIO::ERR_ACCESS_DENIED, url.path() );
+           error( KIO::ERR_ACCESS_DENIED, _path );
         else
-           error( KIO::ERR_DOES_NOT_EXIST, url.path() );
+           error( KIO::ERR_DOES_NOT_EXIST, _path );
         return;
     }
 
     if ( S_ISDIR( buff.st_mode ) ) {
-        error( KIO::ERR_IS_DIRECTORY, url.path() );
+        error( KIO::ERR_IS_DIRECTORY, _path );
         return;
     }
     if ( !S_ISREG( buff.st_mode ) ) {
-        error( KIO::ERR_CANNOT_OPEN_FOR_READING, url.path() );
+        error( KIO::ERR_CANNOT_OPEN_FOR_READING, _path );
         return;
     }
 
     int fd = KDE_open( _path.data(), O_RDONLY);
     if ( fd < 0 ) {
-        error( KIO::ERR_CANNOT_OPEN_FOR_READING, url.path() );
+        error( KIO::ERR_CANNOT_OPEN_FOR_READING, _path );
         return;
     }
 
@@ -281,7 +293,7 @@ void FileProtocol::get( const KUrl& url )
 
     // Determine the mimetype of the file to be retrieved, and emit it.
     // This is mandatory in all slaves (for KRun/BrowserRun to work).
-    KMimeType::Ptr mt = KMimeType::findByURL( url, buff.st_mode, true /* local URL */ );
+    KMimeType::Ptr mt = KMimeType::findByUrl( url, buff.st_mode, true /* local URL */ );
     emit mimeType( mt->name() );
 
     KIO::filesize_t processed_size = 0;
@@ -302,10 +314,9 @@ void FileProtocol::get( const KUrl& url )
         }
     }
 
-    totalSize( buff.st_size );
-
     char buffer[ MAX_IPC_SIZE ];
     QByteArray array;
+    bool mimetypeEmitted = false;
 
     while( 1 )
     {
@@ -314,14 +325,27 @@ void FileProtocol::get( const KUrl& url )
        {
           if (errno == EINTR)
               continue;
-          error( KIO::ERR_COULD_NOT_READ, url.path());
+          error( KIO::ERR_COULD_NOT_READ, _path );
           close(fd);
           return;
        }
        if (n == 0)
           break; // Finished
 
-       array = array.fromRawData(buffer, n);
+       array = QByteArray::fromRawData(buffer, n);
+
+       // get the mime type and set the total size ...
+       if(!mimetypeEmitted)
+       {
+           mimetypeEmitted = true;
+           KMimeType::Ptr mime = KMimeType::findByNameAndContent(url.fileName(), array);
+           kDebug(7102) << "Emitting mimetype " << mime->name() << endl;
+           mimeType( mime->name() );
+           // Emit total size AFTER mimetype
+           totalSize( buff.st_size );
+       }
+
+
        data( array );
        array.clear();
 
@@ -357,6 +381,142 @@ write_all(int fd, const char *buf, size_t len)
    return 0;
 }
 
+
+void FileProtocol::open( const KUrl& url, int access )
+{
+    kDebug(7101) << "FileProtocol::open " << url.url() << endl;
+
+    QByteArray _path( QFile::encodeName(url.toLocalFile()));
+    KDE_struct_stat buff;
+    if ( KDE_stat( _path.data(), &buff ) == -1 ) {
+        if ( errno == EACCES )
+           error( KIO::ERR_ACCESS_DENIED, _path );
+        else
+           error( KIO::ERR_DOES_NOT_EXIST, _path );
+        return;
+    }
+
+    if ( S_ISDIR( buff.st_mode ) ) {
+        error( KIO::ERR_IS_DIRECTORY, _path );
+        return;
+    }
+    if ( !S_ISREG( buff.st_mode ) ) {
+        error( KIO::ERR_CANNOT_OPEN_FOR_READING, _path );
+        return;
+    }
+
+    int flags = 0;
+    int rw_access = access & 0x3;
+    if (rw_access == 1) flags = O_RDONLY;
+    else
+    if (rw_access == 2) flags = O_WRONLY;
+    else
+    if (rw_access == 3) flags = O_RDWR;
+
+    if (rw_access != 1 && (access & 0x4)) flags |= O_CREAT;
+
+    int fd = KDE_open( _path.data(), flags);
+    if ( fd < 0 ) {
+        error( KIO::ERR_CANNOT_OPEN_FOR_READING, _path );
+        return;
+    }
+    // Determine the mimetype of the file to be retrieved, and emit it.
+    // This is mandatory in all slaves (for KRun/BrowserRun to work).
+    KMimeType::Ptr mt = KMimeType::findByUrl( url, buff.st_mode, true /* local URL */ );
+    emit mimeType( mt->name() );
+
+    totalSize( buff.st_size );
+    position( 0 );
+
+    emit opened();
+
+    QByteArray array;
+
+    // Command-loop:
+    int cmd = CMD_NONE;
+    while (true) {
+        QByteArray args;
+        int stat = appconn->read(&cmd, args);
+        if ( stat == -1 )
+        {   // error
+            kDebug( 7101 ) << "File::open -- connection error" << endl;
+            break;
+        }
+        QDataStream stream( args );
+        switch( cmd ) {
+        case CMD_READ: {
+            kDebug( 7101 ) << "File::open -- read" << endl;
+            int bytes;
+            stream >> bytes;
+            QVarLengthArray<char> buffer(bytes);
+        read_retry:
+            int res = ::read(fd, buffer.data(), bytes);
+            if (res >= 0) {
+                array = array.fromRawData(buffer.data(), res);
+                data( array );
+                array.clear();
+                continue;
+            } else {
+                if (errno == EINTR) goto read_retry;
+                // empty array designates eof
+                data(QByteArray());
+                error( KIO::ERR_COULD_NOT_READ, _path );
+            }
+            break;
+        }
+        case CMD_WRITE: {
+            kDebug( 7101 ) << "File::open -- write" << endl;
+            QByteArray &buffer = args;
+
+            if (write_all( fd, buffer.data(), buffer.size()))
+            {
+                if ( errno == ENOSPC ) // disk full
+                {
+                    error( KIO::ERR_DISK_FULL, _path );
+                    break;
+                }
+                else
+                {
+                    kWarning(7101) << "Couldn't write. Error:" << strerror(errno) << endl;
+                    error( KIO::ERR_COULD_NOT_WRITE, _path );
+                    break;
+                }
+            } else {
+                written(buffer.size());
+            }
+            continue;
+        }
+        case CMD_SEEK: {
+            kDebug( 7101 ) << "File::open -- seek" << endl;
+            qulonglong offset;
+            stream >> offset;
+            int res = KDE_lseek(fd, offset, SEEK_SET);
+            if (res != -1) {
+                position( offset );
+            } else {
+                error(KIO::ERR_COULD_NOT_SEEK, _path );
+                break;
+            }
+            continue;
+        }
+        case CMD_NONE:
+            kDebug( 7101 ) << "File::open -- none " << endl;
+            continue;
+        case CMD_CLOSE:
+            kDebug( 7101 ) << "File::open -- close " << endl;
+            break;
+        default:
+            kDebug( 7101 ) << "File::open -- invalid command: " << cmd << endl;
+            cmd = CMD_CLOSE;
+            break;
+        }
+        break;
+    }
+    kDebug( 7101 ) << "File::open -- done " << endl;
+    ::close( fd );
+    finished();
+}
+
 static bool
 same_inode(const KDE_struct_stat &src, const KDE_struct_stat &dest)
 {
@@ -369,7 +529,7 @@ same_inode(const KDE_struct_stat &src, const KDE_struct_stat &dest)
 
 void FileProtocol::put( const KUrl& url, int _mode, bool _overwrite, bool _resume )
 {
-    QString dest_orig = url.path();
+    QString dest_orig = url.toLocalFile();
     QByteArray _dest_orig( QFile::encodeName(dest_orig));
 
     kDebug(7101) << "put(): " << dest_orig << ", mode=" << _mode << endl;
@@ -588,8 +748,8 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
 {
     kDebug(7101) << "copy(): " << src << " -> " << dest << ", mode=" << _mode << endl;
 
-    QByteArray _src( QFile::encodeName(src.path()));
-    QByteArray _dest( QFile::encodeName(dest.path()));
+    QByteArray _src( QFile::encodeName(src.toLocalFile()));
+    QByteArray _dest( QFile::encodeName(dest.toLocalFile()));
     KDE_struct_stat buff_src;
 #ifdef HAVE_POSIX_ACL
     acl_t acl;
@@ -597,9 +757,9 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
 
     if ( KDE_stat( _src.data(), &buff_src ) == -1 ) {
         if ( errno == EACCES )
-           error( KIO::ERR_ACCESS_DENIED, src.path() );
+           error( KIO::ERR_ACCESS_DENIED, _src );
         else
-           error( KIO::ERR_DOES_NOT_EXIST, src.path() );
+           error( KIO::ERR_DOES_NOT_EXIST, _src );
 	return;
     }
 
@@ -618,19 +778,19 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
     {
         if (S_ISDIR(buff_dest.st_mode))
         {
-           error( KIO::ERR_DIR_ALREADY_EXIST, dest.path() );
+           error( KIO::ERR_DIR_ALREADY_EXIST, _dest );
            return;
         }
 
-	if ( same_inode( buff_dest, buff_src) ) 
+	if ( same_inode( buff_dest, buff_src) )
 	{
-	    error( KIO::ERR_IDENTICAL_FILES, dest.path() );
+	    error( KIO::ERR_IDENTICAL_FILES, _dest );
 	    return;
 	}
 
         if (!_overwrite)
         {
-           error( KIO::ERR_FILE_ALREADY_EXIST, dest.path() );
+           error( KIO::ERR_FILE_ALREADY_EXIST, _dest );
            return;
         }
 
@@ -646,7 +806,7 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
 
     int src_fd = KDE_open( _src.data(), O_RDONLY);
     if ( src_fd < 0 ) {
-	error( KIO::ERR_CANNOT_OPEN_FOR_READING, src.path() );
+	error( KIO::ERR_CANNOT_OPEN_FOR_READING, _src );
 	return;
     }
 
@@ -665,9 +825,9 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
     if ( dest_fd < 0 ) {
 	kDebug(7101) << "###### COULD NOT WRITE " << dest.url() << endl;
         if ( errno == EACCES ) {
-            error( KIO::ERR_WRITE_ACCESS_DENIED, dest.path() );
+            error( KIO::ERR_WRITE_ACCESS_DENIED, _dest );
         } else {
-            error( KIO::ERR_CANNOT_OPEN_FOR_WRITING, dest.path() );
+            error( KIO::ERR_CANNOT_OPEN_FOR_WRITING, _dest );
         }
         close(src_fd);
         return;
@@ -718,17 +878,17 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
             kDebug(7101) << "sendfile() error:" << strerror(errno) << endl;
             if ( errno == ENOSPC ) // disk full
             {
-                error( KIO::ERR_DISK_FULL, dest.path());
+                error( KIO::ERR_DISK_FULL, _dest );
                 remove( _dest.data() );
             }
             else {
                 error( KIO::ERR_SLAVE_DEFINED,
                         i18n("Cannot copy file from %1 to %2. (Errno: %3)",
-                          src.path() ,  dest.path() ,  errno ) );
+                          src.toLocalFile(), dest.toLocalFile(), errno ) );
             }
           } else
 #endif
-          error( KIO::ERR_COULD_NOT_READ, src.path());
+          error( KIO::ERR_COULD_NOT_READ, _src );
           close(src_fd);
           close(dest_fd);
 #ifdef HAVE_POSIX_ACL
@@ -748,13 +908,13 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
 
            if ( errno == ENOSPC ) // disk full
            {
-              error( KIO::ERR_DISK_FULL, dest.path());
+              error( KIO::ERR_DISK_FULL, _dest );
               remove( _dest.data() );
            }
            else
            {
               kWarning(7101) << "Couldn't write[2]. Error:" << strerror(errno) << endl;
-              error( KIO::ERR_COULD_NOT_WRITE, dest.path());
+              error( KIO::ERR_COULD_NOT_WRITE, _dest );
            }
 #ifdef HAVE_POSIX_ACL
            if (acl) acl_free(acl);
@@ -773,7 +933,7 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
     if (close( dest_fd))
     {
         kWarning(7101) << "Error when closing file descriptor[2]:" << strerror(errno) << endl;
-        error( KIO::ERR_COULD_NOT_WRITE, dest.path());
+        error( KIO::ERR_COULD_NOT_WRITE, _dest );
 #ifdef HAVE_POSIX_ACL
         if (acl) acl_free(acl);
 #endif
@@ -791,7 +951,7 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
        {
         // Eat the error if the filesystem apparently doesn't support chmod.
         if ( KIO::testFileSystemFlag( _dest, KIO::SupportsChmod ) )
-            warning( i18n( "Could not change permissions for\n%1" ,  dest.path() ) );
+            warning( i18n( "Could not change permissions for\n%1" ,  dest.toLocalFile() ) );
        }
     }
 #ifdef HAVE_POSIX_ACL
@@ -804,7 +964,7 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
     ut.modtime = buff_src.st_mtime;
     if ( ::utime( _dest.data(), &ut ) != 0 )
     {
-        kWarning() << QString::fromLatin1("Couldn't preserve access and modification time for\n%1").arg( dest.path() ) << endl;
+        kWarning() << QString::fromLatin1("Couldn't preserve access and modification time for\n%1").arg( _dest.data() ) << endl;
     }
 
     processedSize( buff_src.st_size );
@@ -814,14 +974,14 @@ void FileProtocol::copy( const KUrl &src, const KUrl &dest,
 void FileProtocol::rename( const KUrl &src, const KUrl &dest,
                            bool _overwrite )
 {
-    QByteArray _src( QFile::encodeName(src.path()));
-    QByteArray _dest( QFile::encodeName(dest.path()));
+    QByteArray _src( QFile::encodeName(src.toLocalFile()));
+    QByteArray _dest( QFile::encodeName(dest.toLocalFile()));
     KDE_struct_stat buff_src;
     if ( KDE_lstat( _src.data(), &buff_src ) == -1 ) {
         if ( errno == EACCES )
-           error( KIO::ERR_ACCESS_DENIED, src.path() );
+           error( KIO::ERR_ACCESS_DENIED, _src );
         else
-           error( KIO::ERR_DOES_NOT_EXIST, src.path() );
+           error( KIO::ERR_DOES_NOT_EXIST, _src );
         return;
     }
 
@@ -831,19 +991,19 @@ void FileProtocol::rename( const KUrl &src, const KUrl &dest,
     {
         if (S_ISDIR(buff_dest.st_mode))
         {
-           error( KIO::ERR_DIR_ALREADY_EXIST, dest.path() );
+           error( KIO::ERR_DIR_ALREADY_EXIST, _dest );
            return;
         }
 
-	if ( same_inode( buff_dest, buff_src) ) 
+	if ( same_inode( buff_dest, buff_src) )
 	{
-	    error( KIO::ERR_IDENTICAL_FILES, dest.path() );
+	    error( KIO::ERR_IDENTICAL_FILES, _dest );
 	    return;
 	}
 
         if (!_overwrite)
         {
-           error( KIO::ERR_FILE_ALREADY_EXIST, dest.path() );
+           error( KIO::ERR_FILE_ALREADY_EXIST, _dest );
            return;
         }
     }
@@ -851,16 +1011,16 @@ void FileProtocol::rename( const KUrl &src, const KUrl &dest,
     if ( ::rename( _src.data(), _dest.data()))
     {
         if (( errno == EACCES ) || (errno == EPERM)) {
-            error( KIO::ERR_ACCESS_DENIED, dest.path() );
+            error( KIO::ERR_ACCESS_DENIED, _dest );
         }
         else if (errno == EXDEV) {
            error( KIO::ERR_UNSUPPORTED_ACTION, QLatin1String("rename"));
         }
         else if (errno == EROFS) { // The file is on a read-only filesystem
-           error( KIO::ERR_CANNOT_DELETE, src.path() );
+           error( KIO::ERR_CANNOT_DELETE, _src );
         }
         else {
-           error( KIO::ERR_CANNOT_RENAME, src.path() );
+           error( KIO::ERR_CANNOT_RENAME, _src );
         }
         return;
     }
@@ -910,7 +1070,7 @@ void FileProtocol::symlink( const QString &target, const KUrl &dest, bool overwr
 
 void FileProtocol::del( const KUrl& url, bool isfile)
 {
-    QByteArray _path( QFile::encodeName(url.path()));
+    QByteArray _path( QFile::encodeName(url.toLocalFile()));
     /*****
      * Delete files
      *****/
@@ -922,11 +1082,11 @@ void FileProtocol::del( const KUrl& url, bool isfile)
 
 	if ( unlink( _path.data() ) == -1 ) {
             if ((errno == EACCES) || (errno == EPERM))
-               error( KIO::ERR_ACCESS_DENIED, url.path());
+               error( KIO::ERR_ACCESS_DENIED, _path );
             else if (errno == EISDIR)
-               error( KIO::ERR_IS_DIRECTORY, url.path());
+               error( KIO::ERR_IS_DIRECTORY, _path );
             else
-               error( KIO::ERR_CANNOT_DELETE, url.path() );
+               error( KIO::ERR_CANNOT_DELETE, _path );
 	    return;
 	}
     } else {
@@ -939,10 +1099,10 @@ void FileProtocol::del( const KUrl& url, bool isfile)
 
       if ( ::rmdir( _path.data() ) == -1 ) {
 	if ((errno == EACCES) || (errno == EPERM))
-	  error( KIO::ERR_ACCESS_DENIED, url.path());
+	  error( KIO::ERR_ACCESS_DENIED, _path );
 	else {
 	  kDebug( 7101 ) << "could not rmdir " << perror << endl;
-	  error( KIO::ERR_COULD_NOT_RMDIR, url.path() );
+	  error( KIO::ERR_COULD_NOT_RMDIR, _path );
 	  return;
 	}
       }
@@ -1072,8 +1232,11 @@ void FileProtocol::stat( const KUrl & url )
      * stat("/is/unaccessible/") -> EPERM            H.Z.
      * This is the reason for the -1
      */
+#ifdef Q_WS_WIN
+    QByteArray _path( QFile::encodeName(url.toLocalFile()) );
+#else
     QByteArray _path( QFile::encodeName(url.path(KUrl::RemoveTrailingSlash)));
-
+#endif
     QString sDetails = metaData(QLatin1String("details"));
     int details = sDetails.isEmpty() ? 2 : sDetails.toInt();
     kDebug(7101) << "FileProtocol::stat details=" << details << endl;
@@ -1081,7 +1244,7 @@ void FileProtocol::stat( const KUrl & url )
     UDSEntry entry;
     if ( !createUDSEntry( url.fileName(), _path, entry, details, true /*with acls*/ ) )
     {
-        error( KIO::ERR_DOES_NOT_EXIST, url.path(KUrl::RemoveTrailingSlash) );
+        error( KIO::ERR_DOES_NOT_EXIST, _path );
         return;
     }
 #if 0
@@ -1108,17 +1271,47 @@ void FileProtocol::listDir( const KUrl& url)
 	finished();
 	return;
     }
+    QByteArray _path( QFile::encodeName(url.toLocalFile()) );
+#ifdef Q_WS_WIN
+    QDir dir( _path );
+    dir.setFilter(QDir::AllEntries);
+    dir.setSorting(QDir::Size | QDir::Reversed);
 
-    QByteArray _path( QFile::encodeName(url.path()));
+    if ( !dir.exists() ) {
+        kDebug(7101) << "========= ERR_DOES_NOT_EXIST  =========" << endl;
+        error( KIO::ERR_DOES_NOT_EXIST, _path );
+        return;
+    }
 
+   	// don't knwo how to check
+   	// error( KIO::ERR_IS_FILE, url.path() );
+
+    if ( !dir.isReadable() ) {
+        kDebug(7101) << "========= ERR_CANNOT_ENTER_DIRECTORY =========" << endl;
+        error( KIO::ERR_CANNOT_ENTER_DIRECTORY, _path );
+        return;
+    }
+    QFileInfoList list = dir.entryInfoList();
+    totalSize( list.size() );
+
+    UDSEntry entry;
+    for (int i = 0; i < list.size(); ++i) {
+        QFileInfo fileInfo = list.at(i);
+        entry.clear();
+        if ( createUDSEntry( fileInfo.fileName(),
+                             fileInfo.absoluteFilePath().toAscii() /* we can use the filename as relative path*/,
+                             entry, 2, true ) )
+            listEntry( entry, false);
+    }
+#else
     KDE_struct_stat buff;
     if ( KDE_stat( _path.data(), &buff ) == -1 ) {
-	error( KIO::ERR_DOES_NOT_EXIST, url.path() );
+	error( KIO::ERR_DOES_NOT_EXIST, _path );
 	return;
     }
 
     if ( !S_ISDIR( buff.st_mode ) ) {
-	error( KIO::ERR_IS_FILE, url.path() );
+	error( KIO::ERR_IS_FILE, _path );
 	return;
     }
 
@@ -1132,11 +1325,13 @@ void FileProtocol::listDir( const KUrl& url)
 #ifdef ENOMEDIUM
 	case ENOMEDIUM:
             error( ERR_SLAVE_DEFINED,
-                   i18n( "No media in device for %1" ,  url.path() ) );
+                   i18n( "No media in device for %1", url.toLocalFile() ) );
             break;
+#else
+        case ENOENT: // just to avoid the warning
 #endif
         default:
-            error( KIO::ERR_CANNOT_ENTER_DIRECTORY, url.path() );
+            error( KIO::ERR_CANNOT_ENTER_DIRECTORY, _path );
             break;
         }
 	return;
@@ -1146,7 +1341,6 @@ void FileProtocol::listDir( const KUrl& url)
     // should be passed intact to createUDSEntry to avoid problems with
     // files where QFile::encodeName(QFile::decodeName(a)) != a.
     QList<QByteArray> entryNames;
-
     while ( ( ep = KDE_readdir( dp ) ) != 0L )
 	entryNames.append( ep->d_name );
 
@@ -1180,13 +1374,15 @@ void FileProtocol::listDir( const KUrl& url)
                              entry, 2, true ) )
           listEntry( entry, false);
     }
+#endif
 
     listEntry( entry, true ); // ready
 
     kDebug(7101) << "============= COMPLETED LIST ============" << endl;
 
+#ifndef Q_WS_WIN
     chdir(path_buffer);
-
+#endif
     finished();
 }
 
@@ -1299,8 +1495,10 @@ void FileProtocol::mount( bool _ro, const char *_fstype, const QString& _dev, co
 #else
 
 
-    KTempFile tmpFile;
-    QByteArray tmpFileName = QFile::encodeName(tmpFile.name());
+    KTemporaryFile tmpFile;
+    tmpFile.setAutoRemove(false);
+    tmpFile.open();
+    QByteArray tmpFileName = QFile::encodeName(tmpFile.fileName());
     QByteArray dev;
     if ( _dev.startsWith( "LABEL=" ) ) { // turn LABEL=foo into -L foo (#71430)
         QString labelName = _dev.mid( 6 );
@@ -1408,8 +1606,10 @@ void FileProtocol::unmount( const QString& _point )
 {
     QByteArray buffer;
 
-    KTempFile tmpFile;
-    QByteArray tmpFileName = QFile::encodeName(tmpFile.name());
+    KTemporaryFile tmpFile;
+    tmpFile.setAutoRemove(false);
+    tmpFile.open();
+    QByteArray tmpFileName = QFile::encodeName(tmpFile.fileName());
     QString err;
 
 #ifdef HAVE_VOLMGT
@@ -1510,7 +1710,7 @@ void FileProtocol::unmount( const QString& _point )
 
     err = testLogFile( tmpFileName );
     if ( err.isEmpty() )
-	finished();
+        finished();
     else
         error( KIO::ERR_COULD_NOT_UNMOUNT, err );
 }
