@@ -16,9 +16,16 @@
     along with this library; see the file COPYING.LIB.  If not, write to
     the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
     Boston, MA 02110-1301, USA.
+*/
 
+/*
     Recommended reading explaining FTP details and quirks:
       http://cr.yp.to/ftp.html  (by D.J. Bernstein)
+
+    RFC:
+      RFC  959 "File Transfer Protocol (FTP)"
+      RFC 1635 "How to Use Anonymous FTP"
+      RFC 2428 "FTP Extensions for IPv6 and NATs" (defines EPRT and EPSV)
 */
 
 
@@ -55,8 +62,7 @@
 #include <kdebug.h>
 #include <kglobal.h>
 #include <klocale.h>
-#include <kinstance.h>
-#include <kmimemagic.h>
+#include <kcomponentdata.h>
 #include <kmimetype.h>
 #include <kio/ioslave_defaults.h>
 #include <kio/slaveconfig.h>
@@ -133,7 +139,7 @@ using namespace KNetwork;
 extern "C" int KDE_EXPORT kdemain( int argc, char **argv )
 {
   KLocale::setMainCatalog("kdelibs");
-  KInstance instance( "kio_ftp" );
+  KComponentData componentData( "kio_ftp" );
   ( void ) KGlobal::locale();
 
   kDebug(7102) << "Starting " << getpid() << endl;
@@ -244,8 +250,7 @@ void Ftp::ftpCloseControlConnection()
 const char* Ftp::ftpResponse(int iOffset)
 {
   assert(m_control != NULL);    // must have control connection socket
-  QByteArray data = m_control->readLine();
-  const char *pTxt = data.data();
+  const char *pTxt = m_lastControlLine.data();
 
   // read the next line ...
   if(iOffset < 0)
@@ -258,7 +263,9 @@ const char* Ftp::ftpResponse(int iOffset)
     // be stored. Some servers (OpenBSD) send a single "nnn-" followed by
     // optional lines that start with a space and a final "nnn text" line.
     do {
-      int nBytes = data.size();
+      m_lastControlLine = m_control->readLine();
+      pTxt = m_lastControlLine.data();
+      int nBytes = m_lastControlLine.size();
       int iCode  = atoi(pTxt);
       if(iCode > 0) m_iRespCode = iCode;
 
@@ -397,10 +404,8 @@ bool Ftp::ftpOpenControlConnection( const QString &host, int port )
   closeConnection();
   QString sErrorMsg;
   m_control = new KStreamSocket;
+  m_control->setBlocking(true);
 
-#ifdef __GNUC__
-#warning "FIXME KDE4: reenable 'host not found' error"
-#endif
   // now connect to the server and read the login message ...
   m_control->setTimeout(connectTimeout() * 1000);
   int iErrorCode = m_control->connect(host, serv) ? 0 : ERR_COULD_NOT_CONNECT;
@@ -418,7 +423,10 @@ bool Ftp::ftpOpenControlConnection( const QString &host, int port )
   }
   else
   {
-      sErrorMsg = QString("%1: %2").arg(host).arg(m_control->errorString());
+    if (m_control->error() == KSocketBase::LookupFailure)
+      iErrorCode = ERR_UNKNOWN_HOST;
+
+    sErrorMsg = QString("%1: %2").arg(host).arg(m_control->errorString());
   }
 
   // if there was a problem - report it ...
@@ -501,7 +509,7 @@ bool Ftp::ftpLogin()
       info.readOnly = (!m_user.isEmpty() && m_user != FTP_LOGIN);
 
       bool disablePassDlg = config()->readEntry( "DisablePassDlg", false );
-      if ( disablePassDlg || !openPassDlg( info, errorMsg ) )
+      if ( disablePassDlg || !openPasswordDialog( info, errorMsg ) )
       {
         error( ERR_USER_CANCELED, m_host );
         return false;
@@ -667,7 +675,7 @@ bool Ftp::ftpSendCmd( const QByteArray& cmd, int maxretries )
   // Send the message...
   QByteArray buf = cmd;
   buf += "\r\n";      // Yes, must use CR/LF - see http://cr.yp.to/ftp/request.html
-  int num = m_control->write(buf.data(), buf.length());
+  int num = m_control->write(buf);
 
   // If we were able to successfully send the command, then we will
   // attempt to read the response. Otherwise, take action to re-attempt
@@ -740,7 +748,7 @@ bool Ftp::ftpSendCmd( const QByteArray& cmd, int maxretries )
 /*
  * ftpOpenPASVDataConnection - set up data connection, using PASV mode
  *
- * return 1 if successful, 0 otherwise
+ * return 0 if successful, ERR_INTERNAL otherwise
  * doesn't set error message, since non-pasv mode will always be tried if
  * this one fails
  */
@@ -787,16 +795,17 @@ int Ftp::ftpOpenPASVDataConnection()
   }
 
   // Make hostname and port number ...
-  KInetSocketAddress address((KIpAddress((i[0] << 24) | (i[1] << 16) |
-                                         (i[2] << 8) | i[3]) ), i[4] << 8 | i[5]);
+  KInetSocketAddress address((KIpAddress((i[3] << 24) | (i[2] << 16) |
+                                         (i[1] << 8) | i[0]) ), i[4] << 8 | i[5]);
 
   // now connect the data socket ...
   m_data = new KStreamSocket("PASV");
+  m_data->setBlocking(true);
   m_data->setResolutionEnabled(false);
   m_data->setTimeout(connectTimeout() * 1000);
 
   kDebug(7102) << "Connecting to " << address.toString() << endl;
-  return m_data->connect(address.nodeName(), address.serviceName());
+  return m_data->connect(address.nodeName(), address.serviceName()) ? 0 : ERR_INTERNAL;
 }
 
 /*
@@ -905,6 +914,7 @@ int Ftp::ftpOpenPortDataConnection()
     return ERR_INTERNAL;
 
   KServerSocket server;
+  server.setBlocking(true);
   server.setAcceptBuffered(false);
   server.setResolutionEnabled(false);
   server.setFamily(KResolver::InternetFamily);
@@ -912,7 +922,7 @@ int Ftp::ftpOpenPortDataConnection()
 
   {
     // yes, we are sure this is a KInetSocketAddress
-    const KInetSocketAddress &sin = m_control->localAddress().asInet();
+    KInetSocketAddress sin = m_control->localAddress().asInet();
 
     // setting port to 0 will make us bind to a random, free port
     server.bind(sin.nodeName(), QLatin1String("0"));
@@ -921,12 +931,11 @@ int Ftp::ftpOpenPortDataConnection()
   if (!server.listen(1))
     return ERR_COULD_NOT_LISTEN;
 
-  const KInetSocketAddress &sin = server.localAddress().asInet();
+  KInetSocketAddress sin = server.localAddress().asInet();
 
   QString command;
   if (sin.ianaFamily() == 1)
   {
-    command = QLatin1String("PORT %1,%2,%3,%4,%5,%6");
     struct
     {
       quint32 ip4;
@@ -935,16 +944,12 @@ int Ftp::ftpOpenPortDataConnection()
     data.ip4 = sin.ipAddress().IPv4Addr();
     data.port = sin.port();
 
-    char *pData = reinterpret_cast<char*>(&data);
-    command.arg(pData[0]).arg(pData[1]).arg(pData[2]).arg(pData[3])
-      .arg(pData[4]).arg(pData[5]);
+    unsigned char *pData = reinterpret_cast<unsigned char*>(&data);
+    command.sprintf("PORT %d,%d,%d,%d,%d,%d",pData[0],pData[1],pData[2],pData[3],pData[4],pData[5]);
   }
   else
   {
-    command = QLatin1String("EPRT |%1|%2|%3|");
-    command.arg(sin.ianaFamily())
-      .arg(sin.nodeName())
-      .arg(sin.port());
+    command = QString("EPRT |%1|%2|%3|").arg(sin.ianaFamily()).arg(sin.nodeName()).arg(sin.port());
   }
 
   if( ftpSendCmd(command.toLatin1()) && (m_iRespType == 2) )
@@ -1029,7 +1034,7 @@ bool Ftp::ftpCloseCommand()
   kDebug(7102) << "ftpCloseCommand: reading command result" << endl;
   m_bBusy = false;
 
-  if(ftpResponse(-1) <= 0 || (m_iRespType != 2) )
+  if(!ftpResponse(-1) || (m_iRespType != 2) )
   {
     kDebug(7102) << "ftpCloseCommand: no transfer complete message" << endl;
     return false;
@@ -1179,7 +1184,7 @@ void Ftp::ftpCreateUDSEntry( const QString & filename, FtpEntry& ftpEnt, UDSEntr
   {
     entry.insert( UDS_LINK_DEST, ftpEnt.link );
 
-    KMimeType::Ptr mime = KMimeType::findByURL( KUrl("ftp://host/" + filename ) );
+    KMimeType::Ptr mime = KMimeType::findByUrl( KUrl("ftp://host/" + filename ) );
     // Links on ftp sites are often links to dirs, and we have no way to check
     // that. Let's do like Netscape : assume dirs generally.
     // But we do this only when the mimetype can't be known from the filename.
@@ -1822,11 +1827,11 @@ Ftp::StatusCode Ftp::ftpGet(int& iError, int iCopyFile, const KUrl& url, KIO::fi
     if(!mimetypeEmitted)
     {
       mimetypeEmitted = true;
-      array = array.fromRawData(buffer, n);
-      KMimeMagicResult * result = KMimeMagic::self()->findBufferFileType(array, url.fileName());
+      array = QByteArray::fromRawData(buffer, n);
+      KMimeType::Ptr mime = KMimeType::findByNameAndContent(url.fileName(), array);
       array.clear();
-      kDebug(7102) << "ftpGet: Emitting mimetype " << result->mimeType() << endl;
-      mimeType( result->mimeType() );
+      kDebug(7102) << "ftpGet: Emitting mimetype " << mime->name() << endl;
+      mimeType( mime->name() );
       if( m_size != UnknownSize )	// Emit total size AFTER mimetype
         totalSize( m_size );
     }
@@ -1834,9 +1839,9 @@ Ftp::StatusCode Ftp::ftpGet(int& iError, int iCopyFile, const KUrl& url, KIO::fi
     // write output file or pass to data pump ...
     if(iCopyFile == -1)
     {
-       array = array.fromRawData(buffer, n);
-       data( array );
-       array.clear();
+        array = QByteArray::fromRawData(buffer, n);
+        data( array );
+        array.clear();
     }
     else if( (iError = WriteToFile(iCopyFile, buffer, n)) != 0)
        return statusClientError;              // client side error
@@ -2059,7 +2064,7 @@ Ftp::StatusCode Ftp::ftpPut(int& iError, int iCopyFile, const KUrl& dest_url,
 
     if (result > 0)
     {
-      m_data->write( buffer.data(), buffer.size() );
+      m_data->write( buffer );
       processed_size += result;
       processedSize (processed_size);
     }
@@ -2210,14 +2215,14 @@ void Ftp::copy( const KUrl &src, const KUrl &dest, int permissions, bool overwri
 
   if(bSrcLocal && !bDestLocal)                    // File -> Ftp
   {
-    sCopyFile = src.path();
+    sCopyFile = src.toLocalFile();
     kDebug(7102) << "Ftp::copy local file '" << sCopyFile << "' -> ftp '" << dest.path() << "'" << endl;
     cs = ftpCopyPut(iError, iCopyFile, sCopyFile, dest, permissions, overwrite);
     if( cs == statusServerError) sCopyFile = dest.url();
   }
   else if(!bSrcLocal && bDestLocal)               // Ftp -> File
   {
-    sCopyFile = dest.path();
+    sCopyFile = dest.toLocalFile();
     kDebug(7102) << "Ftp::copy ftp '" << src.path() << "' -> local file '" << sCopyFile << "'" << endl;
     cs = ftpCopyGet(iError, iCopyFile, sCopyFile, src, permissions, overwrite);
     if( cs == statusServerError ) sCopyFile = src.url();
