@@ -58,9 +58,11 @@
 
 #if PLATFORM(WIN_OS)
 #define copysign(x, y) _copysign(x, y)
+#define snprintf _snprintf
+#if !COMPILER(GCC)
 #define isfinite(x) _finite(x)
 #define strncasecmp(x, y, z) strnicmp(x, y, z)
-#define snprintf _snprintf
+#endif
 #endif
 
 inline int gmtoffset(const tm& t)
@@ -284,7 +286,7 @@ static void fillStructuresUsingTimeArgs(ExecState *exec, const List &args, int m
     double milliseconds = 0;
     int idx = 0;
     int numArgs = args.size();
-    
+
     // JS allows extra trailing arguments -- ignore them
     if (numArgs > maxArgs)
         numArgs = maxArgs;
@@ -300,20 +302,20 @@ static void fillStructuresUsingTimeArgs(ExecState *exec, const List &args, int m
         t->tm_min = 0;
         milliseconds += args[idx++]->toInt32(exec) * msPerMinute;
     }
-    
+
     // seconds
     if (maxArgs >= 2 && idx < numArgs) {
         t->tm_sec = 0;
         milliseconds += args[idx++]->toInt32(exec) * msPerSecond;
     }
-    
+
     // milliseconds
     if (idx < numArgs) {
         milliseconds += roundValue(exec, args[idx]);
     } else {
         milliseconds += *ms;
     }
-    
+
     *ms = milliseconds;
 }
 
@@ -325,19 +327,19 @@ static void fillStructuresUsingDateArgs(ExecState *exec, const List &args, int m
 {
     int idx = 0;
     int numArgs = args.size();
-  
+
     // JS allows extra trailing arguments -- ignore them
     if (numArgs > maxArgs)
         numArgs = maxArgs;
-  
+
     // years
     if (maxArgs >= 3 && idx < numArgs)
         t->tm_year = args[idx++]->toInt32(exec) - 1900;
-  
+
     // months
     if (maxArgs >= 2 && idx < numArgs)
         t->tm_mon = args[idx++]->toInt32(exec);
-  
+
     // days
     if (idx < numArgs) {
         t->tm_mday = 0;
@@ -359,7 +361,7 @@ bool DateInstance::getTime(tm &t, int &offset) const
     double milli = internalValue()->getNumber();
     if (isNaN(milli))
         return false;
-    
+
     millisecondsToTM(milli, false, &t);
     offset = gmtoffset(t);
     return true;
@@ -370,7 +372,7 @@ bool DateInstance::getUTCTime(tm &t) const
     double milli = internalValue()->getNumber();
     if (isNaN(milli))
         return false;
-    
+
     millisecondsToTM(milli, true, &t);
     return true;
 }
@@ -380,7 +382,7 @@ bool DateInstance::getTime(double &milli, int &offset) const
     milli = internalValue()->getNumber();
     if (isNaN(milli))
         return false;
-    
+
     tm t;
     millisecondsToTM(milli, false, &t);
     offset = gmtoffset(t);
@@ -392,7 +394,7 @@ bool DateInstance::getUTCTime(double &milli) const
     milli = internalValue()->getNumber();
     if (isNaN(milli))
         return false;
-    
+
     return true;
 }
 
@@ -436,7 +438,7 @@ static void millisecondsToTM(double milli, bool utc, tm *t)
       m += gmtoffset(*t) * msPerSecond;
     t->tm_wday = weekDay(m);
   }
-}    
+}
 
 
 // ------------------------------ DatePrototype -----------------------------
@@ -522,7 +524,7 @@ JSValue *DateProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const
   if (!thisObj->inherits(&DateInstance::info))
     return throwError(exec, TypeError);
 
-  DateInstance* thisDateObj = static_cast<DateInstance*>(thisObj); 
+  DateInstance* thisDateObj = static_cast<DateInstance*>(thisObj);
 
   JSValue *result = 0;
   UString s;
@@ -559,10 +561,24 @@ JSValue *DateProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const
       case GetSeconds:
       case GetMilliSeconds:
       case GetTimezoneOffset:
+      case SetMilliSeconds:
+      case SetSeconds:
+      case SetMinutes:
+      case SetHours:
+      case SetDate:
+      case SetMonth:
+      case SetFullYear:
         return jsNaN();
     }
   }
   
+  if (id == SetTime) {
+    double milli = roundValue(exec, args[0]);
+    result = jsNumber(milli);
+    thisDateObj->setInternalValue(result);
+    return result;
+  }
+
   double secs = floor(milli / msPerSecond);
   double ms = milli - secs * msPerSecond;
 
@@ -632,11 +648,6 @@ JSValue *DateProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const
     return jsNumber(ms);
   case GetTimezoneOffset:
     return jsNumber(-gmtoffset(t) / 60);
-  case SetTime:
-    milli = roundValue(exec, args[0]);
-    result = jsNumber(milli);
-    thisDateObj->setInternalValue(result);
-    break;
   case SetMilliSeconds:
     fillStructuresUsingTimeArgs(exec, args, 1, &ms, &t);
     break;
@@ -653,13 +664,13 @@ JSValue *DateProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const
     fillStructuresUsingDateArgs(exec, args, 1, &ms, &t);
     break;
   case SetMonth:
-    fillStructuresUsingDateArgs(exec, args, 2, &ms, &t);    
+    fillStructuresUsingDateArgs(exec, args, 2, &ms, &t);
     break;
   case SetFullYear:
     fillStructuresUsingDateArgs(exec, args, 3, &ms, &t);
     break;
   case SetYear:
-    t.tm_year = args[0]->toInt32(exec) >= 1900 ? args[0]->toInt32(exec) - 1900 : args[0]->toInt32(exec);
+    t.tm_year = (args[0]->toInt32(exec) > 99 || args[0]->toInt32(exec) < 0) ? args[0]->toInt32(exec) - 1900 : args[0]->toInt32(exec);
     break;
   }
 
@@ -669,14 +680,12 @@ JSValue *DateProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj, const
     result = jsNumber(makeTime(&t, ms, utc));
     thisDateObj->setInternalValue(result);
   }
-  
+
   return result;
 }
 
 // ------------------------------ DateObjectImp --------------------------------
 
-// TODO: MakeTime (15.9.11.1) etc. ?
-
 DateObjectImp::DateObjectImp(ExecState *exec,
                              FunctionPrototype *funcProto,
                              DatePrototype *dateProto)
@@ -756,7 +765,7 @@ JSObject *DateObjectImp::construct(ExecState *exec, const List &args)
       value = makeTime(&t, ms, false);
     }
   }
-  
+
   DateInstance *ret = new DateInstance(exec->lexicalInterpreter()->builtinDatePrototype());
   ret->setInternalValue(jsNumber(timeClip(value)));
   return ret;
@@ -816,11 +825,21 @@ JSValue *DateObjectFuncImp::callAsFunction(ExecState* exec, JSObject*, const Lis
 
 static inline double ymdhmsToSeconds(long year, int mon, int day, int hour, int minute, int second)
 {
+    // in which case is the floor() needed? breaks day value of
+    // "new Date('Thu Nov 5 2065 18:15:30 GMT+0500')"
+#if 0
     double days = (day - 32075)
         + floor(1461 * (year + 4800.0 + (mon - 14) / 12) / 4)
         + 367 * (mon - 2 - (mon - 14) / 12 * 12) / 12
         - floor(3 * ((year + 4900.0 + (mon - 14) / 12) / 100) / 4)
         - 2440588;
+#else
+    double days = (day - 32075)
+            + 1461 * (year + 4800 + (mon - 14) / 12) / 4
+            + 367 * (mon - 2 - (mon - 14) / 12 * 12) / 12
+            - 3 * ((year + 4900 + (mon - 14) / 12) / 100) / 4
+            - 2440588;
+#endif
     return ((days * hoursPerDay + hour) * minutesPerHour + minute) * secondsPerMinute + second;
 }
 
@@ -871,7 +890,7 @@ static double makeTime(tm *t, double ms, bool utc)
     }
 
     double yearOffset = 0.0;
-    if (t->tm_year < (1970 - 1900) || t->tm_year > (2038 - 1900)) {
+    if (t->tm_year < (1971 - 1900) || t->tm_year > (2037 - 1900)) {
         // we'll fool mktime() into believing that this year is within
         // its normal, portable range (1970-2038) by setting tm_year to
         // 2000 or 2001 and adding the difference in milliseconds later.
@@ -886,23 +905,28 @@ static double makeTime(tm *t, double ms, bool utc)
 
     // Determine whether DST is in effect. mktime() can't do this for us because
     // it doesn't know about ms and yearOffset.
-    // NOTE: Casting values of large magnitude to time_t (long) will 
+    // NOTE: Casting values of large magnitude to time_t (long) will
     // produce incorrect results, but there's no other option when calling localtime_r().
-    if (!utc) { 
-        time_t tval = mktime(t) + (time_t)((ms + yearOffset) / 1000);  
-        tm t3 = *localtime(&tval);  
-        t->tm_isdst = t3.tm_isdst;  
+    if (!utc) {
+        time_t tval = mktime(t) + (time_t)((ms + yearOffset) / 1000);
+        tm t3 = *localtime(&tval);
+        t->tm_isdst = t3.tm_isdst;
     }
 
     return (mktime(t) + utcOffset) * msPerSecond + ms + yearOffset;
 }
 
+inline static bool isSpaceLike(char c)
+{
+    return isspace(c) || c == ',' || c == ':' || c == '-';
+}
+
 inline static void skipSpacesAndComments(const char *&s)
 {
     int nesting = 0;
     char ch;
     while ((ch = *s)) {
-        if (!isspace(ch)) {
+        if (!isSpaceLike(ch)) {
             if (ch == '(')
                 nesting++;
             else if (ch == ')' && nesting > 0)
@@ -953,7 +977,7 @@ static double parseDate(const UString &date)
 
     CString dateCString = date.UTF8String();
     const char *dateString = dateCString.c_str();
-     
+
     // Skip leading space
     skipSpacesAndComments(dateString);
 
@@ -1062,7 +1086,7 @@ static double parseDate(const UString &date)
         if (errno)
             return NaN;
     }
-    
+
     // Don't fail if the time is missing.
     long hour = 0;
     long minute = 0;
@@ -1071,20 +1095,21 @@ static double parseDate(const UString &date)
         dateString = newPosStr;
     else {
         // ' 23:12:40 GMT'
-        if (!isspace(*newPosStr)) {
-            if (*newPosStr != ':')
-                return NaN;
+        if (*newPosStr == ':') {
             // There was no year; the number was the hour.
             year = -1;
-        } else {
-            // in the normal case (we parsed the year), advance to the next number
+        } else if (isSpaceLike(*newPosStr)) {
+            // in the normal case (we parsed the year), advance to the
+            // next number
             dateString = ++newPosStr;
             skipSpacesAndComments(dateString);
+        } else {
+            return NaN;
         }
 
         hour = strtol(dateString, &newPosStr, 10);
         // Do not check for errno here since we want to continue
-        // even if errno was set becasue we are still looking
+        // even if errno was set because we are still looking
         // for the timezone!
 
         // Read a number? If not, this might be a timezone name.
@@ -1121,7 +1146,7 @@ static double parseDate(const UString &date)
                 if (errno)
                     return NaN;
                 dateString = newPosStr;
-            
+
                 if (second < 0 || second > 59)
                     return NaN;
             }
@@ -1149,10 +1174,11 @@ static double parseDate(const UString &date)
     bool haveTZ = false;
     int offset = 0;
 
-    // Don't fail if the time zone is missing. 
+    // Don't fail if the time zone is missing.
     // Some websites omit the time zone (4275206).
     if (*dateString) {
-        if (strncasecmp(dateString, "GMT", 3) == 0 || strncasecmp(dateString, "UTC", 3) == 0) {
+        if (strncasecmp(dateString, "GMT", 3) == 0 ||
+            strncasecmp(dateString, "UTC", 3) == 0) {
             dateString += 3;
             haveTZ = true;
         }
@@ -1171,6 +1197,7 @@ static double parseDate(const UString &date)
             if (*dateString != ':') {
                 offset = ((o / 100) * 60 + (o % 100)) * sgn;
             } else { // GMT+05:00
+                dateString++;
                 long o2 = strtol(dateString, &newPosStr, 10);
                 if (errno)
                     return NaN;
@@ -1198,9 +1225,9 @@ static double parseDate(const UString &date)
             return NaN;
         dateString = newPosStr;
     }
-     
+
     skipSpacesAndComments(dateString);
-     
+
     // Trailing garbage
     if (*dateString)
         return NaN;
