@@ -20,7 +20,7 @@
 #include "qpainter_binding.h"
 #include "object_binding.h"
 #include "static_binding.h"
-#include "global.h"
+#include "kjseglobal.h"
 
 #include <kjs/object.h>
 #include <qdebug.h>
@@ -66,7 +66,7 @@ QPaintDevice *extractPaintDevice( KJS::ExecState *exec, KJS::JSValue *arg)
     }
     else
     {
-        ValueBinding *valueImp = extractBindingImp<ValueBinding>(exec,arg);
+        VariantBinding *valueImp = extractBindingImp<VariantBinding>(exec,arg);
         if( valueImp && (valueImp->variant().type() == QVariant::Pixmap ||
                 valueImp->variant().type() == QVariant::Image ))
         {
@@ -77,38 +77,38 @@ QPaintDevice *extractPaintDevice( KJS::ExecState *exec, KJS::JSValue *arg)
 }
 
 START_OBJECT_METHOD( callPainterBegin, QPainter )
-  result = KJS::Boolean(false);
+  result = KJS::jsBoolean(false);
   QPaintDevice *device = extractPaintDevice(exec, args[0]);
   if( device )
   {
-    result = KJS::Boolean(object->begin(device));
+    result = KJS::jsBoolean(object->begin(device));
   } else {
-    result = KJS::Boolean(false);
+    result = KJS::jsBoolean(false);
   }
 END_OBJECT_METHOD
 
 START_OBJECT_METHOD( callPainterEnd, QPainter )
-    result = KJS::Boolean(object->end());
+    result = KJS::jsBoolean(object->end());
 END_OBJECT_METHOD
 
 START_OBJECT_METHOD( callbackground, QPainter )
     QBrush cppValue = object->background();
-    result = KJSEmbed::createValue(exec, "QBrush", cppValue );
+    result = KJSEmbed::createVariant(exec, "QBrush", cppValue );
 END_OBJECT_METHOD
 
 START_OBJECT_METHOD( callbackgroundMode, QPainter )
     Qt::BGMode cppValue = object->backgroundMode();
-    result = KJS::Number(cppValue);
+    result = KJS::jsNumber(cppValue);
 END_OBJECT_METHOD
 
 START_OBJECT_METHOD( callboundingRect, QPainter )
     if( args.size() == 3 )
     {
-        QRect arg0 = KJSEmbed::extractValue<QRect>(exec,args, 0);
+        QRect arg0 = KJSEmbed::extractVariant<QRect>(exec,args, 0);
         int arg1 = KJSEmbed::extractInt(exec, args, 1);
         QString arg2 = KJSEmbed::extractQString(exec, args, 2);
         QRect cppValue = object->boundingRect(arg0, arg1, arg2);
-        result = KJSEmbed::createValue(exec, "QRect", cppValue );
+        result = KJSEmbed::createVariant(exec, "QRect", cppValue );
     }
     else if( args.size() == 6)
     {
@@ -119,24 +119,24 @@ START_OBJECT_METHOD( callboundingRect, QPainter )
         int arg4 = KJSEmbed::extractInt(exec, args, 4);
         QString arg5 = KJSEmbed::extractQString(exec, args, 5);
         QRect cppValue = object->boundingRect(arg0, arg1, arg2, arg3, arg4, arg5);
-        result = KJSEmbed::createValue(exec, "QRect", cppValue );
+        result = KJSEmbed::createVariant(exec, "QRect", cppValue );
     }
 END_OBJECT_METHOD
 
 START_OBJECT_METHOD( callbrush, QPainter )
     QBrush cppValue = object->brush();
-    result = KJSEmbed::createValue(exec, "QBrush", cppValue );
+    result = KJSEmbed::createVariant(exec, "QBrush", cppValue );
 END_OBJECT_METHOD
 
 START_OBJECT_METHOD( callbrushOrigin, QPainter )
     QPoint cppValue = object->brushOrigin();
-    result = KJSEmbed::createValue(exec, "QPoint", cppValue );
+    result = KJSEmbed::createVariant(exec, "QPoint", cppValue );
 END_OBJECT_METHOD
 
 START_OBJECT_METHOD( calldrawArc, QPainter )
     if( args.size() == 3 )
     {
-        QRect arg0 = KJSEmbed::extractValue<QRect>(exec,args, 0);
+        QRect arg0 = KJSEmbed::extractVariant<QRect>(exec,args, 0);
         int arg1 = KJSEmbed::extractInt(exec, args, 1);
         int arg2 = KJSEmbed::extractInt(exec, args, 2);
         object->drawArc(arg0, arg1, arg2);
@@ -156,7 +156,7 @@ END_OBJECT_METHOD
 START_OBJECT_METHOD( calldrawChord, QPainter )
     if( args.size() == 3 )
     {
-        QRect arg0 = KJSEmbed::extractValue<QRect>(exec,args, 0);
+        QRect arg0 = KJSEmbed::extractVariant<QRect>(exec,args, 0);
         int arg1 = KJSEmbed::extractInt(exec, args, 1);
         int arg2 = KJSEmbed::extractInt(exec, args, 2);
         object->drawChord(arg0, arg1, arg2);
@@ -174,7 +174,7 @@ START_OBJECT_METHOD( calldrawChord, QPainter )
 END_OBJECT_METHOD
 
 START_OBJECT_METHOD( calldrawConvexPolygon, QPainter )
-    QPolygon arg0 = KJSEmbed::extractValue<QPolygon>(exec,args, 0);
+    QPolygon arg0 = KJSEmbed::extractVariant<QPolygon>(exec,args, 0);
     object->drawConvexPolygon(arg0);
 END_OBJECT_METHOD
 
@@ -189,7 +189,7 @@ START_OBJECT_METHOD( calldrawEllipse, QPainter )
     }
     else if ( args.size() == 1 )
     {
-        QRect arg0 = KJSEmbed::extractValue<QRect>(exec,args, 0);
+        QRect arg0 = KJSEmbed::extractVariant<QRect>(exec,args, 0);
         object->drawEllipse(arg0);
     }
 END_OBJECT_METHOD
@@ -197,15 +197,15 @@ END_OBJECT_METHOD
 START_OBJECT_METHOD( calldrawImage, QPainter )
     if ( args.size() == 2 )
     {
-        QPoint arg0 = KJSEmbed::extractValue<QPoint>(exec,args, 0);
-        QImage arg1 = KJSEmbed::extractValue<QImage>(exec,args, 1);
+        QPoint arg0 = KJSEmbed::extractVariant<QPoint>(exec,args, 0);
+        QImage arg1 = KJSEmbed::extractVariant<QImage>(exec,args, 1);
         object->drawImage(arg0, arg1);
     }
     else if ( args.size() == 4 )
     {
-        QPoint arg0 = KJSEmbed::extractValue<QPoint>(exec,args, 0);
-        QImage arg1 = KJSEmbed::extractValue<QImage>(exec,args, 1);
-        QRect arg2 = KJSEmbed::extractValue<QRect>(exec,args, 2);
+        QPoint arg0 = KJSEmbed::extractVariant<QPoint>(exec,args, 0);
+        QImage arg1 = KJSEmbed::extractVariant<QImage>(exec,args, 1);
+        QRect arg2 = KJSEmbed::extractVariant<QRect>(exec,args, 2);
         Qt::ImageConversionFlags arg3 = (Qt::ImageConversionFlags)KJSEmbed::extractInt(exec, args, 3);
         object->drawImage(arg0, arg1, arg2, arg3);
     }
@@ -213,7 +213,7 @@ START_OBJECT_METHOD( calldrawImage, QPainter )
     {
         int arg0 = KJSEmbed::extractInt(exec, args, 0);
         int arg1 = KJSEmbed::extractInt(exec, args, 1);
-        QImage arg2 = KJSEmbed::extractValue<QImage>(exec,args, 2);
+        QImage arg2 = KJSEmbed::extractVariant<QImage>(exec,args, 2);
         int arg3 = KJSEmbed::extractInt(exec, args, 3);
         int arg4 = KJSEmbed::extractInt(exec, args, 4);
         int arg5 = KJSEmbed::extractInt(exec, args, 5);
@@ -226,13 +226,13 @@ END_OBJECT_METHOD
 START_OBJECT_METHOD( calldrawLine, QPainter )
     if( args.size() == 1 )
     {
-        QLine arg0 = KJSEmbed::extractValue<QLine>(exec,args, 0);
+        QLine arg0 = KJSEmbed::extractVariant<QLine>(exec,args, 0);
         object->drawLine(arg0);
     }
     else if ( args.size() == 2 )
     {
-        QPoint arg0 = KJSEmbed::extractValue<QPoint>(exec,args, 0);
-        QPoint arg1 = KJSEmbed::extractValue<QPoint>(exec,args, 1);
+        QPoint arg0 = KJSEmbed::extractVariant<QPoint>(exec,args, 0);
+        QPoint arg1 = KJSEmbed::extractVariant<QPoint>(exec,args, 1);
         object->drawLine(arg0, arg1);
     }
     else if ( args.size() == 4 )
@@ -248,7 +248,7 @@ END_OBJECT_METHOD
 START_OBJECT_METHOD( calldrawPie, QPainter )
     if( args.size() == 3 )
     {
-        QRect arg0 = KJSEmbed::extractValue<QRect>(exec,args, 0);
+        QRect arg0 = KJSEmbed::extractVariant<QRect>(exec,args, 0);
         int arg1 = KJSEmbed::extractInt(exec, args, 1);
         int arg2 = KJSEmbed::extractInt(exec, args, 2);
         object->drawPie(arg0, arg1, arg2);
@@ -268,15 +268,15 @@ END_OBJECT_METHOD
 START_OBJECT_METHOD( calldrawPixmap, QPainter )
     if ( args.size() == 2)
     {
-        QPoint arg0 = KJSEmbed::extractValue<QPoint>(exec,args, 0);
-        QPixmap arg1 = KJSEmbed::extractValue<QPixmap>(exec,args, 1);
+        QPoint arg0 = KJSEmbed::extractVariant<QPoint>(exec,args, 0);
+        QPixmap arg1 = KJSEmbed::extractVariant<QPixmap>(exec,args, 1);
         object->drawPixmap(arg0, arg1);
     }
     else if ( args.size() == 3 )
     {
-        QPoint arg0 = KJSEmbed::extractValue<QPoint>(exec,args, 0);
-        QPixmap arg1 = KJSEmbed::extractValue<QPixmap>(exec,args, 1);
-        QRect arg2 = KJSEmbed::extractValue<QRect>(exec,args, 2);
+        QPoint arg0 = KJSEmbed::extractVariant<QPoint>(exec,args, 0);
+        QPixmap arg1 = KJSEmbed::extractVariant<QPixmap>(exec,args, 1);
+        QRect arg2 = KJSEmbed::extractVariant<QRect>(exec,args, 2);
         object->drawPixmap(arg0, arg1, arg2);
     }
 END_OBJECT_METHOD
@@ -284,7 +284,7 @@ END_OBJECT_METHOD
 START_OBJECT_METHOD( calldrawPoint, QPainter )
     if( args.size() == 1 )
     {
-        QPoint arg0 = KJSEmbed::extractValue<QPoint>(exec,args, 0);
+        QPoint arg0 = KJSEmbed::extractVariant<QPoint>(exec,args, 0);
         object->drawPoint(arg0);
     }
     else if (args.size() == 2 )
@@ -296,25 +296,25 @@ START_OBJECT_METHOD( calldrawPoint, QPainter )
 END_OBJECT_METHOD
 
 START_OBJECT_METHOD( calldrawPoints, QPainter )
-    QPolygon arg0 = KJSEmbed::extractValue<QPolygon>(exec,args, 0);
+    QPolygon arg0 = KJSEmbed::extractVariant<QPolygon>(exec,args, 0);
     object->drawPoints(arg0);
 END_OBJECT_METHOD
 
 START_OBJECT_METHOD( calldrawPolygon, QPainter )
-    QPolygon arg0 = KJSEmbed::extractValue<QPolygon>(exec,args, 0);
+    QPolygon arg0 = KJSEmbed::extractVariant<QPolygon>(exec,args, 0);
     Qt::FillRule arg1 = (Qt::FillRule)KJSEmbed::extractInt(exec, args, 1,  Qt::OddEvenFill );
     object->drawPolygon(arg0, arg1);
 END_OBJECT_METHOD
 
 START_OBJECT_METHOD( calldrawPolyline, QPainter )
-    QPolygon arg0 = KJSEmbed::extractValue<QPolygon>(exec,args, 0);
+    QPolygon arg0 = KJSEmbed::extractVariant<QPolygon>(exec,args, 0);
     object->drawPolyline(arg0);
 END_OBJECT_METHOD
 
 START_OBJECT_METHOD( calldrawRect, QPainter )
     if (args.size() == 1 )
     {
-        QRect arg0 = KJSEmbed::extractValue<QRect>(exec,args, 0);
+        QRect arg0 = KJSEmbed::extractVariant<QRect>(exec,args, 0);
         object->drawRect(arg0);
     }
     else if ( args.size() == 4 )
@@ -330,7 +330,7 @@ END_OBJECT_METHOD
 START_OBJECT_METHOD( calldrawRoundRect, QPainter )
     if ( args.size() == 2 )
     {
-        QRect arg0 = KJSEmbed::extractValue<QRect>(exec,args, 0);
+        QRect arg0 = KJSEmbed::extractVariant<QRect>(exec,args, 0);
         int arg1 = KJSEmbed::extractInt(exec, args, 1);
         int arg2 = KJSEmbed::extractInt(exec, args, 2);
         object->drawRoundRect(arg0, arg1, arg2);
@@ -350,13 +350,13 @@ END_OBJECT_METHOD
 START_OBJECT_METHOD( calldrawText, QPainter )
     if( args.size() == 2 )
     {
-        QPoint arg0 = KJSEmbed::extractValue<QPoint>(exec,args, 0);
+        QPoint arg0 = KJSEmbed::extractVariant<QPoint>(exec,args, 0);
         QString arg1 = KJSEmbed::extractQString(exec, args, 1);
         object->drawText(arg0, arg1);
     }
     else if ( args.size() == 3 )
     {
-        QRect arg0 = KJSEmbed::extractValue<QRect>(exec,args, 0);
+        QRect arg0 = KJSEmbed::extractVariant<QRect>(exec,args, 0);
         int arg1 = KJSEmbed::extractInt(exec, args, 1);
         QString arg2 = KJSEmbed::extractQString(exec, args, 2);
         QRect* arg3 = 0;
@@ -384,7 +384,7 @@ START_OBJECT_METHOD( calltranslate, QPainter )
     }
     else if( args.size() == 1 )
     {
-        QPoint arg0 = KJSEmbed::extractValue<QPoint>(exec,args, 0);
+        QPoint arg0 = KJSEmbed::extractVariant<QPoint>(exec,args, 0);
         object->translate(arg0);
     }
 END_OBJECT_METHOD
@@ -392,9 +392,9 @@ END_OBJECT_METHOD
 START_OBJECT_METHOD( calldrawTiledPixmap, QPainter)
     if( args.size() == 3 )
     {
-        QRect arg0 = KJSEmbed::extractValue<QRect>(exec,args, 0);
-        QPixmap arg1 = KJSEmbed::extractValue<QPixmap>(exec,args, 1);
-        QPoint arg2 = KJSEmbed::extractValue<QPoint>(exec,args, 2);
+        QRect arg0 = KJSEmbed::extractVariant<QRect>(exec,args, 0);
+        QPixmap arg1 = KJSEmbed::extractVariant<QPixmap>(exec,args, 1);
+        QPoint arg2 = KJSEmbed::extractVariant<QPoint>(exec,args, 2);
         object->drawTiledPixmap(arg0,arg1,arg2);
     }
     else if( args.size() == 7)
@@ -403,7 +403,7 @@ START_OBJECT_METHOD( calldrawTiledPixmap, QPainter)
         int arg1 = KJSEmbed::extractInt(exec, args, 1);
         int arg2 = KJSEmbed::extractInt(exec, args, 2);
         int arg3 = KJSEmbed::extractInt(exec, args, 3);
-        QPixmap arg4 = KJSEmbed::extractValue<QPixmap>(exec,args, 4);
+        QPixmap arg4 = KJSEmbed::extractVariant<QPixmap>(exec,args, 4);
         int arg5 = KJSEmbed::extractInt(exec, args, 5);
         int arg6 = KJSEmbed::extractInt(exec, args, 6);
         object->drawTiledPixmap(arg0,arg1,arg2,arg3,arg4,arg5,arg6);
@@ -421,7 +421,7 @@ START_OBJECT_METHOD( calleraseRect, QPainter)
     }
     else if (args.size() == 1 )
     {
-        QRect arg0 = KJSEmbed::extractValue<QRect>(exec,args, 0);
+        QRect arg0 = KJSEmbed::extractVariant<QRect>(exec,args, 0);
         object->eraseRect(arg0);
     }
 END_OBJECT_METHOD
@@ -498,7 +498,7 @@ START_CTOR( Painter, QPainter, 0 )
         }
         else
         {
-            KJS::throwError( exec, KJS::EvalError, QString("Cannot paint to object %1").arg(args[0]->toString(exec).qstring()) );
+            KJS::throwError( exec, KJS::EvalError, QString("Cannot paint to object %1").arg(toQString(args[0]->toString(exec))));
 	    return 0L;
         }
     }
