@@ -29,13 +29,7 @@ class QSettings;
 
 namespace KJSEmbed
 {
-    class SettingsBinding : public QObjectBinding
-    {
-        public:
-            SettingsBinding( KJS::ExecState *exec, QSettings *settings );
-    };
-    
-    KJS_BINDING( Settings )
+    KJSO_BINDING( SettingsBinding, QSettings, QObjectBinding )
 }
 
 #endif
