@@ -22,7 +22,6 @@
 #include "downloaddialog.moc"
 
 #include <klocale.h>
-#include <ktabctl.h>
 #include <kdebug.h>
 #include <kio/job.h>
 #include <kio/netaccess.h>
@@ -45,7 +44,8 @@
 #include <QTextBrowser>
 #include <QTimer> // hack
 #include <QTreeWidget>
-
+#include <QTabWidget>
+#include <kconfiggroup.h>
 
 using namespace KNS;
 
@@ -117,7 +117,8 @@ void DownloadDialog::init(Engine *engine)
   m_engine = engine;
   d->m_page = NULL;
 
-  connect(this, SIGNAL(currentPageChanged(QWidget*)), SLOT(slotPage(QWidget*)));
+  connect(this, SIGNAL(currentPageChanged(KPageWidgetItem*, KPageWidgetItem*)),
+          this, SLOT(slotPage(KPageWidgetItem*)));
 
   if(!engine)
   {
@@ -172,14 +173,14 @@ void DownloadDialog::slotProviders(Provider::List *list)
     else
       addProvider(p);
     /*if(p == list->getFirst())
-      slotPage(m_frame);*/ // only if !qtbug
+      slotPage(m_item);*/ // only if !qtbug
   }
 }
 
 void DownloadDialog::addProvider(Provider *p)
 {
   QFrame *frame;
-  KTabCtl *ctl;
+  QTabWidget *ctl;
   QWidget *w_d, *w_r, *w_l;
   QWidget *w2;
   QTextBrowser *rt;
@@ -206,30 +207,29 @@ void DownloadDialog::addProvider(Provider *p)
     }
     else
     {
-      pix = KGlobal::iconLoader()->loadIcon(p->icon().path(), K3Icon::Panel);
+      pix = KIconLoader::global()->loadIcon(p->icon().path(), K3Icon::Panel);
       ret = true;
     }
   }
-  if(!ret) pix = KGlobal::iconLoader()->loadIcon("knewstuff", K3Icon::Panel);
+  if(!ret) pix = KIconLoader::global()->loadIcon("knewstuff", K3Icon::Panel);
 
   frame = new QFrame( this );
-  KPageWidgetItem *item = addPage(frame, p->name());
-  item->setHeader( p->name() );
+  m_item = addPage(frame, p->name());
+  m_item->setHeader( p->name() );
   //FIXME: set icon: item->setIcon(pix);
-  m_frame = frame;
 
   w2 = new QWidget(frame);
   w_d = new QWidget(frame);
   w_r = new QWidget(frame);
   w_l = new QWidget(frame);
 
-  ctl = new KTabCtl(frame);
+  ctl = new QTabWidget(frame);
   ctl->addTab(w_r, i18n("Highest Rated"));
   ctl->addTab(w_d, i18n("Most Downloads"));
   ctl->addTab(w_l, i18n("Latest"));
 
   m_curtab = 0;
-  connect(ctl, SIGNAL(tabSelected(int)), SLOT(slotTab(int)));
+  connect(ctl, SIGNAL(currentChanged(int)), SLOT(slotTab(int)));
 
   QHBoxLayout *box = new QHBoxLayout(frame);
   box->addWidget(ctl);
@@ -270,7 +270,7 @@ void DownloadDialog::addProvider(Provider *p)
 
   box->addLayout(vbox);
 
-  connect(rt, SIGNAL(linkClicked(const QString&)), SLOT(slotEmail(const QString&)));
+  connect(rt, SIGNAL(anchorClicked(const QUrl &)), SLOT(slotEmail(const QUrl &)));
 
   connect(in, SIGNAL(clicked()), SLOT(slotInstall()));
   connect(de, SIGNAL(clicked()), SLOT(slotDetails()));
@@ -296,7 +296,7 @@ void DownloadDialog::addProvider(Provider *p)
 
   kDebug() << "addProvider()/end; d->m_lvtmp_r = " << d->m_lvtmp_r << endl;
 
-  if(m_engine) slotPage(frame);
+  if(m_engine) slotPage(m_item);
 
   QTimer::singleShot(100, this, SLOT(slotFinish()));
 }
@@ -334,7 +334,7 @@ void DownloadDialog::slotResult(KJob *job)
         if(d->m_newproviders[p])
         {
           addProvider(p);
-          slotPage(m_frame);
+          slotPage(m_item);
           d->m_newproviders[p] = 0;
         }
       }
@@ -365,8 +365,8 @@ void DownloadDialog::addEntry(Entry *entry, const QStringList& variants)
 
   installed = installStatus(entry);
 
-  if(installed > 0) pix = KGlobal::iconLoader()->loadIcon("ok", K3Icon::Small);
-  else if(installed < 0) pix = KGlobal::iconLoader()->loadIcon("history", K3Icon::Small);
+  if(installed > 0) pix = KIconLoader::global()->loadIcon("ok", K3Icon::Small);
+  else if(installed < 0) pix = KIconLoader::global()->loadIcon("history", K3Icon::Small);
   else pix = QPixmap();
 
   QString lang = KGlobal::locale()->language();
@@ -509,7 +509,7 @@ void DownloadDialog::install(Entry *e)
   KConfigGroup cg(KGlobal::config(), "KNewStuffStatus");
   cg.writeEntry(m_entryname, e->releaseDate());
 
-  QPixmap pix = KGlobal::iconLoader()->loadIcon("ok", K3Icon::Small);
+  QPixmap pix = KIconLoader::global()->loadIcon("ok", K3Icon::Small);
 
   QString lang = KGlobal::locale()->language();
 
@@ -558,7 +558,7 @@ void DownloadDialog::slotInstalled(KJob *job)
     KIO::FileCopyJob *cjob = ::qobject_cast<KIO::FileCopyJob*>(job);
     if(cjob)
     {
-      ret = m_s->install(cjob->destURL().path());
+      ret = m_s->install(cjob->destUrl().path());
     }
     else ret = false;
   }
@@ -690,7 +690,7 @@ void DownloadDialog::slotSelected()
   }
 }
 
-void DownloadDialog::slotEmail(const QString& link)
+void DownloadDialog::slotEmail(const QUrl &link)
 {
   kDebug() << "EMAIL: " << link << endl;
   KToolInvocation::invokeMailer(link);
@@ -717,8 +717,9 @@ Entry *DownloadDialog::getEntry()
   return 0;
 }
 
-void DownloadDialog::slotPage(QWidget *w)
+void DownloadDialog::slotPage(KPageWidgetItem *current)
 {
+  QWidget* w = current->widget();
   Provider *p;
 
   kDebug() << "changed widget!!!" << endl;
@@ -771,7 +772,7 @@ void DownloadDialog::loadProvider(Provider *p)
 
   QMap<QString, QStringList> urls;
 
-  for(QStringList::Iterator it = variants.begin(); it != variants.end(); it++)
+  for(QStringList::Iterator it = variants.begin(); it != variants.end(); /*it++ already done by erase*/)
   {
     QString url = p->downloadUrlVariant((*it)).url();
     if(!urls.contains(url))
