@@ -389,8 +389,8 @@ BrowserExtension::BrowserExtension( KParts::ReadOnlyPart *parent )
 
   connect( m_part, SIGNAL( completed() ),
            this, SLOT( slotCompleted() ) );
-  connect( this, SIGNAL( openURLRequest( const KUrl &, const KParts::URLArgs & ) ),
-           this, SLOT( slotOpenURLRequest( const KUrl &, const KParts::URLArgs & ) ) );
+  connect( this, SIGNAL( openUrlRequest( const KUrl &, const KParts::URLArgs & ) ),
+           this, SLOT( slotOpenUrlRequest( const KUrl &, const KParts::URLArgs & ) ) );
   connect( this, SIGNAL( enableAction( const char *, bool ) ),
            this, SLOT( slotEnableAction( const char *, bool ) ) );
   connect( this, SIGNAL( setActionText( const char *, const QString& ) ),
@@ -403,7 +403,7 @@ BrowserExtension::~BrowserExtension()
   delete d;
 }
 
-void BrowserExtension::setURLArgs( const URLArgs &args )
+void BrowserExtension::setUrlArgs( const URLArgs &args )
 {
   m_args = args;
 }
@@ -438,9 +438,9 @@ void BrowserExtension::restoreState( QDataStream &stream )
   args.xOffset = xOfs;
   args.yOffset = yOfs;
 
-  setURLArgs( args );
+  setUrlArgs( args );
 
-  m_part->openURL( u );
+  m_part->openUrl( u );
 }
 
 bool BrowserExtension::isURLDropHandlingEnabled() const
@@ -456,7 +456,7 @@ void BrowserExtension::setURLDropHandlingEnabled( bool enable )
 void BrowserExtension::slotCompleted()
 {
   //empty the argument stuff, to avoid bogus/invalid values when opening a new url
-  setURLArgs( URLArgs() );
+  setUrlArgs( URLArgs() );
 }
 
 void BrowserExtension::pasteRequest()
@@ -467,56 +467,56 @@ void BrowserExtension::pasteRequest()
     url.remove(QRegExp("[\\ ]*\\n+[\\ ]*"));
 
     // Check if it's a URL
-    QStringList filters = KURIFilter::self()->pluginNames();
+    QStringList filters = KUriFilter::self()->pluginNames();
     filters.removeAll( "kuriikwsfilter" );
     filters.removeAll( "localdomainurifilter" );
-    KURIFilterData filterData;
+    KUriFilterData filterData;
     filterData.setData( url );
     filterData.setCheckForExecutables( false );
-    if ( KURIFilter::self()->filterURI( filterData, filters ) )
+    if ( KUriFilter::self()->filterUri( filterData, filters ) )
     {
         switch ( filterData.uriType() )
 	{
-	    case KURIFilterData::LOCAL_FILE:
-	    case KURIFilterData::LOCAL_DIR:
-	    case KURIFilterData::NET_PROTOCOL:
-	        slotOpenURLRequest( filterData.uri(), KParts::URLArgs() );
+	    case KUriFilterData::LOCAL_FILE:
+	    case KUriFilterData::LOCAL_DIR:
+	    case KUriFilterData::NET_PROTOCOL:
+	        slotOpenUrlRequest( filterData.uri(), KParts::URLArgs() );
 		break;
-	    case KURIFilterData::ERROR:
+	    case KUriFilterData::ERROR:
 		KMessageBox::sorry( m_part->widget(), filterData.errorMsg() );
 		break;
 	    default:
 		break;
 	}
     }
-    else if ( KURIFilter::self()->filterURI( filterData,
+    else if ( KUriFilter::self()->filterUri( filterData,
                     QStringList( QLatin1String( "kuriikwsfilter" ) ) ) &&
               url.length() < 250 )
     {
         if ( KMessageBox::questionYesNo( m_part->widget(),
 		    i18n( "<qt>Do you want to search the Internet for <b>%1</b>?" ,  Qt::escape(url) ),
 		    i18n( "Internet Search" ), KGuiItem( i18n( "&Search" ), "find"),
-		    KStdGuiItem::cancel(), "MiddleClickSearch" ) == KMessageBox::Yes)
-          slotOpenURLRequest( filterData.uri(), KParts::URLArgs() );
+		    KStandardGuiItem::cancel(), "MiddleClickSearch" ) == KMessageBox::Yes)
+          slotOpenUrlRequest( filterData.uri(), KParts::URLArgs() );
     }
 }
 
-void BrowserExtension::slotOpenURLRequest( const KUrl &url, const KParts::URLArgs &args )
+void BrowserExtension::slotOpenUrlRequest( const KUrl &url, const KParts::URLArgs &args )
 {
     //kDebug() << this << " BrowserExtension::slotOpenURLRequest(): url=" << url.url() << endl;
     BrowserExtensionPrivate::DelayedRequest req;
     req.m_delayedURL = url;
     req.m_delayedArgs = args;
     d->m_requests.append( req );
-    QTimer::singleShot( 0, this, SLOT( slotEmitOpenURLRequestDelayed() ) );
+    QTimer::singleShot( 0, this, SLOT( slotEmitOpenUrlRequestDelayed() ) );
 }
 
-void BrowserExtension::slotEmitOpenURLRequestDelayed()
+void BrowserExtension::slotEmitOpenUrlRequestDelayed()
 {
     if (d->m_requests.isEmpty()) return;
     BrowserExtensionPrivate::DelayedRequest req = d->m_requests.front();
     d->m_requests.pop_front();
-    emit openURLRequestDelayed( req.m_delayedURL, req.m_delayedArgs );
+    emit openUrlRequestDelayed( req.m_delayedURL, req.m_delayedArgs );
     // tricky: do not do anything here! (no access to member variables, etc.)
 }
 
@@ -660,7 +660,7 @@ const QList<KParts::ReadOnlyPart*> BrowserHostExtension::frames() const
   return QList<KParts::ReadOnlyPart*>();
 }
 
-bool BrowserHostExtension::openURLInFrame( const KUrl &, const KParts::URLArgs & )
+bool BrowserHostExtension::openUrlInFrame( const KUrl &, const KParts::URLArgs & )
 {
   return false;
 }
@@ -679,23 +679,12 @@ BrowserHostExtension *BrowserHostExtension::childObject( QObject *obj )
     return 0L;
 }
 
-void BrowserExtension::virtual_hook( int, void* )
-{ /*BASE::virtual_hook( id, data );*/ }
-
 BrowserHostExtension *
 BrowserHostExtension::findFrameParent(KParts::ReadOnlyPart *callingPart, const QString &frame)
 {
-    FindFrameParentParams param;
-    param.parent = 0;
-    param.callingPart = callingPart;
-    param.frame = frame;
-    virtual_hook(VIRTUAL_FIND_FRAME_PARENT, &param);
-    return param.parent;
+    return 0;
 }
 
-void BrowserHostExtension::virtual_hook( int, void* )
-{ /*BASE::virtual_hook( id, data );*/ }
-
 LiveConnectExtension::LiveConnectExtension( KParts::ReadOnlyPart *parent ) : QObject( parent) {}
 
 bool LiveConnectExtension::get( const unsigned long, const QString &, Type &, unsigned long &, QString & ) {
