@@ -46,15 +46,15 @@ struct URLArgsPrivate;
 
 /**
  * URLArgs is a set of arguments bundled into a structure,
- * to allow specifying how a URL should be opened by openURL().
- * In other words, this is like arguments to openURL(), but without
- * have to change the signature of openURL() (since openURL is a
+ * to allow specifying how a URL should be opened by openUrl().
+ * In other words, this is like arguments to openUrl(), but without
+ * have to change the signature of openUrl() (since openUrl is a
  * generic KParts method).
  * The parts (with a browser extension) who care about urlargs will
  * use those arguments, others will ignore them.
  *
  * This can also be used the other way round, when a part asks
- * for a URL to be opened (with openURLRequest or createNewWindow).
+ * for a URL to be opened (with openUrlRequest or createNewWindow).
  */
 struct KPARTS_EXPORT URLArgs
 {
@@ -251,7 +251,7 @@ class BrowserExtensionPrivate;
   * The Browser Extension is an extension (yes, no kidding) to
   * KParts::ReadOnlyPart, which allows a better integration of parts
   * with browsers (in particular Konqueror).
-  * Remember that ReadOnlyPart only has openURL(KUrl), with no other settings.
+  * Remember that ReadOnlyPart only has openUrl(KUrl), with no other settings.
   * For full-fledged browsing, we need much more than that, including
   * many arguments about how to open this URL (see URLArgs), allowing
   * parts to save and restore their data into the back/forward history,
@@ -327,27 +327,22 @@ public:
 
   /**
    * Set of flags passed via the popupMenu signal, to ask for some items in the popup menu.
-   *
-   * DefaultPopupItems: default value, no additional menu item
-   * ShowNavigationItems: show "back" and "forward" (usually done when clicking the background of the view, but not an item)
-   * ShowUp: show "up" (same thing, but not over e.g. HTTP). Requires ShowNavigationItems.
-   * ShowReload: show "reload" (usually done when clicking the background of the view, but not an item)
-   * ShowBookmark: show "add to bookmarks" (usually not done on the local filesystem)
-   * ShowCreateDirectory: show "create directory" (usually only done on the background of the view, or
-   *                      in hierarchical views like directory trees, where the new dir would be visible)
-   * ShowTextSelectionItems: set when selecting text, for a popup that only contains text-related items.
-   * NoDeletion: deletion, trashing and renaming not allowed (e.g. parent dir not writeable).
-   *            (this is only needed if the protocol itself supports deletion, unlike e.g. HTTP)
-   * IsLink: show "Bookmark This Link" and other link-related actions (linkactions merging group)
-   * ShowURLOperations: show copy,cut,paste, as well as rename+trash+delete, if NoDeletion is not set.
    */
-  enum PopupFlag { DefaultPopupItems=0x0000, ShowNavigationItems=0x0001,
-         ShowUp=0x0002, ShowReload=0x0004, ShowBookmark=0x0008,
-         ShowCreateDirectory=0x0010, ShowTextSelectionItems=0x0020,
-         NoDeletion=0x0040,
-         IsLink=0x0080,
-         ShowURLOperations=0x0100
+  enum PopupFlag {
+      DefaultPopupItems=0x0000, /**< default value, no additional menu item */ 
+      ShowNavigationItems=0x0001, /**< show "back" and "forward" (usually done when clicking the background of the view, but not an item) */ 
+      ShowUp=0x0002, /**<  show "up" (same thing, but not over e.g. HTTP). Requires ShowNavigationItems. */
+      ShowReload=0x0004, /**< show "reload" (usually done when clicking the background of the view, but not an item) */
+      ShowBookmark=0x0008, /**< show "add to bookmarks" (usually not done on the local filesystem) */
+      ShowCreateDirectory=0x0010, /**<  show "create directory" (usually only done on the background of the view, or
+       *                      in hierarchical views like directory trees, where the new dir would be visible) */
+      ShowTextSelectionItems=0x0020, /**< set when selecting text, for a popup that only contains text-related items. */
+      NoDeletion=0x0040, /**< deletion, trashing and renaming not allowed (e.g. parent dir not writeable).
+       *            (this is only needed if the protocol itself supports deletion, unlike e.g. HTTP) */
+      IsLink=0x0080, /**< show "Bookmark This Link" and other link-related actions (linkactions merging group) */
+      ShowUrlOperations=0x0100 /**< show copy,cut,paste, as well as rename+trash+delete, if NoDeletion is not set. */
   };
+
   Q_DECLARE_FLAGS( PopupFlags, PopupFlag )
 
   /**
@@ -355,7 +350,7 @@ public:
    * This is called by the "hosting" application, to pass parameters to the part.
    * @see URLArgs
    */
-  virtual void setURLArgs( const URLArgs &args );
+  virtual void setUrlArgs( const URLArgs &args );
 
   /**
    * Retrieve the set of parameters to use for opening the URL
@@ -477,7 +472,7 @@ public:
   static BrowserExtension *childObject( QObject *obj );
 
   /**
-   * Asks the hosting browser to perform a paste (using openURLRequestDelayed)
+   * Asks the hosting browser to perform a paste (using openUrlRequestDelayed())
    */
   void pasteRequest();
 
@@ -504,44 +499,44 @@ public:  // yes, those signals are public; don't tell moc :)
    * Asks the host (browser) to open @p url.
    * To set a reload, the x and y offsets, the service type etc., fill in the
    * appropriate fields in the @p args structure.
-   * Hosts should not connect to this signal but to openURLRequestDelayed.
+   * Hosts should not connect to this signal but to openUrlRequestDelayed().
    */
-  void openURLRequest( const KUrl &url, const KParts::URLArgs &args = KParts::URLArgs() );
+  void openUrlRequest( const KUrl &url, const KParts::URLArgs &args = KParts::URLArgs() );
 
   /**
-   * This signal is emitted when openURLRequest is called, after a 0-seconds timer.
+   * This signal is emitted when openUrlRequest() is called, after a 0-seconds timer.
    * This allows the caller to terminate what it's doing first, before (usually)
    * being destroyed. Parts should never use this signal, hosts should only connect
    * to this signal.
    */
-  void openURLRequestDelayed( const KUrl &url, const KParts::URLArgs &args = KParts::URLArgs() );
+  void openUrlRequestDelayed( const KUrl &url, const KParts::URLArgs &args = KParts::URLArgs() );
 
   /**
    * Tells the hosting browser that the part opened a new URL (which can be
    * queried via KParts::Part::url().
    *
    * This helps the browser to update/create an entry in the history.
-   * The part may @em not emit this signal together with openURLRequest().
-   * Emit openURLRequest() if you want the browser to handle a URL the user
+   * The part may @em not emit this signal together with openUrlRequest().
+   * Emit openUrlRequest() if you want the browser to handle a URL the user
    * asked to open (from within your part/document). This signal however is
    * useful if you want to handle URLs all yourself internally, while still
    * telling the hosting browser about new opened URLs, in order to provide
    * a proper history functionality to the user.
    * An example of usage is a html rendering component which wants to emit
    * this signal when a child frame document changed its URL.
-   * Conclusion: you probably want to use openURLRequest() instead.
+   * Conclusion: you probably want to use openUrlRequest() instead.
    */
-  void openURLNotify();
+  void openUrlNotify();
 
   /**
    * Updates the URL shown in the browser's location bar to @p url.
    */
-  void setLocationBarURL( const QString &url );
+  void setLocationBarUrl( const QString &url );
 
   /**
    * Sets the URL of an icon for the currently displayed page.
    */
-  void setIconURL( const KUrl &url );
+  void setIconUrl( const KUrl &url );
 
   /**
    * Asks the hosting browser to open a new window for the given @p url.
@@ -557,7 +552,7 @@ public:  // yes, those signals are public; don't tell moc :)
    * and return a reference to the content part.
    * The request for a reference to the part is only fulfilled/processed
    * if the serviceType is set in the @p args . (otherwise the request cannot be
-   * processed synchroniously.
+   * processed synchronously.
    */
   void createNewWindow( const KUrl &url, const KParts::URLArgs &args,
                         const KParts::WindowArgs &windowArgs, KParts::ReadOnlyPart *&part );
@@ -592,7 +587,8 @@ public:  // yes, those signals are public; don't tell moc :)
    */
   void popupMenu( KXMLGUIClient *client, const QPoint &global, const KFileItemList &items );
 
-  void popupMenu( KXMLGUIClient *client, const QPoint &global, const KFileItemList &items, const KParts::URLArgs &args, KParts::BrowserExtension::PopupFlags i );
+  void popupMenu( KXMLGUIClient *client, const QPoint &global, const KFileItemList &items,
+                  const KParts::URLArgs &args, KParts::BrowserExtension::PopupFlags i );
 
   /**
    * Emit this to make the browser show a standard popup menu
@@ -681,10 +677,16 @@ public:  // yes, those signals are public; don't tell moc :)
    */
   void setPageSecurity( int );
 
+#define KPARTS_BROWSEREXTENSION_HAS_ITEMS_REMOVED
+  /**
+   * Inform the host about items that have been removed.
+   */
+  void itemsRemoved( const KFileItemList &items );
+
 private Q_SLOTS:
   void slotCompleted();
-  void slotOpenURLRequest( const KUrl &url, const KParts::URLArgs &args );
-  void slotEmitOpenURLRequestDelayed();
+  void slotOpenUrlRequest( const KUrl &url, const KParts::URLArgs &args );
+  void slotEmitOpenUrlRequestDelayed();
   void slotEnableAction( const char *, bool );
   void slotSetActionText( const char*, const QString& );
 
@@ -698,9 +700,6 @@ private:
   static ActionNumberMap * s_actionNumberMap;
   static ActionSlotMap * s_actionSlotMap;
   static void createActionSlotMap();
-protected:
-  virtual void virtual_hook( int id, void* data );
-private:
   BrowserExtensionPrivate *d;
 };
 
@@ -735,13 +734,13 @@ public:
    * Returns the part that contains @p frame and that may be accessed
    * by @p callingPart
    */
-  BrowserHostExtension *findFrameParent(KParts::ReadOnlyPart *callingPart, const QString &frame);
+  virtual BrowserHostExtension *findFrameParent(KParts::ReadOnlyPart *callingPart, const QString &frame);
 
   /**
    * Opens the given url in a hosted child frame. The frame name is specified in the
    * frameName variable in the urlArgs argument structure (see KParts::URLArgs ) .
    */
-  virtual bool openURLInFrame( const KUrl &url, const KParts::URLArgs &urlArgs );
+  virtual bool openUrlInFrame( const KUrl &url, const KParts::URLArgs &urlArgs );
 
   /**
    * Queries @p obj for a child object which inherits from this
@@ -749,21 +748,6 @@ public:
    */
   static BrowserHostExtension *childObject( QObject *obj );
 
-protected:
-  /** This 'enum' along with the structure below is NOT part of the public API.
-   * It's going to disappear in KDE 4.0 and is likely to change inbetween.
-   *
-   * @internal
-   */
-  enum { VIRTUAL_FIND_FRAME_PARENT = 0x10 };
-  struct FindFrameParentParams
-  {
-      BrowserHostExtension *parent;
-      KParts::ReadOnlyPart *callingPart;
-      QString frame;
-  };
-
-  virtual void virtual_hook( int id, void* data );
 private:
   class BrowserHostExtensionPrivate;
   BrowserHostExtensionPrivate *d;
