@@ -22,11 +22,12 @@
 #include <kio/job.h>
 #include <kio/jobuidelegate.h>
 #include <kio/scheduler.h>
+#include <kio/copyjob.h>
 #include <klocale.h>
 #include <kprocess.h>
 #include <kstringhandler.h>
 #include <kmimetypetrader.h>
-#include <ktempfile.h>
+#include <ktemporaryfile.h>
 #include <kdebug.h>
 #include <kstandarddirs.h>
 #include <assert.h>
@@ -91,7 +92,7 @@ void BrowserRun::scanFile()
   // Optimization for http/https, findByURL doesn't trust extensions over http.
   if ( m_strURL.query().isEmpty() && !m_strURL.protocol().startsWith("http") )
   {
-    KMimeType::Ptr mime = KMimeType::findByURL( m_strURL );
+    KMimeType::Ptr mime = KMimeType::findByUrl( m_strURL );
     assert( mime );
     if ( mime->name() != "application/octet-stream" || m_bIsLocalFile )
     {
@@ -226,9 +227,12 @@ BrowserRun::NonEmbeddableResult BrowserRun::handleNonEmbeddable( const QString&
                 int extensionPos = fileName.lastIndexOf( '.' );
                 if ( extensionPos != -1 )
                     extension = fileName.mid( extensionPos ); // keep the '.'
-                KTempFile tempFile( QString::null, extension );
+                KTemporaryFile tempFile;
+                tempFile.setSuffix(extension);
+                tempFile.setAutoRemove(false);
+                tempFile.open();
                 KUrl destURL;
-                destURL.setPath( tempFile.name() );
+                destURL.setPath( tempFile.fileName() );
                 KIO::Job *job = KIO::file_copy( m_strURL, destURL, 0600, true /*overwrite*/, false /*no resume*/, true /*progress info*/ );
                 job->ui()->setWindow (m_window);
                 connect( job, SIGNAL( result( KJob *)),
@@ -260,7 +264,7 @@ bool BrowserRun::allowExecution( const QString &mimeType, const KUrl &url )
         return false;
 
     return ( KMessageBox::warningContinueCancel( 0, i18n( "Do you really want to execute '%1'? " ,  url.prettyUrl() ),
-    i18n("Execute File?"), i18n("Execute") ) == KMessageBox::Continue );
+    i18n("Execute File?"), KGuiItem(i18n("Execute")) ) == KMessageBox::Continue );
 }
 
 static QString makeQuestion( const KUrl& url, const QString& mimeType, const QString& suggestedFileName )
@@ -299,7 +303,7 @@ BrowserRun::AskSaveResult BrowserRun::askSave( const KUrl & url, KService::Ptr o
 
     int choice = KMessageBox::questionYesNoCancel(
         0L, question, url.host(),
-        KStdGuiItem::saveAs(), openText,
+        KStandardGuiItem::saveAs(), KGuiItem(openText),
         QLatin1String("askSave")+ mimeType ); // dontAskAgainName, KEEP IN SYNC!!!
 
     return choice == KMessageBox::Yes ? Save : ( choice == KMessageBox::No ? Open : Cancel );
@@ -336,7 +340,7 @@ BrowserRun::AskSaveResult BrowserRun::askEmbedOrSave( const KUrl & url, const QS
 
     int choice = KMessageBox::questionYesNoCancel(
         0L, question, url.host(),
-        KStdGuiItem::saveAs(), KStdGuiItem::open(),
+        KStandardGuiItem::saveAs(), KStandardGuiItem::open(),
         QLatin1String("askEmbedOrSave")+ mimeType ); // dontAskAgainName, KEEP IN SYNC!!!
     return choice == KMessageBox::Yes ? Save : ( choice == KMessageBox::No ? Open : Cancel );
     // SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC
@@ -470,7 +474,7 @@ void BrowserRun::slotCopyToTempFileResult(KJob *job)
 	    job->uiDelegate()->showErrorMessage();
     } else {
         // Same as KRun::foundMimeType but with a different URL
-        (void) (KRun::runUrl( static_cast<KIO::FileCopyJob *>(job)->destURL(), m_sMimeType, m_window ));
+        (void) (KRun::runUrl( static_cast<KIO::FileCopyJob *>(job)->destUrl(), m_sMimeType, m_window ));
     }
     m_bFault = true; // see above
     m_bFinished = true;
