@@ -27,7 +27,8 @@
 #include <qcoreevent.h>
 #include <kxmlguiclient.h>
 
-class KInstance;
+class KIconLoader;
+class KComponentData;
 class QWidget;
 class KAction;
 class KActionCollection;
@@ -81,24 +82,24 @@ public:
 
 protected:
   /**
-   * Set the instance ( KInstance) for this part.
+   * Set the componentData(KComponentData) for this part.
    *
    * Call this *first* in the inherited class constructor,
    * because it loads the i18n catalogs.
    */
-  virtual void setInstance( KInstance *instance );
+  virtual void setComponentData(const KComponentData &componentData);
 
   /**
-   * Set the instance ( KInstance) for this part.
+   * Set the componentData(KComponentData) for this part.
    *
    * Call this *first* in the inherited class constructor,
    * because it loads the i18n catalogs.
    */
-  virtual void setInstance( KInstance *instance, bool loadPlugins );
+  virtual void setComponentData(const KComponentData &componentData, bool loadPlugins);
 
   /**
    * We have three different policies, whether to load new plugins or not. The
-   * value in the KConfig object of the KInstance object always overrides
+   * value in the KConfig object of the KComponentData object always overrides
    * LoadPlugins and LoadPluginsIfEnabled.
    */
   enum PluginLoadingMode {
@@ -138,13 +139,13 @@ protected:
    * }
    * \endcode
    */
-  void loadPlugins( QObject *parent, KXMLGUIClient *parentGUIClient, KInstance *instance );
+  void loadPlugins(QObject *parent, KXMLGUIClient *parentGUIClient, const KComponentData &componentData);
 
   /**
    * Set how plugins should be loaded
    * @param loadingMode see PluginLoadingMode
    *
-   * For a KParts::Part: call this before setInstance.
+   * For a KParts::Part: call this before setComponentData.
    * For a KParts::MainWindow: call this before createGUI.
    */
   void setPluginLoadingMode( PluginLoadingMode loadingMode );
@@ -158,7 +159,7 @@ protected:
    *
    * @param version the interface version that plugins must have in order to be loaded.
    *
-   * For a KParts::Part: call this before setInstance.
+   * For a KParts::Part: call this before setComponentData.
    * For a KParts::MainWindow: call this before createGUI.
    */
   void setPluginInterfaceVersion( int version );
@@ -260,6 +261,15 @@ public:
      */
     bool isSelectable() const;
 
+    /**
+     * Use this icon loader to load any icons that are specific to this part,
+     * i.e. icons installed into this part's own directories as opposed to standard
+     * kde icons. Use KIcon("myicon", iconLoader()).
+     *
+     * Make sure to call setComponentData before calling iconLoader.
+     */
+    KIconLoader* iconLoader();
+
 Q_SIGNALS:
     /**
      * Emitted by the part, to set the caption of the window(s)
@@ -339,16 +349,16 @@ class BrowserExtension;
  * in the simplest way (downloading to a temporary file, then letting the part
  * load from the temporary file).
  * To use the built-in network transparency, you only need to implement
- * openFile(), not openURL().
+ * openFile(), not openUrl().
  *
  * To implement network transparency differently (e.g. for progressive loading,
  * like a web browser does for instance), or to prevent network transparency
- * (but why would you do that?), you can override openURL().
+ * (but why would you do that?), you can override openUrl().
  *
  * KParts Application can use the signals to show feedback while the URL is being loaded.
  *
  * ReadOnlyPart handles the window caption by setting it to the current URL
- * (set in openURL(), and each time the part is activated).
+ * (set in openUrl(), and each time the part is activated).
  * If you want another caption, set it in openFile() and
  * (if the part might ever be used with a part manager) in guiActivateEvent()
  */
@@ -356,6 +366,7 @@ class KPARTS_EXPORT ReadOnlyPart : public Part
 {
   Q_OBJECT
   friend class ReadWritePart;
+  Q_PROPERTY( KUrl url READ url )
 public:
   /**
    * Constructor
@@ -389,32 +400,32 @@ public:
 
 public Q_SLOTS:
   /**
-   * Only reimplement openURL if you don't want the network transparency support
+   * Only reimplement openUrl if you don't want the network transparency support
    * to download from the url into a temporary file (when the url isn't local).
    * Otherwise, reimplement openFile() only .
    *
    * If you reimplement it, don't forget to set the caption, usually with
    * emit setWindowCaption( url.prettyUrl() );
    */
-  virtual bool openURL( const KUrl &url );
+  virtual bool openUrl( const KUrl &url );
 
 public:
   /**
-   *  Returns the currently in part used URL.
+   *  Returns the URL currently opened in this part.
    *
-   *  @return The current used URL.
+   *  @return The current URL.
    */
   KUrl url() const { return m_url; }
 
   /**
    * Called when closing the current url (e.g. document), for instance
-   * when switching to another url (note that openURL() calls it
+   * when switching to another url (note that openUrl() calls it
    * automatically in this case).
    * If the current URL is not fully loaded yet, aborts loading.
    * Deletes the temporary file used when the url is remote.
    * @return always true, but the return value exists for reimplementations
    */
-  virtual bool closeURL();
+  virtual bool closeUrl();
 
   /**
    * This convenience method returns the browserExtension for this part,
@@ -425,7 +436,7 @@ public:
 public:
   /**
    * Initiate sending data to this part.
-   * This is an alternative to openURL, which allows the user of the part
+   * This is an alternative to openUrl, which allows the user of the part
    * to load the data itself, and send it progressively to the part.
    *
    * @param mimeType the type of data that is going to be sent to this part.
@@ -509,7 +520,7 @@ protected Q_SLOTS:
 
 protected:
   /**
-   * If the part uses the standard implementation of openURL(),
+   * If the part uses the standard implementation of openUrl(),
    * it must reimplement this, to open @p m_file.
    * Otherwise simply define it to { return false; }
    */
@@ -576,9 +587,9 @@ public:
   /**
    * Destructor
    * Applications using a ReadWritePart should make sure, before
-   * destroying it, to call closeURL().
+   * destroying it, to call closeUrl().
    * In KMainWindow::queryClose(), for instance, they should allow
-   * closing only if the return value of closeURL() was true.
+   * closing only if the return value of closeUrl() was true.
    * This allows to cancel.
    */
   virtual ~ReadWritePart();
@@ -602,16 +613,16 @@ public:
   /**
    * If the document has been modified, ask the user to save changes.
    * This method is meant to be called from KMainWindow::queryClose().
-   * It will also be called from closeURL().
+   * It will also be called from closeUrl().
    *
-   * @return true if closeURL() can be called without the user losing
+   * @return true if closeUrl() can be called without the user losing
    * important data, false if the user chooses to cancel.
    */
   virtual bool queryClose();
 
   /**
    * Called when closing the current url (e.g. document), for instance
-   * when switching to another url (note that openURL() calls it
+   * when switching to another url (note that openUrl() calls it
    * automatically in this case).
    *
    * If the current URL is not fully loaded yet, aborts loading.
@@ -620,7 +631,7 @@ public:
    *
    * @return false on cancel
    */
-  virtual bool closeURL();
+  virtual bool closeUrl();
 
   /**
    * Call this method instead of the above if you need control if
@@ -628,9 +639,9 @@ public:
    * from KMainWindow::queryClose(), you would not want to prompt
    * again when closing the url.
    *
-   * Equivalent to promptToSave ? closeURL() : ReadOnlyPart::closeURL()
+   * Equivalent to promptToSave ? closeUrl() : ReadOnlyPart::closeUrl()
    */
-  virtual bool closeURL( bool promptToSave );
+  virtual bool closeUrl( bool promptToSave );
 
   /**
    * Save the file to a new location.
@@ -663,7 +674,7 @@ public Q_SLOTS:
   /**
    * Save the file in the location from which it was opened.
    * You can connect this to the "save" action.
-   * Calls saveFile() and saveToURL(), no need to reimplement.
+   * Calls saveFile() and saveToUrl(), no need to reimplement.
    */
   virtual bool save();
 
@@ -695,7 +706,7 @@ protected:
    * in case you want to provide feedback.
    * @return true on success, false on failure.
    */
-  virtual bool saveToURL();
+  virtual bool saveToUrl();
 
 protected Q_SLOTS:
   /**
