@@ -2,17 +2,19 @@
 #include <QTextEdit>
 #include "plugin_spellcheck.h"
 #include <kaction.h>
+#include <kactioncollection.h>
 #include <kgenericfactory.h>
 #include <kmessagebox.h>
 #include <klocale.h>
 #include <kdebug.h>
 
-PluginSpellCheck::PluginSpellCheck( QObject* parent, const char* name, 
+PluginSpellCheck::PluginSpellCheck( QObject* parent,
                                     const QStringList& )
     : Plugin( parent )
 {
-    (void) new KAction( "&Select current line (plugin)", 0, this, SLOT(slotSpellCheck()),
-                        actionCollection(), "spellcheck" );
+    KAction* act = new KAction( "&Select current line (plugin)", this );
+    actionCollection()->addAction( "spellcheck", act );
+    connect(act, SIGNAL(triggered()), this, SLOT(slotSpellCheck()));
 }
 
 PluginSpellCheck::~PluginSpellCheck()
@@ -33,7 +35,7 @@ void PluginSpellCheck::slotSpellCheck()
     }
 }
 
-K_EXPORT_COMPONENT_FACTORY( libspellcheckplugin, 
-                            KGenericFactory<PluginSpellCheck> );
+K_EXPORT_COMPONENT_FACTORY( libspellcheckplugin,
+                            KGenericFactory<PluginSpellCheck> )
 
 #include <plugin_spellcheck.moc>
