@@ -28,7 +28,7 @@
 #include "qpalette.h"
 #include "qpushbutton.h"
 #include "qdrawutil.h"
-#include "q3progressbar.h"
+#include "qprogressbar.h"
 #include "qscrollbar.h"
 #include "qtabbar.h"
 #include "qpointer.h"
@@ -83,7 +83,7 @@ LightStyleV2::~LightStyleV2()
     }
 }
 
-void LightStyleV2::polishPopupMenu( Q3PopupMenu * menu )
+void LightStyleV2::polishPopupMenu( QMenu * menu )
 {
     KStyle::polishPopupMenu(menu);
 }
@@ -776,7 +776,7 @@ void LightStyleV2::drawControl( ControlElement control,
 	    if (! widget || data.isDefault())
 		break;
 
-	    const Q3PopupMenu *popupmenu = (const Q3PopupMenu *) widget;
+	    const QMenu *popupmenu = (const QMenu *) widget;
 	    QMenuItem *mi = data.menuItem();
 	    int tab = data.tabWidth();
 	    int maxpmw = data.maxIconWidth();
@@ -1520,7 +1520,7 @@ QSize LightStyleV2::sizeFromContents( ContentsType contents,
 		break;
 
 	    QMenuItem *mi = data.menuItem();
-	    const Q3PopupMenu *popupmenu = (const Q3PopupMenu *) widget;
+	    const QMenu *popupmenu = (const QMenu *) widget;
 	    int maxpmw = data.maxIconWidth();
 	    int w = contentsSize.width(), h = contentsSize.height();
 
@@ -1561,7 +1561,7 @@ QSize LightStyleV2::sizeFromContents( ContentsType contents,
 	}
     case CT_ProgressBar:
 	{
-	    const Q3ProgressBar* pb = static_cast<const Q3ProgressBar*>(widget);
+	    const QProgressBar* pb = static_cast<const QProgressBar*>(widget);
 	    
 	    //If we have to display the indicator, and we do it on RHS, give some more room
 	    //for it. This tries to match the logic and the spacing in SR_ProgressBarGroove/Contents
