@@ -59,7 +59,7 @@ Backend::Backend(const QString& name, bool isPath) : _name(name), _ref(0) {
 	if (isPath) {
 		_path = name;
 	} else {
-		_path = KGlobal::dirs()->saveLocation("kwallet") + "/" + _name + ".kwl";
+		_path = KGlobal::dirs()->saveLocation("kwallet") + '/' + _name + ".kwl";
 	}
 
 	_open = false;
@@ -239,10 +239,10 @@ static int password2hash(const QByteArray& password, QByteArray& hash) {
 
 
 bool Backend::exists(const QString& wallet) {
-QString path = KGlobal::dirs()->saveLocation("kwallet") + "/" + wallet + ".kwl";
+	QString path = KGlobal::dirs()->saveLocation("kwallet") + '/' + wallet + ".kwl";
 	// Note: 60 bytes is presently the minimum size of a wallet file.
 	//       Anything smaller is junk.
-return QFile::exists(path) && QFileInfo(path).size() >= 60;
+	return QFile::exists(path) && QFileInfo(path).size() >= 60;
 }
 
 
@@ -482,15 +482,14 @@ int Backend::sync(const QByteArray& password) {
 		return -255;  // not open yet
 	}
 
-	KSaveFile sf(_path, 0600);
-	QFile *qf = sf.file();
+	KSaveFile sf(_path);
 
-	if (!qf) {
-		sf.abort();
+	if (!sf.open()) {
 		return -1;		// error opening file
 	}
+	sf.setPermissions(QFile::ReadUser|QFile::WriteUser);
 
-	qf->write(KWMAGIC, KWMAGIC_LEN);
+	sf.write(KWMAGIC, KWMAGIC_LEN);
 
 	// Write the version number
 	QByteArray version(4, 0);
@@ -498,7 +497,7 @@ int Backend::sync(const QByteArray& password) {
 	version[1] = KWALLET_VERSION_MINOR;
 	version[2] = KWALLET_CIPHER_BLOWFISH_CBC;
 	version[3] = KWALLET_HASH_SHA1;
-	qf->write(version, 4);
+	sf.write(version, 4);
 
 	// Holds the hashes we write out
 	QByteArray hashes;
@@ -535,7 +534,7 @@ int Backend::sync(const QByteArray& password) {
 		}
 	}
 
-	qf->write(hashes, hashes.size());
+	sf.write(hashes, hashes.size());
 
 	// calculate the hash of the file
 	SHA1 sha;
@@ -612,16 +611,15 @@ int Backend::sync(const QByteArray& password) {
 	passhash.fill(0);        // passhash is UNUSABLE NOW
 
 	// write the file
-	qf->write(wholeFile, wholeFile.size());
-	if (!sf.close()) {
+	sf.write(wholeFile, wholeFile.size());
+	if (!sf.finalize()) {
 		wholeFile.fill(0);
-		sf.abort();
 		return -4; // write error
 	}
 
 	wholeFile.fill(0);
 
-return 0;
+	return 0;
 }
 
 
@@ -656,13 +654,13 @@ QStringList Backend::entryList() const {
 
 
 Entry *Backend::readEntry(const QString& key) {
-Entry *rc = 0L;
+	Entry *rc = 0L;
 
 	if (_open && hasEntry(key)) {
 		rc = _entries[_folder][key];
 	}
 
-return rc;
+	return rc;
 }
 
 
@@ -696,14 +694,14 @@ bool Backend::createFolder(const QString& f) {
 	folderMd5.update(f.toUtf8());
 	_hashes.insert(MD5Digest(folderMd5.rawDigest()), QList<MD5Digest>());
 
-return true;
+	return true;
 }
 
 
 int Backend::renameEntry(const QString& oldName, const QString& newName) {
-EntryMap& emap = _entries[_folder];
-EntryMap::Iterator oi = emap.find(oldName);
-EntryMap::Iterator ni = emap.find(newName);
+	EntryMap& emap = _entries[_folder];
+	EntryMap::Iterator oi = emap.find(oldName);
+	EntryMap::Iterator ni = emap.find(newName);
 
 	if (oi != emap.end() && ni == emap.end()) {
 		Entry *e = oi.value();
@@ -724,7 +722,7 @@ EntryMap::Iterator ni = emap.find(newName);
 		return 0;
 	}
 
-return -1;
+	return -1;
 }
 
 
@@ -777,7 +775,7 @@ bool Backend::removeEntry(const QString& key) {
 		return true;
 	}
 
-return false;
+	return false;
 }
 
 
@@ -805,7 +803,7 @@ bool Backend::removeFolder(const QString& f) {
 		return true;
 	}
 
-return false;
+	return false;
 }
 
 
