@@ -1,8 +1,9 @@
 #include <qtextstream.h>
+#include <qapplication.h>
 #include <qtimer.h>
 
 #include <kaboutdata.h>
-#include <kapplication.h>
+#include <kcomponentdata.h>
 #include <kcmdlineargs.h>
 #include <kdebug.h>
 #include <kglobal.h>
@@ -26,12 +27,12 @@ void openWallet()
 int main( int argc, char *argv[] )
 {
 	KAboutData aboutData( "kwalletsync", "kwalletsync", "version" );
-	KCmdLineArgs::init( argc, argv, &aboutData );
-	KApplication app( "kwalletsync" );
+	KComponentData componentData(&aboutData);
+	QApplication app( argc, argv );
 
 	// force name with D-BUS
         QDBusReply<QDBusConnectionInterface::RegisterServiceReply> reply
-            = QDBus::sessionBus().interface()->registerService( app.objectName(),
+            = QDBusConnection::sessionBus().interface()->registerService( "org.kde.kwalletsync",
                                                                 QDBusConnectionInterface::ReplaceExistingService );
 
         if ( !reply.isValid() )
