@@ -20,9 +20,11 @@
 #ifndef PHONON_BYTESTREAMINTERFACE_H
 #define PHONON_BYTESTREAMINTERFACE_H
 
+#include <kdelibs_export.h>
+
 namespace Phonon
 {
-class ByteStreamInterface
+class PHONONCORE_EXPORT ByteStreamInterface
 {
 	public:
 		virtual ~ByteStreamInterface() {}
@@ -35,4 +37,4 @@ class ByteStreamInterface
 Q_DECLARE_INTERFACE( Phonon::ByteStreamInterface, "org.kde.Phonon.ByteStreamInterface/0.1" )
 
 #endif // PHONON_BYTESTREAMINTERFACE_H
-// vim: sw=4 ts=4 noet tw=80
+// vim: sw=4 ts=4 tw=80
