@@ -25,7 +25,7 @@
 class AudioFftOutputTest : public QObject
 {
 	Q_OBJECT
-	private slots:
+	private Q_SLOTS:
 		void initTestCase();
 		void testBandwidth();
 		void testRate();
