@@ -36,7 +36,10 @@ namespace Phonon
 			QList<int> audioOutputDeviceListFor( Phonon::Category category ) const;
 			int audioOutputDeviceFor( Phonon::Category category ) const;
 
-		signals:
+                        QList<int> audioCaptureDeviceList() const;
+                        int audioCaptureDevice() const;
+
+                Q_SIGNALS:
 			void audioOutputDeviceConfigChanged();
 
 		private:
