@@ -26,6 +26,7 @@
 #include <QString>
 
 #include <kservice.h>
+class KCModuleProxy;
 
 class BackendSelection : public QWidget, private Ui::BackendSelection
 {
@@ -37,17 +38,21 @@ class BackendSelection : public QWidget, private Ui::BackendSelection
 		void save();
 		void defaults();
 
-	private slots:
+	private Q_SLOTS:
 		void selectionChanged();
 		void up();
 		void down();
 
-	signals:
+ Q_SIGNALS:
 		void changed();
 
 	private:
+        void showBackendKcm(const KService::Ptr &backendService);
+		void loadServices( const KService::List& offers );
 		QHash<QString, KService::Ptr> m_services;
+        QHash<QString, KCModuleProxy *> m_kcms;
+        int m_emptyPage;
 };
 
-// vim: sw=4 ts=4 noet tw=80
+// vim: sw=4 ts=4 tw=80
 #endif // BACKENDSELECTION_H
