@@ -20,9 +20,11 @@
 #ifndef PHONON_MEDIAOBJECTINTERFACE_H
 #define PHONON_MEDIAOBJECTINTERFACE_H
 
+#include <kdelibs_export.h>
+
 namespace Phonon
 {
-class MediaObjectInterface
+class PHONONCORE_EXPORT MediaObjectInterface
 {
 	public:
 		virtual ~MediaObjectInterface() {}
@@ -35,4 +37,4 @@ class MediaObjectInterface
 Q_DECLARE_INTERFACE( Phonon::MediaObjectInterface, "org.kde.Phonon.MediaObjectInterface/0.1" )
 
 #endif // PHONON_MEDIAOBJECTINTERFACE_H
-// vim: sw=4 ts=4 noet tw=80
+// vim: sw=4 ts=4 tw=80
