@@ -36,7 +36,7 @@ namespace Fake
 			BrightnessControl( QObject* parent );
 			~BrightnessControl();
 
-		public slots:
+		public Q_SLOTS:
 			int brightness() const;
 			void setBrightness( int brightness );
 
@@ -48,5 +48,5 @@ namespace Fake
 	};
 }} //namespace Phonon::Fake
 
-// vim: sw=4 ts=4 tw=80 noet
+// vim: sw=4 ts=4 tw=80
 #endif // Phonon_FAKE_BRIGHTNESSCONTROL_H
