@@ -38,7 +38,7 @@ UiBackend::~UiBackend()
 {
 }
 
-QWidget* UiBackend::createVideoWidget( QWidget* parent )
+QObject* UiBackend::createVideoWidget( QWidget* parent )
 {
 	return new Fake::VideoWidget( parent );
 }
@@ -46,4 +46,4 @@ QWidget* UiBackend::createVideoWidget( QWidget* parent )
 }}
 
 #include "backend.moc"
-// vim: sw=4 ts=4 noet
+// vim: sw=4 ts=4
