@@ -34,9 +34,9 @@ namespace Fake
 			UiBackend( QObject* parent, const QStringList& args );
 			virtual ~UiBackend();
 
-			Q_INVOKABLE QWidget*  createVideoWidget( QWidget* parent );
+			Q_INVOKABLE QObject* createVideoWidget( QWidget* parent );
 	};
 }} // namespace Phonon::Fake
 
-// vim: sw=4 ts=4 noet tw=80
+// vim: sw=4 ts=4 tw=80
 #endif // Phonon_FAKE_UI_BACKEND_H
