@@ -18,43 +18,65 @@
 */
 
 #include "mediaobjecttest.h"
+#include "qtesthelper.h"
 #include <cstdlib>
 #include <qtest_kde.h>
 #include <QTime>
 #include <QtDebug>
-#include <phonon/audiopath.h>
 #include <phonon/videopath.h>
+#include <phonon/audiopath.h>
+#include <phonon/audiooutput.h>
+
+#include <unistd.h>
 
 using namespace Phonon;
 
 Q_DECLARE_METATYPE( Phonon::State )
 
-static const qint64 castQVariantToInt64( const QVariant& variant )
+static qint64 castQVariantToInt64( const QVariant& variant )
 {
 	return *reinterpret_cast<const qint64*>( variant.constData() );
 }
 
-static const qint32 castQVariantToInt32( const QVariant& variant )
+static qint32 castQVariantToInt32( const QVariant& variant )
 {
 	return *reinterpret_cast<const qint32*>( variant.constData() );
 }
 
-void MediaObjectTest::startPlayback()
+void MediaObjectTest::init()
+{
+    if (m_media->state() == Phonon::ErrorState) {
+        m_media->setUrl(m_url);
+        if (m_media->state() == Phonon::ErrorState) {
+            waitForSignal(m_media, SIGNAL(stateChanged(Phonon::State, Phonon::State)));
+        }
+        if (m_media->state() == Phonon::LoadingState) {
+            waitForSignal(m_media, SIGNAL(stateChanged(Phonon::State, Phonon::State)));
+        }
+        m_stateChangedSignalSpy->clear();
+    }
+}
+
+void MediaObjectTest::cleanup()
+{
+    m_stateChangedSignalSpy->clear();
+}
+
+void MediaObjectTest::startPlayback(Phonon::State currentState)
 {
 	QCOMPARE( m_stateChangedSignalSpy->count(), 0 );
-	QCOMPARE( m_media->state(), Phonon::StoppedState );
+	QCOMPARE( m_media->state(), currentState);
 	m_media->play();
 	QCOMPARE( m_stateChangedSignalSpy->count(), 1 );
 	QList<QVariant> args = m_stateChangedSignalSpy->takeFirst();
 	Phonon::State newstate = qvariant_cast<Phonon::State>( args.at( 0 ) );
 	Phonon::State oldstate = qvariant_cast<Phonon::State>( args.at( 1 ) );
-	QCOMPARE( oldstate, Phonon::StoppedState );
+	QCOMPARE( oldstate, currentState);
 	QCOMPARE( newstate, m_media->state() );
 	if( newstate == Phonon::BufferingState )
 	{
 		QCOMPARE( m_stateChangedSignalSpy->count(), 0 );
-		while( m_stateChangedSignalSpy->count() == 0 )
-			QCoreApplication::processEvents();
+        waitForSignal(m_media, SIGNAL(stateChanged(Phonon::State, Phonon::State)));
 		QCOMPARE( m_stateChangedSignalSpy->count(), 1 );
 
 		args = m_stateChangedSignalSpy->takeFirst();
@@ -69,6 +91,10 @@ void MediaObjectTest::startPlayback()
 void MediaObjectTest::stopPlayback( Phonon::State currentState )
 {
 	m_media->stop();
+    if (m_stateChangedSignalSpy->count() == 0) {
+        QCOMPARE(m_media->state(), currentState);
+        waitForSignal(m_media, SIGNAL(stateChanged(Phonon::State, Phonon::State)));
+    }
 	QCOMPARE( m_stateChangedSignalSpy->count(), 1 );
 	QList<QVariant> args = m_stateChangedSignalSpy->takeFirst();
 	Phonon::State newstate = qvariant_cast<Phonon::State>( args.at( 0 ) );
@@ -85,6 +111,10 @@ void MediaObjectTest::stopPlayback( Phonon::State currentState )
 void MediaObjectTest::pausePlayback( Phonon::State currentState )
 {
 	m_media->pause();
+    if (m_stateChangedSignalSpy->count() == 0) {
+        QCOMPARE(m_media->state(), currentState);
+        waitForSignal(m_media, SIGNAL(stateChanged(Phonon::State, Phonon::State)));
+    }
 	QCOMPARE( m_stateChangedSignalSpy->count(), 1 );
 	QList<QVariant> args = m_stateChangedSignalSpy->takeFirst();
 	Phonon::State newstate = qvariant_cast<Phonon::State>( args.at( 0 ) );
@@ -94,6 +124,16 @@ void MediaObjectTest::pausePlayback( Phonon::State currentState )
 	QCOMPARE( m_media->state(), Phonon::PausedState );
 }
 
+void MediaObjectTest::waitForSignal(QObject *obj, const char *signalName, int timeout)
+{
+    QEventLoop loop;
+    connect(obj, signalName, &loop, SLOT(quit()));
+    if (timeout > 0) {
+        QTimer::singleShot(timeout, &loop, SLOT(quit()));
+    }
+    loop.exec();
+}
+
 void MediaObjectTest::initTestCase()
 {
 	qRegisterMetaType<Phonon::State>( "Phonon::State" );
@@ -107,6 +147,10 @@ void MediaObjectTest::initTestCase()
 	m_stateChangedSignalSpy = new QSignalSpy( m_media, SIGNAL( stateChanged( Phonon::State, Phonon::State ) ) );
 	QVERIFY( m_stateChangedSignalSpy->isValid() );
 	m_stateChangedSignalSpy->clear();
+
+    setMedia();
+    addPaths();
+    initOutput();
 }
 
 void MediaObjectTest::setMedia()
@@ -123,8 +167,7 @@ void MediaObjectTest::setMedia()
 	{
 		// still in LoadingState, there should be no state change
 		QCOMPARE( emits, 0 );
-		while( m_stateChangedSignalSpy->count() == 0 )
-			QCoreApplication::processEvents();
+        waitForSignal(m_media, SIGNAL(stateChanged(Phonon::State, Phonon::State)));
 		emits = m_stateChangedSignalSpy->count();
 		s = m_media->state();
 	}
@@ -181,6 +224,7 @@ void MediaObjectTest::stopToPause()
 void MediaObjectTest::stopToPlay()
 {
 	startPlayback();
+    ::sleep(1);
 	stopPlayback( Phonon::PlayingState );
 }
 
@@ -222,19 +266,10 @@ void MediaObjectTest::pauseToPause()
 
 void MediaObjectTest::pauseToPlay()
 {
-	startPlayback();
-	pausePlayback( Phonon::PlayingState );
-
-	m_media->play();
-	QCOMPARE( m_stateChangedSignalSpy->count(), 1 );
-	QList<QVariant> args = m_stateChangedSignalSpy->takeFirst();
-	Phonon::State newstate = qvariant_cast<Phonon::State>( args.at( 0 ) );
-	Phonon::State oldstate = qvariant_cast<Phonon::State>( args.at( 1 ) );
-	QCOMPARE( oldstate, Phonon::PausedState );
-	QCOMPARE( newstate, Phonon::PlayingState );
-	QCOMPARE( m_media->state(), Phonon::PlayingState );
-
-	stopPlayback( Phonon::PlayingState );
+    startPlayback();
+    pausePlayback(Phonon::PlayingState);
+    startPlayback(Phonon::PausedState);
+    stopPlayback(Phonon::PlayingState);
 }
 
 void MediaObjectTest::pauseToStop()
@@ -292,13 +327,14 @@ void MediaObjectTest::testSeek()
 
 void MediaObjectTest::testAboutToFinish()
 {
-	m_media->setAboutToFinishTime( 500 );
-	QCOMPARE( m_media->aboutToFinishTime(), qint32( 500 ) );
+    const qint32 requestedAboutToFinishTime = 2000;
+    m_media->setAboutToFinishTime(requestedAboutToFinishTime);
+    QCOMPARE(m_media->aboutToFinishTime(), requestedAboutToFinishTime);
 	QSignalSpy aboutToFinishSpy( m_media, SIGNAL( aboutToFinish( qint32 ) ) );
 	QSignalSpy finishSpy( m_media, SIGNAL( finished() ) );
 	startPlayback();
 	if( m_media->isSeekable() )
-		m_media->seek( m_media->totalTime() - 1000 );
+        m_media->seek(m_media->totalTime() - 4000 - requestedAboutToFinishTime); // give it 4 seconds
 	while( aboutToFinishSpy.count() == 0 && ( m_media->state() == Phonon::PlayingState || m_media->state() == Phonon::BufferingState ) )
 		QCoreApplication::processEvents();
 	// at this point the media should be about to finish playing
@@ -306,18 +342,18 @@ void MediaObjectTest::testAboutToFinish()
 	Phonon::State state = m_media->state();
 	QCOMPARE( aboutToFinishSpy.count(), 1 );
 	const qint32 aboutToFinishTime = castQVariantToInt32( aboutToFinishSpy.first().at( 0 ) );
-	QVERIFY( aboutToFinishTime <= 500 );
-	if( state == Phonon::PlayingState || state == Phonon::BufferingState )
-	{
-		QVERIFY( r <= aboutToFinishTime );
-		while( finishSpy.count() == 0 )
-			QCoreApplication::processEvents();
-	}
-	else
-	{
-		QVERIFY( aboutToFinishTime > 0 );
-	}
-	QCOMPARE( finishSpy.count(), 1 );
+    QVERIFY(aboutToFinishTime <= requestedAboutToFinishTime + 150); // allow it to be up to 150ms too early
+    qDebug() << "received aboutToFinishTime" << aboutToFinishTime << ", requested" << requestedAboutToFinishTime;
+    if (state == Phonon::PlayingState || state == Phonon::BufferingState) {
+        QVERIFY(r <= aboutToFinishTime);
+        waitForSignal(m_media, SIGNAL(finished()));
+    } else {
+        QVERIFY(aboutToFinishTime >= 0);
+    }
+    QCOMPARE(finishSpy.count(), 1);
+
+    m_media->setAboutToFinishTime(0);
+    QCOMPARE(m_media->aboutToFinishTime(), qint32(0));
 
 	QCOMPARE( m_stateChangedSignalSpy->count(), 1 );
 	QList<QVariant> args = m_stateChangedSignalSpy->takeFirst();
@@ -326,6 +362,148 @@ void MediaObjectTest::testAboutToFinish()
 	QCOMPARE( oldstate, Phonon::PlayingState );
 	QCOMPARE( newstate, Phonon::StoppedState );
 	QCOMPARE( m_media->state(), Phonon::StoppedState );
+    QCoreApplication::processEvents();
+    QCOMPARE(aboutToFinishSpy.count(), 1);
+}
+
+void MediaObjectTest::setMediaAndPlay()
+{
+    m_stateChangedSignalSpy->clear();
+    QCOMPARE(m_stateChangedSignalSpy->count(), 0);
+
+    QSignalSpy lengthSignalSpy(m_media, SIGNAL(length(qint64)));
+    QVERIFY(!m_media->url().isEmpty());
+    Phonon::State state = m_media->state();
+    QVERIFY(state == Phonon::StoppedState || state == Phonon::PlayingState);
+    m_media->setUrl(m_url);
+    // before calling play() we better make sure that if play() finishes very fast that we don't get
+    // called again
+    disconnect(m_media, SIGNAL(finished()), this, SLOT(setMediaAndPlay()));
+    m_stateChangedSignalSpy->clear();
+    state = m_media->state();
+    m_media->play();
+    while (state != Phonon::PlayingState) {
+        QVERIFY(m_media->state() != Phonon::ErrorState);
+        if (m_stateChangedSignalSpy->isEmpty()) {
+            waitForSignal(m_media, SIGNAL(stateChanged(Phonon::State, Phonon::State)), 5000);
+            // if it times out the backend has a bug
+        }
+        QVERIFY(m_stateChangedSignalSpy->count() > 0);
+        QList<QVariant> args = m_stateChangedSignalSpy->takeFirst();
+        Phonon::State oldstate = qvariant_cast<Phonon::State>(args.at(1));
+        QCOMPARE(oldstate, state);
+        state = qvariant_cast<Phonon::State>(args.at(0));
+    }
+
+    /*
+    QCOMPARE(m_url, m_media->url());
+    int emits = m_stateChangedSignalSpy->count();
+    state = m_media->state();
+    if (state == Phonon::LoadingState) {
+        // now in LoadingState, there should be one state change
+        QCOMPARE(emits, 1);
+        QList<QVariant> args = m_stateChangedSignalSpy->takeFirst();
+        Phonon::State newstate = qvariant_cast<Phonon::State>(args.at(0));
+        Phonon::State oldstate = qvariant_cast<Phonon::State>(args.at(1));
+        QVERIFY(oldstate == Phonon::StoppedState || oldstate == Phonon::PlayingState);
+        QCOMPARE(state, newstate);
+
+        waitForSignal(m_media, SIGNAL(stateChanged(Phonon::State, Phonon::State)));
+        emits = m_stateChangedSignalSpy->count();
+        state = m_media->state();
+        QCOMPARE(emits, 1);
+    }
+    QVERIFY(state != Phonon::LoadingState);
+    // there should exactly be one state change
+    QCOMPARE(emits, 1);
+    QList<QVariant> args = m_stateChangedSignalSpy->takeFirst();
+    Phonon::State newstate = qvariant_cast<Phonon::State>(args.at(0));
+    Phonon::State oldstate = qvariant_cast<Phonon::State>(args.at(1));
+
+		QCOMPARE( Phonon::LoadingState, oldstate );
+		QCOMPARE( state, newstate );
+		if( Phonon::ErrorState == state )
+			QFAIL( "Loading the URL put the MediaObject into the ErrorState. Check that PHONON_TESTURL is set to a valid URL." );
+		QCOMPARE( Phonon::StoppedState, state );
+		QCOMPARE( m_stateChangedSignalSpy->count(), 0 );
+
+		// check for length signal
+		QVERIFY( lengthSignalSpy.count() > 0 );
+		args = lengthSignalSpy.takeLast();
+		QCOMPARE( m_media->totalTime(), castQVariantToInt64( args.at( 0 ) ) );
+        */
+
+    emit continueTestPlayOnFinish();
+}
+
+void MediaObjectTest::testPlayOnFinish()
+{
+    connect(m_media, SIGNAL(finished()), SLOT(setMediaAndPlay()));
+    startPlayback();
+    if (m_media->isSeekable()) {
+        m_media->seek(m_media->totalTime() - 4000);
+    }
+    waitForSignal(this, SIGNAL(continueTestPlayOnFinish()), 4000);
+    ::sleep(1);
+    m_stateChangedSignalSpy->clear();
+    stopPlayback(Phonon::PlayingState);
+}
+
+void MediaObjectTest::testPlayBeforeFinish()
+{
+    startPlayback();
+    QCOMPARE(m_stateChangedSignalSpy->size(), 0);
+    Phonon::State state = m_media->state();
+    QCOMPARE(state, Phonon::PlayingState);
+    m_media->setUrl(m_url);
+    m_media->play();
+    if (m_stateChangedSignalSpy->isEmpty()) {
+        waitForSignal(m_media, SIGNAL(stateChanged(Phonon::State, Phonon::State)), 4000);
+        if (0 == m_stateChangedSignalSpy->size()) {
+            QFAIL("no state change after setUrl() and play() were called");
+        }
+    }
+    // first state to reach is StoppedState
+    QList<QVariant> args = m_stateChangedSignalSpy->takeFirst();
+    Phonon::State oldstate = qvariant_cast<Phonon::State>(args.at(1));
+    QCOMPARE(oldstate, state);
+    state = qvariant_cast<Phonon::State>(args.at(0));
+    QCOMPARE(state, Phonon::StoppedState);
+    if (m_stateChangedSignalSpy->isEmpty()) {
+        waitForSignal(m_media, SIGNAL(stateChanged(Phonon::State, Phonon::State)), 4000);
+        if (0 == m_stateChangedSignalSpy->size()) {
+            QFAIL("no state change after StoppedState after setUrl() and play() were called");
+        }
+    }
+    // next LoadingState
+    args = m_stateChangedSignalSpy->takeFirst();
+    oldstate = qvariant_cast<Phonon::State>(args.at(1));
+    QCOMPARE(oldstate, state);
+    state = qvariant_cast<Phonon::State>(args.at(0));
+    QCOMPARE(state, Phonon::LoadingState);
+    if (m_stateChangedSignalSpy->isEmpty()) {
+        waitForSignal(m_media, SIGNAL(stateChanged(Phonon::State, Phonon::State)), 4000);
+        if (0 == m_stateChangedSignalSpy->size()) {
+            QFAIL("no state change after LoadingState after setUrl() and play() were called");
+        }
+    }
+    // next either BufferingState or PlayingState
+    args = m_stateChangedSignalSpy->takeFirst();
+    oldstate = qvariant_cast<Phonon::State>(args.at(1));
+    QCOMPARE(oldstate, state);
+    state = qvariant_cast<Phonon::State>(args.at(0));
+    if (state == Phonon::BufferingState && m_stateChangedSignalSpy->size() == 0) {
+        waitForSignal(m_media, SIGNAL(stateChanged(Phonon::State, Phonon::State)), 4000);
+        if (0 == m_stateChangedSignalSpy->size()) {
+            QFAIL("no state change after BufferingState after setUrl() and play() were called");
+        }
+        args = m_stateChangedSignalSpy->takeFirst();
+        oldstate = qvariant_cast<Phonon::State>(args.at(1));
+        QCOMPARE(oldstate, state);
+        state = qvariant_cast<Phonon::State>(args.at(0));
+    }
+    QCOMPARE(state, Phonon::PlayingState);
+    stopPlayback(Phonon::PlayingState);
 }
 
 void MediaObjectTest::testTickSignal()
@@ -356,14 +534,27 @@ void MediaObjectTest::testTickSignal()
 				// before the tick signal
 				// so: s2 <= s1
 
+				// allow +/-350ms inaccuracy
+				const int inaccuracy = 350;
+
 				QVERIFY( tickTime <= m_media->currentTime() );
-				QVERIFY( s1 >= tickTime );
-				QVERIFY( s2 <= tickTime );
+				QVERIFY( s1 + inaccuracy >= tickTime );
+				QVERIFY( s2 - inaccuracy <= tickTime );
+				/*
+				if( s1 + inaccuracy < tickTime )
+					QWARN( qPrintable( QString( "tickTime %1 is greater than %2 - %3, the time that elapsed since before playback was started" )
+								.arg( tickTime ).arg( s1 + inaccuracy ).arg( inaccuracy ) ) );
+				if( s2 - inaccuracy > tickTime )
+					QWARN( qPrintable( QString( "tickTime %1 is less than %2 + %3, the time that elapsed since after playback was started" )
+								.arg( tickTime ).arg( s2 - inaccuracy ).arg( inaccuracy ) ) );
+				*/
 				QVERIFY( s1 >= lastCount * m_media->tickInterval() );
 				if( s2 > ( lastCount + 1 ) * m_media->tickInterval() )
 					QWARN( qPrintable( QString( "%1. tick came too late: %2ms elapsed while this tick should have come before %3ms" )
 							.arg( lastCount ).arg( s2 ).arg( ( lastCount + 1 ) * m_media->tickInterval() ) ) );
-			}
+            } else if (lastCount == 0 && s2 > 20 * m_media->tickInterval()) {
+                QFAIL("no tick signals are being received");
+            }
 			s2 = start2.elapsed();
 			QCoreApplication::processEvents();
 		}
@@ -412,6 +603,9 @@ void MediaObjectTest::addPaths()
 	}
 	else
 		QWARN( "backend does not allow usage of more than one AudioPath" );
+	delete a1;
+	QCOMPARE( m_media->audioPaths().size(), 0 );
+	a1 = 0;
 
 	m_media->addVideoPath( v1 );
 	QCOMPARE( m_media->videoPaths().size(), 1 ); // one should always work
@@ -445,6 +639,19 @@ void MediaObjectTest::addPaths()
 	}
 	else
 		QWARN( "backend does not allow usage of more than one VideoPath" );
+	delete v1;
+	QCOMPARE( m_media->videoPaths().size(), 0 );
+	v1 = 0;
+}
+
+void MediaObjectTest::initOutput()
+{
+	// AudioPath and AudioOutput are needed else the backend might finish in no time
+	AudioPath* audioPath = new AudioPath( this );
+	AudioOutput* audioOutput = new AudioOutput( Phonon::MusicCategory, this );
+    //audioOutput->setVolume(0.0f);
+	audioPath->addOutput( audioOutput );
+	m_media->addAudioPath( audioPath );
 }
 
 void MediaObjectTest::cleanupTestCase()
@@ -455,4 +662,4 @@ void MediaObjectTest::cleanupTestCase()
 
 QTEST_KDEMAIN( MediaObjectTest, NoGUI )
 #include "mediaobjecttest.moc"
-// vim: sw=4 ts=4 noet
+// vim: sw=4 ts=4
