@@ -25,12 +25,11 @@
 class MethodTest : public QObject
 {
 	Q_OBJECT
-	private slots:
+	private Q_SLOTS:
 		void checkUiBackendMethods_data();
 		void checkUiBackendMethods();
-		//VideoWidget doesn't have any methods
-		//void checkVideoWidgetMethods_data();
-		//void checkVideoWidgetMethods();
+		void checkVideoWidgetMethods_data();
+		void checkVideoWidgetMethods();
 
 	private:
 		void addColumns();
