@@ -2,7 +2,7 @@
 #include <qstring.h>
 
 #include <kimageeffect.h>
-#include <QSvgRenderer>
+#include <ksvgrenderer.h>
 #include <QPainter>
 #include <iostream>
 
@@ -22,13 +22,13 @@ int main(int argc, char **argv)
 	int height = atoi(argv[2]);
 
 	QImage img(width, height, QImage::Format_ARGB32_Premultiplied);
-	QPainter p(&img);
-	QSvgRenderer renderer(QString::fromLocal8Bit(argv[3]));
+	img.fill(0);
 
+	KSvgRenderer renderer(QString::fromLocal8Bit(argv[3]));
 	if(renderer.isValid())
 	{
+	        QPainter p(&img);
 		renderer.render(&p);
-		p.end();
 /*
 		// Apply icon sharpening
 		double factor = 0;
